﻿note
	description: "[
		Implementation of {STS_COMPLEX_NUMBER}
		
		Although {STS_COMPLEX_NUMBER} specification follows the semantics of
		complex arithmetic of ISO C17 (please see {STS_COMPLEX_NUMBER}
		description), this implementation can not leverage the support of the C
		compiler to complex arithmetic, since C compilers are not required to
		offer such support. For example, Microsoft Visual C 2019 compiler does
		not implement complex numbers as specified by ISO C17.
		
		Notice that, as stated at {STS_COMPLEX_NUMBER} description, that
		specification (and this implementation)) does not try to offer a
		high-grade numeric class. Its intent is just to introduce examples of
		how several numeric types and sets may receive a set-theoretic
		treatment.
		
		Ideally, {STS_COMPLEX_NUMBER} could inherit from
		{STS_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, ...]}. However, for the
		sake of regularity, in such a case {STS_RATIONAL_NUMBER} should inherit
		from {STS_HOMOGENEOUS_ORDERED_PAIR [STS_INTEGER_NUMBER, ...]}. But then
		{STS_RATIONAL_NUMBER} would inherit twice from
		{STS_HOMOGENEOUS_ORDERED_PAIR}, and such a double inheritance would be
		valid only if both {STS_HOMOGENEOUS_ORDERED_PAIR} ancestors had the
		same actual generic parameters, which would not be the case. Such a
		mess does not occur here, since {COMPLEX_NUMBER} is not an ancestor of
		{RATIONAL_NUMBER}, and each one of these classes can inherit from the
		appropriate derivation of {STS_HOMOGENEOUS_ORDERED_PAIR}. Inheriting
		from {HOMOGENEOUS_ORDERED_PAIR} is not possible because the fields `a'
		and `b' would change their expansion status, which is a forbidden
		construct. The same observations apply to the hypothesis of inheriting
		from {STS_COMPOSABLE_ORDERED_PAIR} and {COMPOSABLE_ORDERED_PAIR}.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

expanded class
	COMPLEX_NUMBER

inherit
	STS_COMPLEX_NUMBER
		rename
			a as real,
			b as imaginary
		redefine
			default_create,
			out,
			value_is_infinite,
			value_is_finite,
			reciprocal, inverse
		end

	STS_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_EQUALITY]
		rename
			a as real,
			first as real,
			b as imaginary,
			second as imaginary,
			is_in as pair_is_in,
			is_not_in as pair_is_not_in,
			equals as pair_equals,
			unequals as pair_unequals,
			inverse as pair_inverse,
			composed_with_inverse_like as composed alias "∘"
		redefine
			default_create,
			out
		end

	STS_COMPOSABLE_ORDERED_PAIR
		[STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY, STS_REAL_EQUALITY]
		rename
			a as real,
			first as real,
			b as imaginary,
			second as imaginary,
			eq_w as Eq,
			eq_a as Eq,
			eq_b as Eq,
			is_in as pair_is_in,
			is_not_in as pair_is_not_in,
			equals as pair_equals,
			unequals as pair_unequals,
			inverse as pair_inverse,
			composed_with_homogeneous as composed alias "∘",
			composed_with_inverse_like as composed alias "∘"
		redefine
			default_create,
			out
		end

	DEBUG_OUTPUT
		rename
			debug_output as out
		redefine
			default_create,
			out
		end

create
	default_create,
	make,
	make_from_expanded,
	make_from_complex,
	make_from_native_real,
	make_from_native_real_ref,
	make_from_native_double_real,
	make_from_native_double_real_ref,
	make_from_real,
	make_from_native_integer,
	make_from_native_integer_ref,
	make_from_integer,
	make_from_native_natural,
	make_from_native_natural_ref,
	make_from_ordered_pair

convert
	make_from_complex ({STS_COMPLEX_NUMBER}),
	make_from_native_real ({REAL}),
	make_from_native_real_ref ({REAL_REF}),
	make_from_native_double_real ({DOUBLE}),
	make_from_native_double_real_ref ({DOUBLE_REF}),
	make_from_real ({STS_REAL_NUMBER}),
	make_from_native_integer ({INTEGER}),
	make_from_native_integer_ref ({INTEGER_REF}),
	make_from_integer ({STS_INTEGER_NUMBER}),
	make_from_native_natural ({NATURAL}),
	make_from_native_natural_ref ({NATURAL_32_REF}),
	make_from_ordered_pair ({
		STS_ORDERED_PAIR [
			STS_REAL_NUMBER, STS_REAL_NUMBER,
			STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]
			],
		ORDERED_PAIR [
			STS_REAL_NUMBER, STS_REAL_NUMBER,
			STS_NUMERIC_EQUALITY [STS_REAL_NUMBER],
			STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]
			],
		ORDERED_PAIR [
			STS_REAL_NUMBER, STS_REAL_NUMBER,
			STS_REAL_EQUALITY, STS_REAL_EQUALITY
			],

		STS_ORDERED_PAIR [
			REAL_NUMBER, REAL_NUMBER,
			STS_EQUALITY [REAL_NUMBER], STS_EQUALITY [REAL_NUMBER]
			],
		ORDERED_PAIR [
			REAL_NUMBER, REAL_NUMBER,
			STS_NUMERIC_EQUALITY [REAL_NUMBER],
			STS_NUMERIC_EQUALITY [REAL_NUMBER]
			],

		STS_HOMOGENEOUS_ORDERED_PAIR
			[STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER]],
		HOMOGENEOUS_ORDERED_PAIR
			[STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]],
		HOMOGENEOUS_ORDERED_PAIR
			[STS_REAL_NUMBER, STS_REAL_EQUALITY],

		STS_HOMOGENEOUS_ORDERED_PAIR [REAL_NUMBER, STS_EQUALITY [REAL_NUMBER]],
		HOMOGENEOUS_ORDERED_PAIR
			[REAL_NUMBER, STS_NUMERIC_EQUALITY [REAL_NUMBER]]
		}),
	as_ordered_pair: {
		HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_EQUALITY],
		HOMOGENEOUS_ORDERED_PAIR
			[STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]],

		ORDERED_PAIR [
			STS_REAL_NUMBER, STS_REAL_NUMBER,
			STS_REAL_EQUALITY, STS_REAL_EQUALITY
			],
		ORDERED_PAIR [
			STS_REAL_NUMBER, STS_REAL_NUMBER,
			STS_NUMERIC_EQUALITY [STS_REAL_NUMBER],
			STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]
			]
		},
	as_tuple: {
		N_TUPLE [STS_REAL_NUMBER, STS_REAL_EQUALITY],
		N_TUPLE [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]],

		STS_N_TUPLE [STS_REAL_NUMBER, STS_REAL_EQUALITY],
		STS_N_TUPLE [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]
		}

feature {NONE} -- Initialization

	default_create
			-- Create the complex number 0 + 0i.
			--| Why not an empty body? Consider a local z: COMPLEX_NUMBER; create z is issued implicitly every time the routine that declares z starts. That is true even when such a
			--| routine restarts due to a retry inside a rescue clause; if z is no more Zero at that point, a default_create empty body does not revert z to Zero, so violating
			--| default_create post-condition.
		do
			create a
			create b
		ensure then
			real: a ≍ Zero
			imaginary: b ≍ Zero
		end

	make (a_real, a_imaginary: STS_REAL_NUMBER)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`a_real'.`value') and
			-- `imaginary'.`value' = {REAL_NUMBER}.adjusted_value (`a_imaginary'.`value')
		do
			a := converted_real (a_real)
			b := converted_real (a_imaginary)
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (a_real.value)
			imaginary: b.value = {REAL_NUMBER}.adjusted_value (a_imaginary.value)
		end

	make_from_expanded (a_real, a_imaginary: REAL_NUMBER)
			-- Create the complex number `a_real' + `a_imaginary'i.
			-- Useful to create complex numbers out of manifest constants.
		do
			a := a_real
			b := a_imaginary
		ensure
			real: a ≍ a_real
			imaginary: b ≍ a_imaginary
		end

	make_from_complex (z: STS_COMPLEX_NUMBER)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`z'.`a'.`value') and `imaginary'.`value' = {REAL_NUMBER}.adjusted_value (`z'.`b'.`value')
		do
			if attached {COMPLEX_NUMBER} z as exp_z then
				make_from_expanded (exp_z.a, exp_z.b)
			else
				make (z.a, z.b)
			end
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (z.a.value)
			imaginary: b.value = {REAL_NUMBER}.adjusted_value (z.b.value)
		end

	make_from_native_real (v: like real_anchor.value)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`v') and `imaginary' = 0
		do
			create a.make (v)
			create b
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (v)
			imaginary: b ≍ Zero
		end

	make_from_native_real_ref (v_ref: like real_anchor.value.to_reference)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`v_ref'.`item') and `imaginary' = 0
		do
			make_from_native_real (v_ref.item)
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (v_ref.item)
			imaginary: b ≍ Zero
		end

	make_from_native_double_real (v: like real_anchor.value.to_double)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`v'.`truncated_to_real') and `imaginary' = 0
			--| TODO: Get rid of truncated_to_real.
		do
			create a.make_from_native_double_real (v)
			create b
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (v.truncated_to_real)
			imaginary: b ≍ Zero
		end

	make_from_native_double_real_ref (v_ref: like real_anchor.value.to_double.to_reference)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`v_ref'.`truncated_to_real') and `imaginary' = 0
			--| TODO: Get rid of truncated_to_real.
		do
			make_from_native_double_real (v_ref.item)
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (v_ref.truncated_to_real)
			imaginary: b ≍ Zero
		end

	make_from_real (x: STS_REAL_NUMBER)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`x'.`value') and `imaginary'.`value' = 0
		do
			a := converted_real (x)
			create b
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (x.value)
			imaginary: b ≍ Zero
		end

	make_from_native_integer (v: like Integer_anchor.value)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`v'.`to_real') and `imaginary' = 0
		do
			create a.make_from_native_integer (v)
			create b
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (v)
			imaginary: b ≍ Zero
		end

	make_from_native_integer_ref (v_ref: like Integer_anchor.value.to_reference)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`v_ref'.`to_real') and `imaginary' = 0
		do
			make_from_native_integer (v_ref.item)
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (v_ref.to_real)
			imaginary: b ≍ Zero
		end

	make_from_integer (j: STS_INTEGER_NUMBER)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`j'.`value'.`to_real') and `imaginary' = 0
		do
			make_from_native_integer (j.value) -- TODO: convert
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (j.value) -- TODO: convert
			imaginary: b ≍ Zero
		end

	make_from_native_natural (v: NATURAL)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`v'.`to_real_32') and `imaginary' = 0
			--| TODO: Anchor `v'.
		do
			create a.make_from_native_natural (v)
			create b
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (v)
			imaginary: b ≍ Zero
		end

	make_from_native_natural_ref (v_ref: NATURAL_32_REF)
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`v_ref'.`to_real_32') and `imaginary' = 0
			--| TODO: Anchor `v_ref'.
		do
			make_from_native_natural (v_ref.item)
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (v_ref.to_real_32)
			imaginary: b ≍ Zero
		end

	make_from_ordered_pair (ab: STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]])
			-- Create a complex number with `real'.`value' = {REAL_NUMBER}.adjusted_value (`ab'.`a'.`value') and `imaginary'.`value' = {REAL_NUMBER}.adjusted_value (`ab'.`b'.`value')
		do
			make (ab.a, ab.b)
		ensure
			real: a.value = {REAL_NUMBER}.adjusted_value (ab.a.value)
			imaginary: b.value = {REAL_NUMBER}.adjusted_value (ab.b.value)
		end

feature -- Primitive

	a: like real_anchor
			-- Real part of current complex number

	real: like real_anchor
			-- <Precursor>
		do
			Result := a
		ensure then
			definition: Result ≍ a
		end

	b: like real_anchor
			-- Imaginary part of current complex number

	imaginary: like real_anchor
			-- <Precursor>
		do
			Result := b
		ensure then
			definition: Result ≍ b
		end

	eq: STS_REAL_EQUALITY
			-- Equality for real numbers
		once
			Result := {REAL_SET}.eq
		ensure then
			class
		end

feature -- Access

	Zero: COMPLEX_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	One: COMPLEX_NUMBER
			-- <Precursor>
		once
			create Result.make_from_expanded ({REAL_NUMBER}.One, {REAL_NUMBER}.Zero)
		ensure then
			class
		end

	Two: INTEGER_NUMBER
			-- <Precursor>
		once
			create Result.make (2)
		ensure then
			class
		end

	i,
	imaginary_unit: COMPLEX_NUMBER
			-- <Precursor>
		once
			create Result.make_from_expanded ({REAL_NUMBER}.Zero, {REAL_NUMBER}.One)
		ensure then
			class
		end

feature -- Quality

	is_invertible: BOOLEAN = True
			-- <Precursor>
			-- Notice that an implementation akin to IEEE 754, like current one, inverts zero with no major consequence.

feature -- Output

	out: STRING
			-- <Precursor>
		do
			Result := a.out
			if b.sign_bit ≍ Zero or b.is_nan then
				Result.append_character ('+')
			end
				check
					argument_not_void: b.out /= Void -- {REAL_NUMBER}.out definition
				end
			Result.append (b.out)
			if not b.is_finite then
				Result.append_character ('×')
			end
			Result.append_character ('i')
		ensure then
			when_nan_imaginary: b.is_nan implies Result ~ a.out + "+" + b.out + "×i"
			when_negative_infinity_imaginary: b.is_negative_infinity implies Result ~ a.out + b.out + "×i"
			when_positive_infinity_imaginary: b.is_positive_infinity implies Result ~ a.out + "+" + b.out + "×i"
			when_finite_negative_imaginary: b.is_finite and b.sign_bit ≍ One implies Result ~ a.out + b.out + "i"
			when_finite_non_negative_imaginary: b.is_finite and b.sign_bit ≍ Zero implies Result ~ a.out + "+" + b.out + "i"
		end

feature -- Operation

	pair_inverse: like Complex_anchor
			-- The components of current complex number in inverted order, i.e.
			-- `b' + `a'i.
		do
			create Result.make_from_expanded (b, a)
		end

	composed alias "∘" (ab: STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]): like Complex_anchor
			-- Composition of (`a', `b') and `ab`, i.e. the pair (`ab`.`a', `b').
		do
			create Result.make (ab.a, b)
		end

	conjugate: like complex_anchor
			-- <Precursor>
		do
			create Result.make_from_expanded (a, - b)
		end

	plus alias "+" (z: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
		do
			create Result.make (a + z.a, b + z.b)
		end

	minus alias "-" alias "−" (z: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
		do
			create Result.make (a - z.a, b - z.b)
		end

	opposite alias "-" alias "−": like complex_anchor
			-- <Precursor>
		do
			create Result.make_from_expanded (- a, - b)
		end

	product alias "*" alias "×" alias "⋅" (z: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
			--| Notice that this implementation always takes negative zeros into account.
		local
			u, x, prod_a: like a.value
			v, y, prod_b: like b.value
		do
			u := a
			v := b
			x := z.a.value
			y := z.b.value
			if u = 0 then
				if v = 0 then
					if x = 0 then
						if y = 0 then
								check
									Current ≍ Zero -- a = u = b = v = 0
									z ≍ Zero -- z.a = x = z.b = y = 0
								end
							prod_a := u * x - v * y
							prod_b := u * y + v * x
							create Result.make_from_expanded (prod_a, prod_b)
						else
								check
									Current ≍ Zero -- a = u = b = v = 0
									z.is_proper_imaginary -- z.a = x = 0 and z.b = y /= 0
								end
							prod_a := - v * y
							prod_b := u * if value_sign_bit (y) = 0 then 0 else - 1 end + v * x
							create Result.make_from_expanded (prod_a, prod_b)
						end
					else
						if y = 0 then
								check
									Current ≍ Zero -- a = u = b = v = 0
									z.is_proper_real -- z.a = x /= 0 and z.b = y = 0
								end
							prod_a := u * x
							prod_b := u * y + v * if value_sign_bit (x) = 0 then 0 else - 1 end
							create Result.make_from_expanded (prod_a, prod_b)
						else
								check
									Current ≍ Zero -- a = u = b = v = 0
									z.is_proper_complex -- z.a = x /= 0 and z.b = y /= 0
								end
							prod_a := u * x - v * y
							prod_b := u * y + v * x
							create Result.make_from_expanded (prod_a, prod_b)
						end
					end
				else
					if x = 0 then
						if y = 0 then
								check
									is_proper_imaginary -- a = u = 0 and b = v /= 0
									z ≍ Zero -- z.a = x = z.b = y = 0
								end
							prod_a := - v * y
							prod_b := u * y + if value_sign_bit (v) = 0 then 0 else - 1 end * x
							create Result.make_from_expanded (prod_a, prod_b)
						else
								check
									is_proper_imaginary -- a = u = 0 and b = v /= 0
									z.is_proper_imaginary -- z.a = x = 0 and z.b = y /= 0
								end
							prod_a := - v * y
							prod_b := u * if value_sign_bit (y) = 0 then 0 else - 1 end + if value_sign_bit (v) = 0 then 0 else - 1 end * x
							create Result.make_from_expanded (prod_a, prod_b)
						end
					else
						if y = 0 then
								check
									is_proper_imaginary -- a = u = 0 and b = v /= 0
									z.is_proper_real -- z.a = x /= 0 and z.b = y = 0
								end
							prod_a := u * if value_sign_bit (x) = 0 then 0 else - 1 end - if value_sign_bit (v) = 0 then 0 else - 1 end * y
							prod_b := v * x
							create Result.make_from_expanded (prod_a, prod_b)
						else
								check
									is_proper_imaginary -- a = u = 0 and b = v /= 0
									z.is_proper_complex -- z.a = x /= 0 and z.b = y /= 0
								end
							prod_a := - v * y
							prod_b := v * x
							create Result.make_from_expanded (prod_a, prod_b)
						end
					end
				end
			else
				if v = 0 then
					if x = 0 then
						if y = 0 then
								check
									is_proper_real -- a = u /= 0 and b = v = 0
									z ≍ Zero -- z.a = x = z.b = y = 0
								end
							prod_a := u * x
							prod_b := if value_sign_bit (u) = 0 then 0 else - 1 end * y + v * x
							create Result.make_from_expanded (prod_a, prod_b)
						else
								check
									is_proper_real -- a = u /= 0 and b = v = 0
									z.is_proper_imaginary -- z.a = x = 0 and z.b = y /= 0
								end
							prod_a := if value_sign_bit (u) = 0 then 0 else - 1 end * x - v * if value_sign_bit (y) = 0 then 0 else - 1 end
							prod_b := u * y
							create Result.make_from_expanded (prod_a, prod_b)
						end
					else
						if y = 0 then
								check
									is_proper_real -- a = u /= 0 and b = v = 0
									z.is_proper_real -- z.a = x /= 0 and z.b = y = 0
								end
							prod_a := u * x
							prod_b := if value_sign_bit (u) = 0 then 0 else - 1 end * y + v * if value_sign_bit (x) = 0 then 0 else - 1 end
							create Result.make_from_expanded (prod_a, prod_b)
						else
								check
									is_proper_real -- a = u /= 0 and b = v = 0
									z.is_proper_complex -- z.a = x /= 0 and z.b = y /= 0
								end
							prod_a := u * x
							prod_b := u * y
							create Result.make_from_expanded (prod_a, prod_b)
						end
					end
				else
					if x = 0 then
						if y = 0 then
								check
									is_proper_complex -- a = u /= 0 and b = v /= 0
									z ≍ Zero -- z.a = x = z.b = y = 0
								end
							prod_a := u * x - v * y
							prod_b := u * y + v * x
							create Result.make_from_expanded (prod_a, prod_b)
						else
								check
									is_proper_complex -- a = u /= 0 and b = v /= 0
									z.is_proper_imaginary -- z.a = x = 0 and z.b = y /= 0
								end
							prod_a := - v * y
							prod_b := u * y
							create Result.make_from_expanded (prod_a, prod_b)
						end
					else
						if y = 0 then
								check
									is_proper_complex -- a = u /= 0 and b = v /= 0
									z.is_proper_real -- z.a = x /= 0 and z.b = y = 0
								end
							prod_a := u * x
							prod_b := v * x
							create Result.make_from_expanded (prod_a, prod_b)
						else
								check
									is_proper_complex -- a = u /= 0 and b = v /= 0
									z.is_proper_complex -- z.a = x /= 0 and z.b = y /= 0
								end
							Result := product_from_native_reals (u, v, x, y)
						end
					end
				end
			end
		ensure then
			ref_prod: attached product_from_native_reals (a, b, z.a.value, z.b.value) as ref_prod
			complex_times_complex_implementation_a: is_proper_complex and z.is_proper_complex implies Result.a ≍ ref_prod.a
			complex_times_complex_implementation_b: is_proper_complex and z.is_proper_complex implies Result.b ≍ ref_prod.b
		end

	quotient alias "/" alias "÷" (z: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
			-- Notice that this implementation always takes negative zeros into account, and even accepts a divisor equal to zero.
		local
			u, x, quot_a: like a.value
			v, y, quot_b: like b.value
		do
			u := a
			v := b
			x := z.a.value
			y := z.b.value
			if u = 0 then
				if v = 0 then
					if x = 0 then
						if y = 0 then
								check
									Current ≍ Zero -- a = u = b = v = 0
									z ≍ Zero -- z.a = x = z.b = y = 0
								end
							Result := quotient_from_native_reals (u, v, x, y)
						else
								check
									Current ≍ Zero -- a = u = b = v = 0
									z.is_proper_imaginary -- z.a = x = 0 and z.b = y /= 0
									zero_by_imaginary_a_good_divisor: v.divisible (y) -- y /= 0
								end
							quot_a := v / y
							quot_b := v * x - u * if value_sign_bit (y) = 0 then 0 else - 1 end
							create Result.make_from_expanded (quot_a, quot_b)
						end
					else
						if y = 0 then
								check
									Current ≍ Zero -- a = u = b = v = 0
									z.is_proper_real -- z.a = x /= 0 and z.b = y = 0
									zero_by_real_a_good_divisor: u.divisible (x) -- x /= 0
								end
							quot_a := u / x
							quot_b := v * if value_sign_bit (x) = 0 then 0 else - 1 end - u * y
							create Result.make_from_expanded (quot_a, quot_b)
						else
								check
									Current ≍ Zero -- a = u = b = v = 0
									z.is_proper_complex -- z.a = x /= 0 and z.b = y /= 0
								end
							Result := quotient_from_native_reals (u, v, x, y)
						end
					end
				else
					if x = 0 then
						if y = 0 then
								check
									is_proper_imaginary -- a = u = 0 and b = v /= 0
									z ≍ Zero -- z.a = x = z.b = y = 0
								end
							quot_a := v / y -- NOTICE: Precondition ignored.
							quot_b := if value_sign_bit (v) = 0 then 0 else - 1 end * x - u * y
							create Result.make_from_expanded (quot_a, quot_b)
						else
								check
									is_proper_imaginary -- a = u = 0 and b = v /= 0
									z.is_proper_imaginary -- z.a = x = 0 and z.b = y /= 0
									imaginary_by_imaginary_a_good_divisor: v.divisible (y) -- y /= 0
								end
							quot_a := v / y
							quot_b := if value_sign_bit (v) = 0 then 0 else - 1 end * x - u * if value_sign_bit (y) = 0 then 0 else - 1 end
							create Result.make_from_expanded (quot_a, quot_b)
						end
					else
						if y = 0 then
								check
									is_proper_imaginary -- a = u = 0 and b = v /= 0
									z.is_proper_real -- z.a = x /= 0 and z.b = y = 0
									imaginary_by_real_b_good_divisor: v.divisible (x) -- x /= 0
								end
							quot_a := u * if value_sign_bit (x) = 0 then 0 else - 1 end + if value_sign_bit (v) = 0 then 0 else - 1 end * y
							quot_b := v / x
							create Result.make_from_expanded (quot_a, quot_b)
						else
								check
									is_proper_imaginary -- a = u = 0 and b = v /= 0
									z.is_proper_complex -- z.a = x /= 0 and z.b = y /= 0
								end
							Result := quotient_from_native_reals (u, v, x, y)
						end
					end
				end
			else
				if v = 0 then
					if x = 0 then
						if y = 0 then
								check
									is_proper_real -- a = u /= 0 and b = v = 0
									z ≍ Zero -- z.a = x = z.b = y = 0
								end
							quot_a := u / x -- NOTICE: Precondition ignored.
							quot_b := v * x - if value_sign_bit (u) = 0 then 0 else - 1 end * y
							create Result.make_from_expanded (quot_a, quot_b)
						else
								check
									is_proper_real -- a = u /= 0 and b = v = 0
									z.is_proper_imaginary -- z.a = x = 0 and z.b = y /= 0
									real_by_imaginary_b_good_divisor: (- u).divisible (y) -- y /= 0
								end
							quot_a := if value_sign_bit (u) = 0 then 0 else - 1 end * x + v * if value_sign_bit (y) = 0 then 0 else - 1 end
							quot_b := - u / y
							create Result.make_from_expanded (quot_a, quot_b)
						end
					else
						if y = 0 then
								check
									is_proper_real -- a = u /= 0 and b = v = 0
									z.is_proper_real -- z.a = x /= 0 and z.b = y = 0
									real_by_real_a_good_divisor: u.divisible (x) -- x /= 0
								end
							quot_a := u / x
							quot_b := v * if value_sign_bit (x) = 0 then 0 else - 1 end - if value_sign_bit (u) = 0 then 0 else - 1 end * y
							create Result.make_from_expanded (quot_a, quot_b)
						else
								check
									is_proper_real -- a = u /= 0 and b = v = 0
									z.is_proper_complex -- z.a = x /= 0 and z.b = y /= 0
								end
							Result := quotient_from_native_reals (u, v, x, y)
						end
					end
				else
					if x = 0 then
						if y = 0 then
								check
									is_proper_complex -- a = u /= 0 and b = v /= 0
									z ≍ Zero -- z.a = x = z.b = y = 0
								end
							Result := quotient_from_native_reals (u, v, x, y)
						else
								check
									is_proper_complex -- a = u /= 0 and b = v /= 0
									z.is_proper_imaginary -- z.a = x = 0 and z.b = y /= 0
									complex_by_imaginary_a_good_divisor: v.divisible (y) -- y /= 0
									complex_by_imaginary_b_good_divisor: (- u).divisible (y)
								end
							quot_a := v / y
							quot_b := - u / y
							create Result.make_from_expanded (quot_a, quot_b)
						end
					else
						if y = 0 then
								check
									is_proper_complex -- a = u /= 0 and b = v /= 0
									z.is_proper_real -- z.a = x /= 0 and z.b = y = 0
									complex_by_real_a_good_divisor: u.divisible (x) -- x /= 0
									complex_by_real_b_good_divisor: v.divisible (x)
								end
							quot_a := u / x
							quot_b := v / x
							create Result.make_from_expanded (quot_a, quot_b)
						else
								check
									is_proper_complex -- a = u /= 0 and b = v /= 0
									z.is_proper_complex -- z.a = x /= 0 and z.b = y /= 0
								end
							Result := quotient_from_native_reals (u, v, x, y)
						end
					end
				end
			end
		ensure then
				--	bi / 0i = b/0 = b/0+0i (Treat 0 as a purely imaginary number; take signs into account.)
			imaginary_by_zero_a: is_proper_imaginary and z ≍ zero implies Result.a ≍ (b / z.b)
			imaginary_by_zero_b: is_proper_imaginary and z ≍ zero implies Result.b ≍ ((if b.sign_bit.as_real ≍ Zero.a then Zero.a else -One.a end) * z.a - a * z.b)

				-- a / 0 = a/0 = a/0+0i (Treat 0 as a purely real number; take signs into account.)
			real_by_zero_a: is_proper_real and z ≍ zero implies Result.a ≍ (a / z.a)
			real_by_zero_b: is_proper_real and z ≍ zero implies Result.b ≍ (b * z.a - (if a.sign_bit.as_real ≍ Zero.a then Zero.a else -One.a end) * z.b)

				-- (a+bi)/(c+di): Treat zero as a proper complex number.
			ref_quot: attached quotient_from_native_reals (a, b, z.a.value, z.b.value) as ref_quot
			complex_divisor_implementation_a: (Current ≍ Zero or is_proper_complex) and (z ≍ Zero or z.is_proper_complex) or
				(is_proper_imaginary or is_proper_real) and z.is_proper_complex implies Result.a ≍ ref_quot.a
			complex_divisor_implementation_b: (Current ≍ Zero or is_proper_complex) and (z ≍ Zero or z.is_proper_complex) or
				(is_proper_imaginary or is_proper_real) and z.is_proper_complex implies Result.b ≍ ref_quot.b
		end

	reciprocal,
	inverse: like complex_anchor
			-- <Precursor>
			-- Notice that this implementation inverts a complex number even if it equals zero.
		local
			x, quot_a: like a.value
			y, quot_b: like b.value
		do
			x := a.value
			y := b.value
			if x = 0 then
				if y = 0 then
						check
							Current ≍ Zero -- a = x = b = y = 0
						end
					quot_a := 1 / x -- NOTICE: Precondition ignored.
					quot_b := 0 * x - 0 * y
					create Result.make_from_expanded (quot_a, quot_b)
				else
						check
							is_proper_imaginary -- a = x = 0 and b = y /= 0
							real_by_imaginary_b_good_divisor: (- {REAL} 1.0).divisible (y) -- y /= 0
						end
					quot_a := 0 * x + 0 * if value_sign_bit (y) = 0 then 0 else - 1 end
					quot_b := - 1 / y
					create Result.make_from_expanded (quot_a, quot_b)
				end
			else
				if y = 0 then
						check
							is_proper_real -- a = x /= 0 and b = y = 0
							real_by_real_a_good_divisor: ({REAL} 1.0).divisible (x) -- x /= 0
						end
					quot_a := 1 / x
					quot_b := 0 * if value_sign_bit (x) = 0 then 0 else - 1 end - 0 * y
					create Result.make_from_expanded (quot_a, quot_b)
				else
						check
							is_proper_complex -- a = x /= 0 and b = y /= 0
						end
					Result := quotient_from_native_reals (1, 0, x, y)
				end
			end
		end

feature -- Conversion

	as_singleton: SET_OF_ORDERED_PAIRS [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]
			-- The pair (`a', `b') alone into a singleton, i.e. {(`a', `b')}.
		do
--			create Result.make_singleton (Current) -- TODO: System crash! {SET_OF_ORDERED_PAIRS}.converted_pair fails on noting that `Current' is not an {ORDERED_PAIR}.
			create Result.make_singleton (as_ordered_pair)
		end

	as_tuple: N_TUPLE [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- <Precursor>
		do
			Result := {REAL_UNIVERSE}.Unit.after (b).after (a)
		end

	as_ordered_pair: HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Current complex number converted into an effective ordered pair of real numbers
		do
			create Result.make (a, b)
		ensure
			first: Result.a ≍ a
			second: Result.b ≍ b
		end

feature -- Math

	value_is_infinite (v: like real_anchor.value): BOOLEAN
			-- <Precursor>
		do
			Result := {REAL_NUMBER}.value_is_infinite (v)
		end

	value_is_finite (v: like real_anchor.value): BOOLEAN
			-- <Precursor>
		do
			Result := {REAL_NUMBER}.value_is_finite (v)
		end

	hypotenuse (x, y: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			Result := c_hypot (x.value, y.value)
		ensure then
			class
			implementation: Result.value = {REAL_NUMBER}.adjusted_value (c_hypot (x.value, y.value))
		end

	arc_tangent_yx (y, x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			Result := c_atan2 (y.value, x.value)
		ensure then
			class
		end

feature -- Factory

	real_from_value (v: like real_anchor.value): like real_anchor
			-- <Precursor>
		do
			Result := {REAL_NUMBER}.real_from_value (v)
		ensure then
			class
		end

	real_from_double_value (dv: like real_anchor.value.to_double): like real_anchor
			-- <Precursor>
		do
			Result := {REAL_NUMBER}.real_from_double_value (dv)
		ensure then
			class
		end

	converted_real (x: STS_REAL_NUMBER): like real_anchor
			-- `x' converted to a real number like `real_anchor'
		do
			if attached {like real_anchor} x as cx then
				Result := cx
			else
				create Result.make_from_real (x)
			end
		ensure
			class
			definition: Result.value = {like real_anchor}.adjusted_value (x.value)
		end

feature -- Anchor

	complex_anchor: COMPLEX_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	real_anchor: REAL_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	integer_anchor: INTEGER_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

feature {NONE} -- Implementation

	value_is_negative_zero (v: like real_anchor.value): BOOLEAN
			-- Is `v` the representation of a "negative" zero, as defined by IEEE 754?
		do
			Result := v = 0 and value_sign_bit (v) = 1
		ensure
			class
			definition: Result = (v = 0 and value_sign_bit (v) = 1)
		end

	value_is_positive_zero (v: like real_anchor.value): BOOLEAN
			-- Is `v` the representation of a "positive" zero, as defined by IEEE 754?
		do
			Result := v = 0 and value_sign_bit (v) = 0
		ensure
			class
			definition: Result = (v = 0 and value_sign_bit (v) = 0)
		end

	c_hypot (x, y: like real_anchor.value): like real_anchor.value
			-- Square root of the sum of the squares of x and y, without undue overﬂow or underﬂow.			
		external
			"C signature (float, float): float use <math.h>"
		alias
			"hypotf"
		ensure
			class
			commutative: Result = c_hypot (y, x)
			when_unidimensional: y = 0 implies Result = x.abs
			when_negatively_infinite: x.is_negative_infinity implies Result.is_positive_infinity
			when_positively_infinite: x.is_positive_infinity implies Result.is_positive_infinity
		end

	c_atan2 (y, x: like real_anchor.value): like real_anchor.value
			-- Arc tangent of `y'/`x', using the signs of both arguments to determine the quadrant of the return value.
		external
			"C signature (float, float): float use <math.h>"
		alias
			"atan2f"
		ensure
			class
			when_either_is_nan: y.is_nan or x.is_nan implies Result.is_nan
			lower_bound: not (y.is_nan or x.is_nan) implies - Pi <= Result
			upper_bound: Result <= Pi

			negative_zero_by_negative_zero: value_is_negative_zero (y) and value_is_negative_zero (x) implies Result = - Pi
			positive_zero_by_negative_zero: value_is_positive_zero (y) and value_is_negative_zero (x) implies Result = Pi

			negative_zero_by_positive_zero: value_is_negative_zero (y) and value_is_positive_zero (x) implies value_is_negative_zero (Result)
			positive_zero_by_positive_zero: value_is_positive_zero (y) and value_is_positive_zero (x) implies value_is_positive_zero (Result)

			negative_zero_by_negative_real: value_is_negative_zero (y) and value_is_finite (x) and x < zero.a implies Result = - Pi
			positive_zero_by_negative_real: value_is_positive_zero (y) and value_is_finite (x) and x < zero.a implies Result = Pi

			negative_zero_by_positive_real: value_is_negative_zero (y) and x > zero.a and value_is_finite (x) implies value_is_negative_zero (Result)
			positive_zero_by_positive_real: value_is_positive_zero (y) and x > zero.a and value_is_finite (x) implies value_is_positive_zero (Result)

			negative_real_by_negative_zero: value_is_finite (y) and y < zero.a and value_is_negative_zero (x) implies Result = - Pi_2
			nevative_real_by_positive_zero: value_is_finite (y) and y < zero.a and value_is_positive_zero (x) implies Result = - Pi_2

			positive_real_by_negative_zero: y > zero.a and value_is_finite (y) and value_is_negative_zero (x) implies Result = Pi_2
			positive_real_by_positive_zero: y > zero.a and value_is_finite (y) and value_is_positive_zero (x) implies Result = Pi_2

			negative_real_by_negative_infinity: value_is_finite (y) and y < zero.a and x.is_negative_infinity implies Result = - Pi
			positive_real_by_negative_infinity: y > zero.a and value_is_finite (y) and x.is_negative_infinity implies Result = Pi

			negative_real_by_positive_infinity: value_is_finite (y) and y < zero.a and x.is_positive_infinity implies value_is_negative_zero (Result)
			positive_real_by_positive_infinity: y > zero.a and value_is_finite (y) and x.is_positive_infinity implies value_is_positive_zero (Result)

			negative_infinity_by_finite_real: y.is_negative_infinity and value_is_finite (x) implies Result = - Pi_2
			positive_infinity_by_finite_real: y.is_positive_infinity and value_is_finite (x) implies Result = Pi_2

			negative_infinity_by_negative_infinity: y.is_negative_infinity and x.is_negative_infinity implies Result = - 3 × Pi_4
			positive_infinity_by_negative_infinity: y.is_positive_infinity and x.is_negative_infinity implies Result = 3 × Pi_4

			negative_infinity_by_positive_infinity: y.is_negative_infinity and x.is_positive_infinity implies Result = - Pi_4
			positive_infinity_by_positive_infinity: y.is_positive_infinity and x.is_positive_infinity implies Result = Pi_4

			good_divisor: x /= 0 implies y.divisible (x)
			first_quadrant: 0 < y and value_is_finite (y) and 0 < x and value_is_finite (x) implies 0 < Result and Result < Pi_2
			second_quadrant: 0 < y and value_is_finite (y) and value_is_finite (x) and x < 0 implies Pi_2 < Result and Result < Pi
			third_quadrant: value_is_finite (y) and y < 0 and value_is_finite (x) and x < 0 implies -Pi < Result and Result < -Pi_2
			fourth_quadrant: value_is_finite (y) and y < 0 and 0 < x and value_is_finite (x) implies -Pi_2 < Result and Result < 0
		end

	product_from_native_reals (u, v, x, y: like real_anchor.value): like complex_anchor
			-- The complex product (`u'+`v'i)(`x'+`y'i) when both factors are proper complex numbers; the intent is to avoid some undue overflows, underflows and NaNs.
			--| This implementation follows the algorythm presented at section "G.5.1 Multiplicative operators" of ISO/IEC9899:2017
			--| (AKA ISO C17, please see https://en.wikipedia.org/wiki/C17_(C_standard_revision).) available at https://tinyurl.com/244jc9uf.
			--| Beware this implementation may change in order to improve precision or performance.
		local
			l_u, l_v, l_x, l_y, ux, vy, uy, vx, l_prod_a, l_prod_b: like real_anchor.value
			recalc: BOOLEAN
		do
			l_u := u
			l_v := v
			l_x := x
			l_y := y

			ux := l_u * l_x
			vy := l_v * l_y
			uy := l_u * l_y
			vx := l_v * l_x

			l_prod_a := ux - vy
			l_prod_b := uy + vx

			if l_prod_a.is_nan and l_prod_b.is_nan then
					-- Recover infinities that computed as NaN + iNaN.
				if value_is_infinite (l_u) or value_is_infinite (l_v) then
						-- `l_u'+`l_v'i is infinite. "Box" the infinity and change NaNs in the other factor to 0.
					l_u := c_copysign (if value_is_infinite (l_u) then 1 else 0 end, l_u)
					l_v := c_copysign (if value_is_infinite (l_v) then 1 else 0 end, l_v)
					if l_x.is_nan then l_x := c_copysign (0, l_x) end
					if l_y.is_nan then l_y := c_copysign (0, l_y) end
					recalc := True
				end
				if value_is_infinite (l_x) or value_is_infinite (l_y) then
						-- `l_x'+`l_y'i is infinite. "Box" the infinity and change NaNs in the other factor to 0.
					l_x := c_copysign (if value_is_infinite (l_x) then 1 else 0 end, l_x)
					l_y := c_copysign (if value_is_infinite (l_y) then 1 else 0 end, l_y)
					if l_u.is_nan then l_u := c_copysign (0, l_u) end
					if l_v.is_nan then l_v := c_copysign (0, l_v) end
					recalc := True
				end
				if
					not recalc and (value_is_infinite (ux) or value_is_infinite (vy) or value_is_infinite (uy) or value_is_infinite (vx))
				then
						-- Recover infinities from overflow by changing NaNs to 0.
					if l_u.is_nan then l_u := c_copysign (0, l_u) end
					if l_v.is_nan then l_v := c_copysign (0, l_v) end
					if l_x.is_nan then l_x := c_copysign (0, l_x) end
					if l_y.is_nan then l_y := c_copysign (0, l_x) end
					recalc := True
				end
				if recalc then
					l_prod_a := {like real_anchor.value}.positive_infinity * (l_u * l_x - l_v * l_y)
					l_prod_b := {like real_anchor.value}.positive_infinity * (l_u * l_y + l_v * l_x)
				end
			end
			create Result.make_from_expanded (l_prod_a, l_prod_b)
		ensure
			class
			prod_a: attached real_from_value (u * x - v * y) as prod_a
			prod_b: attached real_from_value (u * y + v * x) as prod_b
			real: (Result.a - prod_a).abs ≤ (Result.a.machine_epsilon ∨ prod_a.relative_epsilon)
			imaginary: (Result.b - prod_b).abs ≤ (Result.b.machine_epsilon ∨ prod_b.relative_epsilon)
		end

	quotient_from_native_reals (u, v, x, y: like real_anchor.value): like complex_anchor
			-- The complex quotient (`u'+`v'i)/(`x'+`y'i) when divisor is a proper complex number or zero; the intent is to avoid some undue overflows,
			--| underflows and NaNs.
			--| This implementation follows the algorythm presented at section "G.5.1 Multiplicative operators" of ISO/IEC9899:2017
			--| (AKA ISO C17, please see https://en.wikipedia.org/wiki/C17_(C_standard_revision).) available at https://tinyurl.com/244jc9uf.
			--| Notice, however, that no scaling is necessary, since the values of {REAL_NUMBER}, when attached even to single-precision floating-point numbers
			--| can never underflow nor overflow.
			--| Beware this implementation may change in order to improve precision or performance.
		local
			l_u, l_v, l_x, l_y, l_denom, l_quot_a, l_quot_b: like real_anchor.value
		do
			l_u := u
			l_v := v
			l_x := x
			l_y := y

			l_denom := l_x * l_x + l_y * l_y
			l_quot_a := (l_u * l_x + l_v * l_y) / l_denom
			l_quot_b := (l_v * l_x - l_u * l_y) / l_denom

			if l_quot_a.is_nan and l_quot_b.is_nan then
				if l_denom = 0 and (not l_u.is_nan or not l_v.is_nan) then
						-- TODO: Aren't the source of these signs arbirary?
					l_quot_a := c_copysign ({like real_anchor.value}.positive_infinity, l_x) * l_u
					l_quot_b := c_copysign ({like real_anchor.value}.positive_infinity, l_x) * l_v
				elseif (value_is_infinite (l_u) or value_is_infinite (l_v)) and value_is_finite (l_x) and value_is_finite (l_y) then
					l_u := c_copysign (if value_is_infinite (l_u) then 1 else 0 end, l_u)
					l_v := c_copysign (if value_is_infinite (l_v) then 1 else 0 end, l_v)
					l_quot_a := {like real_anchor.value}.positive_infinity * (l_u * l_x + l_v * l_y)
					l_quot_b := {like real_anchor.value}.positive_infinity * (l_v * l_x - l_u * l_y)
				elseif (value_is_infinite (l_x) or value_is_infinite (l_y)) and value_is_finite (l_u) and value_is_finite (l_v) then
					l_x := c_copysign (if value_is_infinite (l_x) then 1 else 0 end, l_x)
					l_y := c_copysign (if value_is_infinite (l_y) then 1 else 0 end, l_y)
					l_quot_a := 0 * (l_u * l_x + l_v * l_y)
					l_quot_b := 0 * (l_v * l_x - l_u * l_y)
				end
			end
			create Result.make_from_expanded (l_quot_a, l_quot_b)
		ensure
			class
			denom: attached (x * x + y * y) as denom

				-- (u+vi) / (x + yi) = (ux+vy)/(x^2+y^2)+(vx-uy)i/(x^2+y^2)				
			quot_a: attached {REAL_NUMBER} real_from_value ((u * x + v * y) / denom) as quot_a
			quot_b: attached {REAL_NUMBER} real_from_value ((v * x - u * y) / denom) as quot_b
			real: (Result.a - quot_a).abs ≤ (Result.a.machine_epsilon ∨ quot_a.relative_epsilon)
			imaginary: (Result.b - quot_b).abs ≤ (Result.b.machine_epsilon ∨ quot_b.relative_epsilon)
		end

feature {REAL_NUMBER} -- Implementation

	c_copysign (x, y: like real_anchor.value): like real_anchor.value
			-- Value with the magnitude of `x' and the sign of `y'. It produces a NaN (with the sign of `y') if `x' is a NaN.
		external
			"C signature (float, float): float use <math.h>"
		alias
			"copysignf"
		ensure
			class
			keep_magnitude: Result.abs = x.abs
			take_y_sign: value_sign_bit (Result) = value_sign_bit (y)
		end

invariant
	zero_real: Zero.real.value = 0
	zero_imaginary: Zero.imaginary.value = 0
	one_real: One.real.value = 1
	one_imaginary: One.imaginary.value = 0
	Two: Two.value = 2
	i_real: i.real ≍ Zero.real
	i_imaginary: i.imaginary ≍ One.real
	Imaginary_unit_real: Imaginary_unit.real ≍ Zero.real
	Imaginary_unit_imaginary: Imaginary_unit.imaginary ≍ One.real

		-- is_invertible
	scale_log: attached value_logb ((a.abs ∨ b.abs).value) as scale_log
	int_scale_log: attached if value_is_finite (scale_log) then
			scale_log.truncated_to_integer
		else
			0
		end as int_scale_log
	scaled_a: attached (a.value * 2 ^ (- int_scale_log)) as scaled_a
	scaled_b: attached (b.value * 2 ^ (- int_scale_log)) as scaled_b
	denom: attached (scaled_a ^ 2 + scaled_b ^ 2) as denom
	is_invertible_quasi_definition: denom /= 0 implies Is_invertible


note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
