﻿note
	description: "[
		Implementation of {STS_INTEGER_NUMBER}

		This implementation follows {REAL_NUMBER} by limiting its range to the
		range of integer numbers that {REAL_NUMBER} can represent with no
		precision loss. Integer numbers are treated as if they were encoded by
		a four-bit, two's-complement representation.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

expanded class
	INTEGER_NUMBER

inherit
	STS_INTEGER_NUMBER
		redefine
			default_create,
			sign,
			Machine_epsilon,
			ulp, relative_epsilon,
			is_real,
			is_proper_real,
			is_imaginary,
			is_proper_imaginary,
			is_proper_complex,
			is_nan,
			is_negative_infinity,
			is_negative_zero,
			is_positive_zero,
			is_positive_infinity,
			is_infinite,
			is_finite,
--			is_integer,
			real_is_invertible,
			is_invertible,
			out,
			real_three_way_comparison,
			real_min,
			real_max,
			rational_min,
			rational_max,
			min,
			max,
			three_way_comparison,
			rational_multipliable,
			rational_modulus, rational_abs,
			argument, phase,
			real_plus,
			real_minus,
--			real_opposite,
			real_product,
			real_quotient,
			real_reciprocal, real_inverse,
			power,
			square_root,
			value_is_infinite,
			value_is_finite,
			gcd,
			div,
			rem,
			converted_integer,
			integer_product_overflows
		end

	DEBUG_OUTPUT
		rename
			debug_output as out
		undefine
			is_equal
		redefine
			default_create,
			out
		end

create
	default_create,
	make,
	make_from_native_integer_ref,
	make_from_integer,
	make_from_native_natural,
	make_from_native_natural_ref
	--	make_from_natural

create {REAL_NUMBER, RATIONAL_NUMBER, INTEGER_NUMBER, INTEGER_UNIVERSE}
	make_from_stored_value

convert
	make ({INTEGER}),
	make_from_native_integer_ref ({INTEGER_REF}),
	make_from_integer ({STS_INTEGER_NUMBER}),
	make_from_native_natural ({NATURAL}),
	make_from_native_natural_ref ({NATURAL_32_REF}),
		--	make_from_natural ({STS_NATURAL}),
	value: {INTEGER, INTEGER_REF, NUMERIC, COMPARABLE, HASHABLE},
	as_rational: {RATIONAL_NUMBER},
	as_real: {REAL_NUMBER},
	real_value: {REAL, REAL_REF},
	as_complex: {COMPLEX_NUMBER}

feature {NONE} -- Initialization

	default_create
			-- Create an integer number with value = 0
		do
			stored_value := 0
		ensure then
			value: value = 0
		end

	make (v: like value)
			-- Create an integer number out of value `v'.
		local
			trimmed_v: like stored_value
		do
			trimmed_v := v.as_integer_8 & Stored_value_mask
			stored_value := (trimmed_v ⊕ Native_min_value) - Native_min_value
		ensure
			value: value = adjusted_value (v)
		end

	make_from_stored_value (sv: like stored_value)
			-- Create an integer number with `value' equal to `sv'
		require
			big_enough: Native_min_value ≤ sv
			small_enough: sv ≤ Native_max_value
		do
			stored_value := sv
		ensure
			stored_value: stored_value = sv
		end

	make_from_native_integer_ref (v_ref: like native_integer_ref_anchor)
			-- Create an integer number out of value `v_ref'.`item'.
		do
			make (v_ref.item)
		ensure
			value: value = adjusted_value (v_ref.item)
		end

	make_from_integer (i: STS_INTEGER_NUMBER)
			-- Create an integer number with value `i'.`value'.
		do
			if attached {INTEGER_NUMBER} i as exp_i then
				stored_value := exp_i.stored_value
			else
				make (i.value)
			end
		ensure
			adjusted_value: value = adjusted_value (i.value)
		end

	make_from_native_natural (v: like Native_natural_anchor)
			-- Create an integer number with value `v'.`as_integer_32'.
		do
			make (v.as_integer_32)
		ensure
			value: value = adjusted_value (v.as_integer_32)
		end

	make_from_native_natural_ref (v_ref: like native_natural_ref_anchor)
			-- Create an integer number with value `v_ref'.`as_integer_32'.
		do
			make_from_native_natural (v_ref.item)
		ensure
			value: value = adjusted_value (v_ref.as_integer_32)
		end

		--	make_from_natural (n: STS_NATURAL)
		--			-- Create an integer number with value `n'.`integer_value'.
		--		do
		--			make_from_integer (n)
		--		ensure
		--			value: value = adjusted_value (n.integer_value)
		--		end

feature -- Primitive

	real_value: like native_real_anchor
			-- <Precursor>
		do
			Result := stored_value
		end

	value: like native_integer_anchor
			-- <Precursor>
		do
			Result := stored_value
		end

feature -- Access

	sign_bit: like one
			-- <Precursor>
		do
			if stored_value < 0 then
				Result := One
			end
		end

	sign: like integer_anchor
			-- <Precursor>
		do
				check
					not_too_small: Native_min_value ≤ stored_value.sign -- -1 ≤ stored_value.sign
					not_too_big: stored_value.sign ≤ Native_max_value -- stored_value.sign ≤ 1
				end
			create Result.make_from_stored_value (stored_value.sign)
		end

	Zero: INTEGER_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	One: INTEGER_NUMBER
			-- <Precursor>
		once
				check
					not_too_small: Native_min_value ≤ 1 -- By definition
					not_too_big: 1 ≤ Native_max_value -- By definition
				end
			create Result.make_from_stored_value (1)
		ensure then
			class
		end

	Two: INTEGER_NUMBER
			-- <Precursor>
		once
				check
					not_too_small: Native_min_value ≤ 2 -- By definition
					not_too_big: 1 ≤ Native_max_value -- By definition
				end
			create Result.make_from_stored_value (2)
		ensure then
			class
		end

	imaginary_unit: COMPLEX_NUMBER
			-- <Precursor>
		once
			Result := {COMPLEX_NUMBER}.I
		ensure then
			class
		end

	Min_value: INTEGER_NUMBER
			-- Minimum value representable by this implementation of integer numbers
		require else
			Min_value_exists: True
		once
				check
					not_too_small: Native_min_value ≤ Native_min_value -- By definition
					not_too_big: Native_min_value ≤ Native_max_value -- By definition
				end
			create Result.make_from_stored_value (Native_min_value)
		ensure then
			class
			definition: Result ≍ (- Max_value - One)
		end

	Max_value: INTEGER_NUMBER
			-- Maximum value representable by this implementation of integer numbers
		require else
			Max_value_exists: True
		once
				check
					not_too_small: Native_min_value ≤ Native_max_value -- By definition
					not_too_big: Native_max_value ≤ Native_max_value -- By definition
				end
			create Result.make_from_stored_value (Native_max_value)
		ensure then
			class
			definition: Result.value = 2 ^ (stored_value_bits - 1) - 1
		end

	Machine_epsilon: REAL_NUMBER
			-- <Precursor>
		once
			Result := {REAL_NUMBER}.machine_epsilon
		ensure then
			class
		end

	ulp,
	relative_epsilon: like real_anchor
			-- <Precursor>
		do
			Result := as_real.ulp
		end

	previous_float: like real_anchor
			-- <Precursor>
		do
			Result := as_real.previous_float
		ensure then
			definition: Result ≍ as_real.previous_float
		end

	next_float: like real_anchor
			-- <Precursor>
		do
			Result := as_real.next_float
		ensure then
			definition: Result ≍ as_real.next_float
		end

feature -- Quality

	is_real: BOOLEAN = True
			-- <Precursor>

	is_proper_real: BOOLEAN
			-- <Precursor>
		do
			Result := stored_value /= 0
		end

	is_imaginary: BOOLEAN
			-- <Precursor>
		do
			Result := stored_value = 0
		end

	is_proper_imaginary: BOOLEAN = False
			-- <Precursor>

	is_proper_complex: BOOLEAN = False
			-- <Precursor>

	is_nan: BOOLEAN = False
			-- <Precursor>
			-- A two's-complement representation of integer numbers has no NaN.

	is_negative_infinity: BOOLEAN = False
			-- <Precursor>
			-- A two's-complement representation of integer numbers has no infinity.

	is_negative_zero: BOOLEAN = False
			-- <Precursor>

	is_positive_zero: BOOLEAN
			-- <Precursor>
			-- A two's-complement representation of integer numbers has no negative zero.
		do
			Result := stored_value = 0
		end

	is_positive_infinity: BOOLEAN = False
			-- <Precursor>
			-- A two's-complement representation of integer numbers has no infinity.

	is_infinite: BOOLEAN = False
			-- <Precursor>
			-- A two's-complement representation of integer numbers has no infinity.

	is_finite: BOOLEAN = True
			-- <Precursor>
			-- A two's-complement representation of integer numbers has no infinity.

	is_rational: BOOLEAN = True
			-- <Precursor>

	is_integer: BOOLEAN = True
			-- <Precursor>

	real_is_invertible: BOOLEAN = True
			-- <Precursor>
			-- An implementation akin to IEEE 754 inverts zero with no major consequence.

	is_invertible: BOOLEAN
			-- <Precursor>
		do
			Result := stored_value /= 0
		end

	min_value_exists: BOOLEAN = True
			-- <Precursor>

	max_value_exists: BOOLEAN = True
			-- <Precursor>

feature -- Output

	out: STRING
			-- <Precursor>
		do
			Result := stored_value.out
		ensure then
			definition: Result ~ stored_value.out
		end

feature -- Comparison

	real_three_way_comparison (x: STS_REAL_NUMBER): like Integer_anchor
			-- <Precursor>
		local
			v: like Real_anchor.value
		do
			v := x.value
			if stored_value < v then
				Result := - One
			elseif v < stored_value then
				Result := One
			end
		end

	real_min (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			create Result.make (stored_value ∧ x.value)
		end

	real_max (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			create Result.make (stored_value ∨ x.value)
		end

	rational_min (pq: STS_RATIONAL_NUMBER): like Rational_anchor
			-- <Precursor>
		do
			Result := as_rational ∧ pq
		end

	rational_max (pq: STS_RATIONAL_NUMBER): like Rational_anchor
			-- <Precursor>
		do
			Result := as_rational ∨ pq
		end

	three_way_comparison alias "⋚" (i: STS_INTEGER_NUMBER): like integer_anchor
			-- <Precursor>
		local
			v: like value
		do
			v := value ⋚ i.value
				check
					not_too_small: Native_min_value ≤ v.as_integer_8 -- Native_min_value ≤ -1
					not_too_big: v.as_integer_8 ≤ Native_max_value -- 1 ≤ Native_max_value
				end
			create Result.make_from_stored_value (v.as_integer_8)
		end

	min alias "∧" (i: STS_INTEGER_NUMBER): like integer_anchor
			-- <Precursor>
		do
			if attached {INTEGER_NUMBER} i as exp_i then
					check
						not_too_small: -- Native_min_value ≤ stored_value and Native_min_value ≤ exp_i.stored_value
							Native_min_value ≤ (stored_value ∧ exp_i.stored_value)
						not_too_big: -- stored_value ≤ Native_max_value and exp_i.stored_value ≤ Native_max_value
							(stored_value ∧ exp_i.stored_value) ≤ Native_max_value
					end
				create Result.make_from_stored_value (stored_value ∧ exp_i.stored_value)
			else
				create Result.make (stored_value ∧ i.value)
			end
		end

	max alias "∨" (i: STS_INTEGER_NUMBER): like integer_anchor
			-- <Precursor>
		do
			if attached {INTEGER_NUMBER} i as exp_i then
					check
						not_too_small: -- Native_min_value ≤ stored_value and Native_min_value ≤ exp_i.stored_value
							Native_min_value ≤ (stored_value ∨ exp_i.stored_value)
						not_too_big: -- stored_value ≤ Native_max_value and exp_i.stored_value ≤ Native_max_value
							(stored_value ∨ exp_i.stored_value) ≤ Native_max_value
					end
				create Result.make_from_stored_value (stored_value ∨ exp_i.stored_value)
			else
				create Result.make (stored_value ∨ i.value)
			end
		end

feature -- Relationship

	rational_multipliable (pq: STS_RATIONAL_NUMBER): BOOLEAN
			-- May current integer number be multiplied by `pq`?
		do
			Result := True
		end

feature -- Operation

	argument,
	phase: like real_anchor
			-- <Precursor>
		do
			if stored_value < 0 then
				create Result.make (Pi)
			end
		end

	conjugate: like complex_anchor
			-- <Precursor>
		do
			Result := as_complex.conjugate
		end

	complex_plus (z: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex + z
		end

	complex_minus (z: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex - z
		end

	complex_product (z: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex ⋅ z
		end

	complex_quotient (z: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
		do
				check
					good_divisor: as_complex.divisible (z) -- divisible (z)
				end
			Result := as_complex / z
		end

	rational_modulus,
	rational_abs: like rational_anchor
			-- <Precursor>
		do
			Result := as_rational.abs
		end

	real_plus (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			create Result.make (stored_value + x.value)
		end

	real_minus (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			create Result.make (stored_value - x.value)
		end

--	real_opposite: like real_anchor
--			-- <Precursor>
--		do
--			create Result.make (- stored_value.to_real)
--		end

	real_product (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			create Result.make (stored_value × x.value)
		end

	real_quotient (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
				check
					ignore_precondition: stored_value.to_real.divisible (x.value) implies True
				end
			create Result.make (stored_value / x.value)
		end

	real_reciprocal,
	real_inverse: like Real_anchor
			-- <Precursor>
		require else
			always_divisible: True
		do
				check
					good_divisor_ignored: ({REAL} 1.0).divisible (stored_value.to_real) implies True
				end
			create Result.make ({REAL} 1.0 / stored_value)
		end

	power alias "^" (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			create Result.make_from_native_double_real (stored_value ^ x.value)
		end

	square_root alias "√¯": like real_anchor
			-- <Precursor>
		do
			create Result.make ({like Real_math_anchor}.sqrt (stored_value))
		end

	rational_opposite: like rational_anchor
			-- <Precursor>
		do
			Result := - as_rational
		end

	modulus,
	abs: like integer_anchor
			-- <Precursor>
		do
			create Result.make (stored_value.abs)
		end

	plus alias "+" (i: STS_INTEGER_NUMBER): like integer_anchor
			-- <Precursor>
		do
			create Result.make (stored_value + i.value)
		end

	minus alias "-" alias "−" (i: STS_INTEGER_NUMBER): like integer_anchor
			-- <Precursor>
		do
			create Result.make (stored_value - i.value)
		end

	opposite alias "-" alias "−": like integer_anchor
			-- <Precursor>
		do
			create Result.make (- stored_value)
		end

	product alias "*" alias "×" alias "⋅" (i: STS_INTEGER_NUMBER):
		like integer_anchor
			-- <Precursor>
		do
			create Result.make (stored_value × i.value)
		end

	quotient alias "/" alias "÷" (i: STS_INTEGER_NUMBER): like rational_anchor
			-- <Precursor>
		do
			create Result.make (Current, i)
		ensure then
			el_gcd: attached gcd (Current, i) as el_gcd
			non_zero_gcd: el_gcd ≭ Zero -- `i' /= 0
			Current_good_divisor: Current.divisible (el_gcd) -- el_gcd ≭ Zero
			i_good_divisor: i.divisible (el_gcd) -- el_gcd ≭ Zero
			el_p: attached (Current // el_gcd) as el_p
			el_q: attached (i // el_gcd) as el_q
			p_as_it_is: el_q > Zero or
				el_p ≍ {INTEGER_NUMBER}.Min_value or
				el_q ≍ {INTEGER_NUMBER}.Min_value implies
				Result.p ≍ el_p
			q_as_it_is: el_q > Zero or
				el_p ≍ {INTEGER_NUMBER}.Min_value or
				el_q ≍ {INTEGER_NUMBER}.Min_value implies
				Result.q ≍ el_q
			negated_p: el_q < Zero and
				el_p ≭ {INTEGER_NUMBER}.Min_value and
				el_q ≭ {INTEGER_NUMBER}.Min_value implies
				Result.p ≍ - el_p
			negated_q: el_q < Zero and
				el_p ≭ {INTEGER_NUMBER}.Min_value and
				el_q ≭ {INTEGER_NUMBER}.Min_value implies
				Result.q ≍ - el_q
		end

	integer_quotient alias "//" (i: STS_INTEGER_NUMBER): like integer_anchor
			-- <Precursor>
		do
				check
--					good_divisor: stored_value.to_integer_32.divisible (i.value) -- i.value /= 0 ⇐ divisible (i) -- TODO: Segmentation violation.
					good_divisor: i.value /= 0
				end
			create Result.make (stored_value // i.value)
		end

	integer_remainder alias "\\" (i: STS_INTEGER_NUMBER): like integer_anchor
			-- <Precursor>
		do
				check
--					good_divisor: stored_value.to_integer_32.divisible (i.value) -- i.value /= 0 ⇐ divisible (i) -- TODO: Segmentation violation.
					good_divisor: i.value /= 0
				end
			create Result.make (stored_value \\ i.value) -- TODO: make_from_stored_value?
		end

feature -- Conversion

	as_complex: like complex_anchor
			-- Current integer number represented as a complex number
		do
			create Result.make_from_integer (Current)
		ensure
			real: Result.a ≍ Current
			Imaginary: Result.b ≍ Zero
		end

	as_real: like real_anchor
			-- Current integer number represented as a real number
		do
			create Result.make_from_native_integer (stored_value)
		ensure
			value: Result.value = real_value
		end

	as_rational: like Rational_anchor
			-- Current integer number represented as a rational number
		do
			create Result.make_from_integer (Current)
		ensure
			numerator: Result.p ≍ Current
			denominator: Result.q ≍ One
		end

feature -- Math

	value_is_infinite (v: like real_value): BOOLEAN
			-- <Precursor>
		do
			Result := {REAL_NUMBER}.value_is_infinite (v)
		end

	value_is_finite (v: like real_value): BOOLEAN
			-- <Precursor>
		do
			Result := {REAL_NUMBER}.value_is_finite (v)
		end

	hypotenuse (x, y: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			Result := {COMPLEX_NUMBER}.hypotenuse (x, y)
		ensure then
			class
		end

	arc_tangent_yx (y, x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
			Result := {COMPLEX_NUMBER}.arc_tangent_yx (y, x)
		ensure then
			class
		end

	splitted (q: STS_REAL_NUMBER): HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- <Precursor>
		do
			Result := as_real.splitted (q)
		end

	gcd (i, j: STS_INTEGER_NUMBER): like Integer_anchor
			-- <Precursor>
		do
			Result := {RATIONAL_NUMBER}.gcd (i, j)
		ensure then
			class
		end

	div (i, j: STS_INTEGER_NUMBER): like Integer_anchor
			-- <Precursor>
		do
			Result := {RATIONAL_NUMBER}.div (i, j)
		ensure then
			class
		end

	rem (i, j: STS_INTEGER_NUMBER): like Integer_anchor
			-- <Precursor>
		do
			Result := {RATIONAL_NUMBER}.rem (i, j)
		ensure then
			class
		end

feature -- Factory

	real_from_value (v: like real_value): like real_anchor
			-- <Precursor>
		do
			Result := {COMPLEX_NUMBER}.real_from_value (v)
		ensure then
			class
		end

	real_from_double_value (dv: like real_value.to_double): like real_anchor
			-- <Precursor>
		do
			Result := {COMPLEX_NUMBER}.real_from_double_value (dv)
		ensure then
			class
		end

	converted_integer (i: STS_INTEGER_NUMBER): like integer_anchor
			-- <Precursor>
		do
			Result := {RATIONAL_NUMBER}.converted_integer (i)
		ensure then
			class
		end

feature -- Predicate

	integer_product_overflows (i, j: STS_INTEGER_NUMBER): BOOLEAN
			-- <Precursor>
		do
			Result := {RATIONAL_NUMBER}.integer_product_overflows (i, j)
		ensure then
			class
		end

feature -- Implementation

	real_adjusted_value (v: like real_value): like real_value
			-- <Precursor>
		do
			Result := {REAL_NUMBER}.adjusted_value (v)
		ensure then
			class
			definition: Result = {REAL_NUMBER}.adjusted_value (v)
		end

	adjusted_value (v: like value): like value
			-- <Precursor>
			--| I'd like to thank KamilCuk (https://stackoverflow.com/users/9072753/kamilcuk) for helping me with this little bit-brushing trick, as seen at
			--| https://stackoverflow.com/questions/66955285/how-to-simulate-twos-complement-arithmetics-upon-5-bit-representation/66993332#66993332.
			--| Please see also https://graphics.stanford.edu/%7Eseander/bithacks.html#VariableSignExtend.
		local
			trimmed_v, sign_extended_v: like stored_value
		do
			trimmed_v := v.as_integer_8 & Stored_value_mask
			sign_extended_v := (trimmed_v ⊕ Native_min_value) - Native_min_value
			Result := sign_extended_v
		ensure then
			class
			definition: Result = (((v.as_integer_8 & Stored_value_mask) ⊕ Native_min_value) - Native_min_value)
		end

feature -- Anchor

	complex_anchor: COMPLEX_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	real_anchor: REAL_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	rational_anchor: RATIONAL_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	integer_anchor: INTEGER_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	native_integer_ref_anchor: like native_integer_anchor.to_reference
			-- Anchor for a reference to a native integer number
		do
			create Result
		ensure
			class
		end

	native_natural_anchor: NATURAL
			-- Anchor for a native natural number
		once
		ensure
			class
		end

	native_natural_ref_anchor: like native_natural_anchor.to_reference
			-- Anchor for a reference to a native natural number
		do
			create Result
		ensure
			class
		end

feature {INTEGER_NUMBER, INTEGER_UNIVERSE} -- Implementation

	stored_value: INTEGER_8
			-- Bit pattern of the `value' of current integer number

feature {REAL_NUMBER, RATIONAL_NUMBER, INTEGER_UNIVERSE} -- Implementation

	native_min_value: like stored_value = -8
			-- Native minimum value representable by this implementation of integer numbers

	native_max_value: like stored_value = 7
			-- Native maximum value representable by this implementation of integer numbers

feature {INTEGER_UNIVERSE} -- Implementation

	stored_value_bits: NATURAL_8 = 4
			-- Number of significant bits in `stored_value'

	stored_value_mask: like stored_value = 0b1111
			-- Binary mask for the (stored) value of an integer number

invariant
	zero_real: Zero.identity.real_value = 0
	zero_imaginary: Zero.Zero.real_value = 0
	one_real: One.identity.real_value = 1
	one_imaginary: One.Zero.real_value = 0
	Two: Two.value = 2
	Imaginary_unit_real: Imaginary_unit.real ≍ Zero.identity
	Imaginary_unit_imaginary: Imaginary_unit.imaginary ≍ One.identity
--	Min_value: Min_value ≍ (- Max_value - One) -- TODO: It fails when `Max_value' is the first once routine to be called in {INTEGER_NUMBER}.
--	Max_value: Max_value.value = 2 ^ (stored_value_bits - 1) - 1 -- TODO: It fails when `Max_value' is the first once routine to be called in {INTEGER_NUMBER}.
	Machine_epsilon: Machine_epsilon ≍ One.relative_epsilon
	is_real_definition: Is_real = Zero ≍ Zero
	is_proper_imaginary_definition: Is_proper_imaginary = (is_imaginary and Zero ≭ Zero)
	is_proper_complex_definition: Is_proper_complex = (not is_real and not is_imaginary)
	is_nan_definition: Is_nan = real_value.is_nan
	total_order: Is_nan implies Current ≍ Current
	is_negative_infinity_definition: Is_negative_infinity = real_value.is_negative_infinity
	is_negative_zero_definition: is_negative_zero = (Current ≍ Zero and sign_bit ≍ One)
	is_positive_infinity_definition: Is_positive_infinity = real_value.is_positive_infinity
	is_infinite_complex_definition: Is_infinite = (identity.Is_infinite or Zero.Is_infinite)
	is_infinite_real_definition: Is_infinite = (Is_negative_infinity or Is_positive_infinity)
	is_finite_complex_definition: Is_finite = (identity.Is_finite and Zero.Is_finite)
	is_finite_real_definition: Is_finite = not (Is_nan or Is_infinite)

		-- is_rational
	is_rational_definition: is_rational = (is_real and identity.is_rational)
	splitted_one: attached splitted (one) as splitted_one
	a: attached splitted_one.a as a
	b: attached splitted_one.b as b
	is_rational_real_definition: is_rational = (
		(a.is_integer or zero.max_value_exists and then (a - one) ≍ zero.max_value) and
		(b.is_integer or zero.max_value_exists and then (b - one) ≍ zero.max_value)
		)

	is_integer_complex_definition: Is_integer = (Is_real and identity.Is_integer)
	is_integer_real_definition: Is_integer = Current.real_equals (identity)
		good_divisor: identity.divisible (One) -- By definition
	is_integer_rational_definition: Is_integer = (identity \\ One) ≍ Zero

		-- real_is_invertible
	scale_log: attached value_logb ((identity.rational_abs.real_max (zero.rational_abs)).value) as scale_log
	int_scale_log: attached if value_is_finite (scale_log) then
			scale_log.truncated_to_integer
		else
			0
		end as int_scale_log
	scaled_a: attached (identity.real_value * 2 ^ (- int_scale_log)) as scaled_a
	scaled_b: attached (zero.real_value * 2 ^ (- int_scale_log)) as scaled_b
	denom: attached (scaled_a ^ 2 + scaled_b ^ 2) as denom
	real_is_invertible_quasi_definition: denom /= 0 implies real_is_invertible
	real_is_invertible_real_quasi_definition: Current.real_unequals (zero) implies real_is_invertible

	Native_min_value: Native_min_value = - Native_max_value - 1
	Native_max_value: Native_max_value = 2 ^ (Stored_value_bits - 1) - 1
	Stored_value_bits: Stored_value_bits = {REAL_NUMBER}.Mantissa_width + 2
	Stored_value_mask: Stored_value_mask = (2 ^ Stored_value_bits - 1).truncated_to_integer

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
