﻿note
	description: "Implementation of {STS_COMPLEX_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	COMPLEX_UNIVERSE

inherit
	STS_COMPLEX_UNIVERSE
		rename
			Cardinality as Cardinality
		undefine
			with,
			without,
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_subset,
			is_trivial_superset,
			is_proper_subset,
			is_proper_superset,
			is_disjoint,
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
			complemented,
			intersected,
			subtracted,
			subtracted_symmetricaly,
			crossed_tuples,
			equality_holds
		redefine
			default_create,
			is_singleton,
			Cardinality,
			out,
			without_zero,
			trivial_subsets,
			proper_subsets,
			complement,
			squared,
			diagonal
		end

	UNIVERSE [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
		undefine
			eq,
			subtracted,
			o, empty_set,
			singleton,
			converted_element,
			unit,
			any_anchor,
			set_anchor,
			subset_anchor,
			element_to_element,
			element_to_natural,
			element_to_boolean,
			element_to_tuple,
			element_to_sets,
			tuple_extensor
		redefine
			default_create,
			make_extended,
			make_from_set,
			any,
			is_singleton,
			Cardinality,
			out,
			subsets,
			powerset,
			trivial_subsets,
			proper_subsets,
			squared,
			diagonal,
			as_singleton,
			as_tuple
		end

	COMPLEX_SET
		rename
			Cardinality as Cardinality,
			superset_anchor as itself
		undefine
			make_empty,
			make_singleton,
			is_empty,
			has,
			with,
			extended,
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_subset,
			is_trivial_superset,
			is_proper_subset,
			is_proper_superset,
			is_disjoint,
			complemented,
			united,
			intersected,
			subtracted_symmetricaly,
			itself
		redefine
			default_create,
			make_extended,
			make_from_set,
			make_from_numeric_set,
			any,
			is_singleton,
			Cardinality,
			out,
			without_zero,
			subsets,
			powerset,
			trivial_subsets,
			proper_subsets,
			complement,
			squared,
			diagonal,
			as_singleton,
			as_tuple,
			as_set_of_pairs
		end

create
	default_create,
	make_from_universe

convert
	make_from_universe ({
		STS_UNIVERSE [STS_COMPLEX_NUMBER, STS_NUMERIC_EQUALITY [STS_COMPLEX_NUMBER]],
		UNIVERSE [STS_COMPLEX_NUMBER, STS_NUMERIC_EQUALITY [STS_COMPLEX_NUMBER]],

		STS_UNIVERSE [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY],
		UNIVERSE [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY],

		STS_COMPLEX_UNIVERSE
		}),
	as_set_of_pairs: {
		SET_OF_ORDERED_PAIRS [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY],
		STS_SET_OF_ORDERED_PAIRS [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY],

		SET_OF_ORDERED_PAIRS [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER], STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]],
		STS_SET_OF_ORDERED_PAIRS [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER], STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]
		}

feature {NONE} -- Initialization

	default_create
			-- Create the universe of complex numbers, i.e. ℂ.
		local
			real_numbers, r1, r2: like Real_universe.others
			z: like any_anchor
		do
			from
				others := O
					check
						is_disjoint: R.O.is_disjoint (R) -- R.O.is_empty
					end
				real_numbers := R.O.batch_extended (R)
				r1 := real_numbers
			invariant
				building_up: others ≍ {
					TRANSFORMER [
						STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
						STS_COMPLEX_NUMBER,
						STS_ORDERED_PAIR_EQUALITY [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
						STS_COMPLEX_EQUALITY
						]
					}.set_map ((R ∖ r1) × R, agent complex_from_pair)
			until
				r1.is_empty
			loop
				from
					r2 := real_numbers
				invariant
					r1_is_not_empty: not r1.is_empty -- Outer loop is not finished.
					still_building_up: others ≍ (
						{
							TRANSFORMER [
								STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
								STS_COMPLEX_NUMBER,
								STS_ORDERED_PAIR_EQUALITY [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
								STS_COMPLEX_EQUALITY
								]
							}.set_map ((R ∖ r1) × R, agent complex_from_pair) ∪ {
								TRANSFORMER [
									STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
									STS_COMPLEX_NUMBER,
									STS_ORDERED_PAIR_EQUALITY [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
									STS_COMPLEX_EQUALITY
									]
								}.set_map (
									{
										TRANSFORMER [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]
										}.singleton_crossed_set (r1.any, R ∖ r2), agent complex_from_pair
									)
						)
				until
					r2.is_empty
				loop
						check
								-- Loops are not finished.
							r1_is_not_empty: not r1.is_empty
							r2_is_not_empty: not r2.is_empty
						end
					create z.make (r1.any, r2.any)
						check
							does_not_have: others ∌ z -- loop invariant
						end
					others := others.extended (z)
					r2 := r2.others
				variant
					r2_n: natural_as_integer (# r2)
				end
				r1 := r1.others
			variant
				r1_n: natural_as_integer (# r1)
			end
				check
					is_not_empty: not others.is_empty -- not R.is_empty
				end
			z := others.any
			others := others.others
			create any_storage.put (z)
		ensure then
			definition: Current ≍ {
				TRANSFORMER [
					STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
					STS_COMPLEX_NUMBER,
					STS_ORDERED_PAIR_EQUALITY [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
					STS_COMPLEX_EQUALITY
					]
				}.set_map (× R, agent complex_from_pair)
		end

	make_extended (a: STS_COMPLEX_NUMBER; s: STS_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY])
			-- Turn `any' and `others', respectively, into `a` and `s`.
		do
			check
					-- NOTICE: Make these hold!
				same_any: a ≍ Any -- Otherwise we must give up of keeping `Any' as a once query.
				almost_the_universe: # s = Instance_free_cardinality - 1
			then
				check
					is_not_empty_already: not Is_empty
					any_already_defined: Eq (Any, a)
					others_already_defined: others ≍ s
				end
			end
		end

	make_from_set (s: STS_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY])
			-- Turn current universe equal to `s`.
			-- TODO: Re-test it.
		do
			check
				the_universe: # s = # Current
			then
				check
					nothing_to_do: Current ≍ s
				end
			end
		end

	make_from_numeric_set (s: STS_SET [STS_COMPLEX_NUMBER, STS_NUMERIC_EQUALITY [STS_COMPLEX_NUMBER]])
			-- Turn current universe equal to `s`.
		do
			check
				the_universe: # s = # Current
			then
				check
					nothing_to_do: s ≍ Current
				end
			end
		end

	make_from_universe (univ: STS_UNIVERSE [STS_COMPLEX_NUMBER, STS_NUMERIC_EQUALITY [STS_COMPLEX_NUMBER]])
			-- Create a universe equal to `univ'.
		do
			others := O.numeric_converted (univ.others)
				check
					is_not_empty: not univ.is_empty -- `univ' is a universe.
				end
			create any_storage.put (converted_element (univ.any))
		ensure
			same_universe: univ ≍ Current
		end

feature -- Primitive

	any: COMPLEX_NUMBER
			-- An arbitrary representable complex number
		once
			Result := Precursor {COMPLEX_SET}
		end

feature -- Quality

	is_singleton: BOOLEAN = False
			-- Does current universe have exactly one element?

feature -- Measurement

	cardinality: NATURAL
			-- <Precursor>
		once
			Result := Precursor {UNIVERSE}
		end

	instance_free_cardinality alias "#": NATURAL
			-- Number of elements in current kind of universe
		once
			Result := {REAL_UNIVERSE}.Instance_free_cardinality * {REAL_UNIVERSE}.Instance_free_cardinality
		ensure then
			class
			definition: Result = (# R) ^ 2
		end

feature -- Output

	out: STRING
			-- <Precursor>
		once
			Result := Precursor {COMPLEX_SET}
		end

feature -- Restriction

	without_zero: COMPLEX_SET
			-- Every representable complex number but `zero'
		once
			Result := Precursor {COMPLEX_SET}
		end

feature -- Operation

	subsets: SUBSET_FAMILY [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
			-- Every possible representable set of complex numbers
		once
			Result := Precursor {COMPLEX_SET}
		end

	powerset: SUBSET_FAMILY [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
			-- Every possible representable set of complex numbers
		once
			Result := Precursor {COMPLEX_SET}
		end

	trivial_subsets: SUBSET_FAMILY [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
			-- Family with only the empty set and the universe of complex numbers
		once
			Result := Precursor {COMPLEX_SET}
		end

	proper_subsets: SUBSET_FAMILY [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
			-- Every non-trivial subset of current universe
		once
			Result := Precursor {COMPLEX_SET}
		end

	complement alias "∁": COMPLEX_SET
			-- <Precursor>
		once
			Result := O
		end

	squared alias "×": SET_OF_ORDERED_PAIRS [STS_COMPLEX_NUMBER, STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY, STS_COMPLEX_EQUALITY]
			-- Current universe crossed with itself, i.e. every representable pair in ℂ².
		once
			Result := Precursor {COMPLEX_SET}
		end

	diagonal alias "∆": SET_OF_ORDERED_PAIRS [STS_COMPLEX_NUMBER, STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY, STS_COMPLEX_EQUALITY]
			-- Every representable complex number z, mapped to the respective identical pair (z,z), i.e. {(z,z) | z ∈ ℂ}
		once
			Result := Precursor {COMPLEX_SET}
		end

feature -- Conversion

	as_singleton: SUBSET_FAMILY [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
			-- Current universe alone into a singleton, i.e. {ℂ}.
		once
			Result := Precursor {COMPLEX_SET}
		end

	as_tuple: N_TUPLE [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
			-- Current complex universe converted to an n-tuple whose terms are ordered as they are accessed by current universe
			-- NOTICE: Please see the comment at {SET}.as_tuple.
		once
			Result := Precursor {COMPLEX_SET}
		end

	as_set_of_pairs: SET_OF_ORDERED_PAIRS [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]
			-- Current complex universe converted into a set of ordered pairs
		once
			Result := Precursor {COMPLEX_SET}
		end

feature -- Factory

	R,
	Real_universe: REAL_UNIVERSE
			-- Universe of all real numbers, i.e. ℝ.
		once
			Result := {REAL_SET}.R
		ensure
			class
		end

feature -- Mapping

	complex_from_pair (ab: STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]]): COMPLEX_NUMBER
			-- `ab' converted to a complex number
		do
			create Result.make_from_ordered_pair (ab)
		ensure
			class
			real: Result.a ≍ ab.a
			imaginary: Result.b ≍ ab.b
		end

invariant
	attached_stored_any: attached any_storage
	Any: Any ~ any_storage.item
	Is_singleton: Is_singleton = (not is_empty and others.is_empty)
	Instance_free_cardinality: # Current = (# R) ^ 2
	Without_zero:; -- TOO MUCH EXPENSIVE: Without_zero ≍ (Current / Zero)
--	Out_not_void: Out /= Void -- TODO: right_trimmed: VOID_TARGET raised (VOID_TARGET)
--	Out: Out ~ "{" + As_tuple.terms_out + "}" -- TODO: right_trimmed: VOID_TARGET raised (VOID_TARGET)
	Subsets:; -- INFEASIBLE: Subsets ≍ Element_to_sets.set_reduction (Current, O.as_singleton, agent sets_plus_same_sets_with_element)
	Subsets_superset:; -- INFEASIBLE: Subsets.superset ≍ Current
	Powerset:; -- INFEASIBLE: Powerset ≍ Element_to_sets.set_reduction (Current, O.as_singleton, agent sets_plus_same_sets_with_element)
	Powerset_superset:; -- INFEASIBLE: Powerset.superset ≍ Current
	Trivial_subsets: Trivial_subsets ≍ (As_singleton & O)
	Trivial_subsets_superset: Trivial_subsets.superset ≍ Current
	Proper_subsets:; -- INFEASIBLE: Proper_subsets ≍ (Powerset / Current / O)
	Proper_subsets_superset:; -- INFEASIBLE: Proper_subsets.superset ≍ Current
	is_subset: Current ⊆ C -- `C' is current universe.
	complement: ∁ Current ≍ (Current ∁ C)
	Squared:; -- TOO MUCH EXPENSIVE: Squared ≍ (Current × Current)
	Diagonal:; -- TOO MUCH EXPENSIVE: Diagonal ≍ (× Current).filtered_xy (agent equality_holds)
	As_tuple:; -- TOO MUCH EXPENSIVE: As_tuple ≍ Element_to_tuple.set_reduction (Current, Unit, agent {STS_N_TUPLE [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]}.before)
	As_singleton_has_current_set: Current ∈ As_singleton
	As_singleton_has_nothing_else: As_singleton.is_singleton
	As_singleton_superset: As_singleton.superset ≍ Current
	As_set_of_pairs_every_complex_number:; -- TOO MUCH EXPENSIVE: Current |∀ agent set_has_complex_number (As_set_of_pairs, ?)
	As_set_of_pairs_nothing_else: # As_set_of_pairs = Current.Cardinality

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
