﻿note
	description: "Implementation of {STS_INTEGER_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	INTEGER_UNIVERSE

inherit
	STS_INTEGER_UNIVERSE
		rename
			Cardinality as Cardinality
		undefine
			with,
			without,
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_subset,
			is_trivial_superset,
			is_proper_subset,
			is_proper_superset,
			is_disjoint,
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
			complemented,
			intersected,
			subtracted,
			subtracted_symmetricaly,
			crossed_tuples,
			equality_holds
		redefine
			default_create,
			is_singleton,
			cardinality,
			out,
			without_zero,
			non_positive,
			non_negative,
			negative,
			positive,
			unbounded_interval,
			trivial_subsets,
			proper_subsets,
			complement,
			squared,
			diagonal
		end

	UNIVERSE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
		undefine
			eq,
			subtracted,
			o, empty_set,
			converted_element,
			unit,
			any_anchor,
			set_anchor,
			subset_anchor,
			element_to_element,
			element_to_natural,
			element_to_boolean,
			element_to_tuple,
			element_to_sets,
			tuple_extensor
		redefine
			default_create,
			make_extended,
			make_from_set,
			any,
			is_singleton,
			cardinality,
			out,
			subsets,
			powerset,
			trivial_subsets,
			proper_subsets,
			squared,
			diagonal,
			as_singleton,
			as_tuple
		end

	INTEGER_SET
		rename
			Cardinality as Cardinality,
			superset_anchor as itself
		undefine
			make_empty,
			make_singleton,
			is_empty,
			has,
			with,
			extended,
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_subset,
			is_trivial_superset,
			is_proper_subset,
			is_proper_superset,
			is_disjoint,
			complemented,
			united,
			intersected,
			subtracted_symmetricaly,
			itself
		redefine
			default_create,
			make_extended,
			make_from_set,
			make_from_numeric_set,
			any,
			is_singleton,
			cardinality,
			out,
			without_zero,
			non_positive,
			non_negative,
			negative,
			positive,
			unbounded_interval,
			subsets,
			powerset,
			trivial_subsets,
			proper_subsets,
			complement,
			squared,
			diagonal,
			as_singleton,
			as_tuple
		end

create
	default_create,
	make_from_universe

convert
	make_from_universe ({
		STS_UNIVERSE [STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]],
		UNIVERSE [STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]],

		STS_UNIVERSE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY],
		UNIVERSE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY],

		STS_INTEGER_UNIVERSE
		}),
	as_complex_set: {
		COMPLEX_SET,
		STS_COMPLEX_SET,

		NUMERIC_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY],
		STS_NUMERIC_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
		},
	as_real_set: {
		REAL_SET,
		STS_REAL_SET,

		NUMERIC_SET [STS_REAL_NUMBER, STS_REAL_EQUALITY],
		STS_NUMERIC_SET [STS_REAL_NUMBER, STS_REAL_EQUALITY]
		}

feature {NONE} -- Initialization

	default_create
			-- Create the universe of integer numbers, i.e. ℤ.
		local
			sv: like Any_anchor.stored_value
			i: like Any_anchor
		do
			from
				create others.make_empty
				sv := {INTEGER_NUMBER}.Native_max_value
			invariant
				building_up: others ≍ {
					TRANSFORMER [
						like Any_anchor.stored_value, STS_INTEGER_NUMBER,
						STS_OBJECT_EQUALITY [like Any_anchor.stored_value], STS_INTEGER_EQUALITY
						]
					}.set_map (
						stored_value_interval (sv, {INTEGER_NUMBER}.Native_max_value) / sv,
						agent integer_from_stored_value
						)
			until
				sv = {INTEGER_NUMBER}.Native_min_value
			loop
					check
							-- Loop variant: {INTEGER_NUMBER}.Native_min_value < sv ≤ {INTEGER_NUMBER}.Native_max_value
						big_enough: {INTEGER_NUMBER}.Native_min_value ≤ sv
						small_enough: sv ≤ {INTEGER_NUMBER}.Native_max_value
					end
				create i.make_from_stored_value (sv)
					check
							-- Loop invariant: others = (stored_value_interval (sv, {INTEGER_NUMBER}.Native_max_value) / sv) ↦ agent integer_from_stored_value
						does_not_have: others ∌ i
					end
				others := others.extended (i)
				sv := sv - 1
			variant
				sv: sv - {INTEGER_NUMBER}.Native_min_value
			end
			create any_storage.put (create {INTEGER_NUMBER}.make_from_stored_value (sv))
		ensure then
			definition: Current ≍ (
				{
					TRANSFORMER [
						like Any_anchor.stored_value, STS_INTEGER_NUMBER,
						STS_OBJECT_EQUALITY [like Any_anchor.stored_value], STS_INTEGER_EQUALITY
						]
					}.set_map (
						stored_value_interval ({INTEGER_NUMBER}.Native_min_value, {INTEGER_NUMBER}.Native_max_value),
						agent integer_from_stored_value
						)
				)
		end

	make_extended (a: STS_INTEGER_NUMBER; s: STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY])
			-- Turn `any' and `others', respectively, into `a` and `s`.
		do
			check
					-- NOTICE: Make these hold!
				same_any: a ≍ Any -- Otherwise we must give up of keeping `Any' as a once query.
				almost_the_universe: # s = Instance_free_cardinality - 1
			then
				check
					is_not_empty_already: not Is_empty
					any_already_defined: Eq (Any, a)
					others_already_defined: others ≍ s
				end
			end
		end

	make_from_set (s: STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY])
			-- Turn current universe equal to `s`.
			-- TODO: Re-test it.
		do
			check
				the_universe: # s = # Current
			then
				check
					nothing_to_do: Current ≍ s
				end
			end
		end

	make_from_numeric_set (s: STS_SET [STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]])
			-- Turn current universe equal to `s`.
		do
			check
				the_universe: # s = # Current
			then
				check
					nothing_to_do: s ≍ Current
				end
			end
		end

	make_from_universe (univ: STS_UNIVERSE [STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]])
			-- Create a universe equal to `univ'.
			--| TODO: Possibly redundant, wrong and useless.
		do
			others := O.numeric_converted (univ.others)
				check
					is_not_empty: not univ.is_empty -- `univ' is a universe.
				end
			create any_storage.put (converted_element (univ.any))
		ensure
			same_universe: univ ≍ Current
		end

feature -- Primitive

	any: INTEGER_NUMBER
			-- An arbitrary representable integer number
		once
			Result := Precursor {UNIVERSE}
		end

feature -- Quality

	is_singleton: BOOLEAN = False
			-- Does current universe have exactly one element?

feature -- Measurement

	cardinality: NATURAL
			-- <Precursor>
		once
			Result := Precursor {UNIVERSE}
		end

	instance_free_cardinality alias "#": NATURAL
			-- Number of elements in current kind of universe
		once
			Result := ({INTEGER_NUMBER}.Native_max_value - {INTEGER_NUMBER}.Native_min_value + 1).as_natural_32
		ensure then
			class
			definition: Result = 2 ^ {INTEGER_NUMBER}.Stored_value_bits
		end

feature -- Output

	out: STRING
			-- <Precursor>
		once
			Result := Precursor {INTEGER_SET}
		end

feature -- Restriction

	without_zero: INTEGER_SET
			-- Every representable integer number but `zero'
		once
			Result := Precursor {INTEGER_SET}
		end

	non_positive: INTEGER_SET
			-- Every non-positive representable integer number
		once
			Result := Precursor {INTEGER_SET}
		end

	non_negative: INTEGER_SET
			-- Every non-negative representable integer number, i.e. every representable natural number.
		once
			Result := Precursor {INTEGER_SET}
		end

	negative: INTEGER_SET
			-- Every negative representable integer number
		once
			Result := Non_positive / Zero
		end

	positive: INTEGER_SET
			-- Every positive representable integer number
		once
			Result := Non_negative / Zero
		end

feature -- Interval

	unbounded_interval: INTEGER_UNIVERSE
			-- Current universe itself, i.e. ℤ.
		once
			Result := Current
		end

feature -- Operation

	subsets: SUBSET_FAMILY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- Every possible representable set of integer numbers
		once
			Result := Precursor {INTEGER_SET}
		end

	powerset: SUBSET_FAMILY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- Every possible representable set of integer numbers
		once
			Result := Precursor {INTEGER_SET}
		end

	trivial_subsets: SUBSET_FAMILY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- Family with only the empty set and the universe of integer numbers
		once
			Result := Precursor {INTEGER_SET}
		end

	proper_subsets: SUBSET_FAMILY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- Every non-trivial subset of current universe
		once
			Result := Precursor {INTEGER_SET}
		end

	complement alias "∁": INTEGER_SET
			-- <Precursor>
		once
			Result := O
		end

	squared alias "×": SET_OF_ORDERED_PAIRS [STS_INTEGER_NUMBER, STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY]
			-- Current universe crossed with itself, i.e. every representable pair in ℤ².
		once
			Result := Precursor {INTEGER_SET}
		end

	diagonal alias "∆": SET_OF_ORDERED_PAIRS [STS_INTEGER_NUMBER, STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY]
			-- Every representable integer number i, mapped to the respective identical pair (i,i), i.e. {(i,i) | i ∈ ℤ}
		once
			Result := Precursor {INTEGER_SET}
		end

feature -- Conversion

	as_singleton: SUBSET_FAMILY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- Current universe alone into a singleton, i.e. {ℤ}.
		once
			Result := Precursor {INTEGER_SET}
		end

	as_tuple: N_TUPLE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- Current universe converted to an n-tuple whose terms are ordered as they are accessed by current universe
			-- NOTICE: Please see the comment at {SET}.as_tuple.
		once
			Result := Precursor {INTEGER_SET}
		end

feature {INTEGER_UNIVERSE} -- Implementation

	i_is_within_interval (a, i, b: like Any_anchor.stored_value): BOOLEAN
			-- Is `i' within the interval {`a'..`b'}?
		do
			Result := a ≤ i and i ≤ b
		ensure
			class
			definition: Result = (a ≤ i and i ≤ b)
		end

feature {NONE} -- Implementation

	stored_value_interval
		(a, b: like Any_anchor.stored_value): SET [like Any_anchor.stored_value, STS_OBJECT_EQUALITY [like Any_anchor.stored_value]]
			-- Set of integer nunbers i, where `a' ≤ i ≤ `b'
		local
			i: like Any_anchor.stored_value
		do
			create Result.make_empty
			if a ≤ b then
				from
					i := b
					Result := Result.extended (i)
				invariant
					every_integer_number_within_interval: # Result = (b.as_integer_16 - i.as_integer_16  + 1).as_natural_32
					nothing_else: Result |∀ agent Z.i_is_within_interval (i, ?, b)
				until
					i = a
				loop
					i := i - 1
					Result := Result.extended (i)
				variant
					interval: i.as_integer_64 - a.as_integer_64
				end
			end
		ensure
			class
			when_interval_is_empty: a > b implies Result.is_empty
			every_integer_number_within_interval: a ≤ b implies # Result = (b.as_integer_16 - a.as_integer_16  + 1).as_natural_32
			nothing_else: Result |∀ agent Z.i_is_within_interval (a, ?, b)
		end

	integer_from_stored_value (sv: like Any_anchor.stored_value): like Any_anchor
			-- Integer number whose bit pattern equals `sv' & {INTEGER_NUMBER}.Value_stored_value_mask
		do
				check
						-- By definition
					big_enough: {INTEGER_NUMBER}.Native_min_value ≤ (
						((sv & {INTEGER_NUMBER}.Stored_value_mask) ⊕ {INTEGER_NUMBER}.Native_min_value) - {INTEGER_NUMBER}.Native_min_value
						)
					small_enough: (
						((sv & {INTEGER_NUMBER}.Stored_value_mask) ⊕ {INTEGER_NUMBER}.Native_min_value) - {INTEGER_NUMBER}.Native_min_value
						) ≤ {INTEGER_NUMBER}.Native_max_value
				end
			create Result.make_from_stored_value (
				((sv & {INTEGER_NUMBER}.Stored_value_mask) ⊕ {INTEGER_NUMBER}.Native_min_value) - {INTEGER_NUMBER}.Native_min_value
				)
		ensure
			definition: Result.stored_value = (
				((sv & {INTEGER_NUMBER}.Stored_value_mask) ⊕ {INTEGER_NUMBER}.Native_min_value) - {INTEGER_NUMBER}.Native_min_value
				)
		end

invariant
	attached_stored_any: attached any_storage
	Any: Any ~ any_storage.item
	Instance_free_cardinality: # Current = 2 ^ {INTEGER_NUMBER}.Stored_value_bits
	Without_zero: Without_zero ≍ (Current / Zero)
	Non_positive: Non_positive ≍ left_unbounded_right_closed_interval (Zero)
	Non_negative: Non_negative ≍ left_closed_right_unbounded_interval (Zero)
	Negative: Negative ≍ left_unbounded_right_open_interval (Zero)
	Positive: Positive ≍ left_open_right_unbounded_interval (Zero)
	Unbounded_interval: Unbounded_interval ≍ Current
	As_tuple: As_tuple ≍ Element_to_tuple.set_reduction (Current, Unit, agent {STS_N_TUPLE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]}.before)
	Is_singleton: Is_singleton = (not is_empty and others.is_empty)
	Out_not_void: Out /= Void
	Out: Out ~ "{" + As_tuple.terms_out + "}"
	As_singleton_has_current_set: Current ∈ As_singleton
	As_singleton_has_nothing_else: As_singleton.is_singleton
	As_singleton_superset: As_singleton.superset ≍ Current
	Subsets:; -- TOO MUCH EXPENSIVE: Subsets ≍ Element_to_sets.set_reduction (Current, O.as_singleton, agent sets_plus_same_sets_with_element)
--	Subsets_superset: Subsets.superset ≍ Current -- TODO: It makes {MEMORY}.objects_instance_of_type to crash!
	Powerset:; -- TOO MUCH EXPENSIVE: Powerset ≍ Element_to_sets.set_reduction (Current, O.as_singleton, agent sets_plus_same_sets_with_element)
--	Powerset_superset: Powerset.superset ≍ Current -- TODO: It makes {MEMORY}.objects_instance_of_type to crash!
	Trivial_subsets: Trivial_subsets ≍ (As_singleton & O)
	Trivial_subsets_superset: Trivial_subsets.superset ≍ Current
	Proper_subsets:; -- TOO MUCH EXPENSIVE: Proper_subsets ≍ (Powerset / Current / O)
	Proper_subsets_superset:; -- TOO MUCH EXPENSIVE: Proper_subsets.superset ≍ Current
	Squared: Squared ≍ (Current × Current)
	Diagonal: Diagonal ≍ (× Current).filtered_xy (agent equality_holds)
	is_subset: Current ⊆ Z -- `Z' is current universe.
	complement: ∁ Current ≍ (Current ∁ Z)

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
