﻿note
	description: "(Partial) implementation of {STS_NUMERIC_SET}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	NUMERIC_SET [A -> STS_COMPLEX_NUMBER, EQ -> STS_NUMERIC_EQUALITY [A] create default_create end]

inherit
	STS_NUMERIC_SET [A, EQ]
		undefine
			has,
			with,
			without,
			cardinality,
			out,
			is_trivial_subset,
			is_proper_subset,
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
			trivial_subsets,
			diagonal,
			crossed_tuples,
			equality_holds
		redefine
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_superset,
			is_proper_superset,
			is_disjoint,
			complemented,
			intersected,
			subtracted,
			subtracted_symmetricaly
		end

	SET [A, EQ]
		undefine
			make_singleton,
			make_extended,
			extended,
			set_anchor,
			subset_anchor,
			superset_anchor
		redefine
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_superset,
			is_proper_superset,
			is_disjoint,
			complemented,
			united,
			intersected,
			subtracted,
			subtracted_symmetricaly
		end

feature -- Access

	u,
	universe: UNIVERSE [A, EQ]
			-- <Precursor>
		deferred
		end

feature -- Comparison

	equals alias "≍" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			s_card: like cardinality
		do
			if Current = s or Current ~ s then
				Result := True
			else
				s_card := # s
				if s_card = # u then
					Result := # Current = s_card
				elseif # Current = s_card then
					Result := subset_relation_holds (Current, s)
				end
			end
		end

	is_subset alias "⊆" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			s_card: like cardinality
		do
			if Current = s or Current ~ s then
				Result := True
			else
				s_card := # s
				if s_card = # u then
					Result := True
				elseif # Current ≤ s_card then
					Result := subset_relation_holds (Current, s)
				end
			end
		end

	is_superset alias "⊇" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			card: like cardinality
		do
			if Current = s or Current ~ s then
				Result := True
			else
				card := # Current
				if card = # u then
					Result := True
				elseif card ≥ # s then
					Result := subset_relation_holds (s, Current)
				end
			end
		end

	is_comparable (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			card, s_card: like cardinality
		do
			if Current = s or Current ~ s then
				Result := True
			else
				s_card := # s
				if s_card = # u then
					Result := True
				else
					card := # Current
					if card ≤ s_card then
						Result := subset_relation_holds (Current, s)
					elseif card = # u then
						Result := True
					else
						Result := subset_relation_holds (s, Current)
					end
				end
			end
		end

	is_strict_subset alias "⊂" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			card, s_card: like cardinality
		do
			if Current /= s and Current /~ s then
				card := # Current
				if card < # u then
					s_card := # s
					if s_card = # u then
						Result := True
					elseif card < s_card then
						Result := subset_relation_holds (Current, s)
					end
				end
			end
		end

	is_strict_superset alias "⊃" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			card, s_card: like cardinality
		do
			if Current /= s and Current /~ s then
				s_card := # s
				if s_card < # u then
					card := # Current
					if card = # u then
						Result := True
					elseif s_card < card then
						Result := subset_relation_holds (s, Current)
					end
				end
			end
		end

	is_trivial_superset (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := s.is_empty or Current ≍ s
		end

	is_proper_superset (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := not s.is_empty and Current ⊃ s
		end

	is_disjoint (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			card, s_card: like cardinality
		do
			card := # Current
			if card = # u then
				Result := s.is_empty
			else
				s_card := # s
				if s_card = # u then
					Result := card = 0
				else
					Result := Precursor {SET}(s)
				end
			end
		end

feature -- Operation

	complemented alias "∁" (s: STS_SET [A, EQ]): like set_anchor
			-- <Precursor>
		do
			if # Current = # u then
				Result := o
			else
				Result := Precursor {SET}(s)
			end
		end

	united alias "∪" (s: STS_SET [A, EQ]): like superset_anchor
			-- <Precursor>
		do
			if # Current = # u or # s = # u then
				Result := converted (u)
			else
				Result := Precursor {SET}(s)
			end
		end

	intersected alias "∩" (s: STS_SET [A, EQ]): like subset_anchor
			-- <Precursor>
		do
			if # Current = # u then
				Result := converted (s)
			elseif # s = # u then
				Result := Current
			else
				Result := Precursor {SET}(s)
			end
		end

	subtracted alias "∖" (s: STS_SET [A, EQ]): like subset_anchor
			-- <Precursor>
		do
			if # s = # u then
				Result := o
			else
				Result := Precursor {SET}(s)
			end
		end

	subtracted_symmetricaly alias "⊖" (s: STS_SET [A, EQ]): like set_anchor
			-- <Precursor>
		local
			curr_subtr_s: like set_anchor
			s_subtr_curr: STS_SET [A, EQ]
		do
			if # Current = # u then
				Result := Current ∖ s
			elseif # s = # u then
				if is_empty then
					Result := converted (s)
				else
					Result := converted (s | agent does_not_have)
				end
			else
				if s.is_empty then
					curr_subtr_s := Current
				else
					curr_subtr_s := Current | agent s.does_not_have
				end
				if is_empty then
					s_subtr_curr := s
				else
					s_subtr_curr := s | agent does_not_have
				end
					check
						is_disjoint: curr_subtr_s.is_disjoint (s_subtr_curr) -- curr_subtr_s ⊆ Current and Current.is_disjoint (s_subtr_curr)
					end
				Result := curr_subtr_s.batch_extended (s_subtr_curr)
			end
		end

feature -- Anchor

	set_anchor,
	subset_anchor,
	superset_anchor: NUMERIC_SET [A, EQ]
			-- <Precursor>
		deferred
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
