﻿note
	description: "Implementation of {STS_REAL_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	REAL_UNIVERSE

inherit
	STS_REAL_UNIVERSE
		rename
			Cardinality as Cardinality
		undefine
			with,
			without,
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_subset,
			is_trivial_superset,
			is_proper_subset,
			is_proper_superset,
			is_disjoint,
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
			complemented,
			intersected,
			subtracted,
			subtracted_symmetricaly,
			crossed_tuples,
			equality_holds
		redefine
			default_create,
			is_singleton,
			cardinality,
			out,
			without_zero,
			non_positive,
			non_negative,
			negative,
			positive,
			unbounded_interval,
			trivial_subsets,
			proper_subsets,
			complement,
			squared,
			diagonal
		end

	UNIVERSE [STS_REAL_NUMBER, STS_REAL_EQUALITY]
		undefine
			eq,
			subtracted,
			o, empty_set,
			converted_element,
			unit,
			any_anchor,
			set_anchor,
			subset_anchor,
			element_to_element,
			element_to_natural,
			element_to_boolean,
			element_to_tuple,
			element_to_sets,
			tuple_extensor
		redefine
			default_create,
			make_extended,
			make_from_set,
			any,
			is_singleton,
			cardinality,
			out,
			subsets,
			powerset,
			trivial_subsets,
			proper_subsets,
			squared,
			diagonal,
			as_singleton,
			as_tuple
		end

	REAL_SET
		rename
			Cardinality as Cardinality,
			superset_anchor as itself
		undefine
			make_empty,
			make_singleton,
			is_empty,
			has,
			with,
			extended,
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_subset,
			is_trivial_superset,
			is_proper_subset,
			is_proper_superset,
			is_disjoint,
			complemented,
			united,
			intersected,
			subtracted_symmetricaly,
			itself
		redefine
			default_create,
			make_extended,
			make_from_set,
			make_from_numeric_set,
			any,
			is_singleton,
			cardinality,
			out,
			without_zero,
			non_positive,
			non_negative,
			negative,
			positive,
			unbounded_interval,
			subsets,
			powerset,
			trivial_subsets,
			proper_subsets,
			complement,
			squared,
			diagonal,
			as_singleton,
			as_tuple
		end

create
	default_create,
	make_from_universe

convert
	make_from_universe ({
		STS_UNIVERSE [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]],
		UNIVERSE [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]],

		STS_UNIVERSE [STS_REAL_NUMBER, STS_REAL_EQUALITY],
		UNIVERSE [STS_REAL_NUMBER, STS_REAL_EQUALITY],

		STS_REAL_UNIVERSE
		}),
	as_complex_set: {
		COMPLEX_SET,
		STS_COMPLEX_SET,

		NUMERIC_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY],
		STS_NUMERIC_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
		}

feature {NONE} -- Initialization

	default_create
			-- Create the universe of real numbers, i.e. ℝ.
		local
			bp: like Any_anchor.value_bit_pattern
			x: like Any_anchor
		do
				-- Non-negative real numbers
			from
				bp := {REAL_NUMBER}.Max_exponent_bit_pattern |<< {REAL_NUMBER}.Mantissa_width
					check
						no_extra_bit: bp & (⊝ {REAL_NUMBER}.Value_bit_pattern_mask) = 0 -- As assigned above.
					end
				create x.make_from_bit_pattern (bp)
				create others.make_singleton (x)
			invariant
				building_non_negatives_up: others ≍ {
					TRANSFORMER [
						like Any_anchor.value_bit_pattern, STS_REAL_NUMBER,
						STS_OBJECT_EQUALITY [like Any_anchor.value_bit_pattern], STS_REAL_EQUALITY
						]
					}.set_map (
						bit_pattern_interval (bp, {REAL_NUMBER}.Positive_infinity.value_bit_pattern),
						agent real_from_bit_pattern
						)
			until
				bp = 0
			loop
				bp := bp - 1
					check
						no_extra_bit: -- bp < {REAL_NUMBER}.Max_exponent_bit_pattern |<< {REAL_NUMBER}.Mantissa_width
							bp & (⊝ {REAL_NUMBER}.Value_bit_pattern_mask) = 0
					end
				create x.make_from_bit_pattern (bp)
					check
						does_not_have: others ∌ x -- TODO: Loop invariant.
					end
				others := others.extended (x)
			variant
				bp: natural_as_integer (bp)
			end

				-- Negative real numbers.
			from
				bp := ({NATURAL_8} 1 |<< ({REAL_NUMBER}.Exponent_width + {REAL_NUMBER}.Mantissa_width)) | 1
					check
						no_extra_bit: bp & (⊝ {REAL_NUMBER}.Value_bit_pattern_mask) = 0 -- As assigned above.
					end
				create x.make_from_bit_pattern (bp)
					check
						does_not_have: others ∌ x -- So far, `others' has only non-negative numbers.
					end
				others := others.extended (x)
			invariant
				building_non_negatives_up: others ≍ {
					TRANSFORMER [
						like Any_anchor.value_bit_pattern, STS_REAL_NUMBER,
						STS_OBJECT_EQUALITY [like Any_anchor.value_bit_pattern], STS_REAL_EQUALITY
						]
					}.set_map (
						bit_pattern_interval (0, {REAL_NUMBER}.Positive_infinity.value_bit_pattern) ∪
						bit_pattern_interval ({REAL_NUMBER}.Zero.previous_float.value_bit_pattern, bp),
						agent real_from_bit_pattern
						)
			until
				bp = ({NATURAL_8} 1 |<< ({REAL_NUMBER}.Exponent_width + {REAL_NUMBER}.Mantissa_width)) |
					({REAL_NUMBER}.Max_exponent_bit_pattern |<< {REAL_NUMBER}.Mantissa_width)
			loop
				bp := bp + 1
					check
						no_extra_bit:
								-- bp ≤ ({NATURAL_8} 1 |<< ({REAL_NUMBER}.Exponent_width + {REAL_NUMBER}.Mantissa_width)) |
								-- 	({REAL_NUMBER}.Max_exponent_bit_pattern |<< {REAL_NUMBER}.Mantissa_width)
							bp & (⊝ {REAL_NUMBER}.Value_bit_pattern_mask) = 0
					end
				create x.make_from_bit_pattern (bp)
					check
						does_not_have: others ∌ x -- TODO: Loop invariant.
					end
				others := others.extended (x)
			variant
				ascending_bp: natural_as_integer (
					(
						({NATURAL_8} 1 |<< ({REAL_NUMBER}.Exponent_width + {REAL_NUMBER}.Mantissa_width)) |
						({REAL_NUMBER}.Max_exponent_bit_pattern |<< {REAL_NUMBER}.Mantissa_width)
						) - bp
					)
			end

			create any_storage.put ({like any}.Nan)
		ensure then
			definition: Current ≍ (
				{
					TRANSFORMER [
						like Any_anchor.value_bit_pattern, STS_REAL_NUMBER,
						STS_OBJECT_EQUALITY [like Any_anchor.value_bit_pattern], STS_REAL_EQUALITY
						]
					}.set_map (
						bit_pattern_interval (0, {REAL_NUMBER}.Positive_infinity.value_bit_pattern) ∪
						bit_pattern_interval ({REAL_NUMBER}.Zero.previous_float.value_bit_pattern, {REAL_NUMBER}.Negative_infinity.value_bit_pattern),
						agent real_from_bit_pattern
						) & {REAL_NUMBER}.Nan
				)
		end

	make_extended (a: STS_REAL_NUMBER; s: STS_SET [STS_REAL_NUMBER, STS_REAL_EQUALITY])
			-- Turn `any' and `others', respectively, into `a` and `s`.
		do
			check
					-- NOTICE: Make these hold!
				same_any: a ≍ Any -- Otherwise we must give up of keeping `Any' as a once query.
				almost_the_universe: # s = Instance_free_cardinality - 1
			then
				check
					is_not_empty_already: not Is_empty
					any_already_defined: Eq (Any, a)
					others_already_defined: others ≍ s
				end
			end
		end

	make_from_set (s: STS_SET [STS_REAL_NUMBER, STS_REAL_EQUALITY])
			-- Turn current universe equal to `s`.
			-- TODO: Re-test it.
		do
			check
				the_universe: # s = # Current
			then
				check
					nothing_to_do: Current ≍ s
				end
			end
		end

	make_from_numeric_set (s: STS_SET [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]])
			-- Turn current universe equal to `s`.
		do
			check
				the_universe: # s = # Current
			then
				check
					nothing_to_do: s ≍ Current
				end
			end
		end

	make_from_universe (univ: STS_UNIVERSE [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]])
			-- Create a universe equal to `univ'.
			--| TODO: Maybe redundant, wrong and useless.
		do
			others := O.numeric_converted (univ.others)
				check
					is_not_empty: not univ.is_empty -- `univ' is a universe.
				end
			create any_storage.put (converted_element (univ.any))
		ensure
			same_universe: univ ≍ Current
		end

feature -- Primitive

	any: REAL_NUMBER
			-- An arbitrary representable real number
		once
			Result := Precursor {UNIVERSE}
		end

feature -- Quality

	is_singleton: BOOLEAN = False
			-- Does current universe have exactly one element?

feature -- Measurement

	cardinality: NATURAL
			-- <Precursor>
		once
			Result := Precursor {UNIVERSE}
		end

	instance_free_cardinality alias "#": NATURAL
			-- Number of elements in current kind of universe
		once
			Result := {NATURAL} 1 |<< ({REAL_NUMBER}.Mantissa_width + {REAL_NUMBER}.Exponent_width + 1) -
				({NATURAL} 1 |<< ({REAL_NUMBER}.Mantissa_width + 1) - 2)
		ensure then
			class
			definition: Result = 2 ^ ({REAL_NUMBER}.Mantissa_width + {REAL_NUMBER}.Exponent_width + 1) - (2 ^ ({REAL_NUMBER}.Mantissa_width + 1) - 2)
		end

feature -- Output

	out: STRING
			-- <Precursor>
		once
			Result := Precursor {REAL_SET}
		end

feature -- Restriction

	without_zero: REAL_SET
			-- Every representable real number but `zero'
		once
			Result := Precursor {REAL_SET}
		end

	non_positive: REAL_SET
			-- Every non-positive representable real number
		once
			Result := Precursor {REAL_SET}
		end

	non_negative: REAL_SET
			-- Every non-negative representable real number
		once
			Result := Precursor {REAL_SET}
		end

	negative: REAL_SET
			-- Every negative representable real number
		once
			Result := Non_positive / Zero
		end

	positive: REAL_SET
			-- Every positive representable real number
		once
			Result := Non_negative / Zero
		end

feature -- Interval

	unbounded_interval: REAL_UNIVERSE
			-- Current universe itself, i.e. ℝ.
		once
			Result := Current
		end

feature -- Operation

	subsets: SUBSET_FAMILY [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Every possible representable set of real numbers
		once
			Result := Precursor {REAL_SET}
		end

	powerset: SUBSET_FAMILY [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Every possible representable set of real numbers
		once
			Result := Precursor {REAL_SET}
		end

	trivial_subsets: SUBSET_FAMILY [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Family with only the empty set and the universe of real numbers
		once
			Result := Precursor {REAL_SET}
		end

	proper_subsets: SUBSET_FAMILY [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Every non-trivial subset of current universe
		once
			Result := Precursor {REAL_SET}
		end

	complement alias "∁": REAL_SET
			-- <Precursor>
		once
			Result := O
		end

	squared alias "×": SET_OF_ORDERED_PAIRS [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]
			-- Current universe crossed with itself, i.e. every representable pair in ℝ².
		once
			Result := Precursor {REAL_SET}
		end

	diagonal alias "∆": SET_OF_ORDERED_PAIRS [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]
			-- Every representable real number x, mapped to the respective identical pair (x,x), i.e. {(x,x) | x ∈ ℝ}
		once
			Result := Precursor {REAL_SET}
		end

feature -- Conversion

	as_singleton: SUBSET_FAMILY [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Current universe alone into a singleton, i.e. {ℝ}.
		once
			Result := Precursor {REAL_SET}
		end

	as_tuple: N_TUPLE [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Current universe converted to an n-tuple whose terms are ordered as they are accessed by current universe
			-- NOTICE: Please see the comment at {SET}.as_tuple.
		once
			Result := Precursor {REAL_SET}
		end

feature {REAL_UNIVERSE} -- Implementation

	n_is_within_interval (a, n, b: like Any_anchor.value_bit_pattern): BOOLEAN
			-- Is `n' within the interval {`a'..`b'}?
		do
			Result := a ≤ n and n ≤ b
		ensure
			class
			definition: Result = (a ≤ n and n ≤ b)
		end

feature {NONE} -- Implementation

	bit_pattern_interval (a, b: like Any_anchor.value_bit_pattern):
		SET [like Any_anchor.value_bit_pattern, STS_OBJECT_EQUALITY [like Any_anchor.value_bit_pattern]]
			-- Set of natural nunbers n, where `a' ≤ n ≤ `b'
		local
			n: like Any_anchor.value_bit_pattern
		do
			create Result.make_empty
			if a ≤ b then
				from
					n := b
					Result := Result.extended (n)
				invariant
					every_natural_number_within_interval: # Result = (b - n  + 1)
					nothing_else: Result |∀ agent R.n_is_within_interval (n, ?, b)
				until
					n = a
				loop
					n := n - 1
					Result := Result.extended (n)
				variant
					interval: natural_as_integer (n - a)
				end
			end
		ensure
			class
			when_interval_is_empty: a > b implies Result.is_empty
			every_natural_number_within_interval: a ≤ b implies # Result = (b - a  + 1)
			nothing_else: Result |∀ agent R.n_is_within_interval (a, ?, b)
		end

	real_from_bit_pattern (bp: like Any_anchor.value_bit_pattern): like Any_anchor
			-- Real number whose bit pattern equals `bp' & {REAL_NUMBER}.Value_bit_pattern_mask
		do
				check
					no_extra_bit: (bp & {REAL_NUMBER}.Value_bit_pattern_mask) & (⊝ {REAL_NUMBER}.Value_bit_pattern_mask) = 0 -- By definifion.
				end
			create Result.make_from_bit_pattern (bp & {REAL_NUMBER}.Value_bit_pattern_mask)
		ensure
			definition: Result.value_bit_pattern = (bp & {REAL_NUMBER}.Value_bit_pattern_mask)
		end

invariant
	attached_stored_any: attached any_storage
	Any: Any ~ any_storage.item
	Instance_free_cardinality: # Current = 2 ^ ({REAL_NUMBER}.Mantissa_width + {REAL_NUMBER}.Exponent_width + 1) - (2 ^ ({REAL_NUMBER}.Mantissa_width + 1) - 2)
	Without_zero: Without_zero ≍ (Current / Zero)
	Non_positive: Non_positive ≍ left_unbounded_right_closed_interval (Zero)
	Non_negative: Non_negative ≍ left_closed_right_unbounded_interval (Zero)
	Negative: Negative ≍ left_unbounded_right_open_interval (Zero)
	Positive: Positive ≍ left_open_right_unbounded_interval (Zero)
	Unbounded_interval: Unbounded_interval ≍ Current
	As_tuple: As_tuple ≍ Element_to_tuple.set_reduction (Current, Unit, agent {STS_N_TUPLE [STS_REAL_NUMBER, STS_REAL_EQUALITY]}.before)
	Is_singleton: Is_singleton = (not is_empty and others.is_empty)
	Out_not_void: Out /= Void
	Out: Out ~ "{" + As_tuple.terms_out + "}"
	As_singleton_has_current_set: Current ∈ As_singleton
	As_singleton_has_nothing_else: As_singleton.is_singleton
	As_singleton_superset: As_singleton.superset ≍ Current
	Subsets:; -- INFEASIBLE: Subsets ≍ Element_to_sets.set_reduction (Current, O.as_singleton, agent sets_plus_same_sets_with_element)
	Subsets_superset:; -- INFEASIBLE: Subsets.superset ≍ Current
	Powerset:; -- INFEASIBLE: Powerset ≍ Element_to_sets.set_reduction (Current, O.as_singleton, agent sets_plus_same_sets_with_element)
	Powerset_superset:; -- INFEASIBLE: Powerset.superset ≍ Current
	Trivial_subsets: Trivial_subsets ≍ (As_singleton & O)
	Trivial_subsets_superset: Trivial_subsets.superset ≍ Current
	Proper_subsets:; -- INFEASIBLE: Proper_subsets ≍ (Powerset / Current / O)
	Proper_subsets_superset:; -- INFEASIBLE: Proper_subsets.superset ≍ Current
	Squared:; -- TOO MUCH EXPENSIVE: Squared ≍ (Current × Current)
	Diagonal: Diagonal ≍ (× Current).filtered_xy (agent equality_holds)
	is_subset: Current ⊆ R -- `R' is current universe.
	complement: ∁ Current ≍ (Current ∁ R)

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
