﻿note
	description: "(Partial) implementation of {STS_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	UNIVERSE [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	STS_UNIVERSE [A, EQ]
		rename
			Cardinality as Cardinality
		undefine
			without,
			out,
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
			trivial_subsets,
			subtracted,
			subtracted_symmetricaly,
			diagonal,
			crossed,
			crossed_tuples,
			equality_holds
		redefine -- TODO: doubtful optimizations...
			with,
			cardinality,
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_subset,
			is_trivial_superset,
			is_proper_subset,
			is_proper_superset,
			is_disjoint,
			complemented,
			intersected
		end

	SET [A, EQ]
		rename
			Cardinality as Cardinality,
			superset_anchor as itself
		undefine
			has
		redefine
			make_empty,
			make_singleton,
			make_extended,
			make_from_set,
			is_empty,
			any,
			with,
			extended,
			cardinality,
			equals,
			is_subset,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_strict_superset,
			is_trivial_subset,
			is_trivial_superset,
			is_proper_subset,
			is_proper_superset,
			is_disjoint,
			complemented,
			united,
			intersected,
			subtracted_symmetricaly,
			itself
		end

feature {NONE} -- Initialization

	make_empty
			-- Do nothing. No way to fulfill the contract.
		do
		end

	make_singleton (a: A)
			-- Do nothing. No way to fulfill the contract.
		do
		end

	make_extended (a: A; s: STS_SET [A, EQ])
			-- Turn `any' and `others', respectively, into `a' and `s'.
		do
			check
					-- NOTICE: Make it hold!
				almost_the_universe: # s = instance_free_cardinality - 1 -- This ensures that `s' & `a' is the universe.
			then
				Precursor {SET}(a, s)
			end
		end

	make_from_set (s: STS_SET [A, EQ])
			-- Turn current universe equal to `s`.
		do
			check
				the_universe: # s = instance_free_cardinality -- NOTICE: Make it hold!
			then
				create others.make_empty
				others := others.converted (s.others)
					check
						is_not_empty: not s.is_empty -- 0 < # s = instance_free_cardinality
					end
				create any_storage.put (converted_element (s.any))
			end
		end

feature -- Primitive

	is_empty: BOOLEAN = False
			-- <Precursor>

	any: like any_anchor
			-- <Precursor>
		require else
			universe_is_never_empty: True
		do
			check
				attached_stored_any: attached any_storage -- not `is_empty'
			then
				Result := any_storage.item
			end
		end

feature -- Construction

	with alias "&" (a: A): like itself
			-- <Precursor>
			-- Current universe itself
		do
			Result := Current
		end

	extended (a: A): like itself
			-- <Precursor>
			-- NOTICE: The precondition never holds for a universe.
		do
			Result := Current
		ensure then
			impossible_precondition: False
		end

feature -- Measurement

	cardinality: like natural_anchor
			-- <Precursor>
		do
			Result := instance_free_cardinality
		ensure then
			no_instance_needed: Result = instance_free_cardinality
		end

	instance_free_cardinality alias "#": like natural_anchor
			-- Number of elements in current kind of universe. It must be known without iterating over the elements of the universe.
		deferred
		ensure
			class
		end

feature -- Comparison

	equals alias "≍" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := Current = s or Current ~ s or attached {STS_UNIVERSE [A, EQ]} s or # Current = # s
		end

	is_subset alias "⊆" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := Current ≍ s
		end

	is_superset alias "⊇" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := True
		end

	is_comparable (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := True
		end

	is_not_comparable (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := False
		end

	is_strict_subset alias "⊂" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := False
		end

	is_strict_superset alias "⊃" (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := Current ≭ s
		end

	is_trivial_subset (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := Current ≍ s
		end

	is_trivial_superset (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := s.is_empty or Current ≍ s
		end

	is_proper_subset (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := False
		end

	is_proper_superset (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := not s.is_empty and # s < # Current
		end

	is_disjoint (s: STS_SET [A, EQ]): BOOLEAN
			-- <Precursor>
		do
			Result := s.is_empty
		end

feature -- Operation

	complemented alias "∁" (s: STS_SET [A, EQ]): like subset_anchor
			-- <Precursor>
		do
			Result := o
		end

	united alias "∪" (s: STS_SET [A, EQ]): like itself
			-- <Precursor>
		do
			Result := Current
		end

	intersected alias "∩" (s: STS_SET [A, EQ]): like subset_anchor
			-- <Precursor>
		do
			Result := converted (s)
		end

	subtracted_symmetricaly alias "⊖" (s: STS_SET [A, EQ]): like set_anchor
			-- <Precursor>
		do
			Result := Current ∖ s
		end

feature -- Anchor

	itself: like Current
			-- <Precursor>
		do
			Result := Current
		end

invariant
	is_empty_definition: is_empty = not attached any_storage
	Cardinality: Cardinality = Element_to_natural.set_reduction (Current, 0, agent natural_successor)
	cardinality_needs_no_instance: Cardinality = instance_free_cardinality

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
