﻿note
	description: "Implementation of {STS_TRANSFORMER}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

class
	TRANSFORMER [
		A, B,
		EQ_A -> STS_EQUALITY [A] create default_create end, -- TODO: It may be unecessary.
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	STS_TRANSFORMER [A, B, EQ_A, EQ_B]
		redefine
			set_reduction,
			tuple_left_reduction,
			tuple_indexed_left_reduction,
			tuple_right_reduction,
			tuple_indexed_right_reduction
		end

feature -- Access

	eq_b: EQ_B
			-- <Precursor>
		do
			create Result
		end

feature -- Operation

	cartesian_product (xs: STS_SET [A, STS_EQUALITY [A]]; ys: STS_SET [B, STS_EQUALITY [B]]): like product_anchor
			-- <Precursor>
		local
			l_xs: STS_SET [A, STS_EQUALITY [A]]
		do
			Result := o
			if not ys.is_empty then
				from
					l_xs := xs
				invariant
					every_pair: (xs ∖ l_xs) |∀ agent transformer.every_pair_is_in_set(?, ys, Result)
					nothing_else: Result.for_all_xy (agent transformer.anded (agent (xs ∖ l_xs).has, ?, agent ys.has, ?))
				until
					l_xs.is_empty
				loop
						check
							is_disjoint:
									-- Result ≍ cartesian_product (xs ∖ l_xs, ys)
								Result.is_disjoint (singleton_crossed_set (l_xs.any, ys))
						end
					Result := Result.batch_extended (singleton_crossed_set (l_xs.any, ys))
					l_xs := l_xs.others
				variant
					cardinality: natural_as_integer (# l_xs)
				end
			end
		ensure then
			class
		end

	singleton_crossed_set (x: A; ys: STS_SET [B, STS_EQUALITY [B]]): like product_anchor
			-- Cartesian product of {`x'} and `ys',
		local
			l_ys: STS_SET [B, STS_EQUALITY [B]]
		do
			from
				Result := o
				l_ys := ys
			invariant
				building_up: Result ≍ {TRANSFORMER [B, STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], EQ_B, STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]]}.
					set_map (ys ∖ l_ys, agent transformer.pair (x, ?))
			until
				l_ys.is_empty
			loop
					check
						does_not_have: Result ∌ pair (x, l_ys.any) -- Result ≍ ((ys - l_ys) ↦ agent pair (x, ?))
					end
				Result := Result.extended (pair (x, l_ys.any))
				l_ys := l_ys.others
			variant
				cardinality: (# l_ys).as_integer_64
			end
		ensure
			class
			definition: Result ≍ {TRANSFORMER [B, STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], EQ_B, STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]]}.
				set_map (ys, agent transformer.pair (x, ?))
		end

feature -- Transformation

	set_map (xs: STS_SET [A, STS_EQUALITY [A]]; f: FUNCTION [A, B]): like set_map_anchor
			-- <Precursor>
		local
			l_xs: STS_SET [A, STS_EQUALITY [A]]
		do
			from
				create Result.make_empty
				l_xs := xs
			invariant
				maps_everything: (xs ∖ l_xs) |∀ agent transformer.target_set_has_value(Result, f, ?)
				nothing_else: Result |∀ agent transformer.source_set_has_argument(f, xs ∖ l_xs, ?)
			until
				l_xs.is_empty
			loop
				Result := Result & f (l_xs.any)
				l_xs := l_xs.others
			variant
				cardinality: natural_as_integer (# l_xs)
			end
		ensure then
			class
		end

	set_reduction (xs: STS_SET [A, STS_EQUALITY [A]]; start: B; f: FUNCTION [B, A, B]): B
			-- <Precursor>
		local
			l_xs: STS_SET [A, STS_EQUALITY [A]]
		do
			from
				Result := start
				l_xs := xs
			invariant
				building_up: eq_b (Result, tuple_left_reduction (xs.as_tuple.right_trimmed (# l_xs), start, f))
					-- NOTICE: This invariant assumes that `xs'.`as_tuple' is implemented like in {STI_SET}.
			until
				l_xs.is_empty
			loop
				Result := f (Result, l_xs.any)
				l_xs := l_xs.others
			variant
				cardinality: natural_as_integer (# l_xs)
			end
		end

	tuple_map (xt: STS_N_TUPLE [A, STS_EQUALITY [A]]; f: FUNCTION [A, B]): N_TUPLE [B, EQ_B]
			-- <Precursor>
		local
			l_xt: STS_N_TUPLE [A, STS_EQUALITY [A]]
		do
			from
				create Result.make_unit
				l_xt := xt.reversed
			invariant
				maps_everything: xt.left_trimmed (l_xt.n).for_all (agent transformer.target_tuple_has_value (Result, ?, f, ?))
				nothing_else: Result.for_all (agent transformer.source_tuple_has_argument (f, xt.left_trimmed (l_xt.n), ?, ?))
			until
				l_xt.is_unit
			loop
				Result := Result.after (f (l_xt.first))
				l_xt := l_xt.tail
			variant
				n: natural_as_integer (l_xt.n)
			end
		ensure then
			class
		end

	tuple_left_reduction (xt: STS_N_TUPLE [A, STS_EQUALITY [A]]; leftmost: B; f: FUNCTION [B, A, B]): B
			-- <Precursor>
		local
			l_xt: STS_N_TUPLE [A, STS_EQUALITY [A]]
		do
			from
				Result := leftmost
				l_xt := xt
			invariant
				building_up: eq_b (Result, tuple_left_reduction (xt.right_trimmed (l_xt.n), leftmost, f))
			until
				l_xt.is_unit
			loop
				Result := f (Result, l_xt.first)
				l_xt := l_xt.tail
			variant
				n: natural_as_integer (l_xt.n)
			end
		end

	tuple_indexed_left_reduction (xt: STS_N_TUPLE [A, STS_EQUALITY [A]]; leftmost: B; i: NATURAL; f: FUNCTION [B, NATURAL, A, B]): B
			-- <Precursor>
		local
			l_xt: STS_N_TUPLE [A, STS_EQUALITY [A]]
			j: NATURAL
		do
			from
				Result := leftmost
				l_xt := xt
				j := i
			invariant
				building_up: eq_b (Result, tuple_indexed_left_reduction (xt.right_trimmed (l_xt.n), leftmost, i, f))
			until
				l_xt.is_unit
			loop
				Result := f (Result, j, l_xt.first)
				l_xt := l_xt.tail
				j := j + 1
			variant
				n: natural_as_integer (l_xt.n)
			end
		end

	tuple_right_reduction (xt: STS_N_TUPLE [A, STS_EQUALITY [A]]; f: FUNCTION [A, B, B]; rightmost: B): B
			-- <Precursor>
		do
			Result := tuple_left_reduction (xt.reversed, rightmost, agent transformer.flipped(f, ?, ?))
		end

	tuple_indexed_right_reduction (xt: STS_N_TUPLE [A, STS_EQUALITY [A]]; i: NATURAL; f: FUNCTION [NATURAL, A, B, B]; rightmost: B): B
			-- <Precursor>
		do
				check
					small_enough: {TRANSFORMER [A, BOOLEAN, EQ_A, STS_OBJECT_EQUALITY [BOOLEAN]]}. -- 0 <= j < xt.n
						tuple_indexed_left_reduction (xt.reversed, True, 0, agent transformer.flipped_index_is_small_enough (xt.n + i, ?, ?, ?))
				end
			Result := tuple_indexed_left_reduction (xt.reversed, rightmost, 0, agent transformer.indexed_flipped (xt.n + i, f, ?, ?, ?))
		end

feature -- Conversion

	natural_as_integer (n: NATURAL): INTEGER_64
			-- `n' converted into an integer value
			-- todo: make it more flexible.
		do
			Result := n.as_integer_64
		ensure
			class
			definition: Result = n.as_integer_64
		end

feature -- Factory

	pair (a: A; b: B): ORDERED_PAIR [A, B, EQ_A, EQ_B]
			-- <Precursor>
		do
			create Result.make (a, b)
		ensure then
			class
		end

	o,
	empty_product: like product_anchor
			-- Cartesian product with no pair
		do
			create Result.make_empty
		ensure
			class
			definition: Result.is_empty
		end

	transformer: like Current
			-- <Precursor>
		do
			create Result
		end

feature -- Reduction
	-- todo: Explain.

	flipped (f: FUNCTION [A, B, B]; acc: B; x: A): B
			-- `f' applied to (`acc', `x') in flipped order, i.e. `f' (`x', `acc').
		do
			Result := f (x, acc)
		ensure
			class
			definition: eq_b (Result, f (x, acc))
		end

	indexed_flipped (m: NATURAL f: FUNCTION [NATURAL, A, B, B] acc: B j: NATURAL x: A): B
			-- `f' applied to (`acc', `j', `x') in flipped order and with adjusted index, i.e. `f' (`m' - `j' - 1, `x', `acc').
		require
			small_enough: j < m
		do
			Result := f (m - j - 1, x, acc)
		ensure
			class
			definition: eq_b (Result, f (m - j - 1, x, acc))
		end

	flipped_index_is_small_enough (m: NATURAL; acc: BOOLEAN; j: NATURAL; x: A): BOOLEAN
			-- Is `acc' true and does the inequality `j' < `m' hold?
		do
			Result := acc and j < m
		ensure
			class
			definition: Result = (acc and j < m)
		end

feature -- Anchor

	product_anchor: SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B]
			-- <Precursor>
		do
			Result := o
		ensure then
			class
		end

	set_map_anchor: SET [B, EQ_B]
			-- <Precursor>
		do
			create Result.make_empty
		ensure then
			class
		end

	tuple_map_anchor: N_TUPLE [B, EQ_B]
			-- <Precursor>
		do
			create Result.make_unit
		ensure then
			class
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
