﻿note
	description: "Implementation of {STS_ORDERED_PAIR}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Wed 2018-11-21 07:53:55 -0300 $"
	revision: "$Revision$"

class
	ORDERED_PAIR [
		A, B,
		EQ_A -> STS_EQUALITY [A] create default_create end,
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]
		redefine
			out
		end

	DEBUG_OUTPUT
		rename
			debug_output as out
		redefine
			out
		end

create
	make, make_from_pair

convert
	make_from_pair ({
		STS_ORDERED_PAIR [A, B, EQ_A, EQ_B],
		STS_COMPOSABLE_ORDERED_PAIR [A, A, B, EQ_A, EQ_A, EQ_B],
		STS_COMPOSABLE_ORDERED_PAIR [B, A, B, EQ_B, EQ_A, EQ_B]
		})

feature {NONE} -- Initialization

	make (a_first: A; a_second: B)
			-- Create the pair (`a_first', `a_second')
		do
			a := a_first
			b := a_second
		ensure
			first: eq_a (a, a_first)
			second: eq_b (b, a_second)
		end

	make_from_pair (ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B])
			-- Create the pair (`ab'.`a', `ab'.`b')
		do
			a := ab.a
			b := ab.b
		ensure
			first: eq_a (a, ab.a)
			second: eq_b (b, ab.b)
		end

feature -- Primitive

	a: A
			-- <Precursor>

	first: like a
			-- <Precursor>
		do
			Result := a
		end

	b: B
			-- <Precursor>

	second: like b
			-- <Precursor>
		do
			Result := b
		end

	eq_a: EQ_A
			-- <Precursor>
		do
			create Result
		ensure then
			class
		end

	eq_b: EQ_B
			-- <Precursor>
		do
			create Result
		ensure then
			class
		end

feature -- Output

	out: STRING
			-- <Precursor>
		do
			Result := "("
				check
					first_argument_not_void: first_out (a) /= Void -- `first_out' definition
				end
			Result.append (first_out (a))
			Result.append_character (',') -- todo: unreachable code?
				check
					second_argument_not_void: second_out (b) /= Void -- `second_out' definition
				end
			Result.append (second_out (b))
			Result.append_character (')')
		ensure then
			definition: Result ~ "(" + first_out (a) + "," + second_out (b) + ")"
		end

	first_out (x: A): STRING
			-- Terse printable representation of `x'
		do
			if attached x then
					check
						other_not_void: x.out /= Void -- {ANY}.`out' definition
					end
				create Result.make_from_separate (x.out)
			else
				Result := "Void"
			end
		ensure
			class
			definition: if attached x then
					Result ~ x.out
				else
					Result ~ "Void"
				end
		end

	second_out (y: B): STRING
			-- Terse printable representation of `y'
		do
			if attached y then
					check
						other_not_void: y.out /= Void -- {ANY}.`out' definition
					end
				create Result.make_from_separate (y.out)
			else
				Result := "Void"
			end
		ensure
			class
			definition: if attached y then
					Result ~ y.out
				else
					Result ~ "Void"
				end
		end

feature -- Operation

	inverse: ORDERED_PAIR [B, A, EQ_B, EQ_A]
			-- <Precursor>
		do
			create Result.make (b, a)
		end

	composed_with_homogeneous (x1x2: STS_ORDERED_PAIR [A, A, EQ_A, EQ_A]): ORDERED_PAIR [A, B, EQ_A, EQ_B]
			-- <Precursor>
		do
			create Result.make (x1x2.a, b)
		end

	composed_with_inverse_like (yx: STS_ORDERED_PAIR [B, A, EQ_B, EQ_A]): COMPOSABLE_ORDERED_PAIR [A, B, B, EQ_A, EQ_B, EQ_B]
			-- <Precursor>
		do
			create Result.make (yx.a, b)
		end

feature -- Conversion

	as_singleton: SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B]
			-- <Precursor>
		do
			create Result.make_singleton (Current)
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
