﻿note
	description: "Implementation of {STS_SET_OF_ORDERED_PAIRS}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

class
	SET_OF_ORDERED_PAIRS [
		A, B,
		EQ_A -> STS_EQUALITY [A] create default_create end,
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	STS_SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B]
		undefine
			with,
			without,
			cardinality,
			out,
			equals,
			is_subset,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
			trivial_subsets,
			complemented,
			intersected,
			subtracted,
			subtracted_symmetricaly,
			diagonal,
			crossed_tuples,
			equality_holds
		redefine
			has
		end

	SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]]
		rename
			converted_element as converted_pair,
			element_to_element as pair_to_pair,
			element_to_natural as pair_to_natural,
			element_to_boolean as pair_to_boolean,
			element_to_tuple as pair_to_tuple,
			element_to_sets as pair_to_sets
		redefine
			has,
			without,
			converted_pair,
			subset_relation_holds,
			any_anchor,
			set_anchor,
			subset_anchor,
			superset_anchor
		end

create
	make_empty,
	make_singleton,
	make_extended,
	make_from_set

convert
	make_from_set ({
		STS_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]],
		SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]],

		STS_TRANSFORMABLE_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], A, STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], EQ_A],
		TRANSFORMABLE_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], A, STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], EQ_A],

		STS_TRANSFORMABLE_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], B, STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], EQ_B],
		TRANSFORMABLE_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], B, STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], EQ_B],

		STS_TRANSFORMABLE_SET [
			STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR [A, B, EQ_A, EQ_B],
			STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]
			],
		TRANSFORMABLE_SET [
			STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR [A, B, EQ_A, EQ_B],
			STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]
			],

		STS_TRANSFORMABLE_SET [
			STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR [B, A, EQ_B, EQ_A],
			STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [B, A, EQ_B, EQ_A]
			],
		TRANSFORMABLE_SET [
			STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR [B, A, EQ_B, EQ_A],
			STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [B, A, EQ_B, EQ_A]
			],

		STS_SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B]
		})

feature -- Membership

	has alias "∋" (ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]): BOOLEAN
			-- Is `ab` an element in current set?
		local
			s: like subset_anchor
			eq_a: EQ_A
			eq_b: EQ_B
		do
			from
				s := Current
				eq_a := ab.eq_a -- Better to avoid recreating an {EQ_A} and an {EQ_B} over and over.
				eq_b := ab.eq_b
			invariant
				not_found_yet: not ((Current ∖ s) |∃ agent equality_holds (ab, ?))
			until
				s.is_empty or else attached s.any as s_any and then (eq_a (ab.a, s_any.a) and eq_b (ab.b, s_any.b))
			loop
				s := s.others
			variant
				cardinality: natural_as_integer (# s)
			end
			Result := not s.is_empty
		end

feature -- Construction

	without alias "/" (ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]): like subset_anchor
			-- Every pair in current set but `ab`
		local
			s: like subset_anchor
			eq_a: EQ_A
			eq_b: EQ_B
		do
			from
				Result := o
				s := Current
				eq_a := ab.eq_a -- Better to avoid recreating an {EQ_A} and an {EQ_B} over and over.
				eq_b := ab.eq_b
			invariant
				building_up: Result ≍ (Current ∖ s)
			until
				s.is_empty or else attached s.any as s_any and then (eq_a (ab.a, s_any.a) and eq_b (ab.b, s_any.b))
			loop
					check
						does_not_have: Result ∌ s.any -- Result ≍ (Current ∖ s)
					end
				Result := Result.extended (s.any)
				s := s.others
			variant
				cardinality: natural_as_integer (# s)
			end
			if s.is_empty then
				Result := Current
			else
					check
						unwanted_pair_found: s.any ≍ ab -- Previous loop exit
					end
				s := s.others
				if not s.is_empty then
						check
							is_disjoint: s.is_disjoint (Result) -- Result ≍ ((Current ∖ s) / ab)
						end
					Result := s.batch_extended (Result)
				end
			end
		end

feature -- Factory

	converted_pair (ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]): like any_anchor
			-- `ab' converted to an ordered pair like `any'
		do
			if attached {like any_anchor} ab as c_ab then
				Result := c_ab
			else
				create {like any_anchor} Result.make (ab.a, ab.b)
			end
		ensure then
			class
		end

feature -- Predicate

	subset_relation_holds (s1, s2: STS_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]]): BOOLEAN
			-- <Precursor>
		local
			l_s1, l_s2: STS_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]]
			ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]
			eq_a: EQ_A
			eq_b: EQ_B
		do
			from
				Result := True
				l_s1 := s1
				create eq_a -- Better to avoid recreating an {EQ_A} and an {EQ_B} over and over.
				create eq_b
			invariant
				definition: Result = ((s1 ∖ l_s1) ⊆ s2)
			until
				not Result or l_s1.is_empty
			loop
				from
					ab := l_s1.any
					Result := False
					l_s2 := s2
				invariant
					not_found_yet: (s2 ∖ l_s2) ∌ ab
				until
					l_s2.is_empty or else attached l_s2.any as s2_any and then (eq_a (ab.a, s2_any.a) and eq_b (ab.b, s2_any.b))
				loop
					l_s2 := l_s2.others
				variant
					cardinality_2: natural_as_integer (# l_s2)
				end
				Result := not l_s2.is_empty
				l_s1 := l_s1.others
			variant
				cardinality_1: natural_as_integer (# l_s1)
			end
		end

feature -- Anchor

	any_anchor: ORDERED_PAIR [A, B, EQ_A, EQ_B]
			-- <Precursor>
		do
			Result := any
		end

	set_anchor,
	subset_anchor,
	superset_anchor: SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B]
			-- <Precursor>
		do
			Result := Current
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
