﻿note
	description: "Partial implementation of {STS_SET_OF_SETS}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	SET_OF_SETS [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	STS_SET_OF_SETS [A, EQ]
		undefine
			has,
			with,
			without,
			cardinality,
			out,
			equals,
			is_subset,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
			trivial_subsets,
			complemented,
			intersected,
			subtracted,
			subtracted_symmetricaly,
			diagonal,
			crossed_tuples,
			equality_holds
		redefine
			sets_intersection
		end

	SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]
		rename
			o as o_s,
			empty_set as empty_set_of_sets,
			unit as unit_s,
			converted_element as converted_member_set,
			element_to_element as set_to_set,
			element_to_natural as set_to_natural,
			element_to_boolean as set_to_boolean,
			element_to_tuple as set_to_tuple,
			n_tuple_anchor as set_tuple_anchor
		undefine
			make_singleton,
			make_extended,
			extended
		redefine
			converted_member_set,
			any_anchor,
			set_anchor,
			subset_anchor,
			superset_anchor,
			set_tuple_anchor
		end

feature -- Operation

	sets_union alias "⋃": like any.superset_anchor
			-- <Precursor>
		local
			ss: like subset_anchor
		do
			from
				Result := o
				ss := Current
			invariant
				every_element: (Current ∖ ss) |∀ agent Result.is_superset
				nothing_else: Result |∀ agent (Current ∖ ss).some_set_has
			until
				ss.is_empty
			loop
				Result := Result ∪ ss.any
				ss := ss.others
			variant
				cardinality: natural_as_integer (# ss)
			end
		end

	sets_intersection alias "⋂": like any.subset_anchor
			-- <Precursor>
		local
			ss: like subset_anchor
		do
			from
				Result := universe
				ss := Current
			invariant
				building_up: Result ≍ (universe | agent (Current ∖ ss).every_set_has)
			until
				Result.is_empty or ss.is_empty
			loop
				Result := Result ∩ ss.any
				ss := ss.others
			variant
				cardinality: natural_as_integer (# ss)
			end
		end

feature -- Factory

	o,
	empty_member_set: like any.subset_anchor
			-- <Precursor>
		do
			if is_empty then
				create Result.make_empty
			else
				Result := any.o
			end
		end

	converted_member_set (s: STS_SET [A, EQ]): like any_anchor
			-- `s' converted to a (member) set like `any'
		do
			Result := o.converted (s)
		end

feature -- Anchor

	any_anchor: SET [A, EQ]
			-- <Precursor>
		do
			Result := any
		end

	set_anchor,
	subset_anchor,
	superset_anchor: SET_OF_SETS [A, EQ]
			-- <Precursor>
		do
			Result := Current
		end

	set_tuple_anchor: SET_TUPLE [A, EQ]
			-- <Precursor>
		do
			Result := unit_s
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
