﻿note
	description: "Implementation of {STS_SET_PARTITION}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

class
	SET_PARTITION [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	STS_SET_PARTITION [A, EQ]
		undefine
			has,
			family_with,
			cardinality,
			out,
			equals,
			is_subset,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
			trivial_subsets,
			complemented,
			intersected,
			subtracted,
			subtracted_symmetricaly,
			diagonal,
			crossed_tuples,
			equality_holds
		redefine
			without,
			is_partition,
			member_sets_are_disjoint,
			partition_intersection
		end

	SET_FAMILY [A, EQ]
		rename
			make_singleton as family_make_singleton,
			make_extended as family_make_extended,
			make_from_set as family_make_from_set,
			with as family_with,
			extended as family_extended,
			batch_extended as family_batch_extended,
			family_union as partition_union alias "⋃",
			family_intersection as partition_intersection alias "⋂",
			o_f as o_p,
			empty_family as empty_partition,
			unit_f as unit_p,
			converted as family_converted,
			subfamily_anchor as subpartition_anchor
		redefine
			family_make_singleton,
			family_make_extended,
			family_make_from_set,
			without,
			family_extended,
			is_partition,
			member_sets_are_disjoint,
			filtered,
			partition_union,
			partition_intersection,
			subpartition_anchor
		end

create
	make_empty,
	make_singleton,
	make_extended,
	make_from_partition

convert
	make_from_partition ({STS_SET_PARTITION [A, EQ]})

feature {NONE} -- Initialization

	family_make_singleton (s: STS_SET [A, EQ])
			-- Turn current set partition into a singleton partition.
			-- Notice: This redefinition is not a creator.
		do
			check
				is_not_empty: not s.is_empty -- NOTICE: make it hold!
			then
				make_singleton (s)
			end
		end

	make_singleton (s: STS_SET [A, EQ])
			-- Create a singleton partition in the form {`s'}.
		require
			is_not_empty: not s.is_empty
		do
			create others.make_empty
			create any_storage.put (others.converted_member_set (s))
		ensure
			is_not_empty: not is_empty
			any: eq (any, s)
			others: others.is_empty
		end

	family_make_extended (s: STS_SET [A, EQ]; ss: STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]])
			-- Update `any' and `others' to `s' and `ss', respectively.
			-- Notice: This redefinition is not a creator.
		local
			old_others: like others
		do
			old_others := others
			create others.make_empty
			check
					-- NOTICE: make these hold!
				cl_ss: attached others.family_converted (ss) as cl_ss
				is_partition: cl_ss.is_partition
				sp: attached others.converted (cl_ss) as sp
				is_not_empty: not s.is_empty
				set_is_disjoint: sp.set_is_disjoint (s)
			then
				make_extended (s, sp)
			end
		rescue
			check
				a_partition_is_never_void: attached old_others
			then
				others := old_others
			end
		end

	make_extended (s: STS_SET [A, EQ]; sp: STS_SET_PARTITION [A, EQ])
			-- Create a partition whose `any' set and `others' are, respectively, `s' and `sp'.
		require
			is_not_empty: not s.is_empty
			set_is_disjoint: sp.set_is_disjoint (s)
		do
			create others.make_empty
			others := others.converted (sp)
			create any_storage.put (converted_member_set (s))
		ensure
			is_not_empty: not is_empty
			any: eq (any, s)
			others: others ≍ sp
		end

	family_make_from_set (ss: STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]])
			-- Make this set partition equal to `ss'.
			-- Notice: This redefinition is not a creator.
		local
			old_others: like others
		do
			old_others := others
			check
				keep_non_emptiness: ss.is_empty implies is_empty -- NOTICE: make it hold!
			then
				create others.make_empty
				check
					cl_ss: attached others.family_converted (ss) as cl_ss
					is_partition: cl_ss.is_partition -- NOTICE: make it hold!
					sp: attached others.converted (cl_ss) as sp
				then
					make_from_partition (sp)
				end
			end
		rescue
			check
				a_partition_is_never_void: attached old_others
			then
				others := old_others
			end
		end

	make_from_partition (sp: STS_SET_PARTITION [A, EQ])
			-- Create a partition equal to `sp`
		do
			if sp.is_empty then
				make_empty
			else
					check
							-- sp.is_partition
						is_not_empty: not sp.any.is_empty
						set_is_disjoint: sp.others.set_is_disjoint (sp.any)
					end
				make_extended (sp.any, sp.others)
			end
		ensure
			same_partition: Current ≍ sp
		end

feature -- Construction

	with alias "&" (s: STS_SET [A, EQ]): like superpartition_anchor
			-- <Precursor>
		do
			if s ∈ Current then
				Result := Current
			else
					check
						set_is_disjoint: set_is_disjoint (s) -- (Current / s) = Current
					end
				Result := extended (s)
			end
		end

	without alias "/" (s: STS_SET [A, EQ]): like subpartition_anchor
			-- Every set in current partition but `s`
		local
			sp: like subpartition_anchor
			l_eq: like eq
		do
			from
				Result := o_p
				sp := Current
				l_eq := eq -- Better to avoid recreating `eq' over and over.
			invariant
				building_up: Result ≍ (Current ∖ sp)
			until
				sp.is_empty or else l_eq (sp.any, s)
			loop
					check
						is_not_empty: not sp.any.is_empty -- sp.any ∈ Current)
						set_is_disjoint: Result.set_is_disjoint (sp.any) -- Result ≍ (Current ∖ sp)
					end
				Result := Result.extended (sp.any)
				sp := sp.others
			variant
				cardinality: natural_as_integer (# sp)
			end
			if sp.is_empty then
				Result := Current
			else
					check
						unwanted_set_found: l_eq (sp.any, s) -- Previous loop exit
					end
				sp := sp.others
				if not sp.is_empty then
						check
							is_partition: Result.is_partition -- By definition
							sets_are_disjoint: sp.sets_are_disjoint (Result) -- Result ≍ ((Current ∖ sp) / s)
						end
					Result := sp.batch_extended (Result)
				end
			end
		end

	family_extended (s: STS_SET [A, EQ]): like superfamily_anchor
			-- Every set in current partition, together with `s`.
		do
			if not s.is_empty and set_is_disjoint (s) then
				Result := extended (s)
			else
				create Result.make_extended (s, Current)
			end
		end

	extended (s: STS_SET [A, EQ]): like superpartition_anchor
			-- Current partition extended with `s'
		require
			is_not_empty: not s.is_empty
			set_is_disjoint: set_is_disjoint (s)
		do
			create Result.make_extended (s, Current)
		ensure
			is_not_empty: not Result.is_empty
			any: eq (Result.any, s)
			others: Result.others ≍ Current
		end

	batch_extended (ss: STS_SET_OF_SETS [A, EQ]): like superpartition_anchor
			-- Current partition extended with every set in `ss'
		require
			is_partition: ss.is_partition
			sets_are_disjoint: sets_are_disjoint (ss)
		local
			l_ss: STS_SET_OF_SETS [A, EQ]
		do
			from
				Result := Current
				l_ss := ss
			invariant
				building_prefixed_ss_up:
					Result.as_tuple.prefix (# (ss ∖ l_ss)).sets ≍ (ss ∖ l_ss)
				suffixed_current: Result.as_tuple.suffix (# Current).sets ≍ Current
			until
				l_ss.is_empty
			loop
					check
						is_not_empty: not l_ss.any.is_empty -- l_ss.is_partition
						set_is_disjoint: Result.set_is_disjoint (l_ss.any) -- Result ≍ (Current ∪ (ss ∖ l_ss))
					end
				Result := Result.extended (l_ss.any)
				l_ss := l_ss.others
			variant
				cardinality: natural_as_integer (# l_ss)
			end
		ensure
			prefixed_ss: Result.as_tuple.prefix (# ss).sets ≍ ss
			suffixed_current: Result.as_tuple.left_trimmed (# ss).sets ≍ Current
		end

feature -- Quality

	is_partition: BOOLEAN = True
			-- <Precursor>

	member_sets_are_disjoint: BOOLEAN = True
			-- <Precursor>

feature -- Operation

	filtered alias "|" (p: PREDICATE [STS_SET [A, EQ]]): like subpartition_anchor
			-- Every set in current partition for which `p` holds
		local
			sp, previous_result, last_segment: like subpartition_anchor
		do
			from
				Result := o_p
				sp := Current
			invariant
				every_compliant_set: (Current ∖ sp) |∀ agent (Current ∖ sp).iff (p, agent Result.has, ?)
				nothing_else: Result ⊆ (Current ∖ sp)
			until
				sp.is_empty
			loop
				if p (sp.any) then
					from
						previous_result := Result
							check
								is_not_empty: not sp.any.is_empty -- sp.any ∈ Current
								set_is_disjoint: Result.set_is_disjoint (sp.any) -- Result ⊆ (Current ∖ sp)
							end
						Result := Result.extended (sp.any)
						last_segment := sp
						sp := sp.others
					invariant
						previous_every_compliant_set: (Current ∖ last_segment) |∀ agent (Current ∖ last_segment).iff(p, agent previous_result.has, ?)
						previous_nothing_else: previous_result ⊆ (Current ∖ last_segment)
						compliant_segment: (last_segment ∖ sp) |∀ p
						is_partition: (last_segment ∖ sp).is_partition -- last_segment ⊆ Current
						sets_are_disjoint: previous_result.sets_are_disjoint (last_segment ∖ sp) -- previous_result ⊆ (Current ∖ last_segment)
						building_up: Result ≍ previous_result.batch_extended (last_segment ∖ sp)
					until
						sp.is_empty or else not p (sp.any)
					loop
							check
								loop_is_not_empty: not sp.any.is_empty -- sp.any ∈ Current)
								loop_set_is_disjoint: Result.set_is_disjoint (sp.any) -- Result ⊆ (Current ∖ sp)
							end
						Result := Result.extended (sp.any)
						sp := sp.others
					variant
						inner_cardinality: natural_as_integer (# sp)
					end
					if sp.is_empty then
							check
								previous_result_is_partition: previous_result.is_partition -- previous_result ⊆ Current
								last_segment_sets_are_disjoint: last_segment.sets_are_disjoint (previous_result) -- previous_result.sets_are_disjoint (last_segment ∖ sp)
							end
							-- loop invariant: (last_segment ∖ sp) |∀ p)
						Result := last_segment.batch_extended (previous_result)
					else
						sp := sp.others
					end
				else
					sp := sp.others
				end
			variant
				outer_cardinality: natural_as_integer (# sp)
			end
		end

	partition_union alias "⋃": like any.superset_anchor
			-- Union of all sets in current partition
		local
			sp: like subpartition_anchor
		do
			from
				Result := o
				sp := Current
			invariant
				building_up: Result ≍ (⋃ (Current ∖ sp))
			until
				sp.is_empty
			loop
					check
						is_disjoint: Result.is_disjoint (sp.any) -- Result ≍ (⋃ (Current ∖ sp))
					end
				Result := Result.batch_extended (sp.any)
				sp := sp.others
			variant
				cardinality: (# sp).as_integer_64
			end
		end

	partition_intersection alias "⋂": like any.subset_anchor
			-- Intersection of all sets in current partition
		do
			if is_singleton then
				Result := any
			else
					-- Since sets in a partition are pair-wise disjoint.
				Result := o
			end
		end

feature -- Factory

	converted (ss: STS_SET_OF_SETS [A, EQ]): like partition_anchor
			-- `ss' converted to a set partition
		require
			is_partition: ss.is_partition
		do
			if attached {like partition_anchor} ss as sp then
				Result := sp
			else
					check
						sets_are_disjoint: o_p.sets_are_disjoint (ss) -- o.is_empty
					end
				Result := o_p.batch_extended (ss)
			end
		ensure
			definition: Result ≍ ss
		end

feature -- Anchor

	partition_anchor,
	subpartition_anchor,
	superpartition_anchor: SET_PARTITION [A, EQ]
			-- <Precursor>
		do
			Result := Current
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
