﻿note
	description: "[
		Object that checks whether the properties verified within set theory hold for an implementation of {STS_COMPOSABLE_ORDERED_PAIR}
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Wed 2018-11-21 07:53:55 -0300 $"
	revision: "$Revision$"

deferred class
	COMPOSABLE_ORDERED_PAIR_PROPERTIES [W, A, B, EQ_W -> STS_EQUALITY [W], EQ_A -> STS_EQUALITY [A], EQ_B -> STS_EQUALITY [B]]

inherit
	ORDERED_PAIR_PROPERTIES [A, B, EQ_A, EQ_B]

feature -- Access

	current_universe_w: STS_TRANSFORMABLE_SET [W, B, EQ_W, EQ_B]
			-- The current "Universe", i.e. set with every object currently in system memory whose type conforms to {W}.
			-- Notice that this "universe" may change from a call to another.
		deferred
		end

	eq_w: EQ_W
			-- Equality for objects like {W}
		deferred
		end

feature -- Properties (Operation)

	composed_ok
		(ab: STS_COMPOSABLE_ORDERED_PAIR [W, A, B, EQ_W, EQ_A, EQ_B]; wa: STS_ORDERED_PAIR [W, A, EQ_W, EQ_A]; ww: STS_ORDERED_PAIR [W, W, EQ_W, EQ_W]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_COMPOSABLE_ORDERED_PAIR}.composed?
		do
			if ab ≽ wa then
				check
					u_w: attached current_universe_w as u_w
					u_b: attached current_universe_b as u_b
					
					definition: (ab ∘ wa).as_singleton ≍ (
						(u_w × u_b).filtered_xy (
							agent (x: W y: B; w: W b: B): BOOLEAN
								do
									Result := eq_w (x, w) and eq_b (y, b)
								end (?, ?, wa.first, ab.b)
							)
						)
				end
				if wa ≽ ww then
					check
						ab_o_wa_succeeds_ww: (ab ∘ wa) ≽ ww
						ab_succeeds_wa_oh_ww: ab ≽ (wa.composed_with_homogeneous (ww))
						associative: ((ab ∘ wa).composed_with_homogeneous (ww)) ≍ (ab ∘ wa.composed_with_homogeneous (ww))
					end
				end
			end
			Result := True
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
