﻿note
	description: "[
		Object that checks whether the properties verified within set theory hold for an implementation of {STS_HOMOGENEOUS_ORDERED_PAIR}
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"

deferred class
	HOMOGENEOUS_ORDERED_PAIR_PROPERTIES [A, EQ -> STS_EQUALITY [A]]

inherit
	ORDERED_PAIR_PROPERTIES [A, A, EQ, EQ]
		rename
			eq_a as eq,
			eq_b as eq,
			current_universe_a as current_universe,
			current_universe_b as current_universe,
			equals_ok as opp_equals_ok,
			composed_with_homogeneous_ok as composed_ok
		end

feature -- Access

	current_universe: STS_TRANSFORMABLE_SET [A, A, EQ, EQ]
			-- The current "Universe", i.e. set with every object currently in system memory whose type conforms to {A}.
			-- Notice that this "universe" may change from a call to another.
		deferred
		end

feature -- Properties (Quality)

	is_identical_ok (xy: STS_HOMOGENEOUS_ORDERED_PAIR [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_HOMOGENEOUS_ORDERED_PAIR}.is_identical?
		do
			check
				definition: xy.is_identical = xy ≍ xy.inverse
			end
			Result := True
		end

feature -- Properties (Comparison)

	equals_ok (x, y: A): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_HOMOGENEOUS_ORDERED_PAIR}.equals?
		do
			check
				inverse_equality: pair (x, y) ≍ pair (y, x) = eq (x, y)
			end
			Result := True
		end

feature -- Properties (Conversion)

	as_tuple_ok (xy: STS_HOMOGENEOUS_ORDERED_PAIR [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_HOMOGENEOUS_ORDERED_PAIR}.as_tuple?
		do
			check
				u: attached current_universe as u
				definition: xy.as_tuple.as_singleton ≍ (
					(u ^ 2) | agent (t: STS_N_TUPLE [A, EQ] ia_xy: STS_HOMOGENEOUS_ORDERED_PAIR [A, EQ]): BOOLEAN
						do
								check
										-- t.n = 2
									small_enough_1: 1 ≤ t.n
									small_enough_2: 2 ≤ t.n
								end
							Result := eq (t [1], ia_xy.a) and eq (t [2], ia_xy.b)
						end (?, xy)
					)
			end
			Result := True
		end

feature -- Factory

	pair (a1, a2: A): STS_HOMOGENEOUS_ORDERED_PAIR [A, EQ]
			-- Ordered pair in the form (`a1', `a2')
		deferred
		ensure
			first: eq (Result.a, a1)
			second: eq (Result.b, a2)
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
