﻿note
	description: "[
		Object that checks whether the properties verified within set theory hold for an implementation of {STS_ORDERED_PAIR_COMPOSER}
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	ORDERED_PAIR_COMPOSER_PROPERTIES [W, A, B, EQ_W -> STS_EQUALITY [W], EQ_A -> STS_EQUALITY [A], EQ_B -> STS_EQUALITY [B]]

inherit
	ELEMENT_PROPERTIES

feature -- Access

	current_universe_w: STS_TRANSFORMABLE_SET [W, B, EQ_W, EQ_B]
			-- The current "universe" of elements like {W}, i.e. set with every object currently in system memory whose type conforms to {W}.
			-- Notice that this "universe" may change from a call to another.
		deferred
		end

	current_universe_b: STS_SET [B, EQ_B]
			-- The current "universe" of elements like {B}, i.e. set with every object currently in system memory whose type conforms to {B}.
			-- Notice that this "universe" may change from a call to another.
		deferred
		end

feature -- Properties (Operation)

	composite_pair_ok (
		ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]
		pc_wab: STS_ORDERED_PAIR_COMPOSER [W, A, B, EQ_W, EQ_A, EQ_B]
		wa: STS_ORDERED_PAIR [W, A, EQ_W, EQ_A]
		ww: STS_ORDERED_PAIR [W, W, EQ_W, EQ_W]
		): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_ORDERED_PAIR_COMPOSER}.composite_pair?
		do
			if ab ≽ wa then
				check
					u_w: attached current_universe_w as u_w
					u_b: attached current_universe_b as u_b
					definition: pc_wab (ab, wa).as_singleton ≍ (
						(u_w × u_b).filtered_xy (
							agent (u_w.transformer).anded (
								agent u_w.equality_holds (?, wa.first), ?,
								agent u_b.equality_holds (?, ab.b), ?
								)
							)
						)
				end
				if wa ≽ ww then
					check
						ab_o_xa_succeeds_xx: pc_wab (ab, wa) ≽ ww
						ab_succeeds_xa_oh_xx: ab ≽ (wa.composed_with_homogeneous (ww))
						associative: (pc_wab (ab, wa).composed_with_homogeneous (ww)) ≍ pc_wab (ab, wa.composed_with_homogeneous (ww))
					end
				end
			end
			Result := True
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
