﻿note
	description: "[
		Object that checks whether the properties verified within set theory hold for an implementation of {STS_SET_OF_ORDERED_PAIRS}
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	SET_OF_ORDERED_PAIRS_PROPERTIES [A, B, EQ_A -> STS_EQUALITY [A], EQ_B -> STS_EQUALITY [B]]

inherit
	SET_PROPERTIES [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]]
		rename
			o as o_ab,
			eq as eq_ab,
			current_universe as current_universe_ab,
			pair as pair_of_pairs,
			universe_anchor as universe_ab_anchor
		end

feature -- Properties (Quantifier)

	exist_xy_ok (s_ab: STS_SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B] p: PREDICATE [A, B]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_OF_ORDERED_PAIRS}.exist_xy?
		do
			check
				by_quantification: s_ab.exist_xy (p) = not s_ab.for_all_xy (agent negated_xy (p, ?, ?))
				by_filtering: s_ab.exist_xy (p) = ((s_ab.filtered_xy (p)) ≭ o_ab)
				by_reduction:
					s_ab.exist_xy (p) = element_to_boolean.set_reduction (s_ab, False, agent s_ab.disjunction (?, agent s_ab.predicate_holds_to_xy (p, ?), ?))
			end
			Result := True
		end

	do_not_exist_xy_ok (s_ab: STS_SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B] p: PREDICATE [A, B]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_OF_ORDERED_PAIRS}.do_not_exist_xy?
		do
			check
				by_quantification: s_ab.do_not_exist_xy (p) = s_ab.for_all_xy (agent negated_xy (p, ?, ?))
				by_filtering: s_ab.do_not_exist_xy (p) = (s_ab.filtered_xy (p) ≍ o_ab)
				by_reduction: s_ab.do_not_exist_xy (p) =
					element_to_boolean.set_reduction (s_ab, True, agent s_ab.conjunction (?, agent negated(agent s_ab.predicate_holds_to_xy(p, ?), ?), ?))
			end
			Result := True
		end

	for_all_xy_ok (s_ab: STS_SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B] p: PREDICATE [A, B]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_OF_ORDERED_PAIRS}.for_all_xy?
		do
			check
				by_quantification: s_ab.for_all_xy (p) = s_ab.do_not_exist_xy (agent negated_xy (p, ?, ?))
				by_filtering: s_ab.for_all_xy (p) = (s_ab.filtered_xy (p) ≍ s_ab)
				by_reduction: s_ab.for_all_xy (p) =
					element_to_boolean.set_reduction (s_ab, True, agent s_ab.conjunction(?, agent s_ab.predicate_holds_to_xy (p, ?), ?))
			end
			Result := True
		end

feature -- Properties (Operation)

	filtered_xy_ok (s_ab: STS_SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B] p: PREDICATE [A, B]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_OF_ORDERED_PAIRS}.filtered_xy?
		do
			check
				u_ab: attached current_universe_ab as u_ab
				definition: s_ab.filtered_xy (p) ≍ (u_ab | agent anded (agent u_ab.predicate_holds_to_xy(p, ?), agent s_ab.has, ?))
				by_intersection: s_ab.filtered_xy (p) ≍ (s_ab ∩ u_ab.filtered_xy (p))
				subset: s_ab.filtered_xy (p) ⊆ s_ab
				only_compliant_elements: s_ab.filtered_xy (p).for_all_xy (p)
				nothing_lost: (s_ab ∖ s_ab.filtered_xy (p)).do_not_exist_xy (p)
				every_compliant_element: s_ab |∀ agent s_ab.iff (agent s_ab.predicate_holds_to_xy (p, ?), agent (s_ab.filtered_xy (p)).has, ?)
				nothing_else: s_ab.filtered_xy (p) ⊆ s_ab
			end
			Result := True
		end

feature -- Predicate

	negated_xy (p: PREDICATE [A, B]; x: A; y: B): BOOLEAN
			-- Logical negation of `p` (`x`, `y'), i.e. is `p` (`x`, `y') false?
		do
			Result := not p (x, y)
		ensure
			class
			definition: Result = not p (x, y)
		end

feature {NONE} -- Anchor

	universe_ab_anchor: STS_SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B]
			-- Anchor for `current_universe_ab'
		deferred
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
