﻿note
	description: "Object that checks whether the properties verified within set theory hold for an implementation of {STS_SET_OF_SETS}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	SET_OF_SETS_PROPERTIES [A, EQ -> STS_EQUALITY [A]]

inherit
	SET_PROPERTIES [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]
		rename
			o as o_s,
			o_ok as o_s_ok,
			current_universe as current_universe_s,
			empty_set_ok as empty_set_of_sets_ok,
			singleton as singleton_s,
			universe_anchor as universe_s_anchor
		end

feature -- Access

	u: like universe_s_anchor.sets_union
			-- "Universe" of elements like {A}
			-- TODO: redefine?
		do
			Result := ⋃ current_universe_s
		ensure
			definition: Result ≍ ⋃ old current_universe_s
		end

	o: like o_s.sets_union
			-- The empty set of elements like {A}
			-- TODO: redefine?
		do
			Result := ⋃ o_s
		ensure
			is_empty: Result.is_empty
		end

feature -- Properties (Quality)

	is_partition_ok (ss: STS_SET_OF_SETS [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {SET_OF_SETS}.is_partition?
		do
			check
				partitioned_union: ss.is_partition = (⋃ ss).is_partitioned (ss)
			end
			Result := True
		end

feature -- Properties (Relationship)

	set_is_disjoint_ok (s: STS_SET [A, EQ]; ss: STS_SET_OF_SETS [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {SET_OF_SETS}.set_is_disjoint?
		do
			check
				disjointness_kept: ss.member_sets_are_disjoint and ss.set_is_disjoint (s) implies (ss & s).member_sets_are_disjoint
			end
			Result := True
		end

feature -- Properties (Operation)

	sets_union_ok (ss: STS_SET_OF_SETS [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {SET_OF_SETS}.sets_union?
		do
			check
				definition: ⋃ ss ≍ (u | agent (x: A; ia_ss: STS_SET_OF_SETS [A, EQ]): BOOLEAN
					do
						Result := ia_ss |∃ agent {STS_SET [A, EQ]}.has (x)
					end (?, ss))
				by_reduction: ⋃ ss ≍ ss.reduced (o, agent {STS_SET [A, EQ]}.united)
			end
			Result := True
		end

	sets_intersection_ok (ss: STS_SET_OF_SETS [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {SET_OF_SETS}.sets_intersection?
		do
			check
				definition: ⋂ ss ≍ ss.reduced (ss.universe, agent {STS_SET [A, EQ]}.intersected)
				within_universe: ⋂ ss ⊆ ss.universe
				nullary_intersection: ss ≍ o_s implies ⋂ ss ≍ ss.universe
			end
			Result := True
		end

feature -- Properties (Factory)

	o_ok (ss: STS_SET_OF_SETS [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_OF_SETS}.o?
			--| To assess the relatioship between `o_ok' and `empty_member_set_ok', please see the note at the header of {STS_SET_PROPERTIES}.subsets_ok.
		do
			check
				definition: ss.o ≍ (
					u | agent (x: A): BOOLEAN
						do
							Result := False
						end
					)
				has_nothing: u |∀ agent (ss.o).does_not_have
				uniqueness: ss.o ≍ o
				no_element: # ss.o = 0
			end
			Result := True
		end

	empty_member_set_ok (ss: STS_SET_OF_SETS [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_OF_SETS}.empty_member_set?
			--| Please see the comment at `o_ok' header.
		do
			check
				definition: ss.empty_member_set ≍ (
					u | agent (x: A): BOOLEAN
						do
							Result := False
						end
					)
				has_nothing: u |∀ agent (ss.empty_member_set).does_not_have
				uniqueness: ss.empty_member_set ≍ o
				no_element: # ss.empty_member_set = 0
			end
			Result := True
		end

feature {NONE} -- Anchor

	universe_s_anchor: STS_SET_OF_SETS [A, EQ]
			-- Anchor for `current_universe_s'
		deferred
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
