﻿note
	description: "Object that checks whether the properties verified within set theory hold for an implementation of {STS_SET_PARTITION}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	SET_PARTITION_PROPERTIES [A, EQ -> STS_EQUALITY [A]]

inherit
	SET_OF_SETS_PROPERTIES [A, EQ]
		rename
			with_ok as family_with_ok
		end

feature -- Properties (Construction)

	with_ok (sp: STS_SET_PARTITION [A, EQ]; s: STS_SET [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_PARTITION}.with?
		do
			if not s.is_empty and (sp / s).set_is_disjoint (s) then
				check
					u_s: attached current_universe_s as u_s
					definition: (sp & s) ≍ (u_s | agent u_s.ored (agent sp.has, agent s.equals, ?))
					inserted: s ∈ (sp & s)
					nothing_lost: sp ⊆ (sp & s)
					nothing_else: (sp & s) |∀ agent sp.ored (agent sp.has, agent s.equals, ?)
				end
			end
			Result := True
		end

feature -- Properties (Operation)

	partition_union_ok (sp: STS_SET_PARTITION [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_PARTITION}.partition_union?
		do
			if sets_union_ok (sp) then
				check
					cardinality: # (⋃ sp) = sp.set_to_natural.set_reduction (
						sp, 0, agent (acc: NATURAL; s: STS_SET [A, EQ]): NATURAL
							do
								Result := acc + # s
							end
						)
				end
				Result := True
			end
		end

	partition_intersection_ok (sp: STS_SET_PARTITION [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SET_PARTITION}.partition_intersection?
		do
			if sets_intersection_ok (sp) then
				check
					non_empty_intersection: sp.is_singleton implies not sp.is_empty and then ⋂ sp ≍ sp.any
					empty_intersection: not sp.is_singleton implies ⋂ sp ≍ o
				end
				Result := True
			end
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
