﻿note
	description: "Object that checks whether the properties verified within set theory hold for an implementation of {STS_SUBSET_FAMILY}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	SUBSET_FAMILY_PROPERTIES [A, EQ -> STS_EQUALITY [A]]

inherit
	SET_OF_SETS_PROPERTIES [A, EQ]

feature -- Properties (Comparison)

	same_family_ok (sf1, sf2: STS_SUBSET_FAMILY [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SUBSET_FAMILY}.same_family?
		do
			check
				same_complement: 2 ^ # sf1.superset <= max_subsets and 2 ^ # sf2.superset <= max_subsets implies sf1.same_family (sf2) = ∁ sf1 ≍ ∁ sf2
			end
			Result := True
		end

feature -- Properties (Operation)

	complemented_family_ok (sf: STS_SUBSET_FAMILY [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SUBSET_FAMILY}.complemented_family?
		do
			check
				power_card: attached (2 ^ # sf.superset) as power_card
				definition: power_card <= max_subsets implies ∁ sf ≍ (sf.superset.powerset ∖ sf)
				subfamily: power_card <= max_subsets implies ∁ sf ⊆ sf.superset.powerset
				disjoint: power_card <= max_subsets implies sf.is_disjoint (∁ sf)
				complementary: power_card <= max_subsets implies (sf ∪ ∁ sf) ≍ sf.superset.powerset
				involutive: power_card <= max_subsets implies ∁ ∁ sf ≍ sf
			end
			Result := True
		end

	family_union_ok (sf: STS_SUBSET_FAMILY [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SUBSET_FAMILY}.family_union?
		do
			if sets_union_ok (sf) then
				check
					definition: ⋃ sf ≍ (sf.superset | agent sf.some_subset_has)
				end
				Result := True
			end
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
