﻿note
	description: "Set of integer numbers, i.e. a subset of ℤ."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	INTEGER_SET

inherit
	ORDERED_NUMERIC_SET [INTEGER_NUMBER, INTEGER_EQUALITY]
		rename
			u as z,
			universe as integer_universe
		redefine
			set_anchor,
			subset_anchor,
			superset_anchor
		end

feature -- Access

	zero: like integer_anchor
			-- The integer number zero
		deferred
		end

feature -- Conversion

	as_complex_set: COMPLEX_SET
			-- Current integer set converted into a set of complex numbers
		deferred
		ensure
--			every_integer_number: Current |∀ agent Result.has -- TODO: Contravariance would be ok.
			every_integer_number: Current |∀ agent complex_set_has_integer_number (Result, ?)
			nothing_else: Result |∀ agent has_complex_number
		end

	as_real_set: REAL_SET
			-- Current integer set converted into a set of real numbers
		deferred
		ensure
			every_integer_number: Current |∀ agent real_set_has_integer_number (Result, ?)
			nothing_else: Result |∀ agent has_real_number
		end

feature -- Predicate

	complex_set_has_integer_number (zs: NUMERIC_SET [COMPLEX_NUMBER, COMPLEX_EQUALITY]; i: INTEGER_NUMBER): BOOLEAN
			-- Does `zs' have `i'?
		do
			Result := zs ∋ i
		ensure
			class
			definition: Result = zs ∋ i
		end

	has_complex_number (ab: COMPLEX_NUMBER): BOOLEAN
			-- Does current set have `ab'?
		do
			Result := ab.is_integer and ab.a.truncated_to_integer ∈ Current
		ensure
			definition: Result = (ab.is_integer and ab.a.truncated_to_integer ∈ Current)
		end

	real_set_has_integer_number (xs: NUMERIC_SET [REAL_NUMBER, REAL_EQUALITY]; i: INTEGER_NUMBER): BOOLEAN
			-- Does `xs' have `i'?
		do
			Result := xs ∋ i
		ensure
			class
			definition: Result = xs ∋ i
		end

	has_real_number (x: REAL_NUMBER): BOOLEAN
			-- Does current set have `x'?
		do
			Result := x.is_integer and x.truncated_to_integer ∈ Current
		ensure
			definition: Result = (x.is_integer and x.truncated_to_integer ∈ Current)
		end

feature -- Anchor

	set_anchor,
	subset_anchor,
	superset_anchor: INTEGER_SET
			-- <Precursor>
		deferred
		end

	integer_anchor: INTEGER_NUMBER
			-- Anchor for integer numbers
		deferred
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
