﻿note
	description: "Set of rational numbers, i.e. a subset of ℚ."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	RATIONAL_SET

inherit
	ORDERED_NUMERIC_SET [RATIONAL_NUMBER, RATIONAL_EQUALITY]
		rename
			u as q,
			universe as rational_universe
		redefine
			set_anchor,
			subset_anchor,
			superset_anchor
		end

feature -- Access

	zero: like integer_anchor
			-- The integer number zero
		deferred
		end

feature -- Conversion

	as_complex_set: COMPLEX_SET
			-- Current rational set converted into a set of complex numbers
		deferred
		ensure
--			every_rational_number: Current |∀ agent Result.has -- TODO: Contravariance would be ok.
			every_rational_number: Current |∀ agent complex_set_has_rational_number (Result, ?)
			nothing_else: Result |∀ agent has_complex_number
		end

	as_real_set: REAL_SET
			-- Current rational set converted into a set of real numbers
		deferred
		ensure
			every_rational_number: Current |∀ agent real_set_has_rational_number (Result, ?)
			nothing_else: Result |∀ agent has_real_number
		end

feature -- Predicate

	complex_set_has_rational_number (zs: NUMERIC_SET [COMPLEX_NUMBER, COMPLEX_EQUALITY]; pq: RATIONAL_NUMBER): BOOLEAN
			-- Does `zs' have `pq'?
		do
			Result := zs ∋ pq
		ensure
			class
			definition: Result = zs ∋ pq
		end

	has_complex_number (z: COMPLEX_NUMBER): BOOLEAN
			-- Does current set have `z'?
		do
				check
					is_rational: z.is_rational implies z.a.is_rational -- By definition.
				end
			Result := z.is_rational and then z.a.to_rational ∈ Current
		ensure
				is_rational: z.is_rational implies z.a.is_rational -- By definition.
			definition: Result = (z.is_rational and then z.a.to_rational ∈ Current)
		end

	real_set_has_rational_number (xs: NUMERIC_SET [REAL_NUMBER, REAL_EQUALITY]; pq: RATIONAL_NUMBER): BOOLEAN
			-- Does `xs' have `i'?
		do
			Result := xs ∋ pq
		ensure
			class
			definition: Result = xs ∋ pq
		end

	has_real_number (x: REAL_NUMBER): BOOLEAN
			-- Does current set have `x'?
		do
			Result := x.is_rational and then x.to_rational ∈ Current
		ensure
			definition: Result = (x.is_rational and then x.to_rational ∈ Current)
		end

feature -- Anchor

	set_anchor,
	subset_anchor,
	superset_anchor: RATIONAL_SET
			-- <Precursor>
		deferred
		end

	integer_anchor: INTEGER_NUMBER
			-- Anchor for integer numbers
		deferred
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
