﻿note
	description: "N-tuple whose terms are sets"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	SET_TUPLE [A, EQ -> EQUALITY [A]]

inherit
	N_TUPLE [SET [A, EQ], SET_EQUALITY [A, EQ]]
		rename
			term as set alias "[]",
			terms as sets,
			unit as unit_s,
			converted_terms as converted_sets,
			term_to_term as set_to_set,
			term_to_index as set_to_index,
			predicate_holds_to_term as predicate_holds_to_set,
			terms_are_equal as sets_are_equal,
			reversed_terms_are_equal as reversed_sets_are_equal,
			indexed_term as indexed_set,
			terms_anchor as sets_anchor
		redefine
			tuple_anchor,
			trimmed_tuple_anchor,
			extended_tuple_anchor,
			sets_anchor
		end

feature -- Quantifier

	for_all_tuples (p: PREDICATE [N_TUPLE [A, EQ]]): BOOLEAN
			-- Does `p' hold for every possible tuple (x_1, x_2, ..., x_`n'), where x_i ∈ `Current' [i]?
		do
			Result := ∏ Current |∀ p
		ensure
			definition: Result = (∏ Current |∀ p)
		end

feature -- Operation

	n_ary_union alias "⋃": like first.superset_anchor
			-- Union of all sets in current set tuple
		do
			Result := ⋃ sets
		ensure
			definition: Result ≍ (⋃ sets)
		end

	n_ary_intersection alias "⋂": like first.subset_anchor
			-- Intersection of all sets in current set tuple
		do
			Result := ⋂ sets
		ensure
			definition: Result ≍ (⋂ sets)
		end

	n_ary_product alias "∏": SET [N_TUPLE [A, EQ], N_TUPLE_EQUALITY [A, EQ]]
			-- Cartesian product of all sets in current set tuple, i.e. set with every tuple (x_1, x_2, ..., x_`n'), where x_i ∈ `set' [i].
		deferred
		ensure
			definition: Result ≍ set_to_tuples.tuple_right_reduction (Current, agent {SET [A, EQ]}.crossed_tuples, unit.as_singleton)
		end

feature -- Factory

	o,
	empty_set: like first.subset_anchor
			-- Set like `first' with no element
		deferred
		ensure
			definition: Result.is_empty
		end

	unit: like first.n_tuple_anchor
			-- N-tuple with no term, i.e. ().
		deferred
		ensure
			definition: Result.is_unit
		end

feature -- Transformer

	set_to_tuples: TRANSFORMER [
		SET [A, EQ], SET [N_TUPLE [A, EQ], N_TUPLE_EQUALITY [A, EQ]],
		SET_EQUALITY [A, EQ], SET_EQUALITY [N_TUPLE [A, EQ], N_TUPLE_EQUALITY [A, EQ]]
		]
			-- Transformer between types derived from {STS_SET [A, EQ]} and {STS_SET [STS_N_TUPLE [A, EQ], STS_N_TUPLE_EQUALITY [A, EQ]]}
		deferred
		end

feature -- Anchor

	tuple_anchor,
	trimmed_tuple_anchor,
	extended_tuple_anchor: SET_TUPLE [A, EQ]
			-- <Precursor>
		deferred
		end

	sets_anchor: SET_FAMILY [A, EQ]
			-- Anchor for set families whose member sets are like those in current set tuple
		deferred
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
