﻿note
	description: "N-tuple equipped with operation that transform objects derived from a generic parameter into objects derived from another generic parameter."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Wed 2018-11-21 07:53:55 -0300 $"
	revision: "$Revision$"

deferred class
	TRANSFORMABLE_TUPLE [A, B, EQ_A -> EQUALITY [A], EQ_B -> EQUALITY [B]]

inherit
	N_TUPLE [A, EQ_A]
		rename
			eq as eq_a,
			mapped as mapped_to_same_kind,
			left_reduced as left_reduced_to_term_like,
			indexed_left_reduced as indexed_left_reduced_to_term_like,
			right_reduced as right_reduced_to_term_like,
			indexed_right_reduced as indexed_right_reduced_to_term_like
		redefine
			tuple_anchor,
			trimmed_tuple_anchor,
			extended_tuple_anchor,
			terms_anchor
		end

feature -- Primitive

	eq_b: EQ_B
			-- Equality for terms like {B}
		deferred
		end

feature -- Transformation

	mapped alias "↦" (f: FUNCTION [A, B]): like transformer.tuple_map
			-- Current n-tuple mapped to an n-tuple of terms like {B}
		do
			Result := transformer.tuple_map (Current, f)
		ensure
			definition: Result ≍ transformer.tuple_map (Current, f)
		end

	left_reduced (leftmost: B; f: FUNCTION [B, A, B]): B
			-- Current n-tuple left-reduced to a value like `leftmost'
		do
			Result := transformer.tuple_left_reduction (Current, leftmost, f)
		ensure
			definition: eq_b (Result, transformer.tuple_left_reduction (Current, leftmost, f))
		end

	indexed_left_reduced (leftmost: B i: NATURAL f: FUNCTION [B, NATURAL, A, B]): B
			-- Current n-tuple indexed left-reduced to a value like `leftmost'
		do
			Result := transformer.tuple_indexed_left_reduction (Current, leftmost, i, f)
		ensure
			definition: eq_b (Result, transformer.tuple_indexed_left_reduction (Current, leftmost, i, f))
		end

	right_reduced (f: FUNCTION [A, B, B]; rightmost: B): B
			-- Current n-tuple right-reduced to a value like `rightmost'
		do
			Result := transformer.tuple_right_reduction (Current, f, rightmost)
		ensure
			definition: eq_b (Result, transformer.tuple_right_reduction (Current, f, rightmost))
		end

	indexed_right_reduced (i: NATURAL; f: FUNCTION [NATURAL, A, B, B]; rightmost: B): B
			-- Current n-tuple indexed right-reduced to a value like `rightmost'
		do
			Result := transformer.tuple_indexed_right_reduction (Current, i, f, rightmost)
		ensure
			definition: eq_b (Result, transformer.tuple_indexed_right_reduction (Current, i, f, rightmost))
		end

feature -- Transformer

	transformer: TRANSFORMER [A, B, EQ_A, EQ_B]
			-- Transformer between types derived from {A} and {B}
		deferred
		end

feature -- Anchor

	tuple_anchor,
	trimmed_tuple_anchor,
	extended_tuple_anchor: TRANSFORMABLE_TUPLE [A, B, EQ_A, EQ_B]
			-- <Precursor>
		deferred
		end

	terms_anchor: TRANSFORMABLE_SET [A, B, EQ_A, EQ_B]
			-- <Precursor>
		deferred
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
