﻿note
	description: "Set of ordered pairs"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	SET_OF_ORDERED_PAIRS [A, B, EQ_A -> EQUALITY [A], EQ_B -> EQUALITY [B]]

inherit
	SET [ORDERED_PAIR [A, B, EQ_A, EQ_B], ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]]
		rename
			element_to_element as pair_to_pair,
			element_to_natural as pair_to_natural,
			element_to_boolean as pair_to_boolean,
			element_to_tuple as pair_to_tuple,
			element_to_sets as pair_to_sets
		redefine
			set_anchor,
			subset_anchor,
			superset_anchor
		end

feature -- Quantifier

	exist_xy (p: PREDICATE [A, B]): BOOLEAN
			-- Does `p' (xy.`a', xy.`b') hold for some pair xy in current set?
			--| NOTICE: See comments at {SET}.exists header.
		do
			Result := Current |∃ agent predicate_holds_to_xy (p, ?)
		ensure
			definition: Result = (Current |∃ agent predicate_holds_to_xy (p, ?))
		end

	do_not_exist_xy (p: PREDICATE [A, B]): BOOLEAN
			-- Does `p' (xy.`a', xy.`b') hold for no pair xy in current set?
			--| NOTICE: See comments at {SET}.exists header.
		do
			Result := not exist_xy (p)
		ensure
			definition: Result = not exist_xy (p)
		end

	for_all_xy (p: PREDICATE [A, B]): BOOLEAN
			-- Does `p' (xy.`a', xy.`b') hold for every pair xy in current set?
			--| NOTICE: See comments at {SET}.exists header.
		do
			Result := Current |∀ agent predicate_holds_to_xy (p, ?)
		ensure
			definition: Result = (Current |∀ agent predicate_holds_to_xy (p, ?))
		end

feature -- Operation

	filtered_xy (p: PREDICATE [A, B]): like subset_anchor
			-- Every ordered pair xy in current set for which `p' (xy.a, xy.b) holds
			--| todo: "|ˣʸ" gets a syntax error.
			--| reported: https://support.eiffel.com/report_detail/19535
		do
			Result := Current | agent predicate_holds_to_xy (p, ?)
		ensure
			definition: Result ≍ (Current | agent predicate_holds_to_xy (p, ?))
		end

feature -- Predicate

	predicate_holds_to_xy (p: PREDICATE [A, B]; xy: ORDERED_PAIR [A, B, EQ_A, EQ_B]): BOOLEAN
			-- Does `p' hold to (x, y) = (`xy'.`a', `xy'.`b')?
			-- NOTICE: Please see the comment at the header of {SET}.ored.
		do
			Result := p (xy.a, xy.b)
		ensure
			class
			definition: Result = p (xy.a, xy.b)
		end

feature -- Anchor

	set_anchor,
	subset_anchor,
	superset_anchor: SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B]
			-- <Precursor>
		deferred
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
