﻿note
	description: "Set whose elements are sets"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

deferred class
	SET_OF_SETS [A, EQ -> EQUALITY [A]]

inherit
	SET [SET [A, EQ], SET_EQUALITY [A, EQ]]
		rename
			o as o_s,
			empty_set as empty_set_of_sets,
			unit as unit_s,
			element_to_element as set_to_set,
			element_to_natural as set_to_natural,
			element_to_boolean as set_to_boolean,
			element_to_tuple as set_to_tuple,
			n_tuple_anchor as set_tuple_anchor
		redefine
			set_anchor,
			subset_anchor,
			superset_anchor,
			set_tuple_anchor
		end

feature -- Primitive

	universe: like any.superset_anchor
			-- Set where the elements of (member) sets of current set (of sets) come from
		deferred
		end

feature -- Quality

	is_partition: BOOLEAN
			-- Is current set (of sets) a partition, i.e. are the (member) sets in current set (of sets) non-empty and pair-wise disjoint?
			-- todo: constant in partitions vs. invariant checking
		do
			Result := o ∉ Current and member_sets_are_disjoint
		ensure
			definition: Result = (o ∉ Current and member_sets_are_disjoint)
		end

	member_sets_are_disjoint: BOOLEAN
			-- Are the (member) sets in current set (of sets) pair-wise disjoint?
			-- todo: constant in partitions vs. invariant checking
		do
			Result := for_all_distinct_pairs (agent {SET [A, EQ]}.is_disjoint)
		ensure
			definition: Result = for_all_distinct_pairs (agent {SET [A, EQ]}.is_disjoint)
		end

feature -- Relationship

	set_is_disjoint (s: SET [A, EQ]): BOOLEAN
			-- Is `s' disjoint from every (member) set in current set (of sets)?
		do
			Result := Current |∀ agent s.is_disjoint
		ensure
			definition: Result = (Current |∀ agent s.is_disjoint)
		end

	sets_are_disjoint (ss: SET [SET [A, EQ], SET_EQUALITY [A, EQ]]): BOOLEAN
			-- Is every (member) set in `ss' disjoint from every (member) set in current set (of sets)?
		do
			Result := ss |∀ agent set_is_disjoint
		ensure
			definition: Result = (ss |∀ agent set_is_disjoint)
		end

feature -- Operation

	sets_union alias "⋃": like any.superset_anchor
			-- Union of all (member) sets in current set (of sets)
		deferred
		ensure
			every_element: Current |∀ agent Result.is_superset
			nothing_else: Result |∀ agent some_set_has
		end

	sets_intersection alias "⋂": like any.subset_anchor
			-- Intersection of all (member) sets in current set (of sets)
		do
			Result := universe | agent every_set_has
		ensure
			definition: Result ≍ (universe | agent every_set_has)
		end

feature -- Factory

	o,
	empty_member_set: like any.subset_anchor
			-- Empty (member) set like `any'
		deferred
		ensure
			definition: Result.is_empty
		end

feature -- Predicate

	some_set_has (x: A): BOOLEAN
			-- Does some (member) set in current set (of sets) have `x'?
			-- NOTICE: Please see the comment at the header of {SET}.ored.
		do
			Result := Current |∃ agent {SET [A, EQ]}.has (x)
		ensure
			definition: Result = (Current |∃ agent {SET [A, EQ]}.has (x))
		end

	every_set_has (x: A): BOOLEAN
			-- Does every (member) set in current set (of sets) have `x'?
			-- NOTICE: Please see the comment at the header of {SET}.ored.
		do
			Result := Current |∀ agent {SET [A, EQ]}.has (x)
		ensure
			definition: Result = (Current |∀ agent {SET [A, EQ]}.has (x))
		end

feature -- Anchor

	set_anchor,
	subset_anchor,
	superset_anchor: SET_OF_SETS [A, EQ]
			-- <Precursor>
		deferred
		end

	set_tuple_anchor: SET_TUPLE [A, EQ]
			-- Anchor for set tuples whose sets conform to the (member) sets of current set (of sets)
		deferred
		end

invariant
	universe_includes_every_member_set: Current |∀ agent universe.is_superset

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
