﻿note
	description: "[
		Ordered pair in the form (sf, s), where sf is a {SUBSET_FAMILY} and s is a {SET}; besides s equals sf.superset.
		The rationale is the need to model the fact that two subset families may be equal according to {SET}.equals and yet they may have different supersets, what may lead some of the
		features of such "equal" families to have different values, e.g. {SUBSET_FAMILY}.complemented_family.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	SUBSET_FAMILY_STRUCTURE [A, EQ -> EQUALITY [A]]

inherit
	ORDERED_PAIR [
		SET [SET [A, EQ], SET_EQUALITY [A, EQ]], SET [A, EQ],
		SET_EQUALITY [SET [A, EQ], SET_EQUALITY [A, EQ]], SET_EQUALITY [A, EQ]
		]
		rename
			a as sf,
			first as subset_family,
			b as s,
			second as superset,
			eq_a as eq_ss,
			eq_b as eq_s
		end

	SUBSET_FAMILY [A, EQ]
		rename
			eq as eq_s,
			is_in as set_is_in,
			is_not_in as set_is_not_in,
			equals as set_equals,
			unequals as set_unequals,
			as_singleton as set_as_singleton,
			family_anchor as family_structure_anchor,
			subfamily_anchor as subfamily_structure_anchor,
			superfamily_anchor as superfamily_structure_anchor
		redefine
			family_structure_anchor,
			subfamily_structure_anchor,
			superfamily_structure_anchor
		end

feature -- Primitive

	sf,
	subset_family: like family_structure_anchor
			-- Subset family of current subset-family structure
		deferred
		end

	s,
	superset: like any.superset_anchor
			-- Set where the elements of subsets in `subset_family' come from
		deferred
		end

feature -- Anchor

	family_structure_anchor,
	subfamily_structure_anchor,
	superfamily_structure_anchor: SUBSET_FAMILY_STRUCTURE [A, EQ]
			-- <Precursor>
		deferred
		end

invariant
	same_subsets: set_equals (sf)
	same_superset: sf.s ≍ s

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
