﻿note
	description: "[
		Set equipped with operations that transform objects derived from a generic parameter into objects derived from another generic parameter.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Wed 2018-11-21 07:53:55 -0300 $"
	revision: "$Revision$"

deferred class
	TRANSFORMABLE_SET [A, B, EQ_A -> EQUALITY [A], EQ_B -> EQUALITY [B]]

inherit
	SET [A, EQ_A]
		rename
			eq as eq_a,
			crossed as crossed_with_same_kind,
			mapped as mapped_to_same_kind,
			reduced as reduced_to_element_like
		redefine
			set_anchor,
			subset_anchor,
			superset_anchor,
			n_tuple_anchor
		end

feature -- Primitive

	eq_b: EQ_B
			-- Equality for elements like {B}
		deferred
		end

feature -- Operation

	crossed alias "×" (s: SET [B, EQ_B]): like transformer.cartesian_product
			-- Current set crossed with `s'
		do
			Result := transformer.cartesian_product (Current, s)
		ensure
			definition: Result ≍ transformer.cartesian_product (Current, s)
		end

feature -- Transformation

	mapped alias "↦" (f: FUNCTION [A, B]): like transformer.set_map
			-- Current set mapped by `f' to a set of elements like {B}
		do
			Result := transformer.set_map (Current, f)
		ensure
			definition: Result ≍ transformer.set_map (Current, f)
		end

	reduced (start: B; f: FUNCTION [B, A, B]): B
			-- Current set reduced by `f 'to a value like `start'
			-- NOTICE: Since the order of the elements in a set is irrelevant, the value of `reduced' may differ for two equal sets, unless `f' is
			-- "commutative", as explained at "Painfully detailed information about ambiguous folds" (https://preview.tinyurl.com/y4qp7zre).
		do
			Result := transformer.set_reduction (Current, start, f)
		ensure
			definition: eq_b (Result, transformer.set_reduction (Current, start, f))
		end

feature -- Factory

	transformer: TRANSFORMER [A, B, EQ_A, EQ_B]
			-- Transformer between types derived from {A} and {B}
		deferred
		end

feature -- Anchor

	set_anchor,
	subset_anchor,
	superset_anchor: TRANSFORMABLE_SET [A, B, EQ_A, EQ_B]
			-- <Precursor>
		deferred
		end

	n_tuple_anchor: TRANSFORMABLE_TUPLE [A, B, EQ_A, EQ_B]
			-- <Precursor>
		deferred
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
