﻿note
	description: "[
		Descendant of {STI_REAL_NUMBER} whose features with restricted access are available to be tested. Notice that inheriting from an expanded class is a
		non-conformant inheritance.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

expanded class
	REAL_NUMBER

inherit
	STI_REAL_NUMBER
		export {REAL_NUMBER_PROPERTIES, REAL_NUMBER_PROPERTIES, NUMBERS_TESTS_RESOURCES}
			value_bit_pattern,
			bit_pattern_from_native_real,
			sign_bit_status,
			exponent_bit_pattern,
			mantissa_bit_pattern,
			Exponent_width,
			Exponent_mask,
			max_exponent_bit_pattern,
			Exponent_bias,
			max_exponent,
			min_normal_exponent,
			min_subnormal_exponent,
			Mantissa_width,
			Mantissa_mask,
			Max_mantissa,
			Mantissa_scale,
			Value_bit_pattern_mask
		redefine
			next_float,
			equals,
			min,
			max,
			modulus, abs,
			argument, phase,
			conjugate,
			opposite
		end

create
	default_create,
	make,
	make_from_native_real_ref,
	make_from_native_double_real,
	make_from_native_double_real_ref,
	make_from_real,
	make_from_native_integer,
	make_from_native_integer_ref,
	make_from_integer,
	make_from_native_natural,
	make_from_native_natural_ref,
	make_abs,
	make_opposite

create {NUMBERS_TESTS_RESOURCES}
	make_from_bit_pattern

convert
	make ({REAL}),
	make_from_native_real_ref ({REAL_REF}),
	make_from_native_double_real ({DOUBLE}),
	make_from_native_double_real_ref ({DOUBLE_REF}),
	make_from_real ({STS_REAL_NUMBER, STI_REAL_NUMBER}),
	make_from_native_integer ({INTEGER}),
	make_from_native_integer_ref({INTEGER_REF}),
	make_from_integer ({STS_INTEGER_NUMBER, STI_INTEGER_NUMBER}), -- TODO: Adapt {STI_INTEGER_NUMBER} instead.
	make_from_native_natural ({NATURAL}),
	make_from_native_natural_ref ({NATURAL_32_REF}),
	value: {REAL, REAL_REF, NUMERIC, COMPARABLE, HASHABLE},
	as_complex: {STI_COMPLEX_NUMBER},
	as_parent: {STI_REAL_NUMBER}

feature -- Access

	next_float: like real_anchor
			-- <Precursor>
		do
			Result := Precursor {STI_REAL_NUMBER}
		end

feature -- Comparison

	equals alias "≍" (x: STS_REAL_NUMBER): BOOLEAN
			-- <Precursor>
		do
			Result := as_parent ≍ x
		end

	min alias "∧" (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
--			Result := Precursor {STI_REAL_NUMBER}(x)
			Result := as_parent ∧ x -- TODO: Don't call Precursor; it stumbles on a bug in the compiler?
		end

	max alias "∨" (x: STS_REAL_NUMBER): like real_anchor
			-- <Precursor>
		do
--			Result := Precursor {STI_REAL_NUMBER}(x)
			Result := as_parent ∨ x -- TODO: Don't call Precursor; it stumbles on a bug in the compiler?
		end

feature -- Operation

	modulus,
	abs: like real_anchor
			-- <Precursor>
		do
			Result := as_parent.abs -- Don't call Precursor; it causes a CATcall.
		end

	argument,
	phase: like real_anchor
			-- <Precursor>
		do
			Result := as_parent.argument -- Don't call Precursor; it causes a CATcall.
		end

	conjugate: like Complex_anchor
			-- <Precursor>
		do
			Result := as_parent.conjugate
		end

	opposite alias "-" alias "−": like real_anchor
			-- <Precursor>
		do
			Result := - as_parent -- Don't call Precursor; it causes a CATcall.
		end

feature -- Conversion

	as_parent: like real_anchor
			-- Current real number converted to the type of its parent
		do
				check
					no_extra_bit: value_bit_pattern & (⊝ Value_bit_pattern_mask) = 0 -- By definition
				end
			create Result.make_from_bit_pattern (value_bit_pattern)
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
