﻿note
	description: "Implementation of {STP_COMPLEX_NUMBER_PROPERTIES}."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	COMPLEX_NUMBER_PROPERTIES

inherit
	STP_COMPLEX_NUMBER_PROPERTIES
		redefine
			default_create,
			machine_epsilon,
			equals_ok,
			unequals_ok
		end

feature {NONE} -- Initialization

	default_create
			-- <Precursor>
		do
			create complex_set_properties
			create real_set_properties
		end

feature -- Access

	Zero: STI_COMPLEX_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	One: STI_COMPLEX_NUMBER
			-- <Precursor>
		once
			Result := {STI_COMPLEX_NUMBER}.One
		ensure then
			class
		end

	Two: STI_INTEGER_NUMBER
			-- <Precursor>
		once
			create Result.make (2)
		ensure then
			class
		end

--	Three: STI_INTEGER_NUMBER
--			-- <Precursor>
--		once
--			create Result.make (3)
--		ensure then
--			class
--		end

	Machine_epsilon: STI_REAL_NUMBER
			-- <Precursor>
		 once
		 	Result := {STI_REAL_NUMBER}.machine_epsilon
		 ensure then
		 	class
		 end

	i,
	imaginary_unit: STI_COMPLEX_NUMBER
			-- <Precursor>
		once
			Result := {STI_COMPLEX_NUMBER}.I
		ensure then
			class
		end

	current_complex_universe: like complex_universe_anchor
			-- <Precursor>
		do
			Result := complex_set_properties.c
		end

	current_real_universe: like real_universe_anchor
			-- <Precursor>
		do
			Result := real_set_properties.r
		end

	complex_set_properties: COMPLEX_SET_PROPERTIES
			-- Object that checks whether the properties verified within set theory hold for an implementation of {STS_COMPLEX_SET}

	real_set_properties: REAL_SET_PROPERTIES
			-- Object that checks whether the properties verified within set theory hold for an implementation of {STS_REAL_SET}

feature -- Properties (Initialization)

	default_create_ok (z: STI_COMPLEX_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_NUMBER}.default_create?
		local
			l_broken_left_absorption, l_broken_right_absorption, l_broken_dividend_absorption: BOOLEAN
			z0: like complex_anchor -- NOTICE: Default created.
		do
			if not (l_broken_left_absorption or l_broken_right_absorption or l_broken_dividend_absorption) then
				check
					zero: z0 ≍ zero
					real: z0.is_real
					imaginary: z0.is_imaginary
					not_proper_real: not z0.is_proper_real
					not_proper_imaginary: not z0.is_proper_imaginary
					not_proper_complex: not z0.is_proper_complex
					equal_to_opposite: z0 ≍ - z0
					neutral_left_term: (z0 + z) ≍ z
					neutral_right_term: (z + z0) ≍ z
					quasi_neutral_minuend: (z0 - z) ≍ - z
					neutral_subtrahend: (z - z0) ≍ z
					absorbing_left_factor: (z0 ⋅ z) ≍ z0
				then
				end
			end
			if not (l_broken_right_absorption or l_broken_dividend_absorption) then
				check
					absorbing_right_factor: (z ⋅ z0) ≍ z0
				then
				end
			end
			if not l_broken_dividend_absorption then
				check
					absorbing_dividend: z0.divisible (z) implies (z0 / z) ≍ z0
				then
				end
			end
			Result := True
		rescue
			print (z) print ('%N')
			if not l_broken_left_absorption and (z0 ⋅ z) ≭ z0 then
				check
					absorbing_left_factor_unfolded_definition_a: (z0 ⋅ z).a ≍ if z ≍ zero then
							zero.a
						elseif z.is_proper_imaginary then
							- (zero.a * z.b)
						elseif z.is_proper_real then
							zero.a * z.a
						else
							zero.a * z.a - zero.a * z.b
						end
					absorbing_left_factor_unfolded_definition_b: (z0 ⋅ z).b ≍ if z ≍zero then
							zero.a
						elseif z.is_proper_imaginary then
							zero.a
						elseif z.is_proper_real then
							zero.a
						else
							zero.a * z.b + zero.a * z.a
						end
					broken_left_absorption:
						z.is_proper_imaginary and not z.b.is_finite or z.is_proper_real and not z.a.is_finite or z.is_proper_complex and not (z.a.is_finite and z.b.is_finite)
					broken_left_absorption_a: z.is_proper_imaginary and not z.b.is_finite or
						z.is_proper_real and not z.a.is_finite or
						z.is_proper_complex and not (z.a.is_finite and z.b.is_finite) implies
						(z0 ⋅ z).a.is_nan
					broken_left_absorption_b: z.is_proper_complex and not (z.a.is_finite and z.b.is_finite) implies (z0 ⋅ z).b.is_nan
				then
					l_broken_left_absorption := True
					retry
				end
			elseif not l_broken_right_absorption and (z ⋅ z0) ≭ z0 then
				check
					absorbing_right_factor_unfolded_definition_a: (z ⋅ z0).a ≍ if z ≍ zero then
							zero.a
						elseif z.is_proper_imaginary then
							- z.b ⋅ zero.a
						elseif z.is_proper_real then
							z.a ⋅ zero.a
						else
							z.a * zero.a - z.b * zero.a
						end
					absorbing_right_factor_unfolded_definition_b: (z ⋅ z0).b ≍ if z ≍ zero then
							zero.a
						elseif z.is_proper_imaginary then
							zero.a
						elseif z.is_proper_real then
							zero.a
						else
							z.a * zero.a + z.b * zero.a
						end
					broken_right_absorption:
						z.is_proper_imaginary and not z.b.is_finite or z.is_proper_real and not z.a.is_finite or z.is_proper_complex and not (z.a.is_finite and z.b.is_finite)
					broken_right_absorption_a: z.is_proper_imaginary and not z.b.is_finite or
						z.is_proper_real and not z.a.is_finite or
						z.is_proper_complex and not (z.a.is_finite and z.b.is_finite) implies
						(z ⋅ z0).a.is_nan
					broken_right_absorption_b: z.is_proper_complex and not (z.a.is_finite and z.b.is_finite) implies (z ⋅ z0).b.is_nan
				then
					l_broken_right_absorption := True
					retry
				end
			elseif not l_broken_dividend_absorption and z0.divisible (z) and (z0 / z) ≭ z0 then
					check
						zero_divisible_zb: z.is_proper_imaginary implies zero.a.divisible (z.b) -- z.b /= 0 <== z.is_proper_imaginary
						zero_divisible_za: z.is_proper_real implies zero.a.divisible (z.a) -- z.a /= 0 <== z.is_proper_real
					end
				check
					absorbing_dividend_unfolded_definition_a: (z0 / z).a ≍ if z.is_proper_imaginary then
							zero.a / z.b
						elseif z.is_proper_real then
							zero.a / z.a
						else
							proper_complex_quotient_implementation (zero, z).a
						end
					absorbing_dividend_unfolded_definition_imaginary: (z0 / z).b ≍ if z.is_proper_imaginary then
							zero.a
						elseif z0.is_proper_real then
							zero.a
						else
							proper_complex_quotient_implementation (zero, z).b
						end
					broken_dividend_absorption: z ≍ Zero or
						z.is_proper_imaginary and z.b.is_nan or
						z.is_proper_real and z.a.is_nan or
						z.is_proper_complex and (z.a.is_nan or z.b.is_nan)
					broken_dividend_absorption_a:
						z ≍ Zero or z.is_proper_imaginary and z.b.is_nan or z.is_proper_real and z.a.is_nan or z.is_proper_complex and (z.a.is_nan or z.b.is_nan) implies
						(z0 / z).a.is_nan
					broken_dividend_absorption_b: z ≍ Zero or z.b.is_proper_complex and (z0.a.is_nan or z0.b.is_nan) implies (z0 / z).b.is_nan
				then
					l_broken_dividend_absorption := True
					retry -- TODO: goto?
				end
			end
		end

feature -- Properties (Comparison)

	equals_ok (z1, z2, z3: STS_COMPLEX_NUMBER): BOOLEAN
			-- <Precursor>
		do
			if Precursor {STP_COMPLEX_NUMBER_PROPERTIES}(z1, z2, z3) then
				if attached {STI_COMPLEX_NUMBER} z1 as exp_z1 then
					if attached {STI_COMPLEX_NUMBER} z2 as exp_z2 then
						Result := expanded_equals_ok (exp_z1, exp_z2)
					else
						Result := expanded_equals_ok (exp_z1, z2)
					end
				else
					if attached {STI_COMPLEX_NUMBER} z2 as exp_z2 then
						Result := expanded_equals_ok (z1, exp_z2)
					else
						Result := expanded_equals_ok (z1, z2)
					end
				end
			end
		end

	expanded_equals_ok (z1, z2: STI_COMPLEX_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_NUMBER}.equals?
		do
			check
				definition: z1 ≍ z2 = z1.pair_equals (z2)
			end
			Result := True
		end

	unequals_ok (z1, z2: STS_COMPLEX_NUMBER): BOOLEAN
			-- <Precursor>
		do
			if Precursor {STP_COMPLEX_NUMBER_PROPERTIES}(z1, z2) then
				if attached {STI_COMPLEX_NUMBER} z1 as exp_z1 then
					if attached {STI_COMPLEX_NUMBER} z2 as exp_z2 then
						Result := expanded_unequals_ok (exp_z1, exp_z2)
					else
						Result := expanded_unequals_ok (exp_z1, z2)
					end
				else
					if attached {STI_COMPLEX_NUMBER} z2 as exp_z2 then
						Result := expanded_unequals_ok (z1, exp_z2)
					else
						Result := expanded_unequals_ok (z1, z2)
					end
				end
			end
		end

	expanded_unequals_ok (z1, z2: STI_COMPLEX_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_NUMBER}.unequals?
		do
			check
				definition: z1 ≭ z2 = z1.pair_unequals (z2)
			end
			Result := True
		end

feature -- Properties (Factory)

	converted_real_ok (x: STS_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_NUMBER}.converted_real?
		do
			check
				same_adjusted_value: attached {like complex_anchor.real_anchor} x implies {like complex_anchor}.converted_real (x).value = x.value
			then
				Result := True
			end
		end

feature -- Anchor

	complex_anchor: STI_COMPLEX_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	real_anchor: STI_REAL_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	integer_anchor: STI_INTEGER_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	complex_universe_anchor: STI_COMPLEX_SET
			-- <Precursor>
		once
			Result := {STI_COMPLEX_SET}.O
		ensure then
			class
		end

	real_universe_anchor: STI_REAL_SET
			-- <Precursor>
		once
			Result := {STI_REAL_SET}.O
		ensure then
			class
		end

feature {NONE} -- Implementation

	proper_complex_product_implementation (z1, z2: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
		do
			Result := z1 ⋅ z2
		end

	proper_complex_quotient_implementation (z1, z2: STS_COMPLEX_NUMBER): like complex_anchor
			-- <Precursor>
		do
			Result := z1 / z2
		end

	max_subsets: NATURAL = 1_048_576 -- 2 ^ 20
			-- <Precursor>

	max_asserted_subsets: NATURAL = 128 -- 2 ^ 7
			-- <Precursor>

	max_asserted_pairs: NATURAL = 289 -- 17 ^ 2
			-- <Precursor>

	max_asserted_elements: NATURAL = 1_682 -- Magic number.
			-- <Precursor>

invariant
	zero_real: Zero.real.value = 0
	zero_imaginary: Zero.imaginary.value = 0
	one_real: One.real.value = 1
	one_imaginary: One.imaginary.value = 0
	Two: Two.value = 2
	i_real: i.real ≍ Zero.real
	i_imaginary: i.imaginary ≍ One.real
	Imaginary_unit_real: Imaginary_unit.real ≍ Zero.real
	Imaginary_unit_imaginary: Imaginary_unit.imaginary ≍ One.real

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
