﻿note
	description: "Implementation of {STP_INTEGER_NUMBER_PROPERTIES}."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	INTEGER_NUMBER_PROPERTIES

inherit
	STP_INTEGER_NUMBER_PROPERTIES
		rename
			machine_epsilon_ok as srnp_machine_epsilon_ok,
			argument_ok as srnp_argument_ok,
			phase_ok as srnp_phase_ok
		undefine
			machine_epsilon,
			complex_equals_ok,
			complex_unequals_ok,
			rational_equals_ok,
			rational_unequals_ok
		redefine
			default_create
		select
			Zero,
			One
		end

	RATIONAL_NUMBER_PROPERTIES
		rename
			Zero as rational_zero,
			One as rational_one,
			default_create_ok as rational_default_create_ok,
			is_in_ok as rational_is_in_ok,
			is_not_in_ok as rational_is_not_in_ok,
			zero_ok as rational_zero_ok,
			one_ok as rational_one_ok,
			is_integer_ok as rational_is_integer_ok,
			equals_ok as rational_equals_ok,
			unequals_ok as rational_unequals_ok,
			is_less_ok as rational_is_less_ok,
			is_less_equal_ok as rational_is_less_equal_ok,
			is_greater_ok as rational_is_greater_ok,
			is_greater_equal_ok as rational_is_greater_equal_ok,
			min_ok as rational_min_ok,
			max_ok as rational_max_ok,
			multipliable_ok as rnp_rational_multipliable_ok,
			divisible_ok as rational_divisible_ok,
			modulus_ok as rational_modulus_ok,
			abs_ok as rational_abs_ok,
			plus_ok as rational_plus_ok,
			identity_ok as rational_identity_ok,
			minus_ok as rational_minus_ok,
			opposite_ok as rational_opposite_ok,
			product_ok as rational_product_ok,
			quotient_ok as rational_quotient_ok,
			power_ok as real_power_ok,
			as_complex_ok as rational_as_complex_ok,
			as_real_ok as rational_as_real_ok
		redefine
			default_create
		end

feature {NONE} -- Initialization

	default_create
			-- <Precursor>
		do
			Precursor {RATIONAL_NUMBER_PROPERTIES}
			create integer_set_properties
		end

feature -- Access

	zero: STI_INTEGER_NUMBER
			-- <Precursor>
			--| TODO: Feature tool cannot show post-conditions.
			--| TODO: What about invariants?
		once
		ensure then
			class
		end

	one: STI_INTEGER_NUMBER
			-- <Precursor>
			--| TODO: Feature tool cannot show post-conditions.
			--| TODO: What about invariants?
		once
			Result := {STI_INTEGER_NUMBER}.One
		ensure then
			class
		end

	current_integer_universe: like integer_universe_anchor
			-- <Precursor>
		do
			Result := integer_set_properties.z
		end

	integer_set_properties: INTEGER_SET_PROPERTIES
			-- Object that checks whether the properties verified within set
			-- theory hold for an implementation of {STS_INTEGER_SET}

feature -- Properties (Initialization)

	default_create_ok (i: STS_INTEGER_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_INTEGER_NUMBER}.default_create?
		local
			broken_dividend_absorption: BOOLEAN
			i0: like integer_anchor -- NOTICE: Default created.
		do
			if not broken_dividend_absorption then
				check
					zero: i0 ≍ zero
					real: i0.is_real
					imaginary: i0.is_imaginary
					not_proper_real: not i0.is_proper_real
					not_proper_imaginary: not i0.is_proper_imaginary
					not_proper_complex: not i0.is_proper_complex
					equal_to_opposite: i0 ≍ - i0
					neutral_left_term: (i0 + i) ≍ i
					neutral_right_term: (i + i0) ≍ i
					quasi_neutral_minuend: (i0 - i) ≍ - i
					neutral_subtrahend: (i - i0) ≍ i
					absorbing_left_factor: (i0 ⋅ i) ≍ i0
					absorbing_right_factor: (i ⋅ i0) ≍ i0
					absorbing_dividend: i0.real_divisible (i) implies i0.real_quotient (i) ≍ i0
				end
			end
			check
				negative_exponent: i < zero implies (i0 ^ i).is_positive_infinity
				to_power_zero: (i0 ^ zero) ≍ one
				absorbing_base: zero < i implies (i0 ^ i) ≍ zero
			end
			Result := True
		rescue
			print (i) print ('%N')
			if
				not broken_dividend_absorption and (
					i0.real_divisible (i) and then i0.real_quotient (i) ≭ i0
					)
			then
				check
					broken_dividend_absorption_cause: i ≍ Zero
					broken_dividend_absorption: i0.real_quotient (i).is_nan
				end
				broken_dividend_absorption := True
				retry -- TODO: goto?
			end
		end

feature -- Properties (Conversion)

	as_complex_ok (i: STI_INTEGER_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_INTEGER_NUMBER}.as_complex?
		do
			check
				integer: i.as_complex.is_integer
				real: i.as_complex.Is_real
				perhaps_proper_real: i.as_complex.is_proper_real = i ≭ Zero
				perhaps_imaginary: i.as_complex.is_imaginary = i ≍ Zero
				not_proper_imaginary: not i.as_complex.Is_proper_imaginary
				not_proper_complex: not i.as_complex.Is_proper_complex
				finite: i.as_complex.is_finite
				never_infinite: not i.as_complex.is_infinite
			end
			Result := True
		end

	as_real_ok (i: STI_INTEGER_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_INTEGER_NUMBER}.as_real?
		do
			check
--				rational: i.as_real.is_rational -- TODO: Do it!
				integer: i.as_real.is_integer
				finite: i.as_real.is_finite
				never_infinite: not i.as_real.is_infinite
				never_negative_infinity: not i.as_real.is_negative_infinity
				never_negative_zero: not i.as_real.is_negative_zero
				never_positive_infinity: not i.as_real.is_positive_infinity
				never_nan: not i.as_real.is_nan
			end
			Result := True
		end

	as_rational_ok (i: STI_INTEGER_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_INTEGER_NUMBER}.as_rational?
		do
			check
				integer: i.as_rational.is_integer
			end
			Result := True
		end

feature -- Anchor

	integer_universe_anchor:
		STI_SET [STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]]
			-- <Precursor>
		once
			create Result.make_empty
		ensure then
			class
		end

invariant
	Rational_one_numerator: Rational_one.p.value = 1
	Rational_one_denominator: Rational_one.q.value = 1

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
