﻿note
	description: "Implementation of {STP_RATIONAL_NUMBER_PROPERTIES}."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	RATIONAL_NUMBER_PROPERTIES

inherit
	STP_RATIONAL_NUMBER_PROPERTIES
		rename
			machine_epsilon_ok as srnp_machine_epsilon_ok,
			argument_ok as srnp_argument_ok,
			phase_ok as srnp_phase_ok
		undefine
			default_create,
			machine_epsilon,
			complex_equals_ok,
			complex_unequals_ok
		redefine
			equals_ok,
			unequals_ok
		select
			Zero,
			One
		end

	REAL_NUMBER_PROPERTIES
		rename
			Zero as real_zero,
			One as real_one,
			i as imaginary_unit,
			default_create_ok as real_default_create_ok,
			is_in_ok as real_is_in_ok,
			is_not_in_ok as real_is_not_in_ok,
			zero_ok as real_zero_ok,
			one_ok as real_one_ok,
			min_value_ok as real_min_value_ok,
			max_value_ok as real_max_value_ok,
			is_nan_ok as real_is_nan_ok,
			is_negative_infinity_ok as real_is_negative_infinity_ok,
			is_positive_infinity_ok as real_is_positive_infinity_ok,
			is_infinite_ok as real_is_infinite_ok,
			is_rational_ok as real_is_rational_ok,
			is_integer_ok as real_is_integer_ok,
			equals_ok as real_equals_ok,
			expanded_equals_ok as complex_expanded_equals_ok,
			expanded_unequals_ok as complex_expanded_unequals_ok,
			unequals_ok as real_unequals_ok,
			is_less_ok as real_is_less_ok,
			is_less_equal_ok as real_is_less_equal_ok,
			is_greater_ok as real_is_greater_ok,
			is_greater_equal_ok as real_is_greater_equal_ok,
			min_ok as real_min_ok,
			max_ok as real_max_ok,
			plus_ok as real_plus_ok,
			identity_ok as real_identity_ok,
			minus_ok as real_minus_ok,
			opposite_ok as real_opposite_ok,
			product_ok as real_product_ok,
			quotient_ok as real_quotient_ok,
			reciprocal_ok as real_reciprocal_ok,
			inverse_ok as real_inverse_ok
		redefine
			Imaginary_unit
		end

feature -- Access

	Zero: STI_RATIONAL_NUMBER
			-- <Precursor>
			--| TODO: Feature tool cannot show the inherited post-conditions.
			--| TODO: What about the invariants?
		once
		ensure then
			class
		end

	One: STI_RATIONAL_NUMBER
			-- <Precursor>
			--| TODO: Feature tool cannot show the inherited post-conditions.
			--| TODO: What about the invariants?
		once
			create Result.make_from_expanded ({INTEGER_NUMBER}.One, {INTEGER_NUMBER}.One)
		ensure then
			class
		end

	imaginary_unit: STI_COMPLEX_NUMBER
			-- <Precursor>
		once
			Result := {STI_COMPLEX_NUMBER}.I
		ensure then
			class
		end

feature -- Properties (Initialization)

	default_create_ok (pq: STS_RATIONAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_RATIONAL_NUMBER}.default_create?
		local
			pq_0: like rational_anchor -- NOTICE: Default created.
		do
			check
				zero: pq_0 ≍ Zero
				real: pq_0.is_real
				imaginary: pq_0.is_imaginary
				not_proper_real: not pq_0.is_proper_real
				not_proper_imaginary: not pq_0.is_proper_imaginary
				not_proper_complex: not pq_0.is_proper_complex
				equal_to_opposite: pq_0 ≍ - pq_0
				neutral_left_term: (pq_0 + pq) ≍ pq
				neutral_right_term: (pq + pq_0) ≍ pq
				quasi_neutral_minuend: (pq_0 - pq) ≍ - pq
				neutral_subtrahend: (pq - pq_0) ≍ pq

					pq_0_multipliable_pq: pq_0.multipliable (pq) -- pq_0.q = 1
				absorbing_left_factor: (pq_0 ⋅ pq) ≍ pq_0

					pq_multipliable_pq_0: pq.multipliable (pq_0) -- pq_0.q = 1
				absorbing_right_factor: (pq ⋅ pq_0) ≍ pq_0

				absorbing_dividend: pq_0.divisible (pq) implies (pq_0 / pq) ≍ pq_0
				negative_exponent: -- TODO: Take a real exponent instead.
					pq < Zero implies (pq_0 ^ pq).is_positive_infinity
				to_power_zero: (pq_0 ^ Zero) ≍ one
				absorbing_base: pq > Zero implies  (pq_0 ^ pq) ≍ Zero
			end
			Result := True
		end

feature -- Properties (Access)

	min_value_ok (pq: STI_RATIONAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_RATIONAL_NUMBER}.min_value?
		do
			check
				definition: pq.Min_value ≍ - {STI_RATIONAL_NUMBER}.Max_value
			end
			Result := True
		end

	max_value_ok (pq: STI_RATIONAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_RATIONAL_NUMBER}.max_value?
		do
			check
				definition: pq.Max_value ≍ - {STI_RATIONAL_NUMBER}.Min_value
			end
			Result := True
		end

feature -- Properties (Comparison)

	equals_ok (pq_1, pq_2, pq_3: STS_RATIONAL_NUMBER): BOOLEAN
			-- <Precursor>
		do
			if Precursor {STP_RATIONAL_NUMBER_PROPERTIES}(pq_1, pq_2, pq_3) then
				if attached {STI_RATIONAL_NUMBER} pq_1 as exp_pq_1 then
					if attached {STI_RATIONAL_NUMBER} pq_2 as exp_pq_2 then
						Result := expanded_equals_ok (exp_pq_1, exp_pq_2)
					else
						Result := expanded_equals_ok (exp_pq_1, pq_2)
					end
				else
					if attached {STI_RATIONAL_NUMBER} pq_2 as exp_pq_2 then
						Result := expanded_equals_ok (pq_1, exp_pq_2)
					else
						Result := expanded_equals_ok (pq_1, pq_2)
					end
				end
			end
		end

	expanded_equals_ok (pq_1, pq_2: STI_RATIONAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_RATIONAL_NUMBER}.equals?
		do
			check
				consistent: pq_1.pair_equals (pq_2) implies pq_1 ≍ pq_2
			end
			Result := True
		end

	unequals_ok (pq_1, pq_2: STS_RATIONAL_NUMBER): BOOLEAN
			-- <Precursor>
		do
			if Precursor {STP_RATIONAL_NUMBER_PROPERTIES}(pq_1, pq_2) then
				if attached {STI_RATIONAL_NUMBER} pq_1 as exp_pq_1 then
					if attached {STI_RATIONAL_NUMBER} pq_2 as exp_pq_2 then
						Result := expanded_unequals_ok (exp_pq_1, exp_pq_2)
					else
						Result := expanded_unequals_ok (exp_pq_1, pq_2)
					end
				else
					if attached {STI_RATIONAL_NUMBER} pq_2 as exp_pq_2 then
						Result := expanded_unequals_ok (pq_1, exp_pq_2)
					else
						Result := expanded_unequals_ok (pq_1, pq_2)
					end
				end
			end
		end

	expanded_unequals_ok (pq_1, pq_2: STI_RATIONAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_RATIONAL_NUMBER}.unequals?
		do
			check
				consistent: pq_1 ≭ pq_2 implies pq_1.pair_unequals (pq_2)
			end
			Result := True
		end

feature -- Properties (Relationship)

	expanded_multipliable_ok
		(pq_1: STI_RATIONAL_NUMBER; pq_2: STS_RATIONAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_RATIONAL_NUMBER}.multipliable?
		do
			if multipliable_ok (pq_1, pq_2) then
				check
					gcd_1: attached gcd (pq_2.p, pq_1.q) as gcd_1
					gcd_2: attached gcd (pq_1.p, pq_2.q) as gcd_2
					good_divisor_1: -- gcd_1 /= 0 ⇐ pq_1.q /= 0
						pq_1.q.divisible (gcd_1)
					good_divisor_2: -- gcd_2 /= 0 ⇐ pq_2.q /= 0
						pq_2.q.divisible (gcd_2)
					accepted_overflow:
						{STI_RATIONAL_NUMBER}.integer_product_overflows
							(pq_1.q // gcd_1, pq_2.q // gcd_2) and
						(pq_1.q // gcd_1) ⋅ (pq_2.q // gcd_2) ≭ Zero implies
						pq_1.multipliable (pq_2)
				end
				Result := True
			end
		end

	divisible_ok (pq_1: STI_RATIONAL_NUMBER; pq_2: STS_RATIONAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_RATIONAL_NUMBER}.divisible?
		do
			check
				good_divisor_1: -- pq_2.q, pq_1.q /= 0
					pq_1.q.divisible (gcd (pq_2.q, pq_1.q))
				good_divisor_2: pq_2.p ≭ Zero implies
					pq_2.p.divisible (gcd (pq_1.p, pq_2.p))
				definition: pq_1.divisible (pq_2) implies
					pq_2.p ≭ Zero and then (
						(pq_1.q // gcd (pq_2.q, pq_1.q)) ⋅
						(pq_2.p // gcd (pq_1.p, pq_2.p))
						) ≭ Zero
			end
			Result := True
		rescue
			print (pq_1) print (", ")
			print (pq_2) print ('%N')
		end

feature -- Properties (Conversion)

	as_real_ok (q: STI_RATIONAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_RATIONAL_NUMBER}.as_real?
		do
			check
--				rational: q.as_real.is_rational -- TODO: Do it!
				finite: q.as_real.is_finite
				never_infinite: not q.as_real.is_infinite
				never_negative_infinity: not q.as_real.is_negative_infinity
				never_negative_zero: not q.as_real.is_negative_zero
				never_positive_infinity: not q.as_real.is_positive_infinity
				never_nan: not q.as_real.is_nan
			end
			Result := True
		end

feature -- Anchor

	rational_anchor: STI_RATIONAL_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

invariant
	Real_zero: Real_zero.value = 0

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
