﻿note
	description: "Implementation of {STP_REAL_PROPERTIES}."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	REAL_NUMBER_PROPERTIES

inherit
	STP_REAL_NUMBER_PROPERTIES
		rename
			machine_epsilon_ok as srnp_machine_epsilon_ok,
			argument_ok as srnp_argument_ok,
			phase_ok as srnp_phase_ok
		undefine
			default_create,
			machine_epsilon,
			complex_equals_ok,
			complex_unequals_ok
		select
			one
		end

	COMPLEX_NUMBER_PROPERTIES
		rename
			default_create_ok as complex_default_create_ok,
			zero as complex_zero,
			one as complex_one,
			is_in_ok as complex_is_in_ok,
			is_not_in_ok as complex_is_not_in_ok,
			zero_ok as complex_zero_ok,
			one_ok as complex_one_ok,
			is_infinite_ok as complex_is_infinite_ok,
			is_integer_ok as complex_is_integer_ok,
			is_rational_ok as complex_is_rational_ok,
			equals_ok as complex_equals_ok,
			unequals_ok as complex_unequals_ok,
			modulus_ok as complex_modulus_ok,
			abs_ok as complex_abs_ok,
			argument_ok as complex_argument_ok,
			phase_ok as complex_phase_ok,
			plus_ok as complex_plus_ok,
			identity_ok as complex_identity_ok,
			minus_ok as complex_minus_ok,
			opposite_ok as complex_opposite_ok,
			product_ok as complex_product_ok,
			quotient_ok as complex_quotient_ok,
			reciprocal_ok as complex_reciprocal_ok,
			inverse_ok as complex_inverse_ok
		redefine
			default_create
		end

feature {NONE} -- Initialization

	default_create
			-- <Precursor>
		do
			Precursor {COMPLEX_NUMBER_PROPERTIES}
			nan := {STI_REAL_NUMBER}.Nan
			negative_infinity := {STI_REAL_NUMBER}.Negative_infinity
			positive_infinity := {STI_REAL_NUMBER}.Positive_infinity
		end

feature -- Access

	Zero: STI_REAL_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	One: STI_REAL_NUMBER
			-- <Precursor>
			--| TODO: Feature tool cannot show the inherited post-conditions.
			--| TODO: What about the invariants?
		once
			Result := {STI_REAL_NUMBER}.One
		ensure then
			class
		end

	nan: STI_REAL_NUMBER
			-- Representation of not a number (NaN)
--		once
--			Result := {STI_REAL_NUMBER}.Nan
--		ensure
--			class
--			definition: Result.is_nan
--		end

	negative_infinity: STI_REAL_NUMBER
			-- Representation of negative infinity
--		once
--			Result := {STI_REAL_NUMBER}.Negative_infinity
--		ensure
--			class
--			definition: Result.is_negative_infinity
--		end

	positive_infinity: STI_REAL_NUMBER
			-- Representation of positive infinity
--		once
--			Result := {STI_REAL_NUMBER}.Positive_infinity
--		ensure
--			class
--			definition: Result.is_positive_infinity
--		end

feature -- Properties (Initialization)

	default_create_ok (x: STS_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.default_create?
		local
			l_broken_left_absorption, l_broken_right_absorption, l_broken_dividend_absorption, l_broken_negative_exponent: BOOLEAN
			x0: like real_anchor -- NOTICE: Default created.
		do
			if not (l_broken_left_absorption or l_broken_right_absorption or l_broken_dividend_absorption) then
				check
					zero: x0 ≍ zero
					real: x0.is_real
					imaginary: x0.is_imaginary
					not_proper_real: not x0.is_proper_real
					not_proper_imaginary: not x0.is_proper_imaginary
					not_proper_complex: not x0.is_proper_complex
					equal_to_opposite: x0 ≍ - x0
					neutral_left_term: (x0 + x) ≍ x
					neutral_right_term: (x + x0) ≍ x
					quasi_neutral_minuend: (x0 - x) ≍ - x
					neutral_subtrahend: (x - x0) ≍ x
					absorbing_left_factor: (x0 ⋅ x) ≍ x0
				end
			end
			if not (l_broken_right_absorption or l_broken_dividend_absorption) then
				check
					absorbing_right_factor: (x ⋅ x0) ≍ x0
				end
			end
			if not l_broken_dividend_absorption then
				check
					absorbing_dividend: x0.divisible (x) implies (x0 / x) ≍ x0
				end
			end
			if not l_broken_negative_exponent then
				check
					negative_exponent: x < zero implies (x0 ^ x).is_positive_infinity
				end
			end
			check
				to_power_zero: (x0 ^ zero) ≍ one
				absorbing_base: x > zero implies (x0 ^ x) ≍ zero
			end
			Result := True
		rescue
			print (x) print ('%N')
			if not l_broken_left_absorption and (x0 ⋅ x) ≭ x0 then
				check
					broken_left_absorption_cause: not x.is_finite
					broken_left_absorption: (x0 ⋅ x).is_nan
				end
				l_broken_left_absorption := True
				retry
			elseif not l_broken_right_absorption and (x ⋅ x0) ≭ x0 then
				check
					broken_right_absorption_cause: not x.is_finite
					broken_right_absorption: (x ⋅ x0).is_nan
				end
				l_broken_right_absorption := True
				retry
			elseif not l_broken_dividend_absorption and x0.divisible (x) and (x0 / x) ≭ x0 then
				check
					broken_dividend_absorption_cause: x ≍ Zero or x.is_nan
					broken_dividend_absorption: (x0 / x).is_nan
				end
				l_broken_dividend_absorption := True
				retry -- TODO: goto?
			elseif not l_broken_negative_exponent and x < zero and not (x0 ^ x).is_positive_infinity then
				check
					broken_negative_exponent_cause: x.is_nan
					broken_negative_exponent_nan_power: (x0 ^ x).is_nan
				end
				l_broken_negative_exponent := True
				retry
			end
		end

	make_abs_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.make_abs?
		local
			l_broken_non_negativity: BOOLEAN
			x_abs: STI_REAL_NUMBER
		do
			if not l_broken_non_negativity then
				create x_abs.make_abs (x)
				check
					when_negative: x < zero implies x_abs ≍ - x
					when_non_negative: x ≥ zero implies x_abs ≍ x
					non_negative: x_abs ≥ zero
				end
			end
			Result := True
		rescue
			print (x) print ('%N')
			if not l_broken_non_negativity then
				check
					nan: x.is_nan
					broken_abs: x_abs.is_nan
				end
				l_broken_non_negativity := True
				retry
			end
		end

	make_opposite_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.make_opposite?
		local
			l_broken_left_opposition, l_broken_right_opposition: BOOLEAN
			x_opposite: like real_anchor
		do
			create x_opposite.make_opposite (x)
			if not l_broken_left_opposition and not l_broken_right_opposition then
				check
					opposite_left_term: (x_opposite + x) ≍ zero
				end
			end
			if not l_broken_right_opposition then
				check
					opposite_right_term: (x + x_opposite) ≍ zero
				end
			end
			check
				involutive: - x_opposite ≍ x
				as_subtraction: x_opposite ≍ (zero - x)
				as_product: x_opposite ≍ (- one ⋅ x)
			end
			check
					divisible: x.divisible (- one) -- - one /= 0
				as_quotient: x_opposite ≍ (x / ( - one))
			end
			Result := True
		rescue
			print (x) print ('%N')
			if not l_broken_left_opposition and (x_opposite + x) ≭ zero then
				check
					broken_left_opposition_cause: not x_opposite.is_finite
					broken_left_opposition: (x_opposite + x).is_nan
				end
				l_broken_left_opposition := True
				retry
			elseif not l_broken_right_opposition and (x + x_opposite) ≭ zero then
				check
					broken_right_opposition_cause: not x_opposite.is_finite
					broken_right_opposition: (x + x_opposite).is_nan
				end
				l_broken_right_opposition := True
				retry
			end
		end

feature -- Properties (Access)

	nan_ok (x, y: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.nan?
		do
			check
				real: x.Nan.Is_real
				proper_real: x.Nan.is_proper_real
				not_imaginary: not x.Nan.is_imaginary
				not_proper_imaginary: not x.Nan.Is_proper_imaginary
				not_proper_complex: not x.Nan.Is_proper_complex
				below_everything: x.Nan ≤ y
				propagating_left_term: (x.Nan + y) ≍ x.Nan
				propagating_right_term: (x + y.Nan) ≍ y.Nan
				propagating_minuend: (x.Nan - y) ≍ x.Nan
				propagating_subtrahend: (x - y.Nan) ≍ y.Nan
				propagating_left_factor: (x.Nan ⋅ y) ≍ x.Nan
				propagating_right_factor: (x - y.Nan) ≍ y.Nan
				propagating_dividend: x.Nan.divisible (y) implies (x.Nan / y) ≍ x.Nan
					divisible: x.divisible (y.Nan)
				propagating_divisor: (x / y.Nan) ≍ y.Nan
				propagating_base: y ≭ Zero implies (x.Nan ^ y) ≍ x.Nan
				propagating_exponent: (x ^ y.Nan) ≍ y.Nan
			end
			Result := True
		end

	negative_infinity_ok (x, y: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.Negative_infinity?
		do
			check
				real: x.Negative_infinity.Is_real
				proper_real: x.Negative_infinity.is_proper_real
				not_imaginary: not x.Negative_infinity.is_imaginary
				not_proper_imaginary: not x.Negative_infinity.Is_proper_imaginary
				not_proper_complex: not x.Negative_infinity.Is_proper_complex
				above_nan: Nan < y.Negative_infinity
				below_everything_else: not y.is_nan implies x.Negative_infinity ≤ y
				absorbing_left_term: y.is_negative_infinity or y.is_finite implies (x.Negative_infinity + y) ≍ x.Negative_infinity
				plus_positive_infinity: (x.Negative_infinity + Positive_infinity) ≍ Nan
				absorbing_right_term: x.is_negative_infinity or x.is_finite implies (x + y.Negative_infinity) ≍ y.Negative_infinity
				positive_infinity_plus: (Positive_infinity + y.Negative_infinity) ≍ Nan

				minus_negative_infinity: (x.Negative_infinity - Negative_infinity) ≍ Nan
				absorbing_minuend: y.is_finite or y.is_positive_infinity implies (x.Negative_infinity - y) ≍ x.Negative_infinity
				negative_infinity_minus: (Negative_infinity - y.Negative_infinity) ≍ Nan
				quasi_absorbing_subtrahend: x.is_finite or x.is_positive_infinity implies (x - y.Negative_infinity) ≍ y.Positive_infinity

				quasi_absorbing_left_factor: not y.is_nan and y ≭ Zero implies (x.Negative_infinity ⋅ y) ≍ (x.Negative_infinity ⋅ y.sign)
				times_zero: (x.Negative_infinity ⋅ Zero) ≍ Nan
				quasi_absorbing_right_factor: not x.is_nan and x ≭ Zero implies (x ⋅ y.Negative_infinity) ≍ (x.sign ⋅ y.Negative_infinity)
				zero_times: (Zero ⋅ y.Negative_infinity) ≍ Nan
					Negative_infinity_divisible_y: y.is_finite and y ≭ Zero implies x.Negative_infinity.divisible (y)
					x_divisible_y_Negative_infinity: x.is_finite implies x.divisible (y.Negative_infinity)
					Negative_infinity_divisible_Negative_infinity: x.Negative_infinity.divisible (Negative_infinity)
					Negative_infinity_divisible_Positive_infinity: x.Negative_infinity.divisible (Positive_infinity)
					Negative_infinity_divisible_y_Negative_infinity: Negative_infinity.divisible (y.Negative_infinity)
					Positive_infinity_divisible_y_Negative_infinity: Positive_infinity.divisible (y.Negative_infinity)
				quasi_absorbing_dividend: y.is_finite and y ≭ Zero implies (x.Negative_infinity / y) ≍ (x.Negative_infinity ⋅ y.sign)
				zero_quotient: x.is_finite implies (x / y.Negative_infinity) ≍ - Zero
				infinite_negative_divisor: (x.Negative_infinity / Negative_infinity) ≍ Nan
				infinite_positive_divisor: (x.Negative_infinity / Positive_infinity) ≍ Nan
				infinite_negative_dividend: (Negative_infinity / y.Negative_infinity) ≍ Nan
				infinite_positive_dividend: (Positive_infinity / y.Negative_infinity) ≍ Nan
				to_zero_exponent: (x.Negative_infinity ^ Zero) ≍ One
				bad_exponent: (one ^ y.Negative_infinity) ≍ Nan -- TODO: Why?
			end
			Result := True
		end

	positive_infinity_ok (x, y: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.Positive_infinity?
		do
			check
				real: x.Positive_infinity.Is_real
				proper_real: x.Positive_infinity.is_proper_real
				not_imaginary: not x.Positive_infinity.is_imaginary
				not_proper_imaginary: not x.Positive_infinity.Is_proper_imaginary
				not_proper_complex: not x.Positive_infinity.Is_proper_complex
				above_everything: x ≤ y.Positive_infinity
				plus_negative_infinity: (x.Positive_infinity + Negative_infinity) ≍ Nan
				absorbing_left_term: y.is_finite or y.is_positive_infinity implies (x.Positive_infinity + y) ≍ x.Positive_infinity
				negative_infinity_plus: (Negative_infinity + y.Positive_infinity) ≍ Nan
				absorbing_right_term: x.is_finite or x.is_positive_infinity implies (x + y.Positive_infinity) ≍ y.Positive_infinity

				absorbing_minuend: y.is_negative_infinity or y.is_finite implies (x.Positive_infinity - y) ≍ x.Positive_infinity
				minus_positive_infinity: (x.Positive_infinity - Positive_infinity) ≍ Nan
				quasi_absorbing_subtrahend: x.is_negative_infinity or x.is_finite implies (x - y.Positive_infinity) ≍ y.Negative_infinity
				positive_infinity_minus: (Positive_infinity - y.Positive_infinity) ≍ Nan

				quasi_absorbing_left_factor: not y.is_nan and y ≭ Zero implies (x.Positive_infinity ⋅ y) ≍ (x.Positive_infinity ⋅ y.sign)
				times_zero: (x.Positive_infinity ⋅ Zero) ≍ Nan
				quasi_absorbing_right_factor: not x.is_nan and x ≭ Zero implies (x ⋅ y.Positive_infinity) ≍ (x.sign ⋅ y.Positive_infinity)
				zero_times: (Zero ⋅ y.Positive_infinity) ≍ Nan
					Positive_infinity_divisible_y: y.is_finite and y ≭ Zero implies x.Positive_infinity.divisible (y)
					x_divisible_y_Positive_infinity: x.is_finite implies x.divisible (y.Positive_infinity)
					Positive_infinity_divisible_Negative_infinity: x.Positive_infinity.divisible (Negative_infinity)
					Positive_infinity_divisible_Positive_infinity: x.Positive_infinity.divisible (Positive_infinity)
					Negative_infinity_divisible_y_Positive_infinity: Negative_infinity.divisible (y.Positive_infinity)
					Positive_infinity_divisible_y_Positive_infinity: Positive_infinity.divisible (y.Positive_infinity)
				quasi_absorbing_dividend: y.is_finite and y ≭ Zero implies (x.Positive_infinity / y) ≍ (x.Positive_infinity ⋅ y.sign)
				zero_quotient: x.is_finite implies (x / y.Positive_infinity) ≍ - Zero
				infinite_negative_divisor: (x.Positive_infinity / Negative_infinity) ≍ Nan
				infinite_positive_divisor: (x.Positive_infinity / Positive_infinity) ≍ Nan
				infinite_negative_dividend: (Negative_infinity / y.Positive_infinity) ≍ Nan
				infinite_positive_dividend: (Positive_infinity / y.Positive_infinity) ≍ Nan
				to_zero_exponent: (x.Negative_infinity ^ Zero) ≍ One
				bad_exponent: (one ^ y.Positive_infinity) ≍ Nan -- TODO: Why?
			end
			Result := True
		end

	min_value_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.Min_value?
		do
			check
				real: x.Min_value.Is_real
				proper_real: x.Min_value.is_proper_real
				not_imaginary: not x.Min_value.is_imaginary
				not_proper_imaginary: not x.Min_value.Is_proper_imaginary
				not_proper_complex: not x.Min_value.Is_proper_complex
				above_nan: Nan < x.Min_value
				above_negative_infinity: Negative_infinity < x.Min_value
			end
			Result := True
		end

	max_value_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.Max_value?
		do
			check
				real: x.Max_value.Is_real
				proper_real: x.Max_value.is_proper_real
				not_imaginary: not x.Max_value.is_imaginary
				not_proper_imaginary: not x.Max_value.Is_proper_imaginary
				not_proper_complex: not x.Max_value.Is_proper_complex
				below_positive_infinity: x.Max_value < Positive_infinity
			end
			Result := True
		end

	machine_epsilon_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_REAL_NUMBER}.Machine_epsilon?
		do
			if srnp_machine_epsilon_ok (x) then
				check
					definition: x.Machine_epsilon.value =
						(1 / 2 ^ {REAL_NUMBER}.Mantissa_width)
				end
				Result := True
			end
		end

	epsilon_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.Epsilon?
		do
			check
				real: x.Epsilon.Is_real
				proper_real: x.Epsilon.is_proper_real
				not_imaginary: not x.Epsilon.is_imaginary
				not_proper_imaginary: not x.Epsilon.Is_proper_imaginary
				not_proper_complex: not x.Epsilon.Is_proper_complex
				positive: Zero < x.Epsilon
			end
			Result := True
		end

feature -- Properties (Operation)

	argument_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.argument?
		do
			if srnp_argument_ok (x) then
				check
					when_negative_zero: x.is_negative_zero implies x.argument.value = x.adjusted_value (Pi)
					when_positive_zero: x.is_positive_zero implies x.argument ≍ Zero
				end
				Result := True
			end
		end

	phase_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.phase?
		do
			if srnp_phase_ok (x) then
				check
					when_negative_zero: x.is_negative_zero implies x.phase.value = x.adjusted_value (Pi)
					when_positive_zero: x.is_positive_zero implies x.phase ≍ Zero
				end
				Result := True
			end
		end

feature -- Properties (Conversion)

	as_complex_ok (x: STI_REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_REAL_NUMBER}.as_complex?
		do
			check
				real: x.as_complex.Is_real
				perhaps_proper_real: x.as_complex.is_proper_real = x ≭ Zero
				perhaps_imaginary: x.as_complex.is_imaginary = x ≍ Zero
				not_proper_imaginary: not x.as_complex.Is_proper_imaginary
				not_proper_complex: not x.as_complex.Is_proper_complex
			end
			Result := True
		end

feature -- Properties (Implementation)

	sign_bit_status_ok (x: REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.sign_bit_status?
		do
			check
				two_states: x.sign_bit_status = 0 or x.sign_bit_status = 1
				when_nan: x.is_nan implies x.sign_bit_status = 0 or x.sign_bit_status = 1
				when_negative: not x.is_nan and x < Zero implies x.sign_bit_status = 1
				when_negative_zero: x.is_negative_zero implies x.sign_bit_status = 1
				when_positive_zero: x.is_positive_zero implies x.sign_bit_status = 0
				when_positive: Zero < x implies x.sign_bit_status = 0
			end
			Result := True
		end

	exponent_bit_pattern_ok (x: REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.exponent_bit_pattern?
		local
			x_abs: REAL_NUMBER
		do
			x_abs := x.abs
			check
				when_nan: x.is_nan implies x.exponent_bit_pattern = (2 ^ {REAL_NUMBER}.Exponent_width - 1)
				when_negative: not x.is_nan and x < Zero implies x.exponent_bit_pattern = x_abs.exponent_bit_pattern
				when_zero: x ≍ Zero implies x.exponent_bit_pattern = 0
				when_subnormal: Zero < x.abs and x.abs < {STI_REAL_NUMBER}.epsilon implies x.exponent_bit_pattern = 0
				when_normal: {STI_REAL_NUMBER}.epsilon ≤ x.abs and x.is_finite implies
					x.exponent_bit_pattern = {STI_REAL_NUMBER}.value_logb (x.value) + {REAL_NUMBER}.Exponent_bias
				when_infinite: x.is_infinite implies x.exponent_bit_pattern = (2 ^ {REAL_NUMBER}.Exponent_width - 1)
			end
			Result := True
		end

	mantissa_bit_pattern_ok (x: REAL_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_REAL_NUMBER}.mantissa_bit_pattern?
		local
			x_abs: REAL_NUMBER
		do
			x_abs := x.abs
			check
				when_nan: x.is_nan implies x.mantissa_bit_pattern /= 0
				when_negative: not x.is_nan and x < Zero implies x.mantissa_bit_pattern = x_abs.mantissa_bit_pattern
				when_zero: x ≍ Zero implies x.mantissa_bit_pattern = 0
				when_subnormal: Zero < x.abs and x.abs < {STI_REAL_NUMBER}.epsilon implies
					0 < x.mantissa_bit_pattern and x.mantissa_bit_pattern <= {REAL_NUMBER}.Max_mantissa
				when_normal: {STI_REAL_NUMBER}.epsilon ≤ x.abs and x.is_finite implies
					0 <= x.mantissa_bit_pattern and x.mantissa_bit_pattern <= {REAL_NUMBER}.Max_mantissa
				when_infinite: x.is_infinite implies x.mantissa_bit_pattern = 0
			end
			Result := True
		end

invariant
	Zero: Zero.value = 0
	Complex_one_real: Complex_one.real.value = 1
	Complex_one_imaginary: Complex_one.imaginary.value = 0
	NaN: nan.is_nan
	negative_infinity: negative_infinity.is_negative_infinity
	positive_infinity: positive_infinity.is_positive_infinity

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
