﻿note
	description: "Implementation of {UNIVERSE_PROPERTIES [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	COMPLEX_UNIVERSE_PROPERTIES

inherit
	UNIVERSE_PROPERTIES [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
		rename
			current_universe as c,
			subtracted_ok as sp_subtracted_ok,
			universe_anchor as c_anchor
		undefine
			c,
			Set_to_set,
			Element_to_pair,
			Element_to_pairs,
			Element_to_boolean,
			Tuple_to_natural,
			Natural_to_real_64,
			Natural_to_tuple_set,
			Element_to_set,
			Element_to_sets,
			Tuple_to_set
		redefine
--			c,
			c_anchor
		end

	COMPLEX_SET_PROPERTIES
		rename
			is_empty_ok as sp_is_empty_ok,
			others_ok as sp_others_ok,
			does_not_have_ok as sp_does_not_have_ok,
			with_ok as sp_with_ok,
			equals_ok as csp_equals_ok,
			is_subset_ok as csp_is_subset_ok,
			is_superset_ok as csp_is_superset_ok,
			is_not_superset_ok as csp_is_not_superset_ok,
			is_comparable_ok as csp_is_comparable_ok,
			is_not_comparable_ok as csp_is_not_comparable_ok,
			is_strict_subset_ok as csp_is_strict_subset_ok,
			is_strict_superset_ok as csp_is_strict_superset_ok,
			is_trivial_subset_ok as csp_is_trivial_subset_ok,
			is_proper_subset_ok as csp_is_proper_subset_ok,
			is_proper_superset_ok as csp_is_proper_superset_ok,
			is_disjoint_ok as csp_is_disjoint_ok,
			complemented_ok as csp_complemented_ok,
			united_ok as csp_united_ok,
			intersected_ok as csp_intersected_ok,
			subtracted_symmetricaly_ok as csp_subtracted_symmetricaly_ok
		redefine
--			c,
			make_empty_ok,
			make_singleton_ok,
			make_extended_ok,
			Set_anchor,
			c_anchor
		end

feature -- Access

--	c: STI_COMPLEX_UNIVERSE
--			-- <Precursor>
--		once
--			Result := {STI_COMPLEX_SET}.C
--		ensure then
--			class
--		end

feature -- Properties (Initialization)

	make_empty_ok: BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_UNIVERSE}.make_empty?
		do
			-- Nothing to do.
		end

	make_singleton_ok (a: STS_COMPLEX_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_UNIVERSE}.make_singleton?
		do
			-- Nothing to do.
		end

	make_extended_ok (z: STS_COMPLEX_NUMBER; s: STS_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_UNIVERSE}.make_extended?
		local
			extension_failed: BOOLEAN
		do
			if
				not extension_failed and (
					agent (ia_z: STS_COMPLEX_NUMBER; ia_s: STS_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]): BOOLEAN
						local
							chk_done: BOOLEAN
						do
							if not chk_done then
								check
									does_not_have: ia_s ∌ ia_z -- This test may be too expensive if assertions are fully checked, e.g. if ia_s is a universe.
								end
								Result := True
							end
						rescue
							if {EXCEPTIONS}.tag_name ~ "does_not_have" then
								chk_done := True
								retry
							end
						end (z, s)
					).item
			then
				check
					cl_c: attached (create {COMPLEX_UNIVERSE}) as cl_c -- TODO: Make an anchor?
					apply_make_extended: (
						agent (ia_c: COMPLEX_UNIVERSE; ia_z: STS_COMPLEX_NUMBER; ia_s: STS_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]): BOOLEAN
							do
									-- It will fail if ia_z and ia_s do not comply with internal {STI_COMPLEX_UNIVERSE}.make_extended requirements.
								ia_c.make_extended (ia_z, ia_s)
								Result := True
							end (cl_c, z, s)
						).item
					still_the_universe: C ≍ cl_c
					definition: cl_c ≍ (s & z)
					decomposed: (cl_c / z) ≍ s
					cardinality: # cl_c = # s + 1
				end
			end
			Result := True
		rescue
			if {EXCEPTIONS}.recipient_name ~ "make_extended" then
				extension_failed := True
				retry
			end
		end

feature -- Properties (Conversion)

	as_set_of_pairs_ok (u: STI_COMPLEX_UNIVERSE): BOOLEAN
			-- Do the properties verified within set theory hold for
			-- {STI_COMPLEX_UNIVERSE}.as_set_of_pairs?
		do
			check
				complex_plane: u ≍ (× u.R)
			end
			Result := True
		end

feature -- Properties (Factory)

	r_ok (u: STI_COMPLEX_UNIVERSE): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_UNIVERSE}.R?
		do
			check
				squared: {
					STI_TRANSFORMER [
						STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
						STS_COMPLEX_NUMBER,
						STS_ORDERED_PAIR_EQUALITY [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
						STS_COMPLEX_EQUALITY
						]
					}.set_map (× u.R, agent u.complex_from_pair) ≍ u
			end
			Result := True
		end

	real_universe_ok (u: STI_COMPLEX_UNIVERSE): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_COMPLEX_UNIVERSE}.Real_universe?
		do
			check
				squared: {
					STI_TRANSFORMER [
						STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
						STS_COMPLEX_NUMBER,
						STS_ORDERED_PAIR_EQUALITY [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]],
						STS_COMPLEX_EQUALITY
						]
					}.set_map (× u.Real_universe, agent u.complex_from_pair) ≍ u
			end
			Result := True
		end

feature -- Anchor

	set_anchor: STI_COMPLEX_UNIVERSE
			-- <Precursor>
		once
			Result := C
		end

	c_anchor: STI_COMPLEX_UNIVERSE
			-- Anchor for `C`
		once
			Result := C
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
