note
	description: "Implementation of {STS_INTEGER_SET_PROPERTIES}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	INTEGER_SET_PROPERTIES

inherit
	STP_INTEGER_SET_PROPERTIES
		undefine
			has_ok,
			with_ok,
			without_ok,
			subsets_ok,
			powerset_ok,
			sp_united_ok,
			sp_intersected_ok,
			sp_subtracted_ok,
			sp_subtracted_symmetricaly_ok,
			mapped_ok,
			singleton_ok
		redefine
			default_create
		end

	ORDERED_NUMERIC_SET_PROPERTIES [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
		rename
			current_universe as z,
			universe_anchor as z_anchor
		redefine
			z,
			set_to_set,
			element_to_pair,
			element_to_pairs,
			element_to_boolean,
			tuple_to_natural,
			natural_to_real_64,
			natural_to_tuple_set,
			element_to_set,
			element_to_sets,
			tuple_to_set
		end

	SET_PROPERTIES [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
		rename
			current_universe as z,
			universe_anchor as z_anchor,
			equals_ok as sp_equals_ok,
			is_subset_ok as sp_is_subset_ok,
			is_superset_ok as sp_is_superset_ok,
			is_not_superset_ok as sp_is_not_superset_ok,
			is_comparable_ok as sp_is_comparable_ok,
			is_not_comparable_ok as sp_is_not_comparable_ok,
			is_strict_subset_ok as sp_is_strict_subset_ok,
			is_strict_superset_ok as sp_is_strict_superset_ok,
			is_proper_subset_ok as sp_is_proper_subset_ok,
			is_proper_superset_ok as sp_is_proper_superset_ok,
			is_disjoint_ok as sp_is_disjoint_ok,
			complemented_ok as sp_complemented_ok,
			united_ok as sp_united_ok,
			intersected_ok as sp_intersected_ok,
			subtracted_ok as sp_subtracted_ok,
			subtracted_symmetricaly_ok as sp_subtracted_symmetricaly_ok
		redefine
			default_create,
			z,
			set_to_set,
			element_to_pair,
			element_to_pairs,
			element_to_boolean,
			tuple_to_natural,
			natural_to_real_64,
			natural_to_tuple_set,
			element_to_set,
			element_to_sets,
			tuple_to_set,
			set_anchor,
			z_anchor
		end

feature {NONE} -- Initialization

	default_create
			-- Initialize current property checker
			--| TODO: what about turning post-conditions into invariants?
		do
			o := {STI_INTEGER_SET}.O
			unit := {STI_INTEGER_SET}.Unit
			eq := {STI_INTEGER_SET}.Eq
		end

feature -- Access

	Zero: STI_INTEGER_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	z: STI_INTEGER_UNIVERSE
			-- The integer "Universe", i.e. set with every representable integer number,
		once
			Result := {STI_INTEGER_SET}.Z
		ensure then
			class
		end

feature -- Properties (Construction)

	numeric_batch_extended_ok (s0: STS_SET [STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]]; s: STI_INTEGER_SET): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_INTEGER_SET}.numeric_batch_extended?
		do
			check
				cardinality: s0.is_disjoint (s) implies # s.numeric_batch_extended (s0) = # s + # s0
			end
			Result := True
		end

feature -- Transformer

	set_to_set: STI_TRANSFORMER [STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_SET_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_SET_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]]
			-- <Precursor>
		once
			create Result
		end

	element_to_pair: STI_TRANSFORMER [STS_INTEGER_NUMBER, STS_ORDERED_PAIR [STS_INTEGER_NUMBER, STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY], STS_INTEGER_EQUALITY, STS_ORDERED_PAIR_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY]]
			-- <Precursor>
		once
			create Result
		end

	element_to_pairs: STI_TRANSFORMER [STS_INTEGER_NUMBER, STS_SET [STS_ORDERED_PAIR [STS_INTEGER_NUMBER, STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY], STS_ORDERED_PAIR_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY]], STS_INTEGER_EQUALITY, STS_SET_EQUALITY [STS_ORDERED_PAIR [STS_INTEGER_NUMBER, STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY], STS_ORDERED_PAIR_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY]]]
			-- <Precursor>
		once
			create Result
		end

	element_to_boolean: STI_TRANSFORMER [STS_INTEGER_NUMBER, BOOLEAN, STS_INTEGER_EQUALITY, STS_OBJECT_EQUALITY [BOOLEAN]]
			-- <Precursor>
		once
			create Result
		end

	tuple_to_natural: STI_TRANSFORMER [STS_N_TUPLE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], NATURAL_32, STS_N_TUPLE_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_OBJECT_EQUALITY [NATURAL_32]]
			-- <Precursor>
		once
			create Result
		end

	natural_to_real_64: STI_TRANSFORMER [NATURAL_32, REAL_64, STS_OBJECT_EQUALITY [NATURAL_32], STS_OBJECT_EQUALITY [REAL_64]]
			-- <Precursor>
		once
			create Result
		end

	natural_to_tuple_set: STI_TRANSFORMER [NATURAL_32, STS_SET [STS_N_TUPLE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_N_TUPLE_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]], STS_OBJECT_EQUALITY [NATURAL_32], STS_SET_EQUALITY [STS_N_TUPLE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_N_TUPLE_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]]]
			-- <Precursor>
		once
			create Result
		end

	element_to_set: STI_TRANSFORMER [STS_INTEGER_NUMBER, STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_INTEGER_EQUALITY, STS_SET_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]]
			-- <Precursor>
		once
			create Result
		end

	element_to_sets: STI_TRANSFORMER [STS_INTEGER_NUMBER, STS_SET [STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_SET_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]], STS_INTEGER_EQUALITY, STS_SET_EQUALITY [STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_SET_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]]]
			-- <Precursor>
		once
			create Result
		end

	tuple_to_set: STI_TRANSFORMER [STS_N_TUPLE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_N_TUPLE_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY], STS_SET_EQUALITY [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]]
			-- <Precursor>
		once
			create Result
		end

feature -- Anchor

	set_anchor: STI_INTEGER_SET
			-- <Precursor>
		once
			Result := {STI_INTEGER_SET}.O
		ensure then
			class
		end

	integer_anchor: STI_INTEGER_NUMBER
			-- <Precursor>
		once
		ensure then
			class
		end

	z_anchor: STI_INTEGER_SET
			-- <Precursor>
		once
			Result := {STI_INTEGER_SET}.O
		ensure then
			class
		end

invariant
	Zero: Zero.complex_equals (Zero.Zero)

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
