﻿note
	description: "Implementation of {UNIVERSE_PROPERTIES [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	INTEGER_UNIVERSE_PROPERTIES

inherit
	UNIVERSE_PROPERTIES [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
		rename
			current_universe as z,
			subtracted_ok as sp_subtracted_ok,
			universe_anchor as z_anchor
		undefine
			z,
			set_to_set,
			element_to_pair,
			element_to_pairs,
			element_to_boolean,
			tuple_to_natural,
			natural_to_real_64,
			natural_to_tuple_set,
			element_to_set,
			tuple_to_set,
			element_to_sets
		redefine
--			z,
			z_anchor
		end

	INTEGER_SET_PROPERTIES
		rename
			is_empty_ok as sp_is_empty_ok,
			others_ok as sp_others_ok,
			does_not_have_ok as sp_does_not_have_ok,
			with_ok as sp_with_ok,
			equals_ok as isp_equals_ok,
			is_subset_ok as isp_is_subset_ok,
			is_superset_ok as isp_is_superset_ok,
			is_not_superset_ok as isp_is_not_superset_ok,
			is_comparable_ok as isp_is_comparable_ok,
			is_not_comparable_ok as isp_is_not_comparable_ok,
			is_strict_subset_ok as isp_is_strict_subset_ok,
			is_strict_superset_ok as isp_is_strict_superset_ok,
			is_trivial_subset_ok as isp_is_trivial_subset_ok,
			is_proper_subset_ok as isp_is_proper_subset_ok,
			is_proper_superset_ok as isp_is_proper_superset_ok,
			is_disjoint_ok as isp_is_disjoint_ok,
			complemented_ok as isp_complemented_ok,
			united_ok as isp_united_ok,
			intersected_ok as isp_intersected_ok,
			subtracted_symmetricaly_ok as isp_subtracted_symmetricaly_ok
		redefine
--			z,
			make_empty_ok,
			make_singleton_ok,
			make_extended_ok,
			Set_anchor,
			z_anchor
		end

feature -- Access

--	z: STI_INTEGER_UNIVERSE
--			-- <Precursor>
--		once
--			Result := {STI_INTEGER_SET}.Z
--		ensure then
--			class
--		end

feature -- Properties (Initialization)

	make_empty_ok: BOOLEAN
			-- Do the properties verified within set theory hold for {STI_INTEGER_UNIVERSE}.make_empty?
		do
			-- Nothing to do.
		end

	make_singleton_ok (x: STS_INTEGER_NUMBER): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_INTEGER_UNIVERSE}.make_singleton?
		do
			-- Nothing to do.
		end

	make_extended_ok (x: STS_INTEGER_NUMBER; s: STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_INTEGER_UNIVERSE}.make_extended?
		local
			extension_failed: BOOLEAN
		do
			if not extension_failed and s ∌ x then
				check
					cl_z: attached (create {INTEGER_UNIVERSE}) as cl_z -- TODO: Make an anchor?
					apply_make_extended: (
						agent (ia_r: INTEGER_UNIVERSE; ia_x: STS_INTEGER_NUMBER; ia_s: STS_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]): BOOLEAN
							do
									-- It will fail if ia_x and ia_s do not comply with internal {STI_INTEGER_UNIVERSE}.make_extended requirements.
								ia_r.make_extended (ia_x, ia_s)
								Result := True
							end (cl_z, x, s)
						).item
					still_the_universe: Z ≍ cl_z
					definition: cl_z ≍ (s & x)
					decomposed: (cl_z / x) ≍ s
					cardinality: # cl_z = # s + 1
				end
			end
			Result := True
		rescue
			if {EXCEPTIONS}.recipient_name ~ "make_extended" then
				extension_failed := True
				retry
			end
		end

feature -- Anchor

	set_anchor: STI_INTEGER_UNIVERSE
			-- <Precursor>
		once
			Result := Z
		end

	z_anchor: STI_INTEGER_UNIVERSE
			-- Anchor for `R`
		once
			Result := Z
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
