﻿note
	description: "Implementation of {STP_TRANSFORMER_PROPERTIES}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

class
	TRANSFORMER_PROPERTIES [
		A, B,
		EQ_A -> STS_EQUALITY [A] create default_create end,
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	STP_TRANSFORMER_PROPERTIES [A, B, EQ_A, EQ_B]
		redefine
			default_create,
			set_map_ok,
			tuple_map_ok,
			pair_ok
		end

feature {NONE} -- Initialization

	default_create
			-- Initialize current property checker
			--| TODO: what about turning post-conditions into invariants?
		do
			create transformable_set_properties
			o_b := transformable_set_properties.o_b
			unit_b := transformable_set_properties.set_properties_b.unit
			eq_b := transformable_set_properties.eq_b
		end

feature -- Access

	o_b: like current_universe_b.subset_anchor
			-- <Precursor>

	unit_b: like current_universe_b.n_tuple_anchor
			-- <Precursor>

	eq_b: EQ_B
			-- <Precursor>

	current_universe_a: like transformable_set_properties.universe_a_anchor
			-- <Precursor>
		do
			Result := transformable_set_properties.current_universe_a
		end

	current_universe_b: like transformable_set_properties.current_universe_b
			-- <Precursor>
		do
			Result := transformable_set_properties.current_universe_b
		end

	transformable_set_properties: TRANSFORMABLE_SET_PROPERTIES [A, B, EQ_A, EQ_B]
			-- Object that checks whether the properties verified within set theory hold for an implementation of {STS_TRANSFORMABLE_SET [A, B, EQ_A, EQ_B]}

feature -- Properties (Operation)

	singleton_crossed_set_ok (tr: STI_TRANSFORMER [A, B, EQ_A, EQ_B] a: A; s: STS_SET [B, EQ_B]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_TRANSFORMER}.singleton_crossed_set?
		do
			check
				u_a: attached current_universe_a as u_a
				u_b: attached current_universe_b as u_b

				definition: tr.singleton_crossed_set (a, s) ≍
					(u_a × u_b).filtered_xy (agent (u_a.transformer).anded (agent u_a.equality_holds (?, a), ?, agent s.has, ?))
			end
			Result := True
		end

feature -- Properties (Transformation)

	set_map_ok (tr: STS_TRANSFORMER [A, B, EQ_A, EQ_B]; s: STS_SET [A, EQ_A]; f: FUNCTION [A, B]): BOOLEAN
			-- <Precursor>
		local
			bs: STS_SET [B, EQ_B]
		do
			bs := tr.set_map (s, f) -- This makes `f' (x) collectible by `current_universe_b', for every x ∈ `s'.
			Result := Precursor {STP_TRANSFORMER_PROPERTIES}(tr, s, f)
		end

	set_reduction_ok (tr: STI_TRANSFORMER [A, B, EQ_A, EQ_B]; s: STI_SET [A, EQ_A]; start: B; f: FUNCTION [B, A, B]; ): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_TRANSFORMER}.set_reduction?
		do
			check
				definition: eq_b (tr.set_reduction (s, start, f), tr.tuple_left_reduction (s.as_tuple, start, f))
					-- NOTICE: This property applies to {STI_TRANSFORMER}.set_reduction and {STI_SET}.as_tuple respective implementations; other implementations
					-- need not comply to such a property. Please see the comment at {STS_SET}.reduced header.
			end
			Result := True
		end

	tuple_map_ok (tr: STS_TRANSFORMER [A, B, EQ_A, EQ_B]; t: STS_N_TUPLE [A, EQ_A]; f: FUNCTION [A, B]): BOOLEAN
			-- <Precursor>
		local
			bs: STS_SET [B, EQ_B]
		do
			bs := tr.set_map (t.terms, f) -- This makes `f' (x) collectible by `current_universe_b', for every x ∈ `t'.`terms'.
			Result := Precursor {STP_TRANSFORMER_PROPERTIES}(tr, t, f)
		end

feature -- Properties (Factory)

	pair_ok (tr: STS_TRANSFORMER [A, B, EQ_A, EQ_B]; a: A; b: B): BOOLEAN
			-- <Precursor>
		local
			sa: STS_SET [A, EQ_A]
			sb: STS_SET [B, EQ_B]
		do
			if transformable_set_properties.type_is_predefined_expanded (({A}).type_id) then
				sa := transformable_set_properties.singleton (a) -- This makes `a' collectible by `current_universe_a'.
			end
			if transformable_set_properties.type_is_predefined_expanded (({B}).type_id) then
				sb := transformable_set_properties.set_properties_b.singleton (b) -- This makes `b' collectible by `current_universe_b'.
			end
			Result := Precursor {STP_TRANSFORMER_PROPERTIES}(tr, a, b)
		end

	o_ok (tr: STI_TRANSFORMER [A, B, EQ_A, EQ_B]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_TRANSFORMER}.o?
		do
			check
				u_a: attached current_universe_a as u_a
				u_b: attached current_universe_b as u_b

				definition: tr.o ≍ (u_a × u_b).o
			end
			Result := True
		end

	empty_product_ok (tr: STI_TRANSFORMER [A, B, EQ_A, EQ_B]): BOOLEAN
			-- Do the properties verified within set theory hold for and {STI_TRANSFORMER}.empty_product?
		do
			check
				u_a: attached current_universe_a as u_a
				u_b: attached current_universe_b as u_b

				definition: tr.empty_product ≍ (u_a × u_b).empty_set
			end
			Result := True
		end

feature -- Transformer

	element_to_set: STI_TRANSFORMER [A, STS_SET [B, EQ_B], EQ_A, STS_SET_EQUALITY [B, EQ_B]]
			-- <Precursor>
		do
			create Result
		ensure then
			class
		end

	term_to_tuple: STI_TRANSFORMER [A, STS_N_TUPLE [B, EQ_B], EQ_A, STS_N_TUPLE_EQUALITY [B, EQ_B]]
			-- <Precursor>
		do
			create Result
		ensure then
			class
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
