﻿note
	description: "Implementation of {STP_N_TUPLE_PROPERTIES}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

class
	N_TUPLE_PROPERTIES [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	STP_N_TUPLE_PROPERTIES [A, EQ]
		redefine
			default_create,
			equals_ok,
			unequals_ok,
			mapped_ok
		end

feature {NONE} -- Initialization

	default_create
			-- <Precursor>
			--| TODO: what about turning post-conditions into invariants?
		do
			create set_properties
			create set_properties_n
			o := set_properties.o
			eq := set_properties.eq
			o_n := set_properties_n.o_a
			unit := set_properties.unit
		end

feature -- Access

	o: like universe_anchor.subset_anchor
			-- <Precursor>
			--| TODO: what about turning post-conditions into invariants?

	eq: EQ
			-- <Precursor>

	o_n: like unit.indices
			-- The empty set of natural numbers

	unit: like universe_anchor.n_tuple_anchor
			-- <Precursor>

	current_universe: like universe_anchor
			-- <Precursor>
		do
			Result := set_properties.current_universe
		end

	current_universe_n: like unit.indices
			-- <Precursor>
		do
			Result := set_properties_n.current_universe_a
		end

	set_properties: SET_PROPERTIES [A, EQ]
			-- Object that checks whether the properties verified within set theory hold for an implementation of {STS_SET}

	set_properties_n: TRANSFORMABLE_SET_PROPERTIES [NATURAL, A, STS_OBJECT_EQUALITY [NATURAL], EQ]
			-- Object that checks whether the properties verified within set theory hold for an implementation of
			-- {STS_TRANSFORMABLE_SET [NATURAL, A, STS_OBJECT_EQUALITY [NATURAL], EQ]}

feature -- Properties (Initialization)

	make_unit_ok: BOOLEAN
			-- Do the properties verified within set theory hold for {STS_N_TUPLE}.make_unit?
		do
			check
				u: attached current_universe as u

				t: attached (create {like tuple_anchor}.make_unit) as t
				has_nothing: u |∀ agent negated (agent t.has, ?)
				uniqueness: t ≍ unit
				no_term: t.n = 0
				empty_terms: t.terms ≍ o
			end
			Result := True
		end

	make_after_ok (a: A; t: STS_N_TUPLE [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_N_TUPLE}.make_after?
		do
			check
				a_t: attached (create {like tuple_anchor}.make_after (a, t)) as a_t
				n: a_t.n = t.n + 1
				terms: a_t.terms ≍ (t.terms & a)
			end
			Result := True
		end

	make_n_ok (n: NATURAL; a: A): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_N_TUPLE}.make_n?
		do
			check
				t: attached (create {like tuple_anchor}.make_n (n, a)) as t
				repeated_terms: t.for_all (
					agent (i: NATURAL; x_i, ia_a: A): BOOLEAN
						do
							Result := eq (x_i, ia_a)
						end (?, ?, a)
					)
			end
			Result := True
		end

feature -- Properties (Output)

	terms_out_ok (t: STI_N_TUPLE [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_N_TUPLE}.terms_out?
		do
			inspect
				t.n
			when 0 then
				check
					no_term: t.terms_out ~ ""
				end
			when 1 then
					check
						is_not_unit: not t.is_unit -- t.n = 1
					end
				check
					first: t.terms_out ~ t.term_out (t.first)
				end
			else -- t.n > 1
					check
						is_not_unit: not t.is_unit -- t.n > 1
					end
				check
					by_tail: t.terms_out ~ t.term_out (t.first) + "," + t.tail.terms_out
				end
			end
			Result := True
		end

feature -- Properties (Comparison)

	equals_ok (t1, t2: STS_N_TUPLE [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			ns: like bounded_indices
		do
			ns := bounded_indices (t1.n ∨ t2.n) -- It will make every i ∈ {1, ..., t1.n.max (t2.n)} collectible by current_universe_n.
			Result := Precursor {STP_N_TUPLE_PROPERTIES}(t1, t2)
		end

	unequals_ok (t1, t2: STS_N_TUPLE [A, EQ]): BOOLEAN
			-- <Precursor>
		local
			ns: like bounded_indices
		do
			ns := bounded_indices (t1.n ∨ t2.n) -- It will make every i ∈ {1, ..., t1.n.max (t2.n)} collectible by current_universe_n.
			Result := Precursor {STP_N_TUPLE_PROPERTIES}(t1, t2)
		end

feature -- Properties (Transformation)

	mapped_ok (t: STS_N_TUPLE [A, EQ]; f: FUNCTION [A, A]): BOOLEAN
			-- <Precursor>
		local
			s: STS_SET [A, EQ]
		do
			s := t.terms ↦ f -- This makes `f' (x) collectible by `current_universe', for every x ∈ `t'.`terms'.
			Result := Precursor {STP_N_TUPLE_PROPERTIES}(t, f)
		end

feature -- Conversion

	natural_as_integer (a_n: NATURAL): INTEGER_64
			-- `a_n' converted into an integer value
			--| TODO: make it more flexible.
			--| TODO: DRY: {SET}.natural_as_integer
		do
			Result := a_n.as_integer_64
		ensure
			class
			definition: Result = a_n.as_integer_64
		end

feature -- Factory

	bounded_indices (n: NATURAL): STI_TRANSFORMABLE_SET [NATURAL, A, STS_OBJECT_EQUALITY [NATURAL], EQ]
			-- Set of indices from 1 up to `n', i.e. {1..`n'}.
		local
			i: NATURAL
		do
			from
				Result := o_n
				i := n
			invariant
				building_up: Result ≍ (bounded_indices (n) ∖ bounded_indices (i))
			until
				i = 0
			loop
					check
						does_not_have: Result ∌ i -- Result ≍ (bounded_indices (n) ∖ bounded_indices (i))
					end
				Result := Result.extended (i)
				i := i - 1
			variant
				i: natural_as_integer (i)
			end
		ensure
			base: n = 0 implies Result.is_empty
			induction: n > 0 implies Result ≍ (bounded_indices (n - 1) & n)
		end

feature -- Transformer

	term_to_tuple: STI_TRANSFORMER [A, STS_N_TUPLE [A, EQ], EQ, STS_N_TUPLE_EQUALITY [A, EQ]]
			-- <Precursor>
		do
			create Result
		ensure then
			class
		end

	term_to_set: STI_TRANSFORMER [A, STS_SET [A, EQ], EQ, STS_SET_EQUALITY [A, EQ]]
			-- <Precursor>
		do
			create Result
		ensure then
			class
		end

feature -- Anchor

	tuple_anchor: STI_N_TUPLE [A, EQ]
			-- Anchor for n-tuples whose properties must be checked
		do
			Result := unit
		end

	universe_anchor: STI_SET [A, EQ]
			-- <Precursor>
		do
			Result := o
		end

feature {NONE} -- Implementation

	max_subsets: NATURAL = 1_048_576 -- 2 ^ 20
			-- <Precursor>

	max_tuples: NATURAL = 46_656 -- 6 ^ 6
			-- <Precursor>

invariant
	o_n_is_empty: o_n.is_empty

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
