﻿note
	description: "Implementation of {STP_SET_PARTITION_PROPERTIES}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

class
	SET_PARTITION_PROPERTIES [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	STP_SET_PARTITION_PROPERTIES [A, EQ]
		undefine
			default_create,
			mapped_ok
		end

	SET_FAMILY_PROPERTIES [A, EQ]
		rename
			make_singleton_ok as sfp_set_make_singleton_ok,
			make_extended_ok as sfp_set_make_extended_ok,
			with_ok as family_with_ok,
			extended_ok as family_extended_ok,
			batch_extended_ok as family_batch_extended_ok,
			family_anchor as partition_anchor
		redefine
			sfp_set_make_singleton_ok,
			sfp_set_make_extended_ok,
			partition_anchor
		end

feature -- Properties (Initialization)

	sfp_set_make_singleton_ok (s: STS_SET [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_SET_PARTITION}.set_make_singleton?
		do
			Result := family_make_singleton_ok (create {like partition_anchor}.make_empty, s, s)
		end

	family_make_singleton_ok (sp: EXPOSED_SET_PARTITION [A, EQ]; s0, s: STS_SET [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_SET_PARTITION}.family_make_singleton?
		do
			if not s.is_empty then
				check
					set_make_singleton_got_rescued: (
						agent (ia_sp: EXPOSED_SET_PARTITION [A, EQ]; ia_s0, ia_s: STS_SET [A, EQ]): BOOLEAN
							require
								is_not_empty: not ia_s.is_empty
							local
								s0_has_failed: BOOLEAN
							do
								if not s0_has_failed then
									ia_sp.family_make_singleton (ia_s0)
								else
									ia_sp.family_make_singleton (ia_s)
								end
								Result := True
							ensure
								ia_s0: not ia_s0.is_empty implies ia_sp ≍ ia_s0.as_singleton
								ia_s: ia_s0.is_empty and not ia_s.is_empty implies ia_sp ≍ ia_s.as_singleton
							rescue
								if not s0_has_failed then
									s0_has_failed := True
									retry
								end
							end
						).item (sp, s0, s)
					is_not_empty: not sp.is_empty
					others: sp.others ≍ o_s
					cardinality: # sp = 1
					union_cardinality: # (⋃ sp) = # sp.any
				end
			end
			Result := True
		ensure
			s0: not s0.is_empty implies sp ≍ s0.as_singleton
			s: s0.is_empty and not s.is_empty implies sp ≍ s.as_singleton
		end

	make_singleton_ok (s: STS_SET [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_SET_PARTITION}.make_singleton?
		do
			if not s.is_empty then
				check
					sp: attached (create {like partition_anchor}.make_singleton (s)) as sp
					is_not_empty: not sp.is_empty
					others: sp.others ≍ o_s
					cardinality: # sp = 1
					union_cardinality: # (⋃ sp) = # s
				end
			end
			Result := True
		end

	sfp_set_make_extended_ok (s: STS_SET [A, EQ]; ss: STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_SET_PARTITION}.set_make_extended?
		do
			Result := family_make_extended_ok (create {like partition_anchor}.make_empty, s, s, ss, ss)
		end

	family_make_extended_ok (sp: EXPOSED_SET_PARTITION [A, EQ]; s0, s: STS_SET [A, EQ]; ss0, ss: STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_SET_PARTITION}.family_make_extended?
		do
			if ss0 ∌ s0 and ss0 ∌ s and ss ∌ s0 and ss ∌ s and sp.family_converted (ss).is_partition and not s.is_empty and sp.family_converted (ss).set_is_disjoint (s) then
				check
					set_make_extended_got_rescued: (
						agent (ia_sp: EXPOSED_SET_PARTITION [A, EQ]; ia_s0, ia_s: STS_SET [A, EQ]; ia_ss0, ia_ss: STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]): BOOLEAN
								-- todo: symplify?
							require
								ia_ss0_does_not_have_ia_s0: ia_ss0 ∌ ia_s0
								ia_ss0_does_not_have_ia_s: ia_ss0 ∌ ia_s
								ia_ss_does_not_have_ia_s0: ia_ss ∌ ia_s0
								ia_ss_does_not_have_ia_s: ia_ss ∌ ia_s
								is_partition: ia_sp.family_converted (ia_ss).is_partition
								is_not_empty: not ia_s.is_empty
								set_is_disjoint: ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s)
							local
								ss0_violates_is_partition,
								s0_violates_is_not_empty,
								ss0_violates_set_is_disjoint_s0,
								ss0_violates_set_is_disjoint_s,
								ss_violates_set_is_disjoint_s0: BOOLEAN
							do
								if not ss0_violates_is_partition then
									if not s0_violates_is_not_empty then
										if not ss0_violates_set_is_disjoint_s0 then
											ia_sp.family_make_extended (ia_s0, ia_ss0)
										elseif not ss0_violates_set_is_disjoint_s then
											ia_sp.family_make_extended (ia_s, ia_ss0)
										elseif not ss_violates_set_is_disjoint_s0 then
											ia_sp.family_make_extended (ia_s0, ia_ss)
										else
											ia_sp.family_make_extended (ia_s, ia_ss)
										end
									elseif not ss0_violates_set_is_disjoint_s then
										ia_sp.family_make_extended (ia_s, ia_ss0)
									else
										ia_sp.family_make_extended (ia_s, ia_ss)
									end
								elseif not s0_violates_is_not_empty then
									if not ss_violates_set_is_disjoint_s0 then
										ia_sp.family_make_extended (ia_s0, ia_ss)
									else
										ia_sp.family_make_extended (ia_s, ia_ss)
									end
								else
									ia_sp.family_make_extended (ia_s, ia_ss)
								end
								Result := True
							ensure
								ia_ss0_ia_s0: ia_sp.family_converted (ia_ss0).is_partition and
									not ia_s0.is_empty and
									ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) implies

									ia_sp ≍ (ia_ss0 & ia_s0)

								ia_ss0_ia_s: ia_sp.family_converted (ia_ss0).is_partition and
									not ia_s0.is_empty and
									not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) and
									ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) or

									ia_sp.family_converted (ia_ss0).is_partition and
									ia_s0.is_empty and
									ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) implies

									ia_sp ≍ (ia_ss0 & ia_s)

								ia_ss_ia_s0: ia_sp.family_converted (ia_ss0).is_partition and
									not ia_s0.is_empty and
									not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) and
									not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) and
									ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) or

									not ia_sp.family_converted (ia_ss0).is_partition and
									not ia_s0.is_empty and
									ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) implies

									ia_sp ≍ (ia_ss & ia_s0)

								ia_ss_ia_s: ia_sp.family_converted (ia_ss0).is_partition and
									not ia_s0.is_empty and
									not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) and
									not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) and
									not ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) or

									ia_sp.family_converted (ia_ss0).is_partition and
									ia_s0.is_empty and
									not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) or

									not ia_sp.family_converted (ia_ss0).is_partition and
									not ia_s0.is_empty and
									not ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) or

									not ia_sp.family_converted (ia_ss0).is_partition and
									ia_s0.is_empty implies

									ia_sp ≍ (ia_ss & ia_s)
							rescue
								if not ss0_violates_is_partition and not ia_sp.family_converted (ia_ss0).is_partition then
									ss0_violates_is_partition := True
									retry
								end
								if not s0_violates_is_not_empty and ia_s0.is_empty then
									s0_violates_is_not_empty := True
									retry
								end
								if not ss0_violates_set_is_disjoint_s0 and not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) then
									ss0_violates_set_is_disjoint_s0 := True
									retry
								end
								if not ss0_violates_set_is_disjoint_s and not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) then
									ss0_violates_set_is_disjoint_s := True
									retry
								end
								if not ss_violates_set_is_disjoint_s0 and not ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) then
									ss_violates_set_is_disjoint_s0 := True
									retry
								end
							end
						).item (sp, s0, s, ss0, ss)
					sp_is_not_empty: not sp.is_empty
					sp_any: attached sp.any as sp_any
					sp_others: attached sp.others as sp_others
					set_is_pair_wise_disjoint: (sp_others / sp_any).set_is_disjoint (sp_any) -- sp.any ∉ sp.others
					definition: sp ≍ (sp_others & sp_any)
					decomposed: (sp / sp_any) ≍ sp_others
					cardinality: # sp = # sp_others + 1
					union_cardinality: # (⋃ sp) = # (⋃ sp_others) + # sp_any
				end
			end
			Result := True
		ensure
			ss0_s0: sp.family_converted (ss0).is_partition and
				not s0.is_empty and
				sp.family_converted (ss0).set_is_disjoint (s0) implies

				sp ≍ (ss0 & s0)

			ss0_s: sp.family_converted (ss0).is_partition and
				not s0.is_empty and
				not sp.family_converted (ss0).set_is_disjoint (s0) and
				sp.family_converted (ss0).set_is_disjoint (s) or

				sp.family_converted (ss0).is_partition and
				s0.is_empty and
				sp.family_converted (ss0).set_is_disjoint (s) implies

				sp ≍ (ss0 & s)

			ss_s0: sp.family_converted (ss0).is_partition and
				not s0.is_empty and
				not sp.family_converted (ss0).set_is_disjoint (s0) and
				not sp.family_converted (ss0).set_is_disjoint (s) and
				sp.family_converted (ss).set_is_disjoint (s0) or

				not sp.family_converted (ss0).is_partition and
				not s0.is_empty and
				sp.family_converted (ss).set_is_disjoint (s0) implies

				sp ≍ (ss & s0)

			ss_s: sp.family_converted (ss0).is_partition and
				not s0.is_empty and
				not sp.family_converted (ss0).set_is_disjoint (s0) and
				not sp.family_converted (ss0).set_is_disjoint (s) and
				not sp.family_converted (ss).set_is_disjoint (s0) or

				sp.family_converted (ss0).is_partition and
				s0.is_empty and
				not sp.family_converted (ss0).set_is_disjoint (s) or

				not sp.family_converted (ss0).is_partition and
				not s0.is_empty and
				not sp.family_converted (ss).set_is_disjoint (s0) or

				not sp.family_converted (ss0).is_partition and
				s0.is_empty implies

				sp ≍ (ss & s)
		end

	make_extended_ok (s: STS_SET [A, EQ]; sp: STI_SET_PARTITION [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_SET_PARTITION}.make_extended?
		local
			esp: like partition_anchor
		do
			if not s.is_empty and sp.set_is_disjoint (s) then
				create esp.make_extended (s, sp)
					check
						set_is_pair_wise_disjoint: (sp / s).set_is_disjoint (s) -- sp.set_is_disjoint (s)						
					end
				check
					definition: esp ≍ (sp & s)
					decomposed: (esp / s) ≍ sp
					cardinality: # esp = # sp + 1
					union_cardinality: # (⋃ esp) = # (⋃ sp) + # s
				end
			end
			Result := True
		end

feature -- Properties (Construction)

	extended_ok (sp: STI_SET_PARTITION [A, EQ]; s: STS_SET [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_SET_PARTITION}.extended?
		do
			if not s.is_empty and sp.set_is_disjoint (s) then
					check
						set_is_pair_wise_disjoint: (sp / s).set_is_disjoint (s) -- sp.set_is_disjoint (s)
					end
				check
					definition: sp.extended (s) ≍ (sp & s)
					decomposed: (sp.extended (s) / s) ≍ sp
					cardinality: # sp.extended (s) = # sp + 1
					union_cardinality: # (⋃ sp.extended (s)) = # (⋃ sp) + # s
				end
			end
			Result := True
		end

	batch_extended_ok (sp: STI_SET_PARTITION [A, EQ]; ss: STS_SET_OF_SETS [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STI_SET_PARTITION}.batch_extended?
		do
			if ss.is_partition and sp.sets_are_disjoint (ss) then
				check
					cardinality: # sp.batch_extended (ss) = # sp + # ss
					union_cardinality: # (⋃ sp.batch_extended (ss)) = # (⋃ sp) + # (⋃ ss)
				end
			end
			Result := True
		end

feature -- Anchor

	partition_anchor: EXPOSED_SET_PARTITION [A, EQ]
			-- Anchor for set partitions whose properties must be checked
		do
			create Result.make_empty
		ensure then
			class
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
