﻿note
	description: "Implementation of {STP_SUBSET_FAMILY_PROPERTIES}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

class
	SUBSET_FAMILY_PROPERTIES [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	STP_SUBSET_FAMILY_PROPERTIES [A, EQ]
		undefine
			default_create,
			mapped_ok
		end

	SET_OF_SETS_PROPERTIES [A, EQ]
		rename
			make_empty_ok as set_make_empty_ok,
			make_extended_ok as set_make_extended_ok,
			extended_ok as set_extended_ok,
			batch_extended_ok as set_batch_extended_ok,
			set_anchor as family_anchor
		redefine
			family_anchor,
			universe_s_anchor
		end

	SET_PROPERTIES [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]
		rename
			o as o_s,
			o_ok as o_s_ok,
			current_universe as current_universe_s,
			make_empty_ok as set_make_empty_ok,
			make_extended_ok as set_make_extended_ok,
			extended_ok as set_extended_ok,
			batch_extended_ok as set_batch_extended_ok,
			empty_set_ok as empty_set_of_sets_ok,
			singleton as singleton_s,
			set_anchor as family_anchor,
			universe_anchor as universe_s_anchor
		undefine
			has_ok,
			with_ok,
			without_ok,
			subsets_ok,
			powerset_ok,
			united_ok,
			intersected_ok,
			subtracted_ok,
			subtracted_symmetricaly_ok,
			singleton_ok
		redefine
			family_anchor,
			universe_s_anchor
		end

feature -- Properties (Initialization)

	make_empty_ok (s: STS_SET [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SUBSET_FAMILY}.make_empty?
		do
			check
				u_s: attached current_universe_s as u_s
				sf: attached (create {like family_anchor}.make_empty (s)) as sf
				has_nothing: u_s |∀ agent {STS_SET [A, EQ]}.is_not_in (sf)
				equality: sf ≍ o_s
				cardinality: # sf = 0
			end
			Result := True
		end

	make_extended_ok (s: STS_SET [A, EQ]; sf: STS_SUBSET_FAMILY [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SUBSET_FAMILY}.make_extended?
		do
			if s ∉ sf and s ⊆ sf.superset then
				check
					esf: attached (create {like family_anchor}.make_extended (s, sf)) as esf
					definition: esf ≍ (sf & s)
					decomposed: (esf / s) ≍ sf
				end
			end
			Result := True
		end

feature -- Properties (Construction)

	extended_ok (s: STS_SET [A, EQ]; sf: STI_SUBSET_FAMILY [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SUBSET_FAMILY}.extended?
		do
			if s ∉ sf and s ⊆ sf.superset then
				check
					definition: sf.extended (s) ≍ (sf & s)
					decomposed: (sf.extended (s) / s) ≍ sf
				end
			end
			Result := True
		end

	batch_extended_ok (sf0: STS_SUBSET_FAMILY [A, EQ]; sf: STI_SUBSET_FAMILY [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SUBSET_FAMILY}.batch_extended?
		do
			if sf.is_disjoint (sf0) and sf0.superset ⊆ sf.superset then
				check
					cardinality: # sf.batch_extended (sf0) = # sf + # sf0
				end
			end
			Result := True
		end

	replacing_superset_ok (s: STS_SET [A, EQ]; sf: STI_SUBSET_FAMILY [A, EQ]): BOOLEAN
			-- Do the properties verified within set theory hold for {STS_SUBSET_FAMILY}.replacing_superset?
		do
			check
				filtered: sf.replacing_superset (s) ≍ (sf | agent s.is_superset)
			end
			Result := True
		end

feature -- Anchor

	family_anchor: STI_SUBSET_FAMILY [A, EQ]
			-- Anchor for subset families whose properties must be checked
		do
			Result := o_s
		end

	universe_s_anchor: STI_SET_FAMILY [A, EQ]
			-- <Precursor>
		do
			Result := o_s
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
