﻿note
	description: "Implementation of {STP_TRANSFORMABLE_SET_PROPERTIES}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"

class
	TRANSFORMABLE_SET_PROPERTIES [A, B, EQ_A -> STS_EQUALITY [A] create default_create end, EQ_B -> STS_EQUALITY [B] create default_create end]

inherit
	STP_TRANSFORMABLE_SET_PROPERTIES [A, B, EQ_A, EQ_B]
		undefine
			has_ok,
			with_ok,
			without_ok,
			subsets_ok,
			powerset_ok,
			sp_united_ok,
			sp_intersected_ok,
			sp_subtracted_ok,
			sp_subtracted_symmetricaly_ok,
			mapped_to_same_kind_ok,
			mapped_ok,
			singleton_ok
		redefine
			default_create
		end

	SET_PROPERTIES [A, EQ_A]
		rename
			o as o_a,
			eq as eq_a,
			current_universe as current_universe_a,
			equals_ok as sp_equals_ok,
			is_subset_ok as sp_is_subset_ok,
			is_disjoint_ok as sp_is_disjoint_ok,
			united_ok as sp_united_ok,
			intersected_ok as sp_intersected_ok,
			subtracted_ok as sp_subtracted_ok,
			subtracted_symmetricaly_ok as sp_subtracted_symmetricaly_ok,
			crossed_ok as crossed_with_same_kind_ok,
			mapped_ok as mapped_to_same_kind_ok,
			reduced_ok as reduced_to_element_like_ok,
			element_to_set as element_to_set_of_same_kind,
			universe_anchor as universe_a_anchor
		redefine
			default_create,
			set_anchor,
			universe_a_anchor
		end

feature {NONE} -- Initialization

	default_create
			-- <Precursor>
			--| todo: what about turning post-conditions into invariants?
		do
			Precursor {SET_PROPERTIES}
			create set_properties_b
			o_b := set_properties_b.o
			eq_b := set_properties_b.eq
			o_ab := o_a × o_b
		end

feature -- Access

	o_b: like current_universe_b.subset_anchor
			-- <Precursor>

	eq_b: EQ_B
			-- <Precursor>

	o_ab: like universe_a_anchor.crossed
			-- <Precursor>

	current_universe_b: like set_properties_b.universe_anchor
			-- <Precursor>
		do
			Result := set_properties_b.current_universe
		end

	set_properties_b: SET_PROPERTIES [B, EQ_B]
			-- Object that checks whether the properties verified within set theory hold for an implementation of {STS_SET [B, EQ_B]}

feature -- Properties (Transformation)

	mapped_ok (s: STS_TRANSFORMABLE_SET [A, B, EQ_A, EQ_B] f: FUNCTION [A, B]): BOOLEAN
			-- <Precursor>
		local
			bs: STS_SET [B, EQ_B]
		do
			bs := s ↦ f -- This makes `f' (x) collectible by `current_universe_b', for every x ∈ `s'.
			Result := Precursor {STP_TRANSFORMABLE_SET_PROPERTIES}(s, f)
		end

feature -- Factory

	element_to_set: STI_TRANSFORMER [A, STS_SET [B, EQ_B], EQ_A, STS_SET_EQUALITY [B, EQ_B]]
			-- <Precursor>
		do
			create Result
		ensure then
			class
		end

feature -- Anchor

	set_anchor,
	universe_a_anchor: STI_TRANSFORMABLE_SET [A, B, EQ_A, EQ_B]
			-- <Precursor>
		do
			Result := o_a
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
