﻿note
	description: "Pool of features available for numeric test classes"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	NUMBERS_TESTS_RESOURCES [A -> STS_COMPLEX_NUMBER, EQ -> STS_NUMERIC_EQUALITY [A] create default_create end]

inherit
	SETS_TESTS_RESOURCES_UNARY [A, EQ]
		rename
			some_integer_ref as some_native_integer_ref,
			some_direct_integer_ref as some_direct_native_integer_ref,
			new_integer_ref as new_native_integer_ref,
			some_integer as some_native_integer
		redefine
			on_prepare,
			some_native_integer,
			some_element,
			some_equality,
			some_pair,
			some_set,
			some_n_tuple
		end

feature -- Access

	two: STI_INTEGER_NUMBER
			-- The integer number 2
--		once
--			create Result.make (2)
--		ensure
--			class
--			definition: Result.value = 2
--		end

	nan: STI_REAL_NUMBER
			-- Representation of not a number (NaN)
--		once
--			Result := {STI_REAL_NUMBER}.Nan
--		ensure
--			class
--			definition: Result.is_nan
--		end

	negative_infinity: STI_REAL_NUMBER
			-- Representation of negative infinity
--		once
--			Result := {STI_REAL_NUMBER}.Negative_infinity
--		ensure
--			class
--			definition: Result.is_negative_infinity
--		end

	positive_infinity: STI_REAL_NUMBER
			-- Representation of positive infinity
--		once
--			Result := {STI_REAL_NUMBER}.Positive_infinity
--		ensure
--			class
--			definition: Result.is_positive_infinity
--		end

feature {NONE} -- Events

	on_prepare
			-- <Precursor>
		do
			two := {STI_INTEGER_NUMBER}.Two
			nan := {STI_REAL_NUMBER}.Nan
			negative_infinity := {STI_REAL_NUMBER}.Negative_infinity
			positive_infinity := {STI_REAL_NUMBER}.Positive_infinity
		end

feature -- Factory (object)

	some_native_real_ref: REAL_REF
			-- Randomly-fetched polymorphic native real reference
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_native_real_ref
			when 1 then
				Result := some_native_real
			end
		end

	some_direct_native_real_ref: like some_native_real_ref
			-- Randomly-fetched monomorphic native real reference
		do
			check
					-- `some_direct_instance' and `new_native_real_ref' definitions
				x: attached {like new_native_real_ref} some_direct_instance (agent new_native_real_ref) as x
			then
				Result := x
					check
							-- `some_direct_instance' and `new_native_real_ref' definitions
						monomorphic: Result.generating_type ~ {detachable like new_native_real_ref}
					end
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_native_real_ref}
		end

	new_native_real_ref: like some_direct_native_real_ref
			-- Randomly-created monomorphic native real reference
		do
			Result := some_native_real.to_reference
		ensure
			class
			monomorphic: Result.generating_type ~ {detachable like some_direct_native_real_ref}
		end

	some_native_real: REAL
			-- Randomly-created native real number
		do
			if next_random_item \\ (# R).as_integer_32 = 0 then
				Result := {REAL}.nan
			else
				Result := next_random_item / ({INTEGER}.Max_value / ({STI_REAL_NUMBER}.Max_value.value + {STI_REAL_NUMBER}.Max_value.ulp.value))
				if next_random_item \\ 2 = 0 then
					Result := - Result
				end
			end
		ensure
			class
		end

	some_native_double_real_ref: like some_native_double_real.to_reference
			-- Randomly-fetched polymorphic native, double_precision real reference.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_native_real_64_ref
			when 1 then
				Result := some_native_double_real
			end
		end

	some_direct_native_real_64_ref: like some_native_double_real_ref
			-- Randomly-fetched monomorphic native, 64-bits real reference.
		do
			check
					-- `some_direct_instance' and `new_native_real_64_ref' definitions
				r64: attached {like new_native_real_64_ref} some_direct_instance (agent new_native_real_64_ref) as r64
			then
				Result := r64
					check
							-- `some_direct_instance' and `new_native_real_64_ref' definitions
						monomorphic: Result.generating_type ~ {detachable like new_native_real_64_ref}
					end
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_native_double_real_ref}
		end

	new_native_real_64_ref: like some_direct_native_real_64_ref
			-- Randomly-created monomorphic native, 64-bits real reference
		do
			Result := some_native_double_real.to_reference
		ensure
--			class
			monomorphic: Result.generating_type ~ {detachable like some_direct_native_real_64_ref}
		end

	some_native_double_real: like some_expanded_real.native_double_real_anchor
			-- Randomly-created native, 64 bits real number
		do
			if next_random_item \\ (# R).as_integer_32 = 0 then
				Result := {REAL_64}.nan
			else
--				Result := next_random_item / ({INTEGER}.Max_value / ({STI_REAL_NUMBER}.Max_value.value + 2)) -- TODO: {STI_REAL_NUMBER}.Max_value fails to call the once body. Why?
				Result := next_random_item / ({INTEGER}.Max_value / ({STI_REAL_NUMBER}.Max_value + {STI_REAL_NUMBER}.Max_value.ulp).value)
				if next_random_item \\ 2 = 0 then
					Result := - Result
				end
			end
--			set_object_pool (object_pool & Result)
--		ensure
--			class
		end

	some_native_integer: INTEGER
			-- Randomly-created native integer number
		do
			Result := next_random_item \\ ({STI_INTEGER_NUMBER}.Max_value.value - {STI_INTEGER_NUMBER}.Min_value.value + 1) +
				{STI_INTEGER_NUMBER}.Min_value.value
		end

	some_native_natural_ref: NATURAL_32_REF
			-- Randomly-fetched polymorphic native natural reference
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_native_natural_ref
			when 1 then
				Result := some_native_natural
			end
		end

	some_direct_native_natural_ref: like some_native_natural_ref
			-- Randomly-fetched monomorphic native natural reference
		do
			check
					-- `some_direct_instance' and `new_native_natural_ref' definitions
				n: attached {like new_native_natural_ref} some_direct_instance (agent new_native_natural_ref) as n
			then
				Result := n
					check
							-- `some_direct_instance' and `new_native_natural_ref' definitions
						monomorphic: Result.generating_type ~ {detachable like new_native_natural_ref}
					end
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_native_natural_ref}
		end

	new_native_natural_ref: like some_direct_native_natural_ref
			-- Randomly-created monomorphic native natural reference
		do
			Result := some_native_natural.to_reference
		ensure
--			class
			monomorphic: Result.generating_type ~ {detachable like some_direct_native_natural_ref}
		end

	some_native_natural: NATURAL
			-- Randomly-created native natural number
		do
--			Result := (next_random_item \\ ({STI_REAL_NUMBER}.Max_value.value.truncated_to_integer + 2)).as_natural_32 -- TODO: {STI_REAL_NUMBER}.Max_value fails to run the once body. Why?
			Result := (next_random_item \\ (some_expanded_real.Max_value.value.truncated_to_integer + 2)).as_natural_32 -- TODO: 2 is a magic number.
--			set_object_pool (object_pool & Result)
--		ensure
--			class
		end

feature -- Factory (element)

	some_element: STS_ELEMENT
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_complex
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

feature -- Factory (complex)

	some_complex: STS_COMPLEX_NUMBER
			-- Randomly-fetched polymorphic complex number
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_expanded_complex
			when 1 then
				Result := some_exposed_complex
			when 2 then
				Result := some_real
			end
		end

	some_expanded_complex: STI_COMPLEX_NUMBER
			-- Randomly-fetched expanded complex number.
		do
			create Result.make (some_real, some_real)
		end

	same_complex (ab: STS_COMPLEX_NUMBER): like some_complex
			-- Complex number equal to `ab'.
		do
			inspect
				next_random_item \\ 6
			when 0 then
				if not ab.is_real or next_random_item \\ 2 = 0 then -- TODO: Same integer etc.
					Result := ab
				else
					inspect
						next_random_item \\ 3
					when 0 then
						create {like some_expanded_complex} Result.make_from_real (ab.a)
					when 1 then
						create {like some_exposed_complex} Result.make_from_real (ab.a)
					when 2 then
						Result := same_real (ab.a)
					end
				end
			when 1 then
				Result := ab.twin
			when 2 then
				create {like some_expanded_complex} Result.make (ab.a, ab.b)
			when 3 then
				create {like some_expanded_complex} Result.make_from_expanded (ab.a, ab.b)
			when 4 then
				create {like some_exposed_complex} Result.make (ab.a, ab.b)
			when 5 then
				create {like some_exposed_complex} Result.make_from_expanded (ab.a, ab.b)
			end
		ensure
			definition: Result ≍ ab
		end

	some_exposed_complex: COMPLEX_NUMBER
			-- Randomly-fetched exposed complex number.
		do
			create Result.make (some_real, some_real)
		end

feature -- Factory (real)

	some_real: STS_REAL_NUMBER
			-- Randomly-fetched polymorphic real number
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_expanded_real
			when 1 then
				Result := some_exposed_real
			when 2 then
				Result := some_rational
			end
		end

	some_expanded_real: STI_REAL_NUMBER
			-- Randomly-fetched expanded real number.
		local
			xt: like R.As_tuple
			i: like some_n_tuple.n
		do
			xt := R.As_tuple
			i := (next_random_item.as_natural_32 \\ xt.n + 1)
				check
						-- As just assigned...
					positive: 0 < i
					small_enough: i ≤ xt.n
				end
			check
				res: attached {STI_REAL_NUMBER} xt [i] as res -- `xt' terms are `R' elements.
			then
				Result := res
			end
		end

	same_real (x: STS_REAL_NUMBER): like some_real
			-- Real number equal to `x'
		do
			if attached {like some_exposed_real} x as exp_x and next_random_item \\ 2 = 0 then
					check
						no_extra_bit: exp_x.value_bit_pattern & (⊝ {like some_exposed_real}.Value_bit_pattern_mask) = 0 -- By definition
					end
				create {like some_exposed_real} Result.make_from_bit_pattern (exp_x.value_bit_pattern)
			else
				inspect
					next_random_item \\ 6
				when 0 then
					if not x.is_integer or next_random_item \\ 2 = 0 then -- TODO: Same natural etc.
						Result := x
					else
						Result := same_integer (x.truncated_to_integer)
					end
				when 1 then
					Result := x.twin
				when 2 then
					create {like some_expanded_real} Result.make_from_real (x)
				when 3 then
					create {like some_expanded_real} Result.make (x.value)
				when 4 then
					create {like some_exposed_real} Result.make_from_real (x)
				when 5 then
					create {like some_exposed_real} Result.make (x.value)
				end
			end
		ensure
			definition: Result ≍ x
		rescue
			print (x.generating_type) print (": ") print (x) print ("; ")
			if attached Result then
				print (Result.generating_type) print (": ")
			end
			print (Result) print ('%N')
		end

	some_exposed_real: REAL_NUMBER
			-- Randomly-fetched exposed real number.
		do
			create Result.make_from_real (some_expanded_real)
		end

feature -- Factory (rational)

	some_rational: STS_RATIONAL_NUMBER
			-- Randomly-fetched polymorphic rational number
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_expanded_rational
			when 1 then
				Result := some_integer
			end
		end

	some_expanded_rational: STI_RATIONAL_NUMBER
			-- Randomly-fetched expanded rational number.
		local
			num, den: like some_integer
		do
			num := some_integer
			from
				den := some_integer
			until
				den ≭ den.zero
			loop
				den := some_integer
			end
			create Result.make (num, den)
		rescue
			print (num) print (", ")
			print (den) print ('%N')
		end

	same_rational (pq: STS_RATIONAL_NUMBER): like some_rational
			-- Rational number equal to `pq'
		do
			if
				pq.is_integer and pq ≥ pq.zero -- TODO: is_natural etc.
				and next_random_item \\ 2 = 0
			then
				inspect
					next_random_item \\ 3
				when 0 then
					Result := pq -- TODO: same_natural etc.
				when 1 then
					create {like some_expanded_rational} Result.make_from_native_natural (pq.truncated_to_integer.value.as_natural_32)
				when 2 then
					create {like some_expanded_rational} Result.make_from_native_natural_ref (pq.truncated_to_integer.value.as_natural_32.to_reference)
--				when 2 then
--					create {like some_expanded_rational} Result.make_from_natural (pq.truncated_to_integer.value.as_natural_32)
				end
			elseif pq.is_integer and next_random_item \\ 2 = 0 then
				inspect
					next_random_item \\ 4
				when 0 then
					Result := same_integer (pq.truncated_to_integer)
				when 1 then
					create {like some_expanded_rational} Result.make_from_native_integer (pq.truncated_to_integer.value)
				when 2 then
					create {like some_expanded_rational} Result.make_from_native_integer_ref (pq.truncated_to_integer.value.to_reference)
				when 3 then
					create {like some_expanded_rational} Result.make_from_integer (pq.truncated_to_integer)
				end
			else
				inspect
					next_random_item \\ 5
				when 0 then
					Result := pq
				when 1 then
					Result := pq.twin
				when 2 then
					create {like some_expanded_rational} Result.make (pq.p, pq.q)
				when 3 then
					create {like some_expanded_rational} Result.make_from_expanded (pq.p, pq.q)
				when 4 then
					create {like some_expanded_rational} Result.make_from_rational (pq)
				end
			end
		ensure
			definition: Result ≍ pq
		rescue
			print (pq.generating_type) print (": ") print (pq) print ("; ")
			if attached Result then
				print (Result.generating_type) print (": ")
			end
			print (Result) print ('%N')
		end

feature -- Factory (integer)

	some_integer: STS_INTEGER_NUMBER
			-- Randomly-fetched polymorphic integer number
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_expanded_integer
			when 1 then
				Result := some_exposed_integer
--			when 1 then
--				Result := some_natural
			end
		end

	some_expanded_integer: STI_INTEGER_NUMBER
			-- Randomly-fetched expanded integer number.
		local
			it: like Z.As_tuple
			i: like some_n_tuple.n
		do
			it := Z.As_tuple
			i := (next_random_item.as_natural_32 \\ it.n + 1)
				check
						-- As just assigned...
					positive: 0 < i
					small_enough: i ≤ it.n
				end
			check
				res: attached {STI_INTEGER_NUMBER} it [i] as res -- `it' terms are `Z' elements.
			then
				Result := res
			end
		end

	same_integer (i: STS_INTEGER_NUMBER): like some_integer
			-- Integer number equal to `i'
		do
			if attached {like some_exposed_integer} i as exp_i and next_random_item \\ 2 = 0 then
					check
							-- {like some_exposed_integer} invariant
						big_enough: {like some_exposed_integer}.Native_min_value ≤ exp_i.stored_value
						small_enough: exp_i.stored_value ≤ {like some_exposed_integer}.Native_max_value
					end
				create {like some_exposed_integer} Result.make_from_stored_value (exp_i.stored_value)
			else
				inspect
					next_random_item \\ 6
				when 0 then
					Result := i
				when 1 then
					Result := i.twin
				when 2 then
					create {like some_expanded_integer} Result.make_from_integer (i)
				when 3 then
					create {like some_expanded_integer} Result.make (i.value)
				when 4 then
					create {like some_exposed_integer} Result.make_from_integer (i)
				when 5 then
					create {like some_exposed_integer} Result.make (i.value)
				end
			end
		ensure
			definition: Result ≍ i
		end

	some_exposed_integer: INTEGER_NUMBER
			-- Randomly-fetched exposed integer number.
		do
			create Result.make_from_integer (some_expanded_integer)
		end

feature -- Factory (equality)

	some_equality: STS_EQUALITY [detachable separate ANY]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 5
			when 0 then
				Result := some_complex_equality
			when 1 then
				Result := some_real_equality
			when 2 then
				Result := some_rational_equality
			when 3 then
				Result := some_integer_equality
			when 4 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	some_complex_equality: STS_EQUALITY [STS_COMPLEX_NUMBER]
			-- Randomly-fetched polymorphic equality for complex numbers
		local
			new_eq: FUNCTION [like some_complex_equality]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				inspect
					next_random_item \\ 2
				when 0 then
					new_eq := agent new_complex_reference_equality
				when 1 then
					new_eq := agent new_complex_object_equality
				end
				check
					eq: attached {like some_complex_equality} some_direct_instance (new_eq) as eq -- `some_direct_instance' definition
				then
					Result := eq
				end
			when 1 then
				Result := some_complex_numeric_equality
			end
		end

	new_complex_reference_equality: STS_REFERENCE_EQUALITY [STS_COMPLEX_NUMBER]
			-- Randomly-created monomorphic reference-equality for complex numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_complex_reference_equality}
		end

	new_complex_object_equality: STS_OBJECT_EQUALITY [STS_COMPLEX_NUMBER]
			-- Randomly-created monomorphic object-equality for complex numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_complex_object_equality}
		end

	some_complex_numeric_equality: STS_NUMERIC_EQUALITY [STS_COMPLEX_NUMBER]
			-- Randomly-fetched polymorphic numeric equality for complex numbers
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_complex_numeric_equality
			when 1 then
				Result := some_complex_proper_equality
			end
		end

	some_direct_complex_numeric_equality: like some_complex_numeric_equality
			-- Randomly-fetched monomorphic numeric equality for complex numbers
		do
			check
				new_eq: attached {FUNCTION [like some_direct_complex_numeric_equality]} agent new_complex_numeric_equality as new_eq

					-- `some_direct_instance' definition
				eq: attached {like some_direct_complex_numeric_equality} some_direct_instance (new_eq) as eq
			then
				Result := eq
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_complex_numeric_equality}
		end

	new_complex_numeric_equality: like some_complex_numeric_equality
			-- Randomly-created monomorphic numeric equality for complex numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_complex_numeric_equality}
		end

	some_complex_proper_equality: STS_COMPLEX_EQUALITY
			-- Randomly-fetched polymorphic proper equality for complex numbers
		do
			check
				new_eq: attached {FUNCTION [like some_complex_proper_equality]} agent new_complex_proper_equality as new_eq

					-- `some_direct_instance' definition
				eq: attached {like some_complex_proper_equality} some_direct_instance (new_eq) as eq
			then
				Result := eq
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_complex_proper_equality}
		end

	new_complex_proper_equality: like some_complex_proper_equality
			-- Randomly-created monomorphic proper equality for complex numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_complex_proper_equality}
		end

	some_real_equality: STS_EQUALITY [STS_REAL_NUMBER]
			-- Randomly-fetched polymorphic equality for real numbers
		local
			new_eq: FUNCTION [like some_real_equality]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				inspect
					next_random_item \\ 2
				when 0 then
					new_eq := agent new_real_reference_equality
				when 1 then
					new_eq := agent new_real_object_equality
				end
				check
					eq: attached {like some_real_equality} some_direct_instance (new_eq) as eq -- `some_direct_instance' definition
				then
					Result := eq
				end
			when 1 then
				Result := some_real_numeric_equality
			end
		end

	new_real_reference_equality: STS_REFERENCE_EQUALITY [STS_REAL_NUMBER]
			-- Randomly-created monomorphic reference-equality for real numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_real_reference_equality}
		end

	new_real_object_equality: STS_OBJECT_EQUALITY [STS_REAL_NUMBER]
			-- Randomly-created monomorphic object-equality for real numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_real_object_equality}
		end

	some_real_numeric_equality: STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]
			-- Randomly-fetched polymorphic numeric equality for real numbers
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_real_numeric_equality
			when 1 then
				Result := some_real_proper_equality
			end
		end

	some_direct_real_numeric_equality: like some_real_numeric_equality
			-- Randomly-fetched monomorphic numeric equality for real numbers
		do
			check
				new_eq: attached {FUNCTION [like some_direct_real_numeric_equality]} agent new_real_numeric_equality as new_eq

					-- `some_direct_instance' definition
				eq: attached {like some_direct_real_numeric_equality} some_direct_instance (new_eq) as eq
			then
				Result := eq
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_real_numeric_equality}
		end

	new_real_numeric_equality: like some_real_numeric_equality
			-- Randomly-created monomorphic numeric equality for real numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_real_numeric_equality}
		end

	some_real_proper_equality: STS_REAL_EQUALITY
			-- Randomly-fetched polymorphic proper equality for real numbers
		do
			check
				new_eq: attached {FUNCTION [like some_real_proper_equality]} agent new_real_proper_equality as new_eq

					-- `some_direct_instance' definition
				eq: attached {like some_real_proper_equality} some_direct_instance (new_eq) as eq
			then
				Result := eq
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_real_proper_equality}
		end

	new_real_proper_equality: like some_real_proper_equality
			-- Randomly-created monomorphic proper equality for real numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_real_proper_equality}
		end

	some_rational_equality: STS_EQUALITY [STS_RATIONAL_NUMBER]
			-- Randomly-fetched polymorphic equality for rational numbers
		local
			new_eq: FUNCTION [like some_rational_equality]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				inspect
					next_random_item \\ 2
				when 0 then
					new_eq := agent new_rational_reference_equality
				when 1 then
					new_eq := agent new_rational_object_equality
				end
				check
					eq: attached {like some_rational_equality} some_direct_instance (new_eq) as eq -- `some_direct_instance' definition
				then
					Result := eq
				end
			when 1 then
				Result := some_rational_numeric_equality
			end
		end

	new_rational_reference_equality: STS_REFERENCE_EQUALITY [STS_RATIONAL_NUMBER]
			-- Randomly-created monomorphic reference-equality for rational numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_rational_reference_equality}
		end

	new_rational_object_equality: STS_OBJECT_EQUALITY [STS_RATIONAL_NUMBER]
			-- Randomly-created monomorphic object-equality for rational numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_rational_object_equality}
		end

	some_rational_numeric_equality: STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]
			-- Randomly-fetched polymorphic numeric equality for rational numbers
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_rational_numeric_equality
			when 1 then
				Result := some_rational_proper_equality
			end
		end

	some_direct_rational_numeric_equality: like some_rational_numeric_equality
			-- Randomly-fetched monomorphic numeric equality for rational numbers
		do
			check
				new_eq: attached {FUNCTION [like some_direct_rational_numeric_equality]} agent new_rational_numeric_equality as new_eq

					-- `some_direct_instance' definition
				eq: attached {like some_direct_rational_numeric_equality} some_direct_instance (new_eq) as eq
			then
				Result := eq
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_rational_numeric_equality}
		end

	new_rational_numeric_equality: like some_rational_numeric_equality
			-- Randomly-created monomorphic numeric equality for rational numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_rational_numeric_equality}
		end

	some_rational_proper_equality: --STS_RATIONAL_EQUALITY -- TODO: Change it!
		STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]
			-- Randomly-fetched polymorphic proper equality for rational numbers
		do
			check
				new_eq: attached {FUNCTION [like some_rational_proper_equality]} agent new_rational_proper_equality as new_eq

					-- `some_direct_instance' definition
				eq: attached {like some_rational_proper_equality} some_direct_instance (new_eq) as eq
			then
				Result := eq
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_rational_proper_equality}
		end

	new_rational_proper_equality: like some_rational_proper_equality
			-- Randomly-created monomorphic proper equality for rational numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_rational_proper_equality}
		end

	some_integer_equality: STS_EQUALITY [STS_INTEGER_NUMBER]
			-- Randomly-fetched polymorphic equality for integer numbers
		local
			new_eq: FUNCTION [like some_integer_equality]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				inspect
					next_random_item \\ 2
				when 0 then
					new_eq := agent new_integer_reference_equality
				when 1 then
					new_eq := agent new_integer_object_equality
				end
				check
					eq: attached {like some_integer_equality} some_direct_instance (new_eq) as eq -- `some_direct_instance' definition
				then
					Result := eq
				end
			when 1 then
				Result := some_integer_math_equality
			end
		end

	new_integer_reference_equality: STS_REFERENCE_EQUALITY [STS_INTEGER_NUMBER]
			-- Randomly-created monomorphic reference-equality for integer numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_integer_reference_equality}
		end

	new_integer_object_equality: STS_OBJECT_EQUALITY [STS_INTEGER_NUMBER]
			-- Randomly-created monomorphic object-equality for integer numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_integer_object_equality}
		end

	some_integer_math_equality: STS_INTEGER_EQUALITY
			-- Randomly-fetched polymorphic mathematical equality for integer numbers
		local
			new_eq: FUNCTION [like some_integer_math_equality]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				new_eq := agent new_integer_math_equality
			when 1 then
				new_eq := agent new_expanded_integer_equality
			end
			check
				eq: attached {like some_integer_math_equality} some_direct_instance (new_eq) as eq -- `some_direct_instance' definition
			then
				Result := eq
			end
		end

	new_integer_math_equality: STS_INTEGER_EQUALITY
			-- Randomly-created monomorphic mathematical equality for polymorphic integer numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_integer_math_equality}
		end

	new_expanded_integer_equality: STS_INTEGER_EQUALITY
			-- Randomly-created monomorphic mathematical equality for expanded integer numbers
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_expanded_integer_equality}
		end

feature -- Factory (ordered pair)

	some_pair: STS_ORDERED_PAIR [detachable separate ANY, detachable separate ANY, STS_EQUALITY [detachable separate ANY], STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 7
			when 0 then
				Result := some_pair_aa
			when 1 then
				Result := some_subset_family_structure_a
			when 2 then
				Result := some_expanded_complex
			when 3 then
				Result := some_exposed_complex
			when 4 then
				Result := some_expanded_rational
			when 5 then
				Result := some_exposed_complex
			when 6 then
				check
					new_a_ta: attached {FUNCTION [like new_pair_a_ta]} agent new_pair_a_ta as new_a_ta
					a_ta: attached {like new_pair_a_ta} some_direct_instance (new_a_ta) as a_ta
				then
					Result := a_ta
				end
			end
		end

feature -- Factory (set)

	some_set: STI_SET [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 5
			when 0 then
				Result := some_complexes
			when 1 then
				Result := some_reals
			when 2 then
				Result := some_rationals
			when 3 then
				Result := some_integers
			when 4 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	some_complexes: STI_SET [STS_COMPLEX_NUMBER, STS_EQUALITY [STS_COMPLEX_NUMBER]]
			-- Randomly-fetched polymorphic set of polymorphic complex numbers
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_complex_references
			when 1 then
				Result := some_complex_objects
			when 2 then
				Result := some_complex_numbers
			end
		end

	some_complex_references: STI_SET [STS_COMPLEX_NUMBER, STS_REFERENCE_EQUALITY [STS_COMPLEX_NUMBER]]
			-- Randomly-fetched set of polymorphic complex numbers compared by their object references
		do
			check
				new_s: attached {FUNCTION [like new_complex_references]} agent new_complex_references as new_s

					-- `some_direct_instance' and `new_complex_references' definition
				s: attached {like new_complex_references} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_complex_references}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_complex_references}
		end

	new_complex_references: like some_complex_references
			-- Randomly-created monomorphic set of polymorphic complex numbers compared by their object references
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_complex
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_complex_references}
		end

	some_complex_objects: STI_SET [STS_COMPLEX_NUMBER, STS_OBJECT_EQUALITY [STS_COMPLEX_NUMBER]]
			-- Randomly-fetched set of polymorphic complex numbers compared by their object values
		do
			check
				new_s: attached {FUNCTION [like new_complex_objects]} agent new_complex_objects as new_s

					-- `some_direct_instance' and `new_complex_objects' definition
				s: attached {like new_complex_objects} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_complex_objects}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_complex_objects}
		end

	new_complex_objects: like some_complex_objects
			-- Randomly-created monomorphic set of polymorphic complex numbers compared by their object values
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_complex
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_complex_objects}
		end

	some_complex_numbers: STI_SET [STS_COMPLEX_NUMBER, STS_NUMERIC_EQUALITY [STS_COMPLEX_NUMBER]]
			-- Randomly-fetched set of polymorphic complex numbers compared by numeric equality.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_s: attached {FUNCTION [like new_complex_numbers]} agent new_complex_numbers as new_s

					s: attached {like new_complex_numbers} some_direct_instance (new_s) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_strict_complex_numbers
			end
		end

	new_complex_numbers: like some_complex_numbers
			-- Randomly-created monomorphic set of polymorphic complex numbers compared by numeric equality.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_complex
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_complex_numbers}
		end

	some_strict_complex_numbers: STI_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
			-- Randomly-fetched set of polymorphic complex numbers compared by numeric equality as defined by {STS_COMPLEX_EQUALITY}.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_s: attached {FUNCTION [like new_strict_complex_numbers]} agent new_strict_complex_numbers as new_s
					s: attached {like new_strict_complex_numbers} some_direct_instance (NEW_S) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_complex_set
			end
		end

	new_strict_complex_numbers: like some_strict_complex_numbers
			-- Randomly-created monomorphic set of polymorphic complex numbers compared by numeric equality as defined by {STS_COMPLEX_EQUALITY}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_complex
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_strict_complex_numbers}
		end

	some_complex_set: STI_COMPLEX_SET
			-- Randomly-fetched polymorphic complex set
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_direct_complex_set
			when 1 then
				Result := some_quick_complex_set
			when 2 then
				Result := some_complex_universe
			end
		end

	some_direct_complex_set: like some_complex_set
			-- Randomly-fetched monomorphic complex set
		do
			check
				new_s: attached {FUNCTION [like new_complex_set]} agent new_complex_set as new_s
				s: attached {like new_complex_set} some_direct_instance (new_s) as s -- `some_direct_instance' and `new_complex_set' definitions
				monomorphic: s.generating_type ~ {detachable like new_complex_set}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_complex_set}
		end

	new_complex_set: like some_direct_complex_set
			-- Randomly-created monomorphic complex set
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_complex
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_complex_set}
		end

	some_quick_complex_set: STI_COMPLEX_SET
			-- Randomly-fetched polymorphic complex set whose assertions are turned off
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_quick_complex_set
			when 1 then
				Result := Quick_c
			end
		end

	some_direct_quick_complex_set: like some_quick_complex_set
			-- Randomly-fetched monomorphic complex set whose assertions are turned off
		do
			check
				new_s: attached {FUNCTION [like some_direct_quick_complex_set]} agent new_quick_complex_set as new_s
				s: attached {like some_direct_quick_complex_set} some_direct_instance (new_s) as s -- `some_direct_instance' and `new_complex_set' definitions
				monomorphic: s.generating_type ~ {detachable like new_quick_complex_set}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_quick_complex_set}
		end

	new_quick_complex_set: like some_direct_quick_complex_set
			-- Randomly-created monomorphic complex set whose assertions are turned off
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_complex
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_quick_complex_set}
		end

	some_complex_universe: STI_COMPLEX_UNIVERSE
			-- Randomly-fetched polymorphic universe of complex numbers, i.e. ℂ.
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := C
			when 1 then
				Result := Exposed_C
			when 2 then
				Result := Quick_c
			end
		end

	C,
	Complex_universe: STI_COMPLEX_UNIVERSE
			-- Universe of all complex numbers, i.e. ℂ.
		once
			Result := {STI_COMPLEX_SET}.C
		ensure
			class
			monomorphic: Result.generating_type ~ {detachable STI_COMPLEX_UNIVERSE}
		end

	Exposed_C,
	Exposed_complex_universe: COMPLEX_UNIVERSE
			-- Universe of all complex numbers, i.e. ℂ, whose features with restricted access are available to be tested.
		once
			create Result
		ensure
			class
			monomorphic: Result.generating_type ~ {detachable COMPLEX_UNIVERSE}
		end

	Quick_C,
	Quick_complex_universe: QUICK_COMPLEX_UNIVERSE
			-- Universe of all complex numbers, i.e. ℂ, meant to run quickly by turning its assertions off.
		once
			create Result
		ensure
			class
			monomorphic: Result.generating_type ~ {detachable QUICK_COMPLEX_UNIVERSE}
		end

	some_reals: STI_SET [STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER]]
			-- Randomly-fetched polymorphic set of polymorphic real numbers
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_real_references
			when 1 then
				Result := some_real_objects
			when 2 then
				Result := some_real_numbers
			end
		end

	some_real_references: STI_SET [STS_REAL_NUMBER, STS_REFERENCE_EQUALITY [STS_REAL_NUMBER]]
			-- Randomly-fetched set of polymorphic real numbers compared by their object references
		do
			check
				new_s: attached {FUNCTION [like new_real_references]} agent new_real_references as new_s

					-- `some_direct_instance' and `new_real_references' definition
				s: attached {like new_real_references} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_real_references}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_real_references}
		end

	new_real_references: like some_real_references
			-- Randomly-created monomorphic set of polymorphic real numbers compared by their object references
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_real
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_real_references}
		end

	some_real_objects: STI_SET [STS_REAL_NUMBER, STS_OBJECT_EQUALITY [STS_REAL_NUMBER]]
			-- Randomly-fetched set of polymorphic real numbers compared by their object values
		do
			check
				new_s: attached {FUNCTION [like new_real_objects]} agent new_real_objects as new_s

					-- `some_direct_instance' and `new_real_objects' definition
				s: attached {like new_real_objects} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_real_objects}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_real_objects}
		end

	new_real_objects: like some_real_objects
			-- Randomly-created monomorphic set of polymorphic real numbers compared by their object values
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_real
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_real_objects}
		end

	some_real_numbers: STI_SET [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]
			-- Randomly-fetched set of polymorphic real numbers compared by numeric equality.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_s: attached {FUNCTION [like new_real_numbers]} agent new_real_numbers as new_s
					s: attached {like new_real_numbers} some_direct_instance (new_s) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_strict_real_numbers
			end
		end

	new_real_numbers: like some_real_numbers
			-- Randomly-created monomorphic set of polymorphic real numbers compared by numeric equality.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_real
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_real_numbers}
		end

	some_strict_real_numbers: STI_SET [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Randomly-fetched set of polymorphic real numbers compared by numeric equality as defined by {STS_REAL_EQUALITY}.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_s: attached {FUNCTION [like new_strict_real_numbers]} agent new_strict_real_numbers as new_s
					s: attached {like new_strict_real_numbers} some_direct_instance (new_s) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_real_set
			end
		end

	new_strict_real_numbers: like some_strict_real_numbers
			-- Randomly-created monomorphic set of polymorphic real numbers compared by numeric equality as defined by {STS_REAL_EQUALITY}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_real
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_strict_real_numbers}
		end

	some_real_set: STI_REAL_SET
			-- Randomly-fetched polymorphic real set
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_real_set
			when 1 then
				Result := some_real_universe
			end
		end

	some_direct_real_set: like some_real_set
			-- Randomly-fetched monomorphic real set
		do
			check
				new_s: attached {FUNCTION [like new_real_set]} agent new_real_set as new_s

					-- `some_direct_instance' and `new_real_set' definitions
				s: attached {like new_real_set} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_real_set}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_real_set}
		end

	new_real_set: like some_direct_real_set
			-- Randomly-created monomorphic real set
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_real
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_real_set}
		end

	some_real_universe: STI_REAL_UNIVERSE
			-- Randomly-fetched polymorphic universe of real numbers, i.e. ℝ.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := R
			when 1 then
				Result := Exposed_R
			end
		end

	R,
	Real_universe: STI_REAL_UNIVERSE
			-- Universe of all real numbers, i.e. ℝ.
		once
			Result := {STI_REAL_SET}.R
		ensure
			class
			monomorphic: Result.generating_type ~ {detachable STI_REAL_UNIVERSE}
		end

	Exposed_R,
	Exposed_real_universe: REAL_UNIVERSE
			-- Universe of all real numbers, i.e. ℝ, whose features with restricted access are available to be tested.
		once
			create Result
		ensure
			class
			monomorphic: Result.generating_type ~ {detachable REAL_UNIVERSE}
		end

	some_rationals: STI_SET [STS_RATIONAL_NUMBER, STS_EQUALITY [STS_RATIONAL_NUMBER]]
			-- Randomly-fetched polymorphic set of polymorphic rational numbers
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_rational_references
			when 1 then
				Result := some_rational_objects
			when 2 then
				Result := some_rational_numbers
			end
		end

	some_rational_references: STI_SET [STS_RATIONAL_NUMBER, STS_REFERENCE_EQUALITY [STS_RATIONAL_NUMBER]]
			-- Randomly-fetched set of polymorphic rational numbers compared by their object references
		do
			check
				new_s: attached {FUNCTION [like new_rational_references]} agent new_rational_references as new_s

					-- `some_direct_instance' and `new_rational_references' definition
				s: attached {like new_rational_references} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_rational_references}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_rational_references}
		end

	new_rational_references: like some_rational_references
			-- Randomly-created monomorphic set of polymorphic rational numbers compared by their object references
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_rational
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_rational_references}
		end

	some_rational_objects: STI_SET [STS_RATIONAL_NUMBER, STS_OBJECT_EQUALITY [STS_RATIONAL_NUMBER]]
			-- Randomly-fetched set of polymorphic rational numbers compared by their object values
		do
			check
				new_s: attached {FUNCTION [like new_rational_objects]} agent new_rational_objects as new_s

					-- `some_direct_instance' and `new_rational_objects' definition
				s: attached {like new_rational_objects} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_rational_objects}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_rational_objects}
		end

	new_rational_objects: like some_rational_objects
			-- Randomly-created monomorphic set of polymorphic rational numbers compared by their object values
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_rational
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_rational_objects}
		end

	some_rational_numbers: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]
			-- Randomly-fetched set of polymorphic rational numbers compared by numeric equality.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_s: attached {FUNCTION [like new_rational_numbers]} agent new_rational_numbers as new_s
					s: attached {like new_rational_numbers} some_direct_instance (new_s) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_strict_rational_numbers
			end
		end

	new_rational_numbers: like some_rational_numbers
			-- Randomly-created monomorphic set of polymorphic rational numbers compared by numeric equality.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_rational
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_rational_numbers}
		end

	some_strict_rational_numbers: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]
--	some_strict_rational_numbers: STI_SET [STS_RATIONAL_NUMBER, STS_RATIONAL_EQUALITY]
			-- Randomly-fetched set of polymorphic rational numbers compared by numeric equality as defined by {STS_RATIONAL_EQUALITY}.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_s: attached {FUNCTION [like new_strict_rational_numbers]} agent new_strict_rational_numbers as new_s
					s: attached {like new_strict_rational_numbers} some_direct_instance (new_s) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_rational_set
			end
		end

	new_strict_rational_numbers: like some_strict_rational_numbers
			-- Randomly-created monomorphic set of polymorphic rational numbers compared by numeric equality as defined by {STS_RATIONAL_EQUALITY}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_rational
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_strict_rational_numbers}
		end

	some_rational_set: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]
--	some_rational_set: STI_RATIONAL_SET
			-- Randomly-fetched polymorphic rational set
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_rational_set
			when 1 then
--				Result := some_rational_universe -- TODO: Change it.
				Result := some_direct_rational_set
			end
		end

	some_direct_rational_set: like some_rational_set
			-- Randomly-fetched monomorphic rational set
		do
			check
				new_s: attached {FUNCTION [like new_rational_set]} agent new_rational_set as new_s

					-- `some_direct_instance' and `new_rational_set' definitions
				s: attached {like new_rational_set} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_rational_set}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_rational_set}
		end

	new_rational_set: like some_direct_rational_set
			-- Randomly-created monomorphic rational set
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_rational
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_rational_set}
		end

	some_rational_universe: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]
--	some_rational_universe: STI_RATIONAL_UNIVERSE
			-- Randomly-fetched polymorphic universe of rational numbers, i.e. ℚ.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := Q
			when 1 then
				Result := Exposed_Q
			end
		end

	Q,
	Rational_universe: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]
--	Rational_universe: STI_RATIONAL_UNIVERSE
			-- Universe of all rational numbers, i.e. ℚ.
--		once
do
--			Result := {STI_RATIONAL_SET}.R
			Result := some_rational_set
		ensure
--			class
			monomorphic: Result.generating_type ~ {detachable STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]}
--			monomorphic: Result.generating_type ~ {detachable STI_RATIONAL_UNIVERSE}
		end

	Exposed_Q,
--	Exposed_rational_universe: RATIONAL_UNIVERSE
	Exposed_rational_universe: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]
			-- Universe of all rational numbers, i.e. ℚ, whose features with restricted access are available to be tested.
		once
--			create Result
			Result := some_rational_set
		ensure
--			class
--			monomorphic: Result.generating_type ~ {detachable RATIONAL_UNIVERSE}
			monomorphic: Result.generating_type ~ {detachable STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]}
		end

	some_integers: STI_SET [STS_INTEGER_NUMBER, STS_EQUALITY [STS_INTEGER_NUMBER]]
			-- Randomly-fetched polymorphic set of polymorphic integer numbers
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_integer_references
			when 1 then
				Result := some_integer_objects
			when 2 then
				Result := some_integer_numbers
			end
		end

	some_integer_references: STI_SET [STS_INTEGER_NUMBER, STS_REFERENCE_EQUALITY [STS_INTEGER_NUMBER]]
			-- Randomly-fetched set of polymorphic integer numbers compared by their object references
		do
			check
				new_s: attached {FUNCTION [like new_integer_references]} agent new_integer_references as new_s

					-- `some_direct_instance' and `new_integer_references' definition
				s: attached {like new_integer_references} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_integer_references}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_integer_references}
		end

	new_integer_references: like some_integer_references
			-- Randomly-created monomorphic set of polymorphic integer numbers compared by their object references
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_integer
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_integer_references}
		end

	some_integer_objects: STI_SET [STS_INTEGER_NUMBER, STS_OBJECT_EQUALITY [STS_INTEGER_NUMBER]]
			-- Randomly-fetched set of polymorphic integer numbers compared by their object values
		do
			check
				new_s: attached {FUNCTION [like new_integer_objects]} agent new_integer_objects as new_s

					-- `some_direct_instance' and `new_integer_objects' definition
				s: attached {like new_integer_objects} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_integer_objects}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_integer_objects}
		end

	new_integer_objects: like some_integer_objects
			-- Randomly-created monomorphic set of polymorphic integer numbers compared by their object values
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_integer
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_integer_objects}
		end

	some_integer_numbers: STI_SET [STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]]
			-- Randomly-fetched set of polymorphic integer numbers compared by numeric equality.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_s: attached {FUNCTION [like new_integer_numbers]} agent new_integer_numbers as new_s

					s: attached {like new_integer_numbers} some_direct_instance (new_s) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_strict_integer_numbers
			end
		end

	new_integer_numbers: like some_integer_numbers
			-- Randomly-created monomorphic set of polymorphic integer numbers compared by numeric equality.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_integer
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_integer_numbers}
		end

	some_strict_integer_numbers: STI_SET [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- Randomly-fetched set of polymorphic integer numbers compared by numeric equality as defined by {STS_INTEGER_EQUALITY}.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_s: attached {FUNCTION [like new_strict_integer_numbers]} agent new_strict_integer_numbers as new_s
					s: attached {like new_strict_integer_numbers} some_direct_instance (new_s) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_integer_set
			end
		end

	new_strict_integer_numbers: like some_strict_integer_numbers
			-- Randomly-created monomorphic set of polymorphic integer numbers compared by numeric equality as defined by {STS_INTEGER_EQUALITY}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_integer
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_strict_integer_numbers}
		end

	some_integer_set: STI_INTEGER_SET
			-- Randomly-fetched polymorphic integer set
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_integer_set
			when 1 then
				Result := some_integer_universe
			end
		end

	some_direct_integer_set: like some_integer_set
			-- Randomly-fetched monomorphic integer set
		do
			check
				new_s: attached {FUNCTION [like new_integer_set]} agent new_integer_set as new_s

					-- `some_direct_instance' and `new_integer_set' definitions
				s: attached {like new_integer_set} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_integer_set}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_integer_set}
		end

	new_integer_set: like some_direct_integer_set
			-- Randomly-created monomorphic integer set
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_integer
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_integer_set}
		end

	some_integer_universe: STI_INTEGER_UNIVERSE
			-- Randomly-fetched polymorphic universe of integer numbers, i.e. ℚ.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := Z
			when 1 then
				Result := Exposed_Z
			end
		end

	Z,
	Integer_universe: STI_INTEGER_UNIVERSE
			-- Universe of all integer numbers, i.e. ℤ.
		once
			Result := {STI_INTEGER_SET}.Z
		ensure
			class
			monomorphic: Result.generating_type ~ {detachable STI_INTEGER_UNIVERSE}
		end

	Exposed_Z,
	Exposed_integer_universe: INTEGER_UNIVERSE
			-- Universe of all integer numbers, i.e. ℤ, whose features with restricted access are available to be tested.
		once
			create Result
		ensure
			class
			monomorphic: Result.generating_type ~ {detachable INTEGER_UNIVERSE}
		end

feature -- Factory (set)

	some_n_tuple: STI_N_TUPLE [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_t: attached {FUNCTION [like new_quick_complex_tuple]} agent new_quick_complex_tuple as new_t
					t: attached {like new_quick_complex_tuple} some_direct_instance (new_t) as t -- `some_direct_instance' definitions
				then
					Result := trimmed_n_tuple (t)
				end
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	new_quick_complex_tuple: QUICK_COMPLEX_TUPLE
			-- Randomly-created monomorphic complex n-tuple whose assertions are turned off
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_complex)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable QUICK_COMPLEX_TUPLE}
		end

invariant
	two: two.value = 2
	nan: nan.is_nan
	negative_infinity: negative_infinity.is_negative_infinity
	positive_infinity: positive_infinity.is_positive_infinity
	monomorphic_C: C.generating_type ~ {detachable STI_COMPLEX_UNIVERSE}
	monomorphic_Complex_universe: Complex_universe.generating_type ~ {detachable STI_COMPLEX_UNIVERSE}
	monomorphic_Exposed_c: Exposed_c.generating_type ~ {detachable COMPLEX_UNIVERSE}
	monomorphic_Exposed_complex_universe: Exposed_complex_universe.generating_type ~ {detachable COMPLEX_UNIVERSE}
	monomorphic_Quick_c: Quick_c.generating_type ~ {detachable QUICK_COMPLEX_UNIVERSE}
	monomorphic_Quick_complex_universe: Quick_complex_universe.generating_type ~ {detachable QUICK_COMPLEX_UNIVERSE}
	monomorphic_R: R.generating_type ~ {detachable STI_REAL_UNIVERSE}
	monomorphic_Real_universe: Real_universe.generating_type ~ {detachable STI_REAL_UNIVERSE}
	monomorphic_Exposed_r: Exposed_r.generating_type ~ {detachable REAL_UNIVERSE}
	monomorphic_Exposed_real_universe: Exposed_real_universe.generating_type ~ {detachable REAL_UNIVERSE}
	monomorphic_Z: Z.generating_type ~ {detachable STI_INTEGER_UNIVERSE}
	monomorphic_Integer_universe: Integer_universe.generating_type ~ {detachable STI_INTEGER_UNIVERSE}
	monomorphic_Exposed_z: Exposed_z.generating_type ~ {detachable INTEGER_UNIVERSE}
	monomorphic_Exposed_integer_universe: Exposed_integer_universe.generating_type ~ {detachable INTEGER_UNIVERSE}

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
