﻿note
	description: "[
		{NUMBERS_TESTS_RESOURCES}, derived for {STS_COMPLEX_NUMBER} generic parameter, compared by numeric value.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

class
	NUMBERS_TESTS_RESOURCES_C

inherit
	NUMBERS_TESTS_RESOURCES [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
		rename
			some_object_a as some_complex
		redefine
			on_prepare,
			some_set_a,
			same_set_a,
			same_n_tuple_a
		end

feature -- Access

	zero: STI_COMPLEX_NUMBER
			-- The complex number 0 + 0i
--		once
--		ensure then
--			class
--			real: Result.a.value = 0
--			imaginary: Result.b.value = 0
--		end

--	One: STI_COMPLEX_NUMBER
--			-- The complex number 1 + 0i
--		once
--			Result := {STI_COMPLEX_NUMBER}.One
--		ensure then
--			class
--			real: Result.a.value = 1
--			imaginary: Result.b.value = 0
--		end

	i,
	imaginary_unit: STI_COMPLEX_NUMBER
			-- The imaginary unit, i.e. the complex number 0 + 1i.
		once
			Result := {STI_COMPLEX_NUMBER}.I
		ensure then
			class
			real: Result.a.value = 0
			imaginary: Result.b.value = 1
		end

feature {NONE} -- Events

	on_prepare
			-- <Precursor>
		do
			create eq_a
		end

feature -- Factory (set)

	some_set_a: STI_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]
			-- Randomly fetched polymorphic set of complex numbers, compared by numeric equality.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_strict_complex_numbers
			else
				Result := Precursor {NUMBERS_TESTS_RESOURCES}
			end
		end

	same_set_a (s: STI_SET [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]): like some_set_a
			-- <Precursor>
		local
			chk_n, chk_cpy: BOOLEAN
			n: NATURAL
		do
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
				print (n) print ('%N')
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			if n > Max_asserted_elements then
				chk_cpy := {ISE_RUNTIME}.check_assert (False)
			end
				if (attached {STI_COMPLEX_UNIVERSE} s as l_c or n = # C)  and next_random_item \\ 2 = 0 then
					inspect
						next_random_item \\ 3
					when 0 then
						create {like C} Result
					when 1 then
						create {like Exposed_c} Result
					when 2 then
						create {like Quick_c} Result
					end
				elseif next_random_item \\ 2 = 0 then
					inspect
						next_random_item \\ 2
					when 0 then
						create {like some_complex_set} Result.make_empty
					when 1 then
						create {like some_quick_complex_set} Result.make_empty
					end
						check
							is_disjoint: Result.is_disjoint (s) -- Result.is_empty
						end
					Result := Result.batch_extended (s)
				else
					Result := Precursor {NUMBERS_TESTS_RESOURCES}(s)
				end
			if chk_cpy then
				chk_cpy := {ISE_RUNTIME}.check_assert (True)
			end
		end

	same_complex_numbers (s: STI_SET [STS_COMPLEX_NUMBER, STS_NUMERIC_EQUALITY [STS_COMPLEX_NUMBER]]): like some_complex_numbers
			-- Set mathematically equal to `s`, with type randomly chosen.
		local
			chk_n, chk_cpy: BOOLEAN
			n: NATURAL
		do
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			if n > Max_asserted_elements then
				chk_cpy := {ISE_RUNTIME}.check_assert (False)
			end
				if attached {like some_set_a} s as sa then
					inspect
						next_random_item \\ 2
					when 0 then
						Result := same_set_a (sa)
					when 1 then
						create Result.make_empty
							check
								is_disjoint: Result.is_disjoint (s) -- Result.is_empty
							end
						Result := Result.batch_extended (s)
					end
				else
					inspect
						next_random_item \\ 3
					when 0 then
						Result := s
					when 1 then
						Result := s.twin
					when 2 then
						Result := same_set_a (create {like some_complex_set}.make_from_numeric_set (s))
					end
				end
			if chk_cpy then
				chk_cpy := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			definition: Result ≍ s
		end

	same_n_tuple_a (t: STI_N_TUPLE [STS_COMPLEX_NUMBER, STS_COMPLEX_EQUALITY]):
		like some_n_tuple_a
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				create {like new_quick_complex_tuple} Result.make_unit
				Result := Result + t
			when 1 then
				Result := Precursor {NUMBERS_TESTS_RESOURCES}(t)
			end
		end

feature -- Predicate

	p_x (x: STS_COMPLEX_NUMBER): BOOLEAN
			-- <Precursor>
		do
			Result := (x.a + x.b).value.truncated_to_integer \\ 2 = 0
		ensure then
			class
		end

	p_x1_x2 (x1, x2: STS_COMPLEX_NUMBER): BOOLEAN
			-- <Precursor>
		do
			Result := (x1.a + x1.b).value.truncated_to_integer \\ 2 /= ((x1.a + x1.b).value.truncated_to_integer + (x2.a + x2.b).value.truncated_to_integer) \\ 2
		ensure then
			class
		end

feature -- Mapping

	f_x (x: STS_COMPLEX_NUMBER): STS_COMPLEX_NUMBER
			-- <Precursor>
		do
			Result := {NUMBERS_TESTS_RESOURCES_R}.f_x (x.a) + {NUMBERS_TESTS_RESOURCES_R}.f_x (x.b) * i
--			x + (create {STI_COMPLEX_NUMBER}.make ({NUMBERS_TESTS_RESOURCES_R}.f_x (x.a), {NUMBERS_TESTS_RESOURCES_R}.f_x (x.b)))
		ensure then
			class
		end

feature -- Reduction

	f_acc_x (acc, x: STS_COMPLEX_NUMBER): STS_COMPLEX_NUMBER
			-- <Precursor>
		do
			Result := acc - x
		ensure then
			class
		end

	f_acc_i_x (acc: STS_COMPLEX_NUMBER; a_i: NATURAL; x: STS_COMPLEX_NUMBER): STS_COMPLEX_NUMBER
			-- Function that reduces (`acc', `a_i', `x') to a value like `acc'
		do
			Result := f_acc_x (f_acc_x (acc, create {STI_COMPLEX_NUMBER}.make_from_native_natural (a_i)), x)
		ensure then
			class
		end

feature {SET_PROPERTIES, NUMBERS_TESTS_RESOURCES} -- Implementation

	max_asserted_elements: NATURAL = 340
			-- <Precursor>

invariant
	Zero_real: Zero.a.value = 0
	Zero_imaginary: Zero.b.value = 0
	i_real: i.a.value = 0
	i_imaginary: i.b.value = 1
	Imaginary_unit_real: Imaginary_unit.a.value = 0
	Imaginary_unit_imaginary: Imaginary_unit.b.value = 1

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
