﻿note
	description: "[
		{NUMBERS_TESTS_RESOURCES}, derived for {STS_RATIONAL_NUMBER} generic
		parameter, compared by numeric value.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

class
	NUMBERS_TESTS_RESOURCES_Q

inherit
	NUMBERS_TESTS_RESOURCES [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]] -- TODO: Change equality!
		rename
			some_object_a as some_rational
		redefine
			on_prepare,
			some_set_a,
			same_set_a
		end

feature -- Access

	zero: STI_RATIONAL_NUMBER
			-- The rational number 0/1
		once
			Result := {STI_RATIONAL_NUMBER}.Zero
		ensure
			class
			numerator: Result.p.value = 0
			denominator: Result.q.value = 1
		end

feature {NONE} -- Events

	on_prepare
			-- <Precursor>
		do
			create eq_a
		end

feature -- Factory (set)

	some_set_a: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]
			-- Randomly fetched polymorphic set of rational numbers, compared by numeric equality.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_strict_rational_numbers
			else
				Result := Precursor {NUMBERS_TESTS_RESOURCES}
			end
		end

	same_set_a (s: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]): like some_set_a
			-- <Precursor>
		local
			chk_n, chk_cpy: BOOLEAN
			n: NATURAL
		do
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
				print (n) print ('%N')
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			if n > Max_asserted_elements then
				chk_cpy := {ISE_RUNTIME}.check_assert (False)
			end
--				if (attached {STI_RATIONAL_UNIVERSE} s or n = # Q) and next_random_item \\ 2 = 0 then -- TODO: Include universe!
--					inspect
--						next_random_item \\ 2
--					when 0 then
--						create {like Q} Result
--					when 1 then
--						create {like Exposed_q} Result
--					end
--				else
					Result := Precursor {NUMBERS_TESTS_RESOURCES}(s)
--				end
			if chk_cpy then
				chk_cpy := {ISE_RUNTIME}.check_assert (True)
			end
		end

	same_rational_numbers (s: STI_SET [STS_RATIONAL_NUMBER, STS_NUMERIC_EQUALITY [STS_RATIONAL_NUMBER]]): like some_rational_numbers
			-- Set mathematically equal to `s`, with type randomly chosen.
		local
			chk_n, chk_cpy: BOOLEAN
			n: NATURAL
		do
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			if n > Max_asserted_elements then
				chk_cpy := {ISE_RUNTIME}.check_assert (False)
			end
				if attached {like some_set_a} s as sa then
					inspect
						next_random_item \\ 2
					when 0 then
						Result := same_set_a (sa)
					when 1 then
						create Result.make_empty
							check
								is_disjoint: Result.is_disjoint (s) -- Result.is_empty
							end
						Result := Result.batch_extended (s)
					end
				else
					inspect
						next_random_item \\ 3
					when 0 then
						Result := s
					when 1 then
						Result := s.twin
					when 2 then
--						Result := same_set_a (create {like some_rational_set}.make_from_numeric_set (s))
						Result := same_set_a (create {like some_rational_set}.make_from_set (s))
					end
				end
			if chk_cpy then
				chk_cpy := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			definition: Result ≍ s
		end

feature -- Predicate

	p_x (x: STS_RATIONAL_NUMBER): BOOLEAN
			-- <Precursor>
		do
			Result := x.value.truncated_to_integer \\ 2 = 0
		ensure then
			class
		end

	p_x1_x2 (x1, x2: STS_RATIONAL_NUMBER): BOOLEAN
			-- <Precursor>
		do
			Result := x1.value.truncated_to_integer \\ 2 /= (x1.value.truncated_to_integer + x2.value.truncated_to_integer) \\ 2
		ensure then
			class
		end

feature -- Mapping

	f_x (x: STS_RATIONAL_NUMBER): STS_RATIONAL_NUMBER
			-- <Precursor>
		local
			new_q: STS_INTEGER_NUMBER
		do
			new_q := {NUMBERS_TESTS_RESOURCES_I}.f_x (x.q)
			if new_q ≭ Zero then
				Result := {NUMBERS_TESTS_RESOURCES_I}.f_x (x.p) / new_q
			else
					check
						good_divisor: {NUMBERS_TESTS_RESOURCES_I}.f_x (x.p).divisible (x.q) -- x.q /= 0 (class invariant)
					end
				Result := {NUMBERS_TESTS_RESOURCES_I}.f_x (x.p) / x.q
			end
		ensure then
			class
		end

feature -- Reduction

	f_acc_x (acc, x: STS_RATIONAL_NUMBER): STS_RATIONAL_NUMBER
			-- <Precursor>
		do
			Result := acc - x
		ensure then
			class
		end

	f_acc_i_x (acc: STS_RATIONAL_NUMBER; i: NATURAL; x: STS_RATIONAL_NUMBER): STS_RATIONAL_NUMBER
			-- <Precursor>
		do
			Result := f_acc_x (f_acc_x (acc, create {STI_RATIONAL_NUMBER}.make_from_native_natural (i)), x)
--			Result := f_acc_x (f_acc_x (acc, create {STI_RATIONAL_NUMBER}), x)
		ensure then
			class
		end

feature {SET_PROPERTIES} -- Implementation

	max_asserted_elements: NATURAL
			-- <Precursor>
		once
			Result := {NUMBERS_TESTS_RESOURCES_C}.max_asserted_elements
		ensure then
			class
		end

invariant
	zero_numerator: Zero.p.value = 0
	zero_denominator: Zero.q.value = 1

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
