﻿note
	description: "[
		{NUMBERS_TESTS_RESOURCES}, derived for {STS_REAL_NUMBER} generic parameter, compared by numeric value.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

class
	NUMBERS_TESTS_RESOURCES_R

inherit
	NUMBERS_TESTS_RESOURCES [STS_REAL_NUMBER, STS_REAL_EQUALITY]
		rename
			some_object_a as some_real
		redefine
			on_prepare,
			some_pair_aa,
			some_set_a,
			same_set_a
		end

feature -- Access

	zero: STI_REAL_NUMBER
			-- The real number 0
		once
		ensure
			class
			definition: Result.value = 0
		end

feature {NONE} -- Events

	on_prepare
			-- <Precursor>
		do
			create eq_a
		end

feature -- Factory (ordered pair)

	some_pair_aa: STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]
			-- Randomly fetched polymorphic ordered pair of real numbers
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_expanded_complex
			when 1 then
				Result := some_exposed_complex
			when 2 then
				Result := Precursor {NUMBERS_TESTS_RESOURCES}
			end
		end

feature -- Factory (set)

	some_set_a: STI_SET [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- Randomly fetched polymorphic set of real numbers, compared by numeric equality.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_strict_real_numbers
			else
				Result := Precursor {NUMBERS_TESTS_RESOURCES}
			end
		end

	same_set_a (s: STI_SET [STS_REAL_NUMBER, STS_REAL_EQUALITY]): like some_set_a
			-- <Precursor>
		local
			chk_n, chk_cpy: BOOLEAN
			n: NATURAL
		do
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
				print (n) print ('%N')
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			if n > Max_asserted_elements then
				chk_cpy := {ISE_RUNTIME}.check_assert (False)
			end
				if (attached {STI_REAL_UNIVERSE} s or n = # R) and next_random_item \\ 2 = 0 then
					inspect
						next_random_item \\ 2
					when 0 then
						create {like R} Result
					when 1 then
						create {like Exposed_r} Result
					end
				else
					Result := Precursor {NUMBERS_TESTS_RESOURCES}(s)
				end
			if chk_cpy then
				chk_cpy := {ISE_RUNTIME}.check_assert (True)
			end
		end

	same_real_numbers (s: STI_SET [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]): like some_real_numbers
			-- Set mathematically equal to `s`, with type randomly chosen.
		local
			chk_n, chk_cpy: BOOLEAN
			n: NATURAL
		do
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			if n > Max_asserted_elements then
				chk_cpy := {ISE_RUNTIME}.check_assert (False)
			end
				if attached {like some_set_a} s as sa then
					inspect
						next_random_item \\ 2
					when 0 then
						Result := same_set_a (sa)
					when 1 then
						create Result.make_empty
							check
								is_disjoint: Result.is_disjoint (s) -- Result.is_empty
							end
						Result := Result.batch_extended (s)
					end
				else
					inspect
						next_random_item \\ 3
					when 0 then
						Result := s
					when 1 then
						Result := s.twin
					when 2 then
						Result := same_set_a (create {like some_real_set}.make_from_numeric_set (s))
					end
				end
			if chk_cpy then
				chk_cpy := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			definition: Result ≍ s
		end

feature -- Predicate

	p_x (x: STS_REAL_NUMBER): BOOLEAN
			-- <Precursor>
		do
			Result := x.value.truncated_to_integer \\ 2 = 0
		ensure then
			class
		end

	p_x1_x2 (x1, x2: STS_REAL_NUMBER): BOOLEAN
			-- <Precursor>
		do
			Result := x1.value.truncated_to_integer \\ 2 /= (x1.value.truncated_to_integer + x2.value.truncated_to_integer) \\ 2
		ensure then
			class
		end

feature -- Mapping

	f_x (x: STS_REAL_NUMBER): STS_REAL_NUMBER
			-- <Precursor>
		do
			Result := x + (create {STI_REAL_NUMBER}.make (2))
		ensure then
			class
		end

feature -- Reduction

	f_acc_x (acc, x: STS_REAL_NUMBER): STS_REAL_NUMBER
			-- <Precursor>
		do
			Result := acc - x
		ensure then
			class
		end

	f_acc_i_x (acc: STS_REAL_NUMBER; i: NATURAL; x: STS_REAL_NUMBER): STS_REAL_NUMBER
			-- <Precursor>
		do
			Result := f_acc_x (f_acc_x (acc, create {STI_REAL_NUMBER}.make_from_native_natural (i)), x)
		ensure then
			class
		end

feature {SET_PROPERTIES} -- Implementation

	max_asserted_elements: NATURAL
			-- <Precursor>
		once
			Result := {NUMBERS_TESTS_RESOURCES_C}.max_asserted_elements
		ensure then
			class
		end

invariant
	Zero: Zero.value = 0

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
