note
	description: "Set meant to be a storage for objects of all kinds. Since such storage gets very slow as it grows this class is meant to have its assertions turned off."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	OBJECT_POOL

inherit
	STI_SET [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]
		redefine
			make_singleton,
			make_extended,
			eq,
			o, empty_set,
			unit,
			element_to_boolean,
			element_to_element,
			element_to_natural,
			element_to_sets,
			element_to_tuple,
			tuple_extensor,
			set_anchor,
			subset_anchor,
			superset_anchor,
			n_tuple_anchor
		end

create
	make_empty,
	make_singleton,
	make_extended,
	make_from_set

convert
	make_from_set ({
		STS_SET [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]],
		STI_SET [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]],
		STS_TRANSFORMABLE_SET [detachable separate ANY, detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_REFERENCE_EQUALITY [detachable separate ANY]],
		STI_TRANSFORMABLE_SET [detachable separate ANY, detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_REFERENCE_EQUALITY [detachable separate ANY]]
		})

feature {NONE} -- Initialization

	make_extended (a: detachable separate ANY; s: STS_SET [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]])
			-- <Precursor>
		do
			others := O
			others := others.converted (s)
			create any_storage.put (converted_element (a))
		end

	make_singleton (a: detachable separate ANY)
			-- <Precursor>
		do
			others := O
			create any_storage.put (converted_element (a))
		end

feature -- Primitive

	Eq: OBJECT_POOL_EQUALITY
			-- <Precursor>
		once
			create Result
		end

feature -- Factory

	O,
	Empty_set: OBJECT_POOL
			-- <Precursor>
		once
			create Result.make_empty
		ensure then
			class
		end

	Unit: OBJECT_POOL_N_TUPLE
			-- <Precursor>
		once
			Result := {OBJECT_POOL_N_TUPLE}.Unit
		end

feature -- Transformer

	Element_to_boolean: STI_TRANSFORMER [detachable separate ANY, BOOLEAN, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_OBJECT_EQUALITY [BOOLEAN]]
			-- Transformer between types derived from {detachable separate ANY} and {BOOLEAN}
		once
			create Result
		end

	Element_to_element: STI_TRANSFORMER [detachable separate ANY, detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_REFERENCE_EQUALITY [detachable separate ANY]]
			-- Transformer between types derived from {detachable separate ANY}
		once
			create Result
		end

	Element_to_natural: STI_TRANSFORMER [detachable separate ANY, NATURAL, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_OBJECT_EQUALITY [NATURAL]]
			-- Transformer between types derived from {detachable separate ANY} and {NATURAL}
		once
			create Result
		end

	Element_to_sets: STI_TRANSFORMER [detachable separate ANY, STS_SET [STS_SET [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]], STS_SET_EQUALITY [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]], STS_REFERENCE_EQUALITY [detachable separate ANY], STS_SET_EQUALITY [STS_SET [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]], STS_SET_EQUALITY [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]]]
			-- Transformer between types derived from {detachable separate ANY} and {
			--	STS_SET [
			--		STS_SET [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]],
			--		STS_SET_EQUALITY [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]
			--		]
			--	}

		once
			create Result
		end

	Element_to_tuple: STI_TRANSFORMER [detachable separate ANY, STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]], STS_REFERENCE_EQUALITY [detachable separate ANY], STS_N_TUPLE_EQUALITY [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]]
			-- Transformer between types derived from {detachable separate ANY} and {STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]}
		once
			create Result
		end

	Tuple_extensor: STI_TRANSFORMER [STS_ORDERED_PAIR [detachable separate ANY, STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]], STS_REFERENCE_EQUALITY [detachable separate ANY], STS_N_TUPLE_EQUALITY [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]], STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]], STS_ORDERED_PAIR_EQUALITY [detachable separate ANY, STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]], STS_REFERENCE_EQUALITY [detachable separate ANY], STS_N_TUPLE_EQUALITY [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]], STS_N_TUPLE_EQUALITY [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]]
			-- Transformer between types derived from {
			--	STS_ORDERED_PAIR [
			--		detachable separate ANY,
			--		STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]],
			--		STS_REFERENCE_EQUALITY [detachable separate ANY],
			--		STS_N_TUPLE_EQUALITY [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]
			--		]
			--	} and {STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]}
		once
			create Result
		end

feature -- Anchor

	set_anchor,
	subset_anchor,
	superset_anchor: OBJECT_POOL
			-- <Precursor>
		do
			Result := Current
		end

	n_tuple_anchor: OBJECT_POOL_N_TUPLE
			-- <Precursor>
		do
			Result := Unit
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
