﻿note
	description: "N-tuple meant to be a storage for objects of all kinds. Since such storage gets very slow as it grows this class is meant to have its assertions turned off."
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	OBJECT_POOL_N_TUPLE

inherit
	STI_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]
		redefine
			make_after,
			make_n,
			eq,
			terms,
			unit,
			converted_terms,
			term_to_index,
			term_to_term,
			tuple_anchor,
			trimmed_tuple_anchor,
			extended_tuple_anchor,
			terms_anchor
		end

create
	make_unit,
	make_after,
	make_n,
	make_from_tuple

convert
	make_from_tuple ({
		STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]],
		STI_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]],
		STS_TRANSFORMABLE_TUPLE [detachable separate ANY, detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_REFERENCE_EQUALITY [detachable separate ANY]],
		STI_TRANSFORMABLE_TUPLE [detachable separate ANY, detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_REFERENCE_EQUALITY [detachable separate ANY]]
		})

feature {NONE} -- Initialization

	make_after (a: detachable separate ANY; t: STS_N_TUPLE [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]])
			-- <Precursor>
		do
			tail := Unit
			tail := tail.converted (t)
			create first_storage.put (a)
		end

	make_n (a_n: NATURAL; a: detachable separate ANY)
			-- <Precursor>
		local
			i: NATURAL
		do
			if a_n = 0 then
				make_unit
			else
				from
					tail := Unit
					i := a_n - 1
				invariant
					n: tail.n = a_n - 1 - i
					repeated_terms: tail.n > 0 implies tail.terms.is_singleton and tail.terms ∋ a
				until
					i = 0
				loop
					tail := tail.after (a)
					i := i - 1
				variant
					i: i.as_integer_64
				end
				create first_storage.put (a)
			end
		end

feature -- Primitive

	eq: OBJECT_POOL_EQUALITY
			-- <Precursor>
		once
			create Result
		end

feature -- Access

	terms: like terms_anchor
			-- <Precursor>
		local
			t: like trimmed_tuple_anchor
		do
			from
				Result := {OBJECT_POOL}.O
				t := Current
			invariant
				positive: right_trimmed (t.n).indices |∀ agent index_is_positive
				small_enough: right_trimmed (t.n).indices |∀ agent index_is_less_equal(?, n)
				building_up: Result ≍ (right_trimmed (t.n).indices ↦ agent term)
			until
				t.is_unit
			loop
				Result := Result & t.first
				t := t.tail
			variant
				n: t.n.as_integer_64
			end
		end

feature -- Factory

	Unit: OBJECT_POOL_N_TUPLE
			-- <Precursor>
		once
			create Result.make_unit
		ensure then
			class
		end

	converted_terms (s: STS_SET [detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY]]): like terms_anchor
			-- <Precursor>
		do
			if attached {like terms_anchor} s as ct then
				Result := ct
			else
				Result := {OBJECT_POOL}.O
				check
					is_disjoint: Result.is_disjoint (s)
				end
				Result := Result.batch_extended (s)
			end
		end

feature -- Transformer

	term_to_index: STI_TRANSFORMER [detachable separate ANY, NATURAL, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_OBJECT_EQUALITY [NATURAL]]
			-- Transformer between types derived from {detachable separate ANY} and {NATURAL}
		once
			create Result
		end

	term_to_term: STI_TRANSFORMER [detachable separate ANY, detachable separate ANY, STS_REFERENCE_EQUALITY [detachable separate ANY], STS_REFERENCE_EQUALITY [detachable separate ANY]]
			-- Transformer between types derived from {detachable separate ANY}
		once
			create Result
		end

feature -- Anchor

	tuple_anchor,
	trimmed_tuple_anchor,
	extended_tuple_anchor: OBJECT_POOL_N_TUPLE
			-- <Precursor>
		do
			Result := Current
		end

	terms_anchor: OBJECT_POOL
			-- <Precursor>
		do
			Result := terms
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
