﻿note
	description: "[
		Pool of features available for test classes that need two generic parameters and their respective equality types
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Wed 2018-11-21 07:53:55 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	SETS_TESTS_RESOURCES_BINARY [
		A, B,
		EQ_A -> STS_EQUALITY [A] create default_create end,
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	SETS_TESTS_RESOURCES_UNARY [A, EQ_A]
		rename
			f_x as f_x_to_same_kind,
			f_acc_x as f_acc_x_to_same_kind,
			f_x_acc as f_x_acc_to_same_kind,
			f_acc_i_x as f_acc_i_x_to_same_kind,
			f_i_x_acc as f_i_x_acc_to_same_kind
		redefine
			some_equality,
			some_pair,
			some_pair_aa,
			some_set,
			some_set_a,
			same_set_a,
			some_pair_composer,
			some_transformer,
			some_n_tuple,
			some_n_tuple_a,
			same_n_tuple_a
		end

feature -- Access

	eq_b: EQ_B
			-- Equality for objects like {B}

	resources_b: SETS_TESTS_RESOURCES_UNARY [B, EQ_B]
			-- Pool of features for objects like {B}
		deferred
		end

feature -- Factory (object)

	some_object_b: B
			-- Randomly fetched object like {B}
		do
			Result := resources_b.some_object_a
		end

feature -- Factory (equality)

	some_equality: STS_EQUALITY [detachable separate ANY]
			-- <Precursor>
		local
			l_new_equality: FUNCTION [like some_equality]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				inspect
					next_random_item \\ 6
				when 0 then
					l_new_equality := agent new_reference_equality_ab
				when 1 then
					l_new_equality := agent new_object_equality_ab
				when 2 then
					l_new_equality := agent new_pair_equality_ab
				when 3 then
					l_new_equality := agent new_reference_equality_ba
				when 4 then
					l_new_equality := agent new_object_equality_ba
				when 5 then
					l_new_equality := agent new_pair_equality_ba
				end
				check
					eq: attached {like some_equality} some_direct_instance (l_new_equality) as eq -- `some_direct_instance' definition
				then
					Result := eq
				end
			when 1 then
				Result := resources_b.some_equality
			when 2 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	new_reference_equality_ab: STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]]
			-- Randomly created monomorphic reference-equality for ordered pairs like ({A}, {B})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_reference_equality_ab}
		end

	new_object_equality_ab: STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]]
			-- Randomly created monomorphic object-equality for ordered pairs like ({A}, {B})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_object_equality_ab}
		end

	new_pair_equality_ab: STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]
			-- Randomly created monomorphic mathematical equality for ordered pairs like ({A}, {B})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_equality_ab}
		end

	new_reference_equality_ba: STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [B, A, EQ_B, EQ_A]]
			-- Randomly created monomorphic reference-equality for ordered pairs like ({B}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_reference_equality_ba}
		end

	new_object_equality_ba: STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [B, A, EQ_B, EQ_A]]
			-- Randomly created monomorphic object-equality for ordered pairs like ({B}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_object_equality_ba}
		end

	new_pair_equality_ba: STS_ORDERED_PAIR_EQUALITY [B, A, EQ_B, EQ_A]
			-- Randomly created monomorphic mathematical equality for ordered pairs like ({B}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_equality_ba}
		end

feature -- Factory (ordered pair)

	some_pair: STS_ORDERED_PAIR [
		detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY], STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 5
			when 0 then
				Result := some_pair_ab
			when 1 then
				Result := some_pair_ba
			when 2 then
				Result := some_pair_bb
			when 3 then
				Result := resources_b.some_subset_family_structure_a
			when 4 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	some_preceding_pair_dsa_a (a: A): STS_ORDERED_PAIR [detachable separate ANY, A, STS_EQUALITY [detachable separate ANY], EQ_A]
			-- Randomly created polymorphic ordered pair in the form (x, `a'), meant to "precede" another pair in the form (`a', y).
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_preceding_pair_aa (a)
			when 1 then
				Result := some_preceding_pair_ba (a)
			end
		ensure
			second: eq_a (Result.b, a)
		end

	some_pair_aa: STS_ORDERED_PAIR [A, A, EQ_A, EQ_A]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_aa: attached {FUNCTION [like some_pair_aa]} agent new_composable_pair_baa as new_aa
					aa: attached {like some_pair_aa} some_direct_instance (new_aa) as aa -- `some_direct_instance' definition
				then
					Result := aa
				end
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	new_composable_pair_baa: STI_COMPOSABLE_ORDERED_PAIR [B, A, A, EQ_B, EQ_A, EQ_A]
			-- Randomly created monomorphic {B}-composable ordered pair like ({A}, {A})
		do
			create Result.make (some_object_a, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_baa}
		end

	some_preceding_pair_aa (a: A): like some_pair_aa
			-- Randomly created polymorphic ordered pair in the form ({A}, `a'), meant to "precede" another pair in the form (`a', y).
		do
			inspect
				next_random_item \\ 4
			when 0 then
				create {like new_pair_aa} Result.make (some_object_a, a)
			when 1 then
				create {like new_composable_pair_aaa} Result.make (some_object_a, a)
			when 2 then
				create {like new_homogeneous_pair_a} Result.make (some_object_a, a)
			when 3 then
				create {like new_composable_pair_baa} Result.make (some_object_a, a)
			end
--				check
--					does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--				end
--			set_object_pool (object_pool.extended (Result))
		ensure
			second: eq_a (Result.b, a)
		end

	some_pair_ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]
			-- Randomly fetched polymorphic ordered pair like ({A}, {B})
		local
			new_ab: FUNCTION [like some_pair_ab]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				new_ab := agent new_pair_ab
			when 1 then
				new_ab := agent new_composable_pair_aab
			when 2 then
				new_ab := agent new_composable_pair_bab
			end
			check
				ab: attached {like some_pair_ab} some_direct_instance (new_ab) as ab -- `some_direct_instance' definition
			then
				Result := ab
			end
		end

	new_pair_ab: STI_ORDERED_PAIR [A, B, EQ_A, EQ_B]
			-- Randomly created monomorphic ordered pair like ({A}, {B})
		do
			create Result.make (some_object_a, some_object_b)
		ensure
			monomorphic: Result.generating_type ~ {detachable STI_ORDERED_PAIR [A, B, EQ_A, EQ_B]}
		end

	new_composable_pair_aab: STI_COMPOSABLE_ORDERED_PAIR [A, A, B, EQ_A, EQ_A, EQ_B]
			-- Randomly created monomorphic {A}-composable ordered pair like ({A}, {B})
		do
			create Result.make (some_object_a, some_object_b)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_aab}
		end

	new_composable_pair_bab: STI_COMPOSABLE_ORDERED_PAIR [B, A, B, EQ_B, EQ_A, EQ_B]
			-- Randomly created monomorphic {B}-composable ordered pair like ({A}, {B})
		do
			create Result.make (some_object_a, some_object_b)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_bab}
		end

	same_pair_ab (ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]): like some_pair_ab
			-- Ordered pair mathematically equal to `ab', with type randomly chosen.
		do
			inspect
				next_random_item \\ 5
			when 0 then
				Result := ab
			when 1 then
				Result := ab.twin
			when 2 then
				create {like new_pair_ab} Result.make (ab.a, ab.b)
			when 3 then
				create {like new_composable_pair_aab} Result.make (ab.a, ab.b)
			when 4 then
				create {like new_composable_pair_bab} Result.make (ab.a, ab.b)
			end
--			if Result /= ab then
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
--			end
		ensure
			first: eq_a (Result.a, ab.a)
			second: eq_b (Result.b, ab.b)
		end

	some_pair_ba: STS_ORDERED_PAIR [B, A, EQ_B, EQ_A]
			-- Randomly fetched polymorphic ordered pair like ({B}, {A})
		local
			new_ba: FUNCTION [like some_pair_ba]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				new_ba := agent new_pair_ba
			when 1 then
				new_ba := agent new_composable_pair_aba
			when 2 then
				new_ba := agent new_composable_pair_bba
			end
			check
				ba: attached {like some_pair_ba} some_direct_instance (new_ba) as ba -- `some_direct_instance' definition
			then
				Result := ba
			end
		end

	new_pair_ba: STI_ORDERED_PAIR [B, A, EQ_B, EQ_A]
			-- Randomly created monomorphic ordered pair like ({B}, {A})
		do
			create Result.make (some_object_b, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable STI_ORDERED_PAIR [B, A, EQ_B, EQ_A]}
		end

	new_composable_pair_aba: STI_COMPOSABLE_ORDERED_PAIR [A, B, A, EQ_A, EQ_B, EQ_A]
			-- Randomly created monomorphic {A}-composable ordered pair like ({B}, {A})
		do
			create Result.make (some_object_b, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_aba}
		end

	new_composable_pair_bba: STI_COMPOSABLE_ORDERED_PAIR [B, B, A, EQ_B, EQ_B, EQ_A]
			-- Randomly created monomorphic {B}-composable ordered pair like ({B}, {A})
		do
			create Result.make (some_object_b, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_bba}
		end

	some_preceding_pair_ba (a: A): like some_pair_ba
			-- Randomly created polymorphic ordered pair in the form ({B}, `a'), meant to "precede" another pair in the form (`a', y).
		do
			inspect
				next_random_item \\ 3
			when 0 then
				create {like new_pair_ba} Result.make (some_object_b, a)
			when 1 then
				create {like new_composable_pair_aba} Result.make (some_object_b, a)
			when 2 then
				create {like new_composable_pair_bba} Result.make (some_object_b, a)
			end
--				check
--					does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--				end
--			set_object_pool (object_pool.extended (Result))
		ensure
			second: eq_a (Result.b, a)
		end

	some_pair_bb: STS_ORDERED_PAIR [B, B, EQ_B, EQ_B]
			-- Randomly fetched polymorphic ordered pair like ({B}, {B})
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					new_bb: attached {FUNCTION [like some_pair_bb]} agent new_composable_pair_abb as new_bb
					bb: attached {like some_pair_bb} some_direct_instance (new_bb) as bb -- `some_direct_instance' definition
				then
					Result := bb
				end
			when 1 then
				Result := resources_b.some_pair_aa
			end
		end

	new_composable_pair_abb: STI_COMPOSABLE_ORDERED_PAIR [A, B, B, EQ_A, EQ_B, EQ_B]
			-- Randomly created monomorphic {A}-composable ordered pair like ({B}, {B})
		do
			create Result.make (some_object_b, some_object_b)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_abb}
		end

	some_preceding_pair_bb (b: B): like some_pair_bb
			-- Randomly created polymorphic ordered pair in the form ({B}, `b`), meant to "precede" another pair in the form (`b`, y).
		do
			inspect
				next_random_item \\ 4
			when 0 then
				create {like resources_b.new_pair_aa} Result.make (some_object_b, b)
			when 1 then
				create {like new_composable_pair_abb} Result.make (some_object_b, b)
			when 2 then
				create {like resources_b.new_composable_pair_aaa} Result.make (some_object_b, b)
			when 3 then
				create {like resources_b.new_homogeneous_pair_a} Result.make (some_object_b, b)
			end
--				check
--					does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--				end
--			set_object_pool (object_pool.extended (Result))
		ensure
			second: eq_b (Result.b, b)
		end

feature -- Factory (set)

	some_set: STI_SET [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		local
			l_new_set: FUNCTION [like some_set]
		do
			inspect
				next_random_item \\ 4
			when 0 then
				inspect
					next_random_item \\ 24
				when 0 then
					l_new_set := agent resources_b.new_references_a
				when 1 then
					l_new_set := agent resources_b.new_objects_a
				when 2 then
					l_new_set := agent new_references_ba
				when 3 then
					l_new_set := agent new_objects_ba
				when 4 then
					l_new_set := agent new_math_set_ba
				when 5 then
					l_new_set := agent new_transformable_set_ba_a
				when 6 then
					l_new_set := agent new_transformable_set_ba_b
				when 7 then
					l_new_set := agent new_set_of_pairs_ba
				when 8 then
					l_new_set := agent new_references_bb
				when 9 then
					l_new_set := agent new_objects_bb
				when 10 then
					l_new_set := agent new_math_set_bb
				when 11 then
					l_new_set := agent new_transformable_set_bb_b
				when 12 then
					l_new_set := agent new_set_of_pairs_bb
				when 13 then
					l_new_set := agent new_references_sb
				when 14 then
					l_new_set := agent new_objects_sb
				when 15 then
					l_new_set := agent new_math_set_sb
				when 16 then
					l_new_set := agent new_transformable_set_sb_tb
				when 17 then
					l_new_set := agent new_set_family_b
				when 18 then
					l_new_set := agent new_set_partition_b
				when 19 then
					l_new_set := agent new_subset_family_b
				when 20 then
					l_new_set := agent new_references_tb
				when 21 then
					l_new_set := agent new_objects_tb
				when 22 then
					l_new_set := agent new_math_set_tb
				when 23 then
					l_new_set := agent new_transformable_set_tb_sb
				end
				check
					s: attached {like some_set} some_direct_instance (l_new_set) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_set_b
			when 2 then
				Result := some_set_ab
			when 3 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	some_set_a: STI_SET [A, EQ_A]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_transformable_set_ab
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	some_transformable_set_ab: STI_TRANSFORMABLE_SET [A, B, EQ_A, EQ_B]
			-- Randomly fetched {B}-transformable set of elements like {A}
		do
			Result := some_direct_transformable_set_ab
		end

	some_direct_transformable_set_ab: like some_transformable_set_ab
			-- Randomly fetched monomorphic {B}-transformable set of elements like {A}
		do
			check
				new_s: attached {FUNCTION [like new_transformable_set_ab]} agent new_transformable_set_ab as new_s

					-- `some_direct_instance' and `new_transformable_set_ab' definitions
				s: attached {like new_transformable_set_ab} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_transformable_set_ab}
			then
				Result := cropped_set (s)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_transformable_set_ab}
		end

	new_transformable_set_ab: like some_direct_transformable_set_ab
			-- Randomly created monomorphic {B}-transformable set of elements like {A}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_object_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_transformable_set_ab}
		end

	same_set_a (s: STI_SET [A, EQ_A]): like some_set_a
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				create {like some_direct_transformable_set_ab} Result.make_empty
					check
						is_disjoint: Result.is_disjoint (s) -- Result.is_empty
					end
				Result := Result.batch_extended (s)
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY} (s)
			end
		end

	same_transformable_set_ab (s: STI_TRANSFORMABLE_SET [A, B, EQ_A, EQ_B]): like some_transformable_set_ab
			-- {B}-transformable set mathematically equal to `s`, with type randomly chosen.
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := s
			when 1 then
				Result := s.twin
			when 2 then
				create {like some_direct_transformable_set_ab} Result.make_empty
					check
						is_disjoint: Result.is_disjoint (s) -- Result.is_empty
					end
				Result := Result.batch_extended (s)
			end
--			if Result /= s then
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
--			end
		ensure
			definition: Result ≍ s
		end

	some_set_b: STI_SET [B, EQ_B]
			-- Randomly fetched polymorphic set of elements like {B}, whose equality is checked by {EQ_B}.
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_transformable_set_ba
			when 1 then
				Result := resources_b.some_set_a
			end
		end

	some_transformable_set_ba: STI_TRANSFORMABLE_SET [B, A, EQ_B, EQ_A]
			-- Randomly fetched {A}-transformable set of elements like {B}
		do
			check
				new_s: attached {FUNCTION [like new_transformable_set_ba]} agent new_transformable_set_ba as new_s
				s: attached {like new_transformable_set_ba} some_direct_instance (new_s) as s -- `some_direct_instance' definition
			then
				Result := cropped_set (s)
			end
		end

	new_transformable_set_ba: like some_transformable_set_ba
			-- Randomly created monomorphic {A}-transformable set of elements like {B}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_object_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_transformable_set_ba}
		end

	some_set_ab: STI_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_EQUALITY [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]]]
			-- Randomly fetched polymorphic set of ordered pairs like ({A}, {B})
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_references_ab
			when 1 then
				Result := some_objects_ab
			when 2 then
				Result := some_math_set_ab
			end
		end

	some_references_ab: STI_SET [
		STS_ORDERED_PAIR [A, B, EQ_A, EQ_B],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]]
		]
			-- Randomly fetched set of ordered pairs like ({A}, {B}), and whose pairs are compared by their object references.
		do
			check
				new_abs: attached {FUNCTION [like new_references_ab]} agent new_references_ab as new_abs

					-- `some_direct_instance' and `new_references_ab' definition
				abs: attached {like new_references_ab} some_direct_instance (new_abs) as abs
				monomorphic: abs.generating_type ~ {detachable like new_references_ab}
			then
				Result := cropped_set (abs)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_references_ab}
		end

	new_references_ab: like some_references_ab
			-- Randomly created monomorphic set of ordered pairs like ({A}, {B}), and whose pairs are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ab
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_references_ab}
		end

	some_objects_ab: STI_SET [
		STS_ORDERED_PAIR [A, B, EQ_A, EQ_B],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]]
		]
			-- Randomly fetched set of ordered pairs like ({A}, {B}), and whose pairs are compared by their object values.
		do
			check
				new_abs: attached {FUNCTION [like new_objects_ab]} agent new_objects_ab as new_abs

					-- `some_direct_instance' and `new_objects_ab' definition
				abs: attached {like new_objects_ab} some_direct_instance (new_abs) as abs
				monomorphic: abs.generating_type ~ {detachable like new_objects_ab}
			then
				Result := cropped_set (abs)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_objects_ab}
		end

	new_objects_ab: like some_objects_ab
			-- Randomly created set of ordered pairs like ({A}, {B}), and whose pairs are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ab
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_objects_ab}
		end

	some_math_set_ab: STI_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B]]
			-- Randomly fetched set of ordered pairs like ({A}, {B}), and whose pairs are compared by their mathematical values.
		local
			l_new_math_set_ab: FUNCTION [like some_math_set_ab]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				inspect
					next_random_item \\ 3
				when 0 then
					l_new_math_set_ab := agent new_math_set_ab
				when 1 then
					l_new_math_set_ab := agent new_transformable_set_ab_a
				when 2 then
					l_new_math_set_ab := agent new_transformable_set_ab_b
				end
				check
					abs: attached {like some_math_set_ab} some_direct_instance (l_new_math_set_ab) as abs -- `some_direct_instance' definition
				then
					Result := cropped_set (abs)
				end
			when 1 then
				Result := some_set_of_pairs_ab
			end
		end

	new_math_set_ab: like some_math_set_ab
			-- Randomly created monomorphic set of ordered pairs like ({A}, {B}), and whose pairs are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ab
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_math_set_ab}
		end

	new_transformable_set_ab_a: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], A, STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], EQ_A]
			-- Randomly created monomorphic {A}-transformable set of ordered pairs like ({A}, {B})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ab
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_ab_a}
		end

	new_transformable_set_ab_b: STI_TRANSFORMABLE_SET [
		STS_ORDERED_PAIR [A, B, EQ_A, EQ_B], B, STS_ORDERED_PAIR_EQUALITY [A, B, EQ_A, EQ_B], EQ_B
		]
			-- Randomly created monomorphic {B}-transformable set of ordered pairs like ({A}, {B})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ab
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_ab_b}
		end

	some_set_of_pairs_ab: STI_SET_OF_ORDERED_PAIRS [A, B, EQ_A, EQ_B]
			-- Randomly fetched set of pairs like ({A}, {B})
		do
			Result := some_direct_set_of_pairs_ab
		end

	some_direct_set_of_pairs_ab: like some_set_of_pairs_ab
			-- Randomly fetched monomorphic set of pairs like ({A}, {B})
		do
			check
				new_abs: attached {FUNCTION [like new_set_of_pairs_ab]} agent new_set_of_pairs_ab as new_abs

					-- `some_direct_instance' and `new_set_of_pairs_ab' definitions
				abs: attached {like new_set_of_pairs_ab} some_direct_instance (new_abs) as abs
				monomorphic: abs.generating_type ~ {detachable like new_set_of_pairs_ab}
			then
				Result := cropped_set (abs)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_set_of_pairs_ab}
		end

	new_set_of_pairs_ab: like some_direct_set_of_pairs_ab
			-- Randomly created monomorphic set of pairs like ({A}, {B})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ab
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_set_of_pairs_ab}
		end

	new_references_ba: STI_SET [
		STS_ORDERED_PAIR [B, A, EQ_B, EQ_A],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [B, A, EQ_B, EQ_A]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {A}), and whose pairs are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ba
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_ba}
		end

	new_objects_ba: STI_SET [
		STS_ORDERED_PAIR [B, A, EQ_B, EQ_A],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [B, A, EQ_B, EQ_A]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {A}), and whose pairs are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ba
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_ba}
		end

	new_math_set_ba: STI_SET [STS_ORDERED_PAIR [B, A, EQ_B, EQ_A], STS_ORDERED_PAIR_EQUALITY [B, A, EQ_B, EQ_A]]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {A}), which are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ba
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_ba}
		end

	new_transformable_set_ba_a: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [B, A, EQ_B, EQ_A], A, STS_ORDERED_PAIR_EQUALITY [B, A, EQ_B, EQ_A], EQ_A]
			-- Randomly created monomorphic {A}-transformable set of ordered pairs like ({B}, {A})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ba
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_ba_a}
		end

	new_transformable_set_ba_b: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [B, A, EQ_B, EQ_A], B, STS_ORDERED_PAIR_EQUALITY [B, A, EQ_B, EQ_A], EQ_B]
			-- Randomly created monomorphic {B}-transformable set of ordered pairs like ({B}, {A})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ba
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_ba_b}
		end

	new_set_of_pairs_ba: STI_SET_OF_ORDERED_PAIRS [B, A, EQ_B, EQ_A]
			-- Randomly created monomorphic set of pairs like ({B}, {A})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ba
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_of_pairs_ba}
		end

	new_references_bb: STI_SET [
		STS_ORDERED_PAIR [B, B, EQ_B, EQ_B],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [B, B, EQ_B, EQ_B]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {B}), which are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_bb}
		end

	new_objects_bb: STI_SET [
		STS_ORDERED_PAIR [B, B, EQ_B, EQ_B],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [B, B, EQ_B, EQ_B]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {B}), which are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_bb}
		end

	new_math_set_bb: STI_SET [STS_ORDERED_PAIR [B, B, EQ_B, EQ_B], STS_ORDERED_PAIR_EQUALITY [B, B, EQ_B, EQ_B]]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {B}), which are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_bb}
		end

	new_transformable_set_bb_b: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [B, B, EQ_B, EQ_B], B, STS_ORDERED_PAIR_EQUALITY [B, B, EQ_B, EQ_B], EQ_B]
			-- Randomly created monomorphic {B}-transformable set of ordered pairs like ({B}, {B})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_bb_b}
		end

	new_set_of_pairs_bb: STI_SET_OF_ORDERED_PAIRS [B, B, EQ_B, EQ_B]
			-- Randomly created monomorphic set of pairs like ({B}, {B})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_of_pairs_bb}
		end

	new_references_sb: STI_SET [STS_SET [B, EQ_B], STS_REFERENCE_EQUALITY [STS_SET [B, EQ_B]]]
			-- Randomly created monomorphic set of sets of elements like {B}, and whose (member) sets are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_sb}
		end

	new_objects_sb: STI_SET [STS_SET [B, EQ_B], STS_OBJECT_EQUALITY [STS_SET [B, EQ_B]]]
			-- Randomly created monomorphic set of sets of elements like {B}, and whose (member) sets are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_sb}
		end

	new_math_set_sb: STI_SET [STS_SET [B, EQ_B], STS_SET_EQUALITY [B, EQ_B]]
			-- Randomly created monomorphic set of sets of elements like {B}, and whose (member) sets are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_sb}
		end

	new_transformable_set_sb_tb: STI_TRANSFORMABLE_SET [
		STS_SET [B, EQ_B], STS_N_TUPLE [B, EQ_B],
		STS_SET_EQUALITY [B, EQ_B], STS_N_TUPLE_EQUALITY [B, EQ_B]
		]
			-- Randomly created monomorphic set of sets of elements like {B}, which is (the outer set) transformable into a set of n-tuples of terms like {B}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_sb_tb}
		end

	new_set_family_b: STI_SET_FAMILY [B, EQ_B]
			-- Randomly created monomorphic family of sets of elements like {B}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable STI_SET_FAMILY [B, EQ_B]}
		end

	new_set_partition_b: STI_SET_PARTITION [B, EQ_B]
			-- Randomly created monomorphic partition of sets of elements like {B}
		local
			s1, s2, si: like some_set_b
			j: NATURAL
		do
			from
				s1 := some_set_b
				create Result.make_empty
				si := s1
			invariant
				building_partition_up: (s1 ∖ si).is_partitioned (Result)
			until
				si.is_empty
			loop
				from
					s2 := s1.o
					check
						positive: 0 < # si -- not si.is_empty
					end
					j := some_index_up_to (# si)
				invariant
					building_s2_up: s2 ≍ (s1 ∖ si ∖ (⋃ Result))
					si_emptiness: j > 0 implies not si.is_empty
					s2_emptiness: j = 0 implies not s2.is_empty
				until
					j = 0
				loop
						check
							si_is_not_empty: not si.is_empty -- j > 0
							s2_does_not_have_si_any: s2 ∌ si.any -- s2 ≍ (s1 ∖ si...)
						end
					s2 := s2.extended (si.any)
					si := si.others
					j := j - 1
				variant
					j: j.as_integer_64
				end
					check
						s2_is_not_empty: not s2.is_empty -- j = 0 implies not s2.is_empty
						set_is_disjoint: Result.set_is_disjoint (s2) -- s2 ≍ (s1 ∖ si ∖ (⋃ Result))
					end
				Result := Result.extended (s2)
			variant
				cardinality: (# si).as_integer_64
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_partition_b}
		end

	new_subset_family_b: STI_SUBSET_FAMILY [B, EQ_B]
			-- Randomly created monomorphic family of subsets of a set of elements like {B}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty (some_set_b)
			loop
				Result := Result & some_set_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_subset_family_b}
		end

	new_references_tb: STI_SET [STS_N_TUPLE [B, EQ_B], STS_REFERENCE_EQUALITY [STS_N_TUPLE [B, EQ_B]]]
			-- Randomly created monomorphic set of n-tuples of terms like {B}, and whose n-tuples are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_tb}
		end

	new_objects_tb: STI_SET [STS_N_TUPLE [B, EQ_B], STS_OBJECT_EQUALITY [STS_N_TUPLE [B, EQ_B]]]
			-- Randomly created monomorphic set of n-tuples of terms like {B}, and whose n-tuples are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_tb}
		end

	new_math_set_tb: STI_SET [STS_N_TUPLE [B, EQ_B], STS_N_TUPLE_EQUALITY [B, EQ_B]]
			-- Randomly created monomorphic set of n-tuples of terms like {B}, and whose n-tuples are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_tb}
		end

	new_transformable_set_tb_sb: STI_TRANSFORMABLE_SET [
		STS_N_TUPLE [B, EQ_B], STS_SET [B, EQ_B],
		STS_N_TUPLE_EQUALITY [B, EQ_B], STS_SET_EQUALITY [B, EQ_B]
		]
			-- Randomly created set of n-tuples of terms like {B}, which is (the outer set) transformable into a set of sets of elements like {B}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_b
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_tb_sb}
		end

feature -- Factory (pair composer)

	some_pair_composer: STI_ORDERED_PAIR_COMPOSER [
		detachable separate ANY, detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY],
		STS_EQUALITY [detachable separate ANY],
		STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		local
			l_new_pair_composer: FUNCTION [like some_pair_composer]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				inspect
					next_random_item \\ 6
				when 0 then
					l_new_pair_composer := agent new_pair_composer_aab
				when 1 then
					l_new_pair_composer := agent new_pair_composer_aba
				when 2 then
					l_new_pair_composer := agent new_pair_composer_abb
				when 3 then
					l_new_pair_composer := agent new_pair_composer_baa
				when 4 then
					l_new_pair_composer := agent new_pair_composer_bab
				when 5 then
					l_new_pair_composer := agent new_pair_composer_bba
				end
				check
					pc: attached {like some_pair_composer} some_direct_instance (l_new_pair_composer) as pc -- `some_direct_instance' definition
				then
					Result := pc
				end
			when 1 then
				Result := resources_b.some_pair_composer
			when 2 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	new_pair_composer_aab: STI_ORDERED_PAIR_COMPOSER [A, A, B, EQ_A, EQ_A, EQ_B]
			-- Randomly created monomorphic composer for ordered pairs like ({A}, {B}) and ({A}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_aab}
		end

	new_pair_composer_aba: STI_ORDERED_PAIR_COMPOSER [A, B, A, EQ_A, EQ_B, EQ_A]
			-- Randomly created monomorphic composer for ordered pairs like ({B}, {A}) and ({A}, {B})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_aba}
		end

	new_pair_composer_abb: STI_ORDERED_PAIR_COMPOSER [A, B, B, EQ_A, EQ_B, EQ_B]
			-- Randomly created monomorphic composer for ordered pairs like ({B}, {B}) and ({A}, {B})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_abb}
		end

	new_pair_composer_baa: STI_ORDERED_PAIR_COMPOSER [B, A, A, EQ_B, EQ_A, EQ_A]
			-- Randomly created monomorphic composer for ordered pairs like ({A}, {A}) and ({B}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_baa}
		end

	new_pair_composer_bab: STI_ORDERED_PAIR_COMPOSER [B, A, B, EQ_B, EQ_A, EQ_B]
			-- Randomly created monomorphic composer for ordered pairs like ({A}, {B}) and ({B}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_bab}
		end

	new_pair_composer_bba: STI_ORDERED_PAIR_COMPOSER [B, B, A, EQ_B, EQ_B, EQ_A]
			-- Randomly created monomorphic composer for ordered pairs like ({B}, {A}) and ({B}, {B})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_bba}
		end

feature -- Factory (transformer)

	some_transformer: STI_TRANSFORMER [
		detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY], STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		local
			l_new_transformer: FUNCTION [like some_transformer]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				inspect
					next_random_item \\ 2
				when 0 then
					l_new_transformer := agent new_transformer_ab
				when 1 then
					l_new_transformer := agent new_transformer_ba
				end
				check
					tr: attached {like some_transformer} some_direct_instance (l_new_transformer) as tr -- `some_direct_instance' definition
				then
					Result := tr
				end
			when 1 then
				Result := resources_b.some_transformer
			when 2 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	new_transformer_ab: STI_TRANSFORMER [A, B, EQ_A, EQ_B]
			-- Randomly created monomorphic transformer from objects like {A} to objects like {B}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformer_ab}
		end

	new_transformer_ba: STI_TRANSFORMER [B, A, EQ_B, EQ_A]
			-- Randomly created monomorphic transformer from objects like {A} to objects like {B}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformer_ba}
		end

feature -- Factory (n-tuple)

	some_n_tuple: STI_N_TUPLE [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		local
			l_new_n_tuple: FUNCTION [like some_n_tuple]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				inspect
					next_random_item \\ 3
				when 0 then
					l_new_n_tuple := agent new_n_tuple_sb
				when 1 then
					l_new_n_tuple := agent new_transformable_tuple_sb_tb
				when 2 then
					l_new_n_tuple := agent new_set_tuple_b
				end
				check
					t: attached {like some_n_tuple} some_direct_instance (l_new_n_tuple) as t -- `some_direct_instance' definition
				then
					Result := trimmed_n_tuple (t)
				end
			when 1 then
				Result := some_n_tuple_b
			when 2 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	some_n_tuple_a: STI_N_TUPLE [A, EQ_A]
			-- <Precursor>
		do
			inspect next_random_item \\ 2
			when 0 then
				Result := some_direct_transformable_tuple_ab
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY}
			end
		end

	some_direct_transformable_tuple_ab: STI_TRANSFORMABLE_TUPLE [A, B, EQ_A, EQ_B]
			-- Randomly fetched monomorphic {B}-transformable n-tuple of terms like {A}
		do
			check
					-- `some_direct_instance' and `new_transformable_tuple_ab' definition
				t: attached {like new_transformable_tuple_ab} some_direct_instance (agent new_transformable_tuple_ab) as t
				monomorphic: t.generating_type ~ {detachable like new_transformable_tuple_ab}
			then
				Result := trimmed_n_tuple (t)
			end
		end

	new_transformable_tuple_ab: like some_direct_transformable_tuple_ab
			-- Randomly created monomorphic {B}-transformable n-tuple of terms like {A}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_object_a)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_transformable_tuple_ab}
		end

	same_n_tuple_a (t: STI_N_TUPLE [A, EQ_A]): like some_n_tuple_a
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				create {like new_transformable_tuple_ab} Result.make_unit
				Result := Result + t
--					if Result /= t then
--						check
--							does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--						end
--					set_object_pool (object_pool.extended (Result))
--				end
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_UNARY} (t)
			end
		end

	some_n_tuple_b: STI_N_TUPLE [B, EQ_B]
			-- Randomly fetched polymorphic n-tuple of terms like {B}
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_transformable_tuple_ba
			when 1 then
				Result := resources_b.some_n_tuple_a
			end
		end

	some_direct_transformable_tuple_ba: STI_TRANSFORMABLE_TUPLE [B, A, EQ_B, EQ_A]
			-- Randomly fetched monomorphic {A}-transformable n-tuple of terms like {B}
		do
			check
					-- `some_direct_instance' and `new_transformable_tuple_ba' definition
				t: attached {like new_transformable_tuple_ba} some_direct_instance (agent new_transformable_tuple_ba) as t
				monomorphic: t.generating_type ~ {detachable like new_transformable_tuple_ba}
			then
				Result := trimmed_n_tuple (t)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_transformable_tuple_ba}
		end

	new_transformable_tuple_ba: like some_direct_transformable_tuple_ba
			-- Randomly created monomorphic {A}-transformable n-tuple of terms like {B}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_object_b)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_transformable_tuple_ba}
		end

	new_n_tuple_sb: STI_N_TUPLE [STS_SET [B, EQ_B], STS_SET_EQUALITY [B, EQ_B]]
			-- Randomly created monomorphic n-tuple of sets of elements like {B}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_b)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_n_tuple_sb}
		end

	new_transformable_tuple_sb_tb: STI_TRANSFORMABLE_TUPLE [
		STS_SET [B, EQ_B], STS_N_TUPLE [B, EQ_B],
		STS_SET_EQUALITY [B, EQ_B], STS_N_TUPLE_EQUALITY [B, EQ_B]
		]
			-- Randomly created monomorphic n-tuple of sets of elements like {B}, which is (the outer n-tuple) transformable into an n-tuple of n-tuples of terms like {B}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_b)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_tuple_sb_tb}
		end

	new_set_tuple_b: STI_SET_TUPLE [B, EQ_B]
			-- Randomly created monomorphic set-tuple of elements like {B}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_b)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_tuple_b}
		end

feature -- Predicate
	-- todo: explain.

	p_y (y: B): BOOLEAN
			-- Predicate over `y'
		do
			Result := resources_b.p_x (y)
		end

feature -- Mapping
	-- todo: explain.

	f_x (x: A): B
			-- Function that maps `x' to a value like {B}
		deferred
		end

feature -- Reduction
	-- todo: explain.

	f_acc_x (acc: B; x: A): B
			-- Function that reduces (`acc', `x') to a value like `acc'
		deferred
		end

	f_x_acc (x: A; acc: B): B
			-- Function that reduces (`x', `acc') to a value like `acc'
		do
			Result := f_acc_x (acc, x)
		end

	f_acc_i_x (acc: B; i: NATURAL; x: A): B
			-- Function that reduces (`acc', `i', `x') to a value like `acc'
		deferred
		end

	f_i_x_acc (i: NATURAL; x: A; acc: B): B
			-- Function that reduces (`i', `x', `acc') to a value like `acc'
		do
			Result := f_acc_i_x (acc, i, x)
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
