﻿note
	description: "[
		Pool of features available for test classes that need three generic
		parameters and their respective equality types
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Wed 2018-11-21 07:53:55 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	SETS_TESTS_RESOURCES_TERNARY [
		W, A, B,
		EQ_W -> STS_EQUALITY [W] create default_create end,
		EQ_A -> STS_EQUALITY [A] create default_create end,
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	SETS_TESTS_RESOURCES_BINARY [A, B, EQ_A, EQ_B]
		redefine
			some_equality,
			some_pair,
			some_preceding_pair_dsa_a,
			some_pair_aa,
			some_preceding_pair_aa,
			some_pair_ab,
			same_pair_ab,
			some_pair_ba,
			some_preceding_pair_ba,
			some_pair_bb,
			some_preceding_pair_bb,
			some_set,
			some_set_a,
			same_set_a,
			some_set_b,
			some_pair_composer,
			some_transformer,
			some_n_tuple,
			some_n_tuple_a,
			same_n_tuple_a,
			some_n_tuple_b
		end

feature -- Access

	eq_w: EQ_W
			-- Equality for objects like {W}

	resources_wa: SETS_TESTS_RESOURCES_BINARY [W, A, EQ_W, EQ_A]
			-- Pool of features that deal with objects like {W} and {A}
		deferred
		end

	resources_wb: SETS_TESTS_RESOURCES_BINARY [W, B, EQ_W, EQ_B]
			-- Pool of features that deal with objects like {W} and {B}
		deferred
		end

feature -- Factory (object)

	some_object_w: W
			-- Randomly fetched object like {W}
		do
			Result := resources_wa.some_object_a
		end

feature -- Factory (equality)

	some_equality: STS_EQUALITY [detachable separate ANY]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := resources_wa.some_equality
			when 1 then
				Result := resources_wb.some_equality
			when 2 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

feature -- Factory (ordered pair)

	some_pair: STS_ORDERED_PAIR [
		detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY], STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 6
			when 0 then
				Result := some_pair_ww
			when 1 then
				Result := some_pair_wa
			when 2 then
				Result := some_pair_wb
			when 3 then
				Result := some_pair_aw
			when 4 then
				Result := some_pair_bw
			when 5 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	some_pair_ww: STS_ORDERED_PAIR [W, W, EQ_W, EQ_W]
			-- Randomly fetched polymorphic ordered pair like ({W}, {W})
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := resources_wa.some_pair_aa
			when 1 then
				Result := resources_wb.some_pair_aa
			end
		end

	some_preceding_pair_ww (w: W): like some_pair_ww
			-- Randomly created polymorphic ordered pair in the form ({W}, `w`), meant to "precede" another pair in the form (`w`, y).
		do
			inspect
				next_random_item \\ 5
			when 0 then
				create {like resources_wa.new_pair_aa} Result.make (some_object_w, w)
			when 1 then
				create {like resources_wa.new_composable_pair_aaa} Result.make (some_object_w, w)
			when 2 then
				create {like resources_wa.new_composable_pair_baa} Result.make (some_object_w, w)
			when 3 then
				create {like resources_wb.new_composable_pair_baa} Result.make (some_object_w, w)
			when 4 then
				create {like resources_wb.new_homogeneous_pair_a} Result.make (some_object_w, w)
			end
--				check
--					does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--				end
--			set_object_pool (object_pool.extended (Result))
		ensure
			second: eq_w (Result.b, w)
		end

	some_pair_wa: STS_ORDERED_PAIR [W, A, EQ_W, EQ_A]
			-- Randomly fetched polymorphic ordered pair like ({W}, {A})
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					wa: attached {like new_composable_pair_bwa} some_direct_instance (agent new_composable_pair_bwa) as wa -- `some_direct_instance' definition
				then
					Result := wa
				end
			when 1 then
				Result := resources_wa.some_pair_ab
			end
		end

	new_composable_pair_bwa: STI_COMPOSABLE_ORDERED_PAIR [B, W, A, EQ_B, EQ_W, EQ_A]
			-- Randomly created monomorphic {B}-composable ordered pair like ({W}, {A})
		do
			create Result.make (some_object_w, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_bwa}
		end

	some_preceding_pair_wa (a: A): like some_pair_wa
			-- Randomly created polymorphic ordered pair in the form ({W}, `a`), meant to "precede" another pair in the form (`a`, y).
		do
			inspect
				next_random_item \\ 4
			when 0 then
				create {like resources_wa.new_pair_ab} Result.make (some_object_w, a)
			when 1 then
				create {like resources_wa.new_composable_pair_aab} Result.make (some_object_w, a)
			when 2 then
				create {like resources_wa.new_composable_pair_bab} Result.make (some_object_w, a)
			when 3 then
				create {like new_composable_pair_bwa} Result.make (some_object_w, a)
			end
--				check
--					does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--				end
--			set_object_pool (object_pool.extended (Result))
		ensure
			second: eq_a (Result.b, a)
		end

	some_pair_wb: STS_ORDERED_PAIR [W, B, EQ_W, EQ_B]
			-- Randomly fetched polymorphic ordered pair like ({W}, {B})
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					wb: attached {like new_composable_pair_awb} some_direct_instance (agent new_composable_pair_awb) as wb -- `some_direct_instance' definition
				then
					Result := wb
				end
			when 1 then
				Result := resources_wb.some_pair_ab
			end
		end

	new_composable_pair_awb: STI_COMPOSABLE_ORDERED_PAIR [A, W, B, EQ_A, EQ_W, EQ_B]
			-- Randomly created monomorphic {A}-composable ordered pair like ({W}, {B})
		do
			create Result.make (some_object_w, some_object_b)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_awb}
		end

	some_pair_aw: STS_ORDERED_PAIR [A, W, EQ_A, EQ_W]
			-- Randomly fetched polymorphic ordered pair like ({A}, {W})
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					aw: attached {like new_composable_pair_baw} some_direct_instance (agent new_composable_pair_baw) as aw -- `some_direct_instance' definition
				then
					Result := aw
				end
			when 1 then
				Result := resources_wa.some_pair_ba
			end
		end

	new_composable_pair_baw: STI_COMPOSABLE_ORDERED_PAIR [B, A, W, EQ_B, EQ_A, EQ_W]
			-- Randomly created monomorphic {B}-composable ordered pair like ({A}, {W})
		do
			create Result.make (some_object_a, some_object_w)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_baw}
		end

	some_preceding_pair_dsa_a (a: A): STS_ORDERED_PAIR [detachable separate ANY, A, STS_EQUALITY [detachable separate ANY], EQ_A]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_preceding_pair_wa (a)
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY} (a)
			end
		end

	some_pair_aa: STS_ORDERED_PAIR [A, A, EQ_A, EQ_A]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := resources_wa.some_pair_bb
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	some_preceding_pair_aa (a: A): like some_pair_aa
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				create {like resources_wa.new_composable_pair_abb} Result.make (some_object_a, a)
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY} (a)
			end
		end

	some_pair_ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_composable_pair_wab
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	some_direct_composable_pair_wab: STI_COMPOSABLE_ORDERED_PAIR [W, A, B, EQ_W, EQ_A, EQ_B]
			-- Randomly fetched monomorphic {W}-composable ordered pair like ({A}, {B})
		do
			check
				new_wab: attached {FUNCTION [like new_composable_pair_wab]} (agent new_composable_pair_wab) as new_wab

					-- `some_direct_instance' and `new_composable_pair_wab' definitions
				ab: attached {like new_composable_pair_wab} some_direct_instance (new_wab) as ab
				monomorphic: ab.generating_type ~ {detachable like new_composable_pair_wab}
			then
				Result := ab
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_composable_pair_wab}
		end

	new_composable_pair_wab: like some_direct_composable_pair_wab
			-- Randomly created monomorphic {W}-composable ordered pair like ({A}, {B})
		do
			create Result.make (some_object_a, some_object_b)
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_composable_pair_wab}
		end

	same_pair_ab (ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]): like some_pair_ab
			-- <Precursor>
		do
			if next_random_item \\ 2 = 0 then
				create {like new_composable_pair_wab} Result.make (ab.a, ab.b)
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
			else
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY} (ab)
			end
		end

	some_pair_bw: STS_ORDERED_PAIR [B, W, EQ_B, EQ_W]
			-- Randomly fetched polymorphic ordered pair like ({B}, {W})
		do
			inspect next_random_item \\ 2
			when 0 then
				check
					bw: attached {like new_composable_pair_abw} some_direct_instance (agent new_composable_pair_abw) as bw -- `some_direct_instance' definition
				then
					Result := bw
				end
			when 1 then
				Result := resources_wb.some_pair_ba
			end
		end

	new_composable_pair_abw: STI_COMPOSABLE_ORDERED_PAIR [A, B, W, EQ_A, EQ_B, EQ_W]
			-- Randomly created monomorphic {A}-composable ordered pair like ({B}, {W})
		do
			create Result.make (some_object_b, some_object_w)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_abw}
		end

	some_pair_ba: STS_ORDERED_PAIR [B, A, EQ_B, EQ_A]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				check
					ba: attached {like new_composable_pair_wba} some_direct_instance (agent new_composable_pair_wba) as ba -- `some_direct_instance' definition
				then
					Result := ba
				end
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	new_composable_pair_wba: STI_COMPOSABLE_ORDERED_PAIR [W, B, A, EQ_W, EQ_B, EQ_A]
			-- Randomly created monomorphic {W}-composable ordered pair like ({B}, {A})
		do
			create Result.make (some_object_b, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_wba}
		end

	some_preceding_pair_ba (a: A): like some_pair_ba
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				create {like new_composable_pair_wba} Result.make (some_object_b, a)
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY} (a)
			end
		end

	some_pair_bb: STS_ORDERED_PAIR [B, B, EQ_B, EQ_B]
			-- <Precursor>
		do
			inspect next_random_item \\ 2
			when 0 then
				Result := resources_wb.some_pair_bb
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	some_preceding_pair_bb (b: B): like some_pair_bb
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				create {like resources_wb.new_composable_pair_abb} Result.make (some_object_b, b)
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY} (b)
			end
		end

feature -- Factory (set)

	some_set: STI_SET [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		local
			l_new_set: FUNCTION [like some_set]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				inspect
					next_random_item \\ 42
				when 0 then
					l_new_set := agent resources_wa.new_references_a
				when 1 then
					l_new_set := agent resources_wa.new_objects_a
				when 2 then
					l_new_set := agent new_references_ww
				when 3 then
					l_new_set := agent new_objects_ww
				when 4 then
					l_new_set := agent new_math_set_ww
				when 5 then
					l_new_set := agent new_transformable_set_ww_w
				when 6 then
					l_new_set := agent new_set_of_pairs_ww
				when 7 then
					l_new_set := agent new_references_wa
				when 8 then
					l_new_set := agent new_objects_wa
				when 9 then
					l_new_set := agent new_math_set_wa
				when 10 then
					l_new_set := agent new_transformable_set_wa_w
				when 11 then
					l_new_set := agent new_transformable_set_wa_a
				when 12 then
					l_new_set := agent new_set_of_pairs_wa
				when 13 then
					l_new_set := agent new_references_wb
				when 14 then
					l_new_set := agent new_objects_wb
				when 15 then
					l_new_set := agent new_math_set_wb
				when 16 then
					l_new_set := agent new_transformable_set_wb_w
				when 17 then
					l_new_set := agent new_transformable_set_wb_b
				when 18 then
					l_new_set := agent new_set_of_pairs_wb
				when 19 then
					l_new_set := agent new_references_aw
				when 20 then
					l_new_set := agent new_objects_aw
				when 21 then
					l_new_set := agent new_math_set_aw
				when 22 then
					l_new_set := agent new_transformable_set_aw_a
				when 23 then
					l_new_set := agent new_transformable_set_aw_w
				when 24 then
					l_new_set := agent new_set_of_pairs_aw
				when 25 then
					l_new_set := agent new_references_bw
				when 26 then
					l_new_set := agent new_objects_bw
				when 27 then
					l_new_set := agent new_math_set_bw
				when 28 then
					l_new_set := agent new_transformable_set_bw_b
				when 29 then
					l_new_set := agent new_transformable_set_bw_w
				when 30 then
					l_new_set := agent new_set_of_pairs_bw
				when 31 then
					l_new_set := agent new_references_sw
				when 32 then
					l_new_set := agent new_objects_sw
				when 33 then
					l_new_set := agent new_math_set_sw
				when 34 then
					l_new_set := agent new_transformable_set_sw_tw
				when 35 then
					l_new_set := agent new_set_family_w
				when 36 then
					l_new_set := agent new_set_partition_w
				when 37 then
					l_new_set := agent new_subset_family_w
				when 38 then
					l_new_set := agent new_references_tw
				when 39 then
					l_new_set := agent new_objects_tw
				when 40 then
					l_new_set := agent new_math_set_tw
				when 41 then
					l_new_set := agent new_transformable_set_tw_sw
				end
				check
					s: attached {like some_set} some_direct_instance (l_new_set) as s -- `some_direct_instance' definition
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_set_w
			when 2 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	some_set_w: STI_SET [W, EQ_W]
			-- Randomly fetched polymorphic set of elements like {W}, whose equality is checked by {EQ_W}.
		do
			inspect next_random_item \\ 2
			when 0 then
				Result := resources_wa.some_set_a
			when 1 then
				Result := resources_wb.some_set_a
			end
		end

	some_set_a: STI_SET [A, EQ_A]
			-- <Precursor>
		do
			inspect next_random_item \\ 2
			when 0 then
				Result := resources_wa.some_transformable_set_ba
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	same_set_a (s: STI_SET [A, EQ_A]): like some_set_a
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				create {like resources_wa.new_transformable_set_ba} Result.make_empty
					check
						is_disjoint: Result.is_disjoint (s) -- Result.is_empty
					end
				Result := Result.batch_extended (s)
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY} (s)
			end
		end

	some_set_b: STI_SET [B, EQ_B]
			-- <Precursor>
		do
			inspect next_random_item \\ 2
			when 0 then
				Result := resources_wb.some_transformable_set_ba
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	new_references_ww: STI_SET [
		STS_ORDERED_PAIR [W, W, EQ_W, EQ_W],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [W, W, EQ_W, EQ_W]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {W}), which are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ww
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_ww}
		end

	new_objects_ww: STI_SET [
		STS_ORDERED_PAIR [W, W, EQ_W, EQ_W],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [W, W, EQ_W, EQ_W]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {W}), which are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ww
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_ww}
		end

	new_math_set_ww:
			STI_SET [STS_ORDERED_PAIR [W, W, EQ_W, EQ_W], STS_ORDERED_PAIR_EQUALITY [W, W, EQ_W, EQ_W]]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {W}), which are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ww
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_ww}
		end

	new_transformable_set_ww_w: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [W, W, EQ_W, EQ_W], W, STS_ORDERED_PAIR_EQUALITY [W, W, EQ_W, EQ_W], EQ_W]
			-- Randomly created monomorphic {W}-transformable set of ordered pairs like ({W}, {W})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ww
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_ww_w}
		end

	new_set_of_pairs_ww: STI_SET_OF_ORDERED_PAIRS [W, W, EQ_W, EQ_W]
			-- Randomly created monomorphic set of pairs like ({W}, {W})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_ww
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_of_pairs_ww}
		end

	new_references_wa: STI_SET [
		STS_ORDERED_PAIR [W, A, EQ_W, EQ_A],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [W, A, EQ_W, EQ_A]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {A}), which are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_wa}
		end

	new_objects_wa: STI_SET [
		STS_ORDERED_PAIR [W, A, EQ_W, EQ_A],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [W, A, EQ_W, EQ_A]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {A}), which are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_wa}
		end

	new_math_set_wa:
			STI_SET [STS_ORDERED_PAIR [W, A, EQ_W, EQ_A], STS_ORDERED_PAIR_EQUALITY [W, A, EQ_W, EQ_A]]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {A}), which are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_wa}
		end

	new_transformable_set_wa_w: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [W, A, EQ_W, EQ_A], W, STS_ORDERED_PAIR_EQUALITY [W, A, EQ_W, EQ_A], EQ_W]
			-- Randomly created monomorphic {W}-transformable set of ordered pairs like ({W}, {A})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_wa_w}
		end

	new_transformable_set_wa_a: STI_TRANSFORMABLE_SET [
		STS_ORDERED_PAIR [W, A, EQ_W, EQ_A], A,
		STS_ORDERED_PAIR_EQUALITY [W, A, EQ_W, EQ_A], EQ_A
		]
			-- Randomly created monomorphic {A}-transformable set of ordered pairs like ({W}, {A})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_wa_a}
		end

	new_set_of_pairs_wa: STI_SET_OF_ORDERED_PAIRS [W, A, EQ_W, EQ_A]
			-- Randomly created monomorphic set of pairs like ({W}, {A})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_of_pairs_wa}
		end

	new_references_wb: STI_SET [
		STS_ORDERED_PAIR [W, B, EQ_W, EQ_B],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [W, B, EQ_W, EQ_B]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {B}), which are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_wb}
		end

	new_objects_wb: STI_SET [
		STS_ORDERED_PAIR [W, B, EQ_W, EQ_B],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [W, B, EQ_W, EQ_B]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {B}), which are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_wb}
		end

	new_math_set_wb: STI_SET [STS_ORDERED_PAIR [W, B, EQ_W, EQ_B], STS_ORDERED_PAIR_EQUALITY [W, B, EQ_W, EQ_B]]
			-- Randomly created monomorphic set of ordered pairs like ({W}, {B}), which are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_wb}
		end

	new_transformable_set_wb_w: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [W, B, EQ_W, EQ_B], W, STS_ORDERED_PAIR_EQUALITY [W, B, EQ_W, EQ_B], EQ_W]
			-- Randomly created monomorphic {W}-transformable set of ordered pairs like ({W}, {B})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_wb_w}
		end

	new_transformable_set_wb_b: STI_TRANSFORMABLE_SET [
		STS_ORDERED_PAIR [W, B, EQ_W, EQ_B], B,
		STS_ORDERED_PAIR_EQUALITY [W, B, EQ_W, EQ_B], EQ_B
		]
			-- Randomly created monomorphic {B}-transformable set of ordered pairs like ({W}, {B})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_wb_b}
		end

	new_set_of_pairs_wb: STI_SET_OF_ORDERED_PAIRS [W, B, EQ_W, EQ_B]
			-- Randomly created monomorphic set of pairs like ({W}, {B})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_wb
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable STI_SET_OF_ORDERED_PAIRS [W, B, EQ_W, EQ_B]}
		end

	new_references_aw: STI_SET [
		STS_ORDERED_PAIR [A, W, EQ_A, EQ_W],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [A, W, EQ_A, EQ_W]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({A}, {W}), which are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_aw}
		end

	new_objects_aw: STI_SET [
		STS_ORDERED_PAIR [A, W, EQ_A, EQ_W],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [A, W, EQ_A, EQ_W]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({A}, {W}), which are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_aw}
		end

	new_math_set_aw: STI_SET [STS_ORDERED_PAIR [A, W, EQ_A, EQ_W], STS_ORDERED_PAIR_EQUALITY [A, W, EQ_A, EQ_W]]
			-- Randomly created monomorphic set of ordered pairs like ({A}, {W}), which are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_aw}
		end

	new_transformable_set_aw_a: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [A, W, EQ_A, EQ_W], A, STS_ORDERED_PAIR_EQUALITY [A, W, EQ_A, EQ_W], EQ_A]
			-- Randomly created monomorphic {A}-transformable set of ordered pairs like ({A}, {W})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_aw_a}
		end

	new_transformable_set_aw_w: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [A, W, EQ_A, EQ_W], W, STS_ORDERED_PAIR_EQUALITY [A, W, EQ_A, EQ_W], EQ_W]
			-- Randomly created monomorphic {W}-transformable set of ordered pairs like ({A}, {W})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_aw_w}
		end

	new_set_of_pairs_aw: STI_SET_OF_ORDERED_PAIRS [A, W, EQ_A, EQ_W]
			-- Randomly created monomorphic set of pairs like ({A}, {W})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_of_pairs_aw}
		end

	new_references_bw: STI_SET [
		STS_ORDERED_PAIR [B, W, EQ_B, EQ_W],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [B, W, EQ_B, EQ_W]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {W}), which are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_bw}
		end

	new_objects_bw: STI_SET [
		STS_ORDERED_PAIR [B, W, EQ_B, EQ_W],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [B, W, EQ_B, EQ_W]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {W}), which are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_bw}
		end

	new_math_set_bw: STI_SET [STS_ORDERED_PAIR [B, W, EQ_B, EQ_W], STS_ORDERED_PAIR_EQUALITY [B, W, EQ_B, EQ_W]]
			-- Randomly created monomorphic set of ordered pairs like ({B}, {W}), which are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_bw}
		end

	new_transformable_set_bw_b: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [B, W, EQ_B, EQ_W], B, STS_ORDERED_PAIR_EQUALITY [B, W, EQ_B, EQ_W], EQ_B]
			-- Randomly created monomorphic {B}-transformable set of ordered pairs like ({B}, {W})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_bw_b}
		end

	new_transformable_set_bw_w: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [B, W, EQ_B, EQ_W], W, STS_ORDERED_PAIR_EQUALITY [B, W, EQ_B, EQ_W], EQ_W]
			-- Randomly created monomorphic {W}-transformable set of ordered pairs like ({B}, {W})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_bw_w}
		end

	new_set_of_pairs_bw: STI_SET_OF_ORDERED_PAIRS [B, W, EQ_B, EQ_W]
			-- Randomly created monomorphic set of pairs like ({B}, {W})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_bw
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_of_pairs_bw}
		end

	new_references_sw: STI_SET [STS_SET [W, EQ_W], STS_REFERENCE_EQUALITY [STS_SET [W, EQ_W]]]
			-- Randomly created monomorphic set of sets of elements like {W}, and whose (member) sets are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_sw}
		end

	new_objects_sw: STI_SET [STS_SET [W, EQ_W], STS_OBJECT_EQUALITY [STS_SET [W, EQ_W]]]
			-- Randomly created monomorphic set of sets of elements like {W}, and whose (member) sets are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_sw}
		end

	new_math_set_sw: STI_SET [STS_SET [W, EQ_W], STS_SET_EQUALITY [W, EQ_W]]
			-- Randomly created monomorphic set of sets of elements like {A}, and whose (member) sets are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_sw}
		end

	new_transformable_set_sw_tw: STI_TRANSFORMABLE_SET [
		STS_SET [W, EQ_W], STS_N_TUPLE [W, EQ_W],
		STS_SET_EQUALITY [W, EQ_W], STS_N_TUPLE_EQUALITY [W, EQ_W]
		]
			-- Randomly created monomorphic set of sets of elements like {W}, which is (the outer set) transformable into a set of n-tuples of terms like {W}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_sw_tw}
		end

	new_set_family_w: STI_SET_FAMILY [W, EQ_W]
			-- Randomly created monomorphic family of sets of elements like {W}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_family_w}
		end

	new_set_partition_w: STI_SET_PARTITION [W, EQ_W]
			-- Randomly created monomorphic partition of sets of elements like {W}
		local
			s1, s2, si: like some_set_w
			j: NATURAL
		do
			from
				s1 := some_set_w
				create Result.make_empty
				si := s1
			invariant
				building_partition_up: (s1 ∖ si).is_partitioned (Result)
			until
				si.is_empty
			loop
				from
					s2 := s1.o
						check
							positive: 0 < # si -- not si.is_empty
						end
					j := some_index_up_to (# si)
				invariant
					building_s2_up: s2 ≍ (s1 ∖ si ∖ (⋃ Result))
					si_emptiness: j > 0 implies not si.is_empty
					s2_emptiness: j = 0 implies not s2.is_empty
				until
					j = 0
				loop
						check
							si_is_not_empty: not si.is_empty -- j > 0
							s2_does_not_have_si_any: s2 ∌ si.any -- s2 ≍ (s1 ∖ si...)
						end
					s2 := s2.extended (si.any)
					si := si.others
					j := j - 1
				variant
					j: j.as_integer_64
				end
					check
						s2_is_not_empty: not s2.is_empty -- j = 0 implies not s2.is_empty
						set_is_disjoint: Result.set_is_disjoint (s2) -- s2 ≍ (s1 ∖ si ∖ (⋃ Result))
					end
				Result := Result.extended (s2)
			variant
				cardinality: (# si).as_integer_64
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_partition_w}
		end

	new_subset_family_w: STI_SUBSET_FAMILY [W, EQ_W]
			-- Randomly created monomorphic family of subsets of a set of elements like {W}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty (some_set_w)
			loop
				Result := Result & some_set_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_subset_family_w}
		end

	new_references_tw: STI_SET [STS_N_TUPLE [W, EQ_W], STS_REFERENCE_EQUALITY [STS_N_TUPLE [W, EQ_W]]]
			-- Randomly created monomorphic set of n-tuples of terms like {W}, and whose n-tuples are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_tw}
		end

	new_objects_tw: STI_SET [STS_N_TUPLE [W, EQ_W], STS_OBJECT_EQUALITY [STS_N_TUPLE [W, EQ_W]]]
			-- Randomly created monomorphic set of n-tuples of terms like {W}, and whose n-tuples are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_tw}
		end

	new_math_set_tw: STI_SET [STS_N_TUPLE [W, EQ_W], STS_N_TUPLE_EQUALITY [W, EQ_W]]
			-- Randomly created monomorphic set of n-tuples of terms like {W}, and whose n-tuples are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_tw}
		end

	new_transformable_set_tw_sw: STI_TRANSFORMABLE_SET [
		STS_N_TUPLE [W, EQ_W], STS_SET [W, EQ_W],
		STS_N_TUPLE_EQUALITY [W, EQ_W], STS_SET_EQUALITY [W, EQ_W]
		]
			-- Randomly created set of n-tuples of terms like {W}, which is (the outer set) transformable into a set of sets of elements like {W}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_w
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_tw_sw}
		end

feature -- Factory (pair composer)

	some_pair_composer: STI_ORDERED_PAIR_COMPOSER [
		detachable separate ANY, detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY],
		STS_EQUALITY [detachable separate ANY],
		STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		local
			l_new_pair_composer: FUNCTION [like some_pair_composer]
		do
			inspect
				next_random_item \\ 5
			when 0 then
				inspect
					next_random_item \\ 5
				when 0 then
					l_new_pair_composer := agent new_pair_composer_wba
				when 1 then
					l_new_pair_composer := agent new_pair_composer_awb
				when 2 then
					l_new_pair_composer := agent new_pair_composer_abw
				when 3 then
					l_new_pair_composer := agent new_pair_composer_bwa
				when 4 then
					l_new_pair_composer := agent new_pair_composer_baw
				end
				check
					pc: attached {like some_pair_composer} some_direct_instance (l_new_pair_composer) as pc -- `some_direct_instance' definition
				then
					Result := pc
				end
			when 1 then
				Result := some_direct_pair_composer_wab
			when 2 then
				Result := resources_wa.some_pair_composer
			when 3 then
				Result := resources_wb.some_pair_composer
			when 4 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	some_direct_pair_composer_wab: STI_ORDERED_PAIR_COMPOSER [W, A, B, EQ_W, EQ_A, EQ_B]
			-- Randomly fetched monomorphic composer for ordered pairs like ({A}, {B}) and ({W}, {A})
		do
			check
				pc: attached {like new_pair_composer_wab} some_direct_instance (agent new_pair_composer_wab) as pc -- `some_direct_instance' definition
				monomorphic: pc.generating_type ~ {detachable like new_pair_composer_wab}
			then
				Result := pc
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_pair_composer_wab}
		end

	new_pair_composer_wab: like some_direct_pair_composer_wab
			-- Randomly created monomorphic composer for ordered pairs like ({A}, {B}) and ({W}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_pair_composer_wab}
		end

	new_pair_composer_wba: STI_ORDERED_PAIR_COMPOSER [W, B, A, EQ_W, EQ_B, EQ_A]
			-- Randomly created monomorphic composer for ordered pairs like ({B}, {A}) and ({W}, {B})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_wba}
		end

	new_pair_composer_awb: STI_ORDERED_PAIR_COMPOSER [A, W, B, EQ_A, EQ_W, EQ_B]
			-- Randomly created monomorphic composer for ordered pairs like ({W}, {B}) and ({A}, {W})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_awb}
		end

	new_pair_composer_abw: STI_ORDERED_PAIR_COMPOSER [A, B, W, EQ_A, EQ_B, EQ_W]
			-- Randomly created monomorphic composer for ordered pairs like ({B}, {W}) and ({A}, {B})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_abw}
		end

	new_pair_composer_bwa: STI_ORDERED_PAIR_COMPOSER [B, W, A, EQ_B, EQ_W, EQ_A]
			-- Randomly created monomorphic composer for ordered pairs like ({W}, {A}) and ({B}, {W})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_bwa}
		end

	new_pair_composer_baw: STI_ORDERED_PAIR_COMPOSER [B, A, W, EQ_B, EQ_A, EQ_W]
			-- Randomly created monomorphic composer for ordered pairs like ({A}, {W}) and ({B}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_baw}
		end

feature -- Factory (transformer)

	some_transformer: STI_TRANSFORMER [
		detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY], STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := resources_wa.some_transformer
			when 1 then
				Result := resources_wb.some_transformer
			when 2 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

feature -- Factory (n-tuple)

	some_n_tuple: STI_N_TUPLE [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		local
			l_new_n_tuple: FUNCTION [like some_n_tuple]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				inspect
					next_random_item \\ 3
				when 0 then
					l_new_n_tuple := agent new_n_tuple_sw
				when 1 then
					l_new_n_tuple := agent new_transformable_tuple_sw_tw
				when 2 then
					l_new_n_tuple := agent new_set_tuple_w
				end
				check
					t: attached {like some_n_tuple} some_direct_instance (l_new_n_tuple) as t -- `some_direct_instance' definition
				then
					Result := trimmed_n_tuple (t)
				end
			when 1 then
				Result := some_n_tuple_w
			when 2 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	some_n_tuple_w: STI_N_TUPLE [W, EQ_W]
			-- Randomly fetched polymorphic n-tuple of terms like {W}
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := resources_wa.some_n_tuple_a
			when 1 then
				Result := resources_wb.some_n_tuple_a
			end
		end

	some_n_tuple_a: STI_N_TUPLE [A, EQ_A]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := resources_wa.some_direct_transformable_tuple_ba
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	same_n_tuple_a (t: STI_N_TUPLE [A, EQ_A]): like some_n_tuple_a
			-- <Precursor>
		do
			inspect next_random_item \\ 2
			when 0 then
				create {like resources_wa.new_transformable_tuple_ba} Result.make_unit
				Result := Result + t
--				if Result /= t then
--						check
--							does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--						end
--					set_object_pool (object_pool.extended (Result))
--				end
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY} (t)
			end
		end

	some_n_tuple_b: STI_N_TUPLE [B, EQ_B]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := resources_wb.some_direct_transformable_tuple_ba
			when 1 then
				Result := Precursor {SETS_TESTS_RESOURCES_BINARY}
			end
		end

	new_n_tuple_sw: STI_N_TUPLE [STS_SET [W, EQ_W], STS_SET_EQUALITY [W, EQ_W]]
			-- Randomly created monomorphic n-tuple of sets of elements like {B}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_w)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_n_tuple_sw}
		end

	new_transformable_tuple_sw_tw: STI_TRANSFORMABLE_TUPLE [
		STS_SET [W, EQ_W], STS_N_TUPLE [W, EQ_W],
		STS_SET_EQUALITY [W, EQ_W], STS_N_TUPLE_EQUALITY [W, EQ_W]
		]
			-- Randomly created monomorphic n-tuple of sets of elements like {W}, which is (the outer n-tuple) transformable into an n-tuple of n-tuples of terms like {W}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_w)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_tuple_sw_tw}
		end

	new_set_tuple_w: STI_SET_TUPLE [W, EQ_W]
			-- Randomly created monomorphic set-tuple of elements like {W}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_w)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_tuple_w}
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
