﻿note
	description: "Pool of features available for test classes that need one generic parameter and its respective equality type"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Wed 2018-11-21 07:53:55 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	SETS_TESTS_RESOURCES_UNARY [A, EQ_A -> STS_EQUALITY [A] create default_create end]

inherit
	SETS_TESTS_RESOURCES

feature -- Access

	eq_a: EQ_A
			-- Equality for objects like {A}

feature -- Factory (object)

	some_object_a: A
			-- Randomly fetched object like {A}
		deferred
		end

	same_object_a (a: A): A
			-- Object equal (by {EQ_A}) to `a'
			-- TODO: Redefine to same_set_a, same_complex_number, etc.
		do
			Result := a
		ensure
			definition: eq_a (Result, a)
		end

	object_twin_a (a: A): A
			-- Object equal (by value) to `a'
		do
			if attached a then
				Result := a.twin
			else
				Result := a
			end
		ensure
			definition: Result ~ a
		end

feature -- Factory (equality)

	some_equality: STS_EQUALITY [detachable separate ANY]
			-- <Precursor>
		local
			l_new_equality: FUNCTION [like some_equality]
		do
			inspect
				next_random_item \\ 12
			when 0 then
				l_new_equality := agent new_eq_a
			when 1 then
				l_new_equality := agent new_reference_equality_a
			when 2 then
				l_new_equality := agent new_object_equality_a
			when 3 then
				l_new_equality := agent new_reference_equality_aa
			when 4 then
				l_new_equality := agent new_object_equality_aa
			when 5 then
				l_new_equality := agent new_pair_equality_aa
			when 6 then
				l_new_equality := agent new_reference_equality_sa
			when 7 then
				l_new_equality := agent new_object_equality_sa
			when 8 then
				l_new_equality := agent new_set_equality_a
			when 9 then
				l_new_equality := agent new_reference_equality_ta
			when 10 then
				l_new_equality := agent new_object_equality_ta
			when 11 then
				l_new_equality := agent new_n_tuple_equality_a
			end
			check
				eq: attached {like some_equality} some_direct_instance (l_new_equality) as eq -- `some_direct_instance' definition
			then
				Result := eq
			end
		end

	new_eq_a: EQ_A
			-- Randomly created monomorphic equality of type {EQ_A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable EQ_A}
		end

	new_reference_equality_a: STS_REFERENCE_EQUALITY [A]
			-- Randomly created monomorphic reference-equality for objects like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_reference_equality_a}
		end

	new_object_equality_a: STS_OBJECT_EQUALITY [A]
			-- Randomly created monomorphic object-equality for objects like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_object_equality_a}
		end

	new_reference_equality_aa: STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [A, A, EQ_A, EQ_A]]
			-- Randomly created monomorphic reference-equality for ordered pairs like ({A}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_reference_equality_aa}
		end

	new_object_equality_aa: STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [A, A, EQ_A, EQ_A]]
			-- Randomly created monomorphic object-equality for ordered pairs like ({A}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_object_equality_aa}
		end

	new_pair_equality_aa: STS_ORDERED_PAIR_EQUALITY [A, A, EQ_A, EQ_A]
			-- Randomly created monomorphic mathematical equality for ordered pairs like ({A}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_equality_aa}
		end

	new_reference_equality_sa: STS_REFERENCE_EQUALITY [STS_SET [A, EQ_A]]
			-- Randomly created monomorphic reference-equality for sets of elements like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_reference_equality_sa}
		end

	new_object_equality_sa: STS_OBJECT_EQUALITY [STS_SET [A, EQ_A]]
			-- Randomly created monomorphic object-equality for sets of elements like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_object_equality_sa}
		end

	new_set_equality_a: STS_SET_EQUALITY [A, EQ_A]
			-- Randomly created monomorphic mathematical equality for sets of elements like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_equality_a}
		end

	new_reference_equality_ta: STS_REFERENCE_EQUALITY [STS_N_TUPLE [A, EQ_A]]
			-- Randomly created monomorphic reference-equality for n-tuples of terms like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_reference_equality_ta}
		end

	new_object_equality_ta: STS_OBJECT_EQUALITY [STS_N_TUPLE [A, EQ_A]]
			-- Randomly created monomorphic object-equality for n-tuples of terms like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_object_equality_ta}
		end

	new_n_tuple_equality_a: STS_N_TUPLE_EQUALITY [A, EQ_A]
			-- Randomly created monomorphic mathematical equality for n-tuples of terms like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_n_tuple_equality_a}
		end

feature -- Factory (ordered pair)

	some_pair: STS_ORDERED_PAIR [
		detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY], STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_pair_aa
			when 1 then
				Result := some_subset_family_structure_a
			when 2 then
				check
					new_a_ta: attached {FUNCTION [like new_pair_a_ta]} agent new_pair_a_ta as new_a_ta
					a_ta: attached {like new_pair_a_ta} some_direct_instance (new_a_ta) as a_ta -- `some_direct_instance' definition
				then
					Result := a_ta
				end
			end
		end

	some_pair_aa: STS_ORDERED_PAIR [A, A, EQ_A, EQ_A]
			-- Randomly fetched polymorphic ordered pair like ({A}, {A})
		local
			new_aa: FUNCTION [like some_pair_aa]
		do
			inspect
				next_random_item \\ 3
			when 0 then
				new_aa := agent new_pair_aa
			when 1 then
				new_aa := agent new_composable_pair_aaa
			when 2 then
				new_aa := agent new_homogeneous_pair_a
			end
			check
				aa: attached {like some_pair_aa} some_direct_instance (new_aa) as aa -- `some_direct_instance' definition
			then
				Result := aa
			end
		end

	new_pair_aa: STI_ORDERED_PAIR [A, A, EQ_A, EQ_A]
			-- Randomly created monomorphic ordered pair like ({A}, {A})
		do
			create Result.make (some_object_a, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_aa}
		end

	new_composable_pair_aaa: STI_COMPOSABLE_ORDERED_PAIR [A, A, A, EQ_A, EQ_A, EQ_A]
			-- Randomly created monomorphic {A}-composable ordered pair like ({A}, {A})
		do
			create Result.make (some_object_a, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_composable_pair_aaa}
		end

	new_homogeneous_pair_a: STI_HOMOGENEOUS_ORDERED_PAIR [A, EQ_A]
			-- Randomly created monomorphic homogeneous ordered pair like ({A}, {A})
		do
			create Result.make (some_object_a, some_object_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_homogeneous_pair_a}
		end

	new_pair_a_ta: STI_ORDERED_PAIR [A, STS_N_TUPLE [A, EQ_A], EQ_A, STS_N_TUPLE_EQUALITY [A, EQ_A]]
			-- Randomly created monomorphic ordered pair like ({A}, {STS_N_TUPLE [A, ...]})
		do
			create Result.make (some_object_a, some_n_tuple_a)
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_a_ta}
		end

feature -- Factory (set)

	some_set: STI_SET [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		local
			l_new_set: FUNCTION [like some_set]
		do
			inspect
				next_random_item \\ 4
			when 0 then
				inspect
					next_random_item \\ 7
				when 0 then
					l_new_set := agent new_references_a
				when 1 then
					l_new_set := agent new_objects_a
				when 2 then
					l_new_set := agent new_references_aa
				when 3 then
					l_new_set := agent new_objects_aa
				when 4 then
					l_new_set := agent new_math_set_aa
				when 5 then
					l_new_set := agent new_transformable_set_aa_a
				when 6 then
					l_new_set := agent new_set_of_pairs_aa
				end
				check
					s: attached {like some_set} some_direct_instance (l_new_set) as s
				then
					Result := cropped_set (s)
				end
			when 1 then
				Result := some_set_a
			when 2 then
				Result := some_set_sa
			when 3 then
				Result := some_set_ta
			end
		end

	cropped_set (s: STI_SET [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]): like s
			-- `s' striped from as many elements as necessary to keep its cardinality at most `Max_count'
		local
			cropped_s: STI_SET [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			chk_n, chk_cropp: BOOLEAN
			n: like {STI_SET [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]}.cardinality
		do
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n ≤ Max_count then
				Result := s
			else
				if n > max_asserted_elements then
					chk_cropp := {ISE_RUNTIME}.check_assert (False)
				end
					cropped_s := trimmed_n_tuple (s.as_tuple).terms
					check
						is_disjoint: s.o.is_disjoint (cropped_s) -- s.o.is_empty
					end
					Result := s.o.batch_extended (cropped_s)
				if chk_cropp then
					chk_cropp := {ISE_RUNTIME}.check_assert (True)
				end
			end
		ensure
			small_enough: # Result ≤ Max_count
			no_change: # s ≤ Max_count implies Result ≍ s
			cropped: Result ⊆ s
		end

	new_references_a: STI_SET [A, STS_REFERENCE_EQUALITY [A]]
			-- Randomly created monomorphic set of elements like {A}, which are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_object_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_a}
		end

	new_objects_a: STI_SET [A, STS_OBJECT_EQUALITY [A]]
			-- Randomly created monomorphic set of elements like {A}, which are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_object_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_a}
		end

	some_set_a: STI_SET [A, EQ_A]
			-- Randomly fetched polymorphic set of elements like {A}, whose equality is checked by {EQ_A}.
			--| TODO: `some_set_sa' in descendants may be enriched.
		local
			new_s: FUNCTION [like some_set_a]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				new_s := agent new_set_a
			when 1 then
				new_s := agent new_transformable_set_aa
			end
			check
				s: attached {like some_set_a} some_direct_instance (new_s) as s -- `some_direct_instance' definition
			then
				Result := cropped_set (s)
			end
		end

	new_set_a: like some_set_a
			-- Randomly created monomorphic set of elements like {A}, whose equality is checked by {EQ_A}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_object_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_set_a}
		end

	new_transformable_set_aa: STI_TRANSFORMABLE_SET [A, A, EQ_A, EQ_A]
			-- Randomly created monomorphic {A}-transformable set of elements like {A}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_object_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_aa}
		end

	same_set_a (s: STI_SET [A, EQ_A]): like some_set_a
			-- Set mathematically equal to `s', with type randomly chosen.
			-- TODO: `same_set_sa' in descendants may be enriched.
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := s
			when 1 then
				Result := s.twin
			when 2 then
				inspect
					next_random_item \\ 2
				when 0 then
					create {like new_set_a} Result.make_empty
				when 1 then
					create {like new_transformable_set_aa} Result.make_empty
				end
					check
						is_disjoint: Result.is_disjoint (s) -- Result.is_empty
					end
				Result := Result.batch_extended (s) -- TODO: make_from_set instead.
			end
		ensure
			definition: Result ≍ s
		end

	new_references_aa: STI_SET [
		STS_ORDERED_PAIR [A, A, EQ_A, EQ_A],
		STS_REFERENCE_EQUALITY [STS_ORDERED_PAIR [A, A, EQ_A, EQ_A]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({A}, {A}), which are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_references_aa}
		end

	new_objects_aa: STI_SET [
		STS_ORDERED_PAIR [A, A, EQ_A, EQ_A],
		STS_OBJECT_EQUALITY [STS_ORDERED_PAIR [A, A, EQ_A, EQ_A]]
		]
			-- Randomly created monomorphic set of ordered pairs like ({A}, {A}), which are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_objects_aa}
		end

	new_math_set_aa: STI_SET [STS_ORDERED_PAIR [A, A, EQ_A, EQ_A], STS_ORDERED_PAIR_EQUALITY [A, A, EQ_A, EQ_A]]
			-- Randomly created monomorphic set of ordered pairs like ({A}, {A}), which are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_math_set_aa}
		end

	new_transformable_set_aa_a: STI_TRANSFORMABLE_SET [STS_ORDERED_PAIR [A, A, EQ_A, EQ_A], A, STS_ORDERED_PAIR_EQUALITY [A, A, EQ_A, EQ_A], EQ_A]
			-- Randomly created monomorphic {A}-transformable set of ordered pairs like ({A}, {A})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_aa_a}
		end

	new_set_of_pairs_aa: STI_SET_OF_ORDERED_PAIRS [A, A, EQ_A, EQ_A]
			-- Randomly created monomorphic set of pairs like ({A}, {A})
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_pair_aa
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_of_pairs_aa}
		end

	some_set_sa: STI_SET [STS_SET [A, EQ_A], STS_EQUALITY [STS_SET [A, EQ_A]]]
			-- Randomly fetched polymorphic set of sets of elements like {A}
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_references_sa
			when 1 then
				Result := some_objects_sa
			when 2 then
				Result := some_math_set_sa
			end
		end

	some_references_sa: STI_SET [STS_SET [A, EQ_A], STS_REFERENCE_EQUALITY [STS_SET [A, EQ_A]]]
			-- Randomly fetched set of sets of elements like {A}, and whose (member) sets are compared by their object references.
		do
			check
				new_ss: attached {FUNCTION [like new_references_sa]} agent new_references_sa as new_ss
				ss: attached {like new_references_sa} some_direct_instance (new_ss) as ss -- `some_direct_instance' definition
			then
				Result := cropped_set (ss)
			end
		end

	new_references_sa: like some_references_sa
			-- Randomly created monomorphic set of sets of elements like {A}, and whose (member) sets are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_references_sa}
		end

	some_objects_sa: STI_SET [STS_SET [A, EQ_A], STS_OBJECT_EQUALITY [STS_SET [A, EQ_A]]]
			-- Randomly fetched set of sets of elements like {A}, and whose (member) sets are compared by their object values.
		do
			check
				new_ss: attached {FUNCTION [like new_objects_sa]} agent new_objects_sa as new_ss
				ss: attached {like new_objects_sa} some_direct_instance (new_ss) as ss -- `some_direct_instance' definition
			then
				Result := cropped_set (ss)
			end
		end

	new_objects_sa: like some_objects_sa
			-- Randomly created monomorphic set of sets of elements like {A}, and whose (member) sets are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_set_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_objects_sa}
		end

	some_math_set_sa: STI_SET [STS_SET [A, EQ_A], STS_SET_EQUALITY [A, EQ_A]]
			-- Randomly fetched polymorphic set of sets of elements like {A}, and whose (member) sets are compared by their mathematical values.
		local
			new_ss: FUNCTION [like some_math_set_sa]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				inspect
					next_random_item \\ 2
				when 0 then
					new_ss := agent new_math_set_sa
				when 1 then
					new_ss := agent new_transformable_set_sa_ta
				end
				check
					ss: attached {like some_math_set_sa} some_direct_instance (new_ss) as ss -- `some_direct_instance' definitions
				then
					Result := cropped_set (ss)
				end
			when 1 then
				Result := some_set_of_sets_a
			end
		end

	new_math_set_sa: like some_math_set_sa
			-- Randomly created monomorphic set of sets of elements like {A}, and whose (member) sets are compared by their mathematical values.
		local
			s: like some_set_a
			chk_n, chk_with, too_big: BOOLEAN
			n: like some_set.cardinality
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				s := some_set_a
				if not too_big then
					chk_n := {ISE_RUNTIME}.check_assert (False)
						n := # s
					chk_n := {ISE_RUNTIME}.check_assert (chk_n)
					if n > max_asserted_elements then
						chk_with := {ISE_RUNTIME}.check_assert (False)
						too_big := True
					end
				end
				Result := Result & s
			end
			if too_big and chk_with then
				chk_with := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_math_set_sa}
		end

	new_transformable_set_sa_ta: STI_TRANSFORMABLE_SET [
		STS_SET [A, EQ_A], STS_N_TUPLE [A, EQ_A],
		STS_SET_EQUALITY [A, EQ_A], STS_N_TUPLE_EQUALITY [A, EQ_A]
		]
			-- Randomly created monomorphic set of sets of elements like {A}, which is (the outer
			-- set) transformable into a set of n-tuples of terms like {A}.
		local
			s: like some_set_a
			chk_n, chk_with, too_big: BOOLEAN
			n: like some_set.cardinality
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				s := some_set_a
				if not too_big then
					chk_n := {ISE_RUNTIME}.check_assert (False)
						n := # s
					chk_n := {ISE_RUNTIME}.check_assert (chk_n)
					if n > max_asserted_elements then
						chk_with := {ISE_RUNTIME}.check_assert (False)
						too_big := True
					end
				end
				Result := Result & s
			end
			if too_big and chk_with then
				chk_with := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_sa_ta}
		end

	some_set_of_sets_a: STI_SET_OF_SETS [A, EQ_A]
			-- Randomly fetched polymorphic set-of-sets of elements like {A}
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_set_family_a
			when 1 then
				Result := some_subset_family_a
			end
		end

	same_set_of_sets_a (ss: STI_SET_OF_SETS [A, EQ_A]): like some_set_of_sets_a
			-- Set mathematically equal to `ss', with type randomly chosen.
		do
			if attached {like some_subset_family_a} ss as sf then
				Result := same_subset_family_a (sf)
			else
				check
					attached {like some_set_family_a} ss as sf
				then
					Result := same_set_family_a (sf)
				end
			end
		ensure
			definition: Result ≍ ss
		end

	some_set_family_a: STI_SET_FAMILY [A, EQ_A]
			-- Randomly fetched family of sets of elements like {A}
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_set_family_a
			when 1 then
				Result := some_set_partition_a
			end
		end

	some_direct_set_family_a: like some_set_family_a -- TODO: rename descendants
			-- Randomly fetched monomorphic family of sets of elements like {A}
		do
			check
				new_nf: attached {FUNCTION [like new_set_family_a]} agent new_set_family_a as new_sf

					-- `some_direct_instance' and `new_set_family_a' definitions
				sf: attached {like new_set_family_a} some_direct_instance (new_sf) as sf
				monomorphic: sf.generating_type ~ {detachable like new_set_family_a}
			then
				Result := cropped_set (sf)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_set_family_a}
		end

	new_set_family_a: like some_direct_set_family_a
			-- Randomly created monomorphic family of sets of elements like {A}
		local
			s: like some_set_a
			chk_n, chk_with, too_big: BOOLEAN
			n: like some_set.cardinality
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				s := some_set_a
				if not too_big then
					chk_n := {ISE_RUNTIME}.check_assert (False)
						n := # s
					chk_n := {ISE_RUNTIME}.check_assert (chk_n)
					if n > max_asserted_elements then
						chk_with := {ISE_RUNTIME}.check_assert (False)
						too_big := True
					end
				end
				Result := Result & s
			end
			if too_big and chk_with then
				chk_with := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_set_family_a}
		end

	same_set_family_a (sf: STI_SET_FAMILY [A, EQ_A]): like some_set_family_a
			-- Set family mathematically equal to `sf', with type randomly chosen.
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := sf
			when 1 then
				Result := sf.twin
			when 2 then
				if attached {like some_set_partition_a} sf as sp and next_random_item \\ 2 = 0 then
					create {like some_set_partition_a} Result.make_from_partition (sp)
				else
					create Result.make_from_set (sf)
				end
			end
		ensure
			definition: Result ≍ sf
		end

	some_set_partition_a: EXPOSED_SET_PARTITION [A, EQ_A]
			-- Randomly fetched partition of sets of elements like {A}
		do
			Result := some_direct_set_partition_a
		end

	some_direct_set_partition_a: like some_set_partition_a
			-- Randomly fetched monomorphic partition of sets of elements like {A}
		local
			chk_n, chk_crp: BOOLEAN
			n: like some_set.cardinality
		do
			check
				new_sp: attached {FUNCTION [like new_set_partition_a]} agent new_set_partition_a as new_sp

					-- `some_direct_instance' and `new_set_partition_a' definitions
				sp: attached {like new_set_partition_a} some_direct_instance (new_sp) as sp
				monomorphic: sp.generating_type ~ {detachable like new_set_partition_a}
			then
				chk_n := {ISE_RUNTIME}.check_assert (False)
					n := # ⋃ sp
				chk_n := {ISE_RUNTIME}.check_assert (chk_n)
				if n > max_asserted_elements then
					chk_crp := {ISE_RUNTIME}.check_assert (False)
				end
					Result := cropped_set (sp)
				if chk_crp then
					chk_crp := {ISE_RUNTIME}.check_assert (True)
				end
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_set_partition_a}
		end

	new_set_partition_a: like some_direct_set_partition_a
			-- Randomly created monomorphic partition of sets of elements like {A}
		local
			s: like some_set_a
			chk_n, chk_part: BOOLEAN
			n: like some_set.cardinality
		do
			s := some_set_a
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_part := {ISE_RUNTIME}.check_assert (False)
			end
				Result := partitioned_set_a (s)
			if chk_part then
				chk_part := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_set_partition_a}
		end

	partitioned_set_a (s: STI_SET [A, EQ_A]): like new_set_partition_a
			-- Random monomorphic partition of `s'
		local
			s2, si: STI_SET [A, EQ_A]
			j: NATURAL
		do
			from
				create Result.make_empty
				si := s
			invariant
				building_partition_up: (s ∖ si).is_partitioned (Result)
			until
				si.is_empty
			loop
				from
					s2 := s.o
						check
							positive: 0 < # si -- not si.is_empty
						end
					j := some_index_up_to (# si)
				invariant
					building_s2_up: s2 ≍ (s ∖ si ∖ (⋃ Result))
					si_emptiness: j > 0 implies not si.is_empty
					s2_emptiness: j = 0 implies not s2.is_empty
				until
					j = 0
				loop
						check
							si_is_not_empty: not si.is_empty -- j > 0
							s2_does_not_have_si_any: s2 ∌ si.any -- s2 ≍ (s ∖ si...)
						end
					s2 := s2.extended (si.any)
					si := si.others
					j := j - 1
				variant
					j: {like some_set}.natural_as_integer (j)
				end
					check
						s2_is_not_empty: not s2.is_empty -- j = 0 implies not s2.is_empty
						set_is_disjoint: Result.set_is_disjoint (s2) -- s2 ≍ (s - si - (⋃ Result))
					end
				Result := Result.extended (s2)
			variant
				cardinality: {like some_set}.natural_as_integer (# si)
			end
		ensure
			partitioned: s.is_partitioned (Result)
			monomorphic: Result.generating_type ~ {detachable like some_direct_set_partition_a}
		end

	some_subset_family_a: STI_SUBSET_FAMILY [A, EQ_A]
			-- Randomly fetched family of subsets of a set of elements like {A}
		do
			inspect
				next_random_item \\ 2
			when 0 then
				Result := some_direct_subset_family_a
			when 1 then
				Result := some_subset_family_structure_a
			end
		end

	some_direct_subset_family_a: like some_subset_family_a
			-- Randomly fetched monomorphic family of subsets of a set of elements like {A}
		do
			check
				new_sf: attached {FUNCTION [like some_direct_subset_family_a]} (agent new_subset_family_a) as new_sf
				sf: attached {like new_subset_family_a} some_direct_instance (new_sf) as sf -- `some_direct_instance' and `new_subset_family_a' definitions
				monomorphic: sf.generating_type ~ {detachable like new_subset_family_a}
			then
				Result := cropped_set (sf)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_subset_family_a}
		end

	new_subset_family_a: like some_direct_subset_family_a
			-- Randomly created monomorphic family of subsets of a set of elements like {A}
		local
			s: like some_set_a
			chk_n, chk_with, too_big: BOOLEAN
			n: like some_set.cardinality
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty (some_set_a)
			loop
				s := some_set_a
				if not too_big then
					chk_n := {ISE_RUNTIME}.check_assert (False)
						n := # s ∨ # Result.superset
					chk_n := {ISE_RUNTIME}.check_assert (chk_n)
					if n > max_asserted_elements then
						chk_with := {ISE_RUNTIME}.check_assert (False)
						too_big := True
					end
				end
				Result := Result & s
			end
			if too_big and chk_with then
				chk_with := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_subset_family_a}
		end

	same_subset_family_a (sf: STI_SUBSET_FAMILY [A, EQ_A]): like some_subset_family_a
			-- Subset family mathematically equal to `sf', with type randomly chosen.
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := sf
			when 1 then
				Result := sf.twin
			when 2 then
				inspect
					next_random_item \\ 2
				when 0 then
					create {like new_subset_family_a} Result.make_empty (same_set_a (sf.superset))
				when 1 then
					create {like new_subset_family_structure_a} Result.make_empty (same_set_a (sf.superset))
				end
					check
						is_disjoint: Result.is_disjoint (sf) -- Result.is_empty
						includes_superset: sf.superset ⊆ Result.superset -- Result.superset ≍ sf.superset
					end
				Result := Result.batch_extended (sf)
			end
		ensure
			definition: Result.same_family (sf)
		end

	some_subset_family_structure_a: EXPOSED_SUBSET_FAMILY_STRUCTURE [A, EQ_A]
			-- Randomly fetched structure of a family of subsets of a set of elements like {A}
		do
			Result := some_direct_subset_family_structure_a
		end

	some_direct_subset_family_structure_a: like some_subset_family_structure_a
			-- Randomly fetched monomorphic structure of a family of subsets of a set of elements like {A}
		do
			check
				new_sfs: attached {FUNCTION [like new_subset_family_structure_a]} (agent new_subset_family_structure_a) as new_sfs

					-- `some_direct_instance' and `new_subset_family_structure_a' definitions
				sfs: attached {like new_subset_family_structure_a} some_direct_instance (new_sfs) as sfs
				monomorphic: sfs.generating_type ~ {detachable like new_subset_family_structure_a}
			then
				Result := cropped_set (sfs)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_subset_family_structure_a}
		end

	new_subset_family_structure_a: like some_direct_subset_family_structure_a
			-- Randomly created monomorphic structure of a family of subsets of a set of elements like {A}
		local
			sf: like some_direct_subset_family_a
			chk_n, chk_mk: BOOLEAN
			n: like some_set.cardinality
		do
			sf := some_direct_subset_family_a
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # sf.superset
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_mk := {ISE_RUNTIME}.check_assert (False)
			end
				create Result.make_from_family (sf) -- TODO: use conversion.
			if chk_mk then
				chk_mk := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_direct_subset_family_structure_a}
		end

	same_subset_family_structure_a (sf: EXPOSED_SUBSET_FAMILY_STRUCTURE [A, EQ_A]): like some_subset_family_structure_a
			-- Subset family structure mathematically equal to `sf`, with type randomly chosen.
		do
			inspect next_random_item \\ 5
			when 0 then
				Result := sf
			when 1 then
				Result := sf.twin
			when 2 then
				create Result.make_from_set (sf)
			when 3 then
				create Result.make_from_family (sf)
			when 4 then
				create {like new_subset_family_structure_a} Result.make_empty (same_set_a (sf.superset))
					check
						is_disjoint: Result.is_disjoint (sf) -- Result.is_empty
						includes_superset: sf.superset ⊆ Result.superset -- Result.superset ≍ sf.superset
					end
				Result := Result.batch_extended (sf) -- TODO: get "same sets" instead.
			end
--			if Result /= sf then
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
--			end
		ensure
			definition: Result ≍ sf
		end

	some_set_ta: STI_SET [STS_N_TUPLE [A, EQ_A], STS_EQUALITY [STS_N_TUPLE [A, EQ_A]]]
			-- Randomly fetched polymorphic set of n-tuples of terms like {A}
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := some_references_ta
			when 1 then
				Result := some_objects_ta
			when 2 then
				Result := some_math_set_ta
			end
		end

	some_references_ta:
			STI_SET [STS_N_TUPLE [A, EQ_A], STS_REFERENCE_EQUALITY [STS_N_TUPLE [A, EQ_A]]]
			-- Randomly fetched set of n-tuples of terms like {A}, and whose n-tuples are compared by their object references.
		do
			check
				ts: attached {like new_references_ta} some_direct_instance (agent new_references_ta) as ts -- `some_direct_instance' definitions
			then
				Result := cropped_set (ts)
			end
		end

	new_references_ta: like some_references_ta
			-- Randomly created monomorphic set of n-tuples of terms like {A}, and whose n-tuples are compared by their object references.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_references_ta}
		end

	some_objects_ta: STI_SET [STS_N_TUPLE [A, EQ_A], STS_OBJECT_EQUALITY [STS_N_TUPLE [A, EQ_A]]]
			-- Randomly fetched set of n-tuples of terms like {A}, and whose n-tuples are compared by their object values.
		do
			check
				ts: attached {like new_objects_ta} some_direct_instance (agent new_objects_ta) as ts -- `some_direct_instance' definitions
			then
				Result := cropped_set (ts)
			end
		end

	new_objects_ta: like some_objects_ta
			-- Randomly created monomorphic set of n-tuples of terms like {A}, and whose n-tuples are compared by their object values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_objects_ta}
		end

	some_math_set_ta: STI_SET [STS_N_TUPLE [A, EQ_A], STS_N_TUPLE_EQUALITY [A, EQ_A]]
			-- Randomly fetched polymorphic set of n-tuples of terms like {A}, and whose n-tuples are compared by their mathematical values.
		local
			l_new_set_ta: FUNCTION [like some_math_set_ta]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				l_new_set_ta := agent new_math_set_ta
			when 1 then
				l_new_set_ta := agent new_transformable_set_ta_sa
			end
			check
				ts: attached {like some_math_set_ta} some_direct_instance (l_new_set_ta) as ts -- `some_direct_instance' definition
			then
				Result := cropped_set (ts)
			end
		end

	new_math_set_ta: like some_math_set_ta
			-- Randomly created monomorphic set of n-tuples of terms like {A}, and whose n-tuples are compared by their mathematical values.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_math_set_ta}
		end

	new_transformable_set_ta_sa: STI_TRANSFORMABLE_SET [
		STS_N_TUPLE [A, EQ_A], STS_SET [A, EQ_A],
		STS_N_TUPLE_EQUALITY [A, EQ_A], STS_SET_EQUALITY [A, EQ_A]
		]
			-- Randomly created set of n-tuples of terms like {A}, which is (the outer set) transformable into a set of sets of elements like {A}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_empty
			loop
				Result := Result & some_n_tuple_a
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_set_ta_sa}
		end

feature -- Factory (pair composer)

	some_pair_composer: STI_ORDERED_PAIR_COMPOSER [
		detachable separate ANY, detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY],
		STS_EQUALITY [detachable separate ANY],
		STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		do
			check
				pc: attached {like new_pair_composer_aaa} some_direct_instance (agent new_pair_composer_aaa) as pc -- `some_direct_instance' definition
			then
				Result := pc
			end
		end

	new_pair_composer_aaa: STI_ORDERED_PAIR_COMPOSER [A, A, A, EQ_A, EQ_A, EQ_A]
			-- Randomly created monomorphic composer for ordered pairs like ({A}, {A}) and ({A}, {A})
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_pair_composer_aaa}
		end

feature -- Factory (transformer)

	some_transformer: STI_TRANSFORMER [
		detachable separate ANY, detachable separate ANY,
		STS_EQUALITY [detachable separate ANY], STS_EQUALITY [detachable separate ANY]
		]
			-- <Precursor>
		do
			check
				tr: attached {like some_transformer} some_direct_instance (agent new_transformer_aa) as tr -- `some_direct_instance' definition
			then
				Result := tr
			end
		end

	new_transformer_aa: STI_TRANSFORMER [A, A, EQ_A, EQ_A]
			-- Randomly created monomorphic transformer from objects like {A} to objects like {A}
		do
			create Result
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformer_aa}
		end

feature -- Factory (n-tuple)

	some_n_tuple: STI_N_TUPLE [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]
			-- <Precursor>
		local
			l_new_n_tuple: FUNCTION [like some_n_tuple]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				inspect
					next_random_item \\ 3
				when 0 then
					l_new_n_tuple := agent new_n_tuple_sa
				when 1 then
					l_new_n_tuple := agent new_transformable_tuple_sa_ta
				when 2 then
					l_new_n_tuple := agent new_set_tuple_a
				end
				check
					t: attached {like some_n_tuple} some_direct_instance (l_new_n_tuple) as t -- `some_direct_instance' definitions
				then
					Result := trimmed_n_tuple (t)
				end
			when 1 then
				Result := some_n_tuple_a
			end
		end

	trimmed_n_tuple (t: STI_N_TUPLE [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]): like t
			-- `t' striped from as many terms as necessary to keep its `n'-arity at most `Max_count'
		local
			chk_n, chk_trimm: BOOLEAN
			n: like {STI_N_TUPLE [detachable separate ANY, STS_EQUALITY [detachable separate ANY]]}.n
		do
			Result := t
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := t.n
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_trimm := {ISE_RUNTIME}.check_assert (False)
			end
				if Result.n > Max_count then
					from
						if next_random_item \\ 2 = 0 then
							Result := Result.prefix (next_random_item.as_natural_32 \\ (Result.n + 1))
						else
							Result := Result.suffix (next_random_item.as_natural_32 \\ (Result.n + 1))
						end
					until
						Result.n ≤ Max_count
					loop
						if next_random_item \\ 2 = 0 then
							Result := Result.prefix (next_random_item.as_natural_32 \\ (Result.n + 1))
						else
							Result := Result.suffix (next_random_item.as_natural_32 \\ (Result.n + 1))
						end
					end
				end
			if chk_trimm then
				chk_trimm := {ISE_RUNTIME}.check_assert (True)
			end
		ensure
			small_enough: Result.n ≤ Max_count
			no_change: t.n ≤ Max_count implies Result ≍ t
			trimmed: Result.terms ⊆ t.terms
		end

	some_n_tuple_a: STI_N_TUPLE [A, EQ_A]
			-- Randomly fetched polymorphic n-tuple of terms like {A}
		local
			new_t: FUNCTION [like some_n_tuple_a]
		do
			inspect
				next_random_item \\ 2
			when 0 then
				new_t := agent new_n_tuple_a
			when 1 then
				new_t := agent new_transformable_tuple_aa
			end
			check
				t: attached {like some_n_tuple_a} some_direct_instance (new_t) as t -- `some_direct_instance' definition
			then
				Result := trimmed_n_tuple (t)
			end
		end

	new_n_tuple_a: like some_n_tuple_a
			-- Randomly created monomorphic n-tuple of terms like {A}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_object_a)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like some_n_tuple_a}
		end

	new_transformable_tuple_aa: STI_TRANSFORMABLE_TUPLE [A, A, EQ_A, EQ_A]
			-- Randomly created monomorphic {A}-transformable n-tuple of terms like {A}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_object_a)
			end
		ensure
			monomorphic:
				Result.generating_type ~ {detachable like new_transformable_tuple_aa}
		end

	same_n_tuple_a (t: STI_N_TUPLE [A, EQ_A]): like some_n_tuple_a
			-- N-tuple mathematically equal to `t', with type randomly chosen.
		do
			inspect
				next_random_item \\ 3
			when 0 then
				Result := t
			when 1 then
				Result := t.twin
			when 2 then
				inspect
					next_random_item \\ 2
				when 0 then
					create {like new_n_tuple_a} Result.make_unit
				when 1 then
					create {like new_transformable_tuple_aa} Result.make_unit
				end
				Result := Result + t
			end
--			if Result /= t then
--					check
--						does_not_have: object_pool ∌ Result -- `Result' is a new reference.
--					end
--				set_object_pool (object_pool.extended (Result))
--			end
		ensure
			definition: Result ≍ t
		end

	new_n_tuple_sa: STI_N_TUPLE [STS_SET [A, EQ_A], STS_SET_EQUALITY [A, EQ_A]]
			-- Randomly created monomorphic n-tuple of sets of elements like {A}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_a)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_n_tuple_sa}
		end

	new_transformable_tuple_sa_ta: STI_TRANSFORMABLE_TUPLE [
		STS_SET [A, EQ_A], STS_N_TUPLE [A, EQ_A],
		STS_SET_EQUALITY [A, EQ_A], STS_N_TUPLE_EQUALITY [A, EQ_A]
		]
			-- Randomly created monomorphic n-tuple of sets of elements like {A}, which is (the outer n-tuple) transformable into an n-tuple of n-tuples of terms like {A}.
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_a)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_transformable_tuple_sa_ta}
		end

	new_set_tuple_a: STI_SET_TUPLE [A, EQ_A]
			-- Randomly created monomorphic set-tuple of elements like {A}
		do
			across
				1 |..| some_count.as_integer_32 as i
			from
				create Result.make_unit
			loop
				Result := Result.after (some_set_a)
			end
		ensure
			monomorphic: Result.generating_type ~ {detachable like new_set_tuple_a}
		end

feature -- Predicate
	-- TODO:explain.

	p_x (x: A): BOOLEAN
			-- Predicate over `x'
		deferred
		end

	p_i_x (i: NATURAL; x: A): BOOLEAN
			-- Predicate over `i' and `x'
		do
			Result := i \\ 2 = 0 xor p_x (x)
		end

	p_x1_x2 (x1, x2: A): BOOLEAN
			-- Predicate over (`x1', `x2')
		deferred
		end

feature -- Mapping
	-- TODO:explain.

	f_x (x: A): A
			-- Function that maps `x' to a value of same kind
		deferred
		end

feature -- Reduction
	-- TODO:explain.

	f_acc_x (acc, x: A): A
			-- Function that reduces (`acc', `x') to a value of same kind
		deferred
		end

	f_x_acc (x, acc: A): A
			-- Function that reduces (`x', `acc') to a value of same kind
		do
			Result := f_acc_x (acc, x)
		end

	f_acc_i_x (acc: A; i: NATURAL; x: A): A
			-- Function that reduces (`acc', `i', `x') to a value like `acc'
		deferred
		end

	f_i_x_acc (i: NATURAL; x, acc: A): A
			-- Function that reduces (`i', `x', `acc') to a value like `acc'
		do
			Result := f_acc_i_x (acc, i, x)
		end

feature {SET_PROPERTIES} -- Implementation

	max_asserted_elements: NATURAL
			-- Maximum number of elements within a set to be checked with assertions enabled
		deferred
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
