﻿note
	description: "Test suite for {STS_ELEMENT}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	ELEMENT_TESTS

inherit
	SETS_TESTS_RESOURCES

feature -- Access

	properties: STP_ELEMENT_PROPERTIES
			-- Object that checks the set-theory properties of {STS_ELEMENT}
		deferred
		end

feature -- Test routines (All)

	test_all
			-- Test {STS_ELEMENT}.
			-- Intended to be run on finalized mode.
		note
			testing: "covers/{STS_ELEMENT}"
		do
			test_is_in
			test_is_not_in
		end

feature -- Test routines (Membership)

	test_is_in
			-- Test {STS_ELEMENT}.is_in.
		note
			testing: "covers/{STS_ELEMENT}.is_in"
		local
			a, a_twin: like element_to_be_tested
			s: like some_elements
--			chk_with: BOOLEAN
		do
			a := element_to_be_tested

--			chk_with := {ISE_RUNTIME}.check_assert (False)
				s := some_element_references & a
--			chk_with := {ISE_RUNTIME}.check_assert (chk_with)

			assert ("a ∈ s", a ∈ s)
			assert ("a ∈ s ok", properties.is_in_ok (a, s))

			s := some_element_references / a
			assert ("not (a ∈ s)", not (a ∈ s))
			assert ("not (a ∈ s) ok", properties.is_in_ok (a, s))

			a := element_to_be_tested
			a_twin := a.twin

--			chk_with := {ISE_RUNTIME}.check_assert (False)
				s := some_element_objects & a
--			chk_with := {ISE_RUNTIME}.check_assert (chk_with)

			assert ("a_twin ∈ s", a_twin ∈ s)
			assert ("a_twin ∈ s ok", properties.is_in_ok (a_twin, s))

			s := some_element_objects / a
			assert ("not (a_twin ∈ s)", not (a_twin ∈ s))
			assert ("not (a_twin ∈ s) ok", properties.is_in_ok (a_twin, s))

			a := element_to_be_tested
			s := some_elements
			assert ("some_elements", a ∈ s ⇒ True)
			assert ("is_in_ok", properties.is_in_ok (a, s))
		end

	test_is_not_in
			-- Test {STS_ELEMENT}.is_not_in.
		note
			testing: "covers&{STS_ELEMENT}.is_not_in"
		local
			a, a_twin: like element_to_be_tested
			s: like some_elements
--			chk_with: BOOLEAN
		do
			a := element_to_be_tested

--			chk_with := {ISE_RUNTIME}.check_assert (False)
				s := some_element_references / a
--			chk_with := {ISE_RUNTIME}.check_assert (chk_with)

			assert ("a ∉ s", a ∉ s)
			assert ("a ∉ s ok", properties.is_not_in_ok (a, s))

			s := some_element_references & a
			assert ("not (a ∉ s)", not (a ∉ s))
			assert ("not (a ∉ s) ok", properties.is_not_in_ok (a, s))

			a := element_to_be_tested
			a_twin := a.twin

--			chk_with := {ISE_RUNTIME}.check_assert (False)
				s := some_element_objects / a
--			chk_with := {ISE_RUNTIME}.check_assert (chk_with)

			assert ("a_twin ∉ s", a_twin ∉ s)
			assert ("a_twin ∉ s ok", properties.is_not_in_ok (a_twin, s))

			s := some_element_objects & a
			assert ("not (a_twin ∉ s)", not (a_twin ∉ s))
			assert ("not (a_twin ∉ s) ok", properties.is_not_in_ok (a_twin, s))

			a := element_to_be_tested
			s := some_elements
			assert ("some_elements", a ∉ s ⇒ True)
			assert ("is_not_in_ok", properties.is_not_in_ok (a, s))
		end

feature {NONE} -- Factory (element to be tested)

	element_to_be_tested: like some_direct_element
			-- Element meant to be under tests
		do
			Result := some_direct_element
		ensure
			monomorphic: Result.generating_type ~ {detachable like element_to_be_tested}
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
