note
	description: "Test suite for {STS_EQUALITY}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	EQUALITY_TESTS [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	ELEMENT_TESTS
		rename
			element_to_be_tested as equality_to_be_tested
		redefine
			equality_to_be_tested
		end

	SETS_TESTS_RESOURCES_UNARY [A, EQ]

feature -- Access

	properties: EQUALITY_PROPERTIES [A, EQ]
			-- Object that checks the set-theory properties of {STS_EQUALITY}
		deferred
		end

feature -- Test routines (Relationship)

	test_holds
			-- Test {STS_EQUALITY}.holds.
		note
			testing: "covers/{STS_EQUALITY}.holds"
		local
			eq: like equality_to_be_tested
			a1, a2: A
		do
			eq := equality_to_be_tested
			a1 := some_object_a
			assert ("same_entity", eq (a1, a1))
			assert ("same_entity_ok", properties.holds_ok (a1, a1, eq))

			eq := equality_to_be_tested
			a1 := some_object_a
			a2 := some_object_a
			assert ("some_object", eq (a1, a2) implies True)
			assert ("some_objects_ok", properties.holds_ok (a1, a2, eq))
		end

	test_holds_successively
			-- Test {STS_EQUALITY}.holds_successively.
		note
			testing: "covers/{STS_EQUALITY}.holds_successively"
		local
			eq: like equality_to_be_tested
			a1, a2, a3: A
		do
			eq := equality_to_be_tested
			a1 := some_object_a
			assert ("same_entity", eq.holds_successively (a1, a1, a1))
			assert ("same_entity_ok", properties.holds_successively_ok (a1, a1, a1, eq))
			
			eq := equality_to_be_tested
			a1 := some_object_a
			a2 := some_object_a
			a3 := some_object_a
			assert ("some_object", eq.holds_successively (a1, a2, a3) implies True)
			assert ("some_objects_ok", properties.holds_successively_ok (a1, a2, a3, eq))
		end

feature {NONE} -- Factory (element to be tested)

	equality_to_be_tested: EQ
			-- Equality meant to be under tests
		do
			check
				new_eq: attached {FUNCTION [EQ]} agent new_eq_a as new_eq
				eq: attached {EQ} some_direct_instance (new_eq) as eq -- `some_direct_instance' and `new_eq_a' definitions
				monomorphic: eq.generating_type ~ {detachable EQ}
			then
				Result := eq
			end
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
