﻿note
	description: "Test suite for {STI_TRANSFORMER}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	TRANSFORMER_TESTS [
		A, B,
		EQ_A -> STS_EQUALITY [A] create default_create end,
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	ELEMENT_TESTS
		rename
			element_to_be_tested as transformer_to_be_tested
		redefine
			transformer_to_be_tested
		end

	SETS_TESTS_RESOURCES_BINARY [A, B, EQ_A, EQ_B]

feature -- Access

	properties: TRANSFORMER_PROPERTIES [A, B, EQ_A, EQ_B]
			-- Object that checks the set-theory properties of {STI_TRANSFORMER}
		deferred
		end

feature -- Test routines (Operation)

	test_cartesian_product
			-- Test {STI_TRANSFORMER}.cartesian_product.
		note
			testing: "covers/{STI_TRANSFORMER}.cartesian_product"
		local
			tr: like transformer_to_be_tested
			s1: like some_set_a
			s2: like some_set_b
		do
			tr := transformer_to_be_tested
			s1 := some_set_a
			s2 := some_set_b
			assert ("cartesian_product", attached tr.cartesian_product (s1, s2))
			assert ("cartesian_product_ok", properties.cartesian_product_ok (tr, s1, s2))
		end

	test_singleton_crossed_set
			-- Test {STI_TRANSFORMER}.singleton_crossed_set.
		note
			testing: "covers/{STI_TRANSFORMER}.singleton_crossed_set"
		local
			tr: like transformer_to_be_tested
			x: A
			s2: like some_set_b
		do
			tr := transformer_to_be_tested
			x := some_object_a
			s2 := some_set_b
			assert ("singleton_crossed_set", attached tr.singleton_crossed_set (x, s2))
			assert ("singleton_crossed_set_ok", properties.singleton_crossed_set_ok (tr, x, s2))
		end

feature -- Test routines (Transformation)

	test_set_map
			-- Test {STI_TRANSFORMER}.set_map.
		note
			testing: "covers/{STI_TRANSFORMER}.set_map"
		local
			tr: like transformer_to_be_tested
			s: like some_set_a
			f: FUNCTION [A, B]
		do
			tr := transformer_to_be_tested
			s := some_set_a
			f := agent f_x
			assert ("set_map", attached tr.set_map (s, f))
			assert ("set_map_ok", properties.set_map_ok (tr, s, f))
		end

	test_set_reduction
			-- Test {STI_TRANSFORMER}.set_reduction.
		note
			testing: "covers/{STI_TRANSFORMER}.set_reduction"
		local
			tr: like transformer_to_be_tested
			s: like some_set_a
			start: like some_object_b
			f: FUNCTION [B, A, B]
		do
			tr := transformer_to_be_tested
			s := some_set_a
			start := some_object_b
			f := agent f_acc_x
			assert ("set_reduction", attached tr.set_reduction (s, start, f) implies True)
			assert ("set_reduction_ok", properties.set_reduction_ok (tr, s, start, f))
		end

	test_tuple_map
			-- Test {STI_TRANSFORMER}.tuple_map.
		note
			testing: "covers/{STI_TRANSFORMER}.tuple_map"
		local
			tr: like transformer_to_be_tested
			t: like some_n_tuple_a
			f: FUNCTION [A, B]
		do
			tr := transformer_to_be_tested
			t := some_n_tuple_a
			f := agent f_x
			assert ("tuple_map", attached tr.tuple_map (t, f))
			assert ("tuple_map_ok", properties.tuple_map_ok (tr, t, f))
		end

	test_tuple_left_reduction
			-- Test {STI_TRANSFORMER}.tuple_left_reduction.
		note
			testing: "covers/{STI_TRANSFORMER}.tuple_left_reduction"
		local
			tr: like transformer_to_be_tested
			t: like some_n_tuple_a
			start: like some_object_b
			f: FUNCTION [B, A, B]
		do
			tr := transformer_to_be_tested
			t := some_n_tuple_a
			start := some_object_b
			f := agent f_acc_x
			assert ("tuple_left_reduction", attached tr.tuple_left_reduction (t, start, f) implies True)
			assert ("tuple_left_reduction_ok", properties.tuple_left_reduction_ok (tr, t, start, f))
		end

	test_tuple_indexed_left_reduction
			-- Test {STI_TRANSFORMER}.tuple_indexed_left_reduction.
		note
			testing: "covers/{STI_TRANSFORMER}.tuple_indexed_left_reduction"
		local
			tr: like transformer_to_be_tested
			t: like some_n_tuple_a
			start: like some_object_b
			i: NATURAL
			f: FUNCTION [B, NATURAL, A, B]
		do
			tr := transformer_to_be_tested
			t := some_n_tuple_a
			start := some_object_b
			i := some_natural
			f := agent f_acc_i_x
			assert ("tuple_indexed_left_reduction", attached tr.tuple_indexed_left_reduction (t, start, i, f) implies True)
			assert ("tuple_indexed_left_reduction_ok", properties.tuple_indexed_left_reduction_ok (tr, t, start, i, f))
		end

	test_tuple_right_reduction
			-- Test {STI_TRANSFORMER}.tuple_right_reduction.
		note
			testing: "covers/{STI_TRANSFORMER}.tuple_right_reduction"
		local
			tr: like transformer_to_be_tested
			t: like some_n_tuple_a
			f: FUNCTION [A, B, B]
			finish: like some_object_b
		do
			tr := transformer_to_be_tested
			t := some_n_tuple_a
			f := agent f_x_acc
			finish := some_object_b
			assert ("tuple_right_reduction",
			attached tr.tuple_right_reduction (t, f, finish) implies True)
			assert ("tuple_right_reduction_ok", properties.tuple_right_reduction_ok (tr, t, f, finish))
		end

	test_tuple_indexed_right_reduction
			-- Test {STI_TRANSFORMER}.tuple_indexed_right_reduction.
		note
			testing: "covers/{STI_TRANSFORMER}.tuple_indexed_right_reduction"
		local
			tr: like transformer_to_be_tested
			t: like some_n_tuple_a
			i: NATURAL
			f: FUNCTION [NATURAL, A, B, B]
			finish: like some_object_b
		do
			tr := transformer_to_be_tested
			t := some_n_tuple_a
			i := some_natural
			f := agent f_i_x_acc
			finish := some_object_b
			assert ("tuple_indexed_right_reduction", attached tr.tuple_indexed_right_reduction (t, i, f, finish) implies True)
			assert ("tuple_indexed_right_reduction_ok", properties.tuple_indexed_right_reduction_ok (tr, t, i, f, finish))
		end

feature -- Test routines (Conversion)

	test_natural_as_integer
			-- Test {STI_TRANSFORMER}.natural_as_integer.
		note
			testing: "covers/{STI_TRANSFORMER}.natural_as_integer"
		do
			assert ("natural_as_integer", attached {like transformer_to_be_tested}.natural_as_integer (some_natural))
		end

feature -- Test routines (Factory)

	test_pair
			-- Test {STI_TRANSFORMER}.pair.
		note
			testing: "covers/{STI_TRANSFORMER}.pair"
		local
			tr: like transformer_to_be_tested
			x: A
			y: B
		do
			tr := transformer_to_be_tested
			x := some_object_a
			y := some_object_b
			assert ("pair", attached tr.pair (x, y))
			assert ("pair_ok", properties.pair_ok (tr, x, y))
		end

	test_o
			-- Test {STI_TRANSFORMER}.o.
		note
			testing: "covers/{STI_TRANSFORMER}.o"
		local
			tr: like transformer_to_be_tested
		do
			tr := transformer_to_be_tested
			assert ("o", attached tr.o)
			assert ("o_ok", properties.o_ok (tr))
		end

	test_empty_product
			-- Test {STI_TRANSFORMER}.empty_product.
		note
			testing: "covers/{STI_TRANSFORMER}.empty_product"
		local
			tr: like transformer_to_be_tested
		do
			tr := transformer_to_be_tested
			assert ("empty_product", attached tr.empty_product)
			assert ("empty_product_ok", properties.empty_product_ok (tr))
		end

feature -- Test routines (Predicate)

	test_anded
			-- Test {STS_TRANSFORMER}.anded.
			-- Test {STI_TRANSFORMER}.anded.
		note
			testing: "covers/{STS_TRANSFORMER}.anded"
			testing: "covers/{STI_TRANSFORMER}.anded"
		local
			p: PREDICATE [A];
			a: A;
			q: PREDICATE [B];
			b: B
		do
			p := agent p_x
			q := agent p_y
			from
				a := some_object_a
				b := some_object_b
			until
				p (a) and q (b)
			loop
				a := some_object_a
				b := some_object_b
			end
			assert ("anded_p_a_q_b", {like transformer_to_be_tested}.anded (p, a, q, b))
			a := some_object_a
			b := some_object_b
			assert ("anded", {like transformer_to_be_tested}.anded (p, a, q, b) implies True)
		end

	test_every_pair_is_in_set
			-- Test {STS_TRANSFORMER}.every_pair_is_in_set.
			-- Test {STI_TRANSFORMER}.every_pair_is_in_set.
		note
			testing: "covers/{STS_TRANSFORMER}.every_pair_is_in_set"
			testing: "covers/{STI_TRANSFORMER}.every_pair_is_in_set"
		local
			a: A
			bs: like some_set_b
			ps: like some_set_of_pairs_ab
		do
			a := some_object_a
			bs := some_set_b
			ps := {like transformer_to_be_tested}.singleton_crossed_set (a, bs)
			assert ("every_ab_is_in_ps", {like transformer_to_be_tested}.every_pair_is_in_set (a, bs, ps))
			a := some_object_a
			bs := some_set_b
			ps := some_set_of_pairs_ab
			assert ("every_pair_is_in_set", {like transformer_to_be_tested}.every_pair_is_in_set (a, bs, ps) implies True)
		end

	test_pair_is_in_set
			-- Test {STS_TRANSFORMER}.pair_is_in_set.
			-- Test {STI_TRANSFORMER}.pair_is_in_set.
		note
			testing: "covers/{STS_TRANSFORMER}.pair_is_in_set"
			testing: "covers/{STI_TRANSFORMER}.pair_is_in_set"
		local
			ps: like some_set_of_pairs_ab
			ab: like some_pair_ab
			a: A;
			b: B
			chk_union: BOOLEAN
		do
			ab := some_pair_ab
			a := ab.a
			b := ab.b

			chk_union := {ISE_RUNTIME}.check_assert (False)
				ps := ab.as_singleton ∪ some_set_of_pairs_ab
			chk_union := {ISE_RUNTIME}.check_assert (chk_union)

			assert ("ab_is_in_ps", {like transformer_to_be_tested}.pair_is_in_set (a, b, ps))
			a := some_object_a
			b := some_object_b
			ps := some_set_of_pairs_ab
			assert ("pair_is_in_set", {like transformer_to_be_tested}.pair_is_in_set (a, b, ps) implies True)
		end

	test_target_set_has_value
			-- Test {STS_TRANSFORMER}.target_set_has_value.
			-- Test {STI_TRANSFORMER}.target_set_has_value.
		note
			testing: "covers/{STS_TRANSFORMER}.target_set_has_value"
			testing: "covers/{STI_TRANSFORMER}.target_set_has_value"
		local
			f: FUNCTION [A, B]
			a: A
			bs: like some_set_b
			chk_ops: BOOLEAN
		do
			f := agent f_x
			a := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				bs := some_set_b & f (a)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("bs_has_f_a", {like transformer_to_be_tested}.target_set_has_value (bs, f, a))
			a := some_object_a
			bs := some_set_b
			assert ("target_set_has_value", {like transformer_to_be_tested}.target_set_has_value (bs, f, a) implies True)
		end

	test_source_set_has_argument
			-- Test {STS_TRANSFORMER}.source_set_has_argument.
			-- Test {STI_TRANSFORMER}.source_set_has_argument.
		note
			testing: "covers/{STS_TRANSFORMER}.source_set_has_argument"
			testing: "covers/{STI_TRANSFORMER}.source_set_has_argument"
		local
			f: FUNCTION [A, B]
			a: A;
			b: B
			a_s: like some_set_a
			chk_with: BOOLEAN
		do
			f := agent f_x
			a := some_object_a

			chk_with := {ISE_RUNTIME}.check_assert (False)
				a_s := some_set_a & a
			chk_with := {ISE_RUNTIME}.check_assert (chk_with)

			b := f (a)
			assert ("a_s_has_b_argument", {like transformer_to_be_tested}.source_set_has_argument (f, a_s, b))

			a_s := some_set_a
			b := some_object_b
			assert ("source_set_has_argument", {like transformer_to_be_tested}.source_set_has_argument (f, a_s, b) implies True)
		end

	test_argument_is_mapped_to_value
			-- Test {STS_TRANSFORMER}.argument_is_mapped_to_value.
			-- Test {STI_TRANSFORMER}.argument_is_mapped_to_value.
		note
			testing: "covers/{STS_TRANSFORMER}.argument_is_mapped_to_value"
			testing: "covers/{STI_TRANSFORMER}.argument_is_mapped_to_value"
		local
			f: FUNCTION [A, B]
			a: A;
			b: B
		do
			f := agent f_x
			a := some_object_a
			b := f (a)
			assert ("a_is_mapped_to_b", {like transformer_to_be_tested}.argument_is_mapped_to_value (f, a, b))

			a := some_object_a
			b := some_object_b
			assert ("argument_is_mapped_to_value", {like transformer_to_be_tested}.argument_is_mapped_to_value (f, a, b) implies True)
		end

	test_target_tuple_has_value
			-- Test {STS_TRANSFORMER}.target_tuple_has_value.
			-- Test {STI_TRANSFORMER}.target_tuple_has_value.
		note
			testing: "covers/{STS_TRANSFORMER}.target_tuple_has_value"
			testing: "covers/{STI_TRANSFORMER}.target_tuple_has_value"
		local
			f: FUNCTION [A, B]
			a: A
			b: B
			bs: like some_set_b
			bt: like some_set_b.as_tuple
			i: NATURAL
			chk_ops: BOOLEAN
		do
			f := agent f_x
			a := some_object_a
			b := f (a)

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				bs := some_set_b & b
				bt := bs.as_tuple
				i := bt.term_to_index.tuple_indexed_left_reduction (
					bt, 1, 1, agent (acc, j: NATURAL; y_j, ia_b: B): NATURAL
						do
							if {like transformer_to_be_tested}.eq_b (y_j, ia_b) then
								Result := j
							else
								Result := acc
							end
						end (?, ?, ?, b)
					)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("bt_has_f_a", {like transformer_to_be_tested}.target_tuple_has_value (bt, i, f, a))

			a := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				bt := some_set_b.as_tuple
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			i := some_index
			assert ("target_tuple_has_value", {like transformer_to_be_tested}.target_tuple_has_value (bt, i, f, a) implies True)
		end

	test_source_tuple_has_argument
			-- Test {STS_TRANSFORMER}.source_tuple_has_argument.
			-- Test {STI_TRANSFORMER}.source_tuple_has_argument.
		note
			testing: "covers/{STS_TRANSFORMER}.source_tuple_has_argument"
			testing: "covers/{STI_TRANSFORMER}.source_tuple_has_argument"
		local
			f: FUNCTION [A, B]
			a: A
			b: B
			a_s: like some_set_a
			a_t: like some_n_tuple_a
			i: NATURAL
			chk_ops: BOOLEAN
		do
			f := agent f_x
			a := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a_s := some_set_a & a
				a_t := a_s.as_tuple
				i := a_t.term_to_index.tuple_indexed_left_reduction (
					a_t, 1, 1, agent  (acc, j: NATURAL; x_j, ia_a: A): NATURAL
						do
							if {like some_n_tuple_a}.eq (x_j, ia_a) then
								Result := j
							else
								Result := acc
							end
						end (?, ?, ?, a)
					)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			b := f (a)
			assert ("a_t_has_b_argument", {like transformer_to_be_tested}.source_tuple_has_argument (f, a_t, i, b))
			
			a_t := some_n_tuple_a
			i := some_index
			b := some_object_b
			assert ("source_tuple_has_argument", {like transformer_to_be_tested}.source_tuple_has_argument (f, a_t, i, b) implies True)
		end

feature -- Test routines (Reduction)

	test_flipped
			-- Test {STI_TRANSFORMER}.flipped.
		note
			testing: "covers/{STI_TRANSFORMER}.flipped"
		do
			assert ("flipped", attached {like transformer_to_be_tested}.flipped (agent f_x_acc, some_object_b, some_object_a) implies True)
		end

	test_indexed_flipped
			-- Test {STI_TRANSFORMER}.indexed_flipped.
		note
			testing: "covers/{STI_TRANSFORMER}.indexed_flipped"
		local
			m: NATURAL
			f: FUNCTION [NATURAL, A, B, B]
			acc: B
			j: NATURAL
			x: A
		do
			f := agent f_i_x_acc
			acc := some_object_b
			x := some_object_a
			from
				m := some_natural
				j := some_natural
			until
				j < m
			loop
				m := some_natural
				j := some_natural
			end
			assert ("indexed_flipped", attached {like transformer_to_be_tested}.indexed_flipped (m, f, acc, j, x) implies True)
		end

	test_flipped_index_is_small_enough
			-- Test {STI_TRANSFORMER}.flipped_index_is_small_enough.
		note
			testing: "covers/{STI_TRANSFORMER}.flipped_index_is_small_enough"
		local
			m: NATURAL
			acc: BOOLEAN
			j: NATURAL
			x: A
		do
			acc := True
			x := some_object_a
			from
				m := some_natural
				j := some_natural
			until
				j < m
			loop
				m := some_natural
				j := some_natural
			end
			assert ("flipped_j_is_small_enough", {like transformer_to_be_tested}.flipped_index_is_small_enough (m, acc, j, x))

			acc := some_natural \\ 2 = 0
			x := some_object_a
			m := some_natural
			j := some_natural
			assert ("flipped_index_is_small_enough", {like transformer_to_be_tested}.flipped_index_is_small_enough (m, acc, j, x) implies True)
		end

feature {NONE} -- Factory (element to be tested)

	transformer_to_be_tested: like new_transformer_ab
			-- Transformer meant to be under tests
		do
			check
				new_tr: attached {FUNCTION [like new_transformer_ab]} agent new_transformer_ab as new_tr

					-- `some_direct_instance' and `new_transformer_ab' definitions
				tr: attached {like new_transformer_ab} some_direct_instance (new_tr) as tr
				monomorphic: tr.generating_type ~ {detachable like new_transformer_ab}
			then
				Result := tr
			end
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
