﻿note
	description: "Test suite for {STI_N_TUPLE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	N_TUPLE_TESTS [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	ELEMENT_TESTS
		rename
			test_is_in as test_element_is_in,
			test_is_not_in as test_element_is_not_in,
			element_to_be_tested as tuple_to_be_tested
		redefine
			tuple_to_be_tested
		end

	SETS_TESTS_RESOURCES_UNARY [A, EQ]

feature -- Access

	properties: N_TUPLE_PROPERTIES [A, EQ]
			-- Object that checks the set-theory properties of {STI_N_TUPLE}
		deferred
		end

feature -- Test routines (Initialization)

	test_make_unit
			-- Test {STI_N_TUPLE}.make_unit.
		note
			testing: "covers/{STI_N_TUPLE}.make_unit"
		do
			assert ("make_unit", attached (create {like tuple_to_be_tested}.make_unit))
			assert ("make_unit_ok", properties.make_unit_ok)
		end

	test_make_after
			-- Test {STI_N_TUPLE}.make_after.
		note
			testing: "covers/{STI_N_TUPLE}.make_after"
		local
			a: like some_object_a
			t: like some_n_tuple_a
		do
			a := some_object_a
			t := some_n_tuple_a
			assert ("make_after", attached (create {like tuple_to_be_tested}.make_after (a, t)))
			assert ("make_after_ok", properties.make_after_ok (a, t))
		end

	test_make_n
			-- Test {STI_N_TUPLE}.make_n.
		note
			testing: "covers/{STI_N_TUPLE}.make_n"
		local
			n: NATURAL
			a: like some_object_a
		do
			n := some_count
			a := some_object_a
			assert ("make_n", attached (create {like tuple_to_be_tested}.make_n (n, a)))
			assert ("make_n_ok", properties.make_n_ok (n, a))
		end

	test_make_from_tuple
			-- Test {STI_N_TUPLE}.make_from_tuple.
		note
			testing: "covers/{STI_N_TUPLE}.make_from_tuple"
		local
			t: like some_n_tuple_a
		do
			t := some_n_tuple_a
			assert ("make_from_tuple", attached (create {like tuple_to_be_tested}.make_from_tuple (t)))
			assert ("convert_from_ta", attached tuple_from_ta (some_n_tuple_a))
			assert ("convert_from_taa", attached tuple_from_taa (new_transformable_tuple_aa))
		end

feature -- Test routines (Primitive)

	test_is_unit
			-- Test {STI_N_TUPLE}.is_unit.
		note
			testing: "covers/{STI_N_TUPLE}.is_unit"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested.unit
			assert ("t_is_unit", t.is_unit)
			assert ("t_is_unit_ok", properties.is_unit_ok (t))

			t := tuple_to_be_tested
			assert ("is_unit", t.is_unit implies True)
			assert ("is_unit_ok", properties.is_unit_ok (t))
		end

	test_first
			-- Test {STI_N_TUPLE}.first.
		note
			testing: "covers/{STI_N_TUPLE}.first"
		local
			t: like tuple_to_be_tested
		do
			from
				t := tuple_to_be_tested
			until
				not t.is_unit
			loop
				t := tuple_to_be_tested
			end
			assert ("non_unit_t_first", attached t.first implies True)
			assert ("non_unit_t_first_ok", properties.first_ok (t))

			t := tuple_to_be_tested
			assert ("first", not t.is_unit implies attached t.first implies True)
			assert ("first_ok", properties.first_ok (t))
		end

	test_tail
			-- Test {STI_N_TUPLE}.tail.
		note
			testing: "covers/{STI_N_TUPLE}.tail"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested.unit
			assert ("unit_tail", attached t.tail)
			assert ("unit_tail_ok", properties.tail_ok (t))

			t := tuple_to_be_tested
			assert ("tail", attached t.tail)
			assert ("tail_ok", properties.tail_ok (t))
		end

	test_eq
			-- Test {STI_N_TUPLE}.eq.
		note
			testing: "covers/{STI_N_TUPLE}.eq"
		do
			assert ("eq", attached {like tuple_to_be_tested}.eq)
		end

feature -- Test routines (Membership)

	test_is_in
			-- Test {STS_N_TUPLE}.is_in.
			-- Test {STI_N_TUPLE}.is_in.
		note
			testing: "covers/{STS_N_TUPLE}.is_in"
			testing: "covers/{STI_N_TUPLE}.is_in"
		local
			t: like tuple_to_be_tested
			ts: like some_set_ta
			chk_with: BOOLEAN
		do
			t := tuple_to_be_tested

			chk_with := {ISE_RUNTIME}.check_assert (False)
				ts := some_references_ta & t
			chk_with := {ISE_RUNTIME}.check_assert (chk_with)

			assert ("same_reference", t ∈ ts)

			t := tuple_to_be_tested

			chk_with := {ISE_RUNTIME}.check_assert (False)
				ts := some_objects_ta & t.twin
			chk_with := {ISE_RUNTIME}.check_assert (chk_with)

			assert ("same_value", t ∈ ts)

			t := tuple_to_be_tested

			chk_with := {ISE_RUNTIME}.check_assert (False)
				ts := some_math_set_ta & same_n_tuple_a (t)
			chk_with := {ISE_RUNTIME}.check_assert (chk_with)

			assert ("same_math_value", t ∈ ts)

			t := tuple_to_be_tested
			ts := some_set_ta
			assert ("some_tuples", t ∈ ts implies True)
		end

	test_is_not_in
			-- Test {STS_N_TUPLE}.is_not_in.
			-- Test {STI_N_TUPLE}.is_not_in.
		note
			testing: "covers/{STS_N_TUPLE}.is_not_in"
			testing: "covers/{STI_N_TUPLE}.is_not_in"
		local
			t: like tuple_to_be_tested
			ts: like some_set_ta
			chk_without: BOOLEAN
		do
			t := tuple_to_be_tested

			chk_without := {ISE_RUNTIME}.check_assert (False)
				ts := some_references_ta / t
			chk_without := {ISE_RUNTIME}.check_assert (chk_without)

			assert ("same_reference", t ∉ ts)

			t := tuple_to_be_tested

			chk_without := {ISE_RUNTIME}.check_assert (False)
				ts := some_objects_ta / t.twin
			chk_without := {ISE_RUNTIME}.check_assert (chk_without)

			assert ("same_value", t ∉ ts)

			t := tuple_to_be_tested

			chk_without := {ISE_RUNTIME}.check_assert (False)
				ts := some_math_set_ta / same_n_tuple_a (t)
			chk_without := {ISE_RUNTIME}.check_assert (chk_without)

			assert ("same_math_value", t ∉ ts)

			t := tuple_to_be_tested
			ts := some_set_ta
			assert ("some_tuples", t ∉ ts implies True)
		end

	test_has
			-- Test {STI_N_TUPLE}.has.
		note
			testing: "covers/{STI_N_TUPLE}.has"
		local
			a: A
			t: like tuple_to_be_tested
			chk_bef: BOOLEAN
		do
			a := some_object_a

			chk_bef := {ISE_RUNTIME}.check_assert (False)
				t := tuple_to_be_tested.before (a)
			chk_bef := {ISE_RUNTIME}.check_assert (chk_bef)

			assert ("t_has_a", t.has (a))
			assert ("t_has_a_ok", properties.has_ok (t, a))

			a := some_object_a
			t := tuple_to_be_tested
			assert ("has", t.has (a) implies True)
			assert ("has_ok", properties.has_ok (t, a))
		end

	test_does_not_have
			-- Test {STS_N_TUPLE}.does_not_have.
			-- Test {STI_N_TUPLE}.does_not_have.
		note
			testing: "covers/{STS_N_TUPLE}.does_not_have"
			testing: "covers/{STI_N_TUPLE}.does_not_have"
		local
			a: A
			t: like tuple_to_be_tested
			chk_ops: BOOLEAN
		do
			a := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
				t := tuple_to_be_tested.terms.without (a).as_tuple
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("t_does_not_have_a", t.does_not_have (a))
			assert ("t_does_not_have_a_ok", properties.does_not_have_ok (t, a))

			a := some_object_a
			t := tuple_to_be_tested
			assert ("does_not_have", t.does_not_have (a) implies True)
			assert ("does_not_have_ok", properties.does_not_have_ok (t, a))
		end

feature -- Test routines (Access)

	test_indices
			-- Test {STI_N_TUPLE}.indices.
		note
			testing: "covers/{STI_N_TUPLE}.indices"
		do
			assert ("indices", attached tuple_to_be_tested.indices)
		end

	test_term
			-- Test {STI_N_TUPLE}.term.
		note
			testing: "covers/{STI_N_TUPLE}.term"
		local
			i: NATURAL
			t: like tuple_to_be_tested
			chk_n: BOOLEAN
		do
			from
				t := tuple_to_be_tested
			until
				not t.is_unit
			loop
				t := tuple_to_be_tested
			end

			chk_n := {ISE_RUNTIME}.check_assert (False)
				i := some_index \\ t.n + 1
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			assert ("positive", 0 < i)
			assert ("small_enough", i <= t.n)
			assert ("t_i", attached t [i] implies True)
			assert ("t_i_ok", properties.term_ok (t, i))

			i := some_index
			t := tuple_to_be_tested
			assert ("term", 0 < i and i <= t.n implies attached t [i] implies True)
			assert ("term_ok", properties.term_ok (t, i))
		end

	test_terms
			-- Test {STI_N_TUPLE}.terms.
		note
			testing: "covers/{STI_N_TUPLE}.terms"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested
			assert ("terms", attached t.terms)
			assert ("terms_ok", properties.terms_ok (t))
		end

	test_front
			-- Test {STI_N_TUPLE}.front.
		note
			testing: "covers/{STI_N_TUPLE}.front"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested.unit
			assert ("unit_front", attached t.front)
			assert ("unit_front_ok", properties.front_ok (t))

			t := tuple_to_be_tested
			assert ("front", attached t.front)
			assert ("front_ok", properties.front_ok (t))
		end

	test_last
			-- Test {STI_N_TUPLE}.last.
		note
			testing: "covers/{STI_N_TUPLE}.last"
		local
			t: like tuple_to_be_tested
		do
			from
				t := tuple_to_be_tested
			until
				not t.is_unit
			loop
				t := tuple_to_be_tested
			end
			assert ("non_unit_t_last", attached t.last implies True)
			assert ("non_unit_t_last_ok", properties.last_ok (t))

			t := tuple_to_be_tested
			assert ("last", not t.is_unit implies attached t.last implies True)
			assert ("last_ok", properties.last_ok (t))
		end

feature -- Test routines (Construction)

	test_before
			-- Test {STI_N_TUPLE}.before.
		note
			testing: "covers/{STI_N_TUPLE}.before"
		local
			a: A
			t: like tuple_to_be_tested
		do
			a := some_object_a
			t := tuple_to_be_tested
			assert ("before", attached t.before (a))
			assert ("before_ok", properties.before_ok (t, a))
		end

	test_after
			-- Test {STI_N_TUPLE}.after.
		note
			testing: "covers/{STI_N_TUPLE}.after"
		local
			a: A
			t: like tuple_to_be_tested
		do
			a := some_object_a
			t := tuple_to_be_tested
			assert ("after", attached t.after (a))
			assert ("after_ok", properties.after_ok (a, t))
		end

feature -- Test routines (Measurement)

	test_n
			-- Test {STI_N_TUPLE}.n.
		note
			testing: "covers/{STI_N_TUPLE}.n"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested
			assert ("n", attached t.n)
			assert ("n_ok", properties.n_ok (t))
		end

feature -- Test routines (Output)

	test_out
			-- Test {STI_N_TUPLE}.out.
		note
			testing: "covers/{STI_N_TUPLE}.out"
		do
			assert ("out", attached tuple_to_be_tested.out)
		end

	test_terms_out
			-- Test {STI_N_TUPLE}.terms_out.
		note
			testing: "covers/{STI_N_TUPLE}.terms_out"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested
			assert ("terms_out", attached t.terms_out)
			assert ("terms_out_ok", properties.terms_out_ok (t))
		end

	test_term_out
			-- Test {STI_N_TUPLE}.term_out.
		note
			testing: "covers/{STI_N_TUPLE}.term_out"
		do
			assert ("term_out", attached tuple_to_be_tested.term_out (some_object_a))
		end

feature -- Test routines (Comparison)

	test_equals
			-- Test {STI_N_TUPLE}.equals.
		note
			testing: "covers/{STI_N_TUPLE}.equals"
		local
			t1: like tuple_to_be_tested
			t2: like some_n_tuple_a
		do
			t1 := tuple_to_be_tested
			t2 := t1
			assert ("same_reference", t1 ≍ t2)
			assert ("same_reference_ok", properties.equals_ok (t1, t2))

			t1 := tuple_to_be_tested
			t2 := same_n_tuple_a (t1)
			assert ("same_tuple", t1 ≍ t2)
			assert ("same_tuple_ok", properties.equals_ok (t1, t2))

			t1 := tuple_to_be_tested
			t2 := some_n_tuple_a
			assert ("some_tuple", t1 ≍ t2 implies True)
			assert ("some_tuple_ok", properties.equals_ok (t1, t2))
		end

	test_unequals
			-- Test {STS_N_TUPLE}.unequals.
			-- Test {STI_N_TUPLE}.unequals.
		note
			testing: "covers/{STS_N_TUPLE}.unequals"
			testing: "covers/{STI_N_TUPLE}.unequals"
		local
			t1: like tuple_to_be_tested
			t2: like some_n_tuple_a
		do
			t1 := tuple_to_be_tested
			t2 := t1
			assert ("same_reference", not (t1 ≭ t2))
			assert ("same_reference_ok", properties.unequals_ok (t1, t2))

			t1 := tuple_to_be_tested
			t2 := same_n_tuple_a (t1)
			assert ("same_tuple", not (t1 ≭ t2))
			assert ("same_tuple_ok", properties.unequals_ok (t1, t2))

			t1 := tuple_to_be_tested
			t2 := some_n_tuple_a
			assert ("some_tuple", not (t1 ≭ t2) implies True)
			assert ("some_tuple_ok", properties.unequals_ok (t1, t2))
		end

feature -- Test routines (Quantifier)

	test_exists
			-- Test {STI_N_TUPLE}.exists.
		note
			testing: "covers/{STI_N_TUPLE}.exists"
		local
			t: like tuple_to_be_tested
			i: NATURAL
			a: A
			chk_ops: BOOLEAN
		do
			from
				i := some_index
				a := some_object_a
			until
				p_i_x (i, a)
			loop
				i := some_index
				a := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				across
					1 |..| (i - 1).as_integer_32 as ic
				from
					t := tuple_to_be_tested.unit
				loop
					t := t.before (some_object_a)
				end
				t := t.before (a)
				across
					t.n.as_integer_32 |..| (some_index.max (t.n) - 1).as_integer_32 as ic
				loop
					t := t.before (some_object_a)
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("t_exists_p_i_x", t.exists (agent p_i_x))
			assert ("t_exists_p_i_x_ok", properties.exists_ok (t, agent p_i_x))

			t := tuple_to_be_tested
			assert ("exists", t.exists (agent p_i_x) implies True)
			assert ("exists_ok", properties.exists_ok (t, agent p_i_x))
		end

	test_does_not_exist
			-- Test {STS_N_TUPLE}.does_not_exist.
			-- Test {STI_N_TUPLE}.does_not_exist.
		note
			testing: "covers/{STS_N_TUPLE}.does_not_exist"
			testing: "covers/{STI_N_TUPLE}.does_not_exist"
		local
			a: A
			t: like tuple_to_be_tested
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				across
					1 |..| (some_count).as_integer_32 as i
				from
					t := tuple_to_be_tested.unit
				loop
					a := some_object_a
					if not p_i_x (t.n + 1, a) then
						t := t.before (a)
					end
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("t_does_not_exist_p_i_x", t.does_not_exist (agent p_i_x))
			assert ("t_does_not_exist_p_i_x_ok", properties.does_not_exist_ok (t, agent p_i_x))

			t := tuple_to_be_tested
			assert ("does_not_exist", t.does_not_exist (agent p_i_x) implies True)
			assert ("does_not_exist_ok", properties.does_not_exist_ok (t, agent p_i_x))
		end

	test_for_all
			-- Test {STI_N_TUPLE}.for_all.
		note
			testing: "covers/{STI_N_TUPLE}.for_all"
		local
			a: A
			t: like tuple_to_be_tested
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				across
					1 |..| (some_count).as_integer_32 as i
				from
					t := tuple_to_be_tested.unit
				loop
					a := some_object_a
					if p_i_x (t.n + 1, a) then
						t := t.before (a)
					end
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("t_for_all_p_i_x", t.for_all (agent p_i_x))
			assert ("t_for_all_p_i_x_ok", properties.for_all_ok (agent p_i_x, t))

			t := tuple_to_be_tested
			assert ("for_all", t.for_all (agent p_i_x) implies True)
			assert ("for_all_ok", properties.for_all_ok (agent p_i_x, t))
		end

feature -- Test routines (Operation)

	test_left_trimmed
			-- Test {STI_N_TUPLE}.left_trimmed.
		note
			testing: "covers/{STI_N_TUPLE}.left_trimmed"
		local
			m: NATURAL
			t: like tuple_to_be_tested
			chk_n: BOOLEAN
		do
			t := tuple_to_be_tested

			chk_n := {ISE_RUNTIME}.check_assert (False)
				from
					m := some_count
				until
					m ≥ t.n
				loop
					m := some_count
				end
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			assert ("nothing_left", attached t.left_trimmed (m))
			assert ("nothing_left_ok", properties.left_trimmed_ok (m, t))

			t := tuple_to_be_tested
			m := some_count
			assert ("left_trimmed", attached t.left_trimmed (m))
			assert ("left_trimmed_ok", properties.left_trimmed_ok (m, t))
		end

	test_right_trimmed
			-- Test {STI_N_TUPLE}.right_trimmed.
		note
			testing: "covers/{STI_N_TUPLE}.right_trimmed"
		local
			m: NATURAL
			t: like tuple_to_be_tested
			chk_n: BOOLEAN
		do
			t := tuple_to_be_tested

			chk_n := {ISE_RUNTIME}.check_assert (False)
				from
					m := some_count
				until
					m >= t.n
				loop
					m := some_count
				end
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			assert ("nothing_left", attached t.right_trimmed (m))
			assert ("nothing_left_ok", properties.right_trimmed_ok (m, t))

			t := tuple_to_be_tested
			m := some_count
			assert ("right_trimmed", attached t.right_trimmed (m))
			assert ("right_trimmed_ok", properties.right_trimmed_ok (m, t))
		end

	test_prefix
			-- Test {STI_N_TUPLE}.prefix.
		note
			testing: "covers/{STI_N_TUPLE}.prefix"
		local
			m: NATURAL
			t: like tuple_to_be_tested
			chk_n: BOOLEAN
		do
			t := tuple_to_be_tested

			chk_n := {ISE_RUNTIME}.check_assert (False)
				from
					m := some_count
				until
					m >= t.n
				loop
					m := some_count
				end
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			assert ("nothing_lost", attached t.prefix (m))
			assert ("nothing_lost_ok", properties.prefix_ok (m, t))

			t := tuple_to_be_tested
			m := some_count
			assert ("prefix", attached t.prefix (m))
			assert ("prefix_ok", properties.prefix_ok (m, t))
		end

	test_suffix
			-- Test {STI_N_TUPLE}.suffix.
		note
			testing: "covers/{STI_N_TUPLE}.suffix"
		local
			m: NATURAL
			t: like tuple_to_be_tested
			chk_n: BOOLEAN
		do
			t := tuple_to_be_tested

			chk_n := {ISE_RUNTIME}.check_assert (False)
				from
					m := some_count
				until
					m >= t.n
				loop
					m := some_count
				end
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			assert ("nothing_lost", attached t.suffix (m))
			assert ("nothing_lost_ok", properties.suffix_ok (m, t))

			t := tuple_to_be_tested
			m := some_count
			assert ("suffix", attached t.suffix (m))
			assert ("suffix_ok", properties.suffix_ok (m, t))
		end

	test_reversed
			-- Test {STI_N_TUPLE}.reversed.
		note
			testing: "covers/{STI_N_TUPLE}.reversed"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested
			assert ("reversed", attached t.reversed)
			assert ("reversed_ok", properties.reversed_ok (t))
		end

	test_replacing
			-- Test {STI_N_TUPLE}.replacing.
		note
			testing: "covers/{STI_N_TUPLE}.replacing"
		local
			i: NATURAL
			a: like some_object_a
			t: like tuple_to_be_tested
			chk_n: BOOLEAN
		do
			from
				t := tuple_to_be_tested
			until
				not t.is_unit
			loop
				t := tuple_to_be_tested
			end

			chk_n := {ISE_RUNTIME}.check_assert (False)
				i := some_index \\ t.n + 1
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)

			a := some_object_a
			assert ("positive", 0 < i)
			assert ("small_enough", i ≤ t.n)
			assert ("t_replacing_i_a", attached t.replacing (i, a))
			assert ("t_replacing_i_a_ok", properties.replacing_ok (t, i, a))

			i := some_index
			a := some_object_a
			t := tuple_to_be_tested
			assert ("replacing", 0 < i and i ≤ t.n implies attached t.replacing (i, a))
			assert ("replacing_ok", properties.replacing_ok (t, i, a))
		end

	test_concatenated
			-- Test {STI_N_TUPLE}.concatenated.
		note
			testing: "covers/{STI_N_TUPLE}.concatenated"
		local
			t1: like tuple_to_be_tested
			t2: like some_n_tuple_a
		do
			t1 := tuple_to_be_tested
			t2 := some_n_tuple_a
			assert ("concatenated", attached (t1 + t2))
			assert ("concatenated_ok", properties.concatenated_ok (t1, t2))
		end

feature -- Test routines (Transformation)

	test_mapped
			-- Test {STS_N_TUPLE}.mapped.
			-- Test {STI_N_TUPLE}.mapped.
		note
			testing: "covers/{STS_N_TUPLE}.mapped"
			testing: "covers/{STI_N_TUPLE}.mapped"
		local
			t: like tuple_to_be_tested
			f: FUNCTION [A, A]
		do
			t := tuple_to_be_tested
			f := agent f_x
			assert ("mapped", attached (t ↦ f))
			assert ("mapped_ok", properties.mapped_ok (t, f))
		end

	test_left_reduced
			-- Test {STS_N_TUPLE}.left_reduced.
			-- Test {STI_N_TUPLE}.left_reduced.
		note
			testing: "covers/{STS_N_TUPLE}.left_reduced"
			testing: "covers/{STI_N_TUPLE}.left_reduced"
		local
			t: like tuple_to_be_tested
			leftmost: like some_object_a
			f: FUNCTION [A, A, A]
		do
			t := tuple_to_be_tested
			leftmost := some_object_a
			f := agent f_acc_x
			assert ("left_reduced", attached t.left_reduced (leftmost, f) implies True)
			assert ("left_reduced_ok", properties.left_reduced_ok (t, leftmost, f))
		end

	test_indexed_left_reduced
			-- Test {STS_N_TUPLE}.indexed_left_reduced.
			-- Test {STI_N_TUPLE}.indexed_left_reduced.
		note
			testing: "covers/{STS_N_TUPLE}.indexed_left_reduced"
			testing: "covers/{STI_N_TUPLE}.indexed_left_reduced"
		local
			t: like tuple_to_be_tested
			leftmost: like some_object_a
			i: NATURAL
			f: FUNCTION [A, NATURAL, A, A]
		do
			t := tuple_to_be_tested
			leftmost := some_object_a
			i := some_natural
			f := agent f_acc_i_x
			assert ("indexed_left_reduced", attached t.indexed_left_reduced (leftmost, i, f) implies True)
			assert ("indexed_left_reduced_ok", properties.indexed_left_reduced_ok (t, leftmost, i, f))
		end

	test_proper_left_reduced
			-- Test {STS_N_TUPLE}.proper_left_reduced.
			-- Test {STI_N_TUPLE}.proper_left_reduced.
		note
			testing: "covers/{STS_N_TUPLE}.proper_left_reduced"
			testing: "covers/{STI_N_TUPLE}.proper_left_reduced"
		local
			t: like tuple_to_be_tested
			f: FUNCTION [A, A, A]
		do
			f := agent f_acc_x
			from
				t := tuple_to_be_tested
			until
				not t.is_unit
			loop
				t := tuple_to_be_tested
			end
			assert ("t_proper_left_reduced_f", attached t.proper_left_reduced (f) implies True)
			assert ("t_proper_left_reduced_f_ok", properties.proper_left_reduced_ok (t, f))

			t := tuple_to_be_tested
			assert ("proper_left_reduced", not t.is_unit implies attached t.proper_left_reduced (f) implies True)
			assert ("proper_left_reduced_ok", properties.proper_left_reduced_ok (t, f))
		end

	test_right_reduced
			-- Test {STS_N_TUPLE}.right_reduced.
			-- Test {STI_N_TUPLE}.right_reduced.
		note
			testing: "covers/{STS_N_TUPLE}.right_reduced"
			testing: "covers/{STI_N_TUPLE}.right_reduced"
		local
			t: like tuple_to_be_tested
			f: FUNCTION [A, A, A]
			rightmost: like some_object_a
		do
			t := tuple_to_be_tested
			f := agent f_x_acc
			rightmost := some_object_a
			assert ("right_reduced", attached t.right_reduced (f, rightmost) implies True)
			assert ("right_reduced_ok", properties.right_reduced_ok (t, f, rightmost))
		end

	test_indexed_right_reduced
			-- Test {STS_N_TUPLE}.indexed_right_reduced.
			-- Test {STI_N_TUPLE}.indexed_right_reduced.
		note
			testing: "covers/{STS_N_TUPLE}.indexed_right_reduced"
			testing: "covers/{STI_N_TUPLE}.indexed_right_reduced"
		local
			t: like tuple_to_be_tested
			i: NATURAL
			f: FUNCTION [NATURAL, A, A, A]
			rightmost: like some_object_a
		do
			t := tuple_to_be_tested
			i := some_natural
			f := agent f_i_x_acc
			rightmost := some_object_a
			assert ("indexed_right_reduced", attached t.indexed_right_reduced (i, f, rightmost) implies True)
			assert ("indexed_right_reduced_ok", properties.indexed_right_reduced_ok (t, i, f, rightmost))
		end

	test_proper_right_reduced
			-- Test {STI_N_TUPLE}.proper_right_reduced.
		note
			testing: "covers/{STI_N_TUPLE}.proper_right_reduced"
		local
			t: like tuple_to_be_tested
			f: FUNCTION [A, A, A]
		do
			f := agent f_x_acc
			from
				t := tuple_to_be_tested
			until
				not t.is_unit
			loop
				t := tuple_to_be_tested
			end
			assert ("t_proper_right_reduced_f", attached t.proper_right_reduced (f) implies True)
			assert ("t_proper_right_reduced_f_ok", properties.proper_right_reduced_ok (t, f))

			t := tuple_to_be_tested
			assert ("proper_right_reduced", not t.is_unit implies attached t.proper_right_reduced (f) implies True)
			assert ("proper_right_reduced_ok", properties.proper_right_reduced_ok (t, f))
		end

feature -- Test routines (Conversion)

	test_natural_as_integer
			-- Test {STI_N_TUPLE}.natural_as_integer.
		note
			testing: "covers/{STI_N_TUPLE}.natural_as_integer"
		do
			assert ("natural_as_integer", attached {like tuple_to_be_tested}.natural_as_integer (some_natural))
		end

	test_as_singleton
			-- Test {STI_N_TUPLE}.as_singleton.
		note
			testing: "covers/{STI_N_TUPLE}.as_singleton"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested
			assert ("as_singleton", attached t.as_singleton)
			assert ("as_singleton_ok", properties.as_singleton_ok (t))
		end

	test_first_tail
			-- Test {STI_N_TUPLE}.first_tail.
		note
			testing: "covers/{STI_N_TUPLE}.first_tail"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested
			assert ("first_tail", not t.is_unit implies attached t.first_tail)
			assert ("first_tail_ok", properties.first_tail_ok (t))
		end

	test_front_last
			-- Test {STI_N_TUPLE}.front_last.
		note
			testing: "covers/{STI_N_TUPLE}.front_last"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested
			assert ("front_last", not t.is_unit implies attached t.front_last)
			assert ("front_last_ok", properties.front_last_ok (t))
		end

feature -- Test routines (Factory)

	test_unit
			-- Test {STI_N_TUPLE}.unit.
		note
			testing: "covers/{STI_N_TUPLE}.unit"
		local
			t: like tuple_to_be_tested
		do
			t := tuple_to_be_tested
			assert ("unit", attached t.unit)
			assert ("unit_ok", properties.unit_ok (t))
		end

	test_converted
			-- Test {STI_N_TUPLE}.converted.
		note
			testing: "covers/{STI_N_TUPLE}.converted"
		do
			assert ("converted", attached tuple_to_be_tested.converted (some_n_tuple_a))
		end

	test_converted_terms
			-- Test {STI_N_TUPLE}.converted_terms.
		note
			testing: "covers/{STI_N_TUPLE}.converted_terms"
		do
			assert ("converted_terms", attached {like tuple_to_be_tested}.converted_terms (some_set_a))
		end

feature -- Test routines (Predicate)

	test_predicate_holds_to_term
			-- Test {STS_N_TUPLE}.predicate_holds_to_term.
			-- Test {STI_N_TUPLE}.predicate_holds_to_term.
		note
			testing: "covers/{STS_N_TUPLE}.predicate_holds_to_term"
			testing: "covers/{STI_N_TUPLE}.predicate_holds_to_term"
		local
			p: PREDICATE [NATURAL, A]
			t: like tuple_to_be_tested
			i: NATURAL
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					p := agent p_i_x
					t := tuple_to_be_tested
					i := some_count ∧ t.n
				until
					(0 < i and i ≤ t.n) and then p (i, t [i])
				loop
					t := tuple_to_be_tested
					i := some_count ∧ t.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("p_holds_to_t_i", t.predicate_holds_to_term (p, i))

			t := tuple_to_be_tested
			i := some_count
			assert ("predicate_holds_to_term", t.predicate_holds_to_term (p, i) implies True)
		end

	test_negated
			-- Test {STS_N_TUPLE}.negated.
			-- Test {STI_N_TUPLE}.negated.
		note
			testing: "covers/{STS_N_TUPLE}.negated"
			testing: "covers/{STI_N_TUPLE}.negated"
		local
			p: PREDICATE [NATURAL, A]
			i: NATURAL;
			a: A
		do
			from
				p := agent p_i_x
				i := some_natural
				a := some_object_a
			until
				not p (i, a)
			loop
				i := some_natural
				a := some_object_a
			end
			assert ("negated_p_i_a", {like tuple_to_be_tested}.negated (p, i, a))

			i := some_natural
			a := some_object_a
			assert ("negated", {like tuple_to_be_tested}.negated (p, i, a) implies True)
		end

	test_terms_are_equal
			-- Test {STS_N_TUPLE}.terms_are_equal.
			-- Test {STI_N_TUPLE}.terms_are_equal.
		note
			testing: "covers/{STS_N_TUPLE}.terms_are_equal"
			testing: "covers/{STI_N_TUPLE}.terms_are_equal"
		local
			t: like tuple_to_be_tested
			i: NATURAL;
			a: A
			chk_ops: BOOLEAN
		do
			from
				t := tuple_to_be_tested
			until
				not t.is_unit
			loop
				t := tuple_to_be_tested
			end

				check
					positive: 0 < t.n -- not t.is_unit
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				i := some_index_up_to (t.n)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- some_index > 0; t.n > 0.
					small_enough: i <= t.n -- i = some_index.min (t.n)
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := t [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a_equals_t_i", t.terms_are_equal (i, a))

			t := tuple_to_be_tested
			i := some_index
			a := some_object_a
			assert ("terms_are_equal", t.terms_are_equal (i, a) implies True)
		end

	test_reversed_terms_are_equal
			-- Test {STS_N_TUPLE}.reversed_terms_are_equal.
			-- Test {STI_N_TUPLE}.reversed_terms_are_equal.
		note
			testing: "covers/{STS_N_TUPLE}.reversed_terms_are_equal"
			testing: "covers/{STI_N_TUPLE}.reversed_terms_are_equal"
		local
			t: like tuple_to_be_tested
			i: NATURAL;
			a: A
			chk_ops: BOOLEAN
		do
			from
				t := tuple_to_be_tested
			until
				not t.is_unit
			loop
				t := tuple_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				t := t.reversed
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < t.n -- not t.is_unit
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				i := some_index_up_to (t.n)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- some_index > 0; t.n > 0.
					small_enough: i <= t.n -- i = some_index.min (t.n)
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := t [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a_equals_t_reversed_i", t.reversed.reversed_terms_are_equal (i, a))

			t := tuple_to_be_tested
			i := some_index
			a := some_object_a
			assert ("reversed_terms_are_equal", t.reversed_terms_are_equal (i, a) implies True)
		end

	test_index_is_positive
			-- Test {STS_N_TUPLE}.index_is_positive.
			-- Test {STI_N_TUPLE}.index_is_positive.
		note
			testing: "covers/{STS_N_TUPLE}.index_is_positive"
			testing: "covers/{STI_N_TUPLE}.index_is_positive"
		local
			i: NATURAL
		do
			i := some_index
			assert ("i_is_positive", {like tuple_to_be_tested}.index_is_positive (i))

			i := some_natural
			assert ("index_is_positive", {like tuple_to_be_tested}.index_is_positive (i) implies True)
		end

	test_index_is_less_equal
			-- Test {STS_N_TUPLE}.index_is_less_equal.
			-- Test {STI_N_TUPLE}.index_is_less_equal.
		note
			testing: "covers/{STS_N_TUPLE}.index_is_less_equal"
			testing: "covers/{STI_N_TUPLE}.index_is_less_equal"
		local
			i, j: NATURAL
		do
			i := some_index
			j := some_index ∨ i
			assert ("i_is_less_equal_j", {like tuple_to_be_tested}.index_is_less_equal (i, j))

			i := some_index
			j := some_index
			assert ("index_is_less_equal",
			{like tuple_to_be_tested}.index_is_less_equal (i, j) implies True
			)
		end

feature -- Test routines (Reduction)

	test_just_index
			-- Test {STS_N_TUPLE}.just_index.
			-- Test {STI_N_TUPLE}.just_index.
		note
			testing: "covers/{STS_N_TUPLE}.just_index"
			testing: "covers/{STI_N_TUPLE}.just_index"
		local
			i: NATURAL
		do
			i := some_index
			assert ("just_i", {like tuple_to_be_tested}.just_index (some_index, i, some_object_a) = i)

			assert ("just_index", attached {like tuple_to_be_tested}.just_index (some_index, some_index, some_object_a))
		end

	test_indexed_term
			-- Test {STI_N_TUPLE}.indexed_term.
		note
			testing: "covers/{STI_N_TUPLE}.indexed_term"
		local
			i, j: NATURAL
			acc, x_j: A
		do
			acc := some_object_a
			x_j := some_object_a
			j := some_index
			i := j
			assert ("x_j", {like tuple_to_be_tested}.eq ({like tuple_to_be_tested}.indexed_term (i, acc, j, x_j), x_j))

			acc := some_object_a
			x_j := some_object_a
			j := some_index
			from
				i := some_index
			until
				i /= j
			loop
				i := some_index
			end
			assert ("acc", {like tuple_to_be_tested}.eq ({like tuple_to_be_tested}.indexed_term (i, acc, j, x_j), acc))

			acc := some_object_a
			x_j := some_object_a
			j := some_index
			i := some_index
			assert ("indexed_term", attached {like tuple_to_be_tested}.indexed_term (i, acc, j, x_j) implies True)
		end

	test_appending_term_out
			-- Test {STI_N_TUPLE}.appending_term_out.
		note
			testing: "covers/{STI_N_TUPLE}.appending_term_out"
		local
			acc: STRING
			i: NATURAL
			a: A
			chk_out: BOOLEAN
		do
			a := some_object_a

			chk_out := {ISE_RUNTIME}.check_assert (False)
				acc := some_element.out
			chk_out := {ISE_RUNTIME}.check_assert (chk_out)

			assert ("appending_first_term_out", {like tuple_to_be_tested}.appending_term_out (acc, 1, a) ~ {like tuple_to_be_tested}.term_out (a))

			a := some_object_a

			chk_out := {ISE_RUNTIME}.check_assert (False)
				acc := some_element.out
			chk_out := {ISE_RUNTIME}.check_assert (chk_out)

				check
					positive: 0 < Max_count -- Max_count definition
				end
			i := some_index_up_to (Max_count)
			assert ("appending_term_out", attached {like tuple_to_be_tested}.appending_term_out (acc, i, a))
		end

feature {NONE} -- Factory (element to be tested)

	tuple_to_be_tested: like new_n_tuple_a
			-- N-tuple meant to be under tests
		do
			check
				new_t: attached {FUNCTION [like new_n_tuple_a]} (agent new_n_tuple_a) as new_t

					-- `some_direct_instance' and `new_n_tuple_a' definitions
				t: attached {like new_n_tuple_a} some_direct_instance (new_t) as t
				monomorphic: t.generating_type ~ {detachable like new_n_tuple_a}
			then
				Result := t
			end
		end

feature {NONE} -- Conversion

	tuple_from_ta (ta: STS_N_TUPLE [A, EQ]): like tuple_to_be_tested
			-- `ta' converted to an n-tuple like `tuple_to_be_tested'
		do
			Result := ta
		ensure
			same_tuple: Result ≍ ta
		end

	tuple_from_taa (taa: STS_TRANSFORMABLE_TUPLE [A, A, EQ, EQ]): like tuple_to_be_tested
			-- `taa' converted to an n-tuple like `tuple_to_be_tested'
		do
			Result := taa
		ensure
			same_tuple: Result ≍ taa
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
