﻿note
	description: "Test suite for {STI_TRANSFORMABLE_TUPLE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	TRANSFORMABLE_TUPLE_TESTS [A, B, EQ_A -> STS_EQUALITY [A] create default_create end, EQ_B -> STS_EQUALITY [B] create default_create end]

inherit
	N_TUPLE_TESTS [A, EQ_A]
		rename
			test_eq as test_eq_a,
			test_mapped as test_mapped_to_same_kind,
			test_left_reduced as test_left_reduced_to_term_like,
			test_indexed_left_reduced as test_indexed_left_reduced_to_term_like,
			test_right_reduced as test_right_reduced_to_term_like,
			test_indexed_right_reduced as test_indexed_right_reduced_to_term_like,
			f_x as f_x_to_same_kind,
			f_acc_x as f_acc_x_to_same_kind,
			f_acc_i_x as f_acc_i_x_to_same_kind,
			f_x_acc as f_x_acc_to_same_kind,
			f_i_x_acc as f_i_x_acc_to_same_kind
		undefine
			some_equality,
			some_pair,
			some_pair_aa,
			some_set,
			some_set_a,
			same_set_a,
			some_pair_composer,
			some_transformer,
			some_n_tuple,
			some_n_tuple_a,
			same_n_tuple_a
		redefine
			properties,
			test_make_from_tuple,
			tuple_to_be_tested,
			tuple_from_ta,
			tuple_from_taa
		end

	SETS_TESTS_RESOURCES_BINARY [A, B, EQ_A, EQ_B]

feature -- Access

	properties: TRANSFORMABLE_TUPLE_PROPERTIES [A, B, EQ_A, EQ_B]
			-- Object that checks the set-theory properties of {STI_TRANSFORMABLE_TUPLE}
		deferred
		end

feature -- Test routines (Initialization)

	test_make_from_tuple
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_TUPLE}.make_from_tuple.
		note
			testing: "covers/{STI_N_TUPLE}.make_from_tuple"
			testing: "covers/{STI_TRANSFORMABLE_TUPLE}.make_from_tuple"
		do
			Precursor {N_TUPLE_TESTS}
			assert ("convert_from_sti_ta", attached tuple_from_sti_ta (some_n_tuple_a))
			assert ("convert_from_sti_taa", attached tuple_from_sti_taa (new_transformable_tuple_aa))
			assert ("convert_from_tab", attached tuple_from_tab (some_direct_transformable_tuple_ab))
		end

feature -- Test routines (Primitive)

	test_eq_b
			-- Test {STI_TRANSFORMABLE_TUPLE}.eq_b.
		note
			testing: "covers/{STI_TRANSFORMABLE_TUPLE}.eq_b"
		do
			assert ("eq_b", attached {like tuple_to_be_tested}.eq_b)
		end

feature -- Test routines (Transformation)

	test_mapped
			-- Test {STS_TRANSFORMABLE_TUPLE}.mapped.
			-- Test {STI_TRANSFORMABLE_TUPLE}.mapped.
		note
			testing: "covers/{STS_TRANSFORMABLE_TUPLE}.mapped"
			testing: "covers/{STI_TRANSFORMABLE_TUPLE}.mapped"
		local
			t: like tuple_to_be_tested
			f: FUNCTION [A, B]
		do
			t := tuple_to_be_tested
			f := agent f_x
			assert ("mapped", attached (t ↦ f))
			assert ("mapped_ok", properties.mapped_ok (t, f))
		end

	test_left_reduced
			-- Test {STS_TRANSFORMABLE_TUPLE}.left_reduced.
			-- Test {STI_TRANSFORMABLE_TUPLE}.left_reduced.
		note
			testing: "covers/{STS_TRANSFORMABLE_TUPLE}.left_reduced"
			testing: "covers/{STI_TRANSFORMABLE_TUPLE}.left_reduced"
		local
			t: like tuple_to_be_tested
			leftmost: like some_object_b
			f: FUNCTION [B, A, B]
		do
			t := tuple_to_be_tested
			leftmost := some_object_b
			f := agent f_acc_x
			assert ("left_reduced", attached t.left_reduced (leftmost, f) implies True)
			assert ("left_reduced_ok", properties.left_reduced_ok (t, leftmost, f))
		end

	test_indexed_left_reduced
			-- Test {STS_TRANSFORMABLE_TUPLE}.indexed_left_reduced.
			-- Test {STI_TRANSFORMABLE_TUPLE}.indexed_left_reduced.
		note
			testing: "covers/{STS_TRANSFORMABLE_TUPLE}.indexed_left_reduced"
			testing: "covers/{STI_TRANSFORMABLE_TUPLE}.indexed_left_reduced"
		local
			t: like tuple_to_be_tested
			leftmost: like some_object_b
			i: NATURAL
			f: FUNCTION [B, NATURAL, A, B]
		do
			t := tuple_to_be_tested
			leftmost := some_object_b
			i := some_natural
			f := agent f_acc_i_x
			assert ("indexed_left_reduced", attached t.indexed_left_reduced (leftmost, i, f) implies True)
			assert ("indexed_left_reduced_ok", properties.indexed_left_reduced_ok (t, leftmost, i, f))
		end

	test_right_reduced
			-- Test {STS_TRANSFORMABLE_TUPLE}.right_reduced.
			-- Test {STI_TRANSFORMABLE_TUPLE}.right_reduced.
		note
			testing: "covers/{STS_TRANSFORMABLE_TUPLE}.right_reduced"
			testing: "covers/{STI_TRANSFORMABLE_TUPLE}.right_reduced"
		local
			t: like tuple_to_be_tested
			f: FUNCTION [A, B, B]
			rightmost: like some_object_b
		do
			t := tuple_to_be_tested
			f := agent f_x_acc
			rightmost := some_object_b
			assert ("right_reduced", attached t.right_reduced (f, rightmost) implies True)
			assert ("right_reduced_ok", properties.right_reduced_ok (t, f, rightmost))
		end

	test_indexed_right_reduced
			-- Test {STS_TRANSFORMABLE_TUPLE}.indexed_right_reduced.
			-- Test {STI_TRANSFORMABLE_TUPLE}.indexed_right_reduced.
		note
			testing: "covers/{STS_TRANSFORMABLE_TUPLE}.indexed_right_reduced"
			testing: "covers/{STI_TRANSFORMABLE_TUPLE}.indexed_right_reduced"
		local
			t: like tuple_to_be_tested
			i: NATURAL
			f: FUNCTION [NATURAL, A, B, B]
			rightmost: like some_object_b
		do
			t := tuple_to_be_tested
			i := some_natural
			f := agent f_i_x_acc
			rightmost := some_object_b
			assert ("indexed_right_reduced", attached t.indexed_right_reduced (i, f, rightmost) implies True)
			assert ("indexed_right_reduced_ok", properties.indexed_right_reduced_ok (t, i, f, rightmost))
		end

feature {NONE} -- Factory (element to be tested)

	tuple_to_be_tested: like some_direct_transformable_tuple_ab
			-- Transformable tuple meant to be under tests
		do
			Result := some_direct_transformable_tuple_ab
		end

feature {NONE} -- Conversion

	tuple_from_ta (ta: STS_N_TUPLE [A, EQ_A]): like tuple_to_be_tested
			-- <Precursor>
		do
			Result := ta
		end

	tuple_from_sti_ta (ta: STI_N_TUPLE [A, EQ_A]): like tuple_to_be_tested
			-- `ta' converted to an n-tuple like `tuple_to_be_tested'
		do
			Result := ta
		ensure
			same_tuple: Result ≍ ta
		end

	tuple_from_taa (taa: STS_TRANSFORMABLE_TUPLE [A, A, EQ_A, EQ_A]): like tuple_to_be_tested
			-- <Precursor>
		do
			Result := taa
		end

	tuple_from_sti_taa (taa: STI_TRANSFORMABLE_TUPLE [A, A, EQ_A, EQ_A]): like tuple_to_be_tested
			-- `taa' converted to an n-tuple like `tuple_to_be_tested'
		do
			Result := taa
		ensure
			same_tuple: Result ≍ taa
		end

	tuple_from_tab (tab: STS_TRANSFORMABLE_TUPLE [A, B, EQ_A, EQ_B]): like tuple_to_be_tested
			-- `tab' converted to an n-tuple like `tuple_to_be_tested'
		do
			Result := tab
		ensure
			same_tuple: Result ≍ tab
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
