﻿note
	description: "Test suite for {STI_COMPOSABLE_ORDERED_PAIR}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	COMPOSABLE_ORDERED_PAIR_TESTS [
		W, A, B,
		EQ_W -> STS_EQUALITY [W] create default_create end,
		EQ_A -> STS_EQUALITY [A] create default_create end,
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	ORDERED_PAIR_TESTS [A, B, EQ_A, EQ_B]
		undefine
			some_equality,
			some_pair,
			some_preceding_pair_dsa_a,
			some_pair_aa,
			some_preceding_pair_aa,
			some_pair_ab,
			same_pair_ab,
			some_pair_ba,
			some_preceding_pair_ba,
			some_pair_bb,
			some_preceding_pair_bb,
			some_set,
			some_set_a,
			same_set_a,
			some_set_b,
			some_pair_composer,
			some_transformer,
			some_n_tuple,
			some_n_tuple_a,
			same_n_tuple_a,
			some_n_tuple_b
		redefine
			test_make_from_pair,
			pair_to_be_tested,
			pair_from_ab,
			pair_from_aab,
			pair_from_bab
		end

	SETS_TESTS_RESOURCES_TERNARY [W, A, B, EQ_W, EQ_A, EQ_B]

feature -- Access

	properties: COMPOSABLE_ORDERED_PAIR_PROPERTIES [W, A, B, EQ_W, EQ_A, EQ_B]
			-- Object that checks the set-theory properties of {STI_COMPOSABLE_ORDERED_PAIR}
		deferred
		end

feature -- Test routines (Initialization)

	test_make_from_pair
			-- Test {STI_ORDERED_PAIR}.make_from_pair.
			-- Test {STI_COMPOSABLE_ORDERED_PAIR}.make_from_pair.
		note
			testing: "covers/{STI_ORDERED_PAIR}.make_from_pair"
			testing: "covers/{STI_COMPOSABLE_ORDERED_PAIR}.make_from_pair"
		do
			Precursor {ORDERED_PAIR_TESTS}
			assert ("convert_from_sti_ab", attached pair_from_sti_ab (some_pair_ab))
			assert ("convert_from_wab", attached pair_from_wab (some_direct_composable_pair_wab)) -- todo: make some_composable_pair_wab?
			assert ("convert_from_sti_aab", attached pair_from_sti_aab (new_composable_pair_aab))
			assert ("convert_from_sti_bab", attached pair_from_sti_bab (new_composable_pair_bab))
		end

feature -- Test routines (Primitive)

	test_eq_w
			-- Test {STI_COMPOSABLE_ORDERED_PAIR}.eq_w.
		note
			testing: "covers/{STI_COMPOSABLE_ORDERED_PAIR}.eq_w"
		do
			assert ("eq_w", attached {like pair_to_be_tested}.eq_w)
		end

feature -- Test routines (Operation)

	test_composed
			-- Test {STI_COMPOSABLE_ORDERED_PAIR}.composed.
		note
			testing: "covers/{STI_COMPOSABLE_ORDERED_PAIR}.composed"
		local
			ab: like pair_to_be_tested
			wa: like some_preceding_pair_wa
			ww: like some_preceding_pair_ww
		do
			ab := pair_to_be_tested
			wa := some_preceding_pair_wa (ab.a)
			assert ("ab_succeeds_wa", ab ≽ wa)
			assert ("ab_o_wa", attached (ab ∘ wa))
			ww := some_preceding_pair_ww (wa.first)
			assert ("wa_succeeds_ww", wa ≽ ww)
			assert ("bw_o_ab_o_wa_o_ww", properties.composed_ok (ab, wa, ww))

			ab := pair_to_be_tested
			wa := some_preceding_pair_wa (some_object_a)
			assert ("composed", ab ≽ wa implies attached (ab ∘ wa))
			ww := some_preceding_pair_ww (some_object_w)
			assert ("composed_ok", properties.composed_ok (ab, wa, ww))
		end

feature {NONE} -- Factory (element to be tested)

	pair_to_be_tested: like some_direct_composable_pair_wab
			-- <Precursor>
		do
			Result := some_direct_composable_pair_wab
		end

feature {NONE} -- Conversion

	pair_from_ab (ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]): like pair_to_be_tested
			-- <Precursor>
		do
			Result := ab
		end

	pair_from_sti_ab (ab: STI_ORDERED_PAIR [A, B, EQ_A, EQ_B]): like pair_to_be_tested
			-- `ab' converted to a pair like `pair_to_be_tested'
		do
			Result := ab
		ensure
			first: eq_a (Result.a, ab.a)
			second: eq_b (Result.b, ab.b)
		end

	pair_from_wab (wab: STS_COMPOSABLE_ORDERED_PAIR [W, A, B, EQ_W, EQ_A, EQ_B]): like pair_to_be_tested
			-- `wab' converted to a pair like `pair_to_be_tested'
		do
			Result := wab
		ensure
			first: eq_a (Result.a, wab.a)
			second: eq_b (Result.b, wab.b)
		end

	pair_from_aab (aab: STS_COMPOSABLE_ORDERED_PAIR [A, A, B, EQ_A, EQ_A, EQ_B]): like pair_to_be_tested
			-- <Precursor>
		do
			Result := aab
		end

	pair_from_sti_aab (aab: STI_COMPOSABLE_ORDERED_PAIR [A, A, B, EQ_A, EQ_A, EQ_B]): like pair_to_be_tested
			-- `aab' converted to a pair like `pair_to_be_tested'
		do
			Result := aab
		ensure
			first: eq_a (Result.a, aab.a)
			second: eq_b (Result.b, aab.b)
		end

	pair_from_bab (bab: STS_COMPOSABLE_ORDERED_PAIR [B, A, B, EQ_B, EQ_A, EQ_B]): like pair_to_be_tested
			-- <Precursor>
		do
			Result := bab
		end

	pair_from_sti_bab (bab: STI_COMPOSABLE_ORDERED_PAIR [B, A, B, EQ_B, EQ_A, EQ_B]): like pair_to_be_tested
			-- `bab' converted to a pair like `pair_to_be_tested'
		do
			Result := bab
		ensure
			first: eq_a (Result.a, bab.a)
			second: eq_b (Result.b, bab.b)
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
