﻿note
	description: "Test suite for {STI_HOMOGENEOUS_ORDERED_PAIR}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	HOMOGENEOUS_ORDERED_PAIR_TESTS [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	ORDERED_PAIR_TESTS [A, A, EQ, EQ]
		rename
			test_composed_with_homogeneous as test_composed
		redefine
			test_make_from_pair,
			test_equals,
			pair_to_be_tested,
				--  TODO: join?
			pair_from_ab,
			pair_from_aab,
			pair_from_bab
		end

feature -- Access

	properties: HOMOGENEOUS_ORDERED_PAIR_PROPERTIES [A, EQ]
			-- Object that checks the set-theory properties of {STI_HOMOGENEOUS_ORDERED_PAIR}
		deferred
		end

--	eq: EQ
--			-- TODO: is this join possible?


feature -- Test routines (Initialization)

	test_make_from_pair
			-- Test {STI_HOMOGENEOUS_ORDERED_PAIR}.make_from_pair.
		note
			testing: "covers/{STI_HOMOGENEOUS_ORDERED_PAIR}.make_from_pair"
		do
			Precursor {ORDERED_PAIR_TESTS}
			assert ("convert_from_sti_ab", attached pair_from_sti_ab (some_pair_ab))
			assert ("convert_from_sti_aaa", attached pair_from_sti_aab (new_composable_pair_aaa))
			assert ("convert_from_from_homogeneous", attached pair_from_homogeneous (new_homogeneous_pair_a)) -- TODO make `some_homogeneous_pair_a'?
		end

feature -- Test routines (Quality)

	test_is_identical
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.is_identical.
			-- Test {STI_HOMOGENEOUS_ORDERED_PAIR}.is_identical.
		note
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.is_identical"
			testing: "covers/{STI_HOMOGENEOUS_ORDERED_PAIR}.is_identical"
		local
			a: A
			a1a2: like pair_to_be_tested
		do
			a := some_object_a
			create a1a2.make (a, same_object_a (a))
			assert ("a1a2.is_identical", a1a2.is_identical)
			assert ("a1a2.is_identical ok", properties.is_identical_ok (a1a2))

			a1a2 := pair_to_be_tested
			assert ("is_identical", a1a2.is_identical implies True)
			assert ("is_identical_ok", properties.is_identical_ok (a1a2))
		end

feature -- Test routines (Comparison)

	test_equals
			-- Test {STS_ORDERED_PAIR}.equals.
			-- Test {STI_ORDERED_PAIR}.equals.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.equals.
			-- Test {STI_HOMOGENEOUS_ORDERED_PAIR}.equals.
		note
			testing: "covers/{STS_ORDERED_PAIR}.equals"
			testing: "covers/{STI_ORDERED_PAIR}.equals"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.equals"
			testing: "covers/{STI_HOMOGENEOUS_ORDERED_PAIR}.equals"
		local
			a, b: A
		do
			Precursor {ORDERED_PAIR_TESTS}
			a := some_object_a
			b := same_object_a (a)
			assert ("a_b_equals_b_a", pair (a, b) ≍ pair (b, a))
			assert ("a_b_equals_b_a_ok", properties.equals_ok (a, b))

			a := some_object_a
			b := some_object_a
			assert ("equals", pair (a, b) ≍ pair (b, a) implies True)
			assert ("equals_ok", properties.equals_ok (a, b))
		end

feature -- Test routines (Conversion)

	test_as_tuple
			-- Test {STI_HOMOGENEOUS_ORDERED_PAIR}.as_tuple.
		note
			testing: "covers/{STI_HOMOGENEOUS_ORDERED_PAIR}.as_tuple"
		local
			a1a2: like pair_to_be_tested
		do
			a1a2 := pair_to_be_tested
			assert ("as_tuple", attached a1a2.as_tuple)
			assert ("as_tuple_ok", properties.as_tuple_ok (a1a2))
			assert ("converted_to_sts_n_tuple", attached pair_as_sts_n_tuple (pair_to_be_tested))
			assert ("converted_to_sti_n_tuple", attached pair_as_sti_n_tuple (pair_to_be_tested))
		end

feature -- Factory

	pair (a1, a2: A): like properties.pair
			-- Ordered pair in the form (`a1`, `a2`)
		do
			Result := properties.pair (a1, a2)
		end

feature {NONE} -- Factory (element to be tested)

	pair_to_be_tested: like new_homogeneous_pair_a
			-- Homogeneous pair meant to be under tests
		do
			check
					-- `some_direct_instance' and `new_homogeneous_pair_a' definitions
				aa: attached {like new_homogeneous_pair_a} some_direct_instance (agent new_homogeneous_pair_a) as aa
				monomorphic: aa.generating_type ~ {detachable like new_homogeneous_pair_a}
			then
				Result := aa
			end
		end

feature {NONE} -- Conversion

	pair_from_ab (ab: STS_ORDERED_PAIR [A, A, EQ, EQ]): like pair_to_be_tested
			-- <Precursor>
		do
			Result := ab
		end

	pair_from_sti_ab (ab: STI_ORDERED_PAIR [A, A, EQ, EQ]): like pair_to_be_tested
			-- `ab` converted to a pair like `pair_to_be_tested'
		do
			Result := ab
		ensure
			first: eq_a (Result.a, ab.a)
			second: eq_b (Result.b, ab.b)
		end

	pair_from_aab (aab: STS_COMPOSABLE_ORDERED_PAIR [A, A, A, EQ, EQ, EQ]): like pair_to_be_tested
			-- <Precursor>
		do
			Result := aab
		end

	pair_from_sti_aab (aab: STI_COMPOSABLE_ORDERED_PAIR [A, A, A, EQ, EQ, EQ]): like pair_to_be_tested
			-- `aab` converted to a pair like `pair_to_be_tested'
		do
			Result := aab
		ensure
			first: eq_a (Result.a, aab.a)
			second: eq_b (Result.b, aab.b)
		end

	pair_from_bab (bab: STS_COMPOSABLE_ORDERED_PAIR [A, A, A, EQ, EQ, EQ]): like pair_to_be_tested
			-- <Precursor>
		do
			Result := bab
		end

	pair_from_homogeneous (a1a2: STS_HOMOGENEOUS_ORDERED_PAIR [A, EQ]): like pair_to_be_tested
			-- `a1a2' converted to a pair like `pair_to_be_tested'
		do
			Result := a1a2
		ensure
			first: eq_a (Result.a, a1a2.a)
			second: eq_b (Result.b, a1a2.b)
		end

	pair_as_sts_n_tuple (a1a2: STI_HOMOGENEOUS_ORDERED_PAIR [A, EQ]): STS_N_TUPLE [A, EQ]
			-- `a1a2' converted to an {STS_N_TUPLE}
		do
			Result := a1a2
		ensure
			pair: Result.n = 2
			first: eq_a (Result [1], a1a2.first)
			second: eq_a (Result [2], a1a2.second)
		end

	pair_as_sti_n_tuple (a1a2: STI_HOMOGENEOUS_ORDERED_PAIR [A, EQ]): STI_N_TUPLE [A, EQ]
			-- `a1a2' converted to an {STI_N_TUPLE}
		do
			Result := a1a2
		ensure
			pair: Result.n = 2
			first: eq_a (Result [1], a1a2.first)
			second: eq_a (Result [2], a1a2.second)
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
