﻿note
	description: "Test suite for {STI_ORDERED_PAIR}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	ORDERED_PAIR_TESTS [
		A, B,
		EQ_A -> STS_EQUALITY [A] create default_create end,
		EQ_B -> STS_EQUALITY [B] create default_create end
		]

inherit
	ELEMENT_TESTS
		rename
			test_is_in as test_element_is_in,
			test_is_not_in as test_element_is_not_in,
			element_to_be_tested as pair_to_be_tested
		redefine
			pair_to_be_tested
		end

	SETS_TESTS_RESOURCES_BINARY [A, B, EQ_A, EQ_B]

feature -- Access

	properties: ORDERED_PAIR_PROPERTIES [A, B, EQ_A, EQ_B]
			-- Object that checks the set-theory properties of {STI_ORDERED_PAIR}
		deferred
		end

feature -- Test routines (Initialization)

	test_make
			-- Test {STI_ORDERED_PAIR}.make.
		note
			testing: "covers/{STI_ORDERED_PAIR}.make"
		do
			assert ("make", attached (create {like pair_to_be_tested}.make (some_object_a, some_object_b)))
		end

	test_make_from_pair
			-- Test {STI_ORDERED_PAIR}.make_from_pair.
		note
			testing: "covers/{STI_ORDERED_PAIR}.make_from_pair"
		do
			assert ("make_from_pair", attached (create {like pair_to_be_tested}.make_from_pair (some_pair_ab)))
			assert ("convert_from_ab", attached pair_from_ab (some_pair_ab))
			assert ("convert_from_aab", attached pair_from_aab (new_composable_pair_aab))
			assert ("convert_from_bab", attached pair_from_bab (new_composable_pair_bab))
		end

feature -- Test routines (Primitive)

	test_a
			-- Test {STI_ORDERED_PAIR}.a.
		note
			testing: "covers/{STI_ORDERED_PAIR}.a"
		do
			assert ("a", attached pair_to_be_tested.a implies True)
		end

	test_first
			-- Test {STI_ORDERED_PAIR}.first.
		note
			testing: "covers/{STI_ORDERED_PAIR}.first"
		do
			assert ("first", attached (pair_to_be_tested.first) implies True)
		end

	test_b
			-- Test {STI_ORDERED_PAIR}.b.
		note
			testing: "covers/{STI_ORDERED_PAIR}.b"
		do
			assert ("b", attached pair_to_be_tested.b implies True)
		end

	test_second
			-- Test {STI_ORDERED_PAIR}.second.
		note
			testing: "covers/{STI_ORDERED_PAIR}.second"
		do
			assert ("second", attached (pair_to_be_tested.second) implies True)
		end

	test_eq_a
			-- Test {STI_ORDERED_PAIR}.eq_a.
		note
			testing: "covers/{STI_ORDERED_PAIR}.eq_a"
		do
			assert ("eq_a", attached pair_to_be_tested.eq_a)
		end

	test_eq_b
			-- Test {STI_ORDERED_PAIR}.eq_b.
		note
			testing: "covers/{STI_ORDERED_PAIR}.eq_b"
		do
			assert ("eq_b", attached pair_to_be_tested.eq_b)
		end

feature -- Test routines (Membership)

	test_is_in
			-- Test {STS_ORDERED_PAIR}.is_in.
			-- Test {STI_ORDERED_PAIR}.is_in.
		note
			testing: "covers/{STS_ORDERED_PAIR}.is_in"
			testing: "covers/{STI_ORDERED_PAIR}.is_in"
		local
			ab: like pair_to_be_tested
			abs: like some_set_ab
			chk_ops: BOOLEAN
		do
			ab := pair_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := some_references_ab & ab
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_reference", ab ∈ abs)
			assert ("same_reference_ok", properties.is_in_ok (ab, abs))

			ab := pair_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := some_objects_ab & ab.twin
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_value", ab ∈ abs)
			assert ("same_value_ok", properties.is_in_ok (ab, abs))

			ab := pair_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := some_math_set_ab & same_pair_ab (ab)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_math_value", ab ∈ abs)
			assert ("same_math_value_ok", properties.is_in_ok (ab, abs))

			ab := pair_to_be_tested
			abs := some_set_ab
			assert ("is_in", ab ∈ abs implies True)
			assert ("is_in_ok", properties.is_in_ok (ab, abs))
		end

	test_is_not_in
			-- Test {STS_ORDERED_PAIR}.is_not_in.
			-- Test {STI_ORDERED_PAIR}.is_not_in.
		note
			testing: "covers/{STS_ORDERED_PAIR}.is_not_in"
			testing: "covers/{STI_ORDERED_PAIR}.is_not_in"
		local
			ab: like pair_to_be_tested
			abs: like some_set_ab
			chk_ops: BOOLEAN
		do
			ab := pair_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := some_references_ab / ab
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_reference", ab ∉ abs)
			assert ("same_reference_ok", properties.is_not_in_ok (ab, abs))

			ab := pair_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := some_objects_ab / ab.twin
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_value", ab ∉ abs)
			assert ("same_value_ok", properties.is_not_in_ok (ab, abs))

			ab := pair_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := some_math_set_ab / same_pair_ab (ab)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_math_value", ab ∉ abs)
			assert ("same_math_value_ok", properties.is_not_in_ok (ab, abs))

			ab := pair_to_be_tested
			abs := some_set_ab
			assert ("some_pairs", ab ∉ abs implies True)
			assert ("is_not_in_ok", properties.is_not_in_ok (ab, abs))
		end

feature -- Test routines (Output)

	test_out
			-- Test {STI_ORDERED_PAIR}.out.
		note
			testing: "covers/{STI_ORDERED_PAIR}.out"
		do
			assert ("out", attached pair_to_be_tested.out)
		end

	test_first_out
			-- Test {STI_ORDERED_PAIR}.first_out.
		note
			testing: "covers/{STI_ORDERED_PAIR}.first_out"
		do
			assert ("first_out", attached {like pair_to_be_tested}.first_out (some_object_a))
		end

	test_second_out
			-- Test {STI_ORDERED_PAIR}.second_out.
		note
			testing: "covers/{STI_ORDERED_PAIR}.second_out"
		do
			assert ("second_out", attached {like pair_to_be_tested}.second_out (some_object_b))
		end

feature -- Test routines (Comparison)

	test_equals
			-- Test {STS_ORDERED_PAIR}.equals.
			-- Test {STI_ORDERED_PAIR}.equals.
		note
			testing: "covers/{STS_ORDERED_PAIR}.equals"
			testing: "covers/{STI_ORDERED_PAIR}.equals"
		local
			ab_1: like pair_to_be_tested
			ab_2: like some_pair_ab
		do
			ab_1 := pair_to_be_tested
			ab_2 := ab_1
			assert ("same_reference", ab_1 ≍ ab_2)
			assert ("same_reference_ok", properties.equals_ok (ab_1, ab_2))

			ab_1 := pair_to_be_tested
			ab_2 := same_pair_ab (ab_1)
			assert ("same_pair", ab_1 ≍ ab_2)
			assert ("same_pair_ok", properties.equals_ok (ab_1, ab_2))

			ab_1 := pair_to_be_tested
			ab_2 := some_pair_ab
			assert ("some_pair", ab_1 ≍ ab_2 implies True)
			assert ("some_pair_ok", properties.equals_ok (ab_1, ab_2))
		end

	test_unequals
			-- Test {STS_ORDERED_PAIR}.unequals.
			-- Test {STI_ORDERED_PAIR}.unequals.
		note
			testing: "covers/{STS_ORDERED_PAIR}.unequals"
			testing: "covers/{STI_ORDERED_PAIR}.unequals"
		local
			ab_1: like pair_to_be_tested
			ab_2: like some_pair_ab
		do
			ab_1 := pair_to_be_tested
			ab_2 := ab_1
			assert ("same_reference", not (ab_1 ≭ ab_2))
			assert ("same_reference_ok", properties.unequals_ok (ab_1, ab_2))

			ab_1 := pair_to_be_tested
			ab_2 := same_pair_ab (ab_1)
			assert ("same_pair", not (ab_1 ≭ ab_2))
			assert ("same_pair_ok", properties.unequals_ok (ab_1, ab_2))

			ab_1 := pair_to_be_tested
			ab_2 := some_pair_ab
			assert ("some_pair", not (ab_1 ≭ ab_2) implies True)
			assert ("some_pair_ok", properties.unequals_ok (ab_1, ab_2))
		end

	test_succeeds
			-- Test {STS_ORDERED_PAIR}.succeeds.
			-- Test {STI_ORDERED_PAIR}.succeeds.
		note
			testing: "covers/{STS_ORDERED_PAIR}.succeeds"
			testing: "covers/{STI_ORDERED_PAIR}.succeeds"
		local
			ab: like pair_to_be_tested
			dsa_a: like some_preceding_pair_dsa_a
		do
			ab := pair_to_be_tested
			dsa_a := some_preceding_pair_dsa_a (ab.a)
			assert ("ab_succeeds_dsa_a", ab ≽ dsa_a)
			assert ("ab_succeeds_dsa_a_ok", properties.succeeds_ok (ab, dsa_a))

			ab := pair_to_be_tested
			dsa_a := some_preceding_pair_dsa_a (some_object_a)
			assert ("succeeds", ab ≽ dsa_a implies True)
			assert ("succeeds_ok", properties.succeeds_ok (ab, dsa_a))
		end

feature -- Test routines (Operation)

	test_inverse
			-- Test {STI_ORDERED_PAIR}.inverse.
		note
			testing: "covers/{STI_ORDERED_PAIR}.inverse"
		local
			ab: like pair_to_be_tested
		do
			ab := pair_to_be_tested
			assert ("inverse", attached ab.inverse)
			assert ("inverse_ok", properties.inverse_ok (ab))
		end

	test_composed_with_homogeneous
			-- Test {STI_ORDERED_PAIR}.composed_with_homogeneous.
		note
			testing: "covers/{STI_ORDERED_PAIR}.composed_with_homogeneous"
		local
			ab: like pair_to_be_tested
			aa_2, aa_1: like some_preceding_pair_aa
		do
			ab := pair_to_be_tested
			aa_2 := some_preceding_pair_aa (ab.a)
			assert ("ab_succeeds_aa_2", ab ≽ aa_2)
			assert ("ab_oh_aa_2", attached (ab.composed_with_homogeneous (aa_2)))
			aa_1 := some_preceding_pair_aa (aa_2.first)
			assert ("aa_2_succeeds_aa_1", aa_2 ≽ aa_1)
			assert ("ab_oh_aa_2_aa_1_ok", properties.composed_with_homogeneous_ok (ab, aa_2, aa_1))

			ab := pair_to_be_tested
			aa_2 := some_preceding_pair_aa (some_object_a)
			aa_1 := some_preceding_pair_aa (some_object_a)
			assert ("composed_with_homogeneous", ab ≽ aa_2 implies attached (ab.composed_with_homogeneous (aa_2)))
			assert ("composed_with_homogeneous_ok", properties.composed_with_homogeneous_ok (ab, aa_2, aa_1))
		end

	test_composed_with_inverse_like
			-- Test {STI_ORDERED_PAIR}.composed_with_inverse_like.
		note
			testing: "covers/{STI_ORDERED_PAIR}.composed_with_inverse_like"
		local
			ab: like pair_to_be_tested
			ba: like some_preceding_pair_ba
			bb: like some_preceding_pair_bb
		do
			ab := pair_to_be_tested
			ba := some_preceding_pair_ba (ab.a)
			assert ("ab_succeeds_ba", ab ≽ ba)
			assert ("ab_oi_ba", attached (ab.composed_with_inverse_like (ba)))
			bb := some_preceding_pair_bb (ba.first)
			assert ("ab_oi_ba_bb_ok", properties.composed_with_inverse_like_ok (ab, ba, bb))

			bb := some_preceding_pair_bb (some_object_b)
			ba := some_preceding_pair_ba (some_object_a)
			ab := pair_to_be_tested
			assert ("composed_with_inverse_like", ab ≽ ba implies attached (ab.composed_with_inverse_like (ba)))
			assert ("composed_with_inverse_like_ok", properties.composed_with_inverse_like_ok (ab, ba, bb))
		end

feature -- Test routines (Conversion)

	test_as_singleton
			-- Test {STI_ORDERED_PAIR}.as_singleton.
		note
			testing: "covers/{STI_ORDERED_PAIR}.as_singleton"
		local
			ab: like pair_to_be_tested
		do
			ab := pair_to_be_tested
			assert ("as_singleton", attached ab.as_singleton)
			assert ("as_singleton_ok", properties.as_singleton_ok (ab))
		end

feature {NONE} -- Factory (element to be tested)

	pair_to_be_tested: like new_pair_ab
			-- Ordered pair meant to be under tests
		do
			check
					-- `some_direct_instance' and `new_pair_ab' definitions
				ab: attached {like new_pair_ab} some_direct_instance (agent new_pair_ab) as ab
				monomorphic: ab.generating_type ~ {detachable like new_pair_ab}
			then
				Result := ab
			end
		end

feature {NONE} -- Conversion

	pair_from_ab (ab: STS_ORDERED_PAIR [A, B, EQ_A, EQ_B]): like pair_to_be_tested
			-- `ab' converted into a pair like `pair_to_be_tested'
		do
			Result := ab
		ensure
			first: eq_a (Result.a, ab.a)
			second: eq_b (Result.b, ab.b)
		end

	pair_from_aab (aab: STS_COMPOSABLE_ORDERED_PAIR [A, A, B, EQ_A, EQ_A, EQ_B]): like pair_to_be_tested
			-- `aab' converted into a pair like `pair_to_be_tested'
		do
			Result := aab
		ensure
			first: eq_a (Result.a, aab.a)
			second: eq_b (Result.b, aab.b)
		end

	pair_from_bab (bab: STS_COMPOSABLE_ORDERED_PAIR [B, A, B, EQ_B, EQ_A, EQ_B]): like pair_to_be_tested
			-- `bab' converted into a pair like `pair_to_be_tested'
		do
			Result := bab
		ensure
			first: eq_a (Result.a, bab.a)
			second: eq_b (Result.b, bab.b)
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
