﻿note
	description: "Test suite for {STI_SET_PARTITION}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	SET_PARTITION_TESTS [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	SET_FAMILY_TESTS [A, EQ]
		rename
			test_make_singleton as test_family_make_singleton,
			test_make_extended as test_family_make_extended,
			test_make_from_set as test_family_make_from_set,
			test_with as test_family_with,
			test_extended as test_family_extended,
			test_batch_extended as test_family_batch_extended,
			test_family_union as test_partition_union,
			test_family_intersection as test_partition_intersection,
			test_o_f as test_o_p,
			test_empty_family as test_empty_partition,
			test_unit_f as test_unit_p,
			test_converted as test_family_converted,
			set_family_to_be_tested as partition_to_be_tested,
			family_from_set_partition as partition_from_sp
		redefine
			test_family_make_singleton,
			test_family_make_extended,
			test_family_make_from_set,
			test_is_empty,
			test_others,
--			test_has,
--			test_does_not_have,
--			test_family_with,
--			test_without,
--			test_is_singleton,
			test_is_partition,
			test_cardinality,
			test_member_sets_are_disjoint,
--			test_equals,
--			test_unequals,
--			test_is_subset,
--			test_is_not_subset,
--			test_is_superset,
--			test_is_not_superset,
--			test_is_comparable,
--			test_is_not_comparable,
--			test_is_strict_subset,
--			test_is_not_strict_subset,
--			test_is_strict_superset,
--			test_is_not_strict_superset,
			test_exists,
			test_exists_pair,
			test_exists_distinct_pair,
			test_partition_union,
			test_partition_intersection,
			test_some_set_has,
			test_every_set_has,
			partition_to_be_tested,
			family_from_sa,
			family_from_sti_sa,
			family_from_sa_a,
			family_from_sti_sa_a,
			family_from_saa,
			family_from_sti_saa,
			family_from_sa_ta,
			family_from_sti_sa_ta,
			family_from_sa_ssa,
			family_from_sti_sa_ssa,
			family_from_sa_sta,
			family_from_sti_sa_sta,
			family_from_sa_aa,
			family_from_sti_sa_aa,
			family_from_set_of_sets,
			family_from_sti_set_of_sets,
			family_from_sf,
			partition_from_sp,
			family_from_subset_family,
			family_from_sti_subset_family,
			family_from_subset_family_structure,
			family_from_sti_subset_family_structure
		end

feature -- Access

	properties: SET_PARTITION_PROPERTIES [A, EQ]
			-- Object that checks the set-theory properties of {STI_SET_PARTITION}
		deferred
		end

feature -- Test routines (Initialization)

	test_family_make_singleton
			-- Test {STI_SET_PARTITION}.family_make_singleton.
		note
			testing: "covers/{STI_SET_PARTITION}.family_make_singleton"
		local
			set_make_singleton_got_rescued: PREDICATE [like partition_to_be_tested, like some_set_a, like some_set_a]
			sp: like partition_to_be_tested
			s0, s: like some_set_a
		do
			set_make_singleton_got_rescued := agent (ia_sp: like partition_to_be_tested; ia_s0, ia_s: like some_set_a): BOOLEAN
				require
					is_not_empty: not ia_s.is_empty
				local
					s0_has_failed: BOOLEAN
				do
					if not s0_has_failed then
						ia_sp.family_make_singleton (ia_s0)
					else
						ia_sp.family_make_singleton (ia_s)
					end
					Result := True
				ensure
					ia_s0: not ia_s0.is_empty ⇒ ia_sp ≍ ia_s0.as_singleton
					ia_s: ia_s0.is_empty and not ia_s.is_empty ⇒ ia_sp ≍ ia_s.as_singleton
				rescue
					if not s0_has_failed then
						s0_has_failed := True
						retry
					end
				end

			sp := partition_to_be_tested
			s0 := some_set_a.o
			from
				s := some_set_a
			until
				not s.is_empty
			loop
				s := some_set_a
			end
			assert ("exception_set_make_singleton", set_make_singleton_got_rescued (sp.twin, s0, s))
			assert ("exception_set_make_singleton_ok", properties.family_make_singleton_ok (sp, s0, s))

			sp := partition_to_be_tested
			s0 := some_set_a
			from
				s := some_set_a
			until
				not s.is_empty
			loop
				s := some_set_a
			end
			assert ("set_make_singleton", set_make_singleton_got_rescued (sp.twin, s0, s))
			assert ("set_make_singleton_ok", properties.family_make_singleton_ok (sp, s0, s))
		end

	test_make_singleton
			-- Test {STI_SET_PARTITION}.make_singleton.
		note
			testing: "covers/{STI_SET_PARTITION}.make_singleton"
		local
			s: like some_set_a
		do
			from
				s := some_set_a
			until
				not s.is_empty
			loop
				s := some_set_a
			end
			assert ("make_singleton", attached (create {like partition_to_be_tested}.make_singleton (s)))
			assert ("make_singleton_ok", properties.make_singleton_ok (s))
		end

	test_family_make_extended
			-- Test {STI_SET_PARTITION}.family_make_extended.
		note
			testing: "covers/{STI_SET_PARTITION}.family_make_extended"
		local
			set_make_extended_got_rescued: PREDICATE [like partition_to_be_tested, like some_set_a, like some_set_a, like some_set_sa, like some_set_sa]
			sp: like partition_to_be_tested
			sp0: like resources_a.some_set_partition_a
			s0, s: like some_set_a
			ss0, ss: like some_set_sa
			chk_ops: BOOLEAN
		do
			set_make_extended_got_rescued := agent (ia_sp: like partition_to_be_tested; ia_s0, ia_s: like some_set_a; ia_ss0, ia_ss: like some_set_sa): BOOLEAN
					-- TODO: symplify?
				require
					ia_ss0_does_not_have_ia_s0: ia_ss0 ∌ ia_s0
					ia_ss0_does_not_have_ia_s: ia_ss0 ∌ ia_s
					ia_ss_does_not_have_ia_s0: ia_ss ∌ ia_s0
					ia_ss_does_not_have_ia_s: ia_ss ∌ ia_s
					is_partition: ia_sp.family_converted (ia_ss).is_partition
					is_not_empty: not ia_s.is_empty
					set_is_disjoint: ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s)
				local
					ss0_violates_is_partition,
					s0_violates_is_not_empty,
					ss0_violates_set_is_disjoint_s0,
					ss0_violates_set_is_disjoint_s,
					ss_violates_set_is_disjoint_s0: BOOLEAN
				do
					if not ss0_violates_is_partition then
						if not s0_violates_is_not_empty then
							if not ss0_violates_set_is_disjoint_s0 then
								ia_sp.family_make_extended (ia_s0, ia_ss0)
							elseif not ss0_violates_set_is_disjoint_s then
								ia_sp.family_make_extended (ia_s, ia_ss0)
							elseif not ss_violates_set_is_disjoint_s0 then
								ia_sp.family_make_extended (ia_s0, ia_ss)
							else
								ia_sp.family_make_extended (ia_s, ia_ss)
							end
						elseif not ss0_violates_set_is_disjoint_s then
							ia_sp.family_make_extended (ia_s, ia_ss0)
						else
							ia_sp.family_make_extended (ia_s, ia_ss)
						end
					elseif not s0_violates_is_not_empty then
						if not ss_violates_set_is_disjoint_s0 then
							ia_sp.family_make_extended (ia_s0, ia_ss)
						else
							ia_sp.family_make_extended (ia_s, ia_ss)
						end
					else
						ia_sp.family_make_extended (ia_s, ia_ss)
					end
					Result := True
				ensure
					ia_ss0_ia_s0: ia_sp.family_converted (ia_ss0).is_partition and
						not ia_s0.is_empty and
						ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) ⇒

						ia_sp ≍ ia_ss0.extended (ia_s0)

					ia_ss0_ia_s: ia_sp.family_converted (ia_ss0).is_partition and
						not ia_s0.is_empty and
						not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) and
						ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) or

						ia_sp.family_converted (ia_ss0).is_partition and
						ia_s0.is_empty and
						ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) ⇒

						ia_sp ≍ ia_ss0.extended (ia_s)

					ia_ss_ia_s0: ia_sp.family_converted (ia_ss0).is_partition and
						not ia_s0.is_empty and
						not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) and
						not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) and
						ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) or

						not ia_sp.family_converted (ia_ss0).is_partition and
						not ia_s0.is_empty and
						ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) ⇒

						ia_sp ≍ ia_ss.extended (ia_s0)

					ia_ss_ia_s: ia_sp.family_converted (ia_ss0).is_partition and
						not ia_s0.is_empty and
						not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) and
						not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) and
						not ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) or

						ia_sp.family_converted (ia_ss0).is_partition and
						ia_s0.is_empty and
						not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) or

						not ia_sp.family_converted (ia_ss0).is_partition and
						not ia_s0.is_empty and
						not ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) or

						not ia_sp.family_converted (ia_ss0).is_partition and
						ia_s0.is_empty ⇒

						ia_sp ≍ ia_ss.extended (ia_s)
				rescue
					if not ss0_violates_is_partition and not ia_sp.family_converted (ia_ss0).is_partition then
						ss0_violates_is_partition := True
						retry
					end
					if not s0_violates_is_not_empty and ia_s0.is_empty then
						s0_violates_is_not_empty := True
						retry
					end
					if not ss0_violates_set_is_disjoint_s0 and not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s0) then
						ss0_violates_set_is_disjoint_s0 := True
						retry
					end
					if not ss0_violates_set_is_disjoint_s and not ia_sp.family_converted (ia_ss0).set_is_disjoint (ia_s) then
						ss0_violates_set_is_disjoint_s := True
						retry
					end
					if not ss_violates_set_is_disjoint_s0 and not ia_sp.family_converted (ia_ss).set_is_disjoint (ia_s0) then
						ss_violates_set_is_disjoint_s0 := True
						retry
					end
				end

			sp := partition_to_be_tested
			s0 := some_set_a
			from
				s := some_set_a
			until
				not s.is_empty
			loop
				s := some_set_a
			end
			assert ("ss0_violates_is_partition: not s.is_empty", not s.is_empty) -- Previous loop

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					ss0 := some_set_sa / s0 / s
				until
					not sp.family_converted (ss0).is_partition
				loop
					ss0 := some_set_sa / s0 / s
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss0_violates_is_partition: ss0 ∌ s0", ss0 ∌ s0) -- ss0 := some_set_sa / s0 / s
			assert ("ss0_violates_is_partition: ss0 ∌ s", ss0 ∌ s) -- ss0 := some_set_sa / s0 / s

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0
				ss := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss0_violates_is_partition: ss ∌ s0", ss ∌ s0) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
			assert ("ss0_violates_is_partition: ss ∌ s", ss ∌ s) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
			assert ("ss0_violates_is_partition: sp.family_converted (ss).is_partition", sp.family_converted (ss).is_partition)  -- ss ≍ sp0
			assert (
				"ss0_violates_is_partition: sp.family_converted (ss).set_is_disjoint (s)", -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
				sp.family_converted (ss).set_is_disjoint (s)
				)
			assert ("ss0_violates_is_partition", set_make_extended_got_rescued (sp.twin, s0, s, ss0, ss))
			assert ("ss0_violates_is_partition_ok", properties.family_make_extended_ok (sp, s0, s, ss0, ss))

			sp := partition_to_be_tested
			s0 := some_set_a.o -- TODO: o.
			from
				s := some_set_a
			until
				not s.is_empty
			loop
				s := some_set_a
			end
			assert ("s0_violates_is_not_empty: not s.is_empty", not s.is_empty) -- Previous loop

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp0 := resources_a.partitioned_set_a (some_set_a ∖ s)
				ss0 := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s0_violates_is_not_empty: ss0 ∌ s0", ss0 ∌ s0) -- ss0 is a partition and s0 = ∅.
			assert ("s0_violates_is_not_empty: ss0 ∌ s", ss0 ∌ s) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) and ss0 ≍ sp0

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp0 := resources_a.partitioned_set_a (some_set_a ∖ s)
				ss := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s0_violates_is_not_empty: ss ∌ s0", ss ∌ s0) -- ss is a partition and s0 = ∅.
			assert ("s0_violates_is_not_empty: ss ∌ s", ss ∌ s) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) and ss ≍ sp0
			assert ("s0_violates_is_not_empty: sp.family_converted (ss).is_partition", sp.family_converted (ss).is_partition) -- ss ≍ sp0
			assert (
				"s0_violates_is_not_empty: sp.family_converted (ss).set_is_disjoint (s)", -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) and ss ≍ sp0
				sp.family_converted (ss).set_is_disjoint (s)
				)
			assert ("s0_violates_is_not_empty", set_make_extended_got_rescued (sp.twin, s0, s, ss0, ss))
			assert ("s0_violates_is_not_empty_ok", properties.family_make_extended_ok (sp, s0, s, ss0, ss))

			sp := partition_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s0 := some_set_a
				until
					# s0 > 1
				loop
					s0 := some_set_a
				end
				from
					s := some_set_a
				until
					not s.is_empty and # (s0 ∖ s) > 1
				loop
					s := some_set_a
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss0_violates_set_is_disjoint_s0: not s.is_empty", not s.is_empty) -- Previous loop

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					sp0 := resources_a.partitioned_set_a (s0 ∖ s)
				until
					# sp0 > 1
				loop
					sp0 := resources_a.partitioned_set_a (s0 ∖ s)
				end
				ss0 := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss0_violates_set_is_disjoint_s0: ss0 ∌ s0", ss0 ∌ s0) -- sp0 := resources_a.partitioned_set_a (s0 ∖ s) and # s0 > 1 and # sp0 > 1 and ss0 ≍ sp0
			assert ("ss0_violates_set_is_disjoint_s0: ss0 ∌ s", ss0 ∌ s) -- sp0 := resources_a.partitioned_set_a (s0 ∖ s) and ss0 ≍ sp0

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0
				ss := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss0_violates_set_is_disjoint_s0: ss ∌ s0", ss ∌ s0) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
			assert ("ss0_violates_set_is_disjoint_s0: ss ∌ s", ss ∌ s) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
			assert ("ss0_violates_set_is_disjoint_s0: sp.family_converted (ss).is_partition", sp.family_converted (ss).is_partition) -- ss ≍ sp0
			assert (
				"ss0_violates_set_is_disjoint_s0: sp.family_converted (ss).set_is_disjoint (s)", -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
				sp.family_converted (ss).set_is_disjoint (s)
				)
			assert ("ss0_violates_set_is_disjoint_s0", set_make_extended_got_rescued (sp.twin, s0, s, ss0, ss))
			assert ("ss0_violates_set_is_disjoint_s0_ok", properties.family_make_extended_ok (sp, s0, s, ss0, ss))

			sp := partition_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s0 := some_set_a
				until
					# s0 > 1
				loop
					s0 := some_set_a
				end
				from
					s := some_set_a
				until
					# (s0 ∩ s) > 1
				loop
					s := some_set_a
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss0_violates_set_is_disjoint_s: not s.is_empty", not s.is_empty) -- # (s0 ∩ s) > 1

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					sp0 := resources_a.partitioned_set_a (s0 ∩ s)
				until
					# sp0 > 1
				loop
					sp0 := resources_a.partitioned_set_a (s0 ∩ s)
				end
				ss0 := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss0_violates_set_is_disjoint_s: ss0 ∌ s0", ss0 ∌ s0) -- sp0 := resources_a.partitioned_set_a (s0 ∩ s) and # (s0 ∩ s) > 1 and # sp0 > 1 and ss0 ≍ sp0
			assert ("ss0_violates_set_is_disjoint_s: ss0 ∌ s", ss0 ∌ s) -- sp0 := resources_a.partitioned_set_a (s0 ∩ s) and # (s0 ∩ s) > 1 and # sp0 > 1 and ss0 ≍ sp0

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0
				ss := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss0_violates_set_is_disjoint_s: ss ∌ s0", ss ∌ s0) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
			assert ("ss0_violates_set_is_disjoint_s: ss ∌ s", ss ∌ s) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
			assert ("ss0_violates_set_is_disjoint_s: sp.family_converted (ss).is_partition", sp.family_converted (ss).is_partition) -- ss ≍ sp0
			assert (
				"ss0_violates_set_is_disjoint_s: sp.family_converted (ss).set_is_disjoint (s)", -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0 and ss ≍ sp0
				sp.family_converted (ss).set_is_disjoint (s)
				)
			assert ("ss0_violates_set_is_disjoint_s", set_make_extended_got_rescued (sp.twin, s0, s, ss0, ss))
			assert ("ss0_violates_set_is_disjoint_s_ok", properties.family_make_extended_ok (sp, s0, s, ss0, ss))

			sp := partition_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s0 := some_set_a
				until
					# s0 > 1
				loop
					s0 := some_set_a
				end
				from
					s := some_set_a
				until
					not s.is_empty and # (s0 ∖ s) > 1
				loop
					s := some_set_a
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss_violates_set_is_disjoint_s0: not s.is_empty", not s.is_empty) -- Previous loop

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					ss0 := some_set_sa / s0 / s
				until
					not sp.family_converted (ss0).is_partition
				loop
					ss0 := some_set_sa / s0 / s
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss_violates_set_is_disjoint_s0: ss0 ∌ s0", ss0 ∌ s0) -- ss0 := some_set_sa / s0 / s
			assert ("ss_violates_set_is_disjoint_s0: ss0 ∌ s", ss0 ∌ s) -- ss0 := some_set_sa / s0 / s

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					sp0 := resources_a.partitioned_set_a (s0 ∖ s)
				until
					# sp0 > 1
				loop
					sp0 := resources_a.partitioned_set_a (s0 ∖ s)
				end
				ss := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss_violates_set_is_disjoint_s0: ss ∌ s0", ss ∌ s0) -- sp0 := resources_a.partitioned_set_a (s0 ∖ s) and # (s0 ∖ s) > 1 and # sp0 > 1 and ss ≍ sp0
			assert ("ss_violates_set_is_disjoint_s0: ss ∌ s", ss ∌ s) -- sp0 := resources_a.partitioned_set_a (s0 ∖ s) ss ≍ sp0
			assert ("ss_violates_set_is_disjoint_s0: sp.family_converted (ss).is_partition", sp.family_converted (ss).is_partition) -- ss ≍ sp0
			assert (
				"ss_violates_set_is_disjoint_s0: sp.family_converted (ss).set_is_disjoint (s)", -- sp0 := resources_a.partitioned_set_a (s0 ∖ s) ss ≍ sp0
				sp.family_converted (ss).set_is_disjoint (s)
				)
			assert ("ss_violates_set_is_disjoint_s0", set_make_extended_got_rescued (sp.twin, s0, s, ss0, ss))
			assert ("ss_violates_set_is_disjoint_s0_ok", properties.family_make_extended_ok (sp, s0, s, ss0, ss))

			sp := partition_to_be_tested
			s0 := some_set_a
			from
				s := some_set_a
			until
				not s.is_empty
			loop
				s := some_set_a
			end
			assert ("set_make_extended: not s.is_empty", not s.is_empty) -- Previous loop

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				ss0 := some_set_sa / s0 / s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("set_make_extended: ss0 ∌ s0", ss0 ∌ s0) -- ss0 := some_set_sa / s0 / s
			assert ("set_make_extended: ss0 ∌ s", ss0 ∌ s) -- ss0 := some_set_sa / s0 / s

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0
				ss := some_set_sa.converted (sp0)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("set_make_extended: ss ∌ s0", ss ∌ s0) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0
			assert ("set_make_extended: ss ∌ s", ss ∌ s) -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0
			assert ("set_make_extended: sp.family_converted (ss).is_partition", sp.family_converted (ss).is_partition) -- ss ≍ sp0
			assert (
				"set_make_extended: sp.family_converted (ss).set_is_disjoint (s)", -- sp0 := resources_a.partitioned_set_a (some_set_a ∖ s) / s0
				sp.family_converted (ss).set_is_disjoint (s)
				)
			assert ("set_make_extended", set_make_extended_got_rescued (sp.twin, s0, s, ss0, ss))
			assert ("set_make_extended_ok", properties.family_make_extended_ok (sp, s0, s, ss0, ss))
		end

	test_make_extended
			-- Test {STI_SET_PARTITION}.make_extended.
		note
			testing: "covers/{STI_SET_PARTITION}.make_extended"
		local
			sp: like partition_to_be_tested
			s: like some_set_a
			chk_disj: BOOLEAN
		do
			chk_disj := {ISE_RUNTIME}.check_assert (False)
				from
					sp := partition_to_be_tested
					s := some_set_a
				until
					not s.is_empty and sp.set_is_disjoint (s)
				loop
					sp := partition_to_be_tested
					s := some_set_a
				end
			chk_disj := {ISE_RUNTIME}.check_assert (chk_disj)

			assert ("make_extended_s_sp", attached (create {like partition_to_be_tested}.make_extended (s, sp)))
			assert ("make_extended_s_sp_ok", properties.make_extended_ok (s, sp))

			s := some_set_a
			sp := partition_to_be_tested
			assert ("make_extended", not s.is_empty and sp.set_is_disjoint (s) ⇒ attached (create {like partition_to_be_tested}.make_extended (s, sp)))
			assert ("make_extended_ok", properties.make_extended_ok (s, sp))
		end

	test_family_make_from_set
			-- Test {STI_SET_PARTITION}.family_make_from_set.
		note
			testing: "covers/{STI_SET_PARTITION}.family_make_from_set"
		local
			family_make_from_set_got_rescued: PREDICATE [like partition_to_be_tested, like some_set_sa, like some_set_sa]
			sp: like partition_to_be_tested
			sts_ss: STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]
			ss0, ss: like some_set_sa
			chk_ops: BOOLEAN
		do
			family_make_from_set_got_rescued := agent (ia_sp: like partition_to_be_tested; ia_ss0, ia_ss: like some_set_sa): BOOLEAN
				require
					keep_non_emptiness: ia_ss.is_empty ⇒ ia_sp.is_empty
					rl_ss: attached ia_sp.others.family_converted (ia_ss) as rl_ss
					is_partition: rl_ss.is_partition
				local
					ss0_has_failed: BOOLEAN
				do
					if not ss0_has_failed then
						ia_sp.family_make_from_set (ia_ss0)
					else
						ia_sp.family_make_from_set (ia_ss)
					end
					Result := True
				ensure
					el_ss0: attached ia_sp.others.family_converted (ia_ss0) as el_ss0
					ia_ss0: (el_ss0.is_empty ⇒ ia_sp.is_empty) and el_ss0.is_partition ⇒ ia_sp ≍ ia_ss0
					ia_ss: el_ss0.is_empty and not ia_sp.is_empty or not el_ss0.is_partition ⇒ ia_sp ≍ ia_ss
				rescue
					if not ss0_has_failed then
						ss0_has_failed := True
						retry
					end
					print (ia_sp) print (", ")
					print (ia_ss0) print (", ")
					print (ia_ss) print ('%N')
				end

			from
				sp := partition_to_be_tested
			until
				not sp.is_empty
			loop
				sp := partition_to_be_tested
			end
			ss0 := some_set_sa.o
			from
				sts_ss := resources_a.some_set_partition_a
			until
				not sts_ss.is_empty
			loop
				sts_ss := resources_a.some_set_partition_a
			end
			ss := sts_ss
			assert ("keep_non_emptiness_violated", family_make_from_set_got_rescued (sp, ss0, ss))

			from
				sp := partition_to_be_tested
			until
				not sp.is_empty
			loop
				sp := partition_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					ss0 := some_set_sa
				until
					not ss0.is_empty and not sp.others.family_converted (ss0).is_partition
				loop
					ss0 := some_set_sa
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			from
				sts_ss := resources_a.some_set_partition_a
			until
				not sts_ss.is_empty
			loop
				sts_ss := resources_a.some_set_partition_a
			end
			ss := sts_ss
			assert ("is_partition_violated", family_make_from_set_got_rescued (sp, ss0, ss))

			sp := partition_to_be_tested
			ss0 := some_set_sa
			from
				sts_ss := resources_a.some_set_partition_a
			until
				sts_ss.is_empty ⇒ sp.is_empty
			loop
				sts_ss := resources_a.some_set_partition_a
			end
			ss := sts_ss
			assert ("set_make_from_set", family_make_from_set_got_rescued (sp, ss0, ss))
		end

	test_make_from_partition
			-- Test {STI_SET_PARTITION}.make_from_partition.
		note
			testing: "covers/{STI_SET_PARTITION}.make_from_partition"
		local
			sp: like resources_a.some_set_partition_a
		do
			sp := resources_a.some_set_partition_a
			assert ("make_from_partition", attached (create {like partition_to_be_tested}.make_from_partition (sp)))
			assert ("convert_from_sp", attached partition_from_sp (resources_a.some_set_partition_a))
		end

feature -- Test routines (Primitive)

	test_is_empty
			-- <Precursor>
			-- Test {STI_SET_OF_SETS}.is_empty.
			-- Test {STI_SET_FAMILY}.is_empty.
			-- Test {STI_SET_PARTITION}.is_empty.
		note
			testing: "covers/{STI_SET}.is_empty"
			testing: "covers/{STI_SET_OF_SETS}.is_empty"
			testing: "covers/{STI_SET_FAMILY}.is_empty"
			testing: "covers/{STI_SET_PARTITION}.is_empty"
		local
			sp: like partition_to_be_tested
			s: like some_set_a
		do
			sp := partition_to_be_tested
			inspect
				next_random_item \\ 2
			when 0 then
				sp := sp.o_p
			when 1 then
				sp := sp ∖ sp
			end
			assert ("sp.is_empty", sp.is_empty)
			assert ("sp.is_empty ok", properties.is_empty_ok (sp))

			from
				sp := partition_to_be_tested
				s := some_set_a
			until
				not s.is_empty and (sp / s).set_is_disjoint (s)
			loop
				sp := partition_to_be_tested
				s := some_set_a
			end
			sp := sp & s
			assert ("not sp.is_empty", not sp.is_empty)
			assert ("not sp.is_empty ok", properties.is_empty_ok (sp))

			sp := partition_to_be_tested
			assert ("is_empty", sp.is_empty ⇒ True)
			assert ("is_empty_ok", properties.is_empty_ok (sp))
		end

	test_others
			-- <Precursor>
			-- Test {STI_SET_OF_SETS}.others.
			-- Test {STI_SET_FAMILY}.others.
			-- Test {STI_SET_PARTITION}.others.
		note
			testing: "covers/{STI_SET}.others"
			testing: "covers/{STI_SET_OF_SETS}.others"
			testing: "covers/{STI_SET_FAMILY}.others"
			testing: "covers/{STI_SET_PARTITION}.others"
		local
			sp: like partition_to_be_tested
			s: like some_set_a
		do
			sp := partition_to_be_tested
			inspect next_random_item \\ 2
			when 0 then
				sp := sp.o_p
			when 1 then
				sp := sp ∖ sp
			end
			assert ("∅.others", attached sp.others)
			assert ("∅.others ok", properties.others_ok (sp))

			from
				sp := partition_to_be_tested
				s := some_set_a
			until
				not s.is_empty and (sp / s).set_is_disjoint (s)
			loop
				sp := partition_to_be_tested
				s := some_set_a
			end
			sp := sp & s
			assert ("sp.others", attached sp.others)
			assert ("sp.others ok", properties.others_ok (sp))

			sp := partition_to_be_tested
			assert ("others", attached sp.others)
			assert ("others_ok", properties.others_ok (sp))
		end

feature -- Test routines (Membership)

--	test_has
--			-- <Precursor>
--			-- Test {STI_SET_OF_SETS}.has.
--			-- Test {STI_SET_FAMILY}.has.
--			-- Test {STI_SET_PARTITION}.has.
--		note
--			testing: "covers/{STI_SET}.has"
--			testing: "covers/{STI_SET_OF_SETS}.has"
--			testing: "covers/{STI_SET_FAMILY}.has"
--			testing: "covers/{STI_SET_PARTITION}.has"
--		local
--			sp: like partition_to_be_tested
--			s: like some_set_a
--		do
--			from
--				sp := partition_to_be_tested
--				s := some_set_a
--			until
--				not s.is_empty and (sp / s).set_is_disjoint (s)
--			loop
--				sp := partition_to_be_tested
--				s := some_set_a
--			end
--			sp := sp & same_set_a (s)
--			assert ("sp_has_s", sp ∋ s)
--			assert ("sp_has_s_ok", properties.has_ok (sp, s))

--			sp := partition_to_be_tested
--			s := some_set_a
--			assert ("has", sp ∋ s ⇒ True)
--			assert ("has_ok", properties.has_ok (sp, s))
--		end

--	test_does_not_have
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.does_not_have.
--			-- Test {STI_SET_OF_SETS}.does_not_have.
--			-- Test {STS_SET_FAMILY}.does_not_have.
--			-- Test {STI_SET_FAMILY}.does_not_have.
--			-- Test {STS_SET_PARTITION}.does_not_have.
--			-- Test {STI_SET_PARTITION}.does_not_have.
--		note
--			testing: "covers/{STS_SET}.does_not_have"
--			testing: "covers/{STI_SET}.does_not_have"
--			testing: "covers/{STS_SET_OF_SETS}.does_not_have"
--			testing: "covers/{STI_SET_OF_SETS}.does_not_have"
--			testing: "covers/{STS_SET_FAMILY}.does_not_have"
--			testing: "covers/{STI_SET_FAMILY}.does_not_have"
--			testing: "covers/{STS_SET_PARTITION}.does_not_have"
--			testing: "covers/{STI_SET_PARTITION}.does_not_have"
--		local
--			s, same_s: like some_set_a
--			sp: like partition_to_be_tested
--		do
--			s := some_set_a
--			sp := partition_to_be_tested / s
--			assert ("sp ∌ s", sp ∌ s)
--			assert ("sp ∌ s ok", properties.does_not_have_ok (sp, s))

--			from
--				sp := partition_to_be_tested
--				s := some_set_a
--			until
--				not s.is_empty and (sp / s).set_is_disjoint (s)
--			loop
--				sp := partition_to_be_tested
--				s := some_set_a
--			end
--			sp := sp & s
--			assert ("not (sp ∌ s)", not (sp ∌ s))
--			assert ("sp ∌ s ok", properties.does_not_have_ok (sp, s))

--			s := some_set_a
--			same_s := same_set_a (s)
--			sp := partition_to_be_tested / s
--			assert ("sp ∌ same_s", sp ∌ same_s)
--			assert ("sp ∌ same_s ok", properties.does_not_have_ok (sp, same_s))

--			from
--				sp := partition_to_be_tested
--				s := some_set_a
--			until
--				not s.is_empty and (sp / s).set_is_disjoint (s)
--			loop
--				sp := partition_to_be_tested
--				s := some_set_a
--			end
--			sp := sp & s
--			same_s := same_set_a (s)
--			assert ("not (sp ∌ same_s)", not (sp ∌ same_s))
--			assert ("sp ∌ same_s ok", properties.does_not_have_ok (sp, same_s))

--			s := some_set_a
--			sp := partition_to_be_tested
--			assert ("does_not_have", sp ∌ s ⇒ True)
--			assert ("does_not_have_ok", properties.does_not_have_ok (sp, s))
--		end

feature -- Test routines (Construction)

--	test_family_with
--			-- <Precursor>
--			-- Test {STI_SET_OF_SETS}.with.
--			-- Test {STI_SET_FAMILY}.with.
--			-- Test {STI_SET_PARTITION}.family_with.
--		note
--			testing: "covers/{STI_SET}.with"
--			testing: "covers/{STI_SET_OF_SETS}.with"
--			testing: "covers/{STI_SET_FAMILY}.with"
--			testing: "covers/{STI_SET_PARTITION}.family_with"
--		local
--			sp: like partition_to_be_tested
--			st: like partition_to_be_tested.as_tuple
--			s: like some_set_a
--			i: like some_n_tuple.n
--		do
--			s := some_set_a
--			sp := partition_to_be_tested / same_set_a (s)
--			assert ("sp ∌ s", attached sp.family_with (s))
--			assert ("sp ∌ s ok", properties.family_with_ok (sp, s))

--			from
--				sp := partition_to_be_tested
--			until
--				not sp.is_empty
--			loop
--				sp := partition_to_be_tested
--			end
--			st := sp.as_tuple
--			i := next_random_item.as_natural_32 -- TODO: Get rid of `as_natural_32'.
--				check
--					good_divisor: i.divisible (st.n) -- 0 < st.n = # sp ⇐ not sp.is_empty
--				end
--			i := i \\ st.n + 1
--				check
--						-- 1 ≤ i ≤ st.n ⇐ i := i \\ st.n + 1
--					positive: 0 < i
--					small_enough: i ≤ st.n
--				end
--			s := same_set_a (st [i])
--			assert ("sp ∋ s", attached sp.family_with (s))
--			assert ("sp ∋ s ok", properties.family_with_ok (sp, s))
--		end

	test_with
			-- Test {STI_SET_PARTITION}.with.
		note
			testing: "covers/{STI_SET_PARTITION}.with"
		local
			sp: like partition_to_be_tested
			s: like some_set_a
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					sp := partition_to_be_tested
					s := some_set_a
				until
					not s.is_empty and (sp / s).set_is_disjoint (s)
				loop
					sp := partition_to_be_tested
					s := some_set_a
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("sp_with_s", attached (sp & s))
			assert ("sp_with_s_ok", properties.with_ok (sp, s))

			sp := partition_to_be_tested
			s := some_set_a
			assert ("with", not s.is_empty and (sp / s).set_is_disjoint (s) ⇒ attached (sp & s))
			assert ("with_ok", properties.with_ok (sp, s))
		end

--	test_without
--			-- Test {STI_SET_PARTITION}.without.
--		note
--			testing: "covers/{STI_SET_PARTITION}.without"
--		local
--			ss: like some_set_sa
--			sp: like partition_to_be_tested
--			s: like some_set_a
--		do
--			s := some_set_a
--			ss := some_set_sa & same_set_a (s)
--			sp := partition_to_be_tested ∖ ss
--			assert ("sp ∌ s", attached (sp / s))
--			assert ("sp ∌ s ok", properties.family_with_ok (sp, s))

--			from
--				sp := partition_to_be_tested
--				s := some_set_a
--			until
--				not s.is_empty and (sp / s).set_is_disjoint (s)
--			loop
--				sp := partition_to_be_tested
--				s := some_set_a
--			end
--			sp := sp & same_set_a (s)
--			assert ("sp ∋ s", attached (sp / s))
--			assert ("sp ∋ s ok", properties.family_with_ok (sp, s))
--		end

	test_extended
			-- Test {STI_SET_PARTITION}.extended.
		note
			testing: "covers/{STI_SET_PARTITION}.extended"
		local
			sp: like partition_to_be_tested
			s: like some_set_a
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					sp := partition_to_be_tested
					s := some_set_a
				until
					not s.is_empty and sp.set_is_disjoint (s)
				loop
					sp := partition_to_be_tested
					s := some_set_a
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("sp_extended_s", attached sp.extended (s))
			assert ("sp_extended_s_ok", properties.extended_ok (sp, s))

			s := some_set_a
			sp := partition_to_be_tested
			assert ("extended", not s.is_empty and sp.set_is_disjoint (s) ⇒ attached sp.extended (s))
			assert ("extended_ok", properties.extended_ok (sp, s))
		end

	test_batch_extended
			-- Test {STI_SET_PARTITION}.batch_extended.
		note
			testing: "covers/{STI_SET_PARTITION}.batch_extended"
		local
			sp: like partition_to_be_tested
			sp0: like resources_a.some_set_partition_a
			ss: like resources_a.some_set_of_sets_a
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					ss := resources_a.some_set_of_sets_a
					sp := partition_to_be_tested
					sp0 := resources_a.some_set_partition_a ∖ sp
						check
							is_disjoint: ss.o_s.is_disjoint (sp0) -- ss.o.is_empty
						end
					ss := ss.o_s.batch_extended (sp0)
				until
					sp.sets_are_disjoint (ss)
				loop
					sp := partition_to_be_tested
					sp0 := resources_a.some_set_partition_a ∖ sp
						check
							is_disjoint: ss.o_s.is_disjoint (sp0) -- ss.o.is_empty
						end
					ss := ss.o_s.batch_extended (sp0)
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("ss_is_partition", ss.is_partition)
			assert ("sp_batch_extended_ss", attached sp.batch_extended (ss))
			assert ("sp_batch_extended_ss_ok", properties.batch_extended_ok (sp, ss))

			sp := partition_to_be_tested
			ss := resources_a.some_set_of_sets_a
			assert ("batch_extended", ss.is_partition and sp.sets_are_disjoint (ss) ⇒ attached sp.batch_extended (ss))
			assert ("batch_extended_ok", properties.batch_extended_ok (sp, ss))
		end

feature -- Test routines (Quality)

--	test_is_singleton
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_singleton.
--			-- Test {STI_SET_OF_SETS}.is_singleton.
--			-- Test {STS_SET_FAMILY}.is_singleton.
--			-- Test {STI_SET_FAMILY}.is_singleton.
--			-- Test {STS_SET_PARTITION}.is_singleton.
--			-- Test {STI_SET_PARTITION}.is_singleton.
--		note
--			testing: "covers/{STS_SET}.is_singleton"
--			testing: "covers/{STI_SET}.is_singleton"
--			testing: "covers/{STS_SET_OF_SETS}.is_singleton"
--			testing: "covers/{STI_SET_OF_SETS}.is_singleton"
--			testing: "covers/{STS_SET_FAMILY}.is_singleton"
--			testing: "covers/{STI_SET_FAMILY}.is_singleton"
--			testing: "covers/{STS_SET_PARTITION}.is_singleton"
--			testing: "covers/{STI_SET_PARTITION}.is_singleton"
--		local
--			sp: like partition_to_be_tested
--			s: like some_set_a
--		do
--			s := some_set_a & some_object_a
--			sp := partition_to_be_tested.o_p
--				check
--					is_not_empty: not s.is_empty -- s := some_set_a & some_object_a
--					set_is_pair_wise_disjoint: (sp / s).set_is_disjoint (s) -- sp.is_empty
--				end
--			sp := sp & s
--			assert ("sp.is_singleton", sp.is_singleton)
--			assert ("sp.is_singleton ok", properties.is_singleton_ok (sp))

--			from
--				sp := partition_to_be_tested
--			until
--				# sp /= 1
--			loop
--				sp := partition_to_be_tested
--			end
--			assert ("not sp.is_singleton", not sp.is_singleton)
--			assert ("not sp.is_singleton ok", properties.is_singleton_ok (sp))

--			sp := partition_to_be_tested
--			assert ("is_singleton", sp.is_singleton ⇒ True)
--			assert ("is_singleton_ok", properties.is_singleton_ok (sp))
--		end

	test_is_partition
			-- Test {STI_SET_PARTITION}.is_partition.
		note
			testing: "covers/{STI_SET_PARTITION}.is_partition"
		local
			sp: like partition_to_be_tested
		do
			sp := partition_to_be_tested
			assert ("is_partition", sp.is_partition)
			assert ("is_partition_ok", properties.is_partition_ok (sp))
		end

	test_member_sets_are_disjoint
			-- Test {STI_SET_PARTITION}.member_sets_are_disjoint.
		note
			testing: "covers/{STI_SET_PARTITION}.member_sets_are_disjoint"
		do
			assert ("member_sets_are_disjoint", partition_to_be_tested.member_sets_are_disjoint)
		end

feature -- Test routines (Measurement)

	test_cardinality
			-- <Precursor>
			-- Test {STI_SET_OF_SETS}.cardinality.
			-- Test {STI_SET_FAMILY}.cardinality.
			-- Test {STI_SET_PARTITION}.cardinality.
		note
			testing: "covers/{STI_SET}.cardinality"
			testing: "covers/{STI_SET_OF_SETS}.cardinality"
			testing: "covers/{STI_SET_FAMILY}.cardinality"
			testing: "covers/{STI_SET_PARTITION}.cardinality"
		local
			sp: like partition_to_be_tested
			s: like some_set_a
		do
			sp := partition_to_be_tested.o_p
			assert ("# sp = 0", # sp = 0)
			assert ("# sp = 0 ok", properties.cardinality_ok (sp))

			from
				s := some_set_a
			until
				not s.is_empty
			loop
				s := some_set_a
			end
				check
					set_is_pair_wise_disjoint: (sp / s).set_is_disjoint (s) -- sp.is_empty
				end
			sp := sp & same_set_a (s)
			assert ("# sp = 1", # sp = 1)
			assert ("# sp = 1 ok", properties.cardinality_ok (sp))

			from
				s := some_set_a
			until
				not s.is_empty and (sp / s).set_is_disjoint (s)
			loop
				s := some_set_a
			end
			sp := sp & same_set_a (s)
			assert ("# sp = 2", # sp = 2)
			assert ("# sp = 2 ok", properties.cardinality_ok (sp))

			from
				s := some_set_a
			until
				not s.is_empty and (sp / s).set_is_disjoint (s)
			loop
				s := some_set_a
			end
			sp := sp & same_set_a (s)
			assert ("# sp = 3", # sp = 3)
			assert ("# sp = 3 ok", properties.cardinality_ok (sp))

			sp := partition_to_be_tested
			assert ("cardinality", attached # sp)
			assert ("cardinality_ok", properties.cardinality_ok (sp))
		end

--feature -- Test routines (Comparison)

--	test_equals
--			-- <Precursor>
--			-- Test {STI_SET_OF_SETS}.equals.
--			-- Test {STI_SET_FAMILY}.equals.
--			-- Test {STI_SET_PARTITION}.equals.
--		note
--			testing: "covers/{STI_SET}.equals"
--			testing: "covers/{STI_SET_OF_SETS}.equals"
--			testing: "covers/{STI_SET_FAMILY}.equals"
--			testing: "covers/{STI_SET_PARTITION}.equals"
--		local
--			s: like some_set_a
--			sp: like partition_to_be_tested
--			ss1, ss2: like some_set_sa
--		do
--			sp := partition_to_be_tested
--			assert ("sp ≍ sp", sp ≍ sp)
--			assert ("sp ≍ sp ok", properties.equals_ok (sp, sp, sp))

--			if next_random_item \\ 2 = 0 then
--				from
--					sp := partition_to_be_tested
--					s := some_set_a
--				until
--					not s.is_empty and (sp / s).set_is_disjoint (s)
--				loop
--					sp := partition_to_be_tested
--					s := some_set_a
--				end
--				sp := sp & same_set_a (s)
--				ss1 := some_set_sa / same_set_a (s)
--			else
--				s := some_set_a
--				sp := partition_to_be_tested / same_set_a (s)
--				ss1 := some_set_sa & same_set_a (s)
--			end
--			assert ("not (sp ≍ ss1)", not (sp ≍ ss1))
--			assert ("(sp ≍ ss1) ok", properties.equals_ok (sp, ss1, some_set_sa))

--			sp := partition_to_be_tested
--			ss1 := same_set_sa (sp)
--			ss2 := same_set_sa (ss1)
--			assert ("sp ≍ ss1", sp ≍ ss1)
--			assert ("sp ≍ ss1 ok", properties.equals_ok (sp, ss1, ss2))

--			sp := partition_to_be_tested
--			ss1 := some_set_sa
--			ss2 := some_set_sa
--			assert ("equals", sp ≍ ss1 ⇒ True)
--			assert ("equals ok", properties.equals_ok (sp, ss1, ss2))
--		end

--	test_unequals
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.unequals.
--			-- Test {STI_SET_OF_SETS}.unequals.
--			-- Test {STS_SET_FAMILY}.unequals.
--			-- Test {STI_SET_FAMILY}.unequals.
--			-- Test {STS_SET_PARTITION}.unequals.
--			-- Test {STI_SET_PARTITION}.unequals.
--		note
--			testing: "covers/{STS_SET}.unequals"
--			testing: "covers/{STI_SET}.unequals"
--			testing: "covers/{STS_SET_OF_SETS}.unequals"
--			testing: "covers/{STI_SET_OF_SETS}.unequals"
--			testing: "covers/{STS_SET_FAMILY}.unequals"
--			testing: "covers/{STI_SET_FAMILY}.unequals"
--			testing: "covers/{STS_SET_PARTITION}.unequals"
--			testing: "covers/{STI_SET_PARTITION}.unequals"
--		local
--			s: like some_set_a
--			sp: like partition_to_be_tested
--			ss: like some_set_sa
--		do
--			sp := partition_to_be_tested
--			assert ("not (sp ≭ sp)", not (sp ≭ sp))
--			assert ("not (sp ≭ sp) ok", properties.unequals_ok (sp, sp))

--			if next_random_item \\ 2 = 0 then
--				from
--					sp := partition_to_be_tested
--					s := some_set_a
--				until
--					not s.is_empty and (sp / s).set_is_disjoint (s)
--				loop
--					sp := partition_to_be_tested
--					s := some_set_a
--				end
--				sp := sp & same_set_a (s)
--				ss := some_set_sa / same_set_a (s)
--			else
--				s := some_set_a
--				sp := partition_to_be_tested / same_set_a (s)
--				ss := some_set_sa & same_set_a (s)
--			end
--			assert ("sp ≭ ss", sp ≭ ss)
--			assert ("sp ≭ ss ok", properties.unequals_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := same_set_sa (sp)
--			assert ("not (sp ≭ ss)", not (sp ≭ ss))
--			assert ("not (sp ≭ ss) ok", properties.unequals_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := some_set_sa
--			assert ("unequals", sp ≭ ss ⇒ True)
--			assert ("unequals_ok", properties.unequals_ok (sp, ss))
--		end

--	test_is_subset
--			-- <Precursor>
--			-- Test {STI_SET_OF_SETS}.is_subset.
--			-- Test {STI_SET_FAMILY}.is_subset.
--			-- Test {STI_SET_PARTITION}.is_subset.
--		note
--			testing: "covers/{STI_SET}.is_subset"
--			testing: "covers/{STI_SET_OF_SETS}.is_subset"
--			testing: "covers/{STI_SET_FAMILY}.is_subset"
--			testing: "covers/{STI_SET_PARTITION}.is_subset"
--		local
--			s: like some_set_a
--			sp: like partition_to_be_tested
--			ss2, ss3: like some_set_sa
--		do
--			ss3 := some_set_sa
--			ss2 := some_set_sa ∩ same_set_sa (ss3)
--			sp := partition_to_be_tested ∩ same_set_sa (ss2)
--			assert ("sp ⊆ ss2", sp ⊆ ss2)
--			assert ("sp ⊆ ss2 ok", properties.is_subset_ok (sp, ss2, ss3))

--			from
--				s := some_set_a
--				sp := partition_to_be_tested
--			until
--				not s.is_empty and (sp / s).set_is_disjoint (s)
--			loop
--				s := some_set_a
--				sp := partition_to_be_tested
--			end
--			sp := sp & same_set_a (s)
--			ss2 := some_set_sa / same_set_a (s)
--			assert ("not (sp ⊆ ss2)", not (sp ⊆ ss2))
--			assert ("(sp ⊆ ss2) ok", properties.is_subset_ok (sp, ss2, some_set_sa))

--			sp := partition_to_be_tested
--			ss2 := some_set_sa
--			ss3 := some_set_sa
--			assert ("is_subset", sp ⊆ ss2 ⇒ True)
--			assert ("is_subset ok", properties.is_subset_ok (sp, ss2, ss3))
--		end

--	test_is_not_subset
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_not_subset.
--			-- Test {STI_SET_OF_SETS}.is_not_subset.
--			-- Test {STS_SET_FAMILY}.is_not_subset.
--			-- Test {STI_SET_FAMILY}.is_not_subset.
--			-- Test {STS_SET_PARTITION}.is_not_subset.
--			-- Test {STI_SET_PARTITION}.is_not_subset.
--		note
--			testing: "covers/{STS_SET}.is_not_subset"
--			testing: "covers/{STI_SET}.is_not_subset"
--			testing: "covers/{STS_SET_OF_SETS}.is_not_subset"
--			testing: "covers/{STI_SET_OF_SETS}.is_not_subset"
--			testing: "covers/{STS_SET_FAMILY}.is_not_subset"
--			testing: "covers/{STI_SET_FAMILY}.is_not_subset"
--			testing: "covers/{STS_SET_PARTITION}.is_not_subset"
--			testing: "covers/{STI_SET_PARTITION}.is_not_subset"
--		local
--			s: like some_set_a
--			sp: like partition_to_be_tested
--			ss: like some_set_sa
--			chk_ops: BOOLEAN
--		do
--			sp := partition_to_be_tested

--			chk_ops := {ISE_RUNTIME}.check_assert (False)
--				from
--					s := some_set_a
--				until
--					not s.is_empty and (sp / s).set_is_disjoint (s)
--				loop
--					s := some_set_a
--				end
--				sp := sp & s
--				ss := some_set_sa / s
--			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

--			assert ("sp ⊈ ss", sp ⊈ ss)
--			assert ("sp ⊈ ss ok", properties.is_not_subset_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := some_set_sa
--			assert ("is_not_subset", sp ⊈ ss ⇒ True)
--			assert ("is_not_subset_ok", properties.is_not_subset_ok (sp, ss))
--		end

--	test_is_superset
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_superset.
--			-- Test {STI_SET_OF_SETS}.is_superset.
--			-- Test {STS_SET_FAMILY}.is_superset.
--			-- Test {STI_SET_FAMILY}.is_superset.
--			-- Test {STS_SET_PARTITION}.is_superset.
--			-- Test {STI_SET_PARTITION}.is_superset.
--		note
--			testing: "covers/{STS_SET}.is_superset"
--			testing: "covers/{STI_SET}.is_superset"
--			testing: "covers/{STS_SET_OF_SETS}.is_superset"
--			testing: "covers/{STI_SET_OF_SETS}.is_superset"
--			testing: "covers/{STS_SET_FAMILY}.is_superset"
--			testing: "covers/{STI_SET_FAMILY}.is_superset"
--			testing: "covers/{STS_SET_PARTITION}.is_superset"
--			testing: "covers/{STI_SET_PARTITION}.is_superset"
--		local
--			s: like some_set_a
--			sp0: like resources_a.some_set_of_sets_a
--			sp: like partition_to_be_tested
--			ss: like some_set_sa
--		do
--			sp := partition_to_be_tested
--			sp0 := resources_a.some_set_partition_a | agent (ia_s: STS_SET [A, EQ]; ia_sp: STS_SET_PARTITION [A, EQ]): BOOLEAN
--				do
--					Result := ia_sp.set_is_disjoint (ia_s)
--				end (?, sp)
--				check
--					is_partition: sp0.is_partition -- sp0 := resources_a.some_set_partition_a | agent...
--					sets_are_disjoint: sp.sets_are_disjoint (sp0) -- Agent above: Result := ia_sp.set_is_disjoint (ia_s)
--				end
--			sp := sp.batch_extended (sp0)
--			ss := same_set_sa (sp0)
--			assert ("sp ⊇ ss", sp ⊇ ss)
--			assert ("sp ⊇ ss ok", properties.is_superset_ok (sp, ss))

--			s := some_set_a
--			sp := partition_to_be_tested / same_set_a (s)
--			ss := some_set_sa & same_set_a (s)
--			assert ("not (sp ⊇ ss)", not (sp ⊇ ss))
--			assert ("not (sp ⊇ ss) ok", properties.is_superset_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := some_set_sa
--			assert ("is_superset", sp ⊇ ss ⇒ True)
--			assert ("is_superset ok", properties.is_superset_ok (sp, ss))
--		end

--	test_is_not_superset
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_not_superset.
--			-- Test {STI_SET_OF_SETS}.is_not_superset.
--			-- Test {STS_SET_FAMILY}.is_not_superset.
--			-- Test {STI_SET_FAMILY}.is_not_superset.
--			-- Test {STS_SET_PARTITION}.is_not_superset.
--			-- Test {STI_SET_PARTITION}.is_not_superset.
--		note
--			testing: "covers/{STS_SET}.is_not_superset"
--			testing: "covers/{STI_SET}.is_not_superset"
--			testing: "covers/{STS_SET_OF_SETS}.is_not_superset"
--			testing: "covers/{STI_SET_OF_SETS}.is_not_superset"
--			testing: "covers/{STS_SET_FAMILY}.is_not_superset"
--			testing: "covers/{STI_SET_FAMILY}.is_not_superset"
--			testing: "covers/{STS_SET_PARTITION}.is_not_superset"
--			testing: "covers/{STI_SET_PARTITION}.is_not_superset"
--		local
--			s: like some_set_a
--			sp0: like resources_a.some_set_of_sets_a
--			sp: like partition_to_be_tested
--			ss: like some_set_sa
--		do
--			s := some_set_a
--			sp := partition_to_be_tested / same_set_a (s)
--			ss := some_set_sa & same_set_a (s)
--			assert ("sp ⊉ ss", sp ⊉ ss)
--			assert ("sp ⊉ ss ok", properties.is_not_superset_ok (sp, ss))

--			sp := partition_to_be_tested
--			sp0 := resources_a.some_set_partition_a | agent (ia_s: STS_SET [A, EQ]; ia_sp: STS_SET_PARTITION [A, EQ]): BOOLEAN
--				do
--					Result := ia_sp.set_is_disjoint (ia_s)
--				end (?, sp)
--				check
--					is_partition: sp0.is_partition -- sp0 := resources_a.some_set_partition_a | agent...
--					sets_are_disjoint: sp.sets_are_disjoint (sp0) -- Agent above: Result := ia_sp.set_is_disjoint (ia_s)
--				end
--			sp := sp.batch_extended (sp0)
--			ss := same_set_sa (sp0)
--			assert ("not (sp ⊉ ss)", not (sp ⊉ ss))
--			assert ("not (sp ⊉ ss) ok", properties.is_not_superset_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := some_set_sa
--			assert ("is_not_superset", sp ⊉ ss ⇒ True)
--			assert ("is_not_superset ok", properties.is_not_superset_ok (sp, ss))
--		end

--	test_is_comparable
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_comparable.
--			-- Test {STI_SET_OF_SETS}.is_comparable.
--			-- Test {STS_SET_FAMILY}.is_comparable.
--			-- Test {STI_SET_FAMILY}.is_comparable.
--			-- Test {STS_SET_PARTITION}.is_comparable.
--			-- Test {STI_SET_PARTITION}.is_comparable.
--		note
--			testing: "covers/{STS_SET}.is_comparable"
--			testing: "covers/{STI_SET}.is_comparable"
--			testing: "covers/{STS_SET_OF_SETS}.is_comparable"
--			testing: "covers/{STI_SET_OF_SETS}.is_comparable"
--			testing: "covers/{STS_SET_FAMILY}.is_comparable"
--			testing: "covers/{STI_SET_FAMILY}.is_comparable"
--			testing: "covers/{STS_SET_PARTITION}.is_comparable"
--			testing: "covers/{STI_SET_PARTITION}.is_comparable"
--		local
--			s1, s2: like some_set_a
--			sp: like partition_to_be_tested
--			ss: like some_set_sa
--		do
--			if next_random_item \\ 2 = 0 then
--				ss := some_set_sa
--				sp := partition_to_be_tested ∩ same_set_sa (ss)
--			else
--				sp := partition_to_be_tested
--				ss := same_set_sa (some_set_sa ∩ sp)
--			end
--			assert ("sp.is_comparable (ss)", sp.is_comparable (ss))
--			assert ("sp.is_comparable (ss) ok", properties.is_comparable_ok (sp, ss))

--			if next_random_item \\ 2 = 0 then
--				from
--					sp := partition_to_be_tested
--					s1 := some_set_a
--				until
--					not s1.is_empty and (sp / s1).set_is_disjoint (s1)
--				loop
--					sp := partition_to_be_tested
--					s1 := some_set_a
--				end
--				from
--					s2 := some_set_a
--				until
--					s1 ≭ s2
--				loop
--					s2 := some_set_a
--				end
--				sp := sp & same_set_a (s1) / same_set_a (s2)
--				ss := (some_set_sa / same_set_a (s1)) & same_set_a (s2)
--			else
--				from
--					sp := partition_to_be_tested
--					s2 := some_set_a
--				until
--					not s2.is_empty and (sp / s2).set_is_disjoint (s2)
--				loop
--					sp := partition_to_be_tested
--					s2 := some_set_a
--				end
--				from
--					s1 := some_set_a
--				until
--					s1 ≭ s2
--				loop
--					s1 := some_set_a
--				end
--				sp := (sp / same_set_a (s1)) & same_set_a (s2)
--				ss := some_set_sa & same_set_a (s1) / same_set_a (s2)
--			end
--			assert ("not sp.is_comparable (ss)", not sp.is_comparable (ss))
--			assert ("not sp.is_comparable (ss) ok", properties.is_comparable_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := some_set_sa
--			assert ("is_comparable", sp.is_comparable (ss) ⇒ True)
--			assert ("is_comparable_ok", properties.is_comparable_ok (sp, ss))
--		end

--	test_is_not_comparable
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_not_comparable.
--			-- Test {STI_SET_OF_SETS}.is_not_comparable.
--			-- Test {STS_SET_FAMILY}.is_not_comparable.
--			-- Test {STI_SET_FAMILY}.is_not_comparable.
--			-- Test {STS_SET_PARTITION}.is_not_comparable.
--			-- Test {STI_SET_PARTITION}.is_not_comparable.
--		note
--			testing: "covers/{STS_SET}.is_not_comparable"
--			testing: "covers/{STI_SET}.is_not_comparable"
--			testing: "covers/{STS_SET_OF_SETS}.is_not_comparable"
--			testing: "covers/{STI_SET_OF_SETS}.is_not_comparable"
--			testing: "covers/{STS_SET_FAMILY}.is_not_comparable"
--			testing: "covers/{STI_SET_FAMILY}.is_not_comparable"
--			testing: "covers/{STS_SET_PARTITION}.is_not_comparable"
--			testing: "covers/{STI_SET_PARTITION}.is_not_comparable"
--		local
--			s1, s2: like some_set_a
--			sp: like partition_to_be_tested
--			ss: like some_set_sa
--		do
--			if next_random_item \\ 2 = 0 then
--				from
--					sp := partition_to_be_tested
--					s1 := some_set_a
--				until
--					not s1.is_empty and (sp / s1).set_is_disjoint (s1)
--				loop
--					sp := partition_to_be_tested
--					s1 := some_set_a
--				end
--				from
--					s2 := some_set_a
--				until
--					s1 ≭ s2
--				loop
--					s2 := some_set_a
--				end
--				sp := sp & same_set_a (s1) / same_set_a (s2)
--				ss := (some_set_sa / same_set_a (s1)) & same_set_a (s2)
--			else
--				from
--					sp := partition_to_be_tested
--					s2 := some_set_a
--				until
--					not s2.is_empty and (sp / s2).set_is_disjoint (s2)
--				loop
--					sp := partition_to_be_tested
--					s2 := some_set_a
--				end
--				from
--					s1 := some_set_a
--				until
--					s1 ≭ s2
--				loop
--					s1 := some_set_a
--				end
--				sp := (sp / same_set_a (s1)) & same_set_a (s2)
--				ss := some_set_sa & same_set_a (s1) / same_set_a (s2)
--			end
--			assert ("sp.is_not_comparable (ss)", sp.is_not_comparable (ss))
--			assert ("sp.is_not_comparable (ss) ok", properties.is_not_comparable_ok (sp, ss))

--			if next_random_item \\ 2 = 0 then
--				ss := some_set_sa
--				sp := partition_to_be_tested ∩ same_set_sa (ss)
--			else
--				sp := partition_to_be_tested
--				ss := same_set_sa (some_set_sa ∩ sp)
--			end
--			assert ("not sp.is_not_comparable (ss)", not sp.is_not_comparable (ss))
--			assert ("not sp.is_not_comparable (ss) ok", properties.is_not_comparable_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := some_set_sa
--			assert ("is_not_comparable", sp.is_not_comparable (ss) ⇒ True)
--			assert ("is_not_comparable_ok", properties.is_not_comparable_ok (sp, ss))
--		end

--	test_is_strict_subset
--			-- <Precursor>
--			-- Test {STI_SET_OF_SETS}.is_strict_subset.
--			-- Test {STI_SET_FAMILY}.is_strict_subset.
--			-- Test {STI_SET_PARTITION}.is_strict_subset.
--		note
--			testing: "covers/{STI_SET}.is_strict_subset"
--			testing: "covers/{STI_SET_OF_SETS}.is_strict_subset"
--			testing: "covers/{STI_SET_FAMILY}.is_strict_subset"
--			testing: "covers/{STI_SET_PARTITION}.is_strict_subset"
--		local
--			sp1, sp2: like partition_to_be_tested
--			chk_ops: BOOLEAN
--		do
--			chk_ops := {ISE_RUNTIME}.check_assert (False)
--				from
--					sp2 := partition_to_be_tested
--				until
--					0 < # sp2
--				loop
--					sp2 := partition_to_be_tested
--				end
--			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

--				check
--					is_partition: -- sp2.as_tuple.suffix (some_index_up_to (# sp2) - 1) ⊂ `sp2'
--						sp2.as_tuple.suffix (some_index_up_to (# sp2) - 1).sets.is_partition
--				end
--			chk_ops := {ISE_RUNTIME}.check_assert (False)
--				sp1 := sp2.converted (sp2.as_tuple.suffix (some_index_up_to (# sp2) - 1).sets)
--			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

--			assert ("sp1_is_strict_subset_sp2", sp1 ⊂ sp2)
--			assert ("sp1_is_strict_subset_sp2_ok", properties.is_strict_subset_ok (sp1, sp2, some_set_sa))

--			sp1 := partition_to_be_tested
--			sp2 := partition_to_be_tested
--			assert ("is_strict_subset", sp1 ⊂ sp2 ⇒ True)
--			assert ("is_strict_subset_ok", properties.is_strict_subset_ok (sp1, sp2, some_set_sa))
--		end

--	test_is_not_strict_subset
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_not_strict_subset.
--			-- Test {STI_SET_OF_SETS}.is_not_strict_subset.
--			-- Test {STS_SET_FAMILY}.is_not_strict_subset.
--			-- Test {STI_SET_FAMILY}.is_not_strict_subset.
--			-- Test {STS_SET_PARTITION}.is_not_strict_subset.
--			-- Test {STI_SET_PARTITION}.is_not_strict_subset.
--		note
--			testing: "covers/{STS_SET}.is_not_strict_subset"
--			testing: "covers/{STI_SET}.is_not_strict_subset"
--			testing: "covers/{STS_SET_OF_SETS}.is_not_strict_subset"
--			testing: "covers/{STI_SET_OF_SETS}.is_not_strict_subset"
--			testing: "covers/{STS_SET_FAMILY}.is_not_strict_subset"
--			testing: "covers/{STI_SET_FAMILY}.is_not_strict_subset"
--			testing: "covers/{STS_SET_PARTITION}.is_not_strict_subset"
--			testing: "covers/{STI_SET_PARTITION}.is_not_strict_subset"
--		local
--			sp1, sp2: like partition_to_be_tested
--			s: like some_set_a
--			chk_ops: BOOLEAN
--		do
--			sp1 := partition_to_be_tested
--			sp2 := partition_to_be_tested

--			chk_ops := {ISE_RUNTIME}.check_assert (False)
--				if sp1 ≭ sp2 then
--					from
--						s := some_set_a
--					until
--						not s.is_empty and (sp1 / s).set_is_disjoint (s)
--					loop
--						s := some_set_a
--					end
--					sp1 := sp1 & s
--					sp2 := sp2 / s
--				end
--			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

--			assert ("sp1 ⊄ sp2", sp1 ⊄ sp2)
--			assert ("sp1 ⊄ sp2 ok", properties.is_not_strict_subset_ok (sp1, sp2))

--			sp1 := partition_to_be_tested
--			sp2 := partition_to_be_tested
--			assert ("is_not_strict_subset", sp1 ⊄ sp2 ⇒ True)
--			assert ("is_not_strict_subset_ok", properties.is_not_strict_subset_ok (sp1, sp2))
--		end

--	test_is_strict_superset
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_strict_superset.
--			-- Test {STI_SET_OF_SETS}.is_strict_superset.
--			-- Test {STS_SET_FAMILY}.is_strict_superset.
--			-- Test {STI_SET_FAMILY}.is_strict_superset.
--			-- Test {STS_SET_PARTITION}.is_strict_superset.
--			-- Test {STI_SET_PARTITION}.is_strict_superset.
--		note
--			testing: "covers/{STS_SET}.is_strict_superset"
--			testing: "covers/{STI_SET}.is_strict_superset"
--			testing: "covers/{STS_SET_OF_SETS}.is_strict_superset"
--			testing: "covers/{STI_SET_OF_SETS}.is_strict_superset"
--			testing: "covers/{STS_SET_FAMILY}.is_strict_superset"
--			testing: "covers/{STI_SET_FAMILY}.is_strict_superset"
--			testing: "covers/{STS_SET_PARTITION}.is_strict_superset"
--			testing: "covers/{STI_SET_PARTITION}.is_strict_superset"
--		local
--			s: like some_set_a
--			sp: like partition_to_be_tested
--			ss: like some_set_sa
--		do
--			from
--				s := some_set_a
--				sp := partition_to_be_tested
--			until
--				not s.is_empty and (sp / s).set_is_disjoint (s)
--			loop
--				s := some_set_a
--				sp := partition_to_be_tested
--			end
--			sp := sp & same_set_a (s)
--			ss := some_set_sa ∩ sp / same_set_a (s)
--			assert ("sp ⊃ ss", sp ⊃ ss)
--			assert ("sp ⊃ ss ok", properties.is_strict_superset_ok (sp, ss))

--			sp := partition_to_be_tested
--			if next_random_item \\ 2 = 0 then
--				ss := same_set_sa (sp)
--			else
--				s := some_set_a
--				sp := sp / same_set_a (s)
--				ss := some_set_sa & same_set_a (s)
--			end
--			assert ("not (sp ⊃ ss)", not (sp ⊃ ss))
--			assert ("not (sp ⊃ ss) ok", properties.is_strict_superset_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := some_set_sa
--			assert ("is_strict_superset", sp ⊃ ss ⇒ True)
--			assert ("is_strict_superset_ok", properties.is_strict_superset_ok (sp, ss))
--		end

--	test_is_not_strict_superset
--			-- <Precursor>
--			-- Test {STS_SET_OF_SETS}.is_not_strict_superset.
--			-- Test {STI_SET_OF_SETS}.is_not_strict_superset.
--			-- Test {STS_SET_FAMILY}.is_not_strict_superset.
--			-- Test {STI_SET_FAMILY}.is_not_strict_superset.
--			-- Test {STS_SET_PARTITION}.is_not_strict_superset.
--			-- Test {STI_SET_PARTITION}.is_not_strict_superset.
--		note
--			testing: "covers/{STS_SET}.is_not_strict_superset"
--			testing: "covers/{STI_SET}.is_not_strict_superset"
--			testing: "covers/{STS_SET_OF_SETS}.is_not_strict_superset"
--			testing: "covers/{STI_SET_OF_SETS}.is_not_strict_superset"
--			testing: "covers/{STS_SET_FAMILY}.is_not_strict_superset"
--			testing: "covers/{STI_SET_FAMILY}.is_not_strict_superset"
--			testing: "covers/{STS_SET_PARTITION}.is_not_strict_superset"
--			testing: "covers/{STI_SET_PARTITION}.is_not_strict_superset"
--		local
--			s: like some_set_a
--			sp: like partition_to_be_tested
--			ss: like some_set_sa
--		do
--			sp := partition_to_be_tested
--			if next_random_item \\ 2 = 0 then
--				ss := same_set_sa (sp)
--			else
--				s := some_set_a
--				sp := sp / same_set_a (s)
--				ss := some_set_sa & same_set_a (s)
--			end
--			assert ("sp ⊅ ss", sp ⊅ ss)
--			assert ("sp ⊅ ss ok", properties.is_not_strict_superset_ok (sp, ss))

--			from
--				s := some_set_a
--				sp := partition_to_be_tested
--			until
--				not s.is_empty and (sp / s).set_is_disjoint (s)
--			loop
--				s := some_set_a
--				sp := partition_to_be_tested
--			end
--			sp := sp & same_set_a (s)
--			ss := some_set_sa ∩ sp / same_set_a (s)
--			assert ("not (sp ⊅ ss)", not (sp ⊅ ss))
--			assert ("not (sp ⊅ ss) ok", properties.is_not_strict_superset_ok (sp, ss))

--			sp := partition_to_be_tested
--			ss := some_set_sa
--			assert ("is_not_strict_superset", sp ⊅ ss ⇒ True)
--			assert ("is_not_strict_superset_ok", properties.is_not_strict_superset_ok (sp, ss))
--		end

feature -- Test routines (Quantifier)

	test_exists
			-- <Precursor>
			-- Test {STI_SET_OF_SETS}.exists.
			-- Test {STI_SET_FAMILY}.exists.
			-- Test {STI_SET_PARTITION}.exists.
		note
			testing: "covers/{STI_SET}.exists"
			testing: "covers/{STI_SET_OF_SETS}.exists"
			testing: "covers/{STI_SET_FAMILY}.exists"
			testing: "covers/{STI_SET_PARTITION}.exists"
		local
			s: like some_set_a
			sp: like partition_to_be_tested
			p: PREDICATE [STS_SET [A, EQ]]
			chk_ops: BOOLEAN
		do
			p := agent p_x

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					sp := partition_to_be_tested
					s := some_set_a
				until
					not s.is_empty and (sp / s).set_is_disjoint (s) and p (s)
				loop
					sp := partition_to_be_tested
					s := some_set_a
				end
				sp := sp & s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("sp_exists_p", sp |∃ p)
			assert ("sp_exists_p_ok", properties.exists_ok (sp, p))

			sp := partition_to_be_tested
			assert ("exists", (sp |∃ p) ⇒ True)
			assert ("exists_ok", properties.exists_ok (sp, p))
		end

	test_exists_pair
			-- <Precursor>
			-- Test {STI_SET_OF_SETS}.exists_pair.
			-- Test {STI_SET_FAMILY}.exists_pair.
			-- Test {STI_SET_PARTITION}.exists_pair.
		note
			testing: "covers/{STI_SET}.exists_pair"
			testing: "covers/{STI_SET_OF_SETS}.exists_pair"
			testing: "covers/{STI_SET_FAMILY}.exists_pair"
			testing: "covers/{STI_SET_PARTITION}.exists_pair"
		local
			s1, s2: like some_set_a
			sp: like partition_to_be_tested
			p: PREDICATE [STS_SET [A, EQ], STS_SET [A, EQ]]
			chk_ops: BOOLEAN
		do
			p := agent p_x1_x2

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s1 := some_set_a
					s2 := some_set_a
				until
					not s1.is_empty and not s2.is_empty and (s1.is_disjoint (s2) or s1 ≍ s2) and p (s1, s2)
				loop
					s1 := some_set_a
					s2 := some_set_a
				end
				sp := partition_to_be_tested
				sp := (sp / s1 / s2) | agent anded (agent s1.is_disjoint, agent s2.is_disjoint, ?)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					is_not_empty: not s1.is_empty -- loop exit
					set_is_pair_wise_disjoint: (sp / s1).set_is_disjoint (s1) -- filter above
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp := sp & s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					is_not_empty: not s2.is_empty -- loop exit
					set_is_pair_wise_disjoint: (sp / s2).set_is_disjoint (s2) -- filter above
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp := sp & s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("sp_exists_pair_p", sp.exists_pair (p))
			assert ("sp_exists_pair_p_ok", properties.exists_pair_ok (sp, p))

			sp := partition_to_be_tested
			assert ("exists_pair", sp.exists_pair (p) ⇒ True)
			assert ("exists_pair_ok", properties.exists_pair_ok (sp, p))
		end

	test_exists_distinct_pair
			-- <Precursor>
			-- Test {STI_SET_OF_SETS}.exists_distinct_pair.
			-- Test {STI_SET_FAMILY}.exists_distinct_pair.
			-- Test {STI_SET_PARTITION}.exists_distinct_pair.
		note
			testing: "covers/{STI_SET}.exists_distinct_pair"
			testing: "covers/{STI_SET_OF_SETS}.exists_distinct_pair"
			testing: "covers/{STI_SET_FAMILY}.exists_distinct_pair"
			testing: "covers/{STI_SET_PARTITION}.exists_distinct_pair"
		local
			s1, s2: like some_set_a
			sp: like partition_to_be_tested
			p: PREDICATE [STS_SET [A, EQ], STS_SET [A, EQ]]
			chk_ops: BOOLEAN
		do
			p := agent p_x1_x2

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s1 := some_set_a
					s2 := some_set_a
				until
					not s1.is_empty and not s2.is_empty and s1.is_disjoint (s2) and p (s1, s2)
				loop
					s1 := some_set_a
					s2 := some_set_a
				end
				sp := partition_to_be_tested
				sp := (sp / s1 / s2) | agent anded(agent s1.is_disjoint, agent s2.is_disjoint, ?)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					is_not_empty: not s1.is_empty -- loop exit
					set_is_pair_wise_disjoint: (sp / s1).set_is_disjoint (s1) -- filter above
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp := sp & s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					is_not_empty: not s2.is_empty -- loop exit
					set_is_pair_wise_disjoint: (sp / s2).set_is_disjoint (s2) -- filter above
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp := sp & s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("sp_exists_distinct_pair_p", sp.exists_distinct_pair (p))
			assert ("sp_exists_distinct_pair_p_ok", properties.exists_distinct_pair_ok (sp, p))

			sp := partition_to_be_tested
			assert ("exists_distinct_pair", sp.exists_distinct_pair (p) ⇒ True)
			assert ("exists_distinct_pair_ok", properties.exists_distinct_pair_ok (sp, p))
		end

feature -- Test routines (Operation)

	test_partition_union
			-- Test {STI_SET_PARTITION}.partition_union.
		note
			testing: "covers/{STI_SET_PARTITION}.partition_union"
		local
			sp: like partition_to_be_tested
		do
			sp := partition_to_be_tested
			assert ("partition_union", attached (⋃ sp))
			assert ("partition_union_ok", properties.partition_union_ok (sp))
		end

	test_partition_intersection
			-- Test {STI_SET_PARTITION}.partition_intersection.
		note
			testing: "covers/{STI_SET_PARTITION}.partition_intersection"
		local
			sp: like partition_to_be_tested
		do
			from
				sp := partition_to_be_tested
			until
				not sp.is_empty
			loop
				sp := partition_to_be_tested
			end
			from
			until
				sp.is_singleton
			loop
				sp := sp.others
			end
			assert ("singleton_sp_intersection", attached (⋂ sp))
			assert ("singleton_sp_ok", properties.partition_intersection_ok (sp))

			from
				sp := partition_to_be_tested
			until
				not sp.is_empty
			loop
				sp := partition_to_be_tested
			end
			assert ("partition_intersection", attached (⋂ sp))
			assert ("partition_intersection_ok", properties.partition_intersection_ok (sp))
		end

feature -- Test routines (Factory)

	test_converted
			-- Test {STI_SET_PARTITION}.converted.
		note
			testing: "covers/{STI_SET_PARTITION}.converted"
		local
			sp: like partition_to_be_tested
			sp0: like resources_a.some_set_partition_a
			ss: like resources_a.some_set_of_sets_a
			chk_ext: BOOLEAN
		do
			sp := partition_to_be_tested
			sp0 := resources_a.some_set_partition_a
			ss := resources_a.some_set_of_sets_a

				check
					is_disjoint: ss.o_s.is_disjoint (sp0) -- ss.o.is_empty
				end
			chk_ext := {ISE_RUNTIME}.check_assert (False)
				ss := ss.o_s.batch_extended (sp0)
			chk_ext := {ISE_RUNTIME}.check_assert (chk_ext)

			assert ("ss_is_partition", ss.is_partition)
			assert ("converted", attached sp.converted (ss))
		end

feature -- Test routines (Predicate)

	test_some_set_has
			-- <Precursor>
			-- Test {STS_SET_FAMILY}.some_set_has.
			-- Test {STI_SET_FAMILY}.some_set_has.
			-- Test {STS_SET_PARTITION}.some_set_has.
			-- Test {STI_SET_PARTITION}.some_set_has.
		note
			testing: "covers/{STS_SET_OF_SETS}.some_set_has"
			testing: "covers/{STI_SET_OF_SETS}.some_set_has"
			testing: "covers/{STS_SET_FAMILY}.some_set_has"
			testing: "covers/{STI_SET_FAMILY}.some_set_has"
			testing: "covers/{STS_SET_PARTITION}.some_set_has"
			testing: "covers/{STI_SET_PARTITION}.some_set_has"
		local
			a: A
			ss: like resources_a.some_set_of_sets_a
			sp: like partition_to_be_tested
			chk_ops: BOOLEAN
		do
			a := some_object_a
			ss := partition_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				ss := ss.converted (ss ↦ agent {STS_SET [A, EQ]}.without(a)) / ss.o
				ss := ss & ((some_set_a ∖ (⋃ ss)) & a)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					is_partition: ss.is_partition
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sp := partition_to_be_tested.converted (ss)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("sp_some_set_has_a", sp.some_set_has (a))

			a := some_object_a
			sp := partition_to_be_tested
			assert ("some_set_has", sp.some_set_has (a) ⇒ True)
		end

	test_every_set_has
			-- <Precursor>
			-- Test {STI_SET_FAMILY}.every_set_has.
			-- Test {STI_SET_PARTITION}.every_set_has.
		note
			testing: "covers/{STS_SET_OF_SETS}.every_set_has"
			testing: "covers/{STI_SET_OF_SETS}.every_set_has"
			testing: "covers/{STI_SET_FAMILY}.every_set_has"
			testing: "covers/{STI_SET_PARTITION}.every_set_has"
		local
			a: A
			s: like some_set_a
			sp: like partition_to_be_tested
			chk_ops: BOOLEAN
		do
			a := some_object_a
			sp := partition_to_be_tested.o_p
			if next_random_item \\ 2 = 0 then
				chk_ops := {ISE_RUNTIME}.check_assert (False)
					s := some_set_a & a
				chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

					check
						is_not_empty: not s.is_empty
						set_is_pair_wise_disjoint: (sp / s).set_is_disjoint (s)
					end
				chk_ops := {ISE_RUNTIME}.check_assert (False)
					sp := sp & s
				chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			end
			assert ("sp_every_set_has_a", sp.every_set_has (a))

			a := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					sp := partition_to_be_tested
				until
					# sp > 1
				loop
					sp := partition_to_be_tested
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("sp_some_set_does_not_have_a", not sp.every_set_has (a))

			a := some_object_a
			sp := partition_to_be_tested
--			assert ("every_set_has", sp.every_set_has (a) ⇒ True)
			assert ("every_set_has", sp.every_set_has (a) ⇒ True)
		end

feature {NONE} -- Factory (element to be tested)

	partition_to_be_tested: like resources_a.some_direct_set_partition_a
			-- Set partition meant to be under tests
		do
			Result := resources_a.some_direct_set_partition_a
		end

feature {NONE} -- Conversion

	family_from_sa (sa: STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]): like resources_a.new_set_family_a
			-- `sa` converted to a set like `resources_a'.`new_set_family_a`
		do
			Result := sa
		end

	family_from_sti_sa (sa: STI_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]): like resources_a.new_set_family_a
			-- `sa' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa
		end

	family_from_sa_a (sa_a: STS_TRANSFORMABLE_SET [STS_SET [A, EQ], A, STS_SET_EQUALITY [A, EQ], EQ]): like resources_a.new_set_family_a
			-- `sa_a' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_a
		end

	family_from_sti_sa_a (sa_a: STI_TRANSFORMABLE_SET [STS_SET [A, EQ], A, STS_SET_EQUALITY [A, EQ], EQ]): like resources_a.new_set_family_a
			-- `sa_a' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_a
		end

	family_from_saa (saa: STS_TRANSFORMABLE_SET [STS_SET [A, EQ], STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ], STS_SET_EQUALITY [A, EQ]]): like resources_a.new_set_family_a
			-- `saa` converted to a set like `resources_a'.`new_set_family_a`
		do
			Result := saa
		end

	family_from_sti_saa (saa: STI_TRANSFORMABLE_SET [STS_SET [A, EQ], STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ], STS_SET_EQUALITY [A, EQ]]): like resources_a.new_set_family_a
			-- `saa' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := saa
		end

	family_from_sa_ta (sa_ta: STS_TRANSFORMABLE_SET [STS_SET [A, EQ], STS_N_TUPLE [A, EQ], STS_SET_EQUALITY [A, EQ], STS_N_TUPLE_EQUALITY [A, EQ]]): like resources_a.new_set_family_a
			-- `sa_ta' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_ta
		end

	family_from_sti_sa_ta (sa_ta: STI_TRANSFORMABLE_SET [STS_SET [A, EQ], STS_N_TUPLE [A, EQ], STS_SET_EQUALITY [A, EQ], STS_N_TUPLE_EQUALITY [A, EQ]]): like resources_a.new_set_family_a
			-- `sa_ta' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_ta
		end

	family_from_sa_ssa (
		sa_ssa: STS_TRANSFORMABLE_SET [
			STS_SET [A, EQ], STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]],
			STS_SET_EQUALITY [A, EQ], STS_SET_EQUALITY [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]
			]
		): like resources_a.new_set_family_a
			-- `sa_ssa' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_ssa
		end

	family_from_sti_sa_ssa (
		sa_ssa: STI_TRANSFORMABLE_SET [
			STS_SET [A, EQ], STS_SET [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]],
			STS_SET_EQUALITY [A, EQ], STS_SET_EQUALITY [STS_SET [A, EQ], STS_SET_EQUALITY [A, EQ]]
			]
		): like resources_a.new_set_family_a
			-- `sa_ssa' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_ssa
		end

	family_from_sa_sta (
		sa_sta: STS_TRANSFORMABLE_SET [
			STS_SET [A, EQ],
			STS_SET [STS_N_TUPLE [A, EQ], STS_N_TUPLE_EQUALITY [A, EQ]],
			STS_SET_EQUALITY [A, EQ],
			STS_SET_EQUALITY [STS_N_TUPLE [A, EQ], STS_N_TUPLE_EQUALITY [A, EQ]]
			]
		): like resources_a.new_set_family_a
			-- `sa_sta' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_sta
		end

	family_from_sti_sa_sta (
		sa_sta: STI_TRANSFORMABLE_SET [
			STS_SET [A, EQ],
			STS_SET [STS_N_TUPLE [A, EQ], STS_N_TUPLE_EQUALITY [A, EQ]],
			STS_SET_EQUALITY [A, EQ],
			STS_SET_EQUALITY [STS_N_TUPLE [A, EQ], STS_N_TUPLE_EQUALITY [A, EQ]]]
		): like resources_a.new_set_family_a
			-- `sa_sta' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_sta
		end

	family_from_sa_aa (
		sa_aa: STS_TRANSFORMABLE_SET [
			STS_SET [A, EQ],
			STS_SET [STS_ORDERED_PAIR [A, A, EQ, EQ], STS_ORDERED_PAIR_EQUALITY [A, A, EQ, EQ]],
			STS_SET_EQUALITY [A, EQ],
			STS_SET_EQUALITY [STS_ORDERED_PAIR [A, A, EQ, EQ], STS_ORDERED_PAIR_EQUALITY [A, A, EQ, EQ]]
			]
		): like resources_a.new_set_family_a
			-- `sa_aa' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_aa
		end

	family_from_sti_sa_aa (
		sa_aa: STI_TRANSFORMABLE_SET [
			STS_SET [A, EQ],
			STS_SET [STS_ORDERED_PAIR [A, A, EQ, EQ], STS_ORDERED_PAIR_EQUALITY [A, A, EQ, EQ]],
			STS_SET_EQUALITY [A, EQ],
			STS_SET_EQUALITY [STS_ORDERED_PAIR [A, A, EQ, EQ], STS_ORDERED_PAIR_EQUALITY [A, A, EQ, EQ]]
			]
		): like resources_a.new_set_family_a
			-- `sa_aa' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sa_aa
		end

	family_from_set_of_sets (ss: STS_SET_OF_SETS [A, EQ]): like resources_a.new_set_family_a
			-- `ss' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := ss
		end

	family_from_sti_set_of_sets (ss: STI_SET_OF_SETS [A, EQ]): like resources_a.new_set_family_a
			-- `ss' converted to a family like `resources_a'.`new_set_family_a'
		do
			Result := ss
		end

	family_from_sf (sf: STS_SET_FAMILY [A, EQ]): like resources_a.new_set_family_a
			-- `sf' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sf
		end

	partition_from_sp (sp: STS_SET_PARTITION [A, EQ]): like partition_to_be_tested
			-- `sp' converted to a partition like `partition_to_be_tested'
		do
			Result := sp
		end

	family_from_subset_family (sf: STS_SUBSET_FAMILY [A, EQ]): like resources_a.new_set_family_a
			-- `sf' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sf
		end

	family_from_sti_subset_family (sf: STI_SUBSET_FAMILY [A, EQ]): like resources_a.new_set_family_a
			-- `sf' converted to a family like `resources_a'.`new_set_family_a'
		do
			Result := sf
		end

	family_from_subset_family_structure (sf: STS_SUBSET_FAMILY_STRUCTURE [A, EQ]): like resources_a.new_set_family_a
			-- `sf' converted to a set like `resources_a'.`new_set_family_a'
		do
			Result := sf
		end

	family_from_sti_subset_family_structure (sf: STS_SUBSET_FAMILY_STRUCTURE [A, EQ]): like resources_a.new_set_family_a
			-- `sf' converted to a family like `resources_a'.`new_set_family_a'
		do
			Result := sf
		end

feature -- Predicate

	anded (p1, p2: PREDICATE [STS_SET [A, EQ]]; x: STS_SET [A, EQ]): BOOLEAN
			-- Do `p1` (`x`) and `p2` (`x`) hold?
		do
			Result := p1 (x) and p2 (x)
		ensure
			class
			definition: Result = (p1 (x) and p2 (x))
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
