﻿note
	description: "Test suite for {STI_SET}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-12-01 19:25:58 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	SET_TESTS [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	ELEMENT_TESTS
		rename
			test_is_in as test_element_is_in,
			test_is_not_in as test_element_is_not_in,
			element_to_be_tested as set_to_be_tested
		redefine
			set_to_be_tested
		end

	SETS_TESTS_RESOURCES_UNARY [A, EQ]
		rename
			eq_a as eq
		end

feature -- Access

	properties: SET_PROPERTIES [A, EQ]
			-- Object that checks the set-theory properties of {STI_SET}
		deferred
		end

feature -- Test routines (Initialization)

	test_make_empty
			-- Test {STI_SET}.make_empty.
		note
			testing: "covers/{STI_SET}.make_empty"
		do
			assert ("make_empty", attached (create {like set_to_be_tested}.make_empty))
			assert ("make_empty_ok", properties.make_empty_ok)
		end

	test_make_singleton
			-- Test {STI_SET}.make_singleton.
		note
			testing: "covers/{STI_SET}.make_singleton"
		local
			a: A
		do
			a := some_object_a
			assert ("make_singleton", attached (create {like set_to_be_tested}.make_singleton (a)))
			assert ("make_singleton_ok", properties.make_singleton_ok (a))
		end

	test_make_extended
			-- Test {STI_SET}.make_extended.
		note
			testing: "covers/{STI_SET}.make_extended"
		local
			a: A
			s: like some_set_a
			chk_ops: BOOLEAN
		do
			a := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_set_a / a
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_does_not_have_a", s ∌ a)
			assert ("make_extended_a_s", attached (create {like set_to_be_tested}.make_extended (a, s)))
			assert ("make_extended_a_s_ok", properties.make_extended_ok (a, s))

			a := some_object_a
			s := some_set_a
			assert ("make_extended", s ∌ a ⇒ attached (create {like set_to_be_tested}.make_extended (a, s)))
			assert ("make_extended_ok", properties.make_extended_ok (a, s))
		end

	test_make_from_set
			-- Test {STI_SET}.make_from_set.
		note
			testing: "covers/{STI_SET}.make_from_set"
		local
			sa: like some_set_a
			saa: like new_transformable_set_aa
			n: NATURAL
			chk_ops, chk_mk: BOOLEAN
		do
			sa := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # sa
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_mk := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("make_from_set", attached (create {like set_to_be_tested}.make_from_set (sa)))
				assert ("set_from_sa", attached set_from_sa (sa))
			if chk_mk then
				chk_mk := {ISE_RUNTIME}.check_assert (True)
			end

			saa := new_transformable_set_aa
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # saa
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_mk := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("set_from_saa", attached set_from_saa (saa)) -- TODO: make some_transformable_set_aa?
			if chk_mk then
				chk_mk := {ISE_RUNTIME}.check_assert (True)
			end
		end

feature -- Test routines (Primitive)

	test_is_empty
			-- Test {STI_SET}.is_empty.
		note
			testing: "covers/{STI_SET}.is_empty"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			inspect
				next_random_item \\ 2
			when 0 then
				s := s.o
			when 1 then
				s := s ∖ s
			end
			assert ("s.is_empty", s.is_empty)
			assert ("s.is_empty ok", properties.is_empty_ok (s))

			s := set_to_be_tested & some_object_a
			assert ("not s.is_empty", not s.is_empty)
			assert ("not s.is_empty ok", properties.is_empty_ok (s))

			s := set_to_be_tested
			assert ("is_empty", s.is_empty ⇒ True)
			assert ("is_empty_ok", properties.is_empty_ok (s))
		end

	test_any
			-- Test {STI_SET}.any.
		note
			testing: "covers/{STI_SET}.any"
		local
			s: like set_to_be_tested
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			assert ("s.any", attached s.any ⇒ True)
			assert ("s.any ok", properties.any_ok (s))

			s := set_to_be_tested
			assert ("any", not s.is_empty ⇒ attached s.any ⇒ True)
			assert ("any_ok", properties.any_ok (s))
		end

	test_others
			-- Test {STI_SET}.others.
		note
			testing: "covers/{STI_SET}.others"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			inspect
				next_random_item \\ 2
			when 0 then
				s := s.o
			when 1 then
				s := s ∖ s
			end
			assert ("∅.others", attached s.others)
			assert ("∅.others ok", properties.others_ok (s))

			s := set_to_be_tested & some_object_a
			assert ("s.others", attached s.others)
			assert ("s.others ok", properties.others_ok (s))

			s := set_to_be_tested
			assert ("others", attached s.others)
			assert ("others_ok", properties.others_ok (s))
		end

	test_eq
			-- Test {STI_SET}.eq.
		note
			testing: "covers/{STI_SET}.eq"
		do
			assert ("eq", attached set_to_be_tested.eq)
		end

feature -- Test routines (Membership)

	test_is_in
			-- Test {STS_SET}.is_in.
			-- Test {STI_SET}.is_in.
		note
			testing: "covers/{STS_SET}.is_in"
			testing: "covers/{STI_SET}.is_in"
		local
			s, s_twin: like set_to_be_tested
			same_s: like some_set_a
			ss: like some_set_sa
		do
			s := set_to_be_tested
			ss := some_references_sa & s
			assert ("s ∈ ss", s ∈ ss)
			assert ("s ∈ ss ok", properties.is_in_ok (s, ss))

			s := set_to_be_tested
			ss := some_references_sa / s
			assert ("not (s ∈ ss)", not (s ∈ ss))
			assert ("not (s ∈ ss) ok", properties.is_in_ok (s, ss))

			s := set_to_be_tested
			s_twin := s.twin
			ss := some_objects_sa & s
			assert ("s_twin ∈ ss", s_twin ∈ ss)
			assert ("s_twin ∈ ss ok", properties.is_in_ok (s_twin, ss))

			s := set_to_be_tested
			s_twin := s.twin
			ss := some_objects_sa / s
			assert ("not (s_twin ∈ ss) ∈ ss", not (s_twin ∈ ss))
			assert ("not (s_twin ∈ ss) ok", properties.is_in_ok (s_twin, ss))

			s := set_to_be_tested
			same_s := same_set_a (s)
			ss := some_math_set_sa & s
			assert ("same_s ∈ ss", same_s ∈ ss)
			assert ("same_s ∈ ss ok", properties.is_in_ok (same_s, ss))

			s := set_to_be_tested
			same_s := same_set_a (s)
			ss := some_math_set_sa / s
			assert ("not (same_s ∈ ss)", not (same_s ∈ ss))
			assert ("not (same_s ∈ ss) ok", properties.is_in_ok (same_s, ss))

			s := set_to_be_tested
			ss := some_set_sa
			assert ("is_in", s ∈ ss ⇒ True)
			assert ("is_in_ok", properties.is_in_ok (s, ss))
		end

	test_is_not_in
			-- Test {STS_SET}.is_not_in.
			-- Test {STI_SET}.is_not_in.
		note
			testing: "covers/{STS_SET}.is_not_in"
			testing: "covers/{STI_SET}.is_not_in"
		local
			s, s_twin: like set_to_be_tested
			same_s: like some_set_a
			ss: like some_set_sa
		do
			s := set_to_be_tested
			ss := some_references_sa / s
			assert ("s ∉ ss", s ∉ ss)
			assert ("s ∉ ss ok", properties.is_not_in_ok (s, ss))

			s := set_to_be_tested
			ss := some_references_sa & s
			assert ("not (s ∉ ss)", not (s ∉ ss))
			assert ("not (s ∉ ss) ok", properties.is_not_in_ok (s, ss))

			s := set_to_be_tested
			s_twin := s.twin
			ss := some_objects_sa / s
			assert ("s_twin ∉ ss", s_twin ∉ ss)
			assert ("s_twin ∉ ss ok", properties.is_not_in_ok (s_twin, ss))

			s := set_to_be_tested
			s_twin := s.twin
			ss := some_objects_sa & s
			assert ("not (s_twin ∉ ss) ∉ ss", not (s_twin ∉ ss))
			assert ("not (s_twin ∉ ss) ok", properties.is_not_in_ok (s_twin, ss))

			s := set_to_be_tested
			same_s := same_set_a (s)
			ss := some_math_set_sa / s
			assert ("same_s ∉ ss", same_s ∉ ss)
			assert ("same_s ∉ ss ok", properties.is_not_in_ok (same_s, ss))

			s := set_to_be_tested
			same_s := same_set_a (s)
			ss := some_math_set_sa & s
			assert ("not (same_s ∉ ss)", not (same_s ∉ ss))
			assert ("not (same_s ∉ ss) ok", properties.is_not_in_ok (same_s, ss))

			s := set_to_be_tested
			ss := some_set_sa
			assert ("is_not_in", s ∉ ss ⇒ True)
			assert ("is_not_in_ok", properties.is_not_in_ok (s, ss))
		end

	test_has
			-- Test {STI_SET}.has.
		note
			testing: "covers/{STI_SET}.has"
		local
			a: A
			s: like set_to_be_tested
			s2: like some_set_a
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s2 := some_set_a ∩ same_set_a (s.others) & same_object_a (s.any)
				check
					is_not_empty: not s2.is_empty -- Above: s2 := some_set_a ... & ...
				end
			a := same_object_a (s2.any)
			assert ("s ∋ a", s ∋ a)
			assert ("s ∋ a ok", properties.has_ok (s, a))

			a := some_object_a
			s := set_to_be_tested / same_object_a (a)
			assert ("not (s ∋ a)", not (s ∋ a))
			assert ("s ∋ a ok", properties.has_ok (s, a))

			a := some_object_a
			s := set_to_be_tested
			assert ("has", s ∋ a ⇒ True)
			assert ("has_ok", properties.has_ok (s, a))
		end

	test_does_not_have
			-- Test {STS_SET}.does_not_have.
			-- Test {STI_SET}.does_not_have.
		note
			testing: "covers/{STS_SET}.does_not_have"
			testing: "covers/{STI_SET}.does_not_have"
		local
			a: A
			s: like set_to_be_tested
			s2: like some_set_a
		do
			a := some_object_a
			s := set_to_be_tested / same_object_a (a)
			assert ("s ∌ a", s ∌ a)
			assert ("s ∌ a ok", properties.does_not_have_ok (s, a))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s2 := some_set_a ∩ same_set_a (s.others) & same_object_a (s.any)
				check
					is_not_empty: not s2.is_empty -- Above: s2 := some_set_a ... & ...
				end
			a := same_object_a (s2.any)
			assert ("not (s ∌ a)", not (s ∌ a))
			assert ("s ∌ same_a ok", properties.does_not_have_ok (s, a))

			a := some_object_a
			s := set_to_be_tested
			assert ("does_not_have", s ∌ a ⇒ True)
			assert ("does_not_have_ok", properties.does_not_have_ok (s, a))
		end

feature -- Test routines (Construction)

	test_with
			-- Test {STI_SET}.with.
		note
			testing: "covers/{STI_SET}.with"
		local
			a: A
			s: like set_to_be_tested
			s2: like some_set_a
		do
			a := some_object_a
			s := set_to_be_tested / same_object_a (a)
			assert ("s ∌ a", attached (s & a))
			assert ("s ∌ a ok", properties.with_ok (s, a))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s2 := some_set_a ∩ same_set_a (s.others) & same_object_a (s.any)
				check
					is_not_empty: not s2.is_empty -- Above: s2 := some_set_a ... & ...
				end
			a := same_object_a (s2.any)
			assert ("s ∋ a", attached (s & a))
			assert ("s ∋ a ok", properties.with_ok (s, a))

			a := some_object_a
			s := set_to_be_tested
			assert ("with", attached (s & a))
			assert ("with_ok", properties.with_ok (s, a))
		end

	test_without
			-- Test {STI_SET}.without.
		note
			testing: "covers/{STI_SET}.without"
		local
			a: A
			s: like set_to_be_tested
			s2: like some_set_a
		do
			a := some_object_a
			s := set_to_be_tested / same_object_a (a)
			assert ("s ∌ a", attached (s / a))
			assert ("s ∌ a ok", properties.with_ok (s, a))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s2 := some_set_a ∩ same_set_a (s.others) & same_object_a (s.any)
				check
					is_not_empty: not s2.is_empty -- Above: s2 := some_set_a ... & ...
				end
			a := same_object_a (s2.any)
			assert ("s ∋ a", attached (s / a))
			assert ("s ∋ a ok", properties.with_ok (s, a))

			a := some_object_a
			s := set_to_be_tested
			assert ("without", attached (s / a))
			assert ("without_ok", properties.without_ok (s, a))
		end

	test_extended
			-- Test {STI_SET}.extended.
		note
			testing: "covers/{STI_SET}.extended"
		local
			a: A
			s: like set_to_be_tested
			chk_ops: BOOLEAN
		do
			a := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested / a
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_does_not_have_a", s ∌ a)
			assert ("s_extended_a", attached s.extended (a))
			assert ("s_extended_a_ok", properties.extended_ok (s, a))

			a := some_object_a
			s := set_to_be_tested
			assert ("extended", s ∌ a ⇒ attached s.extended (a))
			assert ("extended_ok", properties.extended_ok (s, a))
		end

	test_batch_extended
			-- Test {STI_SET}.batch_extended.
		note
			testing: "covers/{STI_SET}.batch_extended"
		local
			s0: like some_set_a
			s: like set_to_be_tested
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			s0 := some_set_a
			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := s ∖ s0
				n := # s0 ∨ # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s_is_disjoint_s0", s.is_disjoint (s0))
				assert ("s_batch_extended_s0", attached s.batch_extended (s0))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s_batch_extended_s0_ok", properties.batch_extended_ok (s0, s))

			s0 := some_set_a
			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s0 ∨ # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("batch_extended", s.is_disjoint (s0) ⇒ attached s.batch_extended (s0))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("batch_extended_ok", properties.batch_extended_ok (s0, s))
		end

feature -- Test routines (Output)

	test_out
			-- Test {STI_SET}.out.
		note
			testing: "covers/{STI_SET}.out"
		local
			chk_tests: BOOLEAN
			n: like some_set.cardinality
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			n := # s
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("out", attached s.out)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
		end

feature -- Test routines (Quality)

	test_is_singleton
			-- Test {STS_SET}.is_singleton.
			-- Test {STI_SET}.is_singleton.
		note
			testing: "covers/{STS_SET}.is_singleton"
			testing: "covers/{STI_SET}.is_singleton"
		local
			s: like set_to_be_tested
		do
			from
				from
					s := set_to_be_tested
				until
					not s.is_empty
				loop
					s := set_to_be_tested
				end
			until
				s.others.is_empty
			loop
				s := s.others
			end
			assert ("s.is_singleton", s.is_singleton)
			assert ("s.is_singleton ok", properties.is_singleton_ok (s))

			from
				s := set_to_be_tested
			until
				# s /= 1
			loop
				s := set_to_be_tested
			end
			assert ("not s.is_singleton", not s.is_singleton)
			assert ("not s.is_singleton ok", properties.is_singleton_ok (s))

			s := set_to_be_tested
			assert ("is_singleton", s.is_singleton ⇒ True)
			assert ("is_singleton_ok", properties.is_singleton_ok (s))
		end

feature -- Test routines (Measurement)

	test_cardinality
			-- Test {STI_SET}.cardinality.
		note
			testing: "covers/{STI_SET}.cardinality"
		local
			s: like set_to_be_tested
			a: A
		do
			s := set_to_be_tested.o
			assert ("# s = 0", # s = 0)
			assert ("# s = 0 ok", properties.cardinality_ok (s))

			s := s & some_object_a
			assert ("# s = 1", # s = 1)
			assert ("# s = 1 ok", properties.cardinality_ok (s))

			from
				a := some_object_a
			until
				s ∌ a
			loop
				a := some_object_a
			end
			s := s & same_object_a (a)
			assert ("# s = 2", # s = 2)
			assert ("# s = 2 ok", properties.cardinality_ok (s))

			from
				a := some_object_a
			until
				s ∌ a
			loop
				a := some_object_a
			end
			s := s & same_object_a (a)
			assert ("# s = 3", # s = 3)
			assert ("# s = 3 ok", properties.cardinality_ok (s))

			s := set_to_be_tested
			assert ("cardinality", attached # s)
			assert ("cardinality_ok", properties.cardinality_ok (s))
		end

feature -- Test routines (Comparison)

	test_equals
			-- Test {STI_SET}.equals.
		note
			testing: "covers/{STI_SET}.equals"
		local
			a: A
			s: like set_to_be_tested
			s1, s2, s3: like some_set_a
		do
			s := set_to_be_tested
			s2 := same_set_a (s)
			s3 := same_set_a (s2)
			assert ("s ≍ s2", s ≍ s2)
			assert ("s ≍ s2 ok", properties.equals_ok (s, s2, s3))

			if next_random_item \\ 2 = 0 then
				from
					s := set_to_be_tested
				until
					not s.is_empty
				loop
					s := set_to_be_tested
				end
				s1 := same_set_a (s)
					check
						is_not_empty: not s1.is_empty -- s1 ≍ s
					end
				s2 := some_set_a / same_object_a (s1.any)
			else
				a := some_object_a
				s := set_to_be_tested / same_object_a (a)
				s2 := some_set_a & same_object_a (a)
			end
			assert ("not (s ≍ s2)", not (s ≍ s2))
			assert ("(s ≍ s2) ok", properties.equals_ok (s, s2, some_set_a))

			s := set_to_be_tested
			s2 := some_set_a
			s3 := some_set_a
			assert ("equals", s ≍ s2 ⇒ True)
			assert ("equals ok", properties.equals_ok (s, s2, s3))
		end

	test_unequals
			-- Test {STS_SET}.unequals.
			-- Test {STI_SET}.unequals.
		note
			testing: "covers/{STS_SET}.unequals"
			testing: "covers/{STI_SET}.unequals"
		local
			a: A
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			if next_random_item \\ 2 = 0 then
				from
					s := set_to_be_tested
				until
					not s.is_empty
				loop
					s := set_to_be_tested
				end
				s1 := same_set_a (s)
					check
						is_not_empty: not s1.is_empty -- s1 ≍ s
					end
				s2 := some_set_a / same_object_a (s1.any)
			else
				a := some_object_a
				s := set_to_be_tested / same_object_a (a)
				s2 := some_set_a & same_object_a (a)
			end
			assert ("s ≭ s2", s ≭ s2)
			assert ("s ≭ s2 ok", properties.unequals_ok (s, s2))

			s := set_to_be_tested
			s2 := same_set_a (s)
			assert ("not (s ≭ s2)", not (s ≭ s2))
			assert ("(s ≭ s2) ok", properties.unequals_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("unequals", s ≭ s2 ⇒ True)
			assert ("unequals ok", properties.unequals_ok (s, s2))
		end

	test_is_subset
			-- Test {STI_SET}.is_subset.
		note
			testing: "covers/{STI_SET}.is_subset"
		local
			s: like set_to_be_tested
			s1, s2, s3: like some_set_a
		do
			s3 := some_set_a
			s2 := some_set_a ∩ same_set_a (s3)
			s := set_to_be_tested ∩ same_set_a (s2)
			assert ("s ⊆ s2", s ⊆ s2)
			assert ("s ⊆ s2 ok", properties.is_subset_ok (s, s2, s3))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s1 := same_set_a (s)
				check
					is_not_empty: not s1.is_empty -- s1 ≍ s
				end
			s2 := some_set_a / same_object_a (s1.any)
			assert ("not (s ⊆ s2)", not (s ⊆ s2))
			assert ("not (s ⊆ s2) ok", properties.is_subset_ok (s, s2, some_set_a))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_subset", s ⊆ s2 ⇒ True)
			assert ("is_subset ok", properties.is_subset_ok (s, s2, some_set_a))
		end

	test_is_not_subset
			-- Test {STS_SET}.is_not_subset.
			-- Test {STI_SET}.is_not_subset.
		note
			testing: "covers/{STS_SET}.is_not_subset"
			testing: "covers/{STI_SET}.is_not_subset"
		local
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s1 := same_set_a (s)
				check
					is_not_empty: not s1.is_empty -- s1 ≍ s
				end
			s2 := some_set_a / same_object_a (s1.any)
			assert ("s ⊈ s2", s ⊈ s2)
			assert ("s ⊈ s2 ok", properties.is_not_subset_ok (s, s2))

			s2 := some_set_a
			s := set_to_be_tested ∩ same_set_a (s2)
			assert ("not (s ⊈ s2)", not (s ⊈ s2))
			assert ("not (s ⊈ s2) ok", properties.is_not_subset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_not_subset", s ⊈ s2 ⇒ True)
			assert ("is_not_subset ok", properties.is_not_subset_ok (s, s2))
		end

	test_is_superset
			-- Test {STS_SET}.is_superset.
			-- Test {STI_SET}.is_superset.
		note
			testing: "covers/{STS_SET}.is_superset"
			testing: "covers/{STI_SET}.is_superset"
		local
			s: like set_to_be_tested
			s2, s3: like some_set_a
		do
			s := set_to_be_tested
			s2 := some_set_a ∩ same_set_a (s)
			assert ("s ⊇ s2", s ⊇ s2)
			assert ("s ⊇ s2 ok", properties.is_superset_ok (s, s2))

			from
				s2 := some_set_a
			until
				not s2.is_empty
			loop
				s2 := some_set_a
			end
			s3 := same_set_a (s2)
				check
					is_not_empty: not s3.is_empty -- s3 ≍ s2
				end
			s := set_to_be_tested / same_object_a (s3.any)
			assert ("not (s ⊇ s2)", not (s ⊇ s2))
			assert ("not (s ⊇ s2) ok", properties.is_superset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_superset", s ⊇ s2 ⇒ True)
			assert ("is_superset ok", properties.is_superset_ok (s, s2))
		end

	test_is_not_superset
			-- Test {STS_SET}.is_not_superset.
			-- Test {STI_SET}.is_not_superset.
		note
			testing: "covers/{STS_SET}.is_not_superset"
			testing: "covers/{STI_SET}.is_not_superset"
		local
			s: like set_to_be_tested
			s2, s3: like some_set_a
		do
			from
				s2 := some_set_a
			until
				not s2.is_empty
			loop
				s2 := some_set_a
			end
			s3 := same_set_a (s2)
				check
					is_not_empty: not s3.is_empty -- s3 ≍ s2
				end
			s := set_to_be_tested / same_object_a (s3.any)
			assert ("s ⊉ s2", s ⊉ s2)
			assert ("s ⊉ s2 ok", properties.is_not_superset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a ∩ same_set_a (s)
			assert ("not (s ⊉ s2)", not (s ⊉ s2))
			assert ("not (s ⊉ s2) ok", properties.is_not_superset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_not_superset", s ⊉ s2 ⇒ True)
			assert ("is_not_superset ok", properties.is_not_superset_ok (s, s2))
		end

	test_is_comparable
			-- Test {STS_SET}.is_comparable.
			-- Test {STI_SET}.is_comparable.
		note
			testing: "covers/{STS_SET}.is_comparable"
			testing: "covers/{STI_SET}.is_comparable"
		local
			a, b: A
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			if next_random_item \\ 2 = 0 then
				s2 := some_set_a
				s := set_to_be_tested ∩ same_set_a (s2)
			else
				s := set_to_be_tested
				s2 := some_set_a ∩ same_set_a (s)
			end
			assert ("s.is_comparable (s2)", s.is_comparable (s2))
			assert ("s.is_comparable (s2) ok", properties.is_comparable_ok (s, s2))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s1 := same_set_a (s)
				check
					is_not_empty: not s1.is_empty -- s1 ≍ s
				end
			a := same_object_a (s1.any)
			from
				b := some_object_a
			until
				not eq (a, b)
			loop
				b := some_object_a
			end
			s := s / same_object_a (b)
			s2 := (some_set_a / same_object_a (a)) & same_object_a (b)
			assert ("not s.is_comparable (s2)", not s.is_comparable (s2))
			assert ("not s.is_comparable (s2) ok", properties.is_comparable_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_comparable", s.is_comparable (s2) ⇒ True)
			assert ("is_comparable_ok", properties.is_comparable_ok (s, s2))
		end

	test_is_not_comparable
			-- Test {STS_SET}.is_not_comparable.
			-- Test {STI_SET}.is_not_comparable.
		note
			testing: "covers/{STS_SET}.is_not_comparable"
			testing: "covers/{STI_SET}.is_not_comparable"
		local
			a, b: A
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s1 := same_set_a (s)
				check
					is_not_empty: not s1.is_empty -- s1 ≍ s
				end
			a := same_object_a (s1.any)
			from
				b := some_object_a
			until
				not eq (a, b)
			loop
				b := some_object_a
			end
			s := s / same_object_a (b)
			s2 := (some_set_a / same_object_a (a)) & same_object_a (b)
			assert ("s.is_not_comparable (s2)", s.is_not_comparable (s2))
			assert ("s.is_not_comparable (s2) ok", properties.is_not_comparable_ok (s, s2))

			if next_random_item \\ 2 = 0 then
				s2 := some_set_a
				s := set_to_be_tested ∩ same_set_a (s2)
			else
				s := set_to_be_tested
				s2 := some_set_a ∩ same_set_a (s)
			end
			assert ("not s.is_not_comparable (s2)", not s.is_not_comparable (s2))
			assert ("not s.is_not_comparable (s2) ok", properties.is_not_comparable_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_not_comparable", s.is_not_comparable (s2) ⇒ True)
			assert ("is_not_comparable_ok", properties.is_not_comparable_ok (s, s2))
		end

	test_is_strict_subset
			-- Test {STI_SET}.is_strict_subset.
		note
			testing: "covers/{STI_SET}.is_strict_subset"
		local
			s: like set_to_be_tested
			s2, s3: like some_set_a
		do
			from
				s3 := some_set_a
			until
				not s3.others.is_empty
			loop
				s3 := some_set_a
			end
				check
					is_not_empty: not s3.is_empty -- not s3.others.is_empty
				end
			s2 := some_set_a ∩ same_set_a (s3.others).others & same_object_a (s3.any)
			s := set_to_be_tested ∩ same_set_a (s2).others
			assert ("s ⊂ s2", s ⊂ s2)
			assert ("s ⊂ s2 ok", properties.is_strict_subset_ok (s, s2, s3))

			s := set_to_be_tested
			if next_random_item \\ 2 = 0 then
				s2 := same_set_a (s)
			else
				from
					s := set_to_be_tested
				until
					not s.is_empty
				loop
					s := set_to_be_tested
				end
				s2 := some_set_a ∩ same_set_a (s).others
			end
			assert ("not (s ⊂ s2)", not (s ⊂ s2))
			assert ("not (s ⊂ s2) ok", properties.is_strict_subset_ok (s, s2, some_set_a))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_strict_subset", s ⊂ s2 ⇒ True)
			assert ("is_strict_subset_ok", properties.is_strict_subset_ok (s, s2, some_set_a))
		end

	test_is_not_strict_subset
			-- Test {STS_SET}.is_not_strict_subset.
			-- Test {STI_SET}.is_not_strict_subset.
		note
			testing: "covers/{STS_SET}.is_not_strict_subset"
			testing: "covers/{STI_SET}.is_not_strict_subset"
		local
			s: like set_to_be_tested
			s2: like some_set_a
		do
			s := set_to_be_tested
			if next_random_item \\ 2 = 0 then
				s2 := same_set_a (s)
			else
				from
					s := set_to_be_tested
				until
					not s.is_empty
				loop
					s := set_to_be_tested
				end
				s2 := some_set_a ∩ same_set_a (s).others
			end
			assert ("s ⊄ s2", s ⊄ s2)
			assert ("s ⊄ s2 ok", properties.is_not_strict_subset_ok (s, s2))

			s2 := some_set_a & some_object_a
			s := set_to_be_tested ∩ same_set_a (s2).others
			assert ("not (s ⊄ s2)", not (s ⊄ s2))
			assert ("not (s ⊄ s2) ok", properties.is_not_strict_subset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_not_strict_subset", s ⊄ s2 ⇒ True)
			assert ("is_not_strict_subset_ok", properties.is_not_strict_subset_ok (s, s2))
		end

	test_is_strict_superset
			-- Test {STS_SET}.is_strict_superset.
			-- Test {STI_SET}.is_strict_superset.
		note
			testing: "covers/{STS_SET}.is_strict_superset"
			testing: "covers/{STI_SET}.is_strict_superset"
		local
			a: A
			s: like set_to_be_tested
			s2: like some_set_a
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s2 := some_set_a ∩ same_set_a (s).others
			assert ("s ⊃ s2", s ⊃ s2)
			assert ("s ⊃ s2 ok", properties.is_strict_superset_ok (s, s2))

			s := set_to_be_tested
			if next_random_item \\ 2 = 0 then
				s2 := same_set_a (s)
			else
				a := some_object_a
				s := s / same_object_a (a)
				s2 := some_set_a & same_object_a (a)
			end
			assert ("not (s ⊃ s2)", not (s ⊃ s2))
			assert ("not (s ⊃ s2) ok", properties.is_strict_superset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_strict_superset", s ⊃ s2 ⇒ True)
			assert ("is_strict_superset_ok", properties.is_strict_superset_ok (s, s2))
		end

	test_is_not_strict_superset
			-- Test {STS_SET}.is_not_strict_superset.
			-- Test {STI_SET}.is_not_strict_superset.
		note
			testing: "covers/{STS_SET}.is_not_strict_superset"
			testing: "covers/{STI_SET}.is_not_strict_superset"
		local
			a: A
			s: like set_to_be_tested
			s2: like some_set_a
		do
			s := set_to_be_tested
			if next_random_item \\ 2 = 0 then
				s2 := same_set_a (s)
			else
				a := some_object_a
				s := s / same_object_a (a)
				s2 := some_set_a & same_object_a (a)
			end
			assert ("s ⊅ s2", s ⊅ s2)
			assert ("s ⊅ s2 ok", properties.is_not_strict_superset_ok (s, s2))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s2 := some_set_a ∩ same_set_a (s).others
			assert ("not (s ⊅ s2)", not (s ⊅ s2))
			assert ("not (s ⊅ s2) ok", properties.is_not_strict_superset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_not_strict_superset", s ⊅ s2 ⇒ True)
			assert ("is_not_strict_superset_ok", properties.is_not_strict_superset_ok (s, s2))
		end

	test_is_trivial_subset
			-- Test {STI_SET}.is_trivial_subset.
		note
			testing: "covers/{STI_SET}.is_trivial_subset"
		local
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			if next_random_item \\ 2 = 0 then
				s := set_to_be_tested.o
				s2 := some_set_a
			else
				s := set_to_be_tested
				s2 := same_set_a (s)
			end
			assert ("s.is_trivial_subset (s2)", s.is_trivial_subset (s2))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s1 := same_set_a (s)
				check
					is_not_empty: not s1.is_empty -- s1 ≍ s
				end
			s2 := some_set_a / same_object_a (s1.any)
			assert ("not s.is_trivial_subset (s2)", not s.is_trivial_subset (s2))

			assert ("is_trivial_subset", set_to_be_tested.is_trivial_subset (some_set_a) ⇒ True)
		end

	test_is_trivial_superset
			-- Test {STS_SET}.is_trivial_superset.
			-- Test {STI_SET}.is_trivial_superset.
		note
			testing: "covers/{STS_SET}.is_trivial_superset"
			testing: "covers/{STI_SET}.is_trivial_superset"
		local
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			s := set_to_be_tested
			if next_random_item \\ 2 = 0 then
				s2 := same_set_a (s)
			else
				s2 := some_set_a.o
			end
			assert ("s.is_trivial_superset (s2)", s.is_trivial_superset (s2))
			assert ("s.is_trivial_superset (s2) ok", properties.is_trivial_superset_ok (s, s2))

			s1 := some_set_a & some_object_a
			s2 := same_set_a (s1)
				check
					is_not_empty: not s2.is_empty -- Above: s1 := some_set_a & some_object_a; s2 ≍ s1
				end
			s := set_to_be_tested / same_object_a (s2.any)
			assert ("not s.is_trivial_superset (s2)", not s.is_trivial_superset (s2))
			assert ("not s.is_trivial_superset (s2) ok", properties.is_trivial_superset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_trivial_superset", s.is_trivial_superset (s2) ⇒ True)
			assert ("is_trivial_superset_ok", properties.is_trivial_superset_ok (s, s2))
		end

	test_is_proper_subset
			-- Test {STI_SET}.is_proper_subset.
		note
			testing: "covers/{STI_SET}.is_proper_subset"
		local
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			from
				s2 := some_set_a
			until
				s2 ⊈ s
			loop
				s2 := some_set_a
			end
			s2 := s2 ∪ same_set_a (s)
			assert ("s.is_proper_subset (s2)", s.is_proper_subset (s2))
			assert ("s.is_proper_subset (s2) ok", properties.is_proper_subset_ok (s, s2))

			inspect
				next_random_item \\ 3
			when 0 then
				s := set_to_be_tested.o
				s2 := some_set_a
			when 1 then
				s := set_to_be_tested
				s2 := same_set_a (s)
			when 2 then
				from
					s := set_to_be_tested
				until
					not s.is_empty
				loop
					s := set_to_be_tested
				end
				s1 := same_set_a (s)
					check
						is_not_empty: not s1.is_empty -- s1 ≍ s
					end
				s2 := some_set_a / same_object_a (s1.any)
			end
			assert ("not s.is_proper_subset (s2)", not s.is_proper_subset (s2))
			assert ("not s.is_proper_subset (s2) ok", properties.is_proper_subset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_proper_subset", s.is_proper_subset (s2) ⇒ True)
			assert ("is_proper_subset_ok", properties.is_proper_subset_ok (s, s2))
		end

	test_is_proper_superset
			-- Test {STS_SET}.is_proper_superset.
			-- Test {STI_SET}.is_proper_superset.
		note
			testing: "covers/{STS_SET}.is_proper_superset"
			testing: "covers/{STI_SET}.is_proper_superset"
		local
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			from
				s := set_to_be_tested
			until
				not s.others.is_empty
			loop
				s := set_to_be_tested
			end
			s1 := same_set_a (s)
				check
					is_not_empty_1: not s1.is_empty -- s1 ≍ s
				end
			s2 := some_set_a ∩ same_set_a (s1.others.others) & same_object_a (s1.any)
			assert ("s.is_proper_superset (s2)", s.is_proper_superset (s2))
			assert ("s.is_proper_superset (s2) ok", properties.is_proper_superset_ok (s, s2))

			inspect
				next_random_item \\ 3
			when 0 then
				s := set_to_be_tested
				s2 := some_set_a.o
			when 1 then
				s := set_to_be_tested
				s2 := same_set_a (s)
			when 2 then
				s1 := some_set_a & some_object_a
				s2 := same_set_a (s1)
					check
						is_not_empty_1: not s1.is_empty -- Above: s1 := some_set_a & some_object_a
					end
				s := set_to_be_tested / same_object_a (s1.any)
			end
			assert ("not s.is_proper_superset (s2)", not s.is_proper_superset (s2))
			assert ("not s.is_proper_superset (s2) ok", properties.is_proper_superset_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_proper_superset", s.is_proper_superset (s2) ⇒ True)
			assert ("is_proper_superset_ok", properties.is_proper_superset_ok (s, s2))
		end

	test_is_disjoint
			-- Test {STI_SET}.is_disjoint.
		note
			testing: "covers/{STI_SET}.is_disjoint"
		local
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			if next_random_item \\ 2 = 0 then
				s := set_to_be_tested
				s2 := some_set_a ∖ same_set_a (s)
			else
				s2 := some_set_a
				s := set_to_be_tested ∖ same_set_a (s2)
			end
			assert ("s.is_disjoint (s2)", s.is_disjoint (s2))
			assert ("s.is_disjoint (s2) ok", properties.is_disjoint_ok (s, s2))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s1 := same_set_a (s)
				check
					is_not_empty: not s1.is_empty -- s1 ≍ s
				end
			s2 := some_set_a & same_object_a (s1.any)
			assert ("not s.is_disjoint (s2)", not s.is_disjoint (s2))
			assert ("not s.is_disjoint (s2) ok", properties.is_disjoint_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("is_disjoint", s.is_disjoint (s2) ⇒ True)
			assert ("is_disjoint_ok", properties.is_disjoint_ok (s, s2))
		end

	test_intersects
			-- Test {STS_SET}.intersects.
			-- Test {STI_SET}.intersects.
		note
			testing: "covers/{STS_SET}.intersects"
			testing: "covers/{STI_SET}.intersects"
		local
			s: like set_to_be_tested
			s1, s2: like some_set_a
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			s1 := same_set_a (s)
				check
					is_not_empty: not s1.is_empty -- s1 ≍ s
				end
			s2 := some_set_a & same_object_a (s1.any)
			assert ("s.intersects (s2)", s.intersects (s2))
			assert ("s.intersects (s2) ok", properties.intersects_ok (s, s2))

			if next_random_item \\ 2 = 0 then
				s := set_to_be_tested
				s2 := some_set_a ∖ same_set_a (s)
			else
				s2 := some_set_a
				s := set_to_be_tested ∖ same_set_a (s2)
			end
			assert ("not s.intersects (s2)", not s.intersects (s2))
			assert ("not s.intersects (s2) ok", properties.intersects_ok (s, s2))

			s := set_to_be_tested
			s2 := some_set_a
			assert ("intersects", s.intersects (s2) ⇒ True)
			assert ("intersects_ok", properties.intersects_ok (s, s2))
		end

feature -- Test routines (Relationship)

	test_is_partitioned
			-- Test {STS_SET}.is_partitioned.
			-- Test {STI_SET}.is_partitioned.
		note
			testing: "covers/{STS_SET}.is_partitioned"
			testing: "covers/{STI_SET}.is_partitioned"
		local
			s1: like set_to_be_tested
			s2, si: like some_set_a
			ss: like some_set_of_sets_a
			j: NATURAL
			chk_ops, chk_tests: BOOLEAN
			n, n_sup: like set_to_be_tested.cardinality
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s1 := set_to_be_tested
					ss := some_set_of_sets_a.o_s
					si := s1
				invariant
					building_ss_up: (s1 ∖ si).is_partitioned (ss)
				until
					si.is_empty
				loop
					from
						s2 := some_set_a.o
							check
								positive: 0 < # si -- not si.is_empty
							end
						j := some_index_up_to (# si)
					invariant
						building_s2_up: s2 ≍ (s1 ∖ si ∖ (⋃ ss))
						si_emptiness: j > 0 ⇒ not si.is_empty
						s2_emptiness: j = 0 ⇒ not s2.is_empty
					until
						j = 0
					loop
							check
								si_is_not_empty: not si.is_empty -- j > 0
								s2_does_not_have_si_any: s2 ∌ si.any -- s2 ≍ (s1 ∖ si...)
							end
						s2 := s2.extended (si.any)
						si := si.others
						j := j - 1
					variant
						j: j.as_integer_64
					end
						check
							ss_does_not_have_s2: ss ∌ s2 -- s2 ≍ (s1 ∖ si ∖ (⋃ ss)); j = 0 ⇒ not s2.is_empty
						end
					ss := ss.extended (s2)
				variant
					cardinality: {like some_set_a}.natural_as_integer (# si)
				end
				n := # s1
				print ("s1: ") print (# s1) print ("; ")
				print ("ss: ") print (ss.set_to_natural.set_map (ss, agent {STS_SET [A, EQ]}.cardinality))
				if attached {STS_SUBSET_FAMILY [A, EQ]} ss as sf then
					print (", ") print (# sf.superset)
				end
				print ('%N')
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1_is_partitioned_ss", s1.is_partitioned (ss))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end

			s1 := set_to_be_tested
			ss := some_set_of_sets_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
				print ("s1: ") print (n) print ("; ")
				n := n ∨ ss.set_to_natural.set_reduction (
					ss, 0, agent (acc: like set_to_be_tested.cardinality; s: STS_SET [A, EQ]): like set_to_be_tested.cardinality
						do
							Result := acc ∨ # s
						end
					)
				print ("ss: ") print (ss.set_to_natural.set_map (ss, agent {STS_SET [A, EQ]}.cardinality))
				if attached {STS_SUBSET_FAMILY [A, EQ]} ss as sf then
					n_sup := # sf.superset
					print (", ") print (n_sup)
					n := n ∨ n_sup
				end
				print ('%N')
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("is_partitioned", s1.is_partitioned (ss) ⇒ True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
		end

feature -- Test routines (Quantifier)

	test_exists
			-- Test {STI_SET}.exists.
		note
			testing: "covers/{STI_SET}.exists"
		local
			a: A
			s: like set_to_be_tested
			p: PREDICATE [A]
			chk_ops: BOOLEAN
		do
			p := agent p_x
			from
				a := some_object_a
			until
				p (a)
			loop
				a := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested & a
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_exists_p", s |∃ p)
			assert ("s_exists_p_ok", properties.exists_ok (s, p))

			s := set_to_be_tested
			assert ("exists", (s |∃ p) ⇒ True)
			assert ("exists_ok", properties.exists_ok (s, p))
		end

	test_does_not_exist
			-- Test {STS_SET}.does_not_exist.
			-- Test {STI_SET}.does_not_exist.
		note
			testing: "covers/{STS_SET}.does_not_exist"
			testing: "covers/{STI_SET}.does_not_exist"
		local
			s: like set_to_be_tested
			p: PREDICATE [A]
			chk_ops: BOOLEAN
		do
			p := agent p_x
			s := set_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := s | agent negated (p, ?)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_does_not_exist_p", s |∄ p)
			assert ("s_does_not_exist_p_ok", properties.does_not_exist_ok (s, p))

			s := set_to_be_tested
			assert ("does_not_exist", s |∄ p ⇒ True)
			assert ("does_not_exist_ok", properties.does_not_exist_ok (s, p))
		end

	test_exists_unique
			-- Test {STI_SET}.exists_unique.
		note
			testing: "covers/{STI_SET}.exists_unique"
		local
			s: like set_to_be_tested
			fs: like some_set_a
			p: PREDICATE [A]
			chk_ops: BOOLEAN
		do
			p := agent p_x

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s := set_to_be_tested
					fs := s | p
				until
					not fs.is_empty
				loop
					s := set_to_be_tested
					fs := s | p
				end
				s := s ∖ fs.others
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_exists_unique_p", s |∃! p)
			assert ("s_exists_unique_p_ok", properties.exists_unique_ok (s, p))

			s := set_to_be_tested
			assert ("exists_unique", s |∃! p ⇒ True)
			assert ("exists_unique_ok", properties.exists_unique_ok (s, p))
		end

	test_exists_pair
			-- Test {STI_SET}.exists_pair.
		note
			testing: "covers/{STI_SET}.exists_pair"
		local
			a, b: A
			s: like set_to_be_tested
			p: PREDICATE [A, A]
			chk_ops, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
		do
			p := agent p_x1_x2
			from
				a := some_object_a
				b := some_object_a
			until
				p (a, b)
			loop
				a := some_object_a
				b := some_object_a
			end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested & a & b
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s_exists_pair_p", s.exists_pair (p))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s_exists_pair_p_ok", properties.exists_pair_ok (s, p))

			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("exists_pair", s.exists_pair (p) ⇒ True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("exists_pair_ok", properties.exists_pair_ok (s, p))
		end

	test_does_not_exist_pair
			-- Test {STS_SET}.does_not_exist_pair.
			-- Test {STI_SET}.does_not_exist_pair.
		note
			testing: "covers/{STS_SET}.does_not_exist_pair"
			testing: "covers/{STI_SET}.does_not_exist_pair"
		local
			p: PREDICATE [A, A]
			s: like set_to_be_tested
			sf: like set_to_be_tested.subsets
			chk_ops: BOOLEAN
		do
			p := agent p_x1_x2
			s := set_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sf := s.powerset
				sf := sf | agent (xs: STS_SET [A, EQ]; ia_p: PREDICATE [A, A]): BOOLEAN
					do
						Result := xs.for_all_pairs (agent pair_negated (ia_p, ?, ?))
					end (?, p)
				s := s ∩ (
					sf.reduced (
						sf.o, agent (acc, xs: STS_SET [A, EQ]): STS_SET [A, EQ]
							do
								Result := if # xs > # acc then xs else acc end
							end
						)
					)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_does_not_exist_pair_p", s.does_not_exist_pair (p))
			assert ("s_does_not_exist_pair_p_ok", properties.does_not_exist_pair_ok (s, p))

			s := set_to_be_tested
			assert ("does_not_exist_pair", s.does_not_exist_pair (p) ⇒ True)
			assert ("does_not_exist_pair_ok", properties.does_not_exist_pair_ok (s, p))
		end

	test_exists_distinct_pair
			-- Test {STI_SET}.exists_distinct_pair.
		note
			testing: "covers/{STI_SET}.exists_distinct_pair"
		local
			a, b: A
			s: like set_to_be_tested
			p: PREDICATE [A, A]
			chk_ops, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
		do
			p := agent p_x1_x2
			from
				a := some_object_a
				b := some_object_a
			until
				p (a, b) and not eq (a, b)
			loop
				a := some_object_a
				b := some_object_a
			end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested & a & b
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s_exists_distinct_pair_p", s.exists_distinct_pair (p))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s_exists_distinct_pair_p_ok", properties.exists_distinct_pair_ok (s, p))

			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("exists_distinct_pair", s.exists_distinct_pair (p) ⇒ True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("exists_distinct_pair_ok", properties.exists_distinct_pair_ok (s, p))
		end

	test_for_all
			-- Test {STI_SET}.for_all.
		note
			testing: "covers/{STI_SET}.for_all"
		local
			s: like set_to_be_tested
			p: PREDICATE [A]
			chk_ops: BOOLEAN
		do
			p := agent p_x

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested | p
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_for_all_p", s |∀ p)
			assert ("s_for_all_p_ok", properties.for_all_ok (s, p))

			s := set_to_be_tested
			assert ("for_all", (s |∀ p) ⇒ True)
			assert ("for_all_ok", properties.for_all_ok (s, p))
		end

	test_for_all_pairs
			-- Test {STI_SET}.for_all_pairs.
		note
			testing: "covers/{STI_SET}.for_all_pairs"
		local
			s: like set_to_be_tested
			sf: STI_SUBSET_FAMILY [A, EQ]
			p: PREDICATE [A, A]
			chk_ops: BOOLEAN
		do
			p := agent p_x1_x2
			s := set_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sf := s.powerset | agent  (xs: STS_SET [A, EQ]; ia_p: PREDICATE [A, A]): BOOLEAN
					do
						Result := (× xs).filtered_xy (ia_p) ≍ (× xs)
					end (?, p)

					check
						is_not_empty: not sf.is_empty -- (²xs) ｜ˣʸ ia_p) ≍ (²xs) holds for {}.
					end
				s := s ∩ sf.proper_reduced (
					agent  (acc, xs: STS_SET [A, EQ]): STS_SET [A, EQ]
						do
							if # acc >= # xs then
								Result := acc
							else
								Result := xs
							end
						end
					)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_for_all_pairs_p", s.for_all_pairs (p))
			assert ("s_for_all_pairs_p_ok", properties.for_all_pairs_ok (s, p))

			s := set_to_be_tested
			assert ("for_all_pairs", s.for_all_pairs (p) ⇒ True)
			assert ("for_all_pairs_ok", properties.for_all_pairs_ok (s, p))
		end

	test_for_all_distinct_pairs
			-- Test {STI_SET}.for_all_distinct_pairs.
		note
			testing: "covers/{STI_SET}.for_all_distinct_pairs"
		local
			s: like set_to_be_tested
			sf: STI_SUBSET_FAMILY [A, EQ]
			p: PREDICATE [A, A]
			chk_ops: BOOLEAN
		do
			p := agent p_x1_x2
			s := set_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				sf := s.powerset | agent  (xs: STS_SET [A, EQ]; ia_p: PREDICATE [A, A]): BOOLEAN
					do
						Result := (× xs).filtered_xy (agent pair_negated (ia_p, ?, ?)) ⊆ (∆ xs)
					end (?, p)

					check
						is_not_empty: not sf.is_empty -- (²xs) ｜ˣʸ agent xs.pair_negated (ia_p, ?, ?)) ⊆ (∆ xs)
					end
				s := s ∩ sf.proper_reduced (
					agent (acc, xs: STS_SET [A, EQ]): STS_SET [A, EQ]
						do
							if # acc >= # xs then
								Result := acc
							else
								Result := xs
							end
						end
					)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s_for_all_distinct_pairs_p", s.for_all_distinct_pairs (p))
			assert ("s_for_all_distinct_pairs_p_ok", properties.for_all_distinct_pairs_ok (s, p))

			s := set_to_be_tested
			assert ("for_all_distinct_pairs", s.for_all_distinct_pairs (p) ⇒ True)
			assert ("for_all_distinct_pairs_ok", properties.for_all_distinct_pairs_ok (s, p))
		end

feature -- Test routines (Operation)

	test_subsets
			-- Test {STI_SET}.subsets.
		note
			testing: "covers/{STI_SET}.subsets"
		local
			a: A
			s0: like some_set_a
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set.cardinality
			--chk_ops,
			chk_tests: BOOLEAN
		do
			a := some_object_a
			s0 := some_set_a
			s1 := set_to_be_tested
			s2 := some_set_a
--			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
--			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if (2 ^ n) ≤ max_subsets then
				if 2 ^ n > max_asserted_subsets then
					chk_tests := {ISE_RUNTIME}.check_assert (False)
				end
					assert ("subsets", attached s1.subsets)
				if chk_tests then
					chk_tests := {ISE_RUNTIME}.check_assert (True)
				end
			end
			assert ("subsets_ok", properties.subsets_ok (a, s0, s1, s2))
		end

	test_powerset
			-- Test {STI_SET}.powerset.
		note
			testing: "covers/{STI_SET}.powerset"
		local
			a: A
			s0: like some_set_a
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			a := some_object_a
			s0 := some_set_a
			s1 := set_to_be_tested
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
				print (n) print ('%N')
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if (2 ^ n) ≤ max_subsets then
				if (2 ^ n) > max_asserted_subsets then
					chk_tests := {ISE_RUNTIME}.check_assert (False)
				end
					assert ("powerset", attached s1.powerset)
				if chk_tests then
					chk_tests := {ISE_RUNTIME}.check_assert (True)
				end
			end
			assert ("powerset_ok", properties.powerset_ok (a, s0, s1, s2))
		end

	test_trivial_subsets
			-- Test {STI_SET}.trivial_subsets.
		note
			testing: "covers/{STI_SET}.trivial_subsets"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			assert ("trivial_subsets", attached s.trivial_subsets)
			assert ("trivial_subsets_ok", properties.trivial_subsets_ok (s))
		end

	test_proper_subsets
			-- Test {STS_SET}.proper_subsets.
			-- Test {STI_SET}.proper_subsets.
		note
			testing: "covers/{STS_SET}.proper_subsets"
			testing: "covers/{STI_SET}.proper_subsets"
		local
			s: like set_to_be_tested
			n: like set_to_be_tested.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			s := set_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			if (2 ^ n) ≤ max_subsets then
				if (2 ^ n) > max_asserted_subsets then
					chk_tests := {ISE_RUNTIME}.check_assert (False)
				end
					assert ("proper_subsets", attached s.proper_subsets)
				if chk_tests then
					chk_tests := {ISE_RUNTIME}.check_assert (True)
				end
			end
			assert ("proper_subsets_ok", properties.proper_subsets_ok (s))
		end

	test_filtered
			-- Test {STI_SET}.filtered.
		note
			testing: "covers/{STI_SET}.filtered"
		local
			s: like set_to_be_tested
			p: PREDICATE [A]
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			p := agent p_x
			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("filtered", attached (s | p))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("filtered_ok", properties.filtered_ok (s, p))
		end

	test_complemented
			-- Test {STI_SET}.complemented.
		note
			testing: "covers/{STI_SET}.complemented"
		local
			s1: like set_to_be_tested
			s2, s3: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			s1 := set_to_be_tested
			s2 := some_set_a
			s3 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2 + # s3
				s3 := s1 ∪ s2 ∪ s3
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
--				assert ("s1_is_subset_s3", s1 ⊆ s3) -- TODO: Syntax error!
				assert ("s1_is_subset_s3", s1 ⊆ s3)
				assert ("s1_complemented_s3", attached (s1 ∁ s3))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1_complemented_s3_ok", properties.complemented_ok (s1, s2, s3))

			s1 := set_to_be_tested
			s2 := some_set_a
			s3 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s3
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("complemented", s1 ⊆ s3 ⇒ attached (s1 ∁ s3))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("complemented_ok", properties.complemented_ok (s1, s2, s3))
		end

	test_united
			-- Test {STI_SET}.united.
		note
			testing: "covers/{STI_SET}.united"
		local
			s1: like set_to_be_tested
			s2, s3, s4: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: NATURAL
		do
			s1 := set_to_be_tested
			s2 := some_set_a
			s3 := some_set_a
			s4 := some_set_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
				print (n) print ('%N')
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("united", attached (s1 ∪ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("united_ok", properties.united_ok (some_object_a, s1, s2, s3, s4))
		end

	test_intersected
			-- Test {STI_SET}.intersected.
		note
			testing: "covers/{STI_SET}.intersected"
		local
			s1: like set_to_be_tested
			s2, s3, s4: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			s1 := set_to_be_tested
			s2 := some_set_a
			s3 := some_set_a
			s4 := some_set_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("intersected", attached (s1 ∩ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end

			assert ("intersected_ok", properties.intersected_ok (some_object_a, s1, s2, s3, s4))
		end

	test_subtracted
			-- Test {STI_SET}.subtracted.
		note
			testing: "covers/{STI_SET}.subtracted"
		local
			s1: like set_to_be_tested
			s2, s3, s4, s5: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
		do
			s1 := set_to_be_tested
			s2 := some_set_a
			s3 := some_set_a
			s4 := some_set_a
			s5 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("subtracted", attached (s1 ∖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("subtracted_ok", properties.subtracted_ok (some_object_a, s1, s2, s3, s4, s5))
		end

	test_subtracted_symmetricaly
			-- Test {STI_SET}.subtracted_symmetricaly.
		note
			testing: "covers/{STI_SET}.subtracted_symmetricaly"
		local
			s1: like set_to_be_tested
			s2, s3: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			s1 := set_to_be_tested
			s2 := some_set_a
			s3 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("subtracted_symmetricaly", attached (s1 ⊖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("subtracted_symmetricaly_ok", properties.subtracted_symmetricaly_ok (some_object_a, s1, s2, s3))
		end

	test_crossed
			-- Test {STS_SET}.crossed.
			-- Test {STI_SET}.crossed.
		note
			testing: "covers/{STS_SET}.crossed"
			testing: "covers/{STI_SET}.crossed"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			s1 := set_to_be_tested
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1 * # s2
				print (n) print ('%N')
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("crossed", attached (s1 × s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("crossed_ok", properties.crossed_ok (s1, s2, some_set_a))
		end

	test_squared
			-- Test {STS_SET}.squared.
			-- Test {STI_SET}.squared.
		note
			testing: "covers/{STS_SET}.squared"
			testing: "covers/{STI_SET}.squared"
		local
			s: like set_to_be_tested
			n: like set_to_be_tested.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n ^ 2 > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("squared", attached (× s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("squared_ok", properties.squared_ok (s))
		end

	test_diagonal
			-- Test {STI_SET}.diagonal.
		note
			testing: "covers/{STI_SET}.diagonal"
		local
			s: like set_to_be_tested
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("diagonal", attached (∆ s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("diagonal_ok", properties.diagonal_ok (s))
		end

	test_raised
			-- Test {STS_SET}.raised.
			-- Test {STI_SET}.raised.
		note
			testing: "covers/{STS_SET}.raised"
			testing: "covers/{STI_SET}.raised"
		local
			s: like set_to_be_tested
			n, m: NATURAL
			chk_ops, chk_tests: BOOLEAN
		do
			s := set_to_be_tested
			n := some_count
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				m := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if (m ^ n) ≤ max_tuples then
				if m ^ n > max_asserted_tuples then
					chk_tests := {ISE_RUNTIME}.check_assert (False)
				end
					assert ("raised", attached (s ^ n))
				if chk_tests then
					chk_tests := {ISE_RUNTIME}.check_assert (True)
				end
			end
			assert ("raised_ok", properties.raised_ok (s, n))
		end

	test_crossed_tuples
			-- Test {STI_SET}.crossed_tuples.
		note
			testing: "covers/{STI_SET}.crossed_tuples"
		local
			s: like set_to_be_tested
			ts: like some_math_set_ta
			chk_n, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
		do
			s := set_to_be_tested
			ts := some_math_set_ta
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s * # ts
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("crossed_tuples", attached s.crossed_tuples (ts))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("crossed_tuples_ok", properties.crossed_tuples_ok (s, ts))
		end

	test_singleton_crossed_tuples
			-- Test {STI_SET}.singleton_crossed_tuples.
		note
			testing: "covers/{STI_SET}.singleton_crossed_tuples"
		local
			s: like set_to_be_tested
			x: A
			ts: like some_math_set_ta
		do
			s := set_to_be_tested
			x := some_object_a
			ts := some_math_set_ta
			assert ("singleton_crossed_tuples", attached (s.singleton_crossed_tuples (x, ts)))
			assert ("singleton_crossed_tuples_ok", properties.singleton_crossed_tuples_ok (s, x, ts))
		end

feature -- Test routines (Transformation)

	test_mapped
			-- Test {STS_SET}.mapped.
			-- Test {STI_SET}.mapped.
		note
			testing: "covers/{STS_SET}.mapped"
			testing: "covers/{STI_SET}.mapped"
		local
			s: like set_to_be_tested
			f: FUNCTION [A, A]
			chk_n, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
		do
			s := set_to_be_tested
			f := agent f_x
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("mapped", attached (s ↦ f))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("mapped_ok", properties.mapped_ok (s, f))
		end

	test_reduced
			-- Test {STS_SET}.reduced.
			-- Test {STI_SET}.reduced.
		note
			testing: "covers/{STS_SET}.reduced"
			testing: "covers/{STI_SET}.reduced"
		local
			s: like set_to_be_tested
			start: A
			f: FUNCTION [A, A, A]
			chk_n, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
		do
			s := set_to_be_tested
			start := some_object_a
			f := agent f_acc_x
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("reduced", attached s.reduced (start, f) ⇒ True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("reduced_ok", properties.reduced_ok (s, start, f))
		end

	test_proper_reduced
			-- Test {STS_SET}.proper_reduced.
			-- Test {STI_SET}.proper_reduced.
		note
			testing: "covers/{STS_SET}.proper_reduced"
			testing: "covers/{STI_SET}.proper_reduced"
		local
			s: like set_to_be_tested
			f: FUNCTION [A, A, A]
			chk_n, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
		do
			f := agent f_acc_x
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s_proper_reduced_f", attached s.proper_reduced (f) ⇒ True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s_proper_reduced_f_ok", properties.proper_reduced_ok (s, f))

			s := set_to_be_tested
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("proper_reduced", not s.is_empty ⇒ attached s.proper_reduced (f) ⇒ True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("proper_reduced_ok", properties.proper_reduced_ok (s, f))
		end

feature -- Test routines (Conversion)

	test_natural_as_integer
			-- Test {STI_SET}.natural_as_integer.
		note
			testing: "covers/{STI_SET}.natural_as_integer"
		do
			assert ("natural_as_integer", attached set_to_be_tested.natural_as_integer (some_natural))
		end

	test_as_singleton
			-- Test {STI_SET}.as_singleton.
		note
			testing: "covers/{STI_SET}.as_singleton"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			assert ("as_singleton", attached s.as_singleton)
			assert ("as_singleton_ok", properties.as_singleton_ok (s))
		end

	test_n_times
			-- Test {STI_SET}.n_times.
		note
			testing: "covers/{STI_SET}.n_times"
		local
			s: like set_to_be_tested
			n: NATURAL
		do
			s := set_to_be_tested
			n := some_count
			assert ("n_times", attached s.n_times (n))
			assert ("n_times_ok", properties.n_times_ok (s, n))
		end

	test_as_tuple
			-- Test {STI_SET}.as_tuple.
		note
			testing: "covers/{STI_SET}.as_tuple"
		local
			s: like set_to_be_tested
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("as_tuple", attached s.as_tuple)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("as_tuple_ok", properties.as_tuple_ok (s))
		end

feature -- Test routines (Factory)

	test_o
			-- Test {STI_SET}.o.
		note
			testing: "covers/{STI_SET}.o"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			assert ("o", attached s.o)
			assert ("o_ok", properties.o_ok (s))
		end

	test_empty_set
			-- Test {STI_SET}.empty_set.
		note
			testing: "covers/{STI_SET}.empty_set"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			assert ("empty_set", attached s.empty_set)
			assert ("empty_set_ok", properties.empty_set_ok (s))
		end

	test_singleton
			-- Test {STI_SET}.singleton.
		note
			testing: "covers/{STI_SET}.singleton"
		local
			s: like set_to_be_tested
			a: A
		do
			s := set_to_be_tested
			a := some_object_a
			assert ("singleton", attached s.singleton (a))
			assert ("singleton_ok", properties.singleton_ok (s, a))
		end

	test_pair
			-- Test {STI_SET}.pair.
		note
			testing: "covers/{STI_SET}.pair"
		local
			a, b: A
			s: like set_to_be_tested
		do
			a := some_object_a
			b := some_object_a
			s := set_to_be_tested
			assert ("pair", attached s.pair (a, b))
			assert ("pair_ok", properties.pair_ok (s, a, b))
		end

	test_converted
			-- Test {STI_SET}.converted.
		note
			testing: "covers/{STI_SET}.converted"
		do
			assert ("converted", attached set_to_be_tested.converted (some_set_a))
		end

	test_converted_element
			-- Test {STI_SET}.converted_element.
		note
			testing: "covers/{STI_SET}.converted_element"
		do
			assert ("converted_element", attached set_to_be_tested.converted_element (some_object_a) ⇒ True)
		end

	test_unit
			-- Test {STI_SET}.unit.
		note
			testing: "covers/{STI_SET}.unit"
		do
			assert ("unit", attached set_to_be_tested.unit)
			assert ("unit_ok", properties.unit_ok (set_to_be_tested))
		end

feature -- Test routines (Predicate)

	test_ored
			-- Test {STS_SET}.ored.
			-- Test {STI_SET}.ored.
		note
			testing: "covers/{STS_SET}.ored"
			testing: "covers/{STI_SET}.ored"
		local
			p1, p2: PREDICATE [A]
			a: A
		do
			from
				p1 := agent p_x
				p2 := agent set_to_be_tested.has
				a := some_object_a
			until
				p1 (a) or p2 (a)
			loop
				p2 := agent set_to_be_tested.has
				a := some_object_a
			end
			assert ("p1_a_or_p2_a", set_to_be_tested.ored (p1, p2, a))

			p2 := agent set_to_be_tested.has
			a := some_object_a
			assert ("ored", set_to_be_tested.ored (p2, p2, a) ⇒ True)
		end

	test_xored
			-- Test {STS_SET}.xored.
			-- Test {STI_SET}.xored.
		note
			testing: "covers/{STS_SET}.xored"
			testing: "covers/{STI_SET}.xored"
		local
			p1, p2: PREDICATE [A]
			a: A
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					p1 := agent p_x
					p2 := agent set_to_be_tested.has
					a := some_object_a
				until
					p1 (a) xor p2 (a)
				loop
					p2 := agent set_to_be_tested.has
					a := some_object_a
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("p1_a_xor_p2_a", set_to_be_tested.xored (p1, p2, a))

			p2 := agent set_to_be_tested.has
			a := some_object_a
			assert ("xored", set_to_be_tested.xored (p2, p2, a) ⇒ True)
		end

	test_iff
			-- Test {STS_SET}.iff.
			-- Test {STI_SET}.iff.
		note
			testing: "covers/{STS_SET}.iff"
			testing: "covers/{STI_SET}.iff"
		local
			p1, p2: PREDICATE [A]
			a: A
			chk_ops, chk_tests: BOOLEAN
			s: like some_set_a
			n: like some_set_a.cardinality
		do
			p1 := agent p_x
			a := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				if p1 (a) then
					s := some_set_a & a
				else
					s := some_set_a / a
				end
				p2 := agent s.has
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("p1_a_iff_p2_a", set_to_be_tested.iff (p1, p2, a))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end

			a := some_object_a
			s := some_set_a
			p2 := agent s.has
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("iff", set_to_be_tested.iff (p2, p2, a) ⇒ True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
		end

	test_equality_holds
			-- Test {STI_SET}.equality_holds.
		note
			testing: "covers/{STI_SET}.equality_holds"
		local
			a1, a2: A
		do
			a1 := some_object_a
			a2 := same_object_a (a1)
			assert ("equality_holds_a1_a2", set_to_be_tested.equality_holds (a1, a2))

			a1 := some_object_a
			a2 := some_object_a
			assert ("equality_holds", set_to_be_tested.equality_holds (a1, a2) ⇒ True)
		end

	test_subset_relation_holds
			-- Test {STI_SET}.subset_relation_holds.
		note
			testing: "covers/{STI_SET}.subset_relation_holds"
		local
			s: like set_to_be_tested
			s1, s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
			a: A
		do
			s := set_to_be_tested
			s1 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s2 := some_set_a ∪ s1
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ⊆ s2", s.subset_relation_holds (s1, s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ⊆ s2 ok", properties.subset_relation_holds_ok (s, s1, s2, some_set_a))

			a := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s1 := some_set_a & a
				s2 := some_set_a / a
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ⊈ s2", not s.subset_relation_holds (s1, s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ⊈ s2 ok", properties.subset_relation_holds_ok (s, s1, s2, some_set_a))

			s1 := some_set_a
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("subset_relation_holds", s.subset_relation_holds (s1, s2) ⇒ True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("subset_relation_holds_ok", properties.subset_relation_holds_ok (s, s1, s2, some_set_a))
		end

feature -- Test routines (Mapping)

	test_second_after_first
			-- Test {STS_SET}.second_after_first.
			-- Test {STI_SET}.second_after_first.
		note
			testing: "covers/{STS_SET}.second_after_first"
			testing: "covers/{STI_SET}.second_after_first"
		do
			assert ("second_after_first", attached set_to_be_tested.second_after_first (new_pair_a_ta))
		end

feature -- Test routines (Reduction)

	test_natural_successor
			-- Test {STS_SET}.natural_successor.
			-- Test {STI_SET}.natural_successor.
		note
			testing: "covers/{STS_SET}.natural_successor"
			testing: "covers/{STI_SET}.natural_successor"
		do
			assert ("natural_successor", attached set_to_be_tested.natural_successor (some_natural, some_object_a))
		end

	test_disjunction
			-- Test {STS_SET}.disjunction.
			-- Test {STI_SET}.disjunction.
		note
			testing: "covers/{STS_SET}.disjunction"
			testing: "covers/{STI_SET}.disjunction"
		local
			p: PREDICATE [A]
			b: BOOLEAN
			a: A
		do
			p := agent p_x
			from
				b := next_random_item \\ 2 = 0
				a := some_object_a
			until
				b or p (a)
			loop
				b := next_random_item \\ 2 = 0
				a := some_object_a
			end
			assert ("disjunction_b_p_a", set_to_be_tested.disjunction (b, p, a))

			b := next_random_item \\ 2 = 0
			a := some_object_a
			assert ("disjunction", set_to_be_tested.disjunction (b, p, a) ⇒ True)
		end

	test_conjunction
			-- Test {STS_SET}.conjunction.
			-- Test {STI_SET}.conjunction.
		note
			testing: "covers/{STS_SET}.conjunction"
			testing: "covers/{STI_SET}.conjunction"
		local
			p: PREDICATE [A]
			b: BOOLEAN
			a: A
		do
			p := agent p_x
			from
				b := next_random_item \\ 2 = 0
				a := some_object_a
			until
				b and p (a)
			loop
				b := next_random_item \\ 2 = 0
				a := some_object_a
			end
			assert ("conjunction_b_p_a", set_to_be_tested.conjunction (b, p, a))

			b := next_random_item \\ 2 = 0
			a := some_object_a
			assert ("conjunction", set_to_be_tested.conjunction (b, p, a) ⇒ True)
		end

	test_sets_plus_same_sets_with_element
			-- Test {STS_SET}.sets_plus_same_sets_with_element.
			-- Test {STI_SET}.sets_plus_same_sets_with_element.
		note
			testing: "covers/{STS_SET}.sets_plus_same_sets_with_element"
			testing: "covers/{STI_SET}.sets_plus_same_sets_with_element"
		local
			s: like set_to_be_tested
			ss: like some_math_set_sa
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			s := set_to_be_tested
			ss := some_math_set_sa
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := ss.element_to_natural.set_reduction (
					ss, # s, agent (acc: NATURAL; sa: STS_SET [A, EQ]): NATURAL
						do
							Result := acc ∨ # sa
						end
					)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("sets_plus_same_sets_with_element", attached s.sets_plus_same_sets_with_element (ss, some_object_a))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
		end

feature {NONE} -- Factory (element to be tested)

	set_to_be_tested: like new_set_a
			-- Set meant to be under tests
		do
			check
				new_s: attached {FUNCTION [like new_set_a]} (agent new_set_a) as new_s

					-- `some_direct_instance' and `new_set_a' definitions
				s: attached {like new_set_a} some_direct_instance (new_s) as s
				monomorphic: s.generating_type ~ {detachable like new_set_a}
			then
				Result := cropped_set (s)
			end
		end

feature {NONE} -- Conversion

	set_from_sa (sa: STS_SET [A, EQ]): like set_to_be_tested
			-- `sa' converted to a set like `set_to_be_tested'
		do
			Result := sa
		ensure
			same_set: Result ≍ sa
		end

	set_from_saa (saa: STS_TRANSFORMABLE_SET [A, A, EQ, EQ]): like set_to_be_tested
			-- `saa' converted to a set like `set_to_be_tested'
		do
			Result := saa
		ensure
			same_set: Result ≍ saa
		end

feature -- Predicate

	negated (p: PREDICATE [A]; x: A): BOOLEAN
			-- Logical negation of `p' (`x'), i.e. is `p' (`x') false?
		do
			Result := not p (x)
		ensure
			class
			definition: Result = not p (x)
		end

	pair_negated (p: PREDICATE [A, A]; x, y: A): BOOLEAN
			-- Logical negation of `p' (`x', `y'), i.e. is `p' (`x', `y') false?
		do
			Result := not p (x, y)
		ensure
			class
			definition: Result = not p (x, y)
		end

feature {NONE} -- Implementation

	max_asserted_tuples: NATURAL
			-- Maximum number of n-tuples within a set to be checked with assertions enabled
		deferred
		end

	max_asserted_subsets: NATURAL
			-- Maximum number of subsets to be checked with assertions enabled
		deferred
		end

	max_subsets: NATURAL
			-- Maximum number of subsets to be checked within feature tests
		deferred
		end

	max_tuples: NATURAL
			-- Maximum number of n-tuples to be checked within feature tests
		deferred
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
