﻿note
	description: "Test suite for {STI_TRANSFORMABLE_SET}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date: Sat 2018-11-10 18:52:01 -0300 $"
	revision: "$Revision$"
	testing: "type/manual"

deferred class
	TRANSFORMABLE_SET_TESTS [A, B, EQ_A -> STS_EQUALITY [A] create default_create end, EQ_B -> STS_EQUALITY [B] create default_create end]

inherit
	SET_TESTS [A, EQ_A]
		rename
			eq as eq_a,
			test_eq as test_eq_a,
			test_crossed as test_crossed_with_same_kind,
			test_mapped as test_mapped_to_same_kind,
			test_reduced as test_reduced_to_element_like,
			f_x as f_x_to_same_kind,
			f_acc_x as f_acc_x_to_same_kind,
			f_x_acc as f_x_acc_to_same_kind,
			f_acc_i_x as f_acc_i_x_to_same_kind,
			f_i_x_acc as f_i_x_acc_to_same_kind
		undefine
			some_equality,
			some_pair,
			some_pair_aa,
			some_set,
			some_set_a,
			same_set_a,
			some_pair_composer,
			some_transformer,
			some_n_tuple,
			some_n_tuple_a,
			same_n_tuple_a
		redefine
			test_make_from_set,
			test_equals,
			test_is_subset,
			test_is_disjoint,
			test_united,
			test_intersected,
			test_subtracted,
			test_subtracted_symmetricaly,
			set_to_be_tested,
			set_from_sa,
			set_from_saa
		end

	SETS_TESTS_RESOURCES_BINARY [A, B, EQ_A, EQ_B]

feature -- Access

	properties: TRANSFORMABLE_SET_PROPERTIES [A, B, EQ_A, EQ_B]
			-- Object that checks the set-theory properties of {STI_TRANSFORMABLE_SET}
		deferred
		end

feature -- Test routines (Initialization)

	test_make_from_set
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_SET}.make_from_set.
		note
			testing: "covers/{STI_SET}.make_from_set"
			testing: "covers/{STI_TRANSFORMABLE_SET}.make_from_set"
		do
			Precursor {SET_TESTS}
			assert ("convert_from_sti_sa", attached set_from_sti_sa (some_set_a))
			assert ("convert_from_sti_saa", attached set_from_sti_saa (new_transformable_set_aa))
			assert ("convert_from_sab", attached set_from_sab (some_transformable_set_ab))
		end

feature -- Test routines (Primitive)

	test_eq_b
			-- Test {STI_TRANSFORMABLE_SET}.eq_b.
		note
			testing: "covers/{STI_TRANSFORMABLE_SET}.eq_b"
		do
			assert ("eq_b", attached {like set_to_be_tested}.eq_b)
		end

feature -- Test routines (Comparison)

	test_equals
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_SET}.equals.
		note
			testing: "covers/{STI_SET}.equals"
			testing: "covers/{STI_TRANSFORMABLE_SET}.equals"
		local
			s1: like set_to_be_tested
			s2: like some_transformable_set_ab
			s3: like some_set_b
		do
			Precursor {SET_TESTS}
			s1 := set_to_be_tested
			s2 := s1
			s3 := some_set_b
			assert ("same_reference", s1 ≍ s2)
			assert ("same_reference_ok", properties.equals_ok (s1, s2, s3))

			s2 := same_transformable_set_ab (s1)
			s3 := some_set_b
			assert ("same_set", s1 ≍ s2)
			assert ("same_set_ok", properties.equals_ok (s1, s2, s3))

			s2 := some_transformable_set_ab
			s3 := some_set_b
			assert ("some_set", s1 ≍ s2 implies True)
			assert ("some_set_ok", properties.equals_ok (s1, s2, s3))
		end

	test_is_subset
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_SET}.is_subset.
		note
			testing: "covers/{STI_SET}.is_subset"
			testing: "covers/{STI_TRANSFORMABLE_SET}.is_subset"
		local
			s1, s2: like set_to_be_tested
			s3: like some_transformable_set_ba
			s4: like some_set_b
			chk_ops: BOOLEAN
		do
			Precursor {SET_TESTS}
			s2 := set_to_be_tested
			s4 := some_set_b

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s1 := s2 ∖ some_set_a
				s3 := some_transformable_set_ba ∩ s4
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("s1_is_subset_s2", s1 ⊆ s2)
			assert ("s3_is_subset_s4", s3 ⊆ s4)
			assert ("s1_3_are_subsets_s2_4_ok", properties.is_subset_ok (s1, s2, s3, s4))

			s1 := set_to_be_tested
			s2 := set_to_be_tested
			s3 := some_transformable_set_ba
			s4 := some_set_b
			assert ("maybe_s1_is_subset_s2", s1 ⊆ s2 implies True)
			assert ("maybe_s3_is_subset_s4", s3 ⊆ s4 implies True)
			assert ("is_subset_ok", properties.is_subset_ok (s1, s2, s3, s4))
		end

	test_is_disjoint
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_SET}.is_disjoint.
		note
			testing: "covers/{STI_SET}.is_disjoint"
			testing: "covers/{STI_TRANSFORMABLE_SET}.is_disjoint"
		local
			s1: like set_to_be_tested
			s2: like some_transformable_set_ab
			s3: like some_set_b
			chk_ops: BOOLEAN
		do
			Precursor {SET_TESTS}
			s1 := set_to_be_tested

			chk_ops := {ISE_RUNTIME}.check_assert (False)
			s2 := set_to_be_tested ∖ s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			s3 := some_set_b
			assert ("s1_is_disjoint_s2", s1.is_disjoint (s2))
			assert ("s1_is_disjoint_s2_ok", properties.is_disjoint_ok (s1, s2, s3))

			s1 := set_to_be_tested
			s2 := some_transformable_set_ab
			s3 := some_set_b
			assert ("is_disjoint", s1.is_disjoint (s2) implies True)
			assert ("is_disjoint_ok", properties.is_disjoint_ok (s1, s2, s3))
		end

feature -- Test routines (Operation)

	test_united
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_SET}.united.
		note
			testing: "covers/{STI_SET}.united"
			testing: "covers/{STI_TRANSFORMABLE_SET}.united"
		local
			s1: like set_to_be_tested
			s2: like some_transformable_set_ab
			s3: like some_transformable_set_ba
		do
			Precursor {SET_TESTS}
			s1 := set_to_be_tested
			s2 := some_transformable_set_ab
			s3 := some_transformable_set_ba
			assert ("united", attached (s1 ∪ s2))
			assert ("united_ok", properties.united_ok (s1, s2, s3))
		end

	test_intersected
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_SET}.intersected.
		note
			testing: "covers/{STI_SET}.intersected"
			testing: "covers/{STI_TRANSFORMABLE_SET}.intersected"
		local
			s1: like set_to_be_tested
			s2: like some_transformable_set_ab
			s3: like some_transformable_set_ba
			s4: like some_set_b
		do
			Precursor {SET_TESTS}
			s1 := set_to_be_tested
			s2 := some_transformable_set_ab
			s3 := some_transformable_set_ba
			s4 := some_set_b
			assert ("intersected_1", attached (s1 ∩ s2))
			assert ("intersected_2", attached (s3 ∩ s4))
			assert ("intersected_ok", properties.intersected_ok (s1, s2, s3, s4))
		end

	test_subtracted
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_SET}.subtracted.
		note
			testing: "covers/{STI_SET}.subtracted"
			testing: "covers/{STI_TRANSFORMABLE_SET}.subtracted"
		local
			s1: like set_to_be_tested
			s2: like some_transformable_set_ab
			s3: like some_transformable_set_ba
		do
			Precursor {SET_TESTS}
			s1 := set_to_be_tested
			s2 := some_transformable_set_ab
			s3 := some_transformable_set_ba
			assert ("subtracted", attached (s1 ∖ s2))
			assert ("subtracted_ok", properties.subtracted_ok (s1, s2, s3))
		end

	test_subtracted_symmetricaly
			-- <Precursor>
			-- Test {STI_TRANSFORMABLE_SET}.subtracted_symmetricaly.
		note
			testing: "covers/{STI_SET}.subtracted_symmetricaly"
			testing: "covers/{STI_TRANSFORMABLE_SET}.subtracted_symmetricaly"
		local
			s1: like set_to_be_tested
			s2: like some_transformable_set_ab
			s3: like some_transformable_set_ba
		do
			Precursor {SET_TESTS}
			s1 := set_to_be_tested
			s2 := some_transformable_set_ab
			s3 := some_transformable_set_ba
			assert ("subtracted_symmetricaly", attached (s1 ⊖ s2))
			assert ("subtracted_symmetricaly_ok", properties.subtracted_symmetricaly_ok (s1, s2, s3))
		end

	test_crossed
			-- Test {STS_TRANSFORMABLE_SET}.crossed.
			-- Test {STI_TRANSFORMABLE_SET}.crossed.
		note
			testing: "covers/{STS_TRANSFORMABLE_SET}.crossed"
			testing: "covers/{STI_TRANSFORMABLE_SET}.crossed"
		local
			s1: like set_to_be_tested
			s2: like some_set_b
		do
			s1 := set_to_be_tested
			s2 := some_set_b
			assert ("crossed", attached (s1 × s2))
			assert ("crossed_ok", properties.crossed_ok (s1, s2))
		end

feature -- Test routines (Transformation)

	test_mapped
			-- Test {STS_TRANSFORMABLE_SET}.mapped.
			-- Test {STI_TRANSFORMABLE_SET}.mapped.
		note
			testing: "covers/{STS_TRANSFORMABLE_SET}.mapped"
			testing: "covers/{STI_TRANSFORMABLE_SET}.mapped"
		local
			s: like set_to_be_tested
			f: FUNCTION [A, B]
		do
			s := set_to_be_tested
			f := agent f_x
			assert ("mapped", attached (s ↦ f))
			assert ("mapped_ok", properties.mapped_ok (s, f))
		end

	test_reduced
			-- Test {STS_TRANSFORMABLE_SET}.reduced.
			-- Test {STI_TRANSFORMABLE_SET}.reduced.
		note
			testing: "covers/{STS_TRANSFORMABLE_SET}.reduced"
			testing: "covers/{STI_TRANSFORMABLE_SET}.reduced"
		local
			s: like set_to_be_tested
			start: like some_object_b
			f: FUNCTION [B, A, B]
		do
			s := set_to_be_tested
			start := some_object_b
			f := agent f_acc_x
			assert ("reduced", attached s.reduced (start, f) implies True)
			assert ("reduced_ok", properties.reduced_ok (s, start, f))
		end

feature {NONE} -- Factory (test element)

	set_to_be_tested: like some_direct_transformable_set_ab
			-- Transformable set meant to be under tests
		do
			Result := some_direct_transformable_set_ab
		end

feature {NONE} -- Conversion

	set_from_sa (sa: STS_SET [A, EQ_A]): like set_to_be_tested
			-- <Precursor>
		do
			Result := sa
		end

	set_from_sti_sa (sa: STI_SET [A, EQ_A]): like set_to_be_tested
			-- `sa' converted to a set like `set_to_be_tested'
		do
			Result := sa
		ensure
			same_set: Result ≍ sa
		end

	set_from_saa (saa: STS_TRANSFORMABLE_SET [A, A, EQ_A, EQ_A]): like set_to_be_tested
			-- <Precursor>
		do
			Result := saa
		end

	set_from_sti_saa (saa: STI_TRANSFORMABLE_SET [A, A, EQ_A, EQ_A]): like set_to_be_tested
			-- `saa' converted to a set like `set_to_be_tested'
		do
			Result := saa
		ensure
			same_set: Result ≍ saa
		end

	set_from_sab (sab: STS_TRANSFORMABLE_SET [A, B, EQ_A, EQ_B]): like set_to_be_tested
			-- `sab' converted to a set like `set_to_be_tested'
		do
			Result := sab
		ensure
			same_set: Result ≍ sab
		end

note
	copyright: "Copyright (c) 2012-2021, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
