﻿note
	description: "Test suite for {STI_INTEGER}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	INTEGER_NUMBER_TESTS

inherit
	ELEMENT_TESTS
		rename
			test_is_in as test_element_is_in,
			test_is_not_in as test_element_is_not_in,
			element_to_be_tested as integer_to_be_tested,
			some_integer_ref as some_native_integer_ref,
			some_direct_integer_ref as some_direct_native_integer_ref,
			new_integer_ref as new_native_integer_ref,
			some_integer as some_native_integer
		undefine
			on_prepare,
			some_native_integer,
			some_element
		redefine
			test_element_is_in,
			test_element_is_not_in,
			integer_to_be_tested
		end

	NUMBERS_TESTS_RESOURCES_I
		redefine
			on_prepare
		end

feature -- Access

	properties: INTEGER_NUMBER_PROPERTIES
			-- Object that checks the set-theory properties of {STI_INTEGER_NUMBER}
--		once
--			create Result
--		ensure then
--			class
--		end

feature {NONE} -- Events

	on_prepare
			-- <Precursor>
		do
			Precursor {NUMBERS_TESTS_RESOURCES_I}
			create properties
		end

feature -- Test routines (Initialization)

	test_default_create
			-- Test {STI_INTEGER_NUMBER}.default_create.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.default_create"
		do
			assert (
				"default_create",
				attached (create {like integer_to_be_tested})
				)
			assert ("zero", (create {like integer_to_be_tested}) ≍ Zero)
			assert (
				"default_create_ok",
				properties.default_create_ok (some_integer)
				)
		end

	test_make
			-- Test {STI_INTEGER_NUMBER}.make.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.make"
		do
			assert (
				"make",
				attached (
					create {like integer_to_be_tested}.make (next_random_item)
					)
				)
			assert (
				"integer_from_native",
				attached integer_from_native (next_random_item)
				)
		end

	test_make_from_stored_value
			-- Test {STI_INTEGER_NUMBER}.make_from_stored_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.make_from_stored_value"
		local
			sv: like integer_to_be_tested.stored_value
		do
			sv := {like integer_to_be_tested}.adjusted_value (next_random_item).as_integer_8
			assert ("big_enough", {like exposed_integer_to_be_tested}.Native_min_value ≤ sv) -- Assignment above.
			assert ("small_enough", sv ≤ {like exposed_integer_to_be_tested}.Native_max_value)
			assert ("make_from_stored_value", attached (create {like exposed_integer_to_be_tested}.make_from_stored_value (sv)))
		end

	test_make_from_native_integer_ref
			-- Test {STI_INTEGER_NUMBER}.make_from_native_integer_ref.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.make_from_native_integer_ref"
		local
			i_ref: like some_native_integer_ref
		do
			from
				i_ref := some_native_integer_ref
			until
				attached i_ref
			loop
				i_ref := some_native_integer_ref
			end
			assert (
				"make_from_native_integer_ref",
				attached (
					create {like integer_to_be_tested}.
						make_from_native_integer_ref (i_ref)
					)
				)
			assert (
				"integer_from_native_integer_ref",
				attached integer_from_native_integer_ref (i_ref)
				)
		end

	test_make_from_integer
			-- Test {STI_INTEGER_NUMBER}.make_from_integer.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.make_from_integer"
		do
			assert (
				"make_from_integer",
				attached (create {like integer_to_be_tested}.make_from_integer
					(some_integer))
				)
			assert ("integer_from_i", attached integer_from_i (some_integer))
		end

	test_make_from_native_natural
			-- Test {STI_INTEGER_NUMBER}.make_from_native_natural.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.make_from_native_natural"
		do
			assert ("make_from_native_natural", attached (create {like integer_to_be_tested}.make_from_native_natural (some_native_natural)))
			assert ("integer_from_native_natural", attached integer_from_native_natural (some_native_natural))
		end

	test_make_from_native_natural_ref
			-- Test {STI_INTEGER_NUMBER}.make_from_native_natural_ref.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.make_from_native_natural_ref"
		do
			assert ("make_from_native_natural_ref", attached (create {like integer_to_be_tested}.make_from_native_natural_ref (some_native_natural_ref)))
			assert ("integer_from_native_natural_ref", attached integer_from_native_natural_ref (some_native_natural_ref))
		end

feature -- Test routines (Primitive)

	test_real_value
			-- Test {STI_INTEGER_NUMBER}.real_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_value"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("real_value", attached i.real_value)
				assert ("native_real_from_integer", attached native_real_from_integer (i))
				assert ("native_real_ref_from_integer", attached native_real_ref_from_integer (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take ℤ instead.
			end
		end

	test_value
			-- Test {STI_INTEGER_NUMBER}.value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.value"
		local
			js: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				js := Z -- TODO: Take ℤ instead.
			until
				js.is_empty
			loop
				i := js.any
				assert ("value", attached i.value)
				assert ("native_from_integer", attached native_from_integer (i))
				assert ("native_ref_from_integer", attached native_ref_from_integer (i))
				assert ("numeric_from_integer", attached numeric_from_integer (i))
				assert ("comparable_from_integer", attached comparable_from_integer (i))
				assert ("hashable_from_integer", attached hashable_from_integer (i))
				js := js.others
			variant
				cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take ℤ instead.
			end
		end

feature -- Test routines (Membership)

	test_element_is_in
			-- <Precursor>
			-- Test {STS_COMPLEX_NUMBER}.element_is_in.
			-- Test {STS_REAL_NUMBER}.element_is_in.
			-- Test {STS_INTEGER_NUMBER}.element_is_in.
			-- Test {STI_INTEGER_NUMBER}.element_is_in.
		note
			testing: "covers/{STS_ELEMENT}.is_in"
			testing: "covers/{STS_COMPLEX_NUMBER}.element_is_in"
			testing: "covers/{STS_REAL_NUMBER}.element_is_in"
			testing: "covers/{STS_INTEGER_NUMBER}.element_is_in"
			testing: "covers/{STI_INTEGER_NUMBER}.element_is_in"
		do
			Precursor {ELEMENT_TESTS}
		end

	test_element_is_not_in
			-- <Precursor>
			-- Test {STS_COMPLEX_NUMBER}.element_is_not_in.
			-- Test {STS_REAL_NUMBER}.element_is_not_in.
			-- Test {STS_INTEGER_NUMBER}.element_is_not_in.
			-- Test {STI_INTEGER_NUMBER}.element_is_not_in.
		note
			testing: "covers/{STS_ELEMENT}.is_not_in"
			testing: "covers/{STS_COMPLEX_NUMBER}.element_is_not_in"
			testing: "covers/{STS_REAL_NUMBER}.element_is_not_in"
			testing: "covers/{STS_INTEGER_NUMBER}.element_is_not_in"
			testing: "covers/{STI_INTEGER_NUMBER}.element_is_not_in"
		do
			Precursor {ELEMENT_TESTS}
		end

	test_complex_is_in
			-- Test {STS_COMPLEX_NUMBER}.is_in.
			-- Test {STS_REAL_NUMBER}.complex_is_in.
			-- Test {STS_INTEGER_NUMBER}.complex_is_in.
			-- Test {STI_INTEGER_NUMBER}.complex_is_in.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_in"
			testing: "covers/{STS_REAL_NUMBER}.complex_is_in"
			testing: "covers/{STS_INTEGER_NUMBER}.complex_is_in"
			testing: "covers/{STI_INTEGER_NUMBER}.complex_is_in"
		local
			i: like integer_to_be_tested
			s: like some_complexes
		do
			i := integer_to_be_tested
			s := some_complex_references & i
			assert ("same_reference", i.complex_is_in (s))
			assert ("same_reference_ok", properties.complex_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_complex_objects & i.twin
			assert ("same_value", i.complex_is_in (s))
			assert ("same_value_ok", properties.complex_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_complex_numbers & same_complex (i)
			assert ("same_math_value", i.complex_is_in (s))
			assert ("same_math_value_ok", properties.complex_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_complexes
			assert ("some_complexes", i.complex_is_in (s) implies True)
			assert ("some_complexes_ok", properties.complex_is_in_ok (i, s))
		end

	test_complex_is_not_in
			-- Test {STS_COMPLEX_NUMBER}.is_not_in.
			-- Test {STS_REAL_NUMBER}.complex_is_not_in.
			-- Test {STS_INTEGER_NUMBER}.complex_is_not_in.
			-- Test {STI_INTEGER_NUMBER}.complex_is_not_in.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_not_in"
			testing: "covers/{STS_REAL_NUMBER}.complex_is_not_in"
			testing: "covers/{STS_INTEGER_NUMBER}.complex_is_not_in"
			testing: "covers/{STI_INTEGER_NUMBER}.complex_is_not_in"
		local
			i: like integer_to_be_tested
			j: like some_complex
			s: like some_complexes
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			i := integer_to_be_tested
			s := some_complex_references / i
			assert ("same_reference", i.complex_is_not_in (s))
			assert ("same_reference ok", properties.complex_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_complex_objects / i.twin
			assert ("same_value", i.complex_is_not_in (s))
			assert ("same_value", properties.complex_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_complex_numbers
			j := same_complex (i)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := s / j
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_number", i.complex_is_not_in (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
			assert ("same_math_value", properties.complex_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_complexes
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("some_complexes", i.complex_is_not_in (s) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
			assert ("some_complexes_ok", properties.complex_is_not_in_ok (i, s))
		end

	test_real_is_in
			-- Test {STS_REAL_NUMBER}.is_in.
			-- Test {STS_INTEGER_NUMBER}.real_is_in.
			-- Test {STI_INTEGER_NUMBER}.real_is_in.
			--| TODO: Errors due to CATcalls.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_in"
			testing: "covers/{STS_INTEGER_NUMBER}.real_is_in"
			testing: "covers/{STI_INTEGER_NUMBER}.real_is_in"
		local
			i: like integer_to_be_tested
			s: like some_reals
		do
			i := integer_to_be_tested
			s := some_real_references & i
			assert ("same_reference", i.real_is_in (s))
			assert ("same_reference ok", properties.real_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_real_objects & i.twin
			assert ("same_object_value", i.real_is_in (s))
			assert ("same_object_value ok", properties.real_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_real_numbers & same_real (i)
			assert ("same_math_value", i.real_is_in (s))
			assert ("same_math_value ok", properties.real_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_strict_real_numbers & same_real (i)
			assert ("same_strict_math_value", i.real_is_in (s))
			assert ("same_strict_math_value ok", properties.real_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_reals
			assert ("some_reals", i.real_is_in (s) implies True)
			assert ("some_reals ok", properties.real_is_in_ok (i, s))
		end

	test_real_is_not_in
			-- Test {STS_REAL_NUMBER}.is_not_in.
			-- Test {STS_INTEGER_NUMBER}.real_is_not_in.
			-- Test {STI_INTEGER_NUMBER}.real_is_not_in.
			--| TODO: Errors due to CATcalls.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_not_in"
			testing: "covers/{STS_INTEGER_NUMBER}.real_is_not_in"
			testing: "covers/{STI_INTEGER_NUMBER}.real_is_not_in"
		local
			i: like integer_to_be_tested
			s: like some_reals
		do
			i := integer_to_be_tested
			s := some_real_references / i
			assert ("same_reference", i.real_is_not_in (s))
			assert ("same_reference ok", properties.real_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_real_objects / i.twin
			assert ("same_object_value", i.real_is_not_in (s))
			assert ("same_object_value ok", properties.real_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_real_numbers / same_real (i)
			assert ("same_math_value", i.real_is_not_in (s))
			assert ("same_math_value ok", properties.real_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_strict_real_numbers / same_real (i)
			assert ("same_strict_math_value", i.real_is_not_in (s))
			assert ("same_strict_math_value ok", properties.real_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_reals
			assert ("some_reals", i.real_is_not_in (s) implies True)
			assert ("some_reals ok", properties.real_is_not_in_ok (i, s))
		end

	test_rational_is_in
			-- Test {STS_RATIONAL_NUMBER}.is_in.
			-- Test {STS_INTEGER_NUMBER}.rational_is_in.
			-- Test {STI_INTEGER_NUMBER}.rational_is_in.
			--| TODO: Errors due to CATcalls.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_in"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_is_in"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_is_in"
		local
			i: like integer_to_be_tested
			s: like some_rationals
		do
			i := integer_to_be_tested
			s := some_rational_references & i
			assert ("same_reference", i.rational_is_in (s))
			assert ("same_reference ok", properties.rational_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_rational_objects & i.twin
			assert ("same_object_value", i.rational_is_in (s))
			assert ("same_object_value ok", properties.rational_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_rational_numbers & same_rational (i)
			assert ("same_math_value", i.rational_is_in (s))
			assert ("same_math_value ok", properties.rational_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_strict_rational_numbers & same_rational (i)
			assert ("same_strict_math_value", i.rational_is_in (s))
			assert
				("same_strict_math_value ok", properties.rational_is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_rationals
			assert ("some_rationals", i.rational_is_in (s) implies True)
			assert ("some_rationals ok", properties.rational_is_in_ok (i, s))
		end

	test_rational_is_not_in
			-- Test {STS_RATIONAL_NUMBER}.is_not_in.
			-- Test {STS_INTEGER_NUMBER}.rational_is_not_in.
			-- Test {STI_INTEGER_NUMBER}.rational_is_not_in.
			--| TODO: Errors due to CATcalls.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_not_in"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_is_not_in"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_is_not_in"
		local
			i: like integer_to_be_tested
			s: like some_rationals
		do
			i := integer_to_be_tested
			s := some_rational_references / i
			assert ("same_reference", i.rational_is_not_in (s))
			assert ("same_reference ok", properties.rational_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_rational_objects / i.twin
			assert ("same_object_value", i.rational_is_not_in (s))
			assert
				("same_object_value ok", properties.rational_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_rational_numbers / same_rational (i)
			assert ("same_math_value", i.rational_is_not_in (s))
			assert ("same_math_value ok", properties.rational_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_strict_rational_numbers / same_rational (i)
			assert ("same_strict_math_value", i.rational_is_not_in (s))
			assert ("same_strict_math_value ok", properties.rational_is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_rationals
			assert ("some_rationals", i.rational_is_not_in (s) implies True)
			assert ("some_rationals ok", properties.rational_is_not_in_ok (i, s))
		end

	test_is_in
			-- Test {STS_INTEGER_NUMBER}.is_in.
			-- Test {STI_INTEGER_NUMBER}.is_in.
			--| TODO: Use {INTEGER_SET} etc.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.is_in"
			testing: "covers/{STI_INTEGER_NUMBER}.is_in"
		local
			i: like integer_to_be_tested
			s: like some_integers
		do
			i := integer_to_be_tested
			s := some_integer_references & i
			assert ("same_reference", i ∈ s)
			assert ("same_reference ok", properties.is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_integer_objects & i.twin
			assert ("same_object_value", i ∈ s)
			assert ("same_object_value ok", properties.is_in_ok (i, s))

--			i := integer_to_be_tested
--			s := some_integer_numbers & same_integer (i)
--			assert ("same_math_value", i ∈ s)
--			assert ("same_math_value ok", properties.is_in_ok (i, s))

--			i := integer_to_be_tested
--			s := some_strict_integer_numbers & same_integer (i)
--			assert ("same_strict_math_value", i ∈ s)
--			assert ("same_strict_math_value ok", properties.is_in_ok (i, s))

			i := integer_to_be_tested
			s := some_integers
			assert ("some_integers", i ∈ s implies True)
			assert ("some_integers ok", properties.is_in_ok (i, s))
		end

	test_is_not_in
			-- Test {STS_INTEGER_NUMBER}.is_not_in.
			-- Test {STI_INTEGER_NUMBER}.is_not_in.
			--| TODO: Use {INTEGER_SET} etc.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.is_not_in"
			testing: "covers/{STI_INTEGER_NUMBER}.is_not_in"
		local
			i: like integer_to_be_tested
			s: like some_integers
		do
			i := integer_to_be_tested
			s := some_integer_references / i
			assert ("same_reference", i ∉ s)
			assert ("same_reference ok", properties.is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_integer_objects / i.twin
			assert ("same_object_value", i ∉ s)
			assert ("same_object_value ok", properties.is_not_in_ok (i, s))

--			i := integer_to_be_tested
--			s := some_integer_numbers / same_integer (i)
--			assert ("same_math_value", i ∉ s)
--			assert ("same_math_value ok", properties.is_not_in_ok (i, s))

--			i := integer_to_be_tested
--			s := some_strict_integer_numbers / same_integer (i)
--			assert ("same_strict_math_value", i ∉ s)
--			assert ("same_strict_math_value ok", properties.is_not_in_ok (i, s))

			i := integer_to_be_tested
			s := some_integers
			assert ("some_integers", i ∉ s implies True)
			assert ("some_integers ok", properties.is_not_in_ok (i, s))
		end

feature -- Test routines (Access)

	test_sign_bit
			-- Test {STI_INTEGER_NUMBER}.sign_bit.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.sign_bit"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("i.sign_bit", attached i.sign_bit)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			i := - integer_to_be_tested.abs ∧ - One
			assert ("negative", i.sign_bit = One)

			i := Zero
			assert ("no negative zero", (- i).sign_bit = Zero)
			assert ("zero", i.sign_bit = Zero)

			i := integer_to_be_tested.abs ∨ One
			assert ("positive", i.sign_bit = Zero)
		end

	test_sign
			-- Test {STI_INTEGER_NUMBER}.sign.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.sign"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("i.sign", attached i.sign)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			i := - integer_to_be_tested.abs ∧ - One
			assert ("negative", i.sign = - One)

			i := Zero
			assert ("no negative zero", (- i).sign = Zero)
			assert ("zero", i.sign = Zero)

			i := integer_to_be_tested.abs ∨ One
			assert ("positive", i.sign = One)
		end

	test_zero
			-- Test {STI_INTEGER_NUMBER}.zero.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.zero"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("zero", attached i.Zero)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("Min_value complex_zero_ok", properties.complex_zero_ok ({like integer_to_be_tested}.Min_value, some_complex))
			assert ("Min_value real_zero_ok", properties.real_zero_ok ({like integer_to_be_tested}.Min_value, some_real))
			assert ("Min_value", properties.zero_ok ({like integer_to_be_tested}.Min_value, some_integer))

			assert ("Zero complex_zero_ok", properties.complex_zero_ok ({like integer_to_be_tested}.Zero, some_complex))
			assert ("Zero real_zero_ok", properties.real_zero_ok ({like integer_to_be_tested}.Zero, some_real))
			assert ("Zero", properties.zero_ok ({like integer_to_be_tested}.Zero, some_integer))

			assert ("Max_value complex_zero_ok", properties.complex_zero_ok ({like integer_to_be_tested}.Max_value, some_complex))
			assert ("Max_value real_zero_ok", properties.real_zero_ok ({like integer_to_be_tested}.Max_value, some_real))
			assert ("Max_value", properties.zero_ok ({like integer_to_be_tested}.Max_value, some_integer))

			i := integer_to_be_tested
			assert ("complex_zero_ok", properties.complex_zero_ok (i, some_complex))
			assert ("real_zero_ok", properties.complex_zero_ok (i, some_real))
			assert ("zero_ok", properties.zero_ok (i, some_integer))
		end

	test_one
			-- Test {STI_INTEGER_NUMBER}.one.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.one"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("one", attached i.One)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("Min_value complex_one_ok", properties.complex_one_ok ({like integer_to_be_tested}.Min_value, some_complex))
			assert ("Min_value real_one_ok", properties.real_one_ok ({like integer_to_be_tested}.Min_value, some_real))
			assert ("Min_value", properties.one_ok ({like integer_to_be_tested}.Min_value, some_integer))

			assert ("Zero complex_one_ok", properties.complex_one_ok ({like integer_to_be_tested}.Zero, some_complex))
			assert ("Zero real_one_ok", properties.real_one_ok ({like integer_to_be_tested}.Zero, some_real))
			assert ("Zero", properties.one_ok ({like integer_to_be_tested}.Zero, some_integer))

			assert ("Max_value complex_one_ok", properties.complex_one_ok ({like integer_to_be_tested}.Max_value, some_complex))
			assert ("Max_value real_one_ok", properties.real_one_ok ({like integer_to_be_tested}.Max_value, some_real))
			assert ("Max_value", properties.one_ok ({like integer_to_be_tested}.Max_value, some_integer))

			i := integer_to_be_tested
			assert ("complex_one_ok", properties.complex_one_ok (i, some_complex))
			assert ("real_one_ok", properties.complex_one_ok (i, some_real))
			assert ("one_ok", properties.one_ok (i, some_integer))
		end

	test_two
			-- Test {STI_INTEGER_NUMBER}.two.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.two"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("two", attached i.Two)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("Min_value", properties.two_ok ({like integer_to_be_tested}.Min_value))
			assert ("Zero", properties.two_ok ({like integer_to_be_tested}.Zero))
			assert ("Max_value", properties.two_ok ({like integer_to_be_tested}.Max_value))
			assert ("two_ok", properties.two_ok (integer_to_be_tested))
		end

	test_imaginary_unit
			-- Test {STI_INTEGER_NUMBER}.imaginary_unit.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.imaginary_unit"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("imaginary_unit", attached i.imaginary_unit)
				assert ("imaginary_unit_ok", properties.imaginary_unit_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_min_value
			-- Test {STI_INTEGER_NUMBER}.min_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.min_value"
		do
			assert ("min_value", attached integer_to_be_tested.Min_value)
		end

	test_max_value
			-- Test {STI_INTEGER_NUMBER}.max_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.max_value"
		do
			assert ("max_value", attached integer_to_be_tested.Max_value)
		end

	test_machine_epsilon
			-- Test {STI_INTEGER_NUMBER}.machine_epsilon.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.machine_epsilon" -- todo: AutoTest gests confused by covers/{STI_INTEGER_NUMBER}.Machine_epsilon
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("Machine_epsilon", attached i.Machine_epsilon)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			i := integer_to_be_tested
			assert (
				"srnp_machine_epsilon_ok",
				properties.srnp_machine_epsilon_ok (i)
				)
		end

	test_ulp
			-- Test {STI_INTEGER_NUMBER}.ulp.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.ulp"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("ulp", attached i.ulp)
				print (i); print ("; "); print (i.ulp); print ('%N')
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("ulp_ok", properties.ulp_ok (integer_to_be_tested))
		end

	test_relative_epsilon
			-- Test {STI_INTEGER_NUMBER}.relative_epsilon.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.relative_epsilon"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("relative_epsilon", attached i.relative_epsilon)
				print (i_s.any); print ("; "); print (i.relative_epsilon); print ('%N')
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("relative_epsilon_ok", properties.relative_epsilon_ok (integer_to_be_tested))
		end

	test_previous_float
			-- Test {STI_INTEGER_NUMBER}.previous_float.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.previous_float"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("previous_float", attached i.previous_float)
				print (i); print ("; "); print (i.previous_float); print ('%N')
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_next_float
			-- Test {STI_INTEGER_NUMBER}.next_float.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.next_float"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("next_float", attached i.next_float)
				print (i); print ("; "); print (i.next_float); print ('%N')
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

feature -- Test routines (Quality)

	test_is_real
			-- Test {STI_INTEGER_NUMBER}.is_real.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_real"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_real", i.is_real)
				assert ("is_real_ok", properties.is_real_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_proper_real
			-- Test {STI_INTEGER_NUMBER}.is_proper_real.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_proper_real"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			i := Zero
			assert ("not i.is_proper_real", not i.is_proper_real)
			assert ("not i.is_proper_real ok", properties.is_proper_real_ok (i))

			from
				i := integer_to_be_tested
			until
				i ≭ Zero
			loop
				i := integer_to_be_tested
			end
			assert ("i.is_proper_real", i.is_proper_real)
			assert ("i.is_proper_real ok", properties.is_proper_real_ok (i))

			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_proper_real", i.is_proper_real implies True)
				assert ("is_proper_real ok", properties.is_proper_real_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_imaginary
			-- Test {STI_INTEGER_NUMBER}.is_imaginary.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_imaginary"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			i := Zero
			assert ("i.is_imaginary", i.is_imaginary)
			assert ("i.is_imaginary ok", properties.is_imaginary_ok (i))

			from
				i := integer_to_be_tested
			until
				i ≭ Zero
			loop
				i := integer_to_be_tested
			end
			assert ("not i.is_imaginary", not i.is_imaginary)
			assert ("not i.is_imaginary ok", properties.is_imaginary_ok (i))

			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_imaginary", i.is_imaginary implies True)
				assert ("is_imaginary ok", properties.is_imaginary_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_proper_imaginary
			-- Test {STI_INTEGER_NUMBER}.is_proper_imaginary.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_proper_imaginary"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_proper_imaginary", not i.is_proper_imaginary)
				assert (
					"is_proper_imaginary_ok",
					properties.is_proper_imaginary_ok (i)
					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_proper_complex
			-- Test {STI_INTEGER_NUMBER}.is_proper_complex.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_proper_complex"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_proper_complex", not i.is_proper_complex)
				assert
					("is_proper_complex_ok", properties.is_proper_complex_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_nan
			-- Test {STI_INTEGER_NUMBER}.is_nan.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_nan"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_nan", not i.is_nan)
				assert
					("real_is_nan_ok", properties.real_is_nan_ok (i, some_real))
				assert ("is_nan_ok", properties.is_nan_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_negative_infinity
			-- Test {STI_INTEGER_NUMBER}.is_negative_infinity.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_negative_infinity"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_negative_infinity", not i.is_negative_infinity)
				assert (
					"real_is_negative_infinity_ok",
					properties.real_is_negative_infinity_ok (i, some_real)
					)
				assert (
					"is_negative_infinity_ok",
					properties.is_negative_infinity_ok (i)
					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_negative_zero
			-- Test {STI_INTEGER_NUMBER}.is_negative_zero.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_negative_zero"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_negative_zero", not i.is_negative_zero)
				assert (
					"is_negative_zero_ok",
					properties.is_negative_zero_ok (i, some_real)
					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_positive_zero
			-- Test {STI_INTEGER_NUMBER}.is_positive_zero.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_positive_zero"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			i := Zero
			assert ("i.is_positive_zero", i.is_positive_zero)
			assert (
				"i.is_positive_zero ok",
				properties.is_positive_zero_ok (i, some_integer)
				)

			from
				i := integer_to_be_tested
			until
				i ≭ Zero
			loop
				i := integer_to_be_tested
			end
			assert ("not i.is_positive_zero", not i.is_positive_zero)
			assert (
				"not i.is_positive_zero ok",
				properties.is_positive_zero_ok (i, some_real)
				)

			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_positive_zero", i.is_positive_zero implies True)
				assert (
					"is_positive_zero_ok",
					properties.is_positive_zero_ok (i, some_real)
					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_positive_infinity
			-- Test {STI_INTEGER_NUMBER}.is_positive_infinity.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_positive_infinity"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_positive_infinity", not i.is_positive_infinity)
				assert (
					"real_is_positive_infinity_ok",
					properties.real_is_positive_infinity_ok (i, some_real)
					)
				assert (
					"is_positive_infinity_ok",
					properties.is_positive_infinity_ok (i)
					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_infinite
			-- Test {STI_INTEGER_NUMBER}.is_infinite.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_infinite"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_infinite", not i.is_infinite)
				assert (
					"complex_is_infinite ok",
					properties.complex_is_infinite_ok (i, some_complex)
					)
				assert (
					"complex_is_infinite ok",
					properties.real_is_infinite_ok (i, some_real)
					)
				assert ("is_infinite", properties.is_infinite_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_finite
			-- Test {STI_INTEGER_NUMBER}.is_finite.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_finite"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_finite", i.is_finite)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_rational
			-- Test {STI_INTEGER_NUMBER}.is_rational.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_rational"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_rational", i.is_rational)
				assert ("is_rational_ok", properties.is_rational_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_integer
			-- Test {STI_INTEGER_NUMBER}.is_integer.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_integer"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("is_integer", i.is_integer)
				assert ("is_integer_ok", properties.is_integer_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_real_is_invertible
			-- Test {STI_INTEGER_NUMBER}.real_is_invertible.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_is_invertible"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z -- TODO: Take ℤ instead.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("real_is_invertible", i.real_is_invertible)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_is_invertible
			-- Test {STI_INTEGER_NUMBER}.is_invertible.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.is_invertible"
		local
			i: like integer_to_be_tested
			js: like Z.subset_anchor
		do
			from
				i := integer_to_be_tested
			until
				i ≭ Zero
			loop
				i := integer_to_be_tested
			end
			assert ("i.is_invertible", i.is_invertible)
			i := Zero
			assert ("not i.is_invertible", not i.is_invertible)

			from
				js := Z  -- TODO: Replace by Z.
			until
				js.is_empty
			loop
				i := js.any
				assert ("is_invertible", i.is_invertible implies True)
				js := js.others
			variant
				cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Replace by like Z.
			end
		end

	test_min_value_exists
			-- Test {STI_INTEGER_NUMBER}.min_value_exists.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.min_value_exists"
		local
			i: like integer_to_be_tested
			js: like Z.subset_anchor
		do
			from
				js := Z  -- TODO: Replace by Z.
			until
				js.is_empty
			loop
				i := js.any
				assert ("min_value_exists", i.min_value_exists)
				js := js.others
			variant
				cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Replace by like Z.
			end
		end

	test_max_value_exists
			-- Test {STI_INTEGER_NUMBER}.max_value_exists.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.max_value_exists"
		local
			i: like integer_to_be_tested
			js: like Z.subset_anchor
		do
			from
				js := Z  -- TODO: Replace by Z.
			until
				js.is_empty
			loop
				i := js.any
				assert ("max_value_exists", i.max_value_exists)
				js := js.others
			variant
				cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Replace by like Z.
			end
		end

feature -- Test routines (Output)

	test_out
			-- Test {STI_INTEGER_NUMBER}.out.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.out"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z  -- TODO: Replace by Z.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("out", attached i.out)
				print (i.out) print ('%N')
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

feature -- Test routines (Comparison)

	test_complex_equals
			-- Test {STS_COMPLEX_NUMBER}.equals.
			-- Test {STS_REAL_NUMBER}.complex_equals.
			-- Test {STS_INTEGER_NUMBER}.complex_equals.
			-- Test {STI_INTEGER_NUMBER}.complex_equals.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.equals"
			testing: "covers/{STS_REAL_NUMBER}.complex_equals"
			testing: "covers/{STS_INTEGER_NUMBER}.complex_equals"
			testing: "covers/{STI_INTEGER_NUMBER}.complex_equals"
		local
			i: like integer_to_be_tested
			ab: like some_complex
		do
			i := integer_to_be_tested
			assert ("same_entity", i.complex_equals (i))
			assert ("same_entity_ok", properties.complex_equals_ok (i, i, i))

			i := integer_to_be_tested
			ab := same_complex (i)
			assert ("same_complex", i.complex_equals (ab))
			assert ("same_complex_ok", properties.complex_equals_ok (i, ab, same_complex (ab)))

			i := integer_to_be_tested
			ab := some_complex
			assert ("complex_equals", i.complex_equals (ab) implies True)
			assert ("complex_equals_ok", properties.complex_equals_ok (i, ab, some_complex))
		end

	test_complex_unequals
			-- Test {STS_COMPLEX_NUMBER}.unequals.
			-- Test {STS_REAL_NUMBER}.complex_unequals.
			-- Test {STS_INTEGER_NUMBER}.complex_unequals.
			-- Test {STI_INTEGER_NUMBER}.complex_unequals.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.unequals"
			testing: "covers/{STS_REAL_NUMBER}.complex_unequals"
			testing: "covers/{STS_INTEGER_NUMBER}.complex_unequals"
			testing: "covers/{STI_INTEGER_NUMBER}.complex_unequals"
		local
			i: like integer_to_be_tested
			ab: like some_complex
		do
			i := integer_to_be_tested
			assert ("same_entity", not i.complex_unequals (i))
			assert ("same_entity_ok", properties.complex_unequals_ok (i, i))

			i := integer_to_be_tested
			ab := same_complex (i)
			assert ("same_complex", not i.complex_unequals (ab))
			assert ("same_complex_ok", properties.complex_unequals_ok (i, ab))

			i := integer_to_be_tested
			ab := some_complex
			assert ("complex_unequals", i.complex_unequals (ab) implies True)
			assert ("complex_unequals_ok", properties.complex_unequals_ok (i, ab))
		end

	test_real_equals
			-- Test {STS_REAL_NUMBER}.equals.
			-- Test {STS_INTEGER_NUMBER}.real_equals.
			-- Test {STI_INTEGER_NUMBER}.real_equals.
		note
			testing: "covers/{STS_REAL_NUMBER}.equals"
			testing: "covers/{STS_INTEGER_NUMBER}.real_equals"
			testing: "covers/{STI_INTEGER_NUMBER}.real_equals"
		local
			i: like integer_to_be_tested
			x: like some_real
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			assert ("same_entity", i.real_equals (i))
			assert ("same_entity_ok", properties.real_equals_ok (i, i, i))

			i := integer_to_be_tested
			x := same_real (i)
			assert ("same_real", i.real_equals (x))
			assert ("same_real_ok", properties.real_equals_ok (i, x, same_real (x)))

			from
				xs := R
			until
				xs.is_empty
			loop
				i := integer_to_be_tested -- TODO: Take every possible integer.
				x := xs.any
				assert ("some_real", i.real_equals (x) implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			assert ("real_equals_ok", properties.real_equals_ok (integer_to_be_tested, some_real, some_real))
		end

	test_real_unequals
			-- Test {STS_REAL_NUMBER}.unequals.
			-- Test {STS_INTEGER_NUMBER}.real_unequals.
			-- Test {STI_INTEGER_NUMBER}.real_unequals.
		note
			testing: "covers/{STS_REAL_NUMBER}.unequals"
			testing: "covers/{STS_INTEGER_NUMBER}.real_unequals"
			testing: "covers/{STI_INTEGER_NUMBER}.real_unequals"
		local
			i: like integer_to_be_tested
			x: like some_real
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			assert ("same_entity", not i.real_unequals (i))
			assert ("same_entity ok", properties.real_unequals_ok (i, i))

			i := integer_to_be_tested
			x := same_real (i)
			assert ("same_real", not (i.real_unequals (x)))
			assert ("same_real ok", properties.real_unequals_ok (i, x))

			from
				xs := R
			until
				xs.is_empty
			loop
				i := integer_to_be_tested -- TODO: Take every possible integer.
				x := xs.any
				assert ("some_real", i.real_unequals (x) implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			assert ("real_unequals_ok", properties.real_unequals_ok (integer_to_be_tested, some_real))
		end

	test_real_is_less
			-- Test {STS_REAL_NUMBER}.is_less.
			-- Test {STS_INTEGER_NUMBER}.real_is_less.
			-- Test {STI_INTEGER_NUMBER}.real_is_less.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_less"
			testing: "covers/{STS_INTEGER_NUMBER}.real_is_less"
			testing: "covers/{STI_INTEGER_NUMBER}.real_is_less"
		local
			i: like integer_to_be_tested
			x, y: like some_real
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			from
				y := some_real
			until
				y > Zero
			loop
				y := some_real
			end
			x := (y ∨ i.ulp) + i
			assert ("i < x", i.real_is_less (x))
			assert ("i < x ok", properties.real_is_less_ok (i, x, some_real))

			i := integer_to_be_tested
			from
				y := some_real
			until
				y ≤ Zero
			loop
				y := some_real
			end
			x := y + i
			assert ("not (i < x)", not i.real_is_less (x))
			assert ("not (i < x) ok", properties.real_is_less_ok (i, x, some_real))

			from
				xs := R
			until
				xs.is_empty
			loop
				i := integer_to_be_tested -- TODO: Take every possible integer.
				x := xs.any
				assert ("real_is_less", i.real_is_less (x) implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			assert ("real_is_less_ok", properties.real_is_less_ok (integer_to_be_tested, some_real, some_real))
		end

	test_real_is_less_equal
			-- Test {STS_REAL_NUMBER}.is_less_equal.
			-- Test {STS_INTEGER_NUMBER}.real_is_less_equal.
			-- Test {STI_INTEGER_NUMBER}.real_is_less_equal.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_less_equal"
			testing: "covers/{STS_INTEGER_NUMBER}.real_is_less_equal"
			testing: "covers/{STI_INTEGER_NUMBER}.real_is_less_equal"
		local
			i: like integer_to_be_tested
			x, y: like some_real
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			from
				y := some_real
			until
				y ≥ Zero
			loop
				y := some_real
			end
			x := y + i
			assert ("i ≤ x", i.real_is_less_equal (x))
			assert ("i ≤ x ok", properties.real_is_less_equal_ok (i, x, some_real))

			i := integer_to_be_tested
			from
				y := some_real
			until
				y < Zero
			loop
				y := some_real
			end
			x := (y ∧ -i.ulp) + i
			assert ("not (i ≤ x)", not i.real_is_less_equal (x))
			assert ("not (i ≤ x) ok", properties.real_is_less_equal_ok (i, x, some_real))

			from
				xs := R
			until
				xs.is_empty
			loop
				i := integer_to_be_tested -- TODO: Take every possible integer.
				x := xs.any
				assert ("real_is_less_equal", i.real_is_less_equal (x) implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			assert ("real_is_less_equal_ok", properties.real_is_less_equal_ok (integer_to_be_tested, some_real, some_real))
		end

	test_real_is_greater
			-- Test {STS_REAL_NUMBER}.is_greater.
			-- Test {STS_INTEGER_NUMBER}.real_is_greater.
			-- Test {STI_INTEGER_NUMBER}.real_is_greater.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_greater"
			testing: "covers/{STS_INTEGER_NUMBER}.real_is_greater"
			testing: "covers/{STI_INTEGER_NUMBER}.real_is_greater"
		local
			i: like integer_to_be_tested
			x, y: like some_real
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			from
				y := some_real
			until
				y < Zero
			loop
				y := some_real
			end
			x := (y ∧ -i.ulp) + i
			assert ("i > x", i.real_is_greater (x))
			assert ("i > x ok", properties.real_is_greater_ok (i, x, some_real))

			i := integer_to_be_tested
			from
				y := some_real
			until
				y ≥ Zero
			loop
				y := some_real
			end
			x := y + i
			assert ("not (i > x)", not i.real_is_greater (x))
			assert ("not (i > x) ok", properties.real_is_greater_ok (i, x, some_real))

			from
				xs := R
			until
				xs.is_empty
			loop
				i := integer_to_be_tested -- TODO: Take every possible integer.
				x := xs.any
				assert ("real_is_greater", i.real_is_greater (x) implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			assert ("real_is_greater_ok", properties.real_is_greater_ok (integer_to_be_tested, some_real, some_real))
		end

	test_real_is_greater_equal
			-- Test {STS_REAL_NUMBER}.is_greater_equal.
			-- Test {STS_INTEGER_NUMBER}.real_is_greater_equal.
			-- Test {STI_INTEGER_NUMBER}.real_is_greater_equal.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_greater_equal"
			testing: "covers/{STS_INTEGER_NUMBER}.real_is_greater_equal"
			testing: "covers/{STI_INTEGER_NUMBER}.real_is_greater_equal"
		local
			i: like integer_to_be_tested
			x, y: like some_real
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			from
				y := some_real
			until
--				y ≤ Zero
				y ≤ Zero
			loop
				y := some_real
			end
			x := y + i
			assert ("i ≥ x", i.real_is_greater_equal (x))
			assert ("i ≥ x ok", properties.real_is_greater_equal_ok (i, x, some_real))

			i := integer_to_be_tested
			from
				y := some_real
			until
				y > Zero
			loop
				y := some_real
			end
			x := (y ∨ i.ulp) + i
			assert ("not (i ≥ x)", not i.real_is_greater_equal (x))
			assert ("not (i ≥ x) ok", properties.real_is_greater_equal_ok (i, x, some_real))

			from
				xs := R
			until
				xs.is_empty
			loop
				i := integer_to_be_tested -- TODO: Take every possible integer.
				x := xs.any
				assert ("real_is_greater_equal", i.real_is_greater_equal (x) implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			assert ("real_is_greater_equal_ok", properties.real_is_greater_equal_ok (integer_to_be_tested, some_real, some_real))
		end

	test_real_three_way_comparison
			-- Test {STI_INTEGER_NUMBER}.real_three_way_comparison.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_three_way_comparison"
		local
			i: like integer_to_be_tested
			x, y, w: like some_real
			i_s: like Z.subset_anchor
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			w := some_real.abs ∨ i.ulp
			y := w + i
			assert ("negative", i.real_three_way_comparison (y) ≍ - One)

			assert ("same_entity", i.real_three_way_comparison (i) ≍ Zero)
			assert ("same_real", i.real_three_way_comparison (same_real (i)) ≍ Zero)

			i := integer_to_be_tested
			w := (- some_real.abs) ∧ (- i.ulp)
			y := w + i
			assert ("positive", i.real_three_way_comparison (y) ≍ One)

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert ("real_three_way_comparison", attached i.real_three_way_comparison (x))
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_real_min
			-- Test {STI_INTEGER_NUMBER}.real_min.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_min"
		local
			i: like integer_to_be_tested
			x, y, w: like some_real
			i_s: like Z.subset_anchor
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			w := some_real.abs ∨ Zero
			y := w + i
			assert ("i", i.real_min (y) ≍ i)
			assert ("i ok", properties.real_min_ok (i, y, some_real))

			i := integer_to_be_tested
			w := (- some_real.abs) ∧ (- i.ulp)
			y := w + i
			assert ("y", i.real_min (y) ≍ y)
			assert ("y ok", properties.real_min_ok (i, y, some_real))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert ("real_min", attached i.real_min (x))
					assert
						("real_min_ok", properties.real_min_ok (i, x, some_real))
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_real_max
			-- Test {STI_INTEGER_NUMBER}.real_max.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_max"
		local
			i: like integer_to_be_tested
			x, y, w: like some_real
			i_s: like Z.subset_anchor
			xs: like R.subset_anchor
		do
			i := integer_to_be_tested
			w := - some_real.abs
			y := w + i
			assert ("i", i.real_max (y) ≍ i)
			assert ("i ok", properties.real_max_ok (i, y, some_real))

			i := integer_to_be_tested
			w := some_real.abs ∨ i.ulp
			y := w + i
			assert ("y", i.real_max (y) ≍ y)
			assert ("y ok", properties.real_max_ok (i, y, some_real))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert ("real_max", attached i.real_max (x))
					assert
						("real_max_ok", properties.real_max_ok (i, x, some_real))
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_rational_equals
			-- Test {STS_RATIONAL_NUMBER}.equals.
			-- Test {STS_INTEGER_NUMBER}.rational_equals.
			-- Test {STI_INTEGER_NUMBER}.rational_equals.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.equals"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_equals"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_equals"
		local
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
			i: like integer_to_be_tested
			pq: like some_rational
		do
			i := integer_to_be_tested
			assert ("same_entity", i.rational_equals (i))
			assert ("same_entity_ok", properties.rational_equals_ok (i, i, i))

			i := integer_to_be_tested
			pq := same_rational (i)
			assert ("same_rational", i.rational_equals (pq))
			assert ("same_rational_ok", properties.rational_equals_ok (i, pq, same_rational (pq)))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := i_s.any
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("some_rational", i.rational_equals (pq) implies True)
					assert ("rational_equals_ok", properties.rational_equals_ok (i, pq, some_rational))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_unequals
			-- Test {STS_RATIONAL_NUMBER}.unequals.
			-- Test {STS_INTEGER_NUMBER}.rational_unequals.
			-- Test {STI_INTEGER_NUMBER}.rational_unequals.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.unequals"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_unequals"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_unequals"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			i := integer_to_be_tested
			assert ("same_entity", not i.rational_unequals (i))
			assert ("same_entity ok", properties.rational_unequals_ok (i, i))

			i := integer_to_be_tested
			pq := same_rational (i)
			assert ("same_rational", not (i.rational_unequals (pq)))
			assert ("same_rational ok", properties.rational_unequals_ok (i, pq))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := i_s.any
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("some_rational", i.rational_unequals (pq) implies True)
					assert ("rational_unequals_ok", properties.rational_unequals_ok (i, pq))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end

		end

	test_rational_is_less
			-- Test {STS_RATIONAL_NUMBER}.is_less.
			-- Test {STS_INTEGER_NUMBER}.rational_is_less.
			-- Test {STI_INTEGER_NUMBER}.rational_is_less.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_less"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_is_less"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_is_less"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq > i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("i < pq", i.rational_is_less (pq))
			assert ("i < pq ok", properties.rational_is_less_ok (i, pq, some_rational))

			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq ≤ i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("not (i < pq)", not i.rational_is_less (pq))
			assert ("not (i < pq) ok", properties.rational_is_less_ok (i, pq, some_rational))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := i_s.any
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("rational_is_less", i.rational_is_less (pq) implies True)
					assert ("rational_is_less_ok", properties.rational_is_less_ok (i, pq, some_rational))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_is_less_equal
			-- Test {STS_RATIONAL_NUMBER}.is_less_equal.
			-- Test {STS_INTEGER_NUMBER}.rational_is_less_equal.
			-- Test {STI_INTEGER_NUMBER}.rational_is_less_equal.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_less_equal"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_is_less_equal"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_is_less_equal"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq ≥ i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("i ≤ pq", i.rational_is_less_equal (pq))
			assert ("i ≤ pq ok", properties.rational_is_less_equal_ok (i, pq, some_rational))

			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq < i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("not (i ≤ pq)", not i.rational_is_less_equal (pq))
			assert ("not (i ≤ pq) ok", properties.rational_is_less_equal_ok (i, pq, some_rational))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := i_s.any
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("rational_is_less_equal", i.rational_is_less_equal (pq) implies True)
					assert ("rational_is_less_equal_ok", properties.rational_is_less_equal_ok (i, pq, some_rational))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_is_greater
			-- Test {STS_RATIONAL_NUMBER}.is_greater.
			-- Test {STS_INTEGER_NUMBER}.rational_is_greater.
			-- Test {STI_INTEGER_NUMBER}.rational_is_greater.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_greater"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_is_greater"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_is_greater"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq < i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("i > pq", i.rational_is_greater (pq))
			assert ("i > pq ok", properties.rational_is_greater_ok (i, pq, some_rational))

			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq ≥ i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("not (i > pq)", not i.rational_is_greater (pq))
			assert ("not (i > pq) ok", properties.rational_is_greater_ok (i, pq, some_rational))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("rational_is_greater", i.rational_is_greater (pq) implies True)
					assert ("rational_is_greater_ok", properties.rational_is_greater_ok (i, pq, some_rational))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Q instead.
			end
		end

	test_rational_is_greater_equal
			-- Test {STS_RATIONAL_NUMBER}.is_greater_equal.
			-- Test {STS_INTEGER_NUMBER}.rational_is_greater_equal.
			-- Test {STI_INTEGER_NUMBER}.rational_is_greater_equal.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_greater_equal"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_is_greater_equal"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_is_greater_equal"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq ≤ i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("i ≥ pq", i.rational_is_greater_equal (pq))
			assert ("i ≥ pq ok", properties.rational_is_greater_equal_ok (i, pq, some_rational))

			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq > i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("not (i ≥ pq)", not i.rational_is_greater_equal (pq))
			assert ("not (i ≥ pq) ok", properties.rational_is_greater_equal_ok (i, pq, some_rational))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("rational_is_greater_equal", i.rational_is_greater_equal (pq) implies True)
					assert ("rational_is_greater_equal_ok", properties.rational_is_greater_equal_ok (i, pq, some_rational))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Q instead.
			end
		end

	test_rational_three_way_comparison
			-- Test {STS_RATIONAL_NUMBER}.three_way_comparison.
			-- Test {STS_INTEGER_NUMBER}.rational_three_way_comparison.
			-- Test {STI_INTEGER_NUMBER}.rational_three_way_comparison.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.three_way_comparison"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_three_way_comparison"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_three_way_comparison"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq > i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("negative", i.rational_three_way_comparison (pq) ≍ - One)

			assert ("same_entity", i.rational_three_way_comparison (i) ≍ Zero)
			assert ("same_rational", i.rational_three_way_comparison (same_rational (i)) ≍ Zero)

			from
				i := integer_to_be_tested
				pq := some_rational
			until
				pq < i
			loop
				i := integer_to_be_tested
				pq := some_rational
			end
			assert ("positive", i.rational_three_way_comparison (pq) ≍ One)

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert (
						"rational_three_way_comparison",
						attached i.rational_three_way_comparison (pq)
						)
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_min
			-- Test {STI_INTEGER_NUMBER}.rational_min.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.rational_min"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("rational_min", attached i.rational_min (pq))
					assert ("rational_min_ok", properties.rational_min_ok (i, pq, some_rational))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Q instead.
			end
		end

	test_rational_max
			-- Test {STI_INTEGER_NUMBER}.rational_max.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.rational_max"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("rational_max", attached i.rational_max (pq))
					assert ("rational_max_ok", properties.rational_max_ok (i, pq, some_rational))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Q instead.
			end
		end

	test_equals
			-- Test {STS_INTEGER_NUMBER}.equals.
			-- Test {STI_INTEGER_NUMBER}.equals.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.equals"
			testing: "covers/{STI_INTEGER_NUMBER}.equals"
		local
			j: like integer_to_be_tested
			k: like some_integer
			js, ks: like Z.subset_anchor -- TODO: Take Z instead.
		do
			j := integer_to_be_tested
			assert ("same_entity", j ≍ j)
			assert ("same_entity_ok", properties.equals_ok (j, j, j))

			j := integer_to_be_tested
			k := same_integer (j)
			assert ("same_integer", j ≍ k)
			assert ("same_integer_ok", properties.equals_ok (j, k, same_integer (k)))

			from
				js := Z
			until
				js.is_empty
			loop
				j := js.any
				from
					ks := Z
				until
					ks.is_empty
				loop
					k := same_integer (ks.any)
					assert ("some_integer", j ≍ k implies True)
					ks := ks.others
				variant
					cardinality: {like some_set}.natural_as_integer (# ks) -- TODO: Take Z instead.
				end
				js := js.others
			variant
				cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
			end
			assert ("equals_ok", properties.equals_ok (integer_to_be_tested, some_integer, some_integer))
		end

	test_unequals
			-- Test {STS_INTEGER_NUMBER}.unequals.
			-- Test {STI_INTEGER_NUMBER}.unequals.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.unequals"
			testing: "covers/{STI_INTEGER_NUMBER}.unequals"
		local
			j: like integer_to_be_tested
			k: like some_integer
			js, ks: like Z.subset_anchor -- TODO: Take Z instead.
		do
			j := integer_to_be_tested
			assert ("same_entity", not (j ≭ j))
			assert ("same_entity_ok", properties.unequals_ok (j, j))

			j := integer_to_be_tested
			k := same_integer (j)
			assert ("same_integer", not (j ≭ k))
			assert ("same_integer_ok", properties.unequals_ok (j, k))

			from
				js := Z
			until
				js.is_empty
			loop
				j := js.any
				from
					ks := Z
				until
					ks.is_empty
				loop
					k := same_integer (ks.any)
					assert ("some_integer", j ≭ k implies True)
					assert ("unequals_ok", properties.unequals_ok (j, k))
					ks := ks.others
				variant
					ks_cardinality: {like some_set}.natural_as_integer (# ks)
				end
				js := js.others
			variant
				js_cardinality: {like some_set}.natural_as_integer (# js)
			end
		end

	test_is_less
			-- Test {STS_INTEGER_NUMBER}.is_less.
			-- Test {STI_INTEGER_NUMBER}.is_less.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.is_less"
			testing: "covers/{STI_INTEGER_NUMBER}.is_less"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested ∧ ({STI_INTEGER_NUMBER}.Max_value - One)
			k := some_integer.abs ∨ One
			if 0 < i then
				k := k ∧ ({STI_INTEGER_NUMBER}.Max_value - i)
			end
			j := k + i
			assert ("i < j", i < j)
			assert ("i < j ok", properties.is_less_ok (i, j, some_integer))

			i := integer_to_be_tested
			k := - some_integer.abs
			if i < 0 then
				k := k ∨ ({STI_INTEGER_NUMBER}.Min_value - i)
			end
			j := k + i
			assert ("not (i < j)", not (i < j))
			assert ("not (i < j) ok", properties.is_less_ok (i, j, some_integer))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("is_less", i < j implies True)
					assert ("is_less_ok", properties.is_less_ok (i, j, some_integer))
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_is_less_equal
			-- Test {STS_INTEGER_NUMBER}.is_less_equal.
			-- Test {STI_INTEGER_NUMBER}.is_less_equal.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.is_less_equal"
			testing: "covers/{STI_INTEGER_NUMBER}.is_less_equal"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested
			k := some_integer.abs ∨ Zero
			if 0 < i then
				k := k ∧ ({STI_INTEGER_NUMBER}.Max_value - i)
			end
			j := k + i
			assert ("i ≤ j", i ≤ j)
			assert ("i ≤ j ok", properties.is_less_equal_ok (i, j, some_integer))

			i := integer_to_be_tested ∨ ({STI_INTEGER_NUMBER}.Min_value + One)
			k := (- some_integer.abs) ∧ (- One)
			if i < 0 then
				k := k ∨ ({STI_INTEGER_NUMBER}.Min_value - i)
			end
			j := k + i
			assert ("not (i ≤ j)", not (i ≤ j))
			assert ("not (i ≤ j) ok", properties.is_less_equal_ok (i, j, some_integer))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("is_less_equal", i ≤ j implies True)
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
			assert ("is_less_equal_ok", properties.is_less_equal_ok (integer_to_be_tested, some_integer, some_integer))
		end

	test_is_greater
			-- Test {STS_INTEGER_NUMBER}.is_greater.
			-- Test {STI_INTEGER_NUMBER}.is_greater.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.is_greater"
			testing: "covers/{STI_INTEGER_NUMBER}.is_greater"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested ∨ ({STI_INTEGER_NUMBER}.Min_value + One)
			k := (- some_integer.abs) ∧ (- One)
			if i < 0 then
				k := k ∨ ({STI_INTEGER_NUMBER}.Min_value - i)
			end
			j := k + i
			assert ("i > j", i > j)
			assert ("i > j ok", properties.is_greater_ok (i, j, some_integer))

			i := integer_to_be_tested
			k := some_integer.abs ∨ Zero
			if 0 < i then
				k := k ∧ ({STI_INTEGER_NUMBER}.Max_value - i)
			end
			j := k + i
			assert ("not (i > j)", not (i > j))
			assert ("not (i > j) ok", properties.is_greater_ok (i, j, some_integer))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("is_greater", i > j implies True)
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
			assert ("is_greater_ok", properties.is_greater_ok (integer_to_be_tested, some_integer, some_integer))
		end

	test_is_greater_equal
			-- Test {STS_INTEGER_NUMBER}.is_greater_equal.
			-- Test {STI_INTEGER_NUMBER}.is_greater_equal.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.is_greater_equal"
			testing: "covers/{STI_INTEGER_NUMBER}.is_greater_equal"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested
			k := - some_integer.abs
			if i < 0 then
				k := k ∨ ({STI_INTEGER_NUMBER}.Min_value - i)
			end
			j := k + i
			assert ("i ≥ j", i ≥ j)
			assert ("i ≥ j ok", properties.is_greater_equal_ok (i, j, some_integer))

			i := integer_to_be_tested ∧ ({STI_INTEGER_NUMBER}.Max_value - One)
			k := some_integer.abs ∨ One
			if 0 < i then
				k := k ∧ ({STI_INTEGER_NUMBER}.Max_value - i)
			end
			j := k + i
			assert ("not (i ≥ j)", not (i ≥ j))
			assert ("not (i ≥ j) ok", properties.is_greater_equal_ok (i, j, some_integer))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("is_greater_equal", i ≥ j implies True)
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
			assert ("is_greater_equal_ok", properties.is_greater_equal_ok (integer_to_be_tested, some_integer, some_integer))
		end

	test_three_way_comparison
			-- Test {STI_INTEGER_NUMBER}.three_way_comparison.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.three_way_comparison"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested ∧ ({STI_INTEGER_NUMBER}.Max_value - One)
			k := some_integer.abs ∨ One
			if 0 < i then
				k := k ∧ ({STI_INTEGER_NUMBER}.Max_value - i)
			end
			j := k + i
			assert ("negative", i ⋚ j ≍ - One)

			i := integer_to_be_tested
			assert ("same_entity", i ⋚ i ≍ Zero)
			assert ("same_integer", i ⋚ same_integer (i) ≍ Zero)

			i := integer_to_be_tested ∨ ({STI_INTEGER_NUMBER}.Min_value + One)
			k := (- some_integer.abs) ∧ (- One)
			if i < 0 then
				k := k ∨ ({STI_INTEGER_NUMBER}.Min_value - i)
			end
			j := k + i
			assert ("positive", i ⋚ j ≍ One)

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("three_way_comparison", attached (i ⋚ j))
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_min
			-- Test {STI_INTEGER_NUMBER}.min.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.min"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested
			k := some_integer.abs ∨ Zero
			if 0 < i then
				k := k ∧ ({STI_INTEGER_NUMBER}.Max_value - i)
			end
			j := k + i
			assert ("i", (i ∧ j) ≍ i)
			assert ("i ok", properties.min_ok (i, j, some_integer))

			i := integer_to_be_tested ∨ ({STI_INTEGER_NUMBER}.Min_value + One)
			k := (- some_integer.abs) ∧ (- One)
			if i < 0 then
				k := k ∨ ({STI_INTEGER_NUMBER}.Min_value - i)
			end
			j := k + i
			assert ("j", (i ∧ j) ≍ j)
			assert ("j ok", properties.min_ok (i, j, some_integer))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("min", attached (i ∧ j))
					assert ("min_ok", properties.min_ok (i, j, some_integer))
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_max
			-- Test {STI_INTEGER_NUMBER}.max.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.max"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested
			k := - some_integer.abs
			if i < 0 then
				k := k ∨ ({STI_INTEGER_NUMBER}.Min_value - i)
			end
			j := k + i
			assert ("i", (i ∨ j) ≍ i)
			assert ("i ok", properties.max_ok (i, j, some_integer))

			i := integer_to_be_tested ∧ ({STI_INTEGER_NUMBER}.Max_value - One)
			k := some_integer.abs ∨ One
			if 0 < i then
				k := k ∧ ({STI_INTEGER_NUMBER}.Max_value - i)
			end
			j := k + i
			assert ("j", (i ∨ j) ≍ j)
			assert ("j ok", properties.max_ok (i, j, some_integer))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("max", attached (i ∨ j))
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
			assert ("max_ok", properties.max_ok (integer_to_be_tested, some_integer, some_integer))
		end

feature -- Test routines (Relationship)

	test_complex_divisible
			-- Test {STS_COMPLEX_NUMBER}.divisible.
			-- Test {STS_REAL_NUMBER}.complex_divisible.
			-- Test {STS_INTEGER_NUMBER}.complex_divisible.
			-- Test {STI_INTEGER_NUMBER}.complex_divisible.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.divisible"
			testing: "covers/{STS_REAL_NUMBER}.complex_divisible"
			testing: "covers/{STS_INTEGER_NUMBER}.complex_divisible"
			testing: "covers/{STI_INTEGER_NUMBER}.complex_divisible"
		local
			i_s: like Z.subset_anchor
			zs: like Quick_c.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					zs := Quick_c -- TODO: Limit to 6085 ints x comps.
				until
					zs.is_empty
				loop
					i := i_s.any
					assert ("complex_divisible", i.complex_divisible (zs.any))
					zs := zs.others
				variant
					cardinality: {like some_set}.natural_as_integer (# zs)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_real_divisible
			-- Test {STS_REAL_NUMBER}.divisible.
			-- Test {STS_INTEGER_NUMBER}.real_divisible.
			-- Test {STI_INTEGER_NUMBER}.real_divisible.
		note
			testing: "covers/{STS_REAL_NUMBER}.divisible"
			testing: "covers/{STS_INTEGER_NUMBER}.real_divisible"
			testing: "covers/{STI_INTEGER_NUMBER}.real_divisible"
		local
			i: like integer_to_be_tested
			x: like some_real
			xs: like R.subset_anchor
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				i := integer_to_be_tested -- TODO: Take every possible integer.
				x := xs.any
				assert ("real_divisible", i.real_divisible (x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_rational_multipliable
			-- Test {STI_INTEGER_NUMBER}.rational_multipliable.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.rational_multipliable"
		local
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
			i: like integer_to_be_tested
			pq: like some_rational
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := i_s.any
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := pqs.any
					assert ("rational_multipliable", i.rational_multipliable (pq))
					assert ("rational_multipliable_ok", properties.rational_multipliable_ok (i, pq))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_divisible
			-- Test {STS_RATIONAL_NUMBER}.divisible.
			-- Test {STS_INTEGER_NUMBER}.rational_divisible.
			-- Test {STI_INTEGER_NUMBER}.rational_divisible.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.divisible"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_divisible"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_divisible"
		local
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
			i: like integer_to_be_tested
			j: like some_integer
			pq: like some_rational
		do
			i := integer_to_be_tested
			from
				j := some_integer
			until
				j ≭ Zero
			loop
				j := some_integer
			end
			pq := Zero / j
			assert ("not i.rational_divisible (pq)", not i.rational_divisible (pq))
			assert ("not i.rational_divisible (pq) ok", properties.rational_divisible_ok (i, pq))

			i := integer_to_be_tested
			from
				pq := some_rational
			until
				pq ≭ Zero
			loop
				pq := some_rational
			end
			assert ("i.rational_divisible (pq)", i.rational_divisible (pq))
			assert ("i.rational_divisible (pq) ok", properties.rational_divisible_ok (i, pq))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("rational_divisible", i.rational_divisible (pq) implies True)
					assert ("rational_divisible_ok", properties.rational_divisible_ok (i, pq))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Q instead.
			end
		end

	test_divisible
			-- Test {STS_INTEGER_NUMBER}.divisible.
			-- Test {STI_INTEGER_NUMBER}.divisible.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.divisible"
			testing: "covers/{STI_INTEGER_NUMBER}.divisible"
		local
			i: like integer_to_be_tested
			j: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested
			from
				j := some_integer
			until
				j ≭ Zero
			loop
				j := some_integer
			end
			assert ("i.divisible (j)", i.divisible (j))

			i := integer_to_be_tested
			j := Zero
			assert ("not i.divisible (j)", not i.divisible (j))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("divisible", i.divisible (j) implies True)
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_divides
			-- Test {STS_INTEGER_NUMBER}.divides.
			-- Test {STI_INTEGER_NUMBER}.divides.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.divides"
			testing: "covers/{STI_INTEGER_NUMBER}.divides"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js, ks: like Z.subset_anchor -- TODO: Take Z instead.
		do
			from
				i := integer_to_be_tested
				k := some_integer
			until
				i ≭ Zero and not properties.product_overflows (i, k)
			loop
				i := integer_to_be_tested
				k := some_integer
			end
			j := k ⋅ i
			assert ("i | j", i | j)
			assert ("i | j ok", properties.divides_ok (i, j, some_integer))

			from
				i := integer_to_be_tested
			until
				i.abs ≭ One
			loop
				i := integer_to_be_tested
			end
			j := (some_integer ⋅ i) + One
			assert ("not (i | j)", not (i | j))
			assert ("not (i | j) ok", properties.divides_ok (i, j, some_integer))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("divides", (i | j) implies True)
					from
						ks := Z
					until
						ks.is_empty
					loop
						k := same_integer (ks.any)
						assert ("divides_ok", properties.divides_ok (i, j, k))
						ks := ks.others
					variant
						ks_cardinality: {like some_set}.natural_as_integer (# ks)
					end
					js := js.others
				variant
					js_cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

feature -- Test routines (Operation)

	test_argument
			-- Test {STI_INTEGER_NUMBER}.argument.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.argument"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z  -- TODO: Replace by Z.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("argument", attached i.argument)
				assert ("srnp_argument_ok", properties.srnp_argument_ok (i))
				assert ("argument_ok", properties.argument_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_phase
			-- Test {STI_INTEGER_NUMBER}.phase.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.phase"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z  -- TODO: Replace by Z.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("phase", attached i.phase)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			i := integer_to_be_tested
			assert ("srnp_phase_ok", properties.srnp_phase_ok (i))
			assert ("phase_ok", properties.phase_ok (i))
		end

	test_conjugate
			-- Test {STI_INTEGER_NUMBER}.conjugate.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.conjugate"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z  -- TODO: Replace by Z.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("conjugate", attached i.conjugate)
				assert
					("conjugate_ok", properties.conjugate_ok (i, some_complex))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_complex_plus
			-- Test {STI_INTEGER_NUMBER}.complex_plus.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.complex_plus"
		local
			i_s: like Z.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			i: like integer_to_be_tested
			ab: like some_complex
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					zt0 := Quick_C.as_tuple
				until
					zt0.n × # i_s ≤ 46_119 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						zt0 := zt0.suffix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# i_s * zt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					i := i_s.any
					ab := same_complex (zt.first)
					assert ("complex_plus", attached i.complex_plus (ab))
					assert
						("complex_plus_ok", properties.complex_plus_ok (i, ab))
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_complex_minus
			-- Test {STI_INTEGER_NUMBER}.complex_minus.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.complex_minus"
		local
			i_s: like Z.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			i: like integer_to_be_tested
			ab: like some_complex
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					zt0 := Quick_C.as_tuple
				until
					zt0.n × # i_s ≤ 56_145 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						zt0 := zt0.suffix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# i_s * zt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					i := i_s.any
					ab := same_complex (zt.first)
					assert ("complex_minus", attached i.complex_minus (ab))
					assert
						("complex_minus_ok", properties.complex_minus_ok (i, ab))
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_complex_product
			-- Test {STI_INTEGER_NUMBER}.complex_product.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.complex_product"
		local
			i_s: like Z.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			i: like integer_to_be_tested
			ab: like some_complex
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					zt0 := Quick_C.as_tuple
				until
					zt0.n × # i_s ≤ 1_731 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# i_s * zt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					i := i_s.any
					ab := same_complex (zt.first)
					assert ("complex_product", attached i.complex_product (ab))
					assert (
						"complex_product_ok",
						properties.complex_product_ok (i, ab, some_complex)
						)
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_complex_quotient
			-- Test {STI_INTEGER_NUMBER}.complex_quotient.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.complex_quotient"
		local
			i_s: like Z.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			i: like integer_to_be_tested
			ab: like some_complex
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					zt0 := Quick_C.as_tuple
				until
					zt0.n × # i_s ≤ 1_712 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						zt0 := zt0.suffix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# i_s * zt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					i := i_s.any
					ab := same_complex (zt.first)
					assert ("complex_quotient", attached i.complex_quotient (ab))
					assert (
						"complex_quotient_ok",
						properties.complex_quotient_ok (i, ab, some_complex)
						)
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_real_plus
			-- Test {STI_INTEGER_NUMBER}.real_plus.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_plus"
		local
			i_s: like Z.subset_anchor
			xt0, xt: like R.as_tuple
			i: like integer_to_be_tested
			x: like some_real
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					xt0 := R.as_tuple
				until
					xt0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					else
						xt0 := xt0.suffix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# i_s * xt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					i := i_s.any
					x := same_real (xt.first)
					assert ("real_plus", attached i.real_plus (x))
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("real_plus_ok", properties.real_plus_ok (integer_to_be_tested, some_real, some_real))
		end

	test_real_minus
			-- Test {STI_INTEGER_NUMBER}.real_minus.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_minus"
		local
			i_s: like Z.subset_anchor
			xt0, xt: like R.as_tuple
			i: like integer_to_be_tested
			x: like some_real
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					xt0 := R.as_tuple
				until
					xt0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					else
						xt0 := xt0.suffix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# i_s * xt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					i := i_s.any
					x := same_real (xt.first)
					assert ("real_minus", attached i.real_minus (x))
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("real_minus_ok", properties.real_minus_ok (integer_to_be_tested, some_real, some_real))
		end

	test_real_opposite
			-- Test {STI_INTEGER_NUMBER}.real_opposite.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_opposite"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("real_opposite", attached i.real_opposite)
--				assert (
--					"real_opposite_ok",
--					properties.real_opposite_ok (integer_to_be_tested)
--					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_real_product
			-- Test {STI_INTEGER_NUMBER}.real_product.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_product"
		local
			i_s: like Z.subset_anchor
			xt0, xt: like R.as_tuple
			i: like integer_to_be_tested
			x: like some_real
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					xt0 := R.as_tuple
				until
					xt0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					else
						xt0 := xt0.suffix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# i_s * xt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					i := i_s.any
					x := same_real (xt.first)
					assert ("real_product", attached i.real_product (x))
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("real_product_ok", properties.real_product_ok (integer_to_be_tested, some_real, some_real))
		end

	test_real_quotient
			-- Test {STI_INTEGER_NUMBER}.real_quotient.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_quotient"
		local
			i_s: like Z.subset_anchor
			xt0, xt: like R.as_tuple
			i: like integer_to_be_tested
			x: like some_real
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					xt0 := R.as_tuple
				until
					xt0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix (
							(next_random_item \\ (xt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						xt0 := xt0.suffix (
							(next_random_item \\ (xt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# i_s * xt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					i := i_s.any
					x := same_real (xt.first)
					assert ("real_quotient", attached i.real_quotient (x))
					assert (
						"real_quotient_ok",
						properties.real_quotient_ok (i, x, some_real)
						)
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_real_reciprocal
			-- Test {STI_INTEGER_NUMBER}.real_reciprocal.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_reciprocal"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z  -- TODO: Replace by Z.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("real_reciprocal", attached i.real_reciprocal)
				assert (
					"real_reciprocal_ok",
					properties.real_reciprocal_ok (i, some_real)
					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_real_inverse
			-- Test {STI_INTEGER_NUMBER}.real_inverse.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_inverse"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z  -- TODO: Replace by Z.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("real_inverse", attached i.real_inverse)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
			assert ("real_inverse_ok", properties.real_inverse_ok (integer_to_be_tested, some_real))
		end

	test_power
			-- Test {STI_INTEGER_NUMBER}.power.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.power"
		local
			i_s: like Z.subset_anchor
			xt0, xt: like R.as_tuple
			i: like integer_to_be_tested
			x: like some_real
		do
			from
				i_s := Z  -- TODO: Replace by Z.
				from
					xt0 := R.as_tuple
				until
					xt0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					else
						xt0 := xt0.suffix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# i_s * xt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					i := i_s.any
					x := same_real (xt.first)
					assert ("power", attached (i ^ x))
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end

			assert ("power_ok", properties.power_ok (integer_to_be_tested))
		end

	test_square_root
			-- Test {STI_INTEGER_NUMBER}.square_root.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.square_root"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z.Non_negative
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("non_negative", i.real_is_greater_equal (Zero)) -- i_s ⊆ Z.Non_negative
				assert ("square_root", attached √¯i)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
			assert ("square_root_ok", properties.square_root_ok (integer_to_be_tested))
		end

	test_rational_modulus
			-- Test {STI_INTEGER_NUMBER}.rational_modulus.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.rational_modulus"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z  -- TODO: Replace by Z.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("rational_modulus", attached i.rational_modulus)
				assert ("rational_modulus_ok", properties.rational_modulus_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_abs
			-- Test {STI_INTEGER_NUMBER}.rational_abs.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.rational_abs"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z  -- TODO: Replace by Z.
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("rational_abs", attached i.rational_abs)
				assert ("rational_abs_ok", properties.rational_abs_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_plus
			-- Test {STS_RATIONAL_NUMBER}.plus.
			-- Test {STS_INTEGER_NUMBER}.rational_plus.
			-- Test {STI_INTEGER_NUMBER}.rational_plus.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.plus"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_plus"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_plus"
		local
			i_s: like Z.subset_anchor
			pqt0, pqt: like Q.as_tuple
			i: like integer_to_be_tested
			pq: like some_rational
		do
			from
				i_s := Z
				from
					pqt0 := Q.as_tuple
				until
					pqt0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						pqt0 := pqt0.prefix ((next_random_item \\ (pqt0.n.as_integer_32 + 1)).as_natural_32)
					else
						pqt0 := pqt0.suffix ((next_random_item \\ (pqt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# i_s * pqt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					pqt := pqt0
				until
					pqt.is_unit
				loop
					i := i_s.any
					pq := same_rational (pqt.first)
					assert ("rational_plus", attached i.rational_plus (pq))
					assert ("rational_plus_ok", properties.rational_plus_ok (i, pq, some_rational))
					pqt := pqt.tail
				variant
					n: {like some_set}.natural_as_integer (pqt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_minus
			-- Test {STS_RATIONAL_NUMBER}.minus.
			-- Test {STS_INTEGER_NUMBER}.rational_minus.
			-- Test {STI_INTEGER_NUMBER}.rational_minus.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.minus"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_minus"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_minus"
		local
			i_s: like Z.subset_anchor
			pqt_0, pqt: like Q.as_tuple
			i: like integer_to_be_tested
			pq: like some_rational
		do
					assert ("rational_minus_ok", properties.rational_minus_ok (-two-two-two, -one/(two+one), one/(two+two)))
			from
				i_s := Z
				from
					pqt_0 := Q.as_tuple
				until
--					pqt_0.n × # i_s ≤ 83_494 -- TODO: Syntax error!
					pqt_0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						pqt_0 := pqt_0.prefix ((next_random_item \\ (pqt_0.n.as_integer_32 + 1)).as_natural_32)
					else
						pqt_0 := pqt_0.suffix ((next_random_item \\ (pqt_0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# i_s * pqt_0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					pqt := pqt_0
				until
					pqt.is_unit
				loop
					i := i_s.any
					pq := same_rational (pqt.first)
					assert ("rational_minus", attached i.rational_minus (pq))
					assert ("rational_minus_ok", properties.rational_minus_ok (i, pq, some_rational))
					pqt := pqt.tail
				variant
					n: {like some_set}.natural_as_integer (pqt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_opposite
			-- Test {STI_INTEGER_NUMBER}.rational_opposite.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.rational_opposite"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("rational_opposite", attached i.rational_opposite)
				assert (
					"rational_opposite_ok",
					properties.rational_opposite_ok (integer_to_be_tested)
					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_product
			-- Test {STS_INTEGER_NUMBER}.rational_product.
			-- Test {STI_INTEGER_NUMBER}.rational_product.
		note
			testing: "covers/{STS_INTEGER_NUMBER}.rational_product"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_product"
		local
			i_s: like Z.subset_anchor
			pqt0, pqt: like Q.as_tuple
			i: like integer_to_be_tested
			pq: like some_rational
		do
			from
				i_s := Z
				from
					pqt0 := Q.as_tuple
				until
					pqt0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						pqt0 := pqt0.prefix ((next_random_item \\ (pqt0.n.as_integer_32 + 1)).as_natural_32)
					else
						pqt0 := pqt0.suffix ((next_random_item \\ (pqt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# i_s * pqt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					pqt := pqt0
				until
					pqt.is_unit
				loop
					i := i_s.any
					pq := same_rational (pqt.first)
					assert ("rational_product", attached i.rational_product (pq))
					assert ("rational_product_ok", properties.rational_product_ok (i, pq, some_rational))
					pqt := pqt.tail
				variant
					n: {like some_set}.natural_as_integer (pqt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rational_quotient
			-- Test {STS_RATIONAL_NUMBER}.quotient.
			-- Test {STS_INTEGER_NUMBER}.rational_quotient.
			-- Test {STI_INTEGER_NUMBER}.rational_quotient.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.quotient"
			testing: "covers/{STS_INTEGER_NUMBER}.rational_quotient"
			testing: "covers/{STI_INTEGER_NUMBER}.rational_quotient"
		local
			i_s: like Z.subset_anchor
			pqt0, pqt: like Q.as_tuple
			i: like integer_to_be_tested
			pq: like some_rational
		do
			from
				i_s := Z
				from
					pqt0 := Q.as_tuple
				until
					pqt0.n × # i_s ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						pqt0 := pqt0.prefix (
							(next_random_item \\ (pqt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						pqt0 := pqt0.suffix (
							(next_random_item \\ (pqt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# i_s * pqt0.n) print ('%N')
			until
				i_s.is_empty
			loop
				from
					pqt := pqt0
				until
					pqt.is_unit
				loop
					i := i_s.any
					pq := same_rational (pqt.first)
					assert ("rational_quotient", i.rational_divisible (pq) implies attached i.rational_quotient (pq))
					assert (
						"rational_quotient_ok",
						properties.rational_quotient_ok (i, pq, some_rational)
						)
					pqt := pqt.tail
				variant
					n: {like some_set}.natural_as_integer (pqt.n)
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_modulus
			-- Test {STI_INTEGER_NUMBER}.modulus.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.modulus"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested ∨ ({STI_INTEGER_NUMBER}.Min_value + One)
			assert ("non-negative modulus", i.modulus ≥ Zero)
			assert ("non-negative modulus ok", properties.modulus_ok (i))

			i := {STI_INTEGER_NUMBER}.Min_value
			assert ("negative modulus", i.modulus < Zero)
			assert ("negative modulus ok", properties.modulus_ok (i))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("modulus", attached i.modulus)
				assert ("modulus_ok", properties.modulus_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_abs
			-- Test {STI_INTEGER_NUMBER}.abs.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.abs"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor -- TODO: Take Z instead.
		do
			i := integer_to_be_tested ∨ ({STI_INTEGER_NUMBER}.Min_value + One)
			assert ("non-negative abs", i.abs ≥ Zero)
			assert ("non-negative abs ok", properties.abs_ok (i))

			i := {STI_INTEGER_NUMBER}.Min_value
			assert ("negative abs", i.abs < Zero)
			assert ("negative abs ok", properties.abs_ok (i))

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("abs", attached i.abs)
				assert ("abs_ok", properties.abs_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_plus
			-- Test {STI_INTEGER_NUMBER}.plus.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.plus"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js, ks: like Z.subset_anchor -- TODO: Take Z instead.
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("plus", attached (i + j))
					from
						ks := Z
					until
						ks.is_empty
					loop
						k := same_integer (ks.any)
						assert ("plus_ok", properties.plus_ok (i, j, k))
						ks := ks.others
					variant
						ks_cardinality:
							{like some_set}.natural_as_integer (# ks) -- TODO: Take Z instead.
					end
					js := js.others
				variant
					js_cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_identity
			-- Test {STS_COMPLEX_NUMBER}.a.
			-- Test {STS_COMPLEX_NUMBER}.real.
			-- Test {STS_COMPLEX_NUMBER}.identity.
			-- Test {STS_REAL_NUMBER}.to_rational.
			-- Test {STS_REAL_NUMBER}.truncated_to_integer.
			-- Test {STS_REAL_NUMBER}.identity.
			-- Test {STS_RATIONAL_NUMBER}.p.
			-- Test {STS_RATIONAL_NUMBER}.numerator.
			-- Test {STS_RATIONAL_NUMBER}.truncated_to_integer.
			-- Test {STS_RATIONAL_NUMBER}.identity.
			-- Test {STS_INTEGER_NUMBER}.identity.
			-- Test {STI_INTEGER_NUMBER}.identity.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.a"
			testing: "covers/{STS_COMPLEX_NUMBER}.real"
			testing: "covers/{STS_COMPLEX_NUMBER}.identity"
			testing: "covers/{STS_REAL_NUMBER}.to_rational"
			testing: "covers/{STS_REAL_NUMBER}.truncated_to_integer"
			testing: "covers/{STS_REAL_NUMBER}.identity"
			testing: "covers/{STS_RATIONAL_NUMBER}.p"
			testing: "covers/{STS_RATIONAL_NUMBER}.numerator"
			testing: "covers/{STS_RATIONAL_NUMBER}.truncated_to_integer"
			testing: "covers/{STS_RATIONAL_NUMBER}.identity"
			testing: "covers/{STS_INTEGER_NUMBER}.identity"
			testing: "covers/{STI_INTEGER_NUMBER}.identity"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("identity", attached (+ i)) -- TODO: Anomalous join.
				assert ("identity_ok", properties.identity_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_minus
			-- Test {STI_INTEGER_NUMBER}.minus.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.minus"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js, ks: like Z.subset_anchor -- TODO: Take Z instead.
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					from
						ks := Z
					until
						ks.is_empty
					loop
						k := same_integer (ks.any)
						assert ("minus", attached (i - j))
						assert ("minus_ok", properties.minus_ok (i, j, k))
						ks := ks.others
					variant
						cardinality: {like some_set}.natural_as_integer (# ks) -- TODO: Take Z instead.
					end
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_opposite
			-- Test {STI_INTEGER_NUMBER}.opposite.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.opposite"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor -- TODO: Take Z instead.
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("opposite", attached (- i))
				assert ("opposite_ok", properties.opposite_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_product
			-- Test {STI_INTEGER_NUMBER}.product.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.product"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js, ks: like Z.subset_anchor -- TODO: Take Z instead.
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					from
						ks := Z
					until
						ks.is_empty
					loop
						k := same_integer (ks.any)
						assert ("product", attached (i ⋅ j))
						assert ("product_ok", properties.product_ok (i, j, k))
						ks := ks.others
					variant
						cardinality: {like some_set}.natural_as_integer (# ks) -- TODO: Take Z instead.
					end
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_quotient
			-- Test {STI_INTEGER_NUMBER}.quotient.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.quotient"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js, ks: like Z.subset_anchor -- TODO: Take Z instead.
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("quotient", i.divisible (j) implies attached (i / j))
					from
						ks := Z
					until
						ks.is_empty
					loop
						k := same_integer (ks.any)
						assert ("quotient_ok", properties.quotient_ok (i, j, k))
						ks := ks.others
					variant
						ks_cardinality: {like some_set}.natural_as_integer (# ks) -- TODO: Take Z instead.
					end
					js := js.others
				variant
					js_cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_integer_quotient
			-- Test {STI_INTEGER_NUMBER}.integer_quotient.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.integer_quotient"
		local
			i: like integer_to_be_tested
			j, k: like some_integer
			i_s, js, ks: like Z.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert (
						"integer_quotient",
						i.divisible (j) implies attached (i // j)
						)
					from
						ks := Z
					until
						ks.is_empty
					loop
						k := same_integer (ks.any)
						assert (
							"integer_quotient_ok",
							properties.integer_quotient_ok (i, j, k)
							)
						ks := ks.others
					variant
						ks_cardinality:
							{like some_set}.natural_as_integer (# ks) -- TODO: Take Z instead.
					end
					js := js.others
				variant
					js_cardinality:
						{like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_integer_remainder
			-- Test {STI_INTEGER_NUMBER}.integer_remainder.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.integer_remainder"
		local
			i: like integer_to_be_tested
			j: like some_integer
			i_s, js: like Z.subset_anchor -- TODO: Take Z instead.
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				from
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("integer_remainder", i.divisible (j) implies attached (i \\ j))
					assert ("integer_remainder_ok", properties.integer_remainder_ok (i, j))
					js := js.others
				variant
					cardinality: {like some_set}.natural_as_integer (# js) -- TODO: Take Z instead.
				end
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Z instead.
			end
		end

	test_reciprocal
			-- Test {STS_RATIONAL_NUMBER}.reciprocal.
			-- Test {STS_INTEGER_NUMBER}.reciprocal.
			-- Test {STI_INTEGER_NUMBER}.reciprocal.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.reciprocal"
			testing: "covers/{STS_INTEGER_NUMBER}.reciprocal"
			testing: "covers/{STI_INTEGER_NUMBER}.reciprocal"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("reciprocal", i.is_invertible implies attached i.reciprocal)
				from
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("reciprocal_ok", properties.reciprocal_ok (i, pq))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Q instead.
			end
		end

	test_inverse
			-- Test {STS_RATIONAL_NUMBER}.inverse.
			-- Test {STS_INTEGER_NUMBER}.inverse.
			-- Test {STI_INTEGER_NUMBER}.inverse.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.inverse"
			testing: "covers/{STS_INTEGER_NUMBER}.inverse"
			testing: "covers/{STI_INTEGER_NUMBER}.inverse"
		local
			i: like integer_to_be_tested
			pq: like some_rational
			i_s: like Z.subset_anchor
			pqs: like Q.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("inverse", i.is_invertible implies attached i.inverse)
				from
					pqs := Q
				until
					pqs.is_empty
				loop
					pq := same_rational (pqs.any)
					assert ("inverse_ok", properties.inverse_ok (i, pq))
					pqs := pqs.others
				variant
					pqs_cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s) -- TODO: Take Q instead.
			end
		end

feature -- Test routines (Conversion)

	test_as_complex
			-- Test {STI_INTEGER_NUMBER}.as_complex.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.as_complex"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("as_complex", attached i.as_complex)
				assert
					("complex_from_integer", attached complex_from_integer (i))
				assert ("as_complex_ok", properties.as_complex_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_as_real
			-- Test {STI_INTEGER_NUMBER}.as_real.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.as_real"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("as_real", attached i.as_real)
				assert ("real_from_integer", attached real_from_integer (i))
				assert ("as_real_ok", properties.as_real_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_as_rational
			-- Test {STI_INTEGER_NUMBER}.as_rational.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.as_rational"
		local
			i: like integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("as_rational", attached i.as_rational)
				assert ("rational_from_integer", attached rational_from_integer (i))
				assert ("as_rational_ok", properties.as_rational_ok (i))
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

feature -- Test routines (Math)

	test_arc_tangent
			-- Test {STS_COMPLEX_NUMBER}.arc_tangent.
			-- Test {STS_REAL_NUMBER}.arc_tangent.
			-- Test {STS_INTEGER_NUMBER}.arc_tangent.
			-- Test {STI_INTEGER_NUMBER}.arc_tangent.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.arc_tangent"
			testing: "covers/{STS_REAL_NUMBER}.arc_tangent"
			testing: "covers/{STS_INTEGER_NUMBER}.arc_tangent"
			testing: "covers/{STI_INTEGER_NUMBER}.arc_tangent"
		do
			assert ("-NaN / -NaN", attached {like integer_to_be_tested}.arc_tangent (-Nan / -Nan))
			assert ("NaN / -NaN", attached {like integer_to_be_tested}.arc_tangent (Nan / -Nan))
			assert ("-Infinity / -NaN", attached {like integer_to_be_tested}.arc_tangent (Negative_infinity / -Nan))
			assert ("Infinity / -NaN", attached {like integer_to_be_tested}.arc_tangent (Positive_infinity / -Nan))
			assert ("0 / -NaN", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / -Nan))
			assert ("-0 / -NaN", attached {like integer_to_be_tested}.arc_tangent (-Zero.as_real / -Nan))
			assert ("0 / -NaN", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / -Nan))
			assert ("a / -NaN", attached {like integer_to_be_tested}.arc_tangent (some_real.value / -Nan))

			assert ("-NaN / NaN", attached {like integer_to_be_tested}.arc_tangent (-Nan / Nan))
			assert ("NaN / NaN", attached {like integer_to_be_tested}.arc_tangent (Nan / Nan))
			assert ("-Infinity / NaN", attached {like integer_to_be_tested}.arc_tangent (Negative_infinity / Nan))
			assert ("Infinity / NaN", attached {like integer_to_be_tested}.arc_tangent (Positive_infinity / Nan))
			assert ("NaN", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Nan))
			assert ("-0 / NaN", attached {like integer_to_be_tested}.arc_tangent (-Zero.as_real / Nan))
			assert ("0 / NaN", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Nan))
			assert ("a / NaN", attached {like integer_to_be_tested}.arc_tangent (some_real.value / Nan))

			assert ("-NaN / -Infinity", attached {like integer_to_be_tested}.arc_tangent (-Nan / Negative_infinity))
			assert ("NaN / -Infinity", attached {like integer_to_be_tested}.arc_tangent (Nan / Negative_infinity))
			assert ("-Infinity / -Infinity", attached {like integer_to_be_tested}.arc_tangent (Negative_infinity / Negative_infinity))
			assert ("Infinity / -Infinity", attached {like integer_to_be_tested}.arc_tangent (Positive_infinity / Negative_infinity))
			assert ("0 / -Infinity", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Negative_infinity))
			assert ("-0 / -Infinity", attached {like integer_to_be_tested}.arc_tangent (-Zero.as_real / Negative_infinity))
			assert ("0 / -Infinity", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Negative_infinity))
			assert ("a / -Infinity", attached {like integer_to_be_tested}.arc_tangent (some_real.value / Negative_infinity))

			assert ("-NaN / Infinity", attached {like integer_to_be_tested}.arc_tangent (-Nan / Positive_infinity))
			assert ("NaN / Infinity", attached {like integer_to_be_tested}.arc_tangent (Nan / Positive_infinity))
			assert ("-Infinity / Infinity", attached {like integer_to_be_tested}.arc_tangent (Negative_infinity / Positive_infinity))
			assert ("Infinity / Infinity", attached {like integer_to_be_tested}.arc_tangent (Positive_infinity / Positive_infinity))
			assert ("0 / Infinity", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Positive_infinity))
			assert ("-0 / Infinity", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Positive_infinity))
			assert ("0 / Infinity", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Positive_infinity))
			assert ("a / Infinity", attached {like integer_to_be_tested}.arc_tangent (some_real.value / Positive_infinity))

			assert ("-NaN / -0", attached {like integer_to_be_tested}.arc_tangent (-Nan / -Zero.as_real))
			assert ("NaN / -0", attached {like integer_to_be_tested}.arc_tangent (Nan / -Zero.as_real))
			assert ("-Infinity / -0", attached {like integer_to_be_tested}.arc_tangent (Negative_infinity / -Zero.as_real))
			assert ("Infinity / -0", attached {like integer_to_be_tested}.arc_tangent (Positive_infinity / -Zero.as_real))
			assert ("0 / -0", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / -Zero.as_real))
			assert ("-0 / -0", attached {like integer_to_be_tested}.arc_tangent (-Zero.as_real / -Zero.as_real))
			assert ("0 / -0", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / -Zero.as_real))
			assert ("a / -0", attached {like integer_to_be_tested}.arc_tangent (some_real.value / -Zero.as_real))

			assert ("-NaN / 0", attached {like integer_to_be_tested}.arc_tangent (-Nan / Zero))
			assert ("-NaN / 0", attached {like integer_to_be_tested}.arc_tangent (-Nan / Zero))
			assert ("NaN / 0", attached {like integer_to_be_tested}.arc_tangent (Nan / Zero))
			assert ("NaN / 0", attached {like integer_to_be_tested}.arc_tangent (Nan / Zero))
			assert ("-Infinity / 0", attached {like integer_to_be_tested}.arc_tangent (Negative_infinity / Zero))
			assert ("-Infinity / 0", attached {like integer_to_be_tested}.arc_tangent (Negative_infinity / Zero))
			assert ("Infinity / 0", attached {like integer_to_be_tested}.arc_tangent (Positive_infinity / Zero))
			assert ("Infinity / 0", attached {like integer_to_be_tested}.arc_tangent (Positive_infinity / Zero))
			assert ("0 / 0", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Zero))
			assert ("0 / 0", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Zero))
			assert ("-0 / 0", attached {like integer_to_be_tested}.arc_tangent (-Zero.as_real / Zero))
			assert ("0 / 0", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / Zero))
			assert ("a / 0", attached {like integer_to_be_tested}.arc_tangent (some_real.value / Zero))
			assert ("a / 0", attached {like integer_to_be_tested}.arc_tangent (some_real.value / Zero))

			assert ("-NaN / b", attached {like integer_to_be_tested}.arc_tangent (-Nan / some_real))
			assert ("NaN / b", attached {like integer_to_be_tested}.arc_tangent (Nan / some_real))
			assert ("-Infinity / b", attached {like integer_to_be_tested}.arc_tangent (Negative_infinity / some_real))
			assert ("Infinity / b", attached {like integer_to_be_tested}.arc_tangent (Positive_infinity / some_real))
			assert ("0 / bi", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / some_real))
			assert ("-0 / b", attached {like integer_to_be_tested}.arc_tangent (-Zero.as_real / some_real))
			assert ("0 / b", attached {like integer_to_be_tested}.arc_tangent (Zero.as_real / some_real))
			assert ("a / b", attached {like integer_to_be_tested}.arc_tangent (some_real.value / some_real.value))
		end

	test_value_logb
			-- Test {STS_COMPLEX_NUMBER}.value_logb.
			-- Test {STS_REAL_NUMBER}.value_logb.
			-- Test {STS_INTEGER_NUMBER}.value_logb.
			-- Test {STI_INTEGER_NUMBER}.value_logb.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.value_logb"
			testing: "covers/{STS_REAL_NUMBER}.value_logb"
			testing: "covers/{STS_INTEGER_NUMBER}.value_logb"
			testing: "covers/{STI_INTEGER_NUMBER}.value_logb"
		do
			assert ("-NaN", attached integer_to_be_tested.value_logb (-Nan))
			assert ("NaN", attached integer_to_be_tested.value_logb (Nan))
			assert ("-Infinity", attached integer_to_be_tested.value_logb (Negative_infinity))
			assert ("Infinity", attached integer_to_be_tested.value_logb (Positive_infinity))
			assert ("-0", attached integer_to_be_tested.value_logb (-0.0))
			assert ("0", attached integer_to_be_tested.value_logb (0))
			assert ("x", attached integer_to_be_tested.value_logb (some_native_real))
		end

	test_pi
			-- Test {STS_COMPLEX_NUMBER}.pi.
			-- Test {STS_REAL_NUMBER}.pi.
			-- Test {STS_INTEGER_NUMBER}.pi.
			-- Test {STI_INTEGER_NUMBER}.pi.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi"
			testing: "covers/{STS_REAL_NUMBER}.pi"
			testing: "covers/{STS_INTEGER_NUMBER}.pi"
			testing: "covers/{STI_INTEGER_NUMBER}.pi"
		do
			assert ("pi", attached integer_to_be_tested.Pi)
		end

	test_pi_2
			-- Test {STS_COMPLEX_NUMBER}.pi_2.
			-- Test {STS_REAL_NUMBER}.pi_2.
			-- Test {STS_INTEGER_NUMBER}.pi_2.
			-- Test {STI_INTEGER_NUMBER}.pi_2.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi_2"
			testing: "covers/{STS_REAL_NUMBER}.pi_2"
			testing: "covers/{STS_INTEGER_NUMBER}.pi_2"
			testing: "covers/{STI_INTEGER_NUMBER}.pi_2"
		do
			assert ("pi_2", attached integer_to_be_tested.Pi_2)
		end

	test_pi_4
			-- Test {STS_COMPLEX_NUMBER}.pi_4.
			-- Test {STS_REAL_NUMBER}.pi_4.
			-- Test {STS_INTEGER_NUMBER}.pi_4.
			-- Test {STI_INTEGER_NUMBER}.pi_4.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi_4"
			testing: "covers/{STS_REAL_NUMBER}.pi_4"
			testing: "covers/{STS_INTEGER_NUMBER}.pi_4"
			testing: "covers/{STI_INTEGER_NUMBER}.pi_4"
		do
			assert ("pi_4", attached integer_to_be_tested.Pi_4)
		end

	test_value_is_infinite
			-- Test {STI_INTEGER_NUMBER}.value_is_infinite.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.value_is_infinite"
		local
			v: like some_native_real
		do
			v := Negative_infinity
			assert (
				"Negative_infinity", integer_to_be_tested.value_is_infinite (v)
				)

			v := Positive_infinity
			assert (
				"Positive_infinity", integer_to_be_tested.value_is_infinite (v)
				)

			from
				v := some_native_real
			until
				not (v.is_negative_infinity or v.is_positive_infinity)
			loop
				v := some_native_real
			end
			assert ("finite", not integer_to_be_tested.value_is_infinite (v))

			assert (
				"value_is_infinite",
				integer_to_be_tested.value_is_infinite
					(some_native_real) implies True
				)
		end

	test_value_is_finite
			-- Test {STI_INTEGER_NUMBER}.value_is_finite.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.value_is_finite"
		local
			v: like some_native_real
		do
			v := Nan
			assert ("NaN", not integer_to_be_tested.value_is_finite (v))

			v := Negative_infinity
			assert (
				"Negative_infinity",
				not integer_to_be_tested.value_is_finite (v)
				)

			v := Positive_infinity
			assert (
				"Positive_infinity",
				not integer_to_be_tested.value_is_finite (v)
				)

			from
				v := some_native_real
			until
				not (
					v.is_nan or
					v.is_negative_infinity or
					v.is_positive_infinity
					)
			loop
				v := some_native_real
			end
			assert ("finite", integer_to_be_tested.value_is_finite (v))

			assert (
				"value_is_finite",
				integer_to_be_tested.value_is_finite
					(some_native_real) implies True
				)
		end

	test_hypotenuse
			-- Test {STI_INTEGER_NUMBER}.hypotenuse.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.hypotenuse"
		local
			zs: like Quick_C.subset_anchor
			ab: like some_expanded_complex
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				ab := zs.any
				assert (
					"hypotenuse",
					attached {like integer_to_be_tested}.hypotenuse
						(ab.a, ab.b)
					)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert (
				"-NaN, -NaN",
				attached {like integer_to_be_tested}.hypotenuse (-Nan, -Nan)
				)
			assert (
				"NaN, -NaN",
				attached {like integer_to_be_tested}.hypotenuse (Nan, -Nan)
				)
			assert (
				"-Infinity, -NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(Negative_infinity, -Nan)
				)
			assert (
				"Infinity, -NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(Positive_infinity, -Nan)
				)
			assert (
				"0, -NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, -Nan)
				)
			assert (
				"-0, -NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(-Zero.as_real, -Nan)
				)
			assert (
				"0, -NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, -Nan)
				)
			assert (
				"a, -NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(some_real, -Nan)
				)

			assert (
				"-NaN, NaN",
				attached {like integer_to_be_tested}.hypotenuse (-Nan, Nan)
				)
			assert (
				"NaN, NaN",
				attached {like integer_to_be_tested}.hypotenuse (Nan, Nan)
				)
			assert (
				"-Infinity, NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(Negative_infinity, Nan)
				)
			assert (
				"Infinity, NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(Positive_infinity, Nan)
				)
			assert (
				"NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Nan)
				)
			assert (
				"-0, NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(-Zero.as_real, Nan)
				)
			assert (
				"0, NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Nan)
				)
			assert (
				"a, NaN",
				attached {like integer_to_be_tested}.hypotenuse
					(some_real, Nan)
				)

			assert (
				"-NaN, -Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(-Nan, Negative_infinity)
				)
			assert (
				"NaN, -Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Nan, Negative_infinity)
				)
			assert (
				"-Infinity, -Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Negative_infinity, Negative_infinity)
				)
			assert (
				"Infinity, -Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Positive_infinity, Negative_infinity)
				)
			assert (
				"0, -Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Negative_infinity)
				)
			assert (
				"-0, -Infinity",
				attached {like integer_to_be_tested}.hypotenuse
				(-Zero.as_real, Negative_infinity)
				)
			assert (
				"0, -Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Negative_infinity)
				)
			assert (
				"a, -Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(some_real, Negative_infinity)
				)

			assert (
				"-NaN, Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(-Nan, Positive_infinity)
				)
			assert (
				"NaN, Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Nan, Positive_infinity)
				)
			assert (
				"-Infinity, Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Negative_infinity, Positive_infinity)
				)
			assert (
				"Infinity, Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Positive_infinity, Positive_infinity)
				)
			assert (
				"0, Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Positive_infinity)
				)
			assert (
				"-0, Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(-Zero.as_real, Positive_infinity)
					)
			assert (
				"0, Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Positive_infinity)
				)
			assert (
				"a, Infinity",
				attached {like integer_to_be_tested}.hypotenuse
					(some_real, Positive_infinity)
					)

			assert (
				"-NaN, -0",
				attached {like integer_to_be_tested}.hypotenuse
					(-Nan, -Zero.as_real)
				)
			assert (
				"NaN, -0",
				attached {like integer_to_be_tested}.hypotenuse
					(Nan, -Zero.as_real)
				)
			assert (
				"-Infinity, -0",
				attached {like integer_to_be_tested}.hypotenuse
					(Negative_infinity, -Zero.as_real)
				)
			assert (
				"Infinity, -0",
				attached {like integer_to_be_tested}.hypotenuse
					(Positive_infinity, -Zero.as_real)
				)
			assert (
				"0, -0",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, -Zero.as_real)
				)
			assert (
				"-0, -0",
				attached {like integer_to_be_tested}.hypotenuse
					(-Zero.as_real, -Zero.as_real)
				)
			assert (
				"0, -0",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, -Zero.as_real)
				)
			assert (
				"a, -0",
				attached {like integer_to_be_tested}.hypotenuse
					(some_real, -Zero.as_real)
				)

			assert (
				"-NaN, 0",
				attached {like integer_to_be_tested}.hypotenuse (-Nan, Zero)
				)
			assert (
				"-NaN, 0",
				attached {like integer_to_be_tested}.hypotenuse (-Nan, Zero)
				)
			assert (
				"NaN, 0",
				attached {like integer_to_be_tested}.hypotenuse (Nan, Zero)
				)
			assert (
				"NaN, 0", attached {like integer_to_be_tested}.hypotenuse
					(Nan, Zero)
				)
			assert (
				"-Infinity, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(Negative_infinity, Zero)
				)
			assert (
				"-Infinity, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(Negative_infinity, Zero)
				)
			assert (
				"Infinity, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(Positive_infinity, Zero)
				)
			assert (
				"Infinity, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(Positive_infinity, Zero)
				)
			assert (
				"0, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Zero)
				)
			assert (
				"0, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Zero)
				)
			assert (
				"-0, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(-Zero.as_real, Zero)
				)
			assert (
				"0, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, Zero)
				)
			assert (
				"a, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(some_real, Zero)
				)
			assert (
				"a, 0",
				attached {like integer_to_be_tested}.hypotenuse
					(some_real, Zero)
				)

			assert (
				"-NaN, b",
				attached {like integer_to_be_tested}.hypotenuse
					(-Nan, some_real)
				)
			assert (
				"NaN, b",
				attached {like integer_to_be_tested}.hypotenuse
					(Nan, some_real)
					)
			assert (
				"-Infinity, b",
				attached {like integer_to_be_tested}.hypotenuse
					(Negative_infinity, some_real)
				)
			assert (
				"Infinity, b",
				attached {like integer_to_be_tested}.hypotenuse
					(Positive_infinity, some_real)
				)
			assert (
				"0, bi", attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, some_real)
				)
			assert (
				"-0, b",
				attached {like integer_to_be_tested}.hypotenuse
					(-Zero.as_real, some_real)
				)
			assert (
				"0, b",
				attached {like integer_to_be_tested}.hypotenuse
					(Zero.as_real, some_real)
				)
			assert (
				"a, b",
				attached {like integer_to_be_tested}.hypotenuse
					(some_real, some_real)
				)
		end

	test_arc_tangent_yx
			-- Test {STI_INTEGER_NUMBER}.arc_tangent_yx.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.arc_tangent_yx"
		local
			zs: like Quick_C.subset_anchor
			ab: like some_expanded_complex
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				ab := zs.any
				assert (
					"arc_tangent_yx",
					attached {like integer_to_be_tested}.arc_tangent_yx
						(ab.a, ab.b)
					)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert (
				"-NaN, -NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Nan, -Nan)
				)
			assert (
				"NaN, -NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx (Nan, -Nan)
				)
			assert (
				"-Infinity, -NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Negative_infinity, -Nan)
				)
			assert (
				"Infinity, -NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Positive_infinity, -Nan)
				)
			assert (
				"0, -NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, -Nan)
				)
			assert (
				"-0, -NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, -Nan)
				)
			assert (
				"0, -NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, -Nan)
				)
			assert (
				"a, -NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(some_real, -Nan)
				)

			assert (
				"-NaN, NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx (-Nan, Nan)
				)
			assert (
				"NaN, NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx (Nan, Nan)
				)
			assert (
				"-Infinity, NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Nan)
				)
			assert (
				"Infinity, NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Nan)
				)
			assert (
				"NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Nan)
				)
			assert (
				"-0, NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, Nan)
				)
			assert (
				"0, NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Nan)
				)
			assert (
				"a, NaN",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(some_real, Nan)
				)

			assert (
				"-NaN, -Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Nan, Negative_infinity)
				)
			assert (
				"NaN, -Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Nan, Negative_infinity)
				)
			assert (
				"-Infinity, -Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Negative_infinity)
				)
			assert (
				"Infinity, -Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Negative_infinity)
				)
			assert (
				"0, -Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Negative_infinity)
				)
			assert (
				"-0, -Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
				(-Zero.as_real, Negative_infinity)
				)
			assert (
				"0, -Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Negative_infinity)
				)
			assert (
				"a, -Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(some_real, Negative_infinity)
				)

			assert (
				"-NaN, Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Nan, Positive_infinity)
				)
			assert (
				"NaN, Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Nan, Positive_infinity)
				)
			assert (
				"-Infinity, Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Positive_infinity)
				)
			assert (
				"Infinity, Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Positive_infinity)
				)
			assert (
				"0, Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Positive_infinity)
				)
			assert (
				"-0, Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, Positive_infinity)
					)
			assert (
				"0, Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Positive_infinity)
				)
			assert (
				"a, Infinity",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(some_real, Positive_infinity)
					)

			assert (
				"-NaN, -0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Nan, -Zero.as_real)
				)
			assert (
				"NaN, -0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Nan, -Zero.as_real)
				)
			assert (
				"-Infinity, -0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Negative_infinity, -Zero.as_real)
				)
			assert (
				"Infinity, -0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Positive_infinity, -Zero.as_real)
				)
			assert (
				"0, -0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, -Zero.as_real)
				)
			assert (
				"-0, -0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, -Zero.as_real)
				)
			assert (
				"0, -0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, -Zero.as_real)
				)
			assert (
				"a, -0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(some_real, -Zero.as_real)
				)

			assert (
				"-NaN, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Nan, Zero)
				)
			assert (
				"-NaN, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Nan, Zero)
				)
			assert (
				"NaN, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx (Nan, Zero)
				)
			assert (
				"NaN, 0", attached {like integer_to_be_tested}.arc_tangent_yx
					(Nan, Zero)
				)
			assert (
				"-Infinity, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Zero)
				)
			assert (
				"-Infinity, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Zero)
				)
			assert (
				"Infinity, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Zero)
				)
			assert (
				"Infinity, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Zero)
				)
			assert (
				"0, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Zero)
				)
			assert (
				"0, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Zero)
				)
			assert (
				"-0, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, Zero)
				)
			assert (
				"0, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Zero)
				)
			assert (
				"a, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(some_real, Zero)
				)
			assert (
				"a, 0",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(some_real, Zero)
				)

			assert (
				"-NaN, b",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Nan, some_real)
				)
			assert (
				"NaN, b",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Nan, some_real)
					)
			assert (
				"-Infinity, b",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Negative_infinity, some_real)
				)
			assert (
				"Infinity, b",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Positive_infinity, some_real)
				)
			assert (
				"0, bi", attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, some_real)
				)
			assert (
				"-0, b",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, some_real)
				)
			assert (
				"0, b",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(Zero.as_real, some_real)
				)
			assert (
				"a, b",
				attached {like integer_to_be_tested}.arc_tangent_yx
					(some_real, some_real)
				)
		end

	test_sqrt
			-- Test {STS_REAL_NUMBER}.sqrt.
			-- Test {STS_INTEGER_NUMBER}.sqrt.
			-- Test {STI_INTEGER_NUMBER}.sqrt.
		note
			testing: "covers/{STS_REAL_NUMBER}.sqrt"
			testing: "covers/{STS_INTEGER_NUMBER}.sqrt"
			testing: "covers/{STI_INTEGER_NUMBER}.sqrt"
		do
			assert ("-NaN", attached integer_to_be_tested.sqrt (-Nan))
			assert ("NaN", attached integer_to_be_tested.sqrt (Nan))
			assert ("-Infinity", attached integer_to_be_tested.sqrt (Negative_infinity))
			assert ("Infinity", attached integer_to_be_tested.sqrt (Positive_infinity))
			assert ("-0", attached integer_to_be_tested.sqrt (-0.0))
			assert ("0", attached integer_to_be_tested.sqrt (0))
			assert ("i", attached integer_to_be_tested.sqrt (some_native_real))
		end

	test_splitted
			-- Test {STI_INTEGER_NUMBER}.splitted.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.splitted"
		local
			i_s: like Z.subset_anchor
			i: like integer_to_be_tested
			js: like some_integer_set
			j: like some_integer
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := i_s.any
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					assert ("splitted", attached i.splitted (j))
					assert ("splitted ok", properties.splitted_ok (i, j))
					js := js.others
				variant
					js_cardinality: {like some_set}.natural_as_integer (# js)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_gcd
			-- Test {STI_INTEGER_NUMBER}.gcd.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.gcd"
		local
			i: like integer_to_be_tested
			js, ks, ms: like Z.subset_anchor
			j, k, m: like some_integer
		do
			from
				js := Z
			until
				js.is_empty
			loop
				from
					j := same_integer (js.any)
					ks := Z
				until
					ks.is_empty
				loop
					k := same_integer (ks.any)
					i := integer_to_be_tested
					assert ("gcd", attached i.gcd (j, k))
					from
						ms := Z
					until
						ms.is_empty
					loop
						m := same_integer (ms.any)
						assert ("gcd_ok", properties.gcd_ok (i, j, k, m))
						ms := ms.others
					variant
						ms_cardinality: {like some_set}.natural_as_integer (# ms)
					end
					ks := ks.others
				variant
					ks_cardinality: {like some_set}.natural_as_integer (# ks)
				end
				js := js.others
			variant
				js_cardinality: {like some_set}.natural_as_integer (# js)
			end
		end

	test_div
			-- Test {STI_INTEGER_NUMBER}.div.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.div"
		local
			i: like integer_to_be_tested
			js, ks: like Z.subset_anchor
			j, k: like some_integer
		do
			from
				js := Z
			until
				js.is_empty
			loop
				from
					j := same_integer (js.any)
					ks := Z.without_zero
				until
					ks.is_empty
				loop
					k := same_integer (ks.any)
						check
							good_divisor: j.divisible (k) -- k ∈ Z.without_zero
						end
					i := integer_to_be_tested
					assert ("div", attached i.div (j, k))
					assert ("div_ok", properties.div_ok (i, j, k))
					ks := ks.others
				variant
					ks_cardinality: {like some_set}.natural_as_integer (# ks)
				end
				js := js.others
			variant
				js_cardinality: {like some_set}.natural_as_integer (# js)
			end
		end

	test_rem
			-- Test {STI_INTEGER_NUMBER}.rem.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.rem"
		local
			i: like integer_to_be_tested
			js, ks: like Z.subset_anchor
			j, k: like some_integer
		do
			from
				js := Z
			until
				js.is_empty
			loop
				from
					j := same_integer (js.any)
					ks := Z.without_zero
				until
					ks.is_empty
				loop
					k := same_integer (ks.any)
						check
							good_divisor: j.divisible (k) -- k ∈ Z.without_zero
						end
					i := integer_to_be_tested
					assert ("rem", attached i.rem (j, k))
					assert ("rem_ok", properties.rem_ok (i, j, k, some_integer))
					ks := ks.others
				variant
					ks_cardinality: {like some_set}.natural_as_integer (# ks)
				end
				js := js.others
			variant
				js_cardinality: {like some_set}.natural_as_integer (# js)
			end
		end

feature -- Test routines (Factory)

	test_real_from_value
			-- Test {STI_INTEGER_NUMBER}.real_from_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_from_value"
		do
			assert
				("-NaN", attached integer_to_be_tested.real_from_value (-Nan))
			assert ("NaN", attached integer_to_be_tested.real_from_value (Nan))
			assert (
				"-Infinity",
				attached integer_to_be_tested.real_from_value
					(Negative_infinity)
				)
			assert (
				"Infinity",
				attached integer_to_be_tested.real_from_value
					(Positive_infinity)
				)
			assert ("-0", attached integer_to_be_tested.real_from_value (-0.0))
			assert ("0", attached integer_to_be_tested.real_from_value (0))
			assert (
				"x",
				attached integer_to_be_tested.real_from_value
					(some_native_real)
				)
		end

	test_real_from_double_value
			-- Test {STI_INTEGER_NUMBER}.real_from_double_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_from_double_value"
		do
			assert (
				"-NaN",
				attached integer_to_be_tested.real_from_double_value
					(-Nan.value)
				)
			assert (
				"NaN",
				attached integer_to_be_tested.real_from_double_value
					(Nan.value)
				)
			assert (
				"-Infinity",
				attached integer_to_be_tested.real_from_double_value
					(Negative_infinity.value)
				)
			assert (
				"Infinity",
				attached integer_to_be_tested.real_from_double_value
					(Positive_infinity.value)
				)
			assert (
				"-0",
				attached integer_to_be_tested.real_from_double_value (-0.0)
				)
			assert (
				"0",
				attached integer_to_be_tested.real_from_double_value (0)
				)
			assert (
				"x",
				attached integer_to_be_tested.real_from_double_value
					(some_native_double_real)
				)
		end

	test_native_real_from_double_value
			-- Test {STS_COMPLEX_NUMBER}.native_real_from_double_value.
			-- Test {STS_REAL_NUMBER}.native_real_from_double_value.
			-- Test {STS_RATIONAL_NUMBER}.native_real_from_double_value.
			-- Test {STS_INTEGER_NUMBER}.native_real_from_double_value.
			-- Test {STI_INTEGER_NUMBER}.native_real_from_double_value.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.native_real_from_double_value"
			testing: "covers/{STS_REAL_NUMBER}.native_real_from_double_value"
			testing: "covers/{STS_RATIONAL_NUMBER}.native_real_from_double_value"
			testing: "covers/{STS_INTEGER_NUMBER}.native_real_from_double_value"
			testing: "covers/{STI_INTEGER_NUMBER}.native_real_from_double_value"
		do
			assert ("-NaN", attached integer_to_be_tested.native_real_from_double_value (-Nan.value))
			assert ("NaN", attached integer_to_be_tested.native_real_from_double_value (Nan.value))
			assert ("-Infinity", attached integer_to_be_tested.native_real_from_double_value (Negative_infinity.value))
			assert ("Infinity", attached integer_to_be_tested.native_real_from_double_value (Positive_infinity.value))
			assert ("-0", attached integer_to_be_tested.native_real_from_double_value (-0.0))
			assert ("0", attached integer_to_be_tested.native_real_from_double_value (0))
			assert ("x", attached integer_to_be_tested.native_real_from_double_value (some_native_double_real))
		end

	test_converted_integer
			-- Test {STI_INTEGER_NUMBER}.converted_integer.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.converted_integer"
		local
			js: like Z.subset_anchor
			j: like Z.Any
		do
			from
				js := Z
			until
				js.is_empty
			loop
				j := js.any
				assert (
					"converted_integer",
					attached integer_to_be_tested.converted_integer (j)
					)
				js := js.others
			variant
				cardinality: {like some_set}.natural_as_integer (# js)
			end
		end

feature -- Test routines (Predicate)

	test_integer_product_overflows
			-- Test {STI_INTEGER_NUMBER}.integer_product_overflows.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.integer_product_overflows"
		local
			js, ks: like Z.subset_anchor
			i: like integer_to_be_tested
			j, k: like some_integer
		do
			if next_random_item \\ 2 = 0 then
				k := - some_integer.abs ∧ - One
				if next_random_item \\ 2 = 0 then
					j := (- some_integer.abs ∧ - One) ∧
						({STI_INTEGER_NUMBER}.Max_value // k - One)
				else
					k := k ∧ - Two
					j := (some_integer.abs ∨ One) ∨
						({STI_INTEGER_NUMBER}.Min_value // k + One)
				end
			else
				k := some_integer.abs ∨ One
				if next_random_item \\ 2 = 0 then
					j := (- some_integer.abs ∧ - One) ∧
						({STI_INTEGER_NUMBER}.Min_value // k - One)
				else
					j := (some_integer.abs ∨ One) ∨
						({STI_INTEGER_NUMBER}.Max_value // k + One)
				end
			end
			i := integer_to_be_tested
			assert (
				"product_overflows (j, k)",
				i.integer_product_overflows (j, k)
				)
			assert (
				"product_overflows (j, k) ok",
				properties.integer_product_overflows_ok (i, j, k)
				)

			if next_random_item \\ 2 = 0 then
				k := - some_integer.abs
				j := some_integer
				if k < Zero then
					if next_random_item \\ 2 = 0 then
						j := - j.abs ∨ ({STI_INTEGER_NUMBER}.Max_value // k)
					else
						j := j.abs ∧ ({STI_INTEGER_NUMBER}.Min_value // k)
					end
				end
			else
				k := some_integer.abs
				j := some_integer
				if k > Zero then
					if next_random_item \\ 2 = 0 then
						j := - j.abs ∨ ({STI_INTEGER_NUMBER}.Min_value // k)
					else
						j := j.abs ∧ ({STI_INTEGER_NUMBER}.Max_value // k)
					end
				end
			end
			i := integer_to_be_tested
			assert (
				"not product_overflows (j, k)",
				not i.integer_product_overflows (j, k)
				)
			assert (
				"not product_overflows (j, k) ok",
				properties.integer_product_overflows_ok (i, j, k)
				)

			from
				js := Z
			until
				js.is_empty
			loop
				from
					j := same_integer (js.any)
					ks := Z
				until
					ks.is_empty
				loop
					k := same_integer (ks.any)
					i := integer_to_be_tested
					assert (
						"product_overflows",
						i.integer_product_overflows (j, k) implies True
						)
					assert (
						"integer_product_overflows_ok",
						properties.integer_product_overflows_ok (i, j, k)
						)
					ks := ks.others
				variant
					ks_cardinality: {like some_set}.natural_as_integer (# ks)
				end
				js := js.others
			variant
				js_cardinality: {like some_set}.natural_as_integer (# js)
			end
		rescue
			print (j) print (", ")
			print (k) print ('%N')
		end

feature -- Test routines (Implementation)

	test_value_sign_bit
			-- Test {STS_COMPLEX_NUMBER}.value_sign_bit.
			-- Test {STS_REAL_NUMBER}.value_sign_bit.
			-- Test {STS_INTEGER_NUMBER}.value_sign_bit.
			-- Test {STI_INTEGER_NUMBER}.value_sign_bit.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.value_sign_bit"
			testing: "covers/{STS_REAL_NUMBER}.value_sign_bit"
			testing: "covers/{STS_INTEGER_NUMBER}.value_sign_bit"
			testing: "covers/{STI_INTEGER_NUMBER}.value_sign_bit"
		local
			v: like some_native_real
		do
			v := - Nan
			assert ("signed nan", integer_to_be_tested.value_sign_bit (v) = 1)

			v := Nan
			assert ("unsigned nan", integer_to_be_tested.value_sign_bit (v) = 0)

			from
				v := some_native_real
			until
				not v.is_nan and v < 0
			loop
				v := some_native_real
			end
			assert ("negative", integer_to_be_tested.value_sign_bit (v) = 1)

			v := 0.0
			assert ("negative zero", integer_to_be_tested.value_sign_bit (- v) = 1)
			assert ("positive zero", integer_to_be_tested.value_sign_bit (v) = 0)

			from
				v := some_native_real
			until
				0 < v
			loop
				v := some_native_real
			end
			assert ("positive", integer_to_be_tested.value_sign_bit (v) = 0)

			assert ("-NaN", attached integer_to_be_tested.value_sign_bit (-Nan))
			assert ("NaN", attached integer_to_be_tested.value_sign_bit (Nan))
			assert ("-Infinity", attached integer_to_be_tested.value_sign_bit (Negative_infinity))
			assert ("Infinity", attached integer_to_be_tested.value_sign_bit (Positive_infinity))
			assert ("-0", attached integer_to_be_tested.value_sign_bit (-0.0))
			assert ("0", attached integer_to_be_tested.value_sign_bit (0))
			assert ("x", attached integer_to_be_tested.value_sign_bit (some_native_real))
		end

	test_real_adjusted_value
			-- Test {STI_INTEGER_NUMBER}.real_adjusted_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.real_adjusted_value"
		do
			assert (
				"-NaN",
				attached integer_to_be_tested.real_adjusted_value (-Nan)
				)
			assert (
				"NaN", attached integer_to_be_tested.real_adjusted_value (Nan)
				)
			assert (
				"-Infinity",
				attached integer_to_be_tested.real_adjusted_value
					(Negative_infinity)
				)
			assert (
				"Infinity",
				attached integer_to_be_tested.real_adjusted_value
					(Positive_infinity)
				)
			assert (
				"-0", attached integer_to_be_tested.real_adjusted_value (-0.0)
				)
			assert ("0", attached integer_to_be_tested.real_adjusted_value (0))
			assert (
				"x", attached integer_to_be_tested.real_adjusted_value
				(some_native_real)
				)
		end

	test_adjusted_value
			-- Test {STI_INTEGER_NUMBER}.adjusted_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.adjusted_value"
		do
			assert ("adjusted_value", attached integer_to_be_tested.adjusted_value (next_random_item))
		end

	test_stored_value
			-- Test {STI_INTEGER_NUMBER}.stored_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.stored_value"
		local
			i: like exposed_integer_to_be_tested
			i_s: like Z.subset_anchor
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert ("stored_value", attached i.stored_value)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_native_min_value
			-- Test {STI_INTEGER_NUMBER}.native_min_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.native_min_value"
		do
			assert ("native_min_value", attached exposed_integer_to_be_tested.Native_min_value)
		end

	test_native_max_value
			-- Test {STI_INTEGER_NUMBER}.native_max_value.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.native_max_value"
		do
			assert ("native_max_value", attached exposed_integer_to_be_tested.Native_max_value)
		end

	test_stored_value_bits
			-- Test {STI_INTEGER_NUMBER}.stored_value_bits.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.stored_value_bits"
		do
			assert ("Stored_value_bits", attached exposed_integer_to_be_tested.Stored_value_bits)
		end

	test_stored_value_mask
			-- Test {STI_INTEGER_NUMBER}.stored_value_mask.
		note
			testing: "covers/{STI_INTEGER_NUMBER}.stored_value_mask"
		do
			assert ("Stored_value_mask", attached exposed_integer_to_be_tested.Stored_value_mask)
		end

feature {NONE} -- Factory (element to be tested)

	integer_to_be_tested: like some_expanded_integer
			-- Integer number meant to be under tests
		do
			Result := some_expanded_integer
		end

	exposed_integer_to_be_tested: like some_exposed_integer
			-- Integer number meant to have under tests its features with restricted access
		do
			Result := some_exposed_integer
		end

feature {NONE} -- Conversion

	integer_from_native (v: like some_native_integer): like integer_to_be_tested
			-- `v' converted to an integer number like `integer_to_be_tested'
		do
			Result := v
		ensure
			adjusted_value: Result.value = {like integer_to_be_tested}.adjusted_value (v)
		end

	integer_from_native_integer_ref (v_ref: attached like some_native_integer_ref): like integer_to_be_tested
			-- `v_ref' converted to an integer number like `integer_to_be_tested'
		do
			Result := v_ref
		ensure
			adjusted_value: Result.value = {like integer_to_be_tested}.adjusted_value (v_ref.item)
		end

	native_from_integer (i: STI_INTEGER_NUMBER): like some_native_integer
			-- `i' converted to a native integer number
		do
			Result := i
		ensure
			value: Result = i.value
		end

	integer_from_i (i: STS_INTEGER_NUMBER): like integer_to_be_tested
			-- `i' converted to a integer number like `integer_to_be_tested'
		do
			Result := i
		ensure
			same_integer: Result ≍ i
		end

	integer_from_native_natural (v: like some_native_natural): like integer_to_be_tested
			-- `v' converted to a integer number like `integer_to_be_tested'
		do
			Result := v
		ensure
			adjusted_value: Result.value = {like integer_to_be_tested}.adjusted_value (v.as_integer_32)
		end

	integer_from_native_natural_ref (v_ref: like some_native_natural_ref): like integer_to_be_tested
			-- `v_ref' converted to a integer number like `integer_to_be_tested'
		do
			Result := v_ref
		ensure
			adjusted_value: Result.value = {like integer_to_be_tested}.adjusted_value (v_ref.as_integer_32)
		end

	native_ref_from_integer (i: STI_INTEGER_NUMBER): like some_native_integer_ref
			-- `i' converted to a native integer number reference
		do
			Result := i
		ensure
			value: Result = i.value
		end

	native_real_from_integer (i: STI_INTEGER_NUMBER): like some_native_real
			-- `i' converted to a native real number
		do
			Result := i
		ensure
			value: Result = i.real_value
		end

	native_real_ref_from_integer (i: STI_INTEGER_NUMBER): like some_native_real_ref
			-- `i' converted to a native real number reference
		do
			Result := i
		ensure
			value: Result = i.real_value
		end

	numeric_from_integer (i: STI_INTEGER_NUMBER): NUMERIC
			-- `i' converted to a {NUMERIC} object
		do
			Result := i
		ensure
			value: Result = i.value
		end

	comparable_from_integer (i: STI_INTEGER_NUMBER): COMPARABLE
			-- `i' converted to a {COMPARABLE} object
		do
			Result := i
		ensure
			value: Result = i.value
		end

	hashable_from_integer (i: STI_INTEGER_NUMBER): HASHABLE
			-- `i' converted to a {HASHABLE} object
		do
			Result := i
		ensure
			value: Result = i.value
		end

	complex_from_integer (i: STI_INTEGER_NUMBER): STI_COMPLEX_NUMBER
			-- `i' converted to an {STI_COMPLEX_NUMBER}
		do
			Result := i
		ensure
			real: Result.a ≍ i
			imaginary: Result.b ≍ Zero
		end

	real_from_integer (i: STI_INTEGER_NUMBER): STI_REAL_NUMBER
			-- `i' converted to an {STI_REAL_NUMBER}
		do
			Result := i
		ensure
			definition: Result ≍ i
		end

	rational_from_integer (i: STI_INTEGER_NUMBER): STI_RATIONAL_NUMBER
			-- `i' converted to an {STI_RATIONAL_NUMBER}
		do
			Result := i
		ensure
			definition: Result ≍ i
		end

invariant
	Zero: Zero.value = 0

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
