﻿note
	description: "Test suite for {STI_REAL_NUMBER}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	REAL_NUMBER_TESTS

inherit
	ELEMENT_TESTS
		rename
			test_is_in as test_element_is_in,
			test_is_not_in as test_element_is_not_in,
			element_to_be_tested as real_to_be_tested,
			some_integer_ref as some_native_integer_ref,
			some_direct_integer_ref as some_direct_native_integer_ref,
			new_integer_ref as new_native_integer_ref,
			some_integer as some_native_integer
		undefine
			some_native_integer,
			some_element
		redefine
			on_prepare,
			test_element_is_in,
			test_element_is_not_in,
			real_to_be_tested
		end

	NUMBERS_TESTS_RESOURCES_R
		rename
			Z as Z_universe
		redefine
			on_prepare
		end

feature -- Access

--	zero: STI_REAL_NUMBER
--			-- The real number 0
--		once
--		ensure then
--			class
--			definition: Result.value = 0
--		end

	One: STI_REAL_NUMBER
			-- The real number 1
		once
			Result := {STI_REAL_NUMBER}.One
		ensure then
			class
			definition: Result.value = 1
		end

	properties: REAL_NUMBER_PROPERTIES
			-- Object that checks the set-theory properties of {STI_REAL_NUMBER}
--		once
--			create Result
--		ensure then
--			class
--		end

feature {NONE} -- Events

	on_prepare
			-- <Precursor
		do
			Precursor {NUMBERS_TESTS_RESOURCES_R}
			create properties
		end

feature -- Test routines (Initialization)

	test_default_create
			-- Test {STI_REAL_NUMBER}.default_create.
		note
			testing: "covers/{STI_REAL_NUMBER}.default_create"
		do
			assert ("default_create", attached (create {like real_to_be_tested}))
			assert ("zero", (create {like real_to_be_tested}) ≍ Zero)

			assert ("-NaN", properties.default_create_ok (-Nan))
			assert ("NaN", properties.default_create_ok (Nan))
			assert ("-Infinity", properties.default_create_ok (Negative_infinity))
			assert ("Infinity", properties.default_create_ok (Positive_infinity))
			assert ("-0", properties.default_create_ok (- Zero))
			assert ("0", properties.default_create_ok (Zero))

			assert ("default_create_ok", properties.default_create_ok (some_real))
		end

	test_make
			-- Test {STI_REAL_NUMBER}.make.
		note
			testing: "covers/{STI_REAL_NUMBER}.make"
		do
			assert ("make", attached (create {like real_to_be_tested}.make (some_native_real)))
			assert ("real_from_native", attached real_from_native (some_native_real))
		end

	test_make_from_bit_pattern
			-- Test {STI_REAL_NUMBER}.make_from_bit_pattern.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_bit_pattern"
		local
			bp: like real_to_be_tested.value_bit_pattern
		do
			bp := next_random_item.as_natural_8 & {like exposed_real_to_be_tested}.Value_bit_pattern_mask -- TODO: Make it more general.
			assert ("no_extra_bit", bp & (⊝ {like exposed_real_to_be_tested}.Value_bit_pattern_mask) = 0) -- Assignment above.
			assert ("make_from_bit_pattern", attached (create {like exposed_real_to_be_tested}.make_from_bit_pattern (bp)))
		end

	test_make_from_native_real_ref
			-- Test {STI_REAL_NUMBER}.make_from_native_real_ref.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_native_real_ref"
		do
			assert ("make_from_native_real_ref", attached (create {like real_to_be_tested}.make_from_native_real_ref (some_native_real_ref)))
			assert ("convert_from_native_real_ref", attached real_from_native_ref (some_native_real_ref))
		end

	test_make_from_native_double_real
			-- Test {STI_REAL_NUMBER}.make_from_native_double_real.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_native_double_real"
		do
			assert ("make_from_native_double_real", attached (create {like real_to_be_tested}.make_from_native_double_real (some_native_double_real)))
			assert ("convert_from_native_double_real", attached real_from_native_double_real (some_native_double_real))
		end

	test_make_from_native_double_real_ref
			-- Test {STI_REAL_NUMBER}.make_from_native_double_real_ref.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_native_double_real_ref"
		do
			assert ("make_from_native_double_real_ref", attached (create {like real_to_be_tested}.make_from_native_double_real_ref (some_native_double_real_ref)))
			assert ("convert_from_native_double_real_ref", attached real_from_native_double_real_ref (some_native_double_real_ref))
		end

	test_make_from_real
			-- Test {STI_REAL_NUMBER}.make_from_real.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_real"
		do
			assert ("make_from_real", attached (create {like real_to_be_tested}.make_from_real (some_real)))
			assert ("real_from_x", attached real_from_x (some_real))
		end

	test_make_from_native_integer
			-- Test {STI_REAL_NUMBER}.make_from_native_integer.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_native_integer"
		do
			assert ("make_from_native_integer", attached (create {like real_to_be_tested}.make_from_native_integer (some_native_integer)))
			assert ("convert_from_native_integer", attached real_from_native_integer (some_native_integer))
		end

	test_make_from_native_integer_ref
			-- Test {STI_REAL_NUMBER}.make_from_native_integer_ref.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_native_integer_ref"
		local
			i_ref: like some_native_integer_ref
		do
			from
				i_ref := some_native_integer_ref
			until
				attached i_ref
			loop
				i_ref := some_native_integer_ref
			end
			assert ("make_from_native_integer_ref", attached (create {like real_to_be_tested}.make_from_native_integer_ref (i_ref)))
			assert ("convert_from_native_integer_ref", attached real_from_native_integer_ref (i_ref))
		end

	test_make_from_integer
			-- Test {STI_REAL_NUMBER}.make_from_integer.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_integer"
		do
			assert (
				"make_from_integer",
				attached (
					create {like real_to_be_tested}.make_from_integer
						(some_integer)
					)
				)
			assert
				("real_from_integer", attached real_from_integer (some_integer))
		end

	test_make_from_native_natural
			-- Test {STI_REAL_NUMBER}.make_from_native_natural.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_native_natural"
		do
			assert (
				"make_from_native_natural",
				attached (
					create {like real_to_be_tested}.make_from_native_natural
						(some_native_natural)
					)
				)
			assert (
				"real_from_native_natural",
				attached real_from_native_natural (some_native_natural)
				)
		end

	test_make_from_native_natural_ref
			-- Test {STI_REAL_NUMBER}.make_from_native_natural_ref.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_from_native_natural_ref"
		do
			assert (
				"make_from_native_natural_ref",
				attached (
					create {like real_to_be_tested}.make_from_native_natural_ref
						(some_native_natural_ref)
					)
				)
			assert (
				"real_from_native_natural_ref",
				attached real_from_native_natural_ref (some_native_natural_ref)
				)
		end

	test_make_abs
			-- Test {STI_REAL_NUMBER}.make_abs.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_abs"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("make_abs", attached (create {like real_to_be_tested}.make_abs (x)))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.make_abs_ok (-Nan))
			assert ("NaN", properties.make_abs_ok (Nan))
			assert ("-Infinity", properties.make_abs_ok (Negative_infinity))
			assert ("Infinity", properties.make_abs_ok (Positive_infinity))
			assert ("-0", properties.make_abs_ok (- Zero))
			assert ("0", properties.make_abs_ok (Zero))

			assert ("make_abs_ok", properties.make_abs_ok (real_to_be_tested))
		end

	test_make_opposite
			-- Test {STI_REAL_NUMBER}.make_opposite.
		note
			testing: "covers/{STI_REAL_NUMBER}.make_opposite"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("make_opposite", attached (create {like real_to_be_tested}.make_opposite (x)))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.make_opposite_ok (-Nan))
			assert ("NaN", properties.make_opposite_ok (Nan))
			assert ("-Infinity", properties.make_opposite_ok (Negative_infinity))
			assert ("Infinity", properties.make_opposite_ok (Positive_infinity))
			assert ("-0", properties.make_opposite_ok (- Zero))
			assert ("0", properties.make_opposite_ok (Zero))

			assert ("make_opposite_ok", properties.make_opposite_ok (real_to_be_tested))
		end

feature -- Test routines (Primitive)

	test_value
			-- Test {STI_REAL_NUMBER}.value.
		note
			testing: "covers/{STI_REAL_NUMBER}.value"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("value", attached x.value)
				assert ("native_from_real", attached native_from_real (x))
				assert
					("native_ref_from_real", attached native_ref_from_real (x))
				assert ("numeric_from_real", attached numeric_from_real (x))
				assert
					("comparable_from_real", attached comparable_from_real (x))
				assert ("hashable_from_real", attached hashable_from_real (x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			x := Nan
			assert ("negative NaN", x.value.is_nan and (- x).value = Nan.value)
			assert (
				"negative NaN sign bit ",
				{like real_to_be_tested}.value_sign_bit ((- x).value) = 1
				)

			x := Zero
			assert ("negative zero", (- x).value = 0)
			assert (
				"negative zero sign bit ",
				{like real_to_be_tested}.value_sign_bit ((- x).value) = 1
				)
		end

feature -- Test routines (Membership)

	test_element_is_in
			-- <Precursor>
			-- Test {STS_COMPLEX_NUMBER}.element_is_in.
			-- Test {STS_REAL_NUMBER}.element_is_in.
			-- Test {STI_REAL_NUMBER}.element_is_in.
		note
			testing: "covers/{STS_ELEMENT}.is_in"
			testing: "covers/{STS_COMPLEX_NUMBER}.element_is_in"
			testing: "covers/{STS_REAL_NUMBER}.element_is_in"
			testing: "covers/{STI_REAL_NUMBER}.element_is_in"
		do
			Precursor {ELEMENT_TESTS}
		end

	test_element_is_not_in
			-- <Precursor>
			-- Test {STS_COMPLEX_NUMBER}.element_is_not_in.
			-- Test {STS_REAL_NUMBER}.element_is_not_in.
			-- Test {STI_REAL_NUMBER}.element_is_not_in.
		note
			testing: "covers/{STS_ELEMENT}.is_not_in"
			testing: "covers/{STS_COMPLEX_NUMBER}.element_is_not_in"
			testing: "covers/{STS_REAL_NUMBER}.element_is_not_in"
			testing: "covers/{STI_REAL_NUMBER}.element_is_not_in"
		do
			Precursor {ELEMENT_TESTS}
		end

	test_complex_is_in
			-- Test {STS_COMPLEX_NUMBER}.is_in.
			-- Test {STS_REAL_NUMBER}.complex_is_in.
			-- Test {STI_REAL_NUMBER}.complex_is_in.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_in"
			testing: "covers/{STS_REAL_NUMBER}.complex_is_in"
			testing: "covers/{STI_REAL_NUMBER}.complex_is_in"
		local
			x: like real_to_be_tested
			s: like some_complexes
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			x := real_to_be_tested
			s := some_complex_references & x
			assert ("same_reference", x.complex_is_in (s))
			assert ("same_reference_ok", properties.complex_is_in_ok (x, s))

			x := real_to_be_tested
			s := some_complex_objects & x.twin
			assert ("same_value", x.complex_is_in (s))
			assert ("same_value_ok", properties.complex_is_in_ok (x, s))

			x := real_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_complex_numbers & same_complex (x)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_math_value", x.complex_is_in (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("same_math_value_ok", properties.complex_is_in_ok (x, s))

			x := real_to_be_tested
			s := some_complexes
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("some_complexes", x.complex_is_in (s) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("some_complexes_ok", properties.complex_is_in_ok (x, s))
		end

	test_complex_is_not_in
			-- Test {STS_COMPLEX_NUMBER}.is_not_in.
			-- Test {STS_REAL_NUMBER}.complex_is_not_in.
			-- Test {STI_REAL_NUMBER}.complex_is_not_in.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_not_in"
			testing: "covers/{STS_REAL_NUMBER}.complex_is_not_in"
			testing: "covers/{STI_REAL_NUMBER}.complex_is_not_in"
		local
			x: like real_to_be_tested
			s: like some_complexes
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			x := real_to_be_tested
			s := some_complex_references / x
			assert ("same_reference", x.complex_is_not_in (s))
			assert ("same_reference ok", Properties.complex_is_not_in_ok (x, s))

			x := real_to_be_tested
			s := some_complex_objects / x.twin
			assert ("same_value", x.complex_is_not_in (s))
			assert ("same_value", Properties.complex_is_not_in_ok (x, s))

			x := real_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_complex_numbers / same_complex (x)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_math_value", x.complex_is_not_in (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("same_math_value", Properties.complex_is_not_in_ok (x, s))

			x := real_to_be_tested
			s := some_complexes
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("some_complexes", x.complex_is_not_in (s) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("some_complexes_ok", Properties.complex_is_not_in_ok (x, s))
		end

	test_is_in
			-- Test {STS_REAL_NUMBER}.is_in.
			-- Test {STI_REAL_NUMBER}.is_in.
			--| TODO: Errors due to CATcalls.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_in"
			testing: "covers/{STI_REAL_NUMBER}.is_in"
		local
			x: like real_to_be_tested
			s: like some_reals
		do
			x := real_to_be_tested
			s := some_real_references & x
			assert ("same_reference", x ∈ s)
			assert ("same_reference ok", Properties.is_in_ok (x, s))

			x := real_to_be_tested
			s := some_real_objects & x.twin
			assert ("same_object_value", x ∈ s)
			assert ("same_object_value ok", Properties.is_in_ok (x, s))

			x := real_to_be_tested
			s := some_real_numbers & same_real (x)
			assert ("same_math_value", x ∈ s)
			assert ("same_math_value ok", Properties.is_in_ok (x, s))

			x := real_to_be_tested
			s := some_strict_real_numbers & same_real (x)
			assert ("same_strict_math_value", x ∈ s)
			assert ("same_strict_math_value ok", Properties.is_in_ok (x, s))

			x := real_to_be_tested
			s := some_reals
			assert ("some_reals", x ∈ s implies True)
			assert ("some_reals ok", Properties.is_in_ok (x, s))
		end

	test_is_not_in
			-- Test {STS_REAL_NUMBER}.is_not_in.
			-- Test {STI_REAL_NUMBER}.is_not_in.
			--| TODO: Errors due to CATcalls.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_not_in"
			testing: "covers/{STI_REAL_NUMBER}.is_not_in"
		local
			x: like real_to_be_tested
			s: like some_reals
		do
			x := real_to_be_tested
			s := some_real_references / x
			assert ("same_reference", x ∉ s)
			assert ("same_reference ok", Properties.is_not_in_ok (x, s))

			x := real_to_be_tested
			s := some_real_objects / x.twin
			assert ("same_object_value", x ∉ s)
			assert ("same_object_value ok", Properties.is_not_in_ok (x, s))

			x := real_to_be_tested
			s := some_real_numbers / same_real (x)
			assert ("same_math_value", x ∉ s)
			assert ("same_math_value ok", Properties.is_not_in_ok (x, s))

			x := real_to_be_tested
			s := some_strict_real_numbers / same_real (x)
			assert ("same_strict_math_value", x ∉ s)
			assert ("same_strict_math_value ok", Properties.is_not_in_ok (x, s))

			x := real_to_be_tested
			s := some_reals
			assert ("some_reals", x ∉ s implies True)
			assert ("some_reals ok", Properties.is_not_in_ok (x, s))
		end

feature -- Test routines (Access)

	test_sign
			-- Test {STI_REAL_NUMBER}.sign.
		note
			testing: "covers/{STI_REAL_NUMBER}.sign"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			x := Nan
			assert ("signed NaN", (- x).sign = - One)
			assert ("NaN", x.sign = - One)

			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("x.sign", attached x.sign)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				x < Zero
			loop
				x := real_to_be_tested
			end
			assert ("negative", x.sign ≍ - One)

			x := Zero
			assert ("negative zero", (- x).sign ≍ Zero)
			assert ("positive zero", x.sign ≍ Zero)

			from
				x := real_to_be_tested
			until
				x > Zero
			loop
				x := real_to_be_tested
			end
			assert ("positive", x.sign ≍ One)
		end

	test_sign_bit
			-- Test {STI_REAL_NUMBER}.sign_bit.
		note
			testing: "covers/{STI_REAL_NUMBER}.sign_bit"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			x := Nan
			assert ("signed NaN", (- x).sign_bit ≍ One)
			assert ("unsigned NaN", x.sign_bit ≍ Zero)

			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("x.sign_bit", attached x.sign_bit)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				x < Zero
			loop
				x := real_to_be_tested
			end
			assert ("negative", x.sign_bit ≍ One)

			x := Zero
			assert ("negative zero", (- x).sign_bit ≍ One)
			assert ("positive zero", x.sign_bit ≍ Zero)

			from
				x := real_to_be_tested
			until
				x > Zero
			loop
				x := real_to_be_tested
			end
			assert ("positive", x.sign_bit ≍ Zero)
		end

	test_zero
			-- Test {STI_REAL_NUMBER}.zero.
		note
			testing: "covers/{STI_REAL_NUMBER}.zero"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("zero", attached x.Zero)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN complex_zero_ok", properties.complex_zero_ok (-Nan, some_complex))
			assert ("-NaN", properties.zero_ok (-Nan, some_real))

			assert ("NaN complex_zero_ok", properties.complex_zero_ok (Nan, some_complex))
			assert ("NaN", properties.zero_ok (Nan, some_real))

			assert ("-Infinity complex_zero_ok", properties.complex_zero_ok (Negative_infinity, some_complex))
			assert ("-Infinity", properties.zero_ok (Negative_infinity, some_real))

			assert ("Infinity complex_zero_ok", properties.complex_zero_ok (Positive_infinity, some_complex))
			assert ("Infinity", properties.zero_ok (Positive_infinity, some_real))

			assert ("-0 complex_zero_ok", properties.complex_zero_ok (- Zero, some_complex))
			assert ("-0", properties.zero_ok (- Zero, some_real))

			assert ("0 complex_zero_ok", properties.complex_zero_ok (Zero, some_complex))
			assert ("0", properties.zero_ok (Zero, some_real))

			x := real_to_be_tested
			assert ("complex_zero_ok", properties.complex_zero_ok (x, some_complex))
			assert ("zero_ok", properties.zero_ok (x, some_real))
		end

	test_one
			-- Test {STI_REAL_NUMBER}.one.
		note
			testing: "covers/{STI_REAL_NUMBER}.one"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("one", attached x.One)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN complex_one_ok", properties.complex_one_ok (-Nan, some_complex))
			assert ("-NaN", properties.one_ok (-Nan, some_real))

			assert ("NaN complex_one_ok", properties.complex_one_ok (Nan, some_complex))
			assert ("NaN", properties.one_ok (Nan, some_real))

			assert ("-Infinity complex_one_ok", properties.complex_one_ok (Negative_infinity, some_complex))
			assert ("-Infinity", properties.one_ok (Negative_infinity, some_real))

			assert ("Infinity complex_one_ok", properties.complex_one_ok (Positive_infinity, some_complex))
			assert ("Infinity", properties.one_ok (Positive_infinity, some_real))

			assert ("-0 complex_one_ok", properties.complex_one_ok (- Zero, some_complex))
			assert ("-0", properties.one_ok (- Zero, some_real))

			assert ("0 complex_one_ok", properties.complex_one_ok (Zero, some_complex))
			assert ("0", properties.one_ok (Zero, some_real))

			x := real_to_be_tested
			assert ("complex_one_ok", properties.complex_one_ok (x, some_complex))
			assert ("one_ok", properties.one_ok (x, some_real))
		end

	test_two
			-- Test {STI_REAL_NUMBER}.two.
		note
			testing: "covers/{STI_REAL_NUMBER}.two"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("two", attached x.Two)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			assert ("two_ok", properties.two_ok (real_to_be_tested))
		end

	test_imaginary_unit
			-- Test {STI_REAL_NUMBER}.imaginary_unit.
		note
			testing: "covers/{STI_REAL_NUMBER}.imaginary_unit"
		local
			xs: like R.subset_anchor
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("imaginary_unit", attached x.imaginary_unit)
				assert ("imaginary_unit_ok", properties.imaginary_unit_ok (x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

				-- The outsiders...
			x := -Nan
			assert ("-Nan", attached x.imaginary_unit)
			assert ("-Nan", properties.imaginary_unit_ok (x))

			x := -0.0
			assert ("-0", attached x.imaginary_unit)
			assert ("-0 ok", properties.imaginary_unit_ok (x))
		end

	test_nan
			-- Test {STI_REAL_NUMBER}.nan.
		note
			testing: "covers/{STI_REAL_NUMBER}.nan"
		local
			x: like real_to_be_tested
		do
			x := real_to_be_tested
			assert ("nan", attached x.Nan)

			assert ("-NaN", properties.nan_ok (x, -Nan))
			assert ("NaN", properties.nan_ok (x, Nan))
			assert ("-Infinity", properties.nan_ok (x, Negative_infinity))
			assert ("Infinity", properties.nan_ok (x, Positive_infinity))
			assert ("-0", properties.nan_ok (x, - Zero))
			assert ("0", properties.nan_ok (x, Zero))

			assert ("nan_ok", properties.nan_ok (x, real_to_be_tested))
		end

	test_negative_infinity
			-- Test {STI_REAL_NUMBER}.negative_infinity.
		note
			testing: "covers/{STI_REAL_NUMBER}.negative_infinity"
		local
			x: like real_to_be_tested
		do
			x := real_to_be_tested
			assert ("negative_infinity", attached x.Negative_infinity)

			assert ("-NaN", properties.negative_infinity_ok (x, -Nan))
			assert ("NaN", properties.negative_infinity_ok (x, Nan))
			assert ("-Infinity", properties.negative_infinity_ok (x, Negative_infinity))
			assert ("Infinity", properties.negative_infinity_ok (x, Positive_infinity))
			assert ("-0", properties.negative_infinity_ok (x, - Zero))
			assert ("0", properties.negative_infinity_ok (x, Zero))

			assert ("negative_infinity_ok", properties.negative_infinity_ok (x, real_to_be_tested))
		end

	test_positive_infinity
			-- Test {STI_REAL_NUMBER}.positive_infinity.
		note
			testing: "covers/{STI_REAL_NUMBER}.positive_infinity"
		local
			x: like real_to_be_tested
		do
			x := real_to_be_tested
			assert ("positive_infinity", attached x.Positive_infinity)

			assert ("-NaN", properties.positive_infinity_ok (x, -Nan))
			assert ("NaN", properties.positive_infinity_ok (x, Nan))
			assert ("-Infinity", properties.positive_infinity_ok (x, Negative_infinity))
			assert ("Infinity", properties.positive_infinity_ok (x, Positive_infinity))
			assert ("-0", properties.positive_infinity_ok (x, - Zero))
			assert ("0", properties.positive_infinity_ok (x, Zero))

			assert ("positive_infinity_ok", properties.positive_infinity_ok (x, real_to_be_tested))
		end

	test_min_value
			-- Test {STI_REAL_NUMBER}.Min_value.
		note
			testing: "covers/{STI_REAL_NUMBER}.min_value"
		local
			x: like real_to_be_tested
		do
			x := real_to_be_tested
			assert ("Min_value", attached x.Min_value)
			assert ("Min_value ok", Properties.min_value_ok (x))
		end

	test_max_value
			-- Test {STI_REAL_NUMBER}.Max_value.
		note
			testing: "covers/{STI_REAL_NUMBER}.max_value"
		local
			x: like real_to_be_tested
		do
			x := real_to_be_tested
			assert ("Min_value", attached x.Max_value)
			assert ("Min_value ok", Properties.max_value_ok (x))
		end

	test_machine_epsilon
			-- Test {STI_REAL_NUMBER}.machine_epsilon.
		note
			testing: "covers/{STI_REAL_NUMBER}.machine_epsilon" -- todo: AutoTest gests confused by covers/{STI_REAL_NUMBER}.Machine_epsilon
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("Machine_epsilon", attached x.Machine_epsilon)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			assert ("machine_epsilon_ok", properties.machine_epsilon_ok (real_to_be_tested))
		end

	test_epsilon
			-- Test {STI_REAL_NUMBER}.epsilon.
		note
			testing: "covers/{STI_REAL_NUMBER}.epsilon"
		local
			x: like real_to_be_tested
		do
			x := real_to_be_tested
			assert ("epsilon", attached x.Epsilon)
			assert ("epsilon ok", Properties.epsilon_ok (x))
		end

	test_ulp
			-- Test {STI_REAL_NUMBER}.ulp.
		note
			testing: "covers/{STI_REAL_NUMBER}.ulp"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("ulp", attached x.ulp)
				print (xs.any); print ("; "); print (x.ulp); print ('%N')
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.ulp_ok (-Nan))
			assert ("NaN", properties.ulp_ok (Nan))
			assert ("-Infinity", properties.ulp_ok (Negative_infinity))
			assert ("Infinity", properties.ulp_ok (Positive_infinity))
			assert ("-0", properties.ulp_ok (- Zero))
			assert ("0", properties.ulp_ok (Zero))

			assert ("ulp_ok", properties.ulp_ok (real_to_be_tested))
		end

	test_relative_epsilon
			-- Test {STI_REAL_NUMBER}.relative_epsilon.
		note
			testing: "covers/{STI_REAL_NUMBER}.relative_epsilon"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("relative_epsilon", attached x.relative_epsilon)
				print (xs.any); print ("; "); print (x.relative_epsilon); print ('%N')
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.relative_epsilon_ok (-Nan))
			assert ("NaN", properties.relative_epsilon_ok (Nan))
			assert ("-Infinity", properties.relative_epsilon_ok (Negative_infinity))
			assert ("Infinity", properties.relative_epsilon_ok (Positive_infinity))
			assert ("-0", properties.relative_epsilon_ok (- Zero))
			assert ("0", properties.relative_epsilon_ok (Zero))

			assert ("relative_epsilon_ok", properties.relative_epsilon_ok (real_to_be_tested))
		end

	test_previous_float
			-- Test {STI_REAL_NUMBER}.previous_float.
		note
			testing: "covers/{STI_REAL_NUMBER}.previous_float"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("previous_float", attached x.previous_float)
				print (x); print ("; "); print (x.previous_float); print ('%N')
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_next_float
			-- Test {STI_REAL_NUMBER}.next_float.
		note
			testing: "covers/{STI_REAL_NUMBER}.next_float"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("next_float", attached x.next_float)
				print (x); print ("; "); print (x.next_float); print ('%N')
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

feature -- Test routines (Quality)

	test_is_real
			-- Test {STI_REAL_NUMBER}.is_real.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_real"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_real", x.is_real)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.is_real_ok (-Nan))
			assert ("NaN", properties.is_real_ok (Nan))
			assert ("-Infinity", properties.is_real_ok (Negative_infinity))
			assert ("Infinity", properties.is_real_ok (Positive_infinity))
			assert ("-0", properties.is_real_ok (- Zero))
			assert ("0", properties.is_real_ok (Zero))

			assert ("is_real_ok", properties.is_real_ok (real_to_be_tested))
		end

	test_is_proper_real
			-- Test {STI_REAL_NUMBER}.is_proper_real.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_proper_real"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_proper_real", x.is_proper_real implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.is_proper_real_ok (-Nan))
			assert ("NaN", properties.is_proper_real_ok (Nan))
			assert ("-Infinity", properties.is_proper_real_ok (Negative_infinity))
			assert ("Infinity", properties.is_proper_real_ok (Positive_infinity))
			assert ("-0", properties.is_proper_real_ok (- Zero))
			assert ("0", properties.is_proper_real_ok (Zero))

			assert ("is_proper_real_ok", properties.is_proper_real_ok (real_to_be_tested))
		end

	test_is_imaginary
			-- Test {STI_REAL_NUMBER}.is_imaginary.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_imaginary"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_imaginary", x.is_imaginary implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.is_imaginary_ok (-Nan))
			assert ("NaN", properties.is_imaginary_ok (Nan))
			assert ("-Infinity", properties.is_imaginary_ok (Negative_infinity))
			assert ("Infinity", properties.is_imaginary_ok (Positive_infinity))
			assert ("-0", properties.is_imaginary_ok (- Zero))
			assert ("0", properties.is_imaginary_ok (Zero))

			assert ("is_imaginary_ok", properties.is_imaginary_ok (real_to_be_tested))
		end

	test_is_proper_imaginary
			-- Test {STI_REAL_NUMBER}.is_proper_imaginary.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_proper_imaginary"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_proper_imaginary", not x.is_proper_imaginary)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.is_proper_imaginary_ok (-Nan))
			assert ("NaN", properties.is_proper_imaginary_ok (Nan))
			assert ("-Infinity", properties.is_proper_imaginary_ok (Negative_infinity))
			assert ("Infinity", properties.is_proper_imaginary_ok (Positive_infinity))
			assert ("-0", properties.is_proper_imaginary_ok (- Zero))
			assert ("0", properties.is_proper_imaginary_ok (Zero))

			assert ("is_proper_imaginary_ok", properties.is_proper_imaginary_ok (real_to_be_tested))
		end

	test_is_proper_complex
			-- Test {STI_REAL_NUMBER}.is_proper_complex.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_proper_complex"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_proper_complex", not x.is_proper_complex)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.is_proper_complex_ok (-Nan))
			assert ("NaN", properties.is_proper_complex_ok (Nan))
			assert ("-Infinity", properties.is_proper_complex_ok (Negative_infinity))
			assert ("Infinity", properties.is_proper_complex_ok (Positive_infinity))
			assert ("-0", properties.is_proper_complex_ok (- Zero))
			assert ("0", properties.is_proper_complex_ok (Zero))

			assert ("is_proper_complex_ok", properties.is_proper_complex_ok (real_to_be_tested))
		end

	test_is_nan
			-- Test {STI_REAL_NUMBER}.is_nan.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_nan"
		local
			x: like real_to_be_tested
			l_check: BOOLEAN
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_nan", x.is_nan implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				Negative_infinity ≤ x and x ≤ Positive_infinity
			loop
				x := real_to_be_tested
			end
			assert ("not x.is_nan", not x.is_nan)
			assert ("not x.is_nan ok", properties.is_nan_ok (x, some_real))

			inspect
				next_random_item \\ 16
			when 0 then
				x := Zero / Zero
			when 1 then
				x := Negative_infinity / Negative_infinity
			when 2 then
				x := Negative_infinity / Positive_infinity
			when 3 then
				x := Positive_infinity / Negative_infinity
			when 4 then
				x := Positive_infinity / Positive_infinity
			when 5 then
				x := Positive_infinity + Negative_infinity
			when 6 then
				x := Negative_infinity + Positive_infinity
			when 7 then
				x := Positive_infinity - Positive_infinity
			when 8 then
				x := Negative_infinity - Negative_infinity
			when 9 then
				from
					x := real_to_be_tested
				until
					x < Zero
				loop
					x := real_to_be_tested
				end
					l_check := {ISE_RUNTIME}.check_assert (False)
				x := √¯ x
					l_check := {ISE_RUNTIME}.check_assert (l_check)
			when 10 then
				from
					x := real_to_be_tested
				until
					x < Zero
				loop
					x := real_to_be_tested
				end
				x := {like real_math_anchor}.log (x)
			when 11 then
				from
					x := real_to_be_tested
				until
					x < Zero
				loop
					x := real_to_be_tested
				end
				x := {like real_math_anchor}.log10 (x)
			when 12 then
				from
					x := real_to_be_tested
				until
					x < Zero
				loop
					x := real_to_be_tested
				end
				x := {like real_math_anchor}.log_2 (x)
			when 13 then
				from
					x := real_to_be_tested
				until
					x < - One or x > One
				loop
					x := real_to_be_tested
				end
				x := {like real_math_anchor}.arc_cosine (x)
			when 14 then
				from
					x := real_to_be_tested
				until
					x < - One or x > One
				loop
					x := real_to_be_tested
				end
				x := {like real_math_anchor}.arc_sine (x)
			when 15 then
				x := real_to_be_tested.Nan
			end

			assert ("x.is_nan", x.is_nan)
			assert ("x.is_nan ok", properties.is_nan_ok (x, some_real))

			assert ("-NaN", properties.is_nan_ok (-Nan, some_real))
			assert ("NaN", properties.is_nan_ok (Nan, some_real))
			assert ("-Infinity", properties.is_nan_ok (Negative_infinity, some_real))
			assert ("Infinity", properties.is_nan_ok (Positive_infinity, some_real))
			assert ("-0", properties.is_nan_ok (- Zero, some_real))
			assert ("0", properties.is_nan_ok (Zero, some_real))

			assert ("is_nan_ok", properties.is_nan_ok (real_to_be_tested, some_real))
		end

	test_is_negative_infinity
			-- Test {STI_REAL_NUMBER}.is_negative_infinity.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_negative_infinity"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_negative_infinity", x.is_negative_infinity implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				x.is_nan or Negative_infinity < x
			loop
				x := real_to_be_tested
			end
			assert ("not x.is_negative_infinity", not x.is_negative_infinity)
			assert ("not x.is_negative_infinity ok", properties.is_negative_infinity_ok (x, some_real))

			inspect
				next_random_item \\ 3
			when 0 then
				from
					x := real_to_be_tested
				until
					Negative_infinity ≤ x and x < Zero
				loop
					x := real_to_be_tested
				end
				x := x / Zero
			when 1 then
				x := - Positive_infinity
			when 2 then
				x := Negative_infinity
			end
			assert ("x.is_negative_infinity", x.is_negative_infinity)
			assert ("x.is_negative_infinity ok", properties.is_negative_infinity_ok (x, some_real))

			assert ("-NaN", properties.is_negative_infinity_ok (-Nan, some_real))
			assert ("NaN", properties.is_negative_infinity_ok (Nan, some_real))
			assert ("-Infinity", properties.is_negative_infinity_ok (Negative_infinity, some_real))
			assert ("Infinity", properties.is_negative_infinity_ok (Positive_infinity, some_real))
			assert ("-0", properties.is_negative_infinity_ok (- Zero, some_real))
			assert ("0", properties.is_negative_infinity_ok (Zero, some_real))

			assert ("is_negative_infinity_ok", properties.is_negative_infinity_ok (real_to_be_tested, some_real))
		end

	test_is_negative_zero
			-- Test {STI_REAL_NUMBER}.is_negative_zero.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_negative_zero"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_negative_zero", not x.is_negative_zero)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				x.is_finite and x < Zero
			loop
				x := real_to_be_tested
			end
			x := if next_random_item \\ 2 = 0 then x ⋅ Zero else Zero / x end
			assert ("x.is_negative_zero", x.is_negative_zero)
			assert ("x.is_negative_zero ok", properties.is_negative_zero_ok (x, some_real))

			from
				x := real_to_be_tested
			until
				x ≭ Zero or x.sign_bit ≍ Zero
			loop
				x := real_to_be_tested
			end
			assert ("not x.is_negative_zero", not x.is_negative_zero)
			assert ("not x.is_negative_zero ok", properties.is_negative_zero_ok (x, some_real))

			assert ("-NaN", properties.is_negative_zero_ok (-Nan, some_real))
			assert ("NaN", properties.is_negative_zero_ok (Nan, some_real))
			assert ("-Infinity", properties.is_negative_zero_ok (Negative_infinity, some_real))
			assert ("Infinity", properties.is_negative_zero_ok (Positive_infinity, some_real))
			assert ("-0", properties.is_negative_zero_ok (- Zero, some_real))
			assert ("0", properties.is_negative_zero_ok (Zero, some_real))

			assert ("is_negative_zero_ok", properties.is_negative_zero_ok (real_to_be_tested, some_real))
		end

	test_is_positive_zero
			-- Test {STI_REAL_NUMBER}.is_positive_zero.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_positive_zero"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_positive_zero", x.is_positive_zero implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				Zero < x and x.is_finite
			loop
				x := real_to_be_tested
			end
			x := if next_random_item \\ 2 = 0 then x ⋅ Zero else Zero / x end
			assert ("x.is_positive_zero", x.is_positive_zero)
			assert ("x.is_positive_zero ok", properties.is_positive_zero_ok (x, some_real))

			from
				x := real_to_be_tested
			until
				x ≭ Zero or x.sign_bit ≍ One
			loop
				x := real_to_be_tested
			end
			assert ("not x.is_positive_zero", not x.is_positive_zero)
			assert ("not x.is_positive_zero ok", properties.is_positive_zero_ok (x, some_real))

			assert ("-NaN", properties.is_positive_zero_ok (-Nan, some_real))
			assert ("NaN", properties.is_positive_zero_ok (Nan, some_real))
			assert ("-Infinity", properties.is_positive_zero_ok (Negative_infinity, some_real))
			assert ("Infinity", properties.is_positive_zero_ok (Positive_infinity, some_real))
			assert ("-0", properties.is_positive_zero_ok (- Zero, some_real))
			assert ("0", properties.is_positive_zero_ok (Zero, some_real))

			assert ("is_positive_zero_ok", properties.is_positive_zero_ok (real_to_be_tested, some_real))
		end

	test_is_positive_infinity
			-- Test {STI_REAL_NUMBER}.is_positive_infinity.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_positive_infinity"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_positive_infinity", x.is_positive_infinity implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				x < Positive_infinity
			loop
				x := real_to_be_tested
			end
			assert ("not x.is_positive_infinity", not x.is_positive_infinity)
			assert ("not x.is_positive_infinity ok", properties.is_positive_infinity_ok (x, some_real))

			inspect
				next_random_item \\ 3
			when 0 then
				from
					x := real_to_be_tested
				until
					x > Zero
				loop
					x := real_to_be_tested
				end
				x := x / Zero
			when 1 then
				x := - Negative_infinity
			when 2 then
				x := Positive_infinity
			end
			assert ("x.is_positive_infinity", x.is_positive_infinity)
			assert ("x.is_positive_infinity ok", properties.is_positive_infinity_ok (x, some_real))

			assert ("-NaN", properties.is_positive_infinity_ok (-Nan, some_real))
			assert ("NaN", properties.is_positive_infinity_ok (Nan, some_real))
			assert ("-Infinity", properties.is_positive_infinity_ok (Negative_infinity, some_real))
			assert ("Infinity", properties.is_positive_infinity_ok (Positive_infinity, some_real))
			assert ("-0", properties.is_positive_infinity_ok (- Zero, some_real))
			assert ("0", properties.is_positive_infinity_ok (Zero, some_real))

			assert ("is_positive_infinity_ok", properties.is_positive_infinity_ok (real_to_be_tested, some_real))
		end

	test_is_infinite
			-- Test {STI_REAL_NUMBER}.is_infinite.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_infinite"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_infinite", x.is_infinite implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				x.is_nan or Negative_infinity < x and x < Positive_infinity
			loop
				x := real_to_be_tested
			end
			assert ("not x.is_infinite", not x.is_infinite)
			assert ("not x.complex_is_infinite ok", properties.complex_is_infinite_ok (x, some_complex))
			assert ("not x.is_infinite ok", properties.is_infinite_ok (x, some_real))

			inspect
				next_random_item \\ 3
			when 0 then
				from
					x := real_to_be_tested
				until
					not x.is_nan and x ≭ Zero
				loop
					x := real_to_be_tested
				end
				x := x / Zero
			when 1 then
				x := Negative_infinity
			when 2 then
				x := Positive_infinity
			end
			assert ("x.is_infinite", x.is_infinite)
			assert ("x.complex_is_infinite ok", properties.complex_is_infinite_ok (x, some_complex))
			assert ("x.is_infinite ok", properties.is_infinite_ok (x, some_real))

			assert ("-NaN complex_is_infinite_ok", properties.complex_is_infinite_ok (-Nan, some_complex))
			assert ("-NaN", properties.is_infinite_ok (-Nan, some_real))

			assert ("NaN complex_is_infinite_ok", properties.complex_is_infinite_ok (Nan, some_complex))
			assert ("NaN", properties.is_infinite_ok (Nan, some_real))

			assert ("-Infinity complex_is_infinite_ok", properties.complex_is_infinite_ok (Negative_infinity, some_complex))
			assert ("-Infinity", properties.is_infinite_ok (Negative_infinity, some_real))

			assert ("Infinity complex_is_infinite_ok", properties.complex_is_infinite_ok (Positive_infinity, some_complex))
			assert ("Infinity", properties.is_infinite_ok (Positive_infinity, some_real))

			assert ("-0 complex_is_infinite_ok", properties.complex_is_infinite_ok (- Zero, some_complex))
			assert ("-0", properties.is_infinite_ok (- Zero, some_real))

			assert ("0 complex_is_infinite_ok", properties.complex_is_infinite_ok (Zero, some_complex))
			assert ("0", properties.is_infinite_ok (Zero, some_real))

			x := real_to_be_tested
			assert ("complex_is_infinite_ok", properties.complex_is_infinite_ok (x, some_complex))
			assert ("is_infinite_ok", properties.is_infinite_ok (x, some_real))
		end

	test_is_finite
			-- Test {STI_REAL_NUMBER}.is_finite.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_finite"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_finite", x.is_finite implies True)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			from
				x := real_to_be_tested
			until
				Negative_infinity < x and x < Positive_infinity
			loop
				x := real_to_be_tested
			end
			assert ("x.is_finite", x.is_finite)

			inspect
				next_random_item \\ 3
			when 0 then
				x := Nan
			when 1 then
				x := Negative_infinity
			when 2 then
				x := Positive_infinity
			end
			assert ("not x.is_finite", not x.is_finite)
		end

	test_is_rational
			-- Test {STI_REAL_NUMBER}.is_rational.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_rational"
		local
			x: like real_to_be_tested
			xs: like some_real_set
			xt: like some_real_set.as_tuple
			i: like some_set.cardinality
		do
			xs := R | agent (y: STS_REAL_NUMBER): BOOLEAN
				do
					Result := y.is_rational
				end
			xt := xs.as_tuple
			i := (next_random_item.as_natural_32 \\ xt.n + 1)
				check
					positive: 0 < i -- next_random_item \\ xt.n + 1 > 0
					small_enough: i ≤ xt.n -- next_random_item \\ xt.n + 1 ≤ xt.n
				end
			x := xt [i]
			assert ("x.is_rational", x.is_rational)
			assert ("x.is_rational ok", properties.is_rational_ok (x))

			xs := R ∖ xs
			xt := xs.as_tuple
			i := (next_random_item.as_natural_32 \\ xt.n + 1)
				check
					positive: 0 < i -- next_random_item \\ xt.n + 1 > 0
					small_enough: i ≤ xt.n -- next_random_item \\ xt.n + 1 ≤ xt.n
				end
			x := xt [i]
			assert ("not x.is_rational", not x.is_rational)
			assert ("not x.is_rational ok", properties.is_rational_ok (x))

			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_rational", x.is_rational implies True)
				assert ("is_rational ok", properties.is_rational_ok (x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_is_integer
			-- Test {STS_REAL_NUMBER}.is_integer.
			-- Test {STI_REAL_NUMBER}.is_integer.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_integer"
			testing: "covers/{STI_REAL_NUMBER}.is_integer"
		local
			x: like real_to_be_tested
			xs: like some_real_set
			xt: like some_real_set.as_tuple
			i: like some_set.cardinality
		do
			xs := R | agent {STS_REAL_NUMBER}.is_integer -- TODO: Take Z instead.
			xt := xs.as_tuple
			i := (next_random_item.as_natural_32 \\ xt.n + 1)
				check
					positive: 0 < i -- next_random_item \\ xt.n + 1 > 0
					small_enough: i ≤ xt.n -- next_random_item \\ xt.n + 1 ≤ xt.n
				end
			x := xt [i]
			assert ("x.is_integer", x.is_integer)

			xs := R ∖ xs
			xt := xs.as_tuple
			i := (next_random_item.as_natural_32 \\ xt.n + 1)
				check
					positive: 0 < i -- next_random_item \\ xt.n + 1 > 0
					small_enough: i ≤ xt.n -- next_random_item \\ xt.n + 1 ≤ xt.n
				end
			x := xt [i]
			assert ("not x.is_integer", not x.is_integer)

			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_integer", x.is_integer implies True)
				assert ("is_integer ok", properties.is_integer_ok (x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_is_invertible
			-- Test {STI_REAL_NUMBER}.is_invertible.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_invertible"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("is_invertible", x.is_invertible)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

feature -- Test routines (Output)

	test_out
			-- Test {STI_REAL_NUMBER}.out.
		note
			testing: "covers/{STI_REAL_NUMBER}.out"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("out", attached x.out)
				print (x.out) print ('%N')
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

feature -- Test routines (Comparison)

	test_complex_equals
			-- Test {STS_COMPLEX_NUMBER}.equals.
			-- Test {STS_REAL_NUMBER}.complex_equals.
			-- Test {STI_REAL_NUMBER}.complex_equals.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.equals"
			testing: "covers/{STS_REAL_NUMBER}.complex_equals"
			testing: "covers/{STI_REAL_NUMBER}.complex_equals"
		local
			x: like real_to_be_tested
			z: like some_complex
		do
			x := real_to_be_tested
			assert ("same_entity", x.complex_equals (x))
			assert ("same_entity_ok", properties.complex_equals_ok (x, x, x))

			x := real_to_be_tested
			z := same_complex (x)
			assert ("same_complex", x.complex_equals (z))
			assert ("same_complex_ok", properties.complex_equals_ok (x, z, same_complex (z)))

			assert ("-NaN", properties.complex_equals_ok (-Nan, some_complex, some_complex))
			assert ("NaN", properties.complex_equals_ok (Nan, some_complex, some_complex))
			assert ("-Infinity", properties.complex_equals_ok (Negative_infinity, some_complex, some_complex))
			assert ("Infinity", properties.complex_equals_ok (Positive_infinity, some_complex, some_complex))
			assert ("-0", properties.complex_equals_ok (- Zero, some_complex, some_complex))
			assert ("0", properties.complex_equals_ok (Zero, some_complex, some_complex))

			assert ("complex_equals_ok", properties.complex_equals_ok (real_to_be_tested, some_complex, some_complex))
		end

	test_complex_unequals
			-- Test {STS_COMPLEX_NUMBER}.unequals.
			-- Test {STS_REAL_NUMBER}.complex_unequals.
			-- Test {STI_REAL_NUMBER}.complex_unequals.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.unequals"
			testing: "covers/{STS_REAL_NUMBER}.complex_unequals"
			testing: "covers/{STI_REAL_NUMBER}.complex_unequals"
		local
			x: like real_to_be_tested
			z: like some_complex
		do
			x := real_to_be_tested
			assert ("same_entity", not x.complex_unequals (x))
			assert ("same_entity_ok", properties.complex_unequals_ok (x, x))

			x := real_to_be_tested
			z := same_complex (x)
			assert ("same_complex", not x.complex_unequals (z))
			assert ("same_complex_ok", properties.complex_unequals_ok (x, z))

			assert ("-NaN", properties.complex_unequals_ok (-Nan, some_complex))
			assert ("NaN", properties.complex_unequals_ok (Nan, some_complex))
			assert ("-Infinity", properties.complex_unequals_ok (Negative_infinity, some_complex))
			assert ("Infinity", properties.complex_unequals_ok (Positive_infinity, some_complex))
			assert ("-0", properties.complex_unequals_ok (- Zero, some_complex))
			assert ("0", properties.complex_unequals_ok (Zero, some_complex))

			assert ("complex_unequals_ok", properties.complex_unequals_ok (real_to_be_tested, some_complex))
		end

	test_equals
			-- Test {STI_REAL_NUMBER}.equals.
		note
			testing: "covers/{STI_REAL_NUMBER}.equals"
		local
			x: like real_to_be_tested
			y: like some_real
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
		do

			x := real_to_be_tested
			assert ("same_entity", x ≍ x)
			assert ("same_entity_ok", properties.equals_ok (x, x, x))

			x := real_to_be_tested
			y := same_real (x)
			assert ("same_real", x ≍ y)
			assert ("same_real_ok", properties.equals_ok (x, y, same_real (y)))

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				x := z.a
				y := same_real (z.b)
				assert ("some_real", x ≍ y implies True)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.equals_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.equals_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.equals_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.equals_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.equals_ok (- Zero, some_real, some_real))
			assert ("0", properties.equals_ok (Zero, some_real, some_real))

			assert ("equals_ok", properties.equals_ok (real_to_be_tested, some_real, some_real))
		end

	test_unequals
			-- Test {STS_REAL_NUMBER}.unequals.
			-- Test {STI_REAL_NUMBER}.unequals.
		note
			testing: "covers/{STS_REAL_NUMBER}.unequals"
			testing: "covers/{STI_REAL_NUMBER}.unequals"
		local
			x: like real_to_be_tested
			y: like some_real
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
		do
			x := real_to_be_tested
			assert ("same_entity", not (x ≭ x))
			assert ("same_entity ok", Properties.unequals_ok (x, x))

			x := real_to_be_tested
			y := same_real (x)
			assert ("same_real", not (x ≭ y))
			assert ("same_real ok", Properties.unequals_ok (x, y))

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				x := z.a
				y := same_real (z.b)
				assert ("some_real", x ≭ y implies True)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.unequals_ok (-Nan, some_real))
			assert ("NaN", properties.unequals_ok (Nan, some_real))
			assert ("-Infinity", properties.unequals_ok (Negative_infinity, some_real))
			assert ("Infinity", properties.unequals_ok (Positive_infinity, some_real))
			assert ("-0", properties.unequals_ok (- Zero, some_real))
			assert ("0", properties.unequals_ok (Zero, some_real))

			assert ("unequals_ok", properties.unequals_ok (real_to_be_tested, some_real))
		end

	test_is_less
			-- Test {STI_REAL_NUMBER}.is_less.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_less"
		local
			x: like real_to_be_tested
			y, z: like some_real
			zs: like Quick_C.subset_anchor
		do
			from
				z := some_real
			until
				z.is_nan or z > Zero
			loop
				z := some_real
			end
			from
				y := some_real
			until
				y.is_negative_infinity and z.is_nan or Negative_infinity < y
			loop
				y := some_real
			end
			x := y - (z ∨ y.ulp)
			assert ("x < y", x < y)
			assert ("x < y ok", properties.is_less_ok (x, y, some_real))

			from
				z := some_real
			until
				z ≥ Zero
			loop
				z := some_real
			end
			y := some_real
			x := y + z
			assert ("not (x < y)", not (x < y))
			assert ("not (x < y) ok", properties.is_less_ok (x, y, some_real))

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				x := zs.any.a
				y := same_real (zs.any.b)
				assert ("is_less", x < y implies True)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.is_less_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.is_less_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.is_less_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.is_less_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.is_less_ok (- Zero, some_real, some_real))
			assert ("0", properties.is_less_ok (Zero, some_real, some_real))

			assert ("is_less_ok", properties.is_less_ok (real_to_be_tested, some_real, some_real))
		end

	test_is_less_equal
			-- Test {STI_REAL_NUMBER}.is_less_equal.
		note
			testing: "covers/{STI_REAL_NUMBER}.is_less_equal"
		local
			x: like real_to_be_tested
			y, z: like some_real
			zs: like Quick_C.subset_anchor
		do
			from
				z := some_real
			until
				z ≥ Zero
			loop
				z := some_real
			end
			y := some_real
			x := y - z
			assert ("x ≤ y", x ≤ y)
			assert ("x ≤ y ok", properties.is_less_equal_ok (x, y, some_real))

			from
				z := some_real
			until
				z > Zero
			loop
				z := some_real
			end
			from
				y := some_real
			until
				y.is_finite
			loop
				y := some_real
			end
			x := y + z.max (y.relative_epsilon)
			assert ("not (x ≤ y)", not (x ≤ y))
			assert ("not (x ≤ y) ok", properties.is_less_equal_ok (x, y, some_real))

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				x := zs.any.a
				y := same_real (zs.any.b)
				assert ("is_less_equal", x ≤ y implies True)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.is_less_equal_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.is_less_equal_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.is_less_equal_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.is_less_equal_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.is_less_equal_ok (- Zero, some_real, some_real))
			assert ("0", properties.is_less_equal_ok (Zero, some_real, some_real))

			assert ("is_less_equal_ok", properties.is_less_equal_ok (real_to_be_tested, some_real, some_real))
		end

	test_is_greater
			-- Test {STS_REAL_NUMBER}.is_greater.
			-- Test {STI_REAL_NUMBER}.is_greater.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_greater"
			testing: "covers/{STI_REAL_NUMBER}.is_greater"
		local
			x: like real_to_be_tested
			y, z: like some_real
			zs: like Quick_C.subset_anchor
		do
			from
				z := some_real
			until
				z > Zero
			loop
				z := some_real
			end
			from
				y := some_real
			until
				y.is_finite
			loop
				y := some_real
			end
			x := y + (z ∨ y.ulp)
			assert ("x > y", x > y)
			assert ("x > y ok", properties.is_greater_ok (x, y, some_real))

			from
				z := some_real
			until
				z ≥ Zero
			loop
				z := some_real
			end
			y := some_real
			x := y - (z ∨ y.ulp)
			assert ("not (x > y)", not (x > y))
			assert ("not (x > y) ok", properties.is_greater_ok (x, y, some_real))

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				x := zs.any.a
				y := same_real (zs.any.b)
				assert ("is_greater", x > y implies True)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.is_greater_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.is_greater_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.is_greater_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.is_greater_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.is_greater_ok (- Zero, some_real, some_real))
			assert ("0", properties.is_greater_ok (Zero, some_real, some_real))

			assert ("is_greater_ok", properties.is_greater_ok (real_to_be_tested, some_real, some_real))
		end

	test_is_greater_equal
			-- Test {STS_REAL_NUMBER}.is_greater_equal.
			-- Test {STI_REAL_NUMBER}.is_greater_equal.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_greater_equal"
			testing: "covers/{STI_REAL_NUMBER}.is_greater_equal"
		local
			x: like real_to_be_tested
			y, z: like some_real
			zs: like Quick_C.subset_anchor
		do
			z := some_real.abs ∨ Zero
			y := some_real
			x := y + z
			assert ("x ≥ y", x ≥ y)
			assert ("x ≥ y ok", properties.is_greater_equal_ok (x, y, some_real))

			y := (some_real ∨ {STI_REAL_NUMBER}.Min_value) ∧ {STI_REAL_NUMBER}.Max_value
			z := some_real.abs ∨ y.ulp
			x := y - z
			assert ("not (x ≥ y)", not (x ≥ y))
			assert ("not (x ≥ y) ok", properties.is_greater_equal_ok (x, y, some_real))

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				x := zs.any.a
				y := same_real (zs.any.b)
				assert ("is_greater_equal", x ≥ y implies True)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.is_greater_equal_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.is_greater_equal_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.is_greater_equal_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.is_greater_equal_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.is_greater_equal_ok (- Zero, some_real, some_real))
			assert ("0", properties.is_greater_equal_ok (Zero, some_real, some_real))

			assert ("is_greater_equal_ok", properties.is_greater_equal_ok (real_to_be_tested, some_real, some_real))
		end

	test_three_way_comparison
			-- Test {STI_REAL_NUMBER}.three_way_comparison.
		note
			testing: "covers/{STI_REAL_NUMBER}.three_way_comparison"
		local
			x: like real_to_be_tested
			y, z: like some_real
			zs: like Quick_C.subset_anchor
		do
			from
				y := some_real
				z := some_real.abs
			until
				y.is_finite and z ≥ y.ulp
			loop
				y := some_real
				z := some_real.abs
			end
			x := y - z
			assert ("negative", x ⋚ y ≍ - One)

			assert ("same_entity", x ⋚ x ≍ Zero)
			assert ("same_real", x ⋚ same_real (x) ≍ Zero)

			from
				y := some_real
				z := some_real.abs
			until
				y.is_finite and z ≥ y.ulp
			loop
				y := some_real
				z := some_real.abs
			end
			x := y + z
			assert ("positive", x ⋚ y ≍ One)

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				x := zs.any.a
				y := same_real (zs.any.b)
				assert ("three_way_comparison", attached (x ⋚ y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end
		end

	test_min
			-- Test {STI_REAL_NUMBER}.min.
		note
			testing: "covers/{STI_REAL_NUMBER}.min"
		local
			x: like real_to_be_tested
			y, z: like some_real
			zs: like Quick_C.subset_anchor
		do
			from
				z := some_real
			until
				z ≥ Zero
			loop
				z := some_real
			end
			y := some_real
			x := y - z
			assert ("x", (x ∧ y) ≍ x)
			assert ("x ok", Properties.min_ok (x, y, some_real))

			from
				z := some_real
			until
				z ≥ Zero
			loop
				z := some_real
			end
			y := real_to_be_tested
			x := y + z
			assert ("y", (x ∧ y) ≍ y)
			assert ("y ok", Properties.min_ok (x, y, some_real))

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				x := zs.any.a
				y := same_real (zs.any.b)
				assert ("min", attached (x ∧ y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.min_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.min_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.min_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.min_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.min_ok (- Zero, some_real, some_real))
			assert ("0", properties.min_ok (Zero, some_real, some_real))

			assert ("min_ok", properties.min_ok (real_to_be_tested, some_real, some_real))
		end

	test_max
			-- Test {STI_REAL_NUMBER}.max.
		note
			testing: "covers/{STI_REAL_NUMBER}.max"
		local
			x: like real_to_be_tested
			y, z: like some_real
			zs: like Quick_C.subset_anchor
		do
			from
				z := some_real
			until
				z ≥ Zero
			loop
				z := some_real
			end
			y := real_to_be_tested
			x := y + z
			assert ("x", (x ∨ y) ≍ x)
			assert ("x ok", Properties.max_ok (x, y, some_real))

			from
				z := some_real
			until
				z ≥ Zero
			loop
				z := some_real
			end
			y := some_real
			x := y - z
			assert ("y", (x ∨ y) ≍ y)
			assert ("y ok", Properties.max_ok (x, y, some_real))

			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				x := zs.any.a
				y := same_real (zs.any.b)
				assert ("max", attached (x ∨ y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.max_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.max_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.max_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.max_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.max_ok (- Zero, some_real, some_real))
			assert ("0", properties.max_ok (Zero, some_real, some_real))

			assert ("max_ok", properties.max_ok (real_to_be_tested, some_real, some_real))
		end

feature -- Test routines (Relationship)

	test_complex_divisible
			-- Test {STS_COMPLEX_NUMBER}.divisible.
			-- Test {STS_REAL_NUMBER}.complex_divisible.
			-- Test {STI_REAL_NUMBER}.complex_divisible.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.divisible"
			testing: "covers/{STS_REAL_NUMBER}.complex_divisible"
			testing: "covers/{STI_REAL_NUMBER}.complex_divisible"
		local
			xs: like some_real_set
			zs: like Quick_C.subset_anchor
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				from
					zs := Quick_C
				until
					zs.is_empty
				loop
					x := xs.any
					assert ("complex_divisible", x.complex_divisible (zs.any))
					zs := zs.others
				variant
					cardinality: {like some_set}.natural_as_integer (# zs)
				end
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_divisible
			-- Test {STS_REAL_NUMBER}.divisible.
			-- Test {STI_REAL_NUMBER}.divisible.
		note
			testing: "covers/{STS_REAL_NUMBER}.divisible"
			testing: "covers/{STI_REAL_NUMBER}.divisible"
		local
			x: like real_to_be_tested
			y: like some_real
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				x := z.a
				y := same_real (z.b)
				assert ("divisible", x.divisible (y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end
		end

feature -- Test routines (Operation)

	test_argument
			-- Test {STI_REAL_NUMBER}.argument.
		note
			testing: "covers/{STI_REAL_NUMBER}.argument"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("argument", attached x.argument)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.argument_ok (-Nan))
			assert ("NaN", properties.argument_ok (Nan))
			assert ("-Infinity", properties.argument_ok (Negative_infinity))
			assert ("Infinity", properties.argument_ok (Positive_infinity))
			assert ("-0", properties.argument_ok (- Zero))
			assert ("0", properties.argument_ok (Zero))

			assert ("argument_ok", properties.argument_ok (real_to_be_tested))
		end

	test_phase
			-- Test {STI_REAL_NUMBER}.phase.
		note
			testing: "covers/{STI_REAL_NUMBER}.phase"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("phase", attached x.phase)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.phase_ok (-Nan))
			assert ("NaN", properties.phase_ok (Nan))
			assert ("-Infinity", properties.phase_ok (Negative_infinity))
			assert ("Infinity", properties.phase_ok (Positive_infinity))
			assert ("-0", properties.phase_ok (- Zero))
			assert ("0", properties.phase_ok (Zero))

			assert ("phase_ok", properties.phase_ok (real_to_be_tested))
		end

	test_conjugate
			-- Test {STI_REAL_NUMBER}.conjugate.
		note
			testing: "covers/{STI_REAL_NUMBER}.conjugate"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("conjugate", attached x.conjugate)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.conjugate_ok (-Nan, some_complex))
			assert ("NaN", properties.conjugate_ok (Nan, some_complex))
			assert ("-Infinity", properties.conjugate_ok (Negative_infinity, some_complex))
			assert ("Infinity", properties.conjugate_ok (Positive_infinity, some_complex))
			assert ("-0", properties.conjugate_ok (- Zero, some_complex))
			assert ("0", properties.conjugate_ok (Zero, some_complex))

			assert ("conjugate_ok", properties.conjugate_ok (real_to_be_tested, some_complex))
		end

	test_complex_plus
			-- Test {STI_REAL_NUMBER}.complex_plus.
		note
			testing: "covers/{STI_REAL_NUMBER}.complex_plus"
		local
			xs: like some_real_set
			zt0, zt: like Quick_C.as_tuple
			x: like real_to_be_tested
			z: like some_complex
		do
			from
				xs := R
				from
					zt0 := Quick_C.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # xs <= 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# xs * zt0.n) print ('%N')
			until
				xs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					x := xs.any
					z := same_complex (zt.first)
					assert ("complex_plus", attached x.complex_plus (z))
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.complex_plus_ok (-Nan, some_complex))
			assert ("NaN", properties.complex_plus_ok (Nan, some_complex))
			assert ("-Infinity", properties.complex_plus_ok (Negative_infinity, some_complex))
			assert ("Infinity", properties.complex_plus_ok (Positive_infinity, some_complex))
			assert ("-0", properties.complex_plus_ok (- Zero, some_complex))
			assert ("0", properties.complex_plus_ok (Zero, some_complex))

			assert ("complex_plus_ok", properties.complex_plus_ok (real_to_be_tested, some_complex))
		end

	test_complex_minus
			-- Test {STI_REAL_NUMBER}.complex_minus.
		note
			testing: "covers/{STI_REAL_NUMBER}.complex_minus"
		local
			xs: like some_real_set
			zt0, zt: like Quick_C.as_tuple
			x: like real_to_be_tested
			z: like some_complex
		do
			from
				xs := R
				from
					zt0 := Quick_C.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # xs <= 83_374 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# xs * zt0.n) print ('%N')
			until
				xs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					x := xs.any
					z := same_complex (zt.first)
					assert ("complex_minus", attached x.complex_minus (z))
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.complex_minus_ok (-Nan, some_complex))
			assert ("NaN", properties.complex_minus_ok (Nan, some_complex))
			assert ("-Infinity", properties.complex_minus_ok (Negative_infinity, some_complex))
			assert ("Infinity", properties.complex_minus_ok (Positive_infinity, some_complex))
			assert ("-0", properties.complex_minus_ok (- Zero, some_complex))
			assert ("0", properties.complex_minus_ok (Zero, some_complex))

			assert ("complex_minus_ok", properties.complex_minus_ok (real_to_be_tested, some_complex))
		end

	test_complex_product
			-- Test {STI_REAL_NUMBER}.complex_product.
		note
			testing: "covers/{STI_REAL_NUMBER}.complex_product"
		local
			xs: like some_real_set
			zt0, zt: like Quick_C.as_tuple
			x: like real_to_be_tested
			z: like some_complex
		do
			from
				xs := R
				from
					zt0 := Quick_C.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # xs <= 83_575 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# xs * zt0.n) print ('%N')
			until
				xs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					x := xs.any
					z := same_complex (zt.first)
					assert ("complex_product", attached x.complex_product (z))
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.complex_product_ok (-Nan, some_complex, some_complex))
			assert ("NaN", properties.complex_product_ok (Nan, some_complex, some_complex))
			assert ("-Infinity", properties.complex_product_ok (Negative_infinity, some_complex, some_complex))
			assert ("Infinity", properties.complex_product_ok (Positive_infinity, some_complex, some_complex))
			assert ("-0", properties.complex_product_ok (- Zero, some_complex, some_complex))
			assert ("0", properties.complex_product_ok (Zero, some_complex, some_complex))

			assert ("complex_product_ok", properties.complex_product_ok (real_to_be_tested, some_complex, some_complex))
		end

	test_complex_quotient
			-- Test {STI_REAL_NUMBER}.complex_quotient.
		note
			testing: "covers/{STI_REAL_NUMBER}.complex_quotient"
		local
			xs: like some_real_set
			zt0, zt: like Quick_c.as_tuple
			x: like real_to_be_tested
			z: like some_complex
		do
			from
				xs := R
				from
					zt0 := Quick_c.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # xs <= 71_011 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# xs * zt0.n) print ('%N')
			until
				xs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					x := xs.any
					z := same_complex (zt.first)
					assert ("complex_quotient", attached x.complex_quotient (z))
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.complex_quotient_ok (-Nan, some_complex, some_complex))
			assert ("NaN", properties.complex_quotient_ok (Nan, some_complex, some_complex))
			assert ("-Infinity", properties.complex_quotient_ok (Negative_infinity, some_complex, some_complex))
			assert ("Infinity", properties.complex_quotient_ok (Positive_infinity, some_complex, some_complex))
			assert ("-0", properties.complex_quotient_ok (- Zero, some_complex, some_complex))
			assert ("0", properties.complex_quotient_ok (Zero, some_complex, some_complex))

			assert ("complex_quotient_ok", properties.complex_quotient_ok (real_to_be_tested, some_complex, some_complex))
		end

	test_modulus
			-- Test {STI_REAL_NUMBER}.modulus.
		note
			testing: "covers/{STI_REAL_NUMBER}.modulus"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("modulus", attached x.modulus)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.modulus_ok (-Nan))
			assert ("NaN", properties.modulus_ok (Nan))
			assert ("-Infinity", properties.modulus_ok (Negative_infinity))
			assert ("Infinity", properties.modulus_ok (Positive_infinity))
			assert ("-0", properties.modulus_ok (- Zero))
			assert ("0", properties.modulus_ok (Zero))

			assert ("modulus_ok", properties.modulus_ok (real_to_be_tested))
		end

	test_abs
			-- Test {STI_REAL_NUMBER}.abs.
		note
			testing: "covers/{STI_REAL_NUMBER}.abs"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("abs", attached x.abs)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.abs_ok (-Nan))
			assert ("NaN", properties.abs_ok (Nan))
			assert ("-Infinity", properties.abs_ok (Negative_infinity))
			assert ("Infinity", properties.abs_ok (Positive_infinity))
			assert ("-0", properties.abs_ok (- Zero))
			assert ("0", properties.abs_ok (Zero))

			assert ("abs_ok", properties.abs_ok (real_to_be_tested))
		end

	test_plus
			-- Test {STS_REAL_NUMBER}.plus.
			-- Test {STI_REAL_NUMBER}.plus.
		note
			testing: "covers/{STS_REAL_NUMBER}.plus"
			testing: "covers/{STI_REAL_NUMBER}.plus"
		local
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
			x: like real_to_be_tested
			y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				x := z.a
				y := same_real (z.b)
				assert ("plus", attached (x + y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.plus_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.plus_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.plus_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.plus_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.plus_ok (- Zero, some_real, some_real))
			assert ("0", properties.plus_ok (Zero, some_real, some_real))

			assert ("plus_ok", properties.plus_ok (real_to_be_tested, some_real, some_real))
		end

	test_identity
			-- Test {STS_COMPLEX_NUMBER}.identity.
			-- Test {STS_REAL_NUMBER}.identity.
			-- Test {STI_REAL_NUMBER}.identity.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.identity"
			testing: "covers/{STS_REAL_NUMBER}.identity"
			testing: "covers/{STI_REAL_NUMBER}.identity"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("identity", attached (+ x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.identity_ok (-Nan, some_complex))
			assert ("NaN", properties.identity_ok (Nan, some_complex))
			assert ("-Infinity", properties.identity_ok (Negative_infinity, some_complex))
			assert ("Infinity", properties.identity_ok (Positive_infinity, some_complex))
			assert ("-0", properties.identity_ok (- Zero, some_complex))
			assert ("0", properties.identity_ok (Zero, some_complex))

			assert ("identity_ok", properties.identity_ok (real_to_be_tested, some_complex))
		end

	test_minus
			-- Test {STS_REAL_NUMBER}.minus.
			-- Test {STI_REAL_NUMBER}.minus.
		note
			testing: "covers/{STS_REAL_NUMBER}.minus"
			testing: "covers/{STI_REAL_NUMBER}.minus"
		local
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
			x: like real_to_be_tested
			y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				x := z.a
				y := same_real (z.b)
				assert ("minus", attached (x - y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.minus_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.minus_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.minus_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.minus_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.minus_ok (- Zero, some_real, some_real))
			assert ("0", properties.minus_ok (Zero, some_real, some_real))

			assert ("minus_ok", properties.minus_ok (real_to_be_tested, some_real, some_real))
		end

	test_opposite
			-- Test {STI_REAL_NUMBER}.opposite.
		note
			testing: "covers/{STI_REAL_NUMBER}.opposite"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("opposite", attached (- x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.opposite_ok (-Nan))
			assert ("NaN", properties.opposite_ok (Nan))
			assert ("-Infinity", properties.opposite_ok (Negative_infinity))
			assert ("Infinity", properties.opposite_ok (Positive_infinity))
			assert ("-0", properties.opposite_ok (- Zero))
			assert ("0", properties.opposite_ok (Zero))

			assert ("opposite_ok", properties.opposite_ok (real_to_be_tested))
		end

	test_product
			-- Test {STS_REAL_NUMBER}.product.
			-- Test {STI_REAL_NUMBER}.product.
		note
			testing: "covers/{STS_REAL_NUMBER}.product"
			testing: "covers/{STI_REAL_NUMBER}.product"
		local
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
			x: like real_to_be_tested
			y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				x := z.a
				y := same_real (z.b)
				assert ("product", attached (x ⋅ y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.product_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.product_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.product_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.product_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.product_ok (- Zero, some_real, some_real))
			assert ("0", properties.product_ok (Zero, some_real, some_real))

			assert ("product_ok", properties.product_ok (real_to_be_tested, some_real, some_real))
		end

	test_quotient
			-- Test {STI_REAL_NUMBER}.quotient.
		note
			testing: "covers/{STI_REAL_NUMBER}.quotient"
		local
			x: like real_to_be_tested
			y: like some_real
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				x := z.a
				y := same_real (z.b)
				assert ("quotient", x.divisible (y) implies attached (x / y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.quotient_ok (-Nan, some_real, some_real))
			assert ("NaN", properties.quotient_ok (Nan, some_real, some_real))
			assert ("-Infinity", properties.quotient_ok (Negative_infinity, some_real, some_real))
			assert ("Infinity", properties.quotient_ok (Positive_infinity, some_real, some_real))
			assert ("-0", properties.quotient_ok (- Zero, some_real, some_real))
			assert ("0", properties.quotient_ok (Zero, some_real, some_real))

			assert ("quotient_ok", properties.quotient_ok (real_to_be_tested, some_real, some_real))
		end

	test_reciprocal
			-- Test {STI_REAL_NUMBER}.reciprocal.
		note
			testing: "covers/{STI_REAL_NUMBER}.reciprocal"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("reciprocal", attached x.reciprocal)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN complex_reciprocal_ok", properties.complex_reciprocal_ok (-Nan, some_complex))
			assert ("-NaN", properties.reciprocal_ok (-Nan, some_real))

			assert ("NaN complex_reciprocal_ok", properties.complex_reciprocal_ok (Nan, some_complex))
			assert ("NaN", properties.reciprocal_ok (Nan, some_real))

			assert ("-Infinity complex_reciprocal_ok", properties.complex_reciprocal_ok (Negative_infinity, some_complex))
			assert ("-Infinity", properties.reciprocal_ok (Negative_infinity, some_real))

			assert ("Infinity complex_reciprocal_ok", properties.complex_reciprocal_ok (Positive_infinity, some_complex))
			assert ("Infinity", properties.reciprocal_ok (Positive_infinity, some_real))

			assert ("-0 complex_reciprocal_ok", properties.complex_reciprocal_ok (- Zero, some_complex))
			assert ("-0", properties.reciprocal_ok (- Zero, some_real))

			assert ("0 complex_reciprocal_ok", properties.complex_reciprocal_ok (Zero, some_complex))
			assert ("0", properties.reciprocal_ok (Zero, some_real))

			x := real_to_be_tested
			assert ("complex_reciprocal_ok", properties.complex_reciprocal_ok (x, some_complex))
			assert ("reciprocal_ok", properties.reciprocal_ok (x, some_real))
		end

	test_inverse
			-- Test {STI_REAL_NUMBER}.inverse.
		note
			testing: "covers/{STI_REAL_NUMBER}.inverse"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("inverse", attached x.inverse)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN complex_inverse_ok", properties.complex_inverse_ok (-Nan, some_complex))
			assert ("-NaN", properties.inverse_ok (-Nan, some_real))

			assert ("NaN complex_inverse_ok", properties.complex_inverse_ok (Nan, some_complex))
			assert ("NaN", properties.inverse_ok (Nan, some_real))

			assert ("-Infinity complex_inverse_ok", properties.complex_inverse_ok (Negative_infinity, some_complex))
			assert ("-Infinity", properties.inverse_ok (Negative_infinity, some_real))

			assert ("Infinity complex_inverse_ok", properties.complex_inverse_ok (Positive_infinity, some_complex))
			assert ("Infinity", properties.inverse_ok (Positive_infinity, some_real))

			assert ("-0 complex_inverse_ok", properties.complex_inverse_ok (- Zero, some_complex))
			assert ("-0", properties.inverse_ok (- Zero, some_real))

			assert ("0 complex_inverse_ok", properties.complex_inverse_ok (Zero, some_complex))
			assert ("0", properties.inverse_ok (Zero, some_real))

			x := real_to_be_tested
			assert ("complex_inverse_ok", properties.complex_inverse_ok (x, some_complex))
			assert ("inverse_ok", properties.inverse_ok (x, some_real))
		end

	test_power
			-- Test {STS_REAL_NUMBER}.power.
			-- Test {STI_REAL_NUMBER}.power.
		note
			testing: "covers/{STS_REAL_NUMBER}.power"
			testing: "covers/{STI_REAL_NUMBER}.power"
		local
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
			x: like real_to_be_tested
			y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				x := z.a
				y := same_real (z.b)
				assert ("power", attached (x ^ y))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN", properties.power_ok (-Nan))
			assert ("NaN", properties.power_ok (Nan))
			assert ("-Infinity", properties.power_ok (Negative_infinity))
			assert ("Infinity", properties.power_ok (Positive_infinity))
			assert ("-0", properties.power_ok (- Zero))
			assert ("0", properties.power_ok (Zero))

			assert ("power_ok", properties.power_ok (real_to_be_tested))
		end

	test_square_root
			-- Test {STS_REAL_NUMBER}.square_root.
			-- Test {STI_REAL_NUMBER}.square_root.
		note
			testing: "covers/{STS_REAL_NUMBER}.square_root"
			testing: "covers/{STI_REAL_NUMBER}.square_root"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("square_root", x ≥ Zero implies attached √¯ x)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.square_root_ok (-Nan))
			assert ("NaN", properties.square_root_ok (Nan))
			assert ("-Infinity", properties.square_root_ok (Negative_infinity))
			assert ("Infinity", properties.square_root_ok (Positive_infinity))
			assert ("-0", properties.square_root_ok (- Zero))
			assert ("0", properties.square_root_ok (Zero))

			assert ("square_root_ok", properties.square_root_ok (real_to_be_tested))
		end

feature -- Test routines (Conversion)

	test_as_complex
			-- Test {STI_REAL_NUMBER}.as_complex.
		note
			testing: "covers/{STI_REAL_NUMBER}.as_complex"
		local
			x: like real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("as_complex", attached x.as_complex)
				assert ("convert_to_complex", attached complex_from_real (x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			x := real_to_be_tested
			assert ("as_complex_ok", properties.as_complex_ok (x))
		end

	test_to_rational
			-- Test {STI_REAL_NUMBER}.to_rational.
		note
			testing: "covers/{STI_REAL_NUMBER}.to_rational"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("to_rational", x.is_rational implies attached x.to_rational)
				assert ("to_rational ok", properties.to_rational_ok (x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_truncated_to_integer
			-- Test {STI_REAL_NUMBER}.truncated_to_integer.
		note
			testing: "covers/{STI_REAL_NUMBER}.truncated_to_integer"
		local
			xs: like some_real_set
			x: like real_to_be_tested
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("truncated_to_integer", attached x.truncated_to_integer)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

feature -- Test routines (Math)

	test_pi
			-- Test {STS_COMPLEX_NUMBER}.pi.
			-- Test {STS_REAL_NUMBER}.pi.
			-- Test {STI_REAL_NUMBER}.pi.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi"
			testing: "covers/{STS_REAL_NUMBER}.pi"
			testing: "covers/{STI_REAL_NUMBER}.pi"
		do
			assert ("pi", attached real_to_be_tested.Pi)
		end

	test_pi_2
			-- Test {STS_COMPLEX_NUMBER}.pi_2.
			-- Test {STS_REAL_NUMBER}.pi_2.
			-- Test {STI_REAL_NUMBER}.pi_2.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi_2"
			testing: "covers/{STS_REAL_NUMBER}.pi_2"
			testing: "covers/{STI_REAL_NUMBER}.pi_2"
		do
			assert ("pi_2", attached real_to_be_tested.Pi_2)
		end

	test_pi_4
			-- Test {STS_COMPLEX_NUMBER}.pi_4.
			-- Test {STS_REAL_NUMBER}.pi_4.
			-- Test {STI_REAL_NUMBER}.pi_4.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi_4"
			testing: "covers/{STS_REAL_NUMBER}.pi_4"
			testing: "covers/{STI_REAL_NUMBER}.pi_4"
		do
			assert ("pi_4", attached real_to_be_tested.Pi_4)
		end

	test_value_is_infinite
			-- Test {STI_REAL_NUMBER}.value_is_infinite.
		note
			testing: "covers/{STI_REAL_NUMBER}.value_is_infinite"
		local
			v: like some_native_real
		do
			v := Negative_infinity
			assert ("Negative_infinity", real_to_be_tested.value_is_infinite (v))

			v := Positive_infinity
			assert ("Positive_infinity", real_to_be_tested.value_is_infinite (v))

			from
				v := some_native_real
			until
				not (v.is_negative_infinity or v.is_positive_infinity)
			loop
				v := some_native_real
			end
			assert ("finite", not real_to_be_tested.value_is_infinite (v))

			assert ("value_is_infinite", real_to_be_tested.value_is_infinite (some_native_real) implies True)
		end

	test_value_is_finite
			-- Test {STI_REAL_NUMBER}.value_is_finite.
		note
			testing: "covers/{STI_REAL_NUMBER}.value_is_finite"
		local
			v: like some_native_real
		do
			v := Nan
			assert ("NaN", not real_to_be_tested.value_is_finite (v))

			v := Negative_infinity
			assert ("Negative_infinity", not real_to_be_tested.value_is_finite (v))

			v := Positive_infinity
			assert ("Positive_infinity", not real_to_be_tested.value_is_finite (v))

			from
				v := some_native_real
			until
				not (v.is_nan or v.is_negative_infinity or v.is_positive_infinity)
			loop
				v := some_native_real
			end
			assert ("finite", real_to_be_tested.value_is_finite (v))

			assert ("value_is_finite", real_to_be_tested.value_is_finite (some_native_real) implies True)
		end

	test_hypotenuse
			-- Test {STI_REAL_NUMBER}.hypotenuse.
		note
			testing: "covers/{STI_REAL_NUMBER}.hypotenuse"
		local
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("hypotenuse", attached {like real_to_be_tested}.hypotenuse (z.a, z.b))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN, -NaN", attached {like real_to_be_tested}.hypotenuse (-Nan, -Nan))
			assert ("NaN, -NaN", attached {like real_to_be_tested}.hypotenuse (Nan, -Nan))
			assert ("-Infinity, -NaN", attached {like real_to_be_tested}.hypotenuse (Negative_infinity, -Nan))
			assert ("Infinity, -NaN", attached {like real_to_be_tested}.hypotenuse (Positive_infinity, -Nan))
			assert ("0, -NaN", attached {like real_to_be_tested}.hypotenuse (Zero, -Nan))
			assert ("-0, -NaN", attached {like real_to_be_tested}.hypotenuse (-Zero, -Nan))
			assert ("0, -NaN", attached {like real_to_be_tested}.hypotenuse (Zero, -Nan))
			assert ("a, -NaN", attached {like real_to_be_tested}.hypotenuse (some_real, -Nan))

			assert ("-NaN, NaN", attached {like real_to_be_tested}.hypotenuse (-Nan, Nan))
			assert ("NaN, NaN", attached {like real_to_be_tested}.hypotenuse (Nan, Nan))
			assert ("-Infinity, NaN", attached {like real_to_be_tested}.hypotenuse (Negative_infinity, Nan))
			assert ("Infinity, NaN", attached {like real_to_be_tested}.hypotenuse (Positive_infinity, Nan))
			assert ("NaN", attached {like real_to_be_tested}.hypotenuse (Zero, Nan))
			assert ("-0, NaN", attached {like real_to_be_tested}.hypotenuse (-Zero, Nan))
			assert ("0, NaN", attached {like real_to_be_tested}.hypotenuse (Zero, Nan))
			assert ("a, NaN", attached {like real_to_be_tested}.hypotenuse (some_real, Nan))

			assert ("-NaN, -Infinity", attached {like real_to_be_tested}.hypotenuse (-Nan, Negative_infinity))
			assert ("NaN, -Infinity", attached {like real_to_be_tested}.hypotenuse (Nan, Negative_infinity))
			assert ("-Infinity, -Infinity", attached {like real_to_be_tested}.hypotenuse (Negative_infinity, Negative_infinity))
			assert ("Infinity, -Infinity", attached {like real_to_be_tested}.hypotenuse (Positive_infinity, Negative_infinity))
			assert ("0, -Infinity", attached {like real_to_be_tested}.hypotenuse (Zero, Negative_infinity))
			assert ("-0, -Infinity", attached {like real_to_be_tested}.hypotenuse (-Zero, Negative_infinity))
			assert ("0, -Infinity", attached {like real_to_be_tested}.hypotenuse (Zero, Negative_infinity))
			assert ("a, -Infinity", attached {like real_to_be_tested}.hypotenuse (some_real, Negative_infinity))

			assert ("-NaN, Infinity", attached {like real_to_be_tested}.hypotenuse (-Nan, Positive_infinity))
			assert ("NaN, Infinity", attached {like real_to_be_tested}.hypotenuse (Nan, Positive_infinity))
			assert ("-Infinity, Infinity", attached {like real_to_be_tested}.hypotenuse (Negative_infinity, Positive_infinity))
			assert ("Infinity, Infinity", attached {like real_to_be_tested}.hypotenuse (Positive_infinity, Positive_infinity))
			assert ("0, Infinity", attached {like real_to_be_tested}.hypotenuse (Zero, Positive_infinity))
			assert ("-0, Infinity", attached {like real_to_be_tested}.hypotenuse (-Zero, Positive_infinity))
			assert ("0, Infinity", attached {like real_to_be_tested}.hypotenuse (Zero, Positive_infinity))
			assert ("a, Infinity", attached {like real_to_be_tested}.hypotenuse (some_real, Positive_infinity))

			assert ("-NaN, -0", attached {like real_to_be_tested}.hypotenuse (-Nan, -Zero))
			assert ("NaN, -0", attached {like real_to_be_tested}.hypotenuse (Nan, -Zero))
			assert ("-Infinity, -0", attached {like real_to_be_tested}.hypotenuse (Negative_infinity, -Zero))
			assert ("Infinity, -0", attached {like real_to_be_tested}.hypotenuse (Positive_infinity, -Zero))
			assert ("0, -0", attached {like real_to_be_tested}.hypotenuse (Zero, -Zero))
			assert ("-0, -0", attached {like real_to_be_tested}.hypotenuse (-Zero, -Zero))
			assert ("0, -0", attached {like real_to_be_tested}.hypotenuse (Zero, -Zero))
			assert ("a, -0", attached {like real_to_be_tested}.hypotenuse (some_real, -Zero))

			assert ("-NaN, 0", attached {like real_to_be_tested}.hypotenuse (-Nan, Zero))
			assert ("-NaN, 0", attached {like real_to_be_tested}.hypotenuse (-Nan, Zero))
			assert ("NaN, 0", attached {like real_to_be_tested}.hypotenuse (Nan, Zero))
			assert ("NaN, 0", attached {like real_to_be_tested}.hypotenuse (Nan, Zero))
			assert ("-Infinity, 0", attached {like real_to_be_tested}.hypotenuse (Negative_infinity, Zero))
			assert ("-Infinity, 0", attached {like real_to_be_tested}.hypotenuse (Negative_infinity, Zero))
			assert ("Infinity, 0", attached {like real_to_be_tested}.hypotenuse (Positive_infinity, Zero))
			assert ("Infinity, 0", attached {like real_to_be_tested}.hypotenuse (Positive_infinity, Zero))
			assert ("0, 0", attached {like real_to_be_tested}.hypotenuse (Zero, Zero))
			assert ("0, 0", attached {like real_to_be_tested}.hypotenuse (Zero, Zero))
			assert ("-0, 0", attached {like real_to_be_tested}.hypotenuse (-Zero, Zero))
			assert ("0, 0", attached {like real_to_be_tested}.hypotenuse (Zero, Zero))
			assert ("a, 0", attached {like real_to_be_tested}.hypotenuse (some_real, Zero))
			assert ("a, 0", attached {like real_to_be_tested}.hypotenuse (some_real, Zero))

			assert ("-NaN, b", attached {like real_to_be_tested}.hypotenuse (-Nan, some_real))
			assert ("NaN, b", attached {like real_to_be_tested}.hypotenuse (Nan, some_real))
			assert ("-Infinity, b", attached {like real_to_be_tested}.hypotenuse (Negative_infinity, some_real))
			assert ("Infinity, b", attached {like real_to_be_tested}.hypotenuse (Positive_infinity, some_real))
			assert ("0, bi", attached {like real_to_be_tested}.hypotenuse (Zero, some_real))
			assert ("-0, b", attached {like real_to_be_tested}.hypotenuse (-Zero, some_real))
			assert ("0, b", attached {like real_to_be_tested}.hypotenuse (Zero, some_real))
			assert ("a, b", attached {like real_to_be_tested}.hypotenuse (some_real, some_real))
		end

	test_arc_tangent_yx
			-- Test {STI_REAL_NUMBER}.arc_tangent_yx.
		note
			testing: "covers/{STI_REAL_NUMBER}.arc_tangent_yx"
		local
			zs: like Quick_C.subset_anchor
			z: like some_expanded_complex
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("arc_tangent_yx", attached {like real_to_be_tested}.arc_tangent_yx (z.b, z.a))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert ("-NaN, -NaN", attached {like real_to_be_tested}.arc_tangent_yx (-Nan, -Nan))
			assert ("NaN, -NaN", attached {like real_to_be_tested}.arc_tangent_yx (Nan, -Nan))
			assert ("-Infinity, -NaN", attached {like real_to_be_tested}.arc_tangent_yx (Negative_infinity, -Nan))
			assert ("Infinity, -NaN", attached {like real_to_be_tested}.arc_tangent_yx (Positive_infinity, -Nan))
			assert ("0, -NaN", attached {like real_to_be_tested}.arc_tangent_yx (Zero, -Nan))
			assert ("-0, -NaN", attached {like real_to_be_tested}.arc_tangent_yx (-Zero, -Nan))
			assert ("0, -NaN", attached {like real_to_be_tested}.arc_tangent_yx (Zero, -Nan))
			assert ("a, -NaN", attached {like real_to_be_tested}.arc_tangent_yx (some_real, -Nan))

			assert ("-NaN, NaN", attached {like real_to_be_tested}.arc_tangent_yx (-Nan, Nan))
			assert ("NaN, NaN", attached {like real_to_be_tested}.arc_tangent_yx (Nan, Nan))
			assert ("-Infinity, NaN", attached {like real_to_be_tested}.arc_tangent_yx (Negative_infinity, Nan))
			assert ("Infinity, NaN", attached {like real_to_be_tested}.arc_tangent_yx (Positive_infinity, Nan))
			assert ("NaN", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Nan))
			assert ("-0, NaN", attached {like real_to_be_tested}.arc_tangent_yx (-Zero, Nan))
			assert ("0, NaN", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Nan))
			assert ("a, NaN", attached {like real_to_be_tested}.arc_tangent_yx (some_real, Nan))

			assert ("-NaN, -Infinity", attached {like real_to_be_tested}.arc_tangent_yx (-Nan, Negative_infinity))
			assert ("NaN, -Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Nan, Negative_infinity))
			assert ("-Infinity, -Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Negative_infinity, Negative_infinity))
			assert ("Infinity, -Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Positive_infinity, Negative_infinity))
			assert ("0, -Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Negative_infinity))
			assert ("-0, -Infinity", attached {like real_to_be_tested}.arc_tangent_yx (-Zero, Negative_infinity))
			assert ("0, -Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Negative_infinity))
			assert ("a, -Infinity", attached {like real_to_be_tested}.arc_tangent_yx (some_real, Negative_infinity))

			assert ("-NaN, Infinity", attached {like real_to_be_tested}.arc_tangent_yx (-Nan, Positive_infinity))
			assert ("NaN, Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Nan, Positive_infinity))
			assert ("-Infinity, Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Negative_infinity, Positive_infinity))
			assert ("Infinity, Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Positive_infinity, Positive_infinity))
			assert ("0, Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Positive_infinity))
			assert ("-0, Infinity", attached {like real_to_be_tested}.arc_tangent_yx (-Zero, Positive_infinity))
			assert ("0, Infinity", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Positive_infinity))
			assert ("a, Infinity", attached {like real_to_be_tested}.arc_tangent_yx (some_real, Positive_infinity))

			assert ("-NaN, -0", attached {like real_to_be_tested}.arc_tangent_yx (-Nan, -Zero))
			assert ("NaN, -0", attached {like real_to_be_tested}.arc_tangent_yx (Nan, -Zero))
			assert ("-Infinity, -0", attached {like real_to_be_tested}.arc_tangent_yx (Negative_infinity, -Zero))
			assert ("Infinity, -0", attached {like real_to_be_tested}.arc_tangent_yx (Positive_infinity, -Zero))
			assert ("0, -0", attached {like real_to_be_tested}.arc_tangent_yx (Zero, -Zero))
			assert ("-0, -0", attached {like real_to_be_tested}.arc_tangent_yx (-Zero, -Zero))
			assert ("0, -0", attached {like real_to_be_tested}.arc_tangent_yx (Zero, -Zero))
			assert ("a, -0", attached {like real_to_be_tested}.arc_tangent_yx (some_real, -Zero))

			assert ("-NaN, 0", attached {like real_to_be_tested}.arc_tangent_yx (-Nan, Zero))
			assert ("-NaN, 0", attached {like real_to_be_tested}.arc_tangent_yx (-Nan, Zero))
			assert ("NaN, 0", attached {like real_to_be_tested}.arc_tangent_yx (Nan, Zero))
			assert ("NaN, 0", attached {like real_to_be_tested}.arc_tangent_yx (Nan, Zero))
			assert ("-Infinity, 0", attached {like real_to_be_tested}.arc_tangent_yx (Negative_infinity, Zero))
			assert ("-Infinity, 0", attached {like real_to_be_tested}.arc_tangent_yx (Negative_infinity, Zero))
			assert ("Infinity, 0", attached {like real_to_be_tested}.arc_tangent_yx (Positive_infinity, Zero))
			assert ("Infinity, 0", attached {like real_to_be_tested}.arc_tangent_yx (Positive_infinity, Zero))
			assert ("0, 0", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Zero))
			assert ("0, 0", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Zero))
			assert ("-0, 0", attached {like real_to_be_tested}.arc_tangent_yx (-Zero, Zero))
			assert ("0, 0", attached {like real_to_be_tested}.arc_tangent_yx (Zero, Zero))
			assert ("a, 0", attached {like real_to_be_tested}.arc_tangent_yx (some_real, Zero))
			assert ("a, 0", attached {like real_to_be_tested}.arc_tangent_yx (some_real, Zero))

			assert ("-NaN, b", attached {like real_to_be_tested}.arc_tangent_yx (-Nan, some_real))
			assert ("NaN, b", attached {like real_to_be_tested}.arc_tangent_yx (Nan, some_real))
			assert ("-Infinity, b", attached {like real_to_be_tested}.arc_tangent_yx (Negative_infinity, some_real))
			assert ("Infinity, b", attached {like real_to_be_tested}.arc_tangent_yx (Positive_infinity, some_real))
			assert ("0, bi", attached {like real_to_be_tested}.arc_tangent_yx (Zero, some_real))
			assert ("-0, b", attached {like real_to_be_tested}.arc_tangent_yx (-Zero, some_real))
			assert ("0, b", attached {like real_to_be_tested}.arc_tangent_yx (Zero, some_real))
			assert ("a, b", attached {like real_to_be_tested}.arc_tangent_yx (some_real, some_real))
		end

	test_arc_tangent
			-- Test {STS_COMPLEX_NUMBER}.arc_tangent.
			-- Test {STS_REAL_NUMBER}.arc_tangent.
			-- Test {STI_REAL_NUMBER}.arc_tangent.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.arc_tangent"
			testing: "covers/{STS_REAL_NUMBER}.arc_tangent"
			testing: "covers/{STI_REAL_NUMBER}.arc_tangent"
		do
			assert ("-NaN / -NaN", attached {like real_to_be_tested}.arc_tangent (-Nan / -Nan))
			assert ("NaN / -NaN", attached {like real_to_be_tested}.arc_tangent (Nan / -Nan))
			assert ("-Infinity / -NaN", attached {like real_to_be_tested}.arc_tangent (Negative_infinity / -Nan))
			assert ("Infinity / -NaN", attached {like real_to_be_tested}.arc_tangent (Positive_infinity / -Nan))
			assert ("0 / -NaN", attached {like real_to_be_tested}.arc_tangent (Zero / -Nan))
			assert ("-0 / -NaN", attached {like real_to_be_tested}.arc_tangent (-Zero / -Nan))
			assert ("0 / -NaN", attached {like real_to_be_tested}.arc_tangent (Zero / -Nan))
			assert ("a / -NaN", attached {like real_to_be_tested}.arc_tangent (some_real.value / -Nan))

			assert ("-NaN / NaN", attached {like real_to_be_tested}.arc_tangent (-Nan / Nan))
			assert ("NaN / NaN", attached {like real_to_be_tested}.arc_tangent (Nan / Nan))
			assert ("-Infinity / NaN", attached {like real_to_be_tested}.arc_tangent (Negative_infinity / Nan))
			assert ("Infinity / NaN", attached {like real_to_be_tested}.arc_tangent (Positive_infinity / Nan))
			assert ("NaN", attached {like real_to_be_tested}.arc_tangent (Zero / Nan))
			assert ("-0 / NaN", attached {like real_to_be_tested}.arc_tangent (-Zero / Nan))
			assert ("0 / NaN", attached {like real_to_be_tested}.arc_tangent (Zero / Nan))
			assert ("a / NaN", attached {like real_to_be_tested}.arc_tangent (some_real.value / Nan))

			assert ("-NaN / -Infinity", attached {like real_to_be_tested}.arc_tangent (-Nan / Negative_infinity))
			assert ("NaN / -Infinity", attached {like real_to_be_tested}.arc_tangent (Nan / Negative_infinity))
			assert ("-Infinity / -Infinity", attached {like real_to_be_tested}.arc_tangent (Negative_infinity / Negative_infinity))
			assert ("Infinity / -Infinity", attached {like real_to_be_tested}.arc_tangent (Positive_infinity / Negative_infinity))
			assert ("0 / -Infinity", attached {like real_to_be_tested}.arc_tangent (Zero / Negative_infinity))
			assert ("-0 / -Infinity", attached {like real_to_be_tested}.arc_tangent (-Zero / Negative_infinity))
			assert ("0 / -Infinity", attached {like real_to_be_tested}.arc_tangent (Zero / Negative_infinity))
			assert ("a / -Infinity", attached {like real_to_be_tested}.arc_tangent (some_real.value / Negative_infinity))

			assert ("-NaN / Infinity", attached {like real_to_be_tested}.arc_tangent (-Nan / Positive_infinity))
			assert ("NaN / Infinity", attached {like real_to_be_tested}.arc_tangent (Nan / Positive_infinity))
			assert ("-Infinity / Infinity", attached {like real_to_be_tested}.arc_tangent (Negative_infinity / Positive_infinity))
			assert ("Infinity / Infinity", attached {like real_to_be_tested}.arc_tangent (Positive_infinity / Positive_infinity))
			assert ("0 / Infinity", attached {like real_to_be_tested}.arc_tangent (Zero / Positive_infinity))
			assert ("-0 / Infinity", attached {like real_to_be_tested}.arc_tangent (Zero / Positive_infinity))
			assert ("0 / Infinity", attached {like real_to_be_tested}.arc_tangent (Zero / Positive_infinity))
			assert ("a / Infinity", attached {like real_to_be_tested}.arc_tangent (some_real.value / Positive_infinity))

			assert ("-NaN / -0", attached {like real_to_be_tested}.arc_tangent (-Nan / -Zero))
			assert ("NaN / -0", attached {like real_to_be_tested}.arc_tangent (Nan / -Zero))
			assert ("-Infinity / -0", attached {like real_to_be_tested}.arc_tangent (Negative_infinity / -Zero))
			assert ("Infinity / -0", attached {like real_to_be_tested}.arc_tangent (Positive_infinity / -Zero))
			assert ("0 / -0", attached {like real_to_be_tested}.arc_tangent (Zero / -Zero))
			assert ("-0 / -0", attached {like real_to_be_tested}.arc_tangent (-Zero / -Zero))
			assert ("0 / -0", attached {like real_to_be_tested}.arc_tangent (Zero / -Zero))
			assert ("a / -0", attached {like real_to_be_tested}.arc_tangent (some_real.value / -Zero))

			assert ("-NaN / 0", attached {like real_to_be_tested}.arc_tangent (-Nan / Zero))
			assert ("-NaN / 0", attached {like real_to_be_tested}.arc_tangent (-Nan / Zero))
			assert ("NaN / 0", attached {like real_to_be_tested}.arc_tangent (Nan / Zero))
			assert ("NaN / 0", attached {like real_to_be_tested}.arc_tangent (Nan / Zero))
			assert ("-Infinity / 0", attached {like real_to_be_tested}.arc_tangent (Negative_infinity / Zero))
			assert ("-Infinity / 0", attached {like real_to_be_tested}.arc_tangent (Negative_infinity / Zero))
			assert ("Infinity / 0", attached {like real_to_be_tested}.arc_tangent (Positive_infinity / Zero))
			assert ("Infinity / 0", attached {like real_to_be_tested}.arc_tangent (Positive_infinity / Zero))
			assert ("0 / 0", attached {like real_to_be_tested}.arc_tangent (Zero / Zero))
			assert ("0 / 0", attached {like real_to_be_tested}.arc_tangent (Zero / Zero))
			assert ("-0 / 0", attached {like real_to_be_tested}.arc_tangent (-Zero / Zero))
			assert ("0 / 0", attached {like real_to_be_tested}.arc_tangent (Zero / Zero))
			assert ("a / 0", attached {like real_to_be_tested}.arc_tangent (some_real.value / Zero))
			assert ("a / 0", attached {like real_to_be_tested}.arc_tangent (some_real.value / Zero))

			assert ("-NaN / b", attached {like real_to_be_tested}.arc_tangent (-Nan / some_real))
			assert ("NaN / b", attached {like real_to_be_tested}.arc_tangent (Nan / some_real))
			assert ("-Infinity / b", attached {like real_to_be_tested}.arc_tangent (Negative_infinity / some_real))
			assert ("Infinity / b", attached {like real_to_be_tested}.arc_tangent (Positive_infinity / some_real))
			assert ("0 / bi", attached {like real_to_be_tested}.arc_tangent (Zero / some_real))
			assert ("-0 / b", attached {like real_to_be_tested}.arc_tangent (-Zero / some_real))
			assert ("0 / b", attached {like real_to_be_tested}.arc_tangent (Zero / some_real))
			assert ("a / b", attached {like real_to_be_tested}.arc_tangent (some_real.value / some_real.value))
		end

	test_value_logb
			-- Test {STS_COMPLEX_NUMBER}.value_logb.
			-- Test {STS_REAL_NUMBER}.value_logb.
			-- Test {STI_REAL_NUMBER}.value_logb.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.value_logb"
			testing: "covers/{STS_REAL_NUMBER}.value_logb"
			testing: "covers/{STI_REAL_NUMBER}.value_logb"
		do
			assert ("-NaN", attached real_to_be_tested.value_logb (-Nan))
			assert ("NaN", attached real_to_be_tested.value_logb (Nan))
			assert ("-Infinity", attached real_to_be_tested.value_logb (Negative_infinity))
			assert ("Infinity", attached real_to_be_tested.value_logb (Positive_infinity))
			assert ("-0", attached real_to_be_tested.value_logb (-0.0))
			assert ("0", attached real_to_be_tested.value_logb (0))
			assert ("x", attached real_to_be_tested.value_logb (some_native_real))
		end

	test_sqrt
			-- Test {STS_REAL_NUMBER}.sqrt.
			-- Test {STI_REAL_NUMBER}.sqrt.
		note
			testing: "covers/{STS_REAL_NUMBER}.sqrt"
			testing: "covers/{STI_REAL_NUMBER}.sqrt"
		do
			assert ("-NaN", attached real_to_be_tested.sqrt (-Nan))
			assert ("NaN", attached real_to_be_tested.sqrt (Nan))
			assert ("-Infinity", attached real_to_be_tested.sqrt (Negative_infinity))
			assert ("Infinity", attached real_to_be_tested.sqrt (Positive_infinity))
			assert ("-0", attached real_to_be_tested.sqrt (-0.0))
			assert ("0", attached real_to_be_tested.sqrt (0))
			assert ("x", attached real_to_be_tested.sqrt (some_native_real))
		end

	test_splitted
			-- Test {STI_REAL_NUMBER}.splitted.
		note
			testing: "covers/{STI_REAL_NUMBER}.splitted"
		local
			xs: like some_real_set
			x: like real_to_be_tested
			i_s: like some_integer_set
			i: like some_integer
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				from
					x := xs.any
					i_s := Z_universe
				until
					i_s.is_empty
				loop
					i := same_integer (i_s.any)
					assert ("splitted", attached x.splitted (i))
					assert ("splitted ok", properties.splitted_ok (x, i))
					i_s := i_s.others
				variant
					i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
				end
				xs := xs.others
			variant
				xs_cardinality: {like some_set}.natural_as_integer (# xs)
			end

				-- Real values not enumerated above
			from
				x := - Zero
				i_s := Z_universe
			until
				i_s.is_empty
			loop
				i := same_integer (i_s.any)
				assert ("splitted -0.0", attached x.splitted (i))
				assert ("splitted -0.0 ok", properties.splitted_ok (x, i))
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
			from
				x := - Nan
				i_s := Z_universe
			until
				i_s.is_empty
			loop
				i := same_integer (i_s.any)
				assert ("splitted -Nan", attached x.splitted (i))
				assert ("splitted -Nan ok", properties.splitted_ok (x, i))
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

feature -- Test routines (Factory)

	test_real_from_value
			-- Test {STI_REAL_NUMBER}.real_from_value.
		note
			testing: "covers/{STI_REAL_NUMBER}.real_from_value"
		do
			assert ("-NaN", attached real_to_be_tested.real_from_value (-Nan))
			assert ("NaN", attached real_to_be_tested.real_from_value (Nan))
			assert ("-Infinity", attached real_to_be_tested.real_from_value (Negative_infinity))
			assert ("Infinity", attached real_to_be_tested.real_from_value (Positive_infinity))
			assert ("-0", attached real_to_be_tested.real_from_value (-0.0))
			assert ("0", attached real_to_be_tested.real_from_value (0))
			assert ("x", attached real_to_be_tested.real_from_value (some_native_real))
		end

	test_real_from_double_value
			-- Test {STI_REAL_NUMBER}.real_from_double_value.
		note
			testing: "covers/{STI_REAL_NUMBER}.real_from_double_value"
		do
			assert ("-NaN", attached real_to_be_tested.real_from_double_value (-Nan.value))
			assert ("NaN", attached real_to_be_tested.real_from_double_value (Nan.value))
			assert ("-Infinity", attached real_to_be_tested.real_from_double_value (Negative_infinity.value))
			assert ("Infinity", attached real_to_be_tested.real_from_double_value (Positive_infinity.value))
			assert ("-0", attached real_to_be_tested.real_from_double_value (-0.0))
			assert ("0", attached real_to_be_tested.real_from_double_value (0))
			assert ("x", attached real_to_be_tested.real_from_double_value (some_native_double_real))
		end

	test_native_real_from_double_value
			-- Test {STS_COMPLEX_NUMBER}.native_real_from_double_value.
			-- Test {STS_REAL_NUMBER}.native_real_from_double_value.
			-- Test {STI_REAL_NUMBER}.native_real_from_double_value.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.native_real_from_double_value"
			testing: "covers/{STS_REAL_NUMBER}.native_real_from_double_value"
			testing: "covers/{STI_REAL_NUMBER}.native_real_from_double_value"
		do
			assert ("-NaN", attached real_to_be_tested.native_real_from_double_value (-Nan.value))
			assert ("NaN", attached real_to_be_tested.native_real_from_double_value (Nan.value))
			assert ("-Infinity", attached real_to_be_tested.native_real_from_double_value (Negative_infinity.value))
			assert ("Infinity", attached real_to_be_tested.native_real_from_double_value (Positive_infinity.value))
			assert ("-0", attached real_to_be_tested.native_real_from_double_value (-0.0))
			assert ("0", attached real_to_be_tested.native_real_from_double_value (0))
			assert ("x", attached real_to_be_tested.native_real_from_double_value (some_native_double_real))
		end

feature -- Test routines (Implementation)

	test_adjusted_value
			-- Test {STI_REAL_NUMBER}.adjusted_value.
		note
			testing: "covers/{STI_REAL_NUMBER}.adjusted_value"
		do
			assert ("-NaN", attached real_to_be_tested.adjusted_value (-Nan))
			assert ("NaN", attached real_to_be_tested.adjusted_value (Nan))
			assert ("-Infinity", attached real_to_be_tested.adjusted_value (Negative_infinity))
			assert ("Infinity", attached real_to_be_tested.adjusted_value (Positive_infinity))
			assert ("-0", attached real_to_be_tested.adjusted_value (-0.0))
			assert ("0", attached real_to_be_tested.adjusted_value (0))
			assert ("x", attached real_to_be_tested.adjusted_value (some_native_real))
		end

	test_value_sign_bit
			-- Test {STS_COMPLEX_NUMBER}.value_sign_bit.
			-- Test {STS_REAL_NUMBER}.value_sign_bit.
			-- Test {STI_REAL_NUMBER}.value_sign_bit.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.value_sign_bit"
			testing: "covers/{STS_REAL_NUMBER}.value_sign_bit"
			testing: "covers/{STI_REAL_NUMBER}.value_sign_bit"
		local
			v: like some_native_real
		do
			v := - Nan
			assert ("signed nan", real_to_be_tested.value_sign_bit (v) = 1)

			v := Nan
			assert ("unsigned nan", real_to_be_tested.value_sign_bit (v) = 0)

			from
				v := some_native_real
			until
				not v.is_nan and v < 0
			loop
				v := some_native_real
			end
			assert ("negative", real_to_be_tested.value_sign_bit (v) = 1)

			v := 0.0
			assert ("negative zero", real_to_be_tested.value_sign_bit (- v) = 1)
			assert ("positive zero", real_to_be_tested.value_sign_bit (v) = 0)

			from
				v := some_native_real
			until
				0 < v
			loop
				v := some_native_real
			end
			assert ("positive", real_to_be_tested.value_sign_bit (v) = 0)

			assert ("-NaN", attached real_to_be_tested.value_sign_bit (-Nan))
			assert ("NaN", attached real_to_be_tested.value_sign_bit (Nan))
			assert ("-Infinity", attached real_to_be_tested.value_sign_bit (Negative_infinity))
			assert ("Infinity", attached real_to_be_tested.value_sign_bit (Positive_infinity))
			assert ("-0", attached real_to_be_tested.value_sign_bit (-0.0))
			assert ("0", attached real_to_be_tested.value_sign_bit (0))
			assert ("x", attached real_to_be_tested.value_sign_bit (some_native_real))
		end

	test_value_bit_pattern
			-- Test {STI_REAL_NUMBER}.value_bit_pattern.
		note
			testing: "covers/{STI_REAL_NUMBER}.value_bit_pattern"
		local
			x: like exposed_real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("value_bit_pattern", attached x.value_bit_pattern)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			x := - Nan
			assert ("signed NaN", x.value_bit_pattern & ({NATURAL_8} 1 |<< ({like exposed_real_to_be_tested}.Exponent_width + {like exposed_real_to_be_tested}.Mantissa_width)) /= 0) -- TODO: Get rid of {NATURAL_8}? Make a function to take the sign bit.

			x := - Zero.value
			assert ("negative zero", x.value_bit_pattern & ({NATURAL_8} 1 |<< ({like exposed_real_to_be_tested}.Exponent_width + {like exposed_real_to_be_tested}.Mantissa_width)) /= 0) -- TODO: Get rid of {NATURAL_8}? Make a function to take the sign bit.
		end

	test_bit_pattern_from_native_real
			-- Test {STI_REAL_NUMBER}.bit_pattern_from_native_real.
		note
			testing: "covers/{STI_REAL_NUMBER}.bit_pattern_from_native_real"
		local
			x: like exposed_real_to_be_tested
		do
			x := exposed_real_to_be_tested
			assert ("signed NaN", x.bit_pattern_from_native_real (- NaN) & ({NATURAL_8} 1 |<< ({like exposed_real_to_be_tested}.Exponent_width + {like exposed_real_to_be_tested}.Mantissa_width)) /= 0) -- TODO: Get rid of {NATURAL_8}? Make a function to take the sign bit.
			assert ("negative zero", x.bit_pattern_from_native_real (- Zero.value) & ({NATURAL_8} 1 |<< ({like exposed_real_to_be_tested}.Exponent_width + {like exposed_real_to_be_tested}.Mantissa_width)) /= 0) -- TODO: Get rid of {NATURAL_8}? Make a function to take the sign bit.
			assert ("bit_pattern_from_native_real", attached x.bit_pattern_from_native_real (some_native_real))
		end

	test_sign_bit_status
			-- Test {STI_REAL_NUMBER}.sign_bit_status.
		note
			testing: "covers/{STI_REAL_NUMBER}.sign_bit_status"
		local
			x: like exposed_real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("sign_bit_pattern", attached x.sign_bit_status)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			x := - Nan
			assert ("signed NaN", x.sign_bit_status = 1)
			x := Nan
			assert ("unsigned NaN", x.sign_bit_status = 0)

			from
				x := exposed_real_to_be_tested
			until
				not x.is_nan and x < Zero
			loop
				x := exposed_real_to_be_tested
			end
			assert ("negative", x.sign_bit_status = 1)
			assert ("negative ok", Properties.sign_bit_status_ok (x))

			x := - Zero
			assert ("negative zero", x.sign_bit_status = 1)
			x := 0
			assert ("positive zero", x.sign_bit_status = 0)

			assert ("-NaN", properties.sign_bit_status_ok (-Nan))
			assert ("NaN", properties.sign_bit_status_ok (Nan))
			assert ("-Infinity", properties.sign_bit_status_ok (Negative_infinity))
			assert ("Infinity", properties.sign_bit_status_ok (Positive_infinity))
			assert ("-0", properties.sign_bit_status_ok (- Zero))
			assert ("0", properties.sign_bit_status_ok (Zero))

			assert ("sign_bit_status_ok", properties.sign_bit_status_ok (real_to_be_tested))
		end

	test_exponent_bit_pattern
			-- Test {STI_REAL_NUMBER}.exponent_bit_pattern.
		note
			testing: "covers/{STI_REAL_NUMBER}.exponent_bit_pattern"
		local
			x: like exposed_real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("exponent_bit_pattern", attached x.exponent_bit_pattern)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.exponent_bit_pattern_ok (-Nan))
			assert ("NaN", properties.exponent_bit_pattern_ok (Nan))
			assert ("-Infinity", properties.exponent_bit_pattern_ok (Negative_infinity))
			assert ("Infinity", properties.exponent_bit_pattern_ok (Positive_infinity))
			assert ("-0", properties.exponent_bit_pattern_ok (- Zero))
			assert ("0", properties.exponent_bit_pattern_ok (Zero))

			assert ("exponent_bit_pattern_ok", properties.exponent_bit_pattern_ok (real_to_be_tested))
		end

	test_mantissa_bit_pattern
			-- Test {STI_REAL_NUMBER}.mantissa_bit_pattern.
		note
			testing: "covers/{STI_REAL_NUMBER}.mantissa_bit_pattern"
		local
			x: like exposed_real_to_be_tested
			xs: like some_real_set
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				assert ("mantissa_bit_pattern", attached x.mantissa_bit_pattern)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end

			assert ("-NaN", properties.mantissa_bit_pattern_ok (-Nan))
			assert ("NaN", properties.mantissa_bit_pattern_ok (Nan))
			assert ("-Infinity", properties.mantissa_bit_pattern_ok (Negative_infinity))
			assert ("Infinity", properties.mantissa_bit_pattern_ok (Positive_infinity))
			assert ("-0", properties.mantissa_bit_pattern_ok (- Zero))
			assert ("0", properties.mantissa_bit_pattern_ok (Zero))

			assert ("mantissa_bit_pattern_ok", properties.mantissa_bit_pattern_ok (real_to_be_tested))
		end

	test_exponent_width
			-- Test {STI_REAL_NUMBER}.exponent_width.
		note
			testing: "covers/{STI_REAL_NUMBER}.exponent_width"
		do
			assert ("exponent_width", attached exposed_real_to_be_tested.exponent_width)
		end

	test_exponent_mask
			-- Test {STI_REAL_NUMBER}.exponent_mask.
		note
			testing: "covers/{STI_REAL_NUMBER}.exponent_mask"
		do
			assert ("exponent_mask", attached exposed_real_to_be_tested.exponent_mask)
		end

	test_max_exponent_bit_pattern
			-- Test {STI_REAL_NUMBER}.max_exponent_bit_pattern.
		note
			testing: "covers/{STI_REAL_NUMBER}.max_exponent_bit_pattern"
		do
			assert ("max_exponent_bit_pattern", attached exposed_real_to_be_tested.max_exponent_bit_pattern)
		end

	test_exponent_bias
			-- Test {STI_REAL_NUMBER}.exponent_bias.
		note
			testing: "covers/{STI_REAL_NUMBER}.exponent_bias"
		do
			assert ("exponent_bias", attached exposed_real_to_be_tested.exponent_bias)
		end

	test_max_exponent
			-- Test {STI_REAL_NUMBER}.max_exponent.
		note
			testing: "covers/{STI_REAL_NUMBER}.max_exponent"
		do
			assert ("max_exponent", attached exposed_real_to_be_tested.max_exponent)
		end

	test_min_normal_exponent
			-- Test {STI_REAL_NUMBER}.min_normal_exponent.
		note
			testing: "covers/{STI_REAL_NUMBER}.min_normal_exponent"
		do
			assert ("min_normal_exponent", attached exposed_real_to_be_tested.min_normal_exponent)
		end

	test_min_subnormal_exponent
			-- Test {STI_REAL_NUMBER}.min_subnormal_exponent.
		note
			testing: "covers/{STI_REAL_NUMBER}.min_subnormal_exponent"
		do
			assert ("min_subnormal_exponent", attached exposed_real_to_be_tested.min_subnormal_exponent)
		end

	test_mantissa_width
			-- Test {STI_REAL_NUMBER}.mantissa_width.
		note
			testing: "covers/{STI_REAL_NUMBER}.mantissa_width"
		do
			assert ("mantissa_mask", attached exposed_real_to_be_tested.mantissa_width)
		end

	test_mantissa_mask
			-- Test {STI_REAL_NUMBER}.mantissa_mask.
		note
			testing: "covers/{STI_REAL_NUMBER}.mantissa_mask"
		do
			assert ("mantissa_mask", attached exposed_real_to_be_tested.mantissa_mask)
		end

	test_max_mantissa
			-- Test {STI_REAL_NUMBER}.max_mantissa.
		note
			testing: "covers/{STI_REAL_NUMBER}.max_mantissa"
		do
			assert ("max_mantissa", attached exposed_real_to_be_tested.max_mantissa)
		end

	test_mantissa_scale
			-- Test {STI_REAL_NUMBER}.mantissa_scale.
		note
			testing: "covers/{STI_REAL_NUMBER}.mantissa_scale"
		do
			assert ("mantissa_scale", attached exposed_real_to_be_tested.mantissa_scale)
		end

feature {NONE} -- Factory (element to be tested)

	real_to_be_tested: like some_expanded_real
			-- Real number meant to be under tests
		do
			Result := some_expanded_real
		end

	exposed_real_to_be_tested: like some_exposed_real
			-- Real number meant to have under tests its features with restricted access
		do
			Result := some_exposed_real
		end

feature {NONE} -- Conversion

	real_from_native (v: like some_native_real): like real_to_be_tested
			-- `v' converted to a real number like `real_to_be_tested'
		do
			Result := v
		ensure
			adjusted_value: Result.value = {like real_to_be_tested}.adjusted_value (v)
		end

	real_from_native_ref (v_ref: like some_native_real_ref): like real_to_be_tested
			-- `v_ref' converted to a real number like `real_to_be_tested'
		do
			Result := v_ref
		ensure
			adjusted_value: Result.value = {like real_to_be_tested}.adjusted_value (v_ref.item)
		end

	real_from_native_double_real (dv: like some_native_double_real): like real_to_be_tested
			-- `dv' converted to a real number like `real_to_be_tested'
		do
			Result := dv
		ensure
			adjusted_value: Result.value = {like real_to_be_tested}.adjusted_value (dv.truncated_to_real)
		end

	real_from_native_double_real_ref (dv_ref: like some_native_double_real_ref): like real_to_be_tested
			-- `dv_ref' converted to a real number like `real_to_be_tested'
		do
			Result := dv_ref
		ensure
			adjusted_value: Result.value = {like real_to_be_tested}.adjusted_value (dv_ref.truncated_to_real)
		end

	real_from_x (x: STS_REAL_NUMBER): like real_to_be_tested
			-- `x' converted to a real number like `real_to_be_tested'
		do
			Result := x
		ensure
			same_real: Result ≍ x
		end

	real_from_native_integer (v: like some_native_integer): like real_to_be_tested
			-- `v' converted to a real number like `real_to_be_tested'
		do
			Result := v
		ensure
			adjusted_value: Result.value = {like real_to_be_tested}.adjusted_value (v)
		end

	real_from_native_integer_ref (v_ref: attached like some_native_integer_ref): like real_to_be_tested
			-- `v_ref' converted to a real number like `real_to_be_tested'
		do
			Result := v_ref
		ensure
			adjusted_value: Result.value = {like real_to_be_tested}.adjusted_value (v_ref.to_real)
		end

	real_from_integer (i: STS_INTEGER_NUMBER): like real_to_be_tested
			-- `i' converted to a real number like `real_to_be_tested'
		do
			Result := i
		ensure
			adjusted_value: Result.value = {like real_to_be_tested}.adjusted_value (i.value)
		end

	real_from_native_natural (v: like some_native_natural): like real_to_be_tested
			-- `v' converted to a real number like `real_to_be_tested'
		do
			Result := v
		ensure
			adjusted_value: Result.value = {like real_to_be_tested}.adjusted_value (v)
		end

	real_from_native_natural_ref (v_ref: like some_native_natural_ref):
		like real_to_be_tested
			-- `v_ref' converted to a real number like `real_to_be_tested'
		do
			Result := v_ref
		ensure
			adjusted_value: Result.value =
				{like real_to_be_tested}.adjusted_value (v_ref.item)
		end

	native_from_real (x: STI_REAL_NUMBER): like some_native_real
			-- `x' converted to a native real number
		do
			Result := x
		ensure
			value: Result = x.value
		end

	native_ref_from_real (x: STI_REAL_NUMBER): like some_native_real_ref
			-- `x' converted to a native real number reference
		do
			Result := x
		ensure
			value: Result = x.value
		end

	numeric_from_real (x: STI_REAL_NUMBER): NUMERIC
			-- `x' converted to a {NUMERIC} object
		do
			Result := x
		ensure
			value: Result = x.value
		end

	comparable_from_real (x: STI_REAL_NUMBER): COMPARABLE
			-- `x' converted to a {COMPARABLE} object
		do
			Result := x
		ensure
			value: Result = x.value
		end

	hashable_from_real (x: STI_REAL_NUMBER): HASHABLE
			-- `x' converted to a {HASHABLE} object
		do
			Result := x
		ensure
			value: Result = x.value
		end

	complex_from_real (x: STI_REAL_NUMBER): STI_COMPLEX_NUMBER
			-- `x' converted to an {STI_COMPLEX_NUMBER}
		do
			Result := x
		ensure
			real: Result.a ≍ x
			imaginary: Result.b ≍ Zero
		end

feature -- Anchor

	real_math_anchor: like real_to_be_tested.real_math_anchor
			-- Anchor for accessing basic mathematical operations on native real
			-- numbers
		do
			Result := {like real_to_be_tested}.real_math_anchor
		ensure
			class
		end

invariant
	Zero: Zero.value = 0
	One: One.value = 1

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
