﻿note
	description: "Test suite for {STI_NUMERIC_SET}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	NUMERIC_SET_TESTS [A -> STS_COMPLEX_NUMBER, EQ -> STS_NUMERIC_EQUALITY [A] create default_create end]

inherit
	SET_TESTS [A, EQ]
		rename
			some_integer_ref as some_native_integer_ref,
			some_direct_integer_ref as some_direct_native_integer_ref,
			new_integer_ref as new_native_integer_ref,
			some_integer as some_native_integer
		undefine
			on_prepare,
			some_native_integer,
			some_element,
			some_equality,
			some_pair,
			some_set,
			some_n_tuple,
			test_make_empty,
			test_make_singleton,
			test_make_extended,
			test_make_from_set,
			set_to_be_tested,
			set_from_sa,
			set_from_saa
		redefine
			test_equals,
			test_is_subset,
			test_is_superset,
			test_is_not_superset,
			test_is_comparable,
			test_is_not_comparable,
			test_is_strict_subset,
			test_is_strict_superset,
			test_is_trivial_subset,
			test_is_proper_subset,
			test_is_proper_superset,
			test_is_disjoint,
			test_complemented,
			test_united,
			test_intersected,
			test_subtracted,
			test_subtracted_symmetricaly
		end

	NUMBERS_TESTS_RESOURCES [A, EQ]
		rename
			eq_a as eq
		end

feature -- Access

	properties: NUMERIC_SET_PROPERTIES [A, EQ]
			-- Object that checks the set-theory properties of {STI_NUMERIC_SET}
		deferred
		end

feature -- Test routines (Access)

	test_zero
			-- Test {STS_NUMERIC_SET}.Zero.
		note
			testing: "covers/{STS_NUMERIC_SET}.zero"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested

			assert ("Zero", attached s.Zero)
			assert ("zero_ok", properties.zero_ok (s, some_object_a))
		end

	test_u
			-- Test {STI_NUMERIC_SET}.u.
		note
			testing: "covers/{STI_NUMERIC_SET}.u"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			assert ("u", attached s.u)
			assert ("u_ok", properties.u_ok (s, some_object_a))
		end

	test_universe
			-- Test {STI_NUMERIC_SET}.universe.
		note
			testing: "covers/{STI_NUMERIC_SET}.universe"
		local
			s: like set_to_be_tested
		do
			s := set_to_be_tested
			assert ("universe", attached s.universe)
			assert ("universe_ok", properties.universe_ok (s, some_object_a))
		end

feature -- Test routines (Comparison)

	test_equals
			-- Test {STI_NUMERIC_SET}.equals.
		note
			testing: "covers/{STI_NUMERIC_SET}.equals"
		local
			a: like some_object_a
			s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := same_set_a (s1)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ≍ s2", s1 ≍ s2)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ≍ s2 ok", properties.equals_ok (s1, s2))

			a := some_object_a
			s1 := set_to_be_tested & a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s2 := some_set_a / a
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (s1 ≍ s2)", not (s1 ≍ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (s1 ≍ s2) ok", properties.equals_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("equals", s1 ≍ s2 implies True)
			assert ("equals_ok", properties.equals_ok (s1, s2))
		end

	test_is_subset
			-- Test {STI_NUMERIC_SET}.is_subset.
		note
			testing: "covers/{STI_NUMERIC_SET}.is_subset"
		local
			a: like some_object_a
			s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
			s2 := same_set_a (s1.u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ⊆ s2", s1 ⊆ s2)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ⊆ s2 ok", properties.is_subset_ok (s1, s2))

			a := some_object_a
			s1 := set_to_be_tested & a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s2 := some_set_a / a
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (s1 ⊆ s2)", not (s1 ⊆ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (s1 ⊆ s2) ok", properties.is_subset_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_subset", s1 ⊆ s2 implies True)
			assert ("is_subset_ok", properties.is_subset_ok (s1, s2))
		end

	test_is_superset
			-- Test {STI_NUMERIC_SET}.is_superset.
		note
			testing: "covers/{STI_NUMERIC_SET}.is_superset"
		local
			a: like some_object_a
			s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ⊇ s2", s1 ⊇ s2)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ⊇ s2 ok", properties.is_superset_ok (s1, s2))

			a := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s1 := set_to_be_tested / a
				s2 := some_set_a & a
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (s1 ⊇ s2)", not (s1 ⊇ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (s1 ⊇ s2) ok", properties.is_superset_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_superset", s1 ⊇ s2 implies True)
			assert ("is_superset_ok", properties.is_superset_ok (s1, s2))
		end

	test_is_not_superset
			-- <Precursor>
			-- Test {STS_NUMERIC_SET}.is_not_superset.
			-- Test {STI_NUMERIC_SET}.is_not_superset.
		note
			testing: "covers/{STS_SET}.is_not_superset"
			testing: "covers/{STI_SET}.is_not_superset"
			testing: "covers/{STS_NUMERIC_SET}.is_not_superset"
			testing: "covers/{STI_NUMERIC_SET}.is_not_superset"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			a: like some_object_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (s1 ⊉ s2)", not (s1 ⊉ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (s1 ⊉ s2) ok", properties.is_not_superset_ok (s1, s2))

			a := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s1 := set_to_be_tested / a
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			s2 := some_set_a & a
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ⊉ s2", s1 ⊉ s2)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ⊉ s2 ok", properties.is_not_superset_ok (s1, s2))
		end

	test_is_comparable
			-- Test {STI_NUMERIC_SET}.is_comparable.
		note
			testing: "covers/{STI_NUMERIC_SET}.is_comparable"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
			a, b: like some_object_a
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ⊥ s2", s1.is_comparable (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ⊥ s2 ok", properties.is_comparable_ok (s1, s2))

			a := some_object_a
			from
				b := some_object_a
			until
				b ≭ a
			loop
				b := some_object_a
			end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s1 := (set_to_be_tested / a) & b
				s2 := (some_set_a & a) / b
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (s1 ⊥ s2)", not s1.is_comparable (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (s1 ⊥ s2) ok", properties.is_comparable_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_comparable", s1.is_comparable (s2) implies True)
			assert ("is_comparable_ok", properties.is_comparable_ok (s1, s2))
		end

	test_is_not_comparable
			-- <Precursor>
			-- Test {STS_NUMERIC_SET}.is_not_comparable.
			-- Test {STI_NUMERIC_SET}.is_not_comparable.
		note
			testing: "covers/{STS_SET}.is_not_comparable"
			testing: "covers/{STI_SET}.is_not_comparable"
			testing: "covers/{STS_NUMERIC_SET}.is_not_comparable"
			testing: "covers/{STI_NUMERIC_SET}.is_not_comparable"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like set_to_be_tested.cardinality
			a, b: like some_object_a
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not s1.is_not_comparable (s2)", not s1.is_not_comparable (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not s1.is_not_comparable (s2) ok", properties.is_not_comparable_ok (s1, s2))

			a := some_object_a
			from
				b := some_object_a
			until
				b ≭ a
			loop
				b := some_object_a
			end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s1 := (set_to_be_tested / a) & b
				s2 := (some_set_a & a) / b
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1.is_not_comparable (s2)", s1.is_not_comparable (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1.is_not_comparable (s2) ok", properties.is_not_comparable_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_not_comparable", s1.is_not_comparable (s2) implies True)
			assert ("is_not_comparable_ok", properties.is_not_comparable_ok (s1, s2))
		end

	test_is_strict_subset
			-- Test {STI_NUMERIC_SET}.is_strict_subset.
		note
			testing: "covers/{STI_NUMERIC_SET}.is_strict_subset"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (s1 ⊂ s2)", not (s1 ⊂ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (s1 ⊂ s2) ok", properties.is_strict_subset_ok (s1, s2))
			assert ("not (s1 ⊂ s2); s3 ok", properties.sp_is_strict_subset_ok (s1, s2, some_set_a))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s2 := some_set_a
				until
					not s2.is_empty
				loop
					s2 := some_set_a
				end
					check
						positive: 0 < # s2 -- not s2.is_empty
					end
				s1 := s1.converted ((s2.as_tuple.suffix (some_index_up_to (# s2) - 1)).terms)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ⊂ s2", s1 ⊂ s2)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ⊂ s2 ok", properties.is_strict_subset_ok (s1, s2))
			assert ("s1 ⊂ s2; s3 ok", properties.sp_is_strict_subset_ok (s1, s2, some_set_a))

			s1 := set_to_be_tested
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("is_strict_subset", s1 ⊂ s2 implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("is_strict_subset_ok", properties.is_strict_subset_ok (s1, s2))
			assert ("sp_is_strict_subset_ok", properties.sp_is_strict_subset_ok (s1, s2, some_set_a))
		end

	test_is_strict_superset
			-- Test {STI_NUMERIC_SET}.is_strict_superset.
		note
			testing: "covers/{STI_NUMERIC_SET}.is_strict_superset"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
			a: like some_object_a
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s2 := some_set_a / some_object_a
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1 ⊃ s2", s1 ⊃ s2)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1 ⊃ s2 ok", properties.is_strict_superset_ok (s1, s2))

			a := some_object_a
			s2 := some_set_a & a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s1 := set_to_be_tested / a
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (s1 ⊃ s2)", not (s1 ⊃ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (s1 ⊃ s2) ok", properties.is_strict_superset_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_strict_superset", s1 ⊃ s2 implies True)
			assert ("is_strict_superset ok", properties.is_strict_superset_ok (s1, s2))
		end

	test_is_trivial_subset
			-- <Precursor>
			-- Test {STI_NUMERIC_SET}.is_trivial_subset.
		note
			testing: "covers/{STI_SET}.is_trivial_subset"
			testing: "covers/{STI_NUMERIC_SET}.is_trivial_subset"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			a: like some_object_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := same_set_a (s1)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1.is_trivial_subset (s2)", s1.is_trivial_subset (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1.is_trivial_subset (s2) ok", properties.is_trivial_subset_ok (s1, s2))

			a := some_object_a
			s1 := set_to_be_tested & a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s2 := some_set_a / a
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not s1.is_trivial_subset (s2)", not s1.is_trivial_subset (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not s1.is_trivial_subset (s2) ok", properties.is_trivial_subset_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_trivial_subset", s1.is_trivial_subset (s2) implies True)
			assert ("is_trivial_subset_ok", properties.is_trivial_subset_ok (s1, s2))
		end

--	test_is_trivial_superset
--			-- Test {STI_NUMERIC_SET}.is_trivial_superset.
--		note
--			testing: "covers/{STI_NUMERIC_SET}.is_trivial_superset"
--		local
--			s1: like set_to_be_tested
--			s2: like some_set_a
--			a: like some_object_a
--			chk_ops, chk_tests: BOOLEAN
--			n: like some_set.cardinality
--		do
--			Precursor {SET_TESTS}

--			s1 := set_to_be_tested
--				check
--					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
--				end
--			s1 := s1.converted (s1.u.others.extended (s1.u.any))
--			if next_random_item \\ 2 = 0 then
--				s2 := same_set_a (s1)
--			else
--				s2 := s1.o
--			end
--			chk_ops := {ISE_RUNTIME}.check_assert (False)
--				n := # s1
--			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
--			if n > max_asserted_elements then
--				chk_tests := {ISE_RUNTIME}.check_assert (False)
--			end
--				assert ("s1.is_trivial_superset (s2)", s1.is_trivial_superset (s2))
--			if chk_tests then
--				chk_tests := {ISE_RUNTIME}.check_assert (True)
--			end
--			assert ("s1.is_trivial_superset (s2) ok", properties.is_trivial_superset_ok (s1, s2))

--			chk_ops := {ISE_RUNTIME}.check_assert (False)
--				from
--					s2 := some_set_a / some_object_a
--				until
--					not s2.is_empty
--				loop
--					s2 := some_set_a / some_object_a
--				end
--				n := # s1 ∨ # s2
--			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
--			if n > max_asserted_elements then
--				chk_tests := {ISE_RUNTIME}.check_assert (False)
--			end
--				assert ("not s1.is_trivial_superset (s2)", not s1.is_trivial_superset (s2))
--			if chk_tests then
--				chk_tests := {ISE_RUNTIME}.check_assert (True)
--			end
--			assert ("not s1.is_trivial_superset (s2) ok", properties.is_trivial_superset_ok (s1, s2))

--			s1 := set_to_be_tested
--			s2 := some_set_a
--			assert ("is_trivial_superset", s1.is_trivial_superset (s2) implies True)
--			assert ("is_trivial_superset_ok", properties.is_trivial_superset_ok (s1, s2))
--		end

	test_is_proper_subset
			-- <Precursor>
			-- Test {STI_NUMERIC_SET}.is_proper_subset.
		note
			testing: "covers/{STI_SET}.is_proper_subset"
			testing: "covers/{STI_NUMERIC_SET}.is_proper_subset"
		local
			s0, s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not s1.is_proper_subset (s2)", not s1.is_proper_subset (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not s1.is_proper_subset (s2) ok", properties.is_proper_subset_ok (s1, s2))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s0 := set_to_be_tested
					s1 := s0.others ∖ some_set_a
				until
					not s1.is_empty
				loop
					s0 := set_to_be_tested
					s1 := s0.others ∖ some_set_a
				end
				s2 := same_set_a (s0)
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1.is_proper_subset (s2)", s1.is_proper_subset (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1.is_proper_subset (s2) ok", properties.is_proper_subset_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_proper_subset", s1.is_proper_subset (s2) implies True)
			assert ("is_proper_subset_ok", properties.is_proper_subset_ok (s1, s2))
		end

	test_is_proper_superset
			-- Test {STI_NUMERIC_SET}.is_proper_superset.
		note
			testing: "covers/{STI_NUMERIC_SET}.is_proper_superset"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			if next_random_item \\ 2 = 0 then
				s2 := same_set_a (s1)
			else
				s2 := s1.o
			end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not s1.is_proper_superset (s2)", not s1.is_proper_superset (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not s1.is_proper_superset (s2) ok", properties.is_proper_superset_ok (s1, s2))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s2 := some_set_a / some_object_a
				until
					not s2.is_empty
				loop
					s2 := some_set_a / some_object_a
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1.is_proper_superset (s2)", s1.is_proper_superset (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1.is_proper_superset (s2) ok", properties.is_proper_superset_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_proper_superset", s1.is_proper_superset (s2) implies True)
			assert ("is_proper_superset ok", properties.is_proper_superset_ok (s1, s2))
		end

	test_is_disjoint
			-- Test {STI_NUMERIC_SET}.is_disjoint.
		note
			testing: "covers/{STI_NUMERIC_SET}.is_disjoint"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a.o -- TODO: Make an o?
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s1.is_disjoint (s2)", s1.is_disjoint (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s1.is_disjoint (s2) ok", properties.is_disjoint_ok (s1, s2))

			from
				s2 := some_set_a
			until
				not s2.is_empty
			loop
				s2 := some_set_a
			end
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not s1.is_disjoint (s2)", not s1.is_disjoint (s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not s1.is_disjoint (s2) ok", properties.is_disjoint_ok (s1, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			assert ("is_disjoint", s1.is_disjoint (s2) implies True)
			assert ("is_disjoint_ok", properties.is_disjoint_ok (s1, s2))
		end

feature -- Test routines (Restriction)

	test_without_zero
			-- Test {STS_NUMERIC_SET}.without_zero.
			-- Test {STI_NUMERIC_SET}.without_zero.
		note
			testing: "covers/{STS_NUMERIC_SET}.without_zero"
			testing: "covers/{STI_NUMERIC_SET}.without_zero"
		local
			s: like set_to_be_tested
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := s / s.zero
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("no change", attached s.without_zero)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("no change ok", Properties.without_zero_ok (s))

			s := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("without_zero", attached s.without_zero)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("without_zero ok", Properties.without_zero_ok (s))
		end

feature -- Test routines (Operation)

	test_complemented
			-- Test {STI_NUMERIC_SET}.complemented.
		note
			testing: "covers/{STI_NUMERIC_SET}.complemented"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := same_set_a (s1)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s1 = # s1.u: s1 ⊆ s2", s1 ⊆ s2) -- s2 := same_set_a (s1)
				assert ("# s1 = # s1.u", attached (s1 ∁ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s1 = # s1.u ok", properties.complemented_ok (s1, some_set_a, s2))

			s1 := set_to_be_tested
			s2 := s1.u
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s2 = s1.u: s1 ⊆ s2", s1 ⊆ s2) -- s2 := s1.u
				assert ("s2 = s1.u", attached (s1 ∁ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s2 = s1.u ok", properties.complemented_ok (s1, some_set_a, s2))

			s1 := set_to_be_tested
			s2 := same_set_a (s1.u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s2 = # s1.u: s1 ⊆ s2", s1 ⊆ s2) -- s2 := same_set_a (s1.u)
				assert ("# s2 = # s1.u", attached (s1 ∁ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s2 = # s1.u ok", properties.complemented_ok (s1, some_set_a, s2))

			s1 := set_to_be_tested
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("complemented", s1 ⊆ s2 implies attached (s1 ∁ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("complemented_ok", properties.complemented_ok (s1, some_set_a, s2))
		end

	test_complement
			-- Test {STS_NUMERIC_SET}.complement.
			-- Test {STI_NUMERIC_SET}.complement.
		note
			testing: "covers/{STS_NUMERIC_SET}.complement"
			testing: "covers/{STI_NUMERIC_SET}.complement"
		local
			s1: like set_to_be_tested
			s2: like set_to_be_tested -- TODO: Create a some_numeric_set_a?
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			s1 := set_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1.u
				s2 := s1.converted (some_set_a)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("complement", attached (∁ s1))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("complement_ok", properties.complement_ok (s1, s2))
		end

	test_united
			-- Test {STI_NUMERIC_SET}.united.
		note
			testing: "covers/{STI_NUMERIC_SET}.united"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s1 = # s1.u", attached (s1 ∪ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s1 = # s1.u ok", properties.united_ok (some_object_a, s1, s2, some_set_a, some_set_a))

			s1 := set_to_be_tested
			s2 := s1.u
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s2 = s1.u", attached (s1 ∪ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s2 = s1.u ok", properties.united_ok (some_object_a, s1, s2, some_set_a, some_set_a))

			s1 := set_to_be_tested
			s2 := same_set_a (s1.u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s2 = # s1.u", attached (s1 ∪ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s2 = # s1.u ok", properties.united_ok (some_object_a, s1, s2, some_set_a, some_set_a))

			s1 := set_to_be_tested
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("united", attached (s1 ∪ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("united_ok", properties.united_ok (some_object_a, s1, s2, some_set_a, some_set_a))
		end

	test_intersected
			-- Test {STI_NUMERIC_SET}.intersected.
		note
			testing: "covers/{STI_NUMERIC_SET}.intersected"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s1 = # s1.u", attached (s1 ∩ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s1 = # s1.u ok", properties.intersected_ok (some_object_a, s1, s2, some_set_a, some_set_a))

			s1 := set_to_be_tested
			s2 := s1.u
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s2 = s1.u", attached (s1 ∩ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s2 = s1.u ok", properties.intersected_ok (some_object_a, s1, s2, some_set_a, some_set_a))

			s1 := set_to_be_tested
			s2 := same_set_a (s1.u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s2 = # s1.u", attached (s1 ∩ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s2 = # s1.u ok", properties.intersected_ok (some_object_a, s1, s2, some_set_a, some_set_a))

			s1 := set_to_be_tested
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("intersected", attached (s1 ∩ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("intersected_ok", properties.intersected_ok (some_object_a, s1, s2, some_set_a, some_set_a))
		end

	test_subtracted
			-- Test {STI_NUMERIC_SET}.subtracted.
		note
			testing: "covers/{STI_NUMERIC_SET}.subtracted"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
			s2 := s1.u
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s2 = s1.u", attached (s1 ∖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s2 = s1.u ok", properties.subtracted_ok (some_object_a, s1, s2, some_set_a, some_set_a, some_set_a))

			s1 := set_to_be_tested
			s2 := same_set_a (s1.u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s2 = # s1.u", attached (s1 ∖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s2 = # s1.u ok", properties.subtracted_ok (some_object_a, s1, s2, some_set_a, some_set_a, some_set_a))

			s1 := set_to_be_tested
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("subtracted", attached (s1 ∖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("subtracted_ok", properties.subtracted_ok (some_object_a, s1, s2, some_set_a, some_set_a, some_set_a))
		end

	test_subtracted_symmetricaly
			-- Test {STI_NUMERIC_SET}.subtracted_symmetricaly.
		note
			testing: "covers/{STI_NUMERIC_SET}.subtracted_symmetricaly"
		local
			s1: like set_to_be_tested
			s2: like some_set_a
			n: like some_set_a.cardinality
			chk_ops, chk_tests: BOOLEAN
		do
			Precursor {SET_TESTS}

			s1 := set_to_be_tested
				check
					does_not_have: s1.u.others ∌ s1.u.any -- {STS_SET} invariant
				end
			s1 := s1.converted (s1.u.others.extended (s1.u.any))
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s1 = # s1.u", attached (s1 ⊖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s1 = # s1.u ok", properties.subtracted_symmetricaly_ok (some_object_a, s1, s2, some_set_a))

			s1 := set_to_be_tested
			s2 := s1.u
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("s2 = s1.u", attached (s1 ⊖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("s2 = s1.u ok", properties.subtracted_symmetricaly_ok (some_object_a, s1, s2, some_set_a))

			s1 := set_to_be_tested
			s2 := same_set_a (s1.u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("# s2 = # s1.u", attached (s1 ⊖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("# s2 = # s1.u ok", properties.subtracted_symmetricaly_ok (some_object_a, s1, s2, some_set_a))

			s1 := set_to_be_tested
			s2 := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s1 ∨ # s2
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("subtracted_symmetricaly", attached (s1 ⊖ s2))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("subtracted_symmetricaly_ok", properties.subtracted_symmetricaly_ok (some_object_a, s1, s2, some_set_a))
		end

feature {NONE} -- Factory (element to be tested)

	set_to_be_tested: STI_NUMERIC_SET [A, EQ]
			-- <Precursor>
		deferred
		end

feature {NONE} -- Conversion

	set_from_sa (sa: STS_SET [A, EQ]): like set_to_be_tested
			-- <Precursor>
		deferred
		end

	set_from_saa (saa: STS_TRANSFORMABLE_SET [A, A, EQ, EQ]): like set_to_be_tested
			-- <Precursor>
		deferred
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
