﻿note
	description: "Test suite for {STI_ORDERED_NUMERIC_SET}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	ORDERED_NUMERIC_SET_TESTS [A -> STS_REAL_NUMBER, EQ -> STS_NUMERIC_EQUALITY [A] create default_create end]

inherit
	NUMERIC_SET_TESTS [A, EQ]

feature -- Access

	zero: A
			-- The number 0
		deferred
		ensure
			definition: Result ≍ Result.zero
		end

	properties: ORDERED_NUMERIC_SET_PROPERTIES [A, EQ]
			-- Object that checks the set-theory properties of {STI_ORDERED_NUMERIC_SET}
		deferred
		end

feature -- Test routines (Restriction)

	test_non_positive
			-- Test {STS_ORDERED_NUMERIC_SET}.non_positive.
			-- Test {STI_ORDERED_NUMERIC_SET}.non_positive.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.non_positive"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.non_positive"
		local
			s: like set_to_be_tested
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s := set_to_be_tested
				until
					s |∃ agent (x: A): BOOLEAN do Result := x ≤ Zero end
				loop
					s := set_to_be_tested
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("non_empty_interval", attached s.non_positive)
			assert ("non_empty_interval ok", Properties.non_positive_ok (s))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested | agent (x: A): BOOLEAN do Result := x ≥ zero end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("possibly_singleton_interval", attached s.non_positive)
			assert ("possibly_singleton_interval ok", Properties.non_positive_ok (s))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested | agent (x: A): BOOLEAN do Result := x > zero end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("empty_interval", attached s.non_positive)
			assert ("empty_interval ok", Properties.non_positive_ok (s))

			s := set_to_be_tested
			assert ("non_positive", attached s.non_positive)
			assert ("non_positive ok", Properties.non_positive_ok (s))
		end

	test_non_negative
			-- Test {STS_ORDERED_NUMERIC_SET}.non_negative.
			-- Test {STI_ORDERED_NUMERIC_SET}.non_negative.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.non_negative"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.non_negative"
		local
			s: like set_to_be_tested
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s := set_to_be_tested
				until
					s |∃ agent (x: A): BOOLEAN do Result := x ≥ Zero end
				loop
					s := set_to_be_tested
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("non_empty_interval", attached s.non_negative)
			assert ("non_empty_interval ok", Properties.non_negative_ok (s))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested | agent (x: A): BOOLEAN do Result := x ≤ zero end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("possibly_singleton_interval", attached s.non_negative)
			assert ("possibly_singleton_interval ok", Properties.non_negative_ok (s))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested | agent (x: A): BOOLEAN do Result := x < zero end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("empty_interval", attached s.non_negative)
			assert ("empty_interval ok", Properties.non_negative_ok (s))

			s := set_to_be_tested
			assert ("non_negative", attached s.non_negative)
			assert ("non_negative ok", Properties.non_negative_ok (s))
		end

	test_negative
			-- Test {STS_ORDERED_NUMERIC_SET}.negative.
			-- Test {STI_ORDERED_NUMERIC_SET}.negative.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.negative"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.negative"
		local
			s: like set_to_be_tested
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s := set_to_be_tested
				until
					s |∃ agent (x: A): BOOLEAN do Result := x < Zero end
				loop
					s := set_to_be_tested
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("non_empty_interval", attached s.negative)
			assert ("non_empty_interval ok", Properties.negative_ok (s))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested | agent (x: A): BOOLEAN do Result := x ≥ Zero end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("empty_interval", attached s.negative)
			assert ("empty_interval ok", Properties.negative_ok (s))

			s := set_to_be_tested
			assert ("negative", attached s.negative)
			assert ("negative ok", Properties.negative_ok (s))
		end

	test_positive
			-- Test {STS_ORDERED_NUMERIC_SET}.positive.
			-- Test {STI_ORDERED_NUMERIC_SET}.positive.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.positive"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.positive"
		local
			s: like set_to_be_tested
			chk_ops: BOOLEAN
		do
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s := set_to_be_tested
				until
					s |∃ agent (x: A): BOOLEAN do Result := x > Zero end
				loop
					s := set_to_be_tested
				end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("non_empty_interval", attached s.positive)
			assert ("non_empty_interval ok", Properties.positive_ok (s))

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := set_to_be_tested | agent (x: A): BOOLEAN do Result := x ≤ Zero end
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("empty_interval", attached s.positive)
			assert ("empty_interval ok", Properties.positive_ok (s))

			s := set_to_be_tested
			assert ("positive", attached s.positive)
			assert ("positive ok", Properties.positive_ok (s))
		end

feature -- Test routines (Interval)

	test_closed_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.closed_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.closed_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.closed_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.closed_interval"
		local
			s: like set_to_be_tested
			xt: like set_to_be_tested.n_tuple_anchor
			a, b, m: A
			i: like set_to_be_tested.n_tuple_anchor.n
			chk_ops: BOOLEAN
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				xt := s.as_tuple
				i := next_random_item.as_natural_32 \\ xt.n + 1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- 0 < i ≤ xt.n
					small_enough: i ≤ xt.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				b := xt [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			m := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.min (m, b)
				b := s.max (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a ≤ b", attached s [a, b])
			assert ("a ≤ b ok", Properties.closed_interval_ok (s, a, b))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				xt := s.as_tuple
				i := next_random_item.as_natural_32 \\ xt.n + 1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- 0 < i ≤ xt.n
					small_enough: i ≤ xt.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := xt [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			b := same_object_a (a)
			assert ("a = b", attached s [a, b])
			assert ("a = b ok", Properties.closed_interval_ok (s, a, b))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			b := some_object_a
			from
				m := some_object_a
			until
				m ≭ b
			loop
				m := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.max (m, b)
				b := s.min (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a > b", attached s [a, b])
			assert ("a > b ok", Properties.closed_interval_ok (s, a, b))

			s := set_to_be_tested
			a := some_object_a
			b := some_object_a
			assert ("closed_interval", attached s [a, b])
			assert ("closed_interval ok", Properties.closed_interval_ok (s, a, b))
		end

	test_left_closed_right_open_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.left_closed_right_open_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.left_closed_right_open_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.left_closed_right_open_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.left_closed_right_open_interval"
		local
			s: like set_to_be_tested
			xt: like set_to_be_tested.n_tuple_anchor
			a, b, m: A
			i: like set_to_be_tested.n_tuple_anchor.n
			chk_ops: BOOLEAN
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				xt := s.as_tuple
				i := next_random_item.as_natural_32 \\ xt.n + 1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- 0 < i ≤ xt.n
					small_enough: i ≤ xt.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				b := xt [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			from
				m := some_object_a
			until
				m ≭ b
			loop
				m := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.min (m, b)
				b := s.max (m, b)
				s := s & a
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a < b", attached s.left_closed_right_open_interval (a, b))
			assert ("a < b ok", Properties.left_closed_right_open_interval_ok (s, a, b))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			a := some_object_a
			b := same_object_a (a)
			assert ("a = b", attached s.left_closed_right_open_interval (a, b))
			assert ("a = b ok", Properties.left_closed_right_open_interval_ok (s, a, b))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			b := some_object_a
			m := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.max (m, b)
				b := s.min (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a ≥ b", attached s.left_closed_right_open_interval (a, b))
			assert ("a ≥ b ok", Properties.left_closed_right_open_interval_ok (s, a, b))

			s := set_to_be_tested
			a := some_object_a
			b := some_object_a
			assert ("left_closed_right_open_interval", attached s.left_closed_right_open_interval (a, b))
			assert ("left_closed_right_open_interval ok", Properties.left_closed_right_open_interval_ok (s, a, b))
		end

	test_left_open_right_closed_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.left_open_right_closed_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.left_open_right_closed_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.left_open_right_closed_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.left_open_right_closed_interval"
		local
			s: like set_to_be_tested
			xt: like set_to_be_tested.n_tuple_anchor
			a, b, m: A
			i: like set_to_be_tested.n_tuple_anchor.n
			chk_ops: BOOLEAN
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				xt := s.as_tuple
				i := next_random_item.as_natural_32 \\ xt.n + 1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- 0 < i ≤ xt.n
					small_enough: i ≤ xt.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				b := xt [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			from
				m := some_object_a
			until
				m ≭ b
			loop
				m := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.min (m, b)
				b := s.max (m, b)
				s := s & b
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a < b", attached s.left_open_right_closed_interval (a, b))
			assert ("a < b ok", Properties.left_open_right_closed_interval_ok (s, a, b))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			a := some_object_a
			b := same_object_a (a)
			assert ("a = b", attached s.left_open_right_closed_interval (a, b))
			assert ("a = b ok", Properties.left_open_right_closed_interval_ok (s, a, b))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			b := some_object_a
			m := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.max (m, b)
				b := s.min (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a ≥ b", attached s.left_open_right_closed_interval (a, b))
			assert ("a ≥ b ok", Properties.left_open_right_closed_interval_ok (s, a, b))

			s := set_to_be_tested
			a := some_object_a
			b := some_object_a
			assert ("left_open_right_closed_interval", attached s.left_open_right_closed_interval (a, b))
			assert ("left_open_right_closed_interval ok", Properties.left_open_right_closed_interval_ok (s, a, b))
		end

	test_open_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.open_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.open_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.open_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.open_interval"
		local
			s: like set_to_be_tested
			a, b, m: A
			chk_ops: BOOLEAN
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			b := some_object_a
			from
				m := some_object_a
			until
				m ≭ b
			loop
				m := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.min (m, b)
				b := s.max (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a < b", attached s (a, b))
			assert ("a < b ok", Properties.open_interval_ok (s, a, b))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			a := some_object_a
			b := same_object_a (a)
			assert ("a = b", attached s (a, b))
			assert ("a = b ok", Properties.open_interval_ok (s, a, b))

			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			b := some_object_a
			from
				m := some_object_a
			until
				m ≭ b
			loop
				m := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.max (m, b)
				b := s.min (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a > b", attached s (a, b))
			assert ("a > b ok", Properties.open_interval_ok (s, a, b))

			s := set_to_be_tested
			a := some_object_a
			b := some_object_a
			assert ("open_interval", attached s (a, b))
			assert ("open_interval ok", Properties.open_interval_ok (s, a, b))
		end

	test_left_closed_right_unbounded_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.left_closed_right_unbounded_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.left_closed_right_unbounded_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.left_closed_right_unbounded_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.left_closed_right_unbounded_interval"
		local
			s: like set_to_be_tested
			xt: like set_to_be_tested.n_tuple_anchor
			a: A
			i: like set_to_be_tested.n_tuple_anchor.n
			chk_ops: BOOLEAN
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				xt := s.as_tuple
				i := next_random_item.as_natural_32 \\ xt.n + 1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- 0 < i ≤ xt.n
					small_enough: i ≤ xt.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := xt [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a ∈ s", attached s.left_closed_right_unbounded_interval (a))
			assert ("a ∈ s ok", Properties.left_closed_right_unbounded_interval_ok (s, a))

			s := set_to_be_tested
			a := some_object_a
			assert ("left_closed_right_unbounded_interval", attached s.left_closed_right_unbounded_interval (a))
			assert ("left_closed_right_unbounded_interval ok", Properties.left_closed_right_unbounded_interval_ok (s, a))
		end

	test_left_open_right_unbounded_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.left_open_right_unbounded_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.left_open_right_unbounded_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.left_open_right_unbounded_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.left_open_right_unbounded_interval"
		local
			s: like set_to_be_tested
			xt: like set_to_be_tested.n_tuple_anchor
			a: A
			i: like set_to_be_tested.n_tuple_anchor.n
			chk_ops: BOOLEAN
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				xt := s.as_tuple
				i := next_random_item.as_natural_32 \\ xt.n + 1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- 0 < i ≤ xt.n
					small_enough: i ≤ xt.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := xt [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a ∈ s", attached s.left_open_right_unbounded_interval (a))
			assert ("a ∈ s ok", Properties.left_open_right_unbounded_interval_ok (s, a))

			s := set_to_be_tested
			a := some_object_a
			assert ("left_open_right_unbounded_interval", attached s.left_open_right_unbounded_interval (a))
			assert ("left_open_right_unbounded_interval ok", Properties.left_open_right_unbounded_interval_ok (s, a))
		end

	test_left_unbounded_right_closed_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.left_unbounded_right_closed_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.left_unbounded_right_closed_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.left_unbounded_right_closed_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.left_unbounded_right_closed_interval"
		local
			s: like set_to_be_tested
			xt: like set_to_be_tested.n_tuple_anchor
			b: A
			i: like set_to_be_tested.n_tuple_anchor.n
			chk_ops: BOOLEAN
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				xt := s.as_tuple
				i := next_random_item.as_natural_32 \\ xt.n + 1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- 0 < i ≤ xt.n
					small_enough: i ≤ xt.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				b := xt [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("b ∈ s", attached s.left_unbounded_right_closed_interval (b))
			assert ("b ∈ s ok", Properties.left_unbounded_right_closed_interval_ok (s, b))

			s := set_to_be_tested
			b := some_object_a
			assert ("left_unbounded_right_closed_interval", attached s.left_unbounded_right_closed_interval (b))
			assert ("left_unbounded_right_closed_interval ok", Properties.left_unbounded_right_closed_interval_ok (s, b))
		end

	test_left_unbounded_right_open_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.left_unbounded_right_open_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.left_unbounded_right_open_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.left_unbounded_right_open_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.left_unbounded_right_open_interval"
		local
			s: like set_to_be_tested
			xt: like set_to_be_tested.n_tuple_anchor
			b: A
			i: like set_to_be_tested.n_tuple_anchor.n
			chk_ops: BOOLEAN
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				xt := s.as_tuple
				i := next_random_item.as_natural_32 \\ xt.n + 1
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

				check
					positive: 0 < i -- 0 < i ≤ xt.n
					small_enough: i ≤ xt.n
				end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				b := xt [i]
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("b ∈ s", attached s.left_unbounded_right_open_interval (b))
			assert ("b ∈ s ok", Properties.left_unbounded_right_open_interval_ok (s, b))

			s := set_to_be_tested
			b := some_object_a
			assert ("left_unbounded_right_open_interval", attached s.left_unbounded_right_open_interval (b))
			assert ("left_unbounded_right_open_interval ok", Properties.left_unbounded_right_open_interval_ok (s, b))
		end

	test_unbounded_interval
			-- Test {STS_ORDERED_NUMERIC_SET}.unbounded_interval.
			-- Test {STI_ORDERED_NUMERIC_SET}.unbounded_interval.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.unbounded_interval"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.unbounded_interval"
		local
			s: like set_to_be_tested
		do
			from
				s := set_to_be_tested
			until
				not s.is_empty
			loop
				s := set_to_be_tested
			end
			assert ("s ≭ o", attached s.unbounded_interval)
			assert ("s ≭ o ok", properties.unbounded_interval_ok (s))

			s := set_to_be_tested.o
			assert ("s ≍ o", attached s.unbounded_interval)
			assert ("s ≍ o ok", properties.unbounded_interval_ok (s))

			s := set_to_be_tested
			assert ("unbounded_interval", attached s.unbounded_interval)
			assert ("unbounded_interval ok", properties.unbounded_interval_ok (s))
		end

feature -- Test routines (Element comparison)

	test_min
			-- Test {STS_ORDERED_NUMERIC_SET}.min.
			-- Test {STI_ORDERED_NUMERIC_SET}.min.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.min"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.min"
		local
			x, y: A
			s: like set_to_be_tested
		do
			from
				x := some_object_a
				y := some_object_a
			until
				x ≤ y
			loop
				x := some_object_a
				y := some_object_a
			end
			s := set_to_be_tested
			assert ("x", s.min (x, y) ≍ x)
			assert ("x ok", Properties.min_ok (s, x, y, some_object_a))

			from
				x := some_object_a
				y := some_object_a
			until
				x > y
			loop
				x := some_object_a
				y := some_object_a
			end
			s := set_to_be_tested
			assert ("y", s.min (x, y) ≍ y)
			assert ("y ok", Properties.min_ok (s, x, y, some_object_a))

			s := set_to_be_tested
			assert ("min_ok", properties.min_ok (s, some_object_a, some_object_a, some_object_a))
		end

	test_max
			-- Test {STS_ORDERED_NUMERIC_SET}.max.
			-- Test {STI_ORDERED_NUMERIC_SET}.max.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.max"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.max"
		local
			x, y: A
			s: like set_to_be_tested
		do
			from
				x := some_object_a
				y := some_object_a
			until
				x ≥ y
			loop
				x := some_object_a
				y := some_object_a
			end
			s := set_to_be_tested
			assert ("x", s.max (x, y) ≍ x)
			assert ("x ok", Properties.max_ok (s, x, y, some_object_a))

			from
				x := some_object_a
				y := some_object_a
			until
				x < y
			loop
				x := some_object_a
				y := some_object_a
			end
			s := set_to_be_tested
			assert ("y", s.max (x, y) ≍ y)
			assert ("y ok", Properties.max_ok (s, x, y, some_object_a))

			s := set_to_be_tested
			assert ("max_ok", properties.max_ok (s, some_object_a, some_object_a, some_object_a))
		end

feature -- Test routines (Predicate)

	test_anded
			-- Test {STS_ORDERED_NUMERIC_SET}.anded.
			-- Test {STI_ORDERED_NUMERIC_SET}.anded.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.anded"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.anded"
		local
			p1, p2: PREDICATE [A]
			a: A
			s: like some_set_a
			chk_ops: BOOLEAN
		do
			p1 := agent p_x
			from
				a := some_object_a
			until
				not p1 (a)
			loop
				a := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_set_a / a
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			p2 := agent s.has
			assert ("not_p1_a_and_not_p2_a", not set_to_be_tested.anded (p1, p2, a))

			s := some_set_a & a
			p2 := agent s.has
			assert ("not_p1_a_and_p2_a", not set_to_be_tested.anded (p1, p2, a))

			from
				a := some_object_a
			until
				p1 (a)
			loop
				a := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_set_a / a
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			p2 := agent s.has
			assert ("p1_a_and_not_p2_a", not set_to_be_tested.anded (p1, p2, a))

			from
				a := some_object_a
			until
				p1 (a)
			loop
				a := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
			s := some_set_a & a
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			p2 := agent s.has
			assert ("p1_a_and_p2_a", set_to_be_tested.anded (p1, p2, a))

			a := some_object_a
			p2 := agent some_set_a.has
			assert ("anded", set_to_be_tested.anded (p2, p2, a) implies True)
		end

	test_number_is_less
			-- Test {STS_ORDERED_NUMERIC_SET}.number_is_less.
			-- Test {STI_ORDERED_NUMERIC_SET}.number_is_less.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.number_is_less"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.number_is_less"
		local
			a, m, b: A
			s: like set_to_be_tested
			chk_ops: BOOLEAN
		do
			s := set_to_be_tested
			b := some_object_a
			from
				m := some_object_a
			until
				m ≭ b
			loop
				m := some_object_a
			end

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.min (m, b)
				b := s.max (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a < b", s.number_is_less (a, b))

			s := set_to_be_tested
			a := some_object_a
			b := same_object_a (a)
			assert ("a = b", not s.number_is_less (a, b))

			s := set_to_be_tested
			b := some_object_a
			m := some_object_a

			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.max (m, b)
				b := s.min (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)

			assert ("a ≥ b", not s.number_is_less (a, b))

			assert ("number_is_less", set_to_be_tested.number_is_less (some_object_a, some_object_a) implies True)
		end

	test_number_is_less_equal
			-- Test {STS_ORDERED_NUMERIC_SET}.number_is_less_equal.
			-- Test {STI_ORDERED_NUMERIC_SET}.number_is_less_equal.
		note
			testing: "covers/{STS_ORDERED_NUMERIC_SET}.number_is_less_equal"
			testing: "covers/{STI_ORDERED_NUMERIC_SET}.number_is_less_equal"
		local
			a, m, b: A
			s: like set_to_be_tested
			chk_ops: BOOLEAN
		do
			s := set_to_be_tested
			b := some_object_a
			m := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.min (m, b)
				b := s.max (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("a ≤ b", s.number_is_less_equal (a, b))

			s := set_to_be_tested
			a := some_object_a
			b := same_object_a (a)
			assert ("a = b", s.number_is_less_equal (a, b))

			s := set_to_be_tested
			b := some_object_a
			from
				m := some_object_a
			until
				m ≭ b
			loop
				m := some_object_a
			end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				a := s.max (m, b)
				b := s.min (m, b)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("a > b", not s.number_is_less_equal (a, b))

			assert ("number_is_less_equal", set_to_be_tested.number_is_less_equal (some_object_a, some_object_a) implies True)
		end

feature {NONE} -- Factory (element to be tested)

	set_to_be_tested: STI_ORDERED_NUMERIC_SET [A, EQ]
			-- <Precursor>
		deferred
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
