﻿note
	description: "Test suite for {STI_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	UNIVERSE_TESTS [A, EQ -> STS_EQUALITY [A] create default_create end]

inherit
	SET_TESTS [A, EQ]
		rename
			set_to_be_tested as universe_to_be_tested
		undefine
			test_make_empty,
			test_make_singleton,
			test_make_extended,
			test_make_from_set,
			universe_to_be_tested
		redefine
			test_is_empty,
			test_others,
			test_has,
			test_does_not_have,
			test_with,
			test_without,
			test_extended,
			test_batch_extended,
			test_is_singleton,
			test_cardinality,
			test_equals,
			test_unequals,
			test_is_subset,
			test_is_not_subset,
			test_is_superset,
			test_is_not_superset,
			test_is_comparable,
			test_is_not_comparable,
			test_is_strict_subset,
			test_is_not_strict_subset,
			test_is_strict_superset,
			test_is_not_strict_superset,
			test_is_trivial_subset,
			test_is_trivial_superset,
			test_is_proper_subset,
			test_is_proper_superset,
			test_is_disjoint,
			test_intersects,
			test_does_not_exist,
			test_exists_unique,
			test_does_not_exist_pair,
			test_for_all,
			test_for_all_pairs,
			test_for_all_distinct_pairs,
			test_complemented,
			test_united,
			test_intersected,
			test_subtracted_symmetricaly,
			set_from_sa,
			set_from_saa
		end

	SETS_TESTS_RESOURCES_UNARY [A, EQ]
		rename
			eq_a as eq
		end

feature -- Access

	properties: UNIVERSE_PROPERTIES [A, EQ]
			-- Object that checks the set-theory properties of {STI_UNIVERSE}
		deferred
		end

feature -- Test routines (Primitive)

	test_is_empty
			-- Test {STI_UNIVERSE}.is_empty.
		note
			testing: "covers/{STI_UNIVERSE}.is_empty"
		local
			u: like universe_to_be_tested
		do
			u := universe_to_be_tested
			assert ("not u.is_empty", not u.is_empty)
			assert ("is_empty_ok", properties.is_empty_ok (u))
		end

	test_others
			-- Test {STI_UNIVERSE}.others.
		note
			testing: "covers/{STI_UNIVERSE}.others"
		local
			u: like universe_to_be_tested
		do
			u := universe_to_be_tested
			assert ("others", attached u.others)
			assert ("others_ok", properties.others_ok (u))
		end

feature -- Test routines (Membership)

	test_has
			-- Test {STS_UNIVERSE}.has.
			-- Test {STI_UNIVERSE}.has.
		note
			testing: "covers/{STS_UNIVERSE}.has"
			testing: "covers/{STI_UNIVERSE}.has"
		local
			a: A
			u: like universe_to_be_tested
		do
			a := some_object_a
			u := universe_to_be_tested
			assert ("has", u ∋ a)
			assert ("has_ok", properties.does_not_have_ok (u, a))
		end

	test_does_not_have
			-- <Precursor>
			-- Test {STS_UNIVERSE}.does_not_have.
			-- Test {STI_UNIVERSE}.does_not_have.
		note
			testing: "covers/{STS_SET}.does_not_have"
			testing: "covers/{STI_SET}.does_not_have"
			testing: "covers/{STS_UNIVERSE}.does_not_have"
			testing: "covers/{STI_UNIVERSE}.does_not_have"
		local
			a: A
			u: like universe_to_be_tested
		do
			a := some_object_a
			u := universe_to_be_tested
			assert ("not (u ∌ a)", not (u ∌ a))
			assert ("does_not_have_ok", properties.does_not_have_ok (u, a))
		end

feature -- Test routines (Construction)

	test_with
			-- Test {STI_UNIVERSE}.with.
		note
			testing: "covers/{STI_UNIVERSE}.with"
		do
			Precursor {SET_TESTS}
			assert ("with_ok", properties.with_ok (universe_to_be_tested, some_object_a))
		end

	test_without
			-- <Precursor>
			-- Test {STI_UNIVERSE}.without.
		note
			testing: "covers/{STI_SET}.without"
			testing: "covers/{STI_UNIVERSE}.without"
		local
			u: like universe_to_be_tested
			a: A
		do
			a := some_object_a
			u := universe_to_be_tested
			assert ("u ∋ a", attached (u / a))
			assert ("u ∋ a ok", properties.sp_with_ok (u, a))
		end

	test_extended
			-- Test {STI_UNIVERSE}.extended.
		note
			testing: "covers/{STI_UNIVERSE}.extended"
		local
			a: A
			u: like universe_to_be_tested
		do
			a := some_object_a
			u := universe_to_be_tested
			assert ("impossible extension", u ∌ a implies attached u.extended (a))
			assert ("extended_ok", properties.extended_ok (u, a))
		end

	test_batch_extended
			-- <Precursor>
			-- Test {STI_UNIVERSE}.batch_extended.
		note
			testing: "covers/{STI_SET}.batch_extended"
			testing: "covers/{STI_UNIVERSE}.batch_extended"
		local
			s: like some_set_a
			u: like universe_to_be_tested
		do
			s := some_set_a
			u := universe_to_be_tested
			assert ("impossible extension", u.is_disjoint (s) implies attached u.batch_extended (s))
			assert ("batch_extended_ok", properties.batch_extended_ok (s, u))
		end

feature -- Test routines (Quality)

	test_is_singleton
			-- <Precursor>
			-- Test {STS_UNIVERSE}.is_singleton.
			-- Test {STI_UNIVERSE}.is_singleton.
		note
			testing: "covers/{STS_SET}.is_singleton"
			testing: "covers/{STI_SET}.is_singleton"
			testing: "covers/{STS_UNIVERSE}.is_singleton"
			testing: "covers/{STI_UNIVERSE}.is_singleton"
		local
			u: like universe_to_be_tested
		do
			u := universe_to_be_tested
			assert ("is_singleton", u.is_singleton implies True)
			assert ("is_singleton_ok", properties.is_singleton_ok (u))
		end

feature -- Test routines (Measurement)

	test_cardinality
			-- Test {STI_UNIVERSE}.cardinality.
		note
			testing: "covers/{STI_UNIVERSE}.cardinality"
		local
			u: like universe_to_be_tested
			k: like some_set.cardinality
			a1, a2: A
		do
			u := universe_to_be_tested
			assert ("cardinality", attached u.cardinality)
			assert ("cardinality_ok", properties.cardinality_ok (u))

			k := # u
			a1 := some_object_a
			u := u & same_object_a (a1)
			assert ("u.cardinality = k", u.cardinality = k)
			assert ("u.cardinality = k ok", properties.cardinality_ok (u))

			from
				a2 := some_object_a
			until
				not eq (a1, a2)
			loop
				a2 := some_object_a
			end
			a1 := a2
			u := u & same_object_a (a1)
			assert ("u.cardinality = k again", u.cardinality = k)
			assert ("u.cardinality = k again ok", properties.cardinality_ok (u))

			from
				a2 := some_object_a
			until
				not eq (a1, a2)
			loop
				a2 := some_object_a
			end
			a1 := a2
			u := u & same_object_a (a1)
			assert ("u.cardinality = k still", u.cardinality = k)
			assert ("u.cardinality = k still ok", properties.cardinality_ok (u))
		end

feature -- Test routines (Comparison)

	test_equals
			-- Test {STI_UNIVERSE}.equals.
		note
			testing: "covers/{STI_UNIVERSE}.equals"
		local
			u: like universe_to_be_tested
			s: like some_set_a
			a: A
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			Precursor {SET_TESTS}

			u := universe_to_be_tested
			s := same_set_a (u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("u ≍ s", u ≍ s)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("u ≍ s ok", properties.equals_ok (u, s))

			u := universe_to_be_tested
			a := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_set_a / a
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (u ≍ s)", not (u ≍ s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (u ≍ s) ok", properties.equals_ok (u, s))

			u := universe_to_be_tested
			s := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("equals", u ≍ s implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("equals ok", properties.equals_ok (u, s))
		end

	test_unequals
			-- <Precursor>
			-- Test {STS_UNIVERSE}.unequals.
			-- Test {STI_UNIVERSE}.unequals.
		note
			testing: "covers/{STS_SET}.unequals"
			testing: "covers/{STI_SET}.unequals"
			testing: "covers/{STS_UNIVERSE}.unequals"
			testing: "covers/{STI_UNIVERSE}.unequals"
		local
			a: A
			u: like universe_to_be_tested
			s: like some_set_a
		do
			u := universe_to_be_tested
			assert ("not (u ≭ u)", not (u ≭ u))
			assert ("not (u ≭ u) ok", properties.unequals_ok (u, u))

			a := some_object_a
			u := universe_to_be_tested & same_object_a (a)
			s := some_set_a / same_object_a (a)
			assert ("u ≭ s", u ≭ s)
			assert ("u ≭ s ok", properties.unequals_ok (u, s))

			u := universe_to_be_tested
			s := same_set_a (u)
			assert ("not (u ≭ s)", not (u ≭ s))
			assert ("not (u ≭ s) ok", properties.unequals_ok (u, s))

			u := universe_to_be_tested
			s := some_set_a
			assert ("unequals", u ≭ s ⇒ True)
			assert ("unequals_ok", properties.unequals_ok (u, s))
		end

	test_is_subset
			-- Test {STI_UNIVERSE}.is_subset.
		note
			testing: "covers/{STI_UNIVERSE}.is_subset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
			a: A
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			u := universe_to_be_tested
			s := same_set_a (u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("u ⊆ s", u ⊆ s)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("u ⊆ s; s2 ok", properties.sp_is_subset_ok (u, s, some_set_a))
			assert ("u ⊆ s ok", properties.is_subset_ok (u, s))

			u := universe_to_be_tested
			a := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_set_a / a
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (u ⊆ s)", not (u ⊆ s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (u ⊆ s); s2 ok", properties.sp_is_subset_ok (u, s, some_set_a))
			assert ("not (u ⊆ s) ok", properties.is_subset_ok (u, s))

			u := universe_to_be_tested
			s := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("is_subset", u ⊆ s implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("sp_is_subset_ok", properties.sp_is_subset_ok (u, s, some_set_a))
			assert ("is_subset_ok", properties.is_subset_ok (u, s))
		end

	test_is_not_subset
			-- <Precursor>
			-- Test {STS_UNIVERSE}.is_not_subset.
			-- Test {STI_UNIVERSE}.is_not_subset.
		note
			testing: "covers/{STS_SET}.is_not_subset"
			testing: "covers/{STI_SET}.is_not_subset"
			testing: "covers/{STS_UNIVERSE}.is_not_subset"
			testing: "covers/{STI_UNIVERSE}.is_not_subset"
		local
			u: like universe_to_be_tested
			s2: like some_set_a
		do
			u := universe_to_be_tested
			s2 := some_set_a / some_object_a
			assert ("u ⊈ s2", u ⊈ s2)
			assert ("u ⊈ s2 ok", properties.is_not_subset_ok (u, s2))

			u := universe_to_be_tested
			s2 := same_set_a (u)
			assert ("not (u ⊈ s2)", u ⊈ s2)
			assert ("not (u ⊈ s2) ok", properties.is_not_subset_ok (u, s2))

			u := universe_to_be_tested
			s2 := some_set_a
			assert ("is_not_subset", u ⊈ s2 ⇒ True)
			assert ("is_not_subset ok", properties.is_not_subset_ok (u, s2))
		end

	test_is_superset
			-- Test {STI_UNIVERSE}.is_superset.
		note
			testing: "covers/{STI_UNIVERSE}.is_superset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
		do
			Precursor {SET_TESTS}

			u := universe_to_be_tested
			s := some_set_a
			assert ("u ⊇ s", u ⊇ s)
			assert ("u ⊇ s ok", properties.is_superset_ok (u, s))
		end

	test_is_not_superset
			-- <Precursor>
			-- Test {STS_UNIVERSE}.is_not_superset.
			-- Test {STI_UNIVERSE}.is_not_superset.
		note
			testing: "covers/{STS_SET}.is_not_superset"
			testing: "covers/{STI_SET}.is_not_superset"
			testing: "covers/{STS_UNIVERSE}.is_not_superset"
			testing: "covers/{STI_UNIVERSE}.is_not_superset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
		do
			u := universe_to_be_tested
			s := some_set_a
			assert ("not (u ⊉ s)", not (u ⊉ s))
			assert ("is_not_superset_ok", properties.is_not_superset_ok (u, s))
		end

	test_is_comparable
			-- Test {STI_UNIVERSE}.is_comparable.
		note
			testing: "covers/{STI_UNIVERSE}.is_comparable"
		local
			u: like universe_to_be_tested
			s: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like universe_to_be_tested.Cardinality
		do
			u := universe_to_be_tested
			s := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # u ∨ # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("u ⊥ s", u.is_comparable (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("u ⊥ s ok", properties.is_comparable_ok (u, s))
		end

	test_is_not_comparable
			-- Test {STI_UNIVERSE}.is_not_comparable.
		note
			testing: "covers/{STI_UNIVERSE}.is_not_comparable"
		local
			u: like universe_to_be_tested
			s: like some_set_a
		do
			u := universe_to_be_tested
			s := some_set_a
			assert ("not u.is_not_comparable (s)", not u.is_not_comparable (s))
			assert ("not u.is_not_comparable (s) ok", properties.is_not_comparable_ok (u, s))
		end

	test_is_strict_subset
			-- Test {STI_UNIVERSE}.is_strict_subset.
		note
			testing: "covers/{STI_UNIVERSE}.is_strict_subset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
		do
			u := universe_to_be_tested
			s := some_set_a
			assert ("not (u ⊂ s)", not (u ⊂ s))
			assert ("sp_is_strict_subset_ok", properties.sp_is_strict_subset_ok (u, s, some_set_a))
			assert ("is_strict_subset_ok", properties.is_strict_subset_ok (u, s))
		end

	test_is_not_strict_subset
			-- <Precursor>
			-- Test {STS_UNIVERSE}.is_not_strict_subset.
			-- Test {STI_UNIVERSE}.is_not_strict_subset.
		note
			testing: "covers/{STS_SET}.is_not_strict_subset"
			testing: "covers/{STI_SET}.is_not_strict_subset"
			testing: "covers/{STS_UNIVERSE}.is_not_strict_subset"
			testing: "covers/{STI_UNIVERSE}.is_not_strict_subset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
		do
			u := universe_to_be_tested
			s := some_set_a
			assert ("is_not_strict_subset", u ⊄ s)
			assert ("is_not_strict_subset_ok", properties.is_not_strict_subset_ok (u, s))
		end

	test_is_strict_superset
			-- Test {STI_UNIVERSE}.is_strict_superset.
		note
			testing: "covers/{STI_UNIVERSE}.is_strict_superset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			u := universe_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_set_a / some_object_a
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("u ⊃ s", u ⊃ s)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("u ⊃ s ok", properties.is_strict_superset_ok (u, s))

			u := universe_to_be_tested
			s := same_set_a (u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (u ⊃ s)", not (u ⊃ s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (u ⊃ s) ok", properties.is_strict_superset_ok (u, s))

			u := universe_to_be_tested
			s := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("is_strict_superset", u ⊃ s implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("is_strict_superset_ok", properties.is_strict_superset_ok (u, s))
		end

	test_is_not_strict_superset
			-- <Precursor>
			-- Test {STS_UNIVERSE}.is_not_strict_superset.
			-- Test {STI_UNIVERSE}.is_not_strict_superset.
		note
			testing: "covers/{STS_SET}.is_not_strict_superset"
			testing: "covers/{STI_SET}.is_not_strict_superset"
			testing: "covers/{STS_UNIVERSE}.is_not_strict_superset"
			testing: "covers/{STI_UNIVERSE}.is_not_strict_superset"
		local
			a: A
			u: like universe_to_be_tested
			s: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			u := universe_to_be_tested
			s := same_set_a (u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("u ⊅ s", u ⊅ s)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("u ⊅ s ok", properties.is_not_strict_superset_ok (u, s))

			u := universe_to_be_tested
			a := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_set_a / a
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not (u ⊅ s)", not (u ⊅ s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not (u ⊅ s) ok", properties.is_not_strict_superset_ok (u, s))

			u := universe_to_be_tested
			s := some_set_a
			assert ("is_not_strict_superset", u ⊅ s implies True)
			assert ("is_not_strict_superset_ok", properties.is_not_strict_superset_ok (u, s))
		end

	test_is_trivial_subset
			-- Test {STI_UNIVERSE}.is_trivial_subset.
		note
			testing: "covers/{STI_UNIVERSE}.is_trivial_subset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
			a: like some_object_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			u := universe_to_be_tested
			s := same_set_a (u)
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("u.is_trivial_subset (s)", u.is_trivial_subset (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("u.is_trivial_subset (s) ok", properties.is_trivial_subset_ok (u, s))

			u := universe_to_be_tested
			a := some_object_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_set_a / a
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not u.is_trivial_subset (s)", not u.is_trivial_subset (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not u.is_trivial_subset (s) ok", properties.is_trivial_subset_ok (u, s))

			u := universe_to_be_tested
			s := some_set_a
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("is_trivial_subset", u.is_trivial_subset (s) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("is_trivial_subset_ok", properties.is_trivial_subset_ok (u, s))
		end

	test_is_trivial_superset
			-- Test {STI_UNIVERSE}.is_trivial_superset.
		note
			testing: "covers/{STI_UNIVERSE}.is_trivial_superset"
		local
			u: like universe_to_be_tested
			s2: like some_set_a
		do
			u := universe_to_be_tested
			if next_random_item \\ 2 = 0 then
				s2 := same_set_a (u)
			else
				s2 := some_set_a.o
			end
			assert ("u.is_trivial_superset (s2)", u.is_trivial_superset (s2))
			assert ("u.is_trivial_superset (s2) ok", properties.is_trivial_superset_ok (u, s2))

			from
				s2 := some_set_a / some_object_a
			until
				not s2.is_empty
			loop
				s2 := some_set_a / some_object_a
			end
			u := universe_to_be_tested
			assert ("not u.is_trivial_superset (s2)", not u.is_trivial_superset (s2))
			assert ("not u.is_trivial_superset (s2) ok", properties.is_trivial_superset_ok (u, s2))

			u := universe_to_be_tested
			s2 := some_set_a
			assert ("is_trivial_superset", u.is_trivial_superset (s2) ⇒ True)
			assert ("is_trivial_superset_ok", properties.is_trivial_superset_ok (u, s2))
		end

	test_is_proper_subset
			-- Test {STI_UNIVERSE}.is_proper_subset.
		note
			testing: "covers/{STI_UNIVERSE}.is_proper_subset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
		do
			u := universe_to_be_tested
			s := some_set_a
			assert ("not u.is_proper_subset (s)", not u.is_proper_subset (s))
			assert ("is_proper_subset_ok", properties.is_proper_subset_ok (u, s))
		end

	test_is_proper_superset
			-- Test {STI_UNIVERSE}.is_proper_superset.
		note
			testing: "covers/{STI_UNIVERSE}.is_proper_superset"
		local
			u: like universe_to_be_tested
			s: like some_set_a
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			u := universe_to_be_tested
			if next_random_item \\ 2 = 0 then
				s := same_set_a (u)
			else
				s := u.o
			end
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("not u.is_proper_superset (s)", not u.is_proper_superset (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("not u.is_proper_superset (s) ok", properties.is_proper_superset_ok (u, s))

			u := universe_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				from
					s := some_set_a / some_object_a
				until
					not s.is_empty
				loop
					s := some_set_a / some_object_a
				end
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("u.is_proper_superset (s)", u.is_proper_superset (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("u.is_proper_superset (s) ok", properties.is_proper_superset_ok (u, s))

			u := universe_to_be_tested
			s := some_set_a
			assert ("is_proper_superset", u.is_proper_superset (s) implies True)
			assert ("is_proper_superset ok", properties.is_proper_superset_ok (u, s))
		end

	test_is_disjoint
			-- Test {STI_UNIVERSE}.is_disjoint.
		note
			testing: "covers/{STI_UNIVERSE}.is_disjoint"
		local
			u: like universe_to_be_tested
			s: like some_set_a
		do
			Precursor {SET_TESTS}

			u := universe_to_be_tested
			s := some_set_a.o -- TODO: Make an o?
			assert ("u.is_disjoint (s)", u.is_disjoint (s))
			assert ("u.is_disjoint (s) ok", properties.is_disjoint_ok (u, s))

			u := universe_to_be_tested
			from
				s := some_set_a
			until
				not s.is_empty
			loop
				s := some_set_a
			end
			assert ("not u.is_disjoint (s)", not u.is_disjoint (s))
			assert ("not u.is_disjoint (s) ok", properties.is_disjoint_ok (u, s))

			u := universe_to_be_tested
			s := some_set_a
			assert ("is_disjoint", u.is_disjoint (s) implies True)
			assert ("is_disjoint_ok", properties.is_disjoint_ok (u, s))
		end

	test_intersects
			-- <Precursor>
			-- Test {STS_UNIVERSE}.intersects.
			-- Test {STI_UNIVERSE}.intersects.
		note
			testing: "covers/{STS_SET}.intersects"
			testing: "covers/{STI_SET}.intersects"
			testing: "covers/{STS_UNIVERSE}.intersects"
			testing: "covers/{STI_UNIVERSE}.intersects"
		local
			u: like universe_to_be_tested
			s2: like some_set_a
		do
			u := universe_to_be_tested
			s2 := some_set_a & some_object_a
			assert ("u.intersects (s2)", u.intersects (s2))
			assert ("u.intersects (s2) ok", properties.intersects_ok (u, s2))

			u := universe_to_be_tested
			s2 := some_set_a.o
			assert ("not u.intersects (s2)", not u.intersects (s2))
			assert ("not u.intersects (s2) ok", properties.intersects_ok (u, s2))

			u := universe_to_be_tested
			s2 := some_set_a
			assert ("intersects", u.intersects (s2) ⇒ True)
			assert ("intersects_ok", properties.intersects_ok (u, s2))
		end

feature -- Test routines (Quantifier)

	test_does_not_exist
			-- <Precursor>
			-- Test {STS_UNIVERSE}.does_not_exist.
			-- Test {STI_UNIVERSE}.does_not_exist.
		note
			testing: "covers/{STS_SET}.does_not_exist"
			testing: "covers/{STI_SET}.does_not_exist"
			testing: "covers/{STS_UNIVERSE}.does_not_exist"
			testing: "covers/{STI_UNIVERSE}.does_not_exist"
		local
			u: like universe_to_be_tested
			p: PREDICATE [A]
		do
			p := agent p_x
			u := universe_to_be_tested
			assert ("does_not_exist", u |∄ p implies True)
			assert ("does_not_exist_ok", properties.does_not_exist_ok (u, p))
		end

	test_exists_unique
			-- <Precursor>
			-- Test {STI_UNIVERSE}.exists_unique.
		note
			testing: "covers/{STI_SET}.exists_unique"
			testing: "covers/{STI_UNIVERSE}.exists_unique"
		local
			u: like universe_to_be_tested
			p: PREDICATE [A]
		do
			p := agent p_x
			u := universe_to_be_tested
			assert ("exists_unique", u |∃! p implies True)
			assert ("exists_unique_ok", properties.exists_unique_ok (u, p))
		end

	test_does_not_exist_pair
			-- <Precursor>
			-- Test {STS_UNIVERSE}.does_not_exist_pair.
			-- Test {STI_UNIVERSE}.does_not_exist_pair.
		note
			testing: "covers/{STS_SET}.does_not_exist_pair"
			testing: "covers/{STI_SET}.does_not_exist_pair"
			testing: "covers/{STS_UNIVERSE}.does_not_exist_pair"
			testing: "covers/{STI_UNIVERSE}.does_not_exist_pair"
		local
			p: PREDICATE [A, A]
			u: like universe_to_be_tested
			chk_n, chk_tests: BOOLEAN
			n: like universe_to_be_tested.Cardinality
		do
			p := agent p_x1_x2
			u := universe_to_be_tested
			chk_n := {ISE_RUNTIME}.check_assert (False)
				n := # u
			chk_n := {ISE_RUNTIME}.check_assert (chk_n)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("does_not_exist_pair", u.does_not_exist_pair (p) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("does_not_exist_pair_ok", properties.does_not_exist_pair_ok (u, p))
		end

	test_for_all
			-- <Precursor>
			-- Test {STI_UNIVERSE}.for_all.
		note
			testing: "covers/{STI_SET}.for_all"
			testing: "covers/{STI_UNIVERSE}.for_all"
		local
			u: like universe_to_be_tested
			p: PREDICATE [A]
			chk_ops, chk_tests: BOOLEAN
			n: like universe_to_be_tested.Cardinality
		do
			p := agent p_x
			u := universe_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # u
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("for_all", (u |∀ p) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("for_all_ok", properties.for_all_ok (u, p))
		end

	test_for_all_pairs
			-- <Precursor>
			-- Test {STI_UNIVERSE}.for_all_pairs.
		note
			testing: "covers/{STI_SET}.for_all_pairs"
			testing: "covers/{STI_UNIVERSE}.for_all_pairs"
		local
			u: like universe_to_be_tested
			p: PREDICATE [A, A]
			chk_ops, chk_tests: BOOLEAN
			n: like universe_to_be_tested.Cardinality
		do
			p := agent p_x1_x2
			u := universe_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # u
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("for_all_pairs", u.for_all_pairs (p) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("for_all_pairs_ok", properties.for_all_pairs_ok (u, p))
		end

	test_for_all_distinct_pairs
			-- <Precursor>
			-- Test {STI_SET}.for_all_distinct_pairs.
		note
			testing: "covers/{STI_SET}.for_all_distinct_pairs"
			testing: "covers/{STI_UNIVERSE}.for_all_distinct_pairs"
		local
			u: like universe_to_be_tested
			p: PREDICATE [A, A]
			chk_ops, chk_tests: BOOLEAN
			n: like universe_to_be_tested.Cardinality
		do
			p := agent p_x1_x2
			u := universe_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # u
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("for_all_distinct_pairs", u.for_all_distinct_pairs (p) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("for_all_distinct_pairs_ok", properties.for_all_distinct_pairs_ok (u, p))
		end

feature -- Test routines (Operation)

	test_complemented
			-- Test {STI_UNIVERSE}.complemented.
		note
			testing: "covers/{STI_UNIVERSE}.complemented"
		do
			Precursor {SET_TESTS}

			assert ("u ∁ s ok", properties.complemented_ok (universe_to_be_tested, same_set_a (universe_to_be_tested)))
			assert ("complemented_ok", properties.complemented_ok (universe_to_be_tested, some_set_a))
		end

	test_united
			-- Test {STI_UNIVERSE}.united.
		note
			testing: "covers/{STI_UNIVERSE}.united"
		do
			Precursor {SET_TESTS}
			assert ("united_ok", properties.united_ok (universe_to_be_tested, some_set_a))
		end

	test_intersected
			-- Test {STI_UNIVERSE}.intersected.
		note
			testing: "covers/{STI_UNIVERSE}.intersected"
		do
			Precursor {SET_TESTS}
			assert ("intersected_ok", properties.intersected_ok (universe_to_be_tested, some_set_a))
		end

	test_subtracted_symmetricaly
			-- Test {STI_UNIVERSE}.subtracted_symmetricaly.
		note
			testing: "covers/{STI_UNIVERSE}.subtracted_symmetricaly"
		do
			Precursor {SET_TESTS}
			assert ("subtracted_symmetricaly_ok", properties.subtracted_symmetricaly_ok (universe_to_be_tested, some_set_a))
		end

feature {NONE} -- Factory (element to be tested)

	universe_to_be_tested: STI_UNIVERSE [A, EQ]
			-- Universe meant to be under tests
		deferred
		end

feature {NONE} -- Conversion

	set_from_sa (sa: STS_SET [A, EQ]): like some_set_a
			-- `sa' converted to a set like `some_set_a'
		do
			Result := sa
		end

	set_from_saa (saa: STS_TRANSFORMABLE_SET [A, A, EQ, EQ]): like some_set_a
			-- `saa' converted to a set like `some_set_a'
		do
			Result := saa
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
