﻿note
	description: "Implementation of {STS_COMPLEX}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

expanded
class -- todo: why not expanded?
	STI_COMPLEX

inherit

	STS_COMPLEX
		rename
				-- Primitive
			a as real,
			b as imaginary
		redefine
				-- Initialization
			default_create,
				-- Output
			out,
				-- Basic operations
			opposite,
--			reciprocal, inverse,
				-- Anchor
			complex_anchor,
			real_anchor
		end

	DEBUG_OUTPUT
		rename
				-- Output
			debug_output as out
		redefine
				-- Initialization
			default_create,
				-- Output
			out
		end

create
	default_create,
	make,
	make_from_native_real,
	make_from_normalized_native_real,
	make_from_native_real_reference,
	make_from_native_real_64,
	make_from_native_real_64_reference,
	make_from_real,
	make_from_rational,
	make_from_native_integer,
	make_from_native_integer_reference,
	make_from_integer,
	make_from_native_natural,
	make_from_native_natural_reference,
	make_from_natural

convert
	make_from_native_real ({REAL}),
	make_from_native_real_reference ({REAL_REF}),
	make_from_native_real_64 ({REAL_64}),
	make_from_native_real_64_reference ({REAL_64_REF}),
	make_from_real ({STS_REAL}),
	make_from_rational ({STS_RATIONAL}),
	make_from_native_integer ({INTEGER}),
	make_from_native_integer_reference({INTEGER_REF}),
	make_from_integer ({STS_INTEGER}),
	make_from_native_natural ({NATURAL}),
	make_from_native_natural_reference ({NATURAL_32_REF}),
	make_from_natural ({STS_NATURAL})

feature {NONE} -- Initialization

	default_create
			-- Create the complex number 0 + 0i
		do
			a := Zero
			b := Zero
		ensure then
			real: a ≍ Zero
			imaginary: b ≍ Zero
		end

	make (a_real, a_imaginary: STS_REAL)
			-- Create the complex number `a_real' + `a_imaginary'i.
		do
			a := converted_real (a_real)
			b := converted_real (a_imaginary)
		ensure
			real: a ≍ a_real
			imaginary: b ≍ a_imaginary
		end

	make_from_native_real (v: REAL)
			-- Create a complex number with `real'.`value' = {STI_REAL}.normalized_value (`v') and
			-- `imaginary' = 0
		do
			a := v
			b := Zero
		ensure
			real: {STI_REAL}.same_value (a.value, {STI_REAL}.normalized_value (v))
			imaginary: b ≍ Zero
		end

	make_from_normalized_native_real (v: REAL)
			-- Create the comnplex number `v' + 0i
		require
			normalized_value: {STI_REAL}.same_value (v, {STI_REAL}.normalized_value (v))
		do
			create a.make_from_normalized_value (v)
			b := Zero
		ensure
			real: {STI_REAL}.same_value (a.value, v)
			imaginary: b ≍ Zero
		end

	make_from_native_real_reference (v_ref: REAL_REF)
			-- Create a complex number with
			-- `real'.`value' = {STI_REAL}.normalized_value (`v_ref'.`item') and `imaginary' = 0
		do
			create a.make_from_native_real_reference (v_ref)
			b := Zero
		ensure
			real: {STI_REAL}.same_value (a.value, {STI_REAL}.normalized_value (v_ref.item))
			imaginary: b ≍ Zero
		end

	make_from_native_real_64 (v: REAL_64)
			-- Create a complex number with
			-- `real'.`value' = {STI_REAL}.normalized_value (`v'.`truncated_to_real') and
			-- `imaginary' = 0
		do
			a := v
			b := Zero
		ensure
			real: {STI_REAL}.same_value (a.value, {STI_REAL}.normalized_value (v.truncated_to_real))
			imaginary: b ≍ Zero
		end

	make_from_native_real_64_reference (v_ref: REAL_64_REF)
			-- Create a complex number with
			-- `real'.`value' = {STI_REAL}.normalized_value (`v_ref'.`truncated_to_real') and
			-- `imaginary' = 0
		do
			a := v_ref
			b := Zero
		ensure
			real: {STI_REAL}.same_value
				(a.value, {STI_REAL}.normalized_value (v_ref.truncated_to_real))
			imaginary: b ≍ Zero
		end

	make_from_real (r: STS_REAL)
			-- Create the complex number `r' + 0i.
		do
			a := converted_real (r)
			b := Zero
		ensure
			real: a ≍ r
			imaginary: b ≍ Zero
		end

	make_from_rational (r: STS_RATIONAL)
			-- Create the complex number `r' + 0i.
		do
			a := r
			b := Zero
		ensure
			real: a ≍ r
			imaginary: b ≍ Zero
		end

	make_from_native_integer (v: INTEGER)
			-- Create a complex number with
			-- `real'.`value' = {STI_REAL}.normalized_value (`v'.`to_real') and `imaginary' = 0
		do
			a := v
			b := Zero
		ensure
			real: {STI_REAL}.same_value (a.value, {STI_REAL}.normalized_value (v))
			imaginary: b ≍ Zero
		end

	make_from_native_integer_reference (v_ref: INTEGER_REF)
			-- Create a complex number with
			-- `real'.`value' = {STI_REAL}.normalized_value (`v_ref'.`to_real') and `imaginary' = 0
		do
			a := v_ref
			b := Zero
		ensure
			real: {STI_REAL}.same_value (a.value, {STI_REAL}.normalized_value (v_ref.to_real))
			imaginary: b ≍ Zero
		end

	make_from_integer (j: STS_INTEGER)
			-- Create the complex number `j' + 0i.
		do
			a := j
			b := Zero
		ensure
			real: a ≍ j
			imaginary: b ≍ Zero
		end

	make_from_native_natural (v: NATURAL)
			-- Create a complex number with
			-- `real'.`value' = {STI_REAL}.normalized_value (`v'.`to_real_32') and `imaginary' = 0
		do
			a := v
			b := Zero
		ensure
			real: {STI_REAL}.same_value (a.value, {STI_REAL}.normalized_value (v))
			imaginary: b ≍ Zero
		end

	make_from_native_natural_reference (v_ref: NATURAL_32_REF)
			-- Create a complex number with
			-- `real'.`value' = {STI_REAL}.normalized_value (`v_ref'.`to_real_32') and
			-- `imaginary' = 0
		do
			a := v_ref
			b := Zero
		ensure
			real: {STI_REAL}.same_value (a.value, {STI_REAL}.normalized_value (v_ref.to_real_32))
			imaginary: b ≍ Zero
		end

	make_from_natural (n: STS_NATURAL)
			-- Create the complex number `n' + 0i.
		do
			a := n
			b := Zero
		ensure
			real: a ≍ n
			imaginary: b ≍ Zero
		end

feature -- Primitive

	a: like real_anchor
			-- Real part of this number

	real: like real_anchor
			-- <Precursor>
		do
			Result := a
		end

	b: like real_anchor
			-- Imaginary part of this number

	imaginary: like real_anchor
			-- <Precursor>
		do
			Result := b
		end

feature -- Access

	Zero: STI_NATURAL
			-- <Precursor>
		once
			Result := {STI_NATURAL}.Zero
		ensure then
			class
		end

	One: STI_NATURAL
			-- <Precursor>
		once
			Result := {STI_NATURAL}.One
		ensure then
			class
		end

feature -- Status report

	exponentiable (other: NUMERIC): BOOLEAN
			-- <Precursor>
		do
		ensure then
			class
			never: not Result
		end

feature -- Output

	out: STRING
			-- <Precursor>
		do
			if b >= Zero then
				Result := a.out + "+" + b.out + "i"
			else
				Result := a.out + "-" + b.abs.out + "i"
			end
		ensure then
			non_negative_imaginary: b >= Zero implies Result ~ a.out + "+" + b.out + "i"
			negative_imaginary: b < Zero implies Result ~ a.out + "-" + b.abs.out + "i"
		end

feature -- Basic operations

	conjugate: like complex_anchor
			-- <Precursor>
		do
			create Result.make (a, - b)
		end

	plus alias "+" (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			create Result.make (a + other.a, b + other.b)
		end

	minus alias "-" (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			create Result.make (a - other.a, b - other.b)
		end

	opposite alias "-": like complex_anchor
			-- <Precursor>
		do
			create Result.make (- a, - b)
		end

	product alias "*" (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			create Result.make (a * other.a - b * other.b, b * other.a + a * other.b)
		end

--	quotient alias "/" (other: STS_COMPLEX): like complex_anchor
--			-- <Precursor>
--		local
--			other_abs_square: STS_REAL
--		do
--			other_abs_square := other.a * other.a + other.b * other.b
--				check
--						-- divisible (other)
--					real_divisible: (a * other.a + b * other.b).divisible (other_abs_square)
--					imaginary_divisible: (b * other.a - a * other.b).divisible (other_abs_square)
--				end
--			create Result.make (
--				(a * other.a + b * other.b) / other_abs_square,
--				(b * other.a - a * other.b) / other_abs_square
--				)
--		end

	reciprocal,
	inverse alias "⁻¹": like complex_anchor
			-- <Precursor>
		local
			abs_square: STS_REAL
		do
			if a ≭ zero then
				if b ≭ zero then
					abs_square := a * a + b * b
						check
								-- `is_invertible' definition
							a.divisible (abs_square)
							(- b).divisible (abs_square)
						end
					create Result.make (a / abs_square, - b / abs_square)
				else
						check
								-- `is_invertible' definition
							a.divisible (a)
							(a / a).divisible (a)
							(- b).divisible (a)
							(- b / a).divisible (a)
						end
					create Result.make (a / a / a, - b / a / a)
				end
			else
					check
							-- `is_invertible' definition
						b ≭ zero
						a.divisible (b)
						(a / b).divisible (b)
						(- b).divisible (b)
						(- b / b).divisible (b)
					end
				create Result.make (a / b / b, - b / b / b)
			end
		end

feature -- Factory

	converted_real (r: STS_REAL): like real_anchor
			-- `r' converted to a real like `real_anchor'
		do
			if attached {like real_anchor} r as cr then
				Result := cr
			else
				Result := r
			end
		ensure
			class
			definition: Result ≍ r
		end

feature -- Anchor

	complex_anchor: STI_COMPLEX
			-- <Precursor>
		do
			Result := Current
		end

	real_anchor: STI_REAL
			-- <Precursor>
		do
			Result := a
		end

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
