﻿note
	description: "Implementation of {STS_INTEGER}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

expanded
class -- todo: why not expanded?
	STI_INTEGER

inherit

	STS_INTEGER
		redefine
				-- Access
--			sign,
				-- Output
			out,
				-- Comparison
--			real_three_way_comparison,
--			rational_three_way_comparison,
			three_way_comparison,
				-- Basic operations
			abs,
			opposite
--			power,
				-- Arithmetic
--			gcd,
				-- Anchor
--			complex_anchor,
--			real_anchor,
--			rational_anchor,
--			integer_anchor,
--			natural_anchor
		end

	DEBUG_OUTPUT
		rename
				-- Output
			debug_output as out
		undefine
				-- Comparison
			is_equal
		redefine
				-- Output
			out
		end

create
	default_create,
	make,
	make_from_reference,
	make_from_integer,
	make_from_native_natural,
	make_from_native_natural_reference
--	make_from_natural

convert
	make ({INTEGER}),
	make_from_reference ({INTEGER_REF}),
	make_from_integer ({STS_INTEGER}),
	make_from_native_natural ({NATURAL}),
	make_from_native_natural_reference ({NATURAL_32_REF}),
--	make_from_natural ({STS_NATURAL}),
	value: {INTEGER, INTEGER_REF, HASHABLE},
	as_rational: {STI_RATIONAL},
	as_real: {STI_REAL},
--	real_value: {REAL, REAL_REF},
	as_complex: {STI_COMPLEX}

feature {NONE} -- Initialization

	make (v: INTEGER)
			-- Create an integer number with value `v'.
		do
			value := normalized_value (v)
		ensure
			value: value = normalized_value (v)
		end

	make_from_reference (v_ref: INTEGER_REF)
			-- Create an integer number with value `v_ref'.`item'.
		do
			make (v_ref.item)
		ensure
			value: value = normalized_value (v_ref.item)
		end

	make_from_integer (i: STS_INTEGER)
			-- Create an integer number with value `i'.`value'.
		do
			make (i.value)
		ensure
			value: value = normalized_value (i.value)
		end

	make_from_native_natural (v: NATURAL)
			-- Create an integer number with value `v'.`as_integer_32'
		do
			make (v.as_integer_32)
		ensure
			value: value = normalized_value (v.as_integer_32)
		end

	make_from_native_natural_reference (v_ref: NATURAL_32_REF)
			-- Create an integer number with value `v_ref'.`as_integer_32'
		do
			make_from_native_natural (v_ref.item)
		ensure
			value: value = normalized_value (v_ref.as_integer_32)
		end

--	make_from_natural (n: STS_NATURAL)
--			-- Create an integer number with value `n'.`integer_value'.
--		do
--			make_from_integer (n)
--		ensure
--			value: value = normalized_value (n.integer_value)
--		end

feature -- Primitive

--	a,
--	real,
--	conjugate: like real_anchor
--			-- <Precursor>
--			-- This very integer number
--		do
--			Result := Current
--		end

--	real_value: REAL
--			-- <Precursor>
--		do
--			Result := {STI_REAL}.normalized_value (value)
--		end

--	p,
--	numerator: like Current
--			-- <Precursor>
--			-- This very integer number
--		do
--			Result := Current
--		end

	value: INTEGER
			-- <Precursor>

feature -- Access

	sign: STI_INTEGER
			-- Sign of this number as an integer value (0, -1 or 1)
		do
			Result := three_way_comparison (zero)
		ensure
			three_way: Result ≍ three_way_comparison (zero)
		end

	B,
	Imaginary,
	Zero: STI_INTEGER
			-- <Precursor>
		once
			create Result
		ensure then
			class
		end

	Q,
	Denominator,
	One: STI_INTEGER
			-- <Precursor>
		once
			Result := 1
		ensure then
			class
		end

	Min_value: INTEGER = - 8
			-- Minimum value representable by this implementation of integer numbers

	Max_value: INTEGER = 7
			-- Maximum value representable by this implementation of integer numbers

feature -- Output

	out: STRING
			-- <Precursor>
		do
			Result := value.out
		ensure then
			definition: Result ~ value.out
		end

feature -- Comparison

	real_three_way_comparison (other: STS_REAL): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_INTEGER}(other) -- todo: is it right?
		end

	rational_three_way_comparison (other: STS_RATIONAL): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_INTEGER}(other) -- todo: is it right?
		end

	three_way_comparison (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_INTEGER}(other) -- todo: is it right?
		end

feature -- Basic operations

	complex_plus (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex + other
		end

	complex_minus (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex - other
		end

	complex_product (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex * other
		end

	real_plus (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real + other
		end

	real_minus (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real - other
		end

	real_product (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real * other
		end

	real_quotient (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real / other
		end

	real_power alias "|^" (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real ^ other
		end

	abs: like integer_anchor
			-- <Precursor>
		do
			Result := value.abs
		end

	plus alias "+" (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := value + other.value
		end

	minus alias "-" (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := value - other.value
		end

	opposite alias "-": like integer_anchor
			-- <Precursor>
		do
			Result := - value
		end

	product alias "*" (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := value * other.value
		end

	integer_quotient alias "//" (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
				check
					good_divisor: --value.divisible ( todo:
						other.value /= 0 --) -- divisible (other)
				end
			Result := value // other.value
		end

	integer_remainder alias "\\" (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
				check
					good_divisor: --value.divisible ( todo:
						other.value /= 0 --) -- divisible (other)
				end
			Result := value \\ other.value
		end

	power alias "^" (other: STS_INTEGER): like rational_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_INTEGER}(other) -- todo: is it right?
		end

feature -- Arithmetic

	gcd (n, d: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := {STI_RATIONAL}.gcd (n, d)
		ensure then
			class
		end

feature -- Conversion

	as_complex: like complex_anchor
			-- This integer number represented as a complex number
		do
			create Result.make_from_integer (Current)
		ensure
			real: Result.a ≍ Current
			Imaginary: Result.b ≍ Zero
		end

	as_real: like real_anchor
			-- This integer number represented as a real number
		do
			create Result.make_from_integer (Current)
		ensure
			value: Result.value = real_value
		end

	as_rational: like rational_anchor
			-- This integer number represented as a rational number
		do
			create Result.make_from_integer (Current)
		ensure
			numerator: Result.p ≍ Current
			denominator: Result.q ≍ One
		end

feature -- Factory

	real_normalized_value (v: REAL): REAL
			-- <Precursor>
		do
			Result := {STI_REAL}.normalized_value (v)
		ensure then
			definition: Result = {STI_REAL}.normalized_value (v)
		end

	reduced (n, d: STS_INTEGER): like rational_anchor
			-- <Precursor>
		do
			Result := {STI_RATIONAL}.reduced (n, d)
		ensure then
			class
		end

	normalized_value (v: INTEGER): INTEGER
			-- The integer value represented by the first `value_bits' significant bits of `v'
		do
			Result := (v |<< ({PLATFORM}.Integer_32_bits - Value_bits)) |>> ({PLATFORM}.Integer_32_bits - Value_bits)
		ensure then
			class
			definition: Result = (v |<< ({PLATFORM}.Integer_32_bits - Value_bits)) |>> ({PLATFORM}.Integer_32_bits - Value_bits)
		end

feature -- Anchor

	complex_anchor: STI_COMPLEX
			-- <Precursor>
		do
			Result := Current
		end

	real_anchor: STI_REAL
			-- <Precursor>
		do
			Result := Current
		end

	rational_anchor: STI_RATIONAL
			-- <Precursor>
		do
			Result := Current
		end

	integer_anchor: STI_INTEGER
			-- <Precursor>
		do
			Result := Current
		end

	natural_anchor: STI_NATURAL
			-- <Precursor>
		do
			Result := Zero
		end

feature {NONE} -- Implementation

	Value_bits: INTEGER = 4
			-- Number of significant bits in `value'

invariant
	Min_value: Min_value = normalized_value ({INTEGER}.Min_value)
	Max_value: Max_value = normalized_value ({INTEGER}.Max_value)

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
