﻿note
	description: "Implementation of {STS_NATURAL}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

--expanded
class -- todo: why not expanded?
	STI_NATURAL

inherit
	STS_NATURAL
		redefine
				-- Output
			out,
				-- Comparison
			real_three_way_comparison,
			rational_three_way_comparison,
			integer_three_way_comparison,
			three_way_comparison,
				-- Basic operations
			opposite,
			power,
			factorial,
				-- Arithmetic
			gcd,
				-- Anchor
			complex_anchor,
			real_anchor,
			rational_anchor,
			integer_anchor,
			natural_anchor
		end

	DEBUG_OUTPUT
		rename
				-- Output
			debug_output as out
		undefine
				-- Comparison
			is_equal
		redefine
				-- Output
			out
		end

create
	default_create, make, make_from_reference, make_from_natural

convert
	make ({NATURAL}),
	make_from_reference ({NATURAL_32_REF}),
	make_from_natural ({STS_NATURAL}),
	as_complex: {STI_COMPLEX},
	as_real: {STI_REAL},
	real_value: {REAL, REAL_REF},
	as_rational: {STI_RATIONAL},
	as_integer: {STI_INTEGER},
	integer_value: {INTEGER, INTEGER_REF},
	value: {NATURAL, NATURAL_32_REF, HASHABLE}

feature {NONE} -- Initialization

	make (v: NATURAL)
			-- Create a natural number with value `v'.
		do
			value := normalized_value (v)
		ensure
			value: value = normalized_value (v)
		end

	make_from_reference (v_ref: NATURAL_32_REF)
			-- Create a natural number with value `v_ref'.`item'.
		do
			make (v_ref.item)
		ensure
			value: value = normalized_value (v_ref.item)
		end

	make_from_natural (n: STS_NATURAL)
			-- Create a natural number with value `n'.`value'.
		do
			make (n.value)
		ensure
			value: value = normalized_value (n.value)
		end

feature -- Primitive

	a,
	real,
	conjugate: like real_anchor
			-- <Precursor>
			-- This very natural number
		do
			Result := Current
		end

	real_value: REAL
			-- <Precursor>
		do
			Result := {STI_REAL}.normalized_value (value)
		end

	p,
	numerator: like Current
			-- <Precursor>
			-- This very natural number
		do
			Result := Current
		end

	integer_value: INTEGER_32
			-- <Precursor>
		do
			Result := value.as_integer_32
		end

	value: NATURAL
			-- <Precursor>

feature -- Access

	B,
	Imaginary,
	Zero: STI_NATURAL
			-- <Precursor>
		once
			create Result
		ensure then
			class
		end

	Q,
	Denominator,
	One: STI_NATURAL
			-- <Precursor>
		once
			Result := {NATURAL} 1
		ensure then
			class
		end

	Max_value: NATURAL = 7
			-- Maximum value representable by this implementation of natural numbers

feature -- Output

	out: STRING
			-- <Precursor>
		do
			Result := value.out
		ensure then
			definition: Result ~ value.out
		end

feature -- Comparison

	real_three_way_comparison (other: STS_REAL): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_NATURAL}(other) -- todo: is it right?
		end

	rational_three_way_comparison (other: STS_RATIONAL): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_NATURAL}(other) -- todo: is it right?
		end

	integer_three_way_comparison (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_NATURAL}(other) -- todo: is it right?
		end

	three_way_comparison (other: STS_NATURAL): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_NATURAL}(other) -- todo: is it right?
		end

feature -- Basic operations

	complex_plus (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex + other
		end

	complex_minus (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex - other
		end

	complex_product (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex * other
		end

	real_plus (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real + other
		end

	real_minus (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real - other
		end

	real_product (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real * other
		end

	real_quotient (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real / other
		end

	real_power alias "|^" (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real ^ other
		end

	integer_plus (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := as_integer + other
		end

	integer_minus (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := as_integer - other
		end

	integer_product (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := as_integer * other
		end

	integer_quotient (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := as_integer // other
		end

	integer_remainder (other: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := as_integer \\ other
		end

	plus alias "+" (other: STS_NATURAL): like natural_anchor
			-- <Precursor>
		do
			Result := value + other.value
		end

	minus alias "-" (other: STS_NATURAL): like natural_anchor
			-- <Precursor>
		do
			Result := value - other.value
		end

	opposite alias "-": like integer_anchor
			-- <Precursor>
		do
			Result := - value.as_integer_32
		end

	product alias "*" (other: STS_NATURAL): like natural_anchor
			-- <Precursor>
		do
			Result := value * other.value
		end

	natural_quotient alias "//" (other: STS_NATURAL): like natural_anchor
			-- <Precursor>
		do
				check
					good_divisor: --value.divisible ( todo:
						other.value /= 0 --) -- divisible (other)
				end
			Result := value // other.value
		end

	natural_remainder alias "\\" (other: STS_NATURAL): like natural_anchor
			-- <Precursor>
		do
				check
					good_divisor: --value.divisible ( todo:
						other.value /= 0 --) -- divisible (other)
				end
			Result := value \\ other.value
		end

	power alias "^" (other: STS_INTEGER): like rational_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_NATURAL}(other) -- todo: is it right?
		end

	factorial alias "﹗": like natural_anchor
			-- <Precursor>
		local
			n: like natural_anchor
		do
			from
				Result := One
				n := Current
			invariant
				building_up: Result ≍ (﹗Current / ﹗n)
			until
				n ≍ Zero
			loop
				Result := Result * n
				n := n - One
			variant
				n: n.integer_value
			end
		end

feature -- Arithmetic

	gcd (n, d: STS_INTEGER): like integer_anchor
			-- <Precursor>
		do
			Result := {STI_RATIONAL}.gcd (n, d)
		ensure then
			class
		end

feature -- Conversion

	as_complex: like complex_anchor
			-- This natural number represented as a complex number
		do
			create Result.make_from_natural (Current)
		ensure
			real: Result.a ≍ Current
			Imaginary: Result.b ≍ Zero
		end

	as_real: like real_anchor
			-- This natural number represented as a real number
		do
			create Result.make_from_natural (Current)
		ensure
			value: Result.value = real_value
		end

	as_rational: like rational_anchor
			-- This natural number represented as a rational number
		do
			create Result.make_from_natural (Current)
		ensure
			numerator: Result.p ≍ Current
			denominator: Result.q ≍ One
		end

	as_integer: like integer_anchor
			-- This natural number represented as an integer number
		do
			create Result.make_from_natural (Current)
		ensure
			value: Result.value = integer_value
		end

feature -- Factory

	real_normalized_value (v: REAL): REAL
			-- <Precursor>
		do
			Result := {STI_REAL}.normalized_value (v)
		ensure then
			definition: Result = {STI_REAL}.normalized_value (v)
		end

	reduced (n, d: STS_INTEGER): like rational_anchor
			-- <Precursor>
		do
			Result := {STI_RATIONAL}.reduced (n, d)
		ensure then
			class
		end

	integer_normalized_value (v: INTEGER): INTEGER
			-- <Precursor>
		do
			Result := {STI_INTEGER}.normalized_value (v)
		ensure then
			class
			definition: Result =  {STI_INTEGER}.normalized_value (v)
		end

	normalized_value (v: NATURAL): NATURAL
			-- The natural value represented by the first `value_bits' significant bits of `v'
		do
			Result := v & Value_bitmask
		ensure then
			class
			definition: Result = v & Value_bitmask
		end

feature -- Anchor

	complex_anchor: STI_COMPLEX
			-- <Precursor>
		do
			Result := Current
		end

	real_anchor: STI_REAL
			-- <Precursor>
		do
			Result := Current
		end

	rational_anchor: STI_RATIONAL
			-- <Precursor>
		do
			Result := Current
		end

	integer_anchor: STI_INTEGER
			-- <Precursor>
		do
			Result := Current
		end

	natural_anchor: STI_NATURAL
			-- <Precursor>
		do
			Result := Current
		end

feature {NONE} -- Implementation

	Value_bitmask: NATURAL = 0b0000_0111
			-- Mask of significant bits in `value'

invariant
	Max_value: Max_value.as_integer_32 = {STI_INTEGER}.Max_value
	Value_bitmask: Value_bitmask = Max_value

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
