﻿note
	description: "Implementation of {STS_COMPLEX_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	STI_COMPLEX_UNIVERSE

inherit

	STS_COMPLEX_UNIVERSE [STI_COMPLEX, STI_COMPLEX_EQUALITY]
		undefine
				-- Construction
			with,
			without,
				-- Measurement
			cardinality,
				-- Output
			out,
				-- Comparison
			equals,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			intersects,
				-- Quantifier
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
				-- Operation
			trivial_subsets,
			complemented,
			intersected,
			subtracted_symmetricaly,
			diagonal,
			crossed_tuples,
				-- Predicate
			equality_holds,
			equality_does_not_hold,
				-- Anchor
			product_anchor,
			square_anchor,
			diagonal_anchor,
			tuple_anchor,
			set_anchor,
			subset_anchor
		redefine
				-- Initialization
			default_create
		end

	STI_COMPLEX_SET
		rename
				-- Anchor
			superset_anchor as itself
		undefine
				-- Membership
			has,
				-- Construction
			with,
			extended,
				-- Comparison
			equals,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			intersects,
				-- Operation
			complemented,
			united,
			intersected,
				-- Anchor
			itself
		redefine
				-- Initialization
			default_create
		end

	STI_UNIVERSE [STI_COMPLEX, STI_COMPLEX_EQUALITY]
		undefine
				-- Anchor
			set_anchor,
			subset_anchor
		redefine
				-- Initialization
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Create the universe of COMPLEX numbers, i.e. ℂ.
		local
			r: STI_REAL_UNIVERSE
			real_plane: like {STI_REAL_UNIVERSE}.squared
		do
			create r
			real_plane := ²r
			create others.make_empty
			others := others.converted (
				{
					STI_TRANSFORMER [
						STS_ORDERED_PAIR [STI_REAL, STI_REAL, STI_REAL_EQUALITY, STI_REAL_EQUALITY],
						STI_COMPLEX,
						STS_PAIR_EQUALITY 
							[STI_REAL, STI_REAL, STI_REAL_EQUALITY, STI_REAL_EQUALITY],
						STI_COMPLEX_EQUALITY
						]
					}.
				set_map (
					real_plane.others, agent (
						ab: STS_ORDERED_PAIR 
							[STI_REAL, STI_REAL, STI_REAL_EQUALITY, STI_REAL_EQUALITY]
						): STI_COMPLEX
						do
							create Result.make (ab.a, ab.b)
						end
					)
				)
				check
					is_not_empty: not real_plane.is_empty -- not r.is_empty
				end
			create stored_any.put (create {STI_COMPLEX}.make (real_plane.any.a, real_plane.any.b)) -- todo: convert ordered pairs into complexes and rationals
		end

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
