﻿note
	description: "Implementation of {STS_NATURAL_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	STI_NATURAL_UNIVERSE

inherit

	STS_NATURAL_UNIVERSE [STI_NATURAL, STI_NATURAL_EQUALITY]
		undefine
				-- Construction
			with,
			without,
				-- Measurement
			cardinality,
				-- Output
			out,
				-- Comparison
			equals,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			intersects,
				-- Quantifier
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
				-- Operation
			trivial_subsets,
			complemented,
			intersected,
			subtracted_symmetricaly,
			diagonal,
			crossed_tuples,
				-- Predicate
			equality_holds,
			equality_does_not_hold,
				-- Anchor
			product_anchor,
			square_anchor,
			diagonal_anchor,
			tuple_anchor,
			set_anchor,
			subset_anchor
		redefine
				-- Initialization
			default_create
		end

	STI_NATURAL_SET
		rename
				-- Anchor
			superset_anchor as itself
		undefine
				-- Membership
			has,
				-- Construction
			with,
			extended,
				-- Comparison
			equals,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			intersects,
				-- Operation
			complemented,
			united,
			intersected,
				-- Anchor
			itself
		redefine
				-- Initialization
			default_create
		end

	STI_UNIVERSE [STI_NATURAL, STI_NATURAL_EQUALITY]
		undefine
				-- Anchor
			set_anchor,
			subset_anchor
		redefine
				-- Initialization
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Create the universe of every natural number representable by this implementation
		local
			n: like any_anchor
		do
			from
				create others.make_empty
				n := {STI_NATURAL}.Max_value
			invariant
				building_up: others ≍ left_open_right_closed_interval (n, {STI_NATURAL}.Max_value)
			until
				n.value = 0
			loop
					check
						does_not_have: others ∌ n -- others = (n, ->)
					end
				others := others.extended (n)
				n := n - {STI_NATURAL}.One
			variant
				n: n.integer_value
			end
			create stored_any.put (zero)
		ensure then
			has_everything: Current ≍ Current [Zero, {STI_NATURAL}.Max_value]
		end

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
