﻿note
	description: "Implementation of {STS_RATIONAL_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	STI_RATIONAL_UNIVERSE

inherit

	STS_RATIONAL_UNIVERSE [STI_RATIONAL, STI_RATIONAL_EQUALITY]
		undefine
				-- Construction
			with,
			without,
				-- Measurement
			cardinality,
				-- Output
			out,
				-- Comparison
			equals,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			intersects,
				-- Quantifier
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
				-- Operation
			trivial_subsets,
			complemented,
			intersected,
			subtracted_symmetricaly,
			diagonal,
			crossed_tuples,
				-- Predicate
			equality_holds,
			equality_does_not_hold,
				-- Anchor
			product_anchor,
			square_anchor,
			diagonal_anchor,
			tuple_anchor,
			set_anchor,
			subset_anchor
		redefine
				-- Initialization
			default_create
		end

	STI_RATIONAL_SET
		rename
				-- Anchor
			superset_anchor as itself
		undefine
				-- Membership
			has,
				-- Construction
			with,
			extended,
				-- Comparison
			equals,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			intersects,
				-- Operation
			complemented,
			united,
			intersected,
				-- Anchor
			itself
		redefine
				-- Initialization
			default_create
		end

	STI_UNIVERSE [STI_RATIONAL, STI_RATIONAL_EQUALITY]
		undefine
				-- Anchor
			set_anchor,
			subset_anchor
		redefine
				-- Initialization
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Create the universe of every rational number representable in this implementation
		local
			n, d: STI_INTEGER
			ab: like any_anchor
		do
			from
				create others.make_empty
				n := {STI_INTEGER}.Min_value
				d := {STI_INTEGER}.Min_value
			invariant
--				building_up: Result â
			until
				d.value = 0
			loop
				create ab.make (n, d)
				if ab.a ≍ n and ab.b ≍ d then
						check
							does_not_have: others ∌ ab -- todo: why?
						end
					others := others.extended (ab)
						check
							good_divisor: d.abs.value /= 0 -- d.value /= 0
						end
					create ab.make (n, d.abs)
						check
							does_not_have: others ∌ ab -- todo: why?
						end
					others := others.extended (ab)
				end
				d := d + {STI_INTEGER}.one
--			variant
--				cartinality: (# s).as_integer_64
			end
			from
				n := n + {STI_INTEGER}.one
			invariant
--				building_up: Result â
			until
				n.value = 0
			loop
				from
					d :={STI_INTEGER}.Min_value
						check
							good_divisor: d.value /= 0 -- {STI_INTEGER}.Min_value /= 0
						end
					create ab.make (n, d)
					if ab.a ≍ n and ab.b ≍ d then
							check
								does_not_have: others ∌ ab -- todo: why?
							end
						others := others.extended (ab)
						create ab.make (n.abs, d)
							check
								does_not_have: others ∌ ab -- todo: why?
							end
						others := others.extended (ab)
					end
					d := {STI_INTEGER}.Max_value
				invariant
--					building_up: Result â
				until
					d.value = 0
				loop
					create ab.make (n, d)
					if ab.a ≍ n and ab.b ≍ d then
							check
								does_not_have: others ∌ ab -- todo: why?
							end
						others := others.extended (ab)
						create ab.make (n.abs, d)
							check
								does_not_have: others ∌ ab -- todo: why?
							end
						others := others.extended (ab)
					end
					d := d - {STI_INTEGER}.one
--				variant
--					cartinality: (# s).as_integer_64
				end
				n := n + {STI_INTEGER}.one
--			variant
--				cartinality: (# s).as_integer_64
			end
			create stored_any.put (zero)
				check
					z: attached (create {STI_INTEGER_UNIVERSE}) as z
					every_rational: {
						STI_TRANSFORMER [
							STS_ORDERED_PAIR [
								STS_INTEGER, STS_INTEGER,
								STS_INTEGER_EQUALITY [STS_INTEGER],
								STS_INTEGER_EQUALITY [STS_INTEGER]
								],
							STS_RATIONAL,
							STS_PAIR_EQUALITY [
								STS_INTEGER, STS_INTEGER,
								STS_INTEGER_EQUALITY [STS_INTEGER],
								STS_INTEGER_EQUALITY [STS_INTEGER]
								],
							STS_RATIONAL_EQUALITY [STS_RATIONAL]
							]
						}.set_map (
							z × z.without_zero,
							agent (
								xy: STS_ORDERED_PAIR [
									STS_INTEGER, STS_INTEGER,
									STS_INTEGER_EQUALITY [STS_INTEGER],
									STS_INTEGER_EQUALITY [STS_INTEGER]
									]
								): STS_RATIONAL
								do
										check
											good_divisor: xy.b ≭ {STI_INTEGER}.Zero -- todo: why?
										end
									Result := {STI_RATIONAL}.reduced (xy.a, xy.b)
								end
							) ≍ Current
				then
				end
		end

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
