﻿note
	description: "Implementation of {STS_REAL_UNIVERSE}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	STI_REAL_UNIVERSE

inherit

	STS_REAL_UNIVERSE [STI_REAL, STI_REAL_EQUALITY]
		undefine
				-- Construction
			with,
			without,
				-- Measurement
			cardinality,
				-- Output
			out,
				-- Comparison
			equals,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			intersects,
				-- Quantifier
			exists,
			exists_unique,
			exists_pair,
			exists_distinct_pair,
			for_all,
			for_all_pairs,
			for_all_distinct_pairs,
				-- Operation
			trivial_subsets,
			complemented,
			intersected,
			subtracted_symmetricaly,
			diagonal,
			crossed_tuples,
				-- Predicate
			equality_holds,
			equality_does_not_hold,
				-- Anchor
			product_anchor,
			square_anchor,
			diagonal_anchor,
			tuple_anchor,
			set_anchor,
			subset_anchor
		redefine
				-- Initialization
			default_create
		end

	STI_REAL_SET
		rename
				-- Anchor
			superset_anchor as itself
		undefine
				-- Membership
			has,
				-- Construction
			with,
			extended,
				-- Comparison
			equals,
			is_superset,
			is_comparable,
			is_not_comparable,
			is_strict_subset,
			is_trivial_subset,
			is_proper_subset,
			is_disjoint,
			intersects,
				-- Operation
			complemented,
			united,
			intersected,
				-- Anchor
			itself
		redefine
				-- Initialization
			default_create
		end

	STI_UNIVERSE [STI_REAL, STI_REAL_EQUALITY]
		undefine
				-- Anchor
			set_anchor,
			subset_anchor
		redefine
				-- Initialization
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Create the universe of real numbers, i.e. ℝ.
		local
			e, m: INTEGER
			v: REAL
			x: STI_REAL
		do
			from
				create others.make_singleton ({STI_REAL}.Nan)
					check
						does_not_have_Negative_infinity: others ∌ {STI_REAL}.Negative_infinity -- todo: why?
					end
				others := others.extended ({STI_REAL}.Negative_infinity)
					check
						does_not_have_Positive_infinity: others ∌ {STI_REAL}.Positive_infinity -- todo: why?
					end
				others := others.extended ({STI_REAL}.Positive_infinity)
				e := {STI_REAL}.Greatest_exponent
			invariant
--				building_up: Result ≍
			until
				e < {STI_REAL}.Least_normal_exponent
			loop
				from
					m := {STI_REAL}.Mantissa_scale.as_integer_32 + {STI_REAL}.Greatest_mantissa.as_integer_32
				invariant
--					building_up: Result ≍
				until
					m < {STI_REAL}.Mantissa_scale.as_integer_32
				loop
					v := (2 ^ e * m / {STI_REAL}.Mantissa_scale).truncated_to_real
						check
							normalized_value: {STI_REAL}.same_value (v, {STI_REAL}.normalized_value (v)) -- `normalized_value' definition
						end
					create x.make_from_normalized_value (v)
						check
							does_not_have_minus_x: others ∌ - x -- todo: why?
						end
					others := others.extended (- x)
						check
							does_not_have_x: others ∌ x -- todo: why?
						end
					others := others.extended (x)
					m := m - 1
				variant
					m_minus_Greatest_mantissa: m - {STI_REAL}.Greatest_mantissa.as_integer_32
				end
				e := e - 1
			variant
				e_plus_Exponent_bias: e + {STI_REAL}.Exponent_bias
			end
			from
--				m := 3
			invariant
--				building_up: Result ≍
			until
				m = 0
			loop
				v := (m / 2 ^ {STI_REAL}.Mantissa_scale).truncated_to_real
					check
						subnormalized_value: {STI_REAL}.same_value (v, {STI_REAL}.normalized_value (v)) -- `normalized_value' definition
					end
				create x.make_from_normalized_value (v)
					check
						does_not_have_minus_subnormal_x: others ∌ - x -- todo: why?
					end
				others := others.extended (- x)
					check
						does_not_have_subnormal_x: others ∌ x -- todo: why?
					end
				others := others.extended (x)
				m := m - 1
			variant
				m: m
			end
			create stored_any.put (Zero)
		end

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
