﻿note
	description: "Implementation of {STS_RATIONAL}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

--expanded
class -- todo: why not expanded?
	STI_RATIONAL

inherit

	STS_RATIONAL
		redefine
				-- Initialization
			default_create,
				-- Access
			sign,
				-- Output
			out,
				-- Comparison
			real_three_way_comparison,
			three_way_comparison,
				-- Basic operations
			power,
				-- Arithmetic
			gcd,
				-- Anchor
			complex_anchor,
			real_anchor,
			rational_anchor,
			integer_anchor,
			natural_anchor
		end

	DEBUG_OUTPUT
		rename
				-- Output
			debug_output as out
		undefine
				-- Comparison
			is_equal
		redefine
				-- Initialization
			default_create,
				-- Output
			out
		end

create
	default_create,
	make,
	make_from_native_integers,
	make_from_native_integer_references,
	make_from_native_naturals,
	make_from_native_natural_references,
	make_from_rational,
	make_from_integer,
	make_from_natural

convert
	make_from_rational ({STS_RATIONAL}),
	make_from_integer ({STS_INTEGER}),
	make_from_natural ({STS_NATURAL}),
	as_real: {STI_REAL},
	as_complex: {STI_COMPLEX},
	value: {REAL, REAL_REF, HASHABLE}

feature {NONE} -- Initialization

	default_create
			-- Create the rational number 0/1.
		do
			p := Zero
			q := One
		ensure then
			numerator: p ≍ Zero
			denominator: q ≍ One
		end

	make (n, d: STS_INTEGER)
			-- Create the rational number `n' / `d'.
		require
			good_divisor: n.divisible (d)
		local
			l_n, l_d, l_gcd: like integer_anchor
		do
			l_gcd := gcd (n, d)
			l_n := converted_integer (n // l_gcd)
			l_d := converted_integer (d // l_gcd)
			if
				l_d.value > 0 or
				l_n.value = {STI_INTEGER}.Min_value or
				l_d.value = {STI_INTEGER}.Min_value
			then
				p := l_n
				q := l_d
			else
				p := - l_n
				q := - l_d
			end
		ensure
			el_gcd: attached gcd (n, d) as el_gcd
			el_n: attached (n // el_gcd) as el_n
			el_d: attached (d // el_gcd) as el_d
			same_numerator: el_d.value > 0 or
				el_n.value = {STI_INTEGER}.Min_value or
				el_d.value = {STI_INTEGER}.Min_value implies
				p ≍ el_n
			same_denominator: el_d.value > 0 or
				el_n.value = {STI_INTEGER}.Min_value or
				el_d.value = {STI_INTEGER}.Min_value implies
				q ≍ el_d
			opposite_numerator: el_d.value < 0 and
				el_n.value /= {STI_INTEGER}.Min_value and
				el_d.value /= {STI_INTEGER}.Min_value implies
				p ≍ - el_n
			opposite_denominator: el_d.value < 0 and
				el_n.value /= {STI_INTEGER}.Min_value and
				el_d.value /= {STI_INTEGER}.Min_value implies
				q ≍ - el_d
		end

	make_from_native_integers (n, d: INTEGER)
			-- Create the rational number `n' / `d'.
		require
			good_divisor: d /= 0 --n.divisible (d)
		local
			l_n, l_d: STI_INTEGER
		do
			l_n := n
			l_d := d
				check
					good_divisor: l_n.divisible (l_d) -- d /= 0
				end
			make (l_n, l_d)
		ensure
			el_n: attached (create {STI_INTEGER}.make (n)) as el_n
			el_d: attached (create {STI_INTEGER}.make (d)) as el_d
			good_divisor: el_n.divisible (el_d) -- d /= 0
			numerator: p ≍ reduced (el_n, el_d).p
			denominator: q ≍ reduced (el_n, el_d).q
		end

	make_from_native_integer_references (n, d: INTEGER_REF)
			-- Create the rational number `n' / `d'.
		require
			good_divisor: d.item /= 0 --n.divisible (d)
		local
			l_n, l_d: STI_INTEGER
		do
			l_n := n
			l_d := d
				check
					good_divisor: l_n.divisible (l_d) -- d.item /= 0
				end
			make (l_n, l_d)
		ensure
			el_n: attached (create {STI_INTEGER}.make_from_reference (n)) as el_n
			el_d: attached (create {STI_INTEGER}.make_from_reference (d)) as el_d
			good_divisor: el_n.divisible (el_d) -- d.item /= 0
			numerator: p ≍ reduced (el_n, el_d).p
			denominator: q ≍ reduced (el_n, el_d).q
		end

	make_from_native_naturals (n, d: NATURAL)
			-- Create the rational number `n' / `d'.
		require
			good_divisor: d /= 0 --n.divisible (d)
		local
			l_n, l_d: STI_NATURAL
		do
			l_n := n
			l_d := d
				check
					good_divisor: l_n.divisible (l_d) -- d /= 0
				end
			make (l_n, l_d)
		ensure
			el_n: attached (create {STI_NATURAL}.make (n)) as el_n
			el_d: attached (create {STI_NATURAL}.make (d)) as el_d
			good_divisor: el_n.divisible (el_d) -- d /= 0
			numerator: p ≍ reduced (el_n, el_d).p
			denominator: q ≍ reduced (el_n, el_d).q
		end

	make_from_native_natural_references (n, d: NATURAL_32_REF)
			-- Create the rational number `n' / `d'.
		require
			good_divisor: d.item /= 0 --n.divisible (d)
		local
			l_n, l_d: STI_NATURAL
		do
			l_n := n
			l_d := d
				check
					good_divisor: l_n.divisible (l_d) -- d.item /= 0
				end
			make (l_n, l_d)
		ensure
			el_n: attached (create {STI_NATURAL}.make_from_reference (n)) as el_n
			el_d: attached (create {STI_NATURAL}.make_from_reference (d)) as el_d
			good_divisor: el_n.divisible (el_d) -- d.item /= 0
			numerator: p ≍ reduced (el_n, el_d).p
			denominator: q ≍ reduced (el_n, el_d).q
		end

	make_from_rational (r: STS_RATIONAL)
			-- Create the rational number `r'.`p' / `r'.`q'.
		do
				check
					good_divisor: r.p.divisible (r.q) -- by definition
				end
			make (r.p, r.q)
		ensure
			numerator: p ≍ r.p
			denominator: q ≍ r.q
		end

	make_from_integer (i: STS_INTEGER)
			-- Create the rational number `i' / 1.
		do
				check
					good_divisor: i.divisible (One) -- 1 /= 0
				end
			make (i, one)
		ensure
			numerator: p ≍ i
			denominator: q ≍ one
		end

	make_from_natural (n: STS_NATURAL)
			-- Create the rational number `n' / 1.
		do
				check
					good_divisor: n.divisible (One) -- 1 /= 0
				end
			make (n, one)
		ensure
			numerator: p ≍ n
			denominator: q ≍ one
		end

feature -- Primitive

	a,
	real,
	conjugate: like real_anchor
			-- <Precursor>
			-- This very rational number
		do
			Result := Current
		end

	value: REAL
			-- <Precursor>
		do
				check
					good_divisor: q.value /= 0 -- todo: p.value.divisible (q.value)
						-- by definition
				end
			Result := {STI_REAL}.normalized_value ((p.value / q.value).truncated_to_real)
		end

	p: like integer_anchor
			-- <Precursor>

	numerator: like integer_anchor
			-- <Precursor>
		do
			Result := p
		end

	q: like integer_anchor
		-- <Precursor>

	denominator: like integer_anchor
			-- <Precursor>
		do
			Result := q
		end

feature -- Access

	sign: like integer_anchor
			-- <Precursor>
		do
			Result := p.sign * q.sign
		end

	B,
	Imaginary,
	Zero: STI_NATURAL
			-- <Precursor>
		once
			Result := {STI_NATURAL}.Zero
		ensure then
			class
		end

	One: STI_NATURAL
			-- <Precursor>
		once
			Result := {STI_NATURAL}.One
		ensure then
			class
		end

feature -- Output

	out: STRING
			-- <Precursor>
		do
			Result := p.out + "/" + q.out
		ensure then
			definition: Result ~ p.out + "/" + q.out
		end

feature -- Comparison

	real_three_way_comparison (other: STS_REAL): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_RATIONAL}(other) -- todo: is it right?
		end

	three_way_comparison (other: STS_RATIONAL): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_RATIONAL}(other) -- todo: is it right?
		end

feature -- Basic operations

	complex_plus (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex + other
		end

	complex_minus (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex - other
		end

	complex_product (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex * other
		end

	real_plus (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real + other
		end

	real_minus (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real - other
		end

	real_product (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real * other
		end

	real_quotient (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real / other
		end

	real_power alias "|^" (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := as_real ^ other
		end

	power alias "^" (other: STS_INTEGER): like rational_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_RATIONAL}(other) -- todo: is it right?
		end

feature -- Arithmetic

	gcd (n, d: STS_INTEGER): like integer_anchor
			-- <Precursor>
		local
			l_n, l_d: like integer_anchor
		do
			from
				l_n := converted_integer (n)
				l_d := converted_integer (d)
			invariant
				base: l_d ≍ Zero implies gcd (l_n, l_d) ≍ l_n.abs
				good_divisor: l_d ≭ Zero implies l_n.divisible (l_d)
				induction: l_d ≭ Zero implies gcd (l_n, l_d) ≍ gcd (l_d, l_n \\ l_d)
			until
				l_d ≍ Zero
			loop
				l_n := l_d
				l_d := l_n \\ l_d
			variant
				distance: (l_n - l_d).abs.value
			end
			Result := l_n
		ensure then
			class
		end

feature -- Conversion

	as_complex: like complex_anchor
			-- This rational number represented as a complex number
		do
			create Result.make_from_rational (Current)
		ensure
			real: Result.a ≍ Current
			Imaginary: Result.b ≍ Zero
		end

	as_real: like real_anchor
			-- This rational number represented as a real number
		do
			create Result.make_from_rational (Current)
		ensure
			value: Result.value = value
		end

feature -- Factory

	normalized_value (v: REAL): REAL
			-- <Precursor>
		do
			Result := {STI_REAL}.normalized_value (v)
		ensure then
			definition: Result = {STI_REAL}.normalized_value (v)
		end

	reduced (n, d: STS_INTEGER): like rational_anchor
			-- <Precursor>
		do
			create Result.make (n, d)
		ensure then
			class
		end

	converted_integer (i: STS_INTEGER): like integer_anchor
			-- `i' converted to an integer like `integer_anchor'
		do
			if attached {like integer_anchor} i as ci then
				Result := ci
			else
				Result := i
			end
		ensure
			class
			definition: Result ≍ i
		end

feature -- Anchor

	complex_anchor: STI_COMPLEX
			-- <Precursor>
		do
			Result := Current
		end

	real_anchor: STI_REAL
			-- <Precursor>
		do
			Result := Current
		end

	rational_anchor: STI_RATIONAL
			-- <Precursor>
		do
			Result := Current
		end

	integer_anchor: STI_INTEGER
			-- <Precursor>
		do
			Result := p
		end

	natural_anchor: STI_NATURAL
			-- <Precursor>
		do
			Result := Zero
		end

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
