﻿note
	description: "Implementation of {STS_REAL}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

expanded
class -- todo: why not expanded?
	STI_REAL

inherit

	STS_REAL
		redefine
				-- Access
			sign,
				-- Output
			out,
				-- Quality
			is_invertible,
				-- Comparison
			three_way_comparison,
				-- Basic operations
			opposite,
			reciprocal, inverse,
				-- Anchor
			complex_anchor,
			real_anchor,
			integer_anchor,
			natural_anchor
		end

	DEBUG_OUTPUT
		rename
				-- Output
			debug_output as out
		undefine
				-- Comparison
			is_equal
		redefine
				-- Output
			out
		end

create
	default_create,
	make,
	make_from_normalized_value,
	make_from_native_real_reference,
	make_from_native_real_64,
	make_from_native_real_64_reference,
	make_from_real,
	make_from_native_integer,
	make_from_native_integer_reference,
	make_from_integer,
	make_from_native_natural,
	make_from_native_natural_reference,
	make_from_natural,
	make_from_rational,
	make_abs,
	make_opposite

convert
	make ({REAL}),
	make_from_native_real_reference ({REAL_REF}),
	make_from_native_real_64 ({REAL_64}),
	make_from_native_real_64_reference ({REAL_64_REF}),
	make_from_real ({STS_REAL}),
	make_from_rational ({STS_RATIONAL}),
	make_from_native_integer ({INTEGER}),
	make_from_native_integer_reference({INTEGER_REF}),
	make_from_integer ({STS_INTEGER}),
	make_from_native_natural ({NATURAL}),
	make_from_native_natural_reference ({NATURAL_32_REF}),
	make_from_natural ({STS_NATURAL}),
	value: {REAL, REAL_REF, HASHABLE},
	as_complex: {STI_COMPLEX}

feature {NONE} -- Initialization

	make (v: REAL)
			-- Create a real number with `value' = `normalized_value' (`v').
		do
			value := normalized_value (v)
		ensure
			value: same_value (value, normalized_value (v))
		end

	make_from_normalized_value (v: REAL)
			-- Create a real number with `value' = `v'.
		require
			normalized_value: same_value (v, normalized_value (v))
		do
			value := v
		ensure
			value: same_value (value, v)
		end

	make_from_native_real_reference (v_ref: REAL_REF)
			-- Create a real number with `value' = `normalized_value' (`v_ref'.`item').
		do
			make (v_ref.item)
		ensure
			value: same_value (value, normalized_value (v_ref.item))
		end

	make_from_native_real_64 (v: REAL_64)
			-- Create a real number with `value' = `normalized_value' (`v'.`truncated_to_real').
		do
			value := normalized_value (v.truncated_to_real)
		ensure
			value: same_value (value, normalized_value (v.truncated_to_real))
		end

	make_from_native_real_64_reference (v_ref: REAL_64_REF)
			-- Create a real number with `value' = `normalized_value' (`v_ref'.`truncated_to_real').
		do
			make_from_native_real_64 (v_ref.item)
		ensure
			value: same_value (value, normalized_value (v_ref.truncated_to_real))
		end

	make_from_real (r: STS_REAL)
			-- Create a real number with `value' = `r'.`value'
		do
				check
					normalized_value: same_value (r.value, normalized_value (r.value))
						-- {STS_REAL} invariant
				end
			make_from_normalized_value (r.value)
		ensure
			value: same_value(value, r.value)
		end

	make_from_native_integer (v: INTEGER)
			-- Create a real number with `value' = `normalized_value' (`v'.`to_real').
		do
			make (v)
		ensure
			value: same_value (value, normalized_value (v))
		end

	make_from_native_integer_reference (v_ref: INTEGER_REF)
			-- Create a real number with `value' =  `normalized_value' (`v_ref'.`to_real').
		do
			make_from_native_integer (v_ref.item)
		ensure
			value: value = normalized_value (v_ref.to_real)
		end

	make_from_integer (i: STS_INTEGER)
			-- Create a real number with `value' = `i'.`real_value'
		do
				check
					normalized_value: same_value (i.real_value, normalized_value (i.real_value))
						-- {STS_INTEGER} invariant
				end
			make_from_normalized_value (i.real_value)
		ensure
			value: same_value(value, i.real_value)
		end

	make_from_native_natural (v: NATURAL)
			-- Create a real number with `value' = `normalized_value' (`v'.`to_real_32').
		do
			make (v)
		ensure
			value: same_value (value, normalized_value (v))
		end

	make_from_native_natural_reference (v_ref: NATURAL_32_REF)
			-- Create a real number with `value' =  `normalized_value' (`v_ref'.`to_real_32').
		do
			make_from_native_natural (v_ref.item)
		ensure
			value: value = normalized_value (v_ref.to_real_32)
		end

	make_from_natural (n: STS_NATURAL)
			-- Create a real number with `value' = `n'.`real_value'
		do
				check
					normalized_value: same_value (n.real_value, normalized_value (n.real_value))
						-- {STS_NATURAL} invariant
				end
			make_from_normalized_value (n.real_value)
		ensure
			value: same_value(value, n.real_value)
		end

	make_from_rational (r: STS_RATIONAL)
			-- Create a real number with `value' = `r'.`value'
		do
				check
					normalized_value: same_value (r.value, normalized_value (r.value))
						-- {STS_RATIONAL} invariant
				end
			make_from_normalized_value (r.value)
		ensure
			value: same_value(value, r.value)
		end

	make_abs (r: STI_REAL)
			-- Create a real number with value `r'.`value'.`abs'.
		do
				check
					normalized_value: same_value (r.value.abs, normalized_value (r.value.abs))
						-- `normalized_value' definition
				end
			make_from_normalized_value (r.value.abs)
		ensure
			value: same_value (value, r.value.abs)
		end

	make_opposite (r: STI_REAL)
			-- Create a real number with value -`r'.`value'
		do
				check
					normalized_value: same_value (- r.value, normalized_value (- r.value))
						-- `normalized_value' definition
				end
			make_from_normalized_value (- r.value)
		ensure
			value: same_value (value, - r.value)
		end

feature -- Primitive

	a,
	real,
	conjugate: like Current
			-- <Precursor>
			-- This very real number
		do
			Result := Current
		end

	value: REAL
			-- <Precursor>

feature -- Access

	sign: like integer_anchor
			-- <Precursor>
		do
			Result := value.sign
		end

	B,
	Imaginary,
	Zero: STI_NATURAL
			-- <Precursor>
		once
			Result := {STI_NATURAL}.Zero
		ensure then
			class
		end

	One: STI_NATURAL
			-- <Precursor>
		once
			Result := {STI_NATURAL}.One
		ensure then
			class
		end

	Nan: STI_REAL
			-- Representation of not a number (NaN)
		once
			Result := {REAL}.nan
		ensure
			class
			definition: Result.is_nan
		end

	Negative_infinity: STI_REAL
			-- Representation of negative infinity
		once
			Result := {REAL}.negative_infinity
		ensure
			class
			definition: Result.is_negative_infinity
		end

	Positive_infinity: STI_REAL
			-- Representation of positive infinity
		once
			Result := {REAL}.positive_infinity
		ensure
			class
			definition: Result.is_positive_infinity
		end

	Min_value: REAL = - 14.0
			-- Minimum value representable by this kind of real number.

	Max_value: REAL = 14.0 -- 2 ^ `greatest_exponent' * (1 + `greatest_mantissa' * `mantissa_unit')
			-- Maximum value representable by this kind of real number.

	Machine_epsilon: REAL = 0.25 -- 2 ^ 0 * (1 + 1 * `mantissa_unit') - 1
			-- The difference between 1 and the least value greater than
			-- 1 that is representable by this kind of real number.

	Epsilon: REAL = 0.3125 -- 2 ^ `least_normal_exponent' * (1 + 1 * `mantissa_unit')
			-- Minimum normalized positive number representable by this kind of real number

feature -- Quality

	is_invertible: BOOLEAN
			-- <Precursor>
		do
			if value /= 0 then
				Result := (divisible (Current) and (- Zero.as_real).divisible (Current)) and then
					(
						(Current / Current).divisible (Current) and
						(- Zero.as_real / Current).divisible (Current)
						)
			end
		end

feature -- Status report

	is_nan: BOOLEAN
			-- Is current the representation of `nan'?
		do
			Result := value.is_nan
		ensure
			definition: Result = value.is_nan
		end

	is_negative_infinity: BOOLEAN
			-- Is current the representation of `negative_infinity'?
		do
			Result := value.is_negative_infinity
		ensure
			definition: Result = value.is_negative_infinity
		end

	is_positive_infinity: BOOLEAN
			-- Is current the representation of `positive_infinity'?
		do
			Result := value.is_positive_infinity
		ensure
			definition: Result = value.is_positive_infinity
		end

feature -- Output

	out: STRING
			-- <Precursor>
		do
			Result := value.out
		ensure then
			definition: Result ~ value.out
		end

feature -- Comparison

--	complex_equals (z: STS_COMPLEX): BOOLEAN
--			-- <Precursor>
--		do
--			Result := same_value (value, z.a.value) and 0 = z.b.value
--		end

	three_way_comparison (other: STS_REAL): like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_REAL}(other) -- todo: is it right?
		end

feature -- Basic operations

	complex_plus (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex + other
		end

	complex_minus (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex - other
		end

	complex_product (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
			Result := as_complex * other
		end

	abs: like real_anchor
			-- <Precursor>
		do
			create Result.make_abs (Current)
		end

	plus alias "+" (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := value + other.value
		end

	minus alias "-" (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := value - other.value
		end

	opposite alias "-": like real_anchor
			-- <Precursor>
		do
			create Result.make_opposite (Current)
		end

	product alias "*" (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := value * other.value
		end

	quotient alias "/" (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
				check
					good_divisor: value.divisible (other.value) -- divisible (other)
				end
			Result := value / other.value
		end

	reciprocal,
	inverse alias "⁻¹": like real_anchor
			-- <Precursor>
		do
				check
					good_divisor: ({REAL} 1.0).divisible (value) -- one.divisible (Current)
				end
			Result := 1 / value
		end

	power alias "^" (other: STS_REAL): like real_anchor
			-- <Precursor>
		do
			Result := value ^ other.value
		end

--feature -- Conversion

--	as_complex: like complex_anchor
--			-- This real number represented as a complex number
--		do
--			create Result.make (Current, zero)
--		end

feature -- Conversion

	as_complex: like complex_anchor
			-- This real number represented as a complex number
		do
			create Result.make_from_real (Current)
		ensure
			real: Result.a ≍ Current
			Imaginary: Result.b ≍ Zero
		end

feature -- Factory

	normalized_value (v: REAL): REAL
			-- The value of `v' adapted to be represented as the value of this real number
			--| The normalized values range as if they were encoded by a 6-bit, base 2,
			--| IEEE 754-like, floating point number with the following parameters:
			--| sign bit: 1 bit;
			--| exponent width: 3 bits;
			--| significand (mantissa) precision: 3 bits (2 explicitly stored).
			--| Conceptual normalized value layout: s_eee_mm
		local
			s, e: INTEGER
			f, exp, m: NATURAL
		do
			if v.is_nan then
				Result := v
			elseif v > Max_value then
				Result := {REAL}.positive_infinity
			elseif v < Min_value then
				Result := {REAL}.negative_infinity
			else
				s := v.sign
				f := real_bit_pattern (v)
				exp := f |>> Native_mantissa_width
				exp := exp & Native_exponent_mask
				m := f |>> (Native_mantissa_width - Mantissa_width)
				m := m & Mantissa_mask
				e := exp.as_integer_32 - Native_exponent_bias
				if Least_normal_exponent <= e then
					Result := s * (2 ^ e).truncated_to_real * (1 + m * Mantissa_unit)
				elseif Least_subnormal_exponent <= e then
					m := m | Mantissa_scale
					m := m |>> (Least_normal_exponent - e)
					Result := s * (2 ^ Least_normal_exponent).truncated_to_real * m * Mantissa_unit
				else
					Result := s * 0
				end
			end
		ensure then
			class
			Nan: v.is_nan implies Result.is_nan
			Positive_infinity: v > Max_value implies Result.is_positive_infinity
			Negative_infinity: v < Min_value implies Result.is_negative_infinity
			el_s: attached v.sign as el_s
			el_f: attached real_bit_pattern (v) as el_f
			el_exp: attached ((el_f |>> Native_mantissa_width) & Native_exponent_mask) as el_exp
			el_m: attached
				((el_f |>> (Native_mantissa_width - Mantissa_width)) & Mantissa_mask) as el_m
			el_e: attached (el_exp.as_integer_32 - Native_exponent_bias) as el_e
			normal_value: not v.is_nan and Min_value <= v and v <= Max_value and
				Least_normal_exponent <= el_e implies
				Result = el_s * (2 ^ el_e).truncated_to_real * (1 + el_m * Mantissa_unit)
			subnormal_value: not v.is_nan and Min_value <= v and v <= Max_value and
				Least_subnormal_exponent <= el_e and el_e < Least_normal_exponent implies
				attached
					((el_m | Mantissa_scale) |>> (Least_normal_exponent - el_e)) as el_subm and then
				Result = el_s *
					(2 ^ Least_normal_exponent).truncated_to_real *
					(el_subm * Mantissa_unit)
			Zero: not v.is_nan and Min_value <= v and v <= Max_value and
				el_e < Least_subnormal_exponent implies Result = el_s * 0
		end

feature -- Anchor

	complex_anchor: STI_COMPLEX
			-- <Precursor>
		do
			Result := Current
		end

	real_anchor: STI_REAL
			-- <Precursor>
		do
			Result := Current
		end

	integer_anchor: STI_INTEGER
			-- <Precursor>
		do
			Result := sign
		end

	natural_anchor: STI_NATURAL
			-- <Precursor>
		do
			Result := zero
		end

feature {STI_REAL_UNIVERSE} -- Implementation

	native_exponent_width: INTEGER = 8
			-- How many bits a native real number uses to represent its exponent

	native_exponent_mask: NATURAL = 0b1111_1111 -- 2 ^ `native_exponent_width' - 1
			-- Binary mask for the exponent of a native real number

	greatest_native_biased_exponent: INTEGER = 0b1111_1111 -- 2 ^ `native_exponent_width' - 1
			-- Maximum biased value for the exponent of a native real number

	native_exponent_bias: INTEGER = 0b0111_1111 -- (`greatest_native_biased_exponent' - 1) // 2
			-- Bias value for the exponent of a native real number

	native_mantissa_width: INTEGER = 23
			-- How many bits a native real number uses to represent its mantissa

	exponent_width: INTEGER = 3
			-- How many bits a real number uses to represent its exponent

	exponent_mask: NATURAL = 0b111 -- 2 ^ `exponent_width' - 1
			-- Binary mask for the exponent of a real number

	greatest_biased_exponent: INTEGER = 0b111 -- 2 ^ `exponent_width' - 1
			-- Maximum biased value for the exponent of a real number

	exponent_bias, -- (`greatest_biased_exponent' - 1) // 2
			-- Bias value for the exponent of a real number
	greatest_exponent: INTEGER = 0b011 -- `greatest_biased_exponent' - 1 - `exponent_bias'
			-- Maximum unbiased value for the exponent of a real number

	least_normal_exponent: INTEGER = - 2 -- 1 - `exponent_bias'
			-- Minimum unbiased value for the exponent of a normal real number

	least_subnormal_exponent: INTEGER = - 4 -- `least_normal_exponent' - `mantissa_width'
			-- Minimum unbiased value for the exponent of a subnormal real number

	mantissa_width: INTEGER = 2
			-- How many bits a real number uses to represent its mantissa

	mantissa_mask,
			-- Binary mask for the exponent of a real number
	greatest_mantissa: NATURAL = 0b11 -- 2 ^ `mantissa_width' - 1
			-- Maximum value for the mantissa of a real number

	mantissa_scale: NATURAL = 0b100 -- 2 ^ `mantissa_width'
			-- Reference agains wich the units of the mantissa of a real number are measured

	mantissa_unit: REAL = 0.25 -- 1 / `mantissa_scale'
			-- Numerical value of a unit in the mantissa of a real number

	real_bit_pattern (r: REAL): NATURAL
			-- The bit pattern of `r' encoded as a {NATURAL} value
			--| Assumptions:
			--| `Result' is encoded as a 32-bit value;
			--| `r' is encoded as a 32-bit, base 2, IEEE 754, floating point number with the
			--| following parameters:
			--| sign bit: 1 bit;
			--| exponent width: 8 bits;
			--| significand (mantissa) precision: 24 bits (23 explicitly stored).
			--| `r' layout: s_eeee_eeee_mmm_mmmm_mmmm_mmmm_mmmm_mmmm
		external
			"C inline"
		alias
			"{
				union {
					EIF_REAL r;
					EIF_NATURAL n;
					} rn;
				rn.r = $r;
				return rn.n;
				}"
		ensure
			class
		end

invariant
	Min_value: not is_nan and not is_negative_infinity implies Min_value <= value
	Max_value: not is_nan and not is_positive_infinity implies value<= Max_value
	Min_value_definition: Min_value = - Max_value
	Max_value_definition:
		Max_value = 2 ^ Greatest_exponent * (1 + Greatest_mantissa * Mantissa_unit)
	Machine_epsilon: Machine_epsilon = 2 ^ 0 * (1 + 1 * Mantissa_unit) - 1
	Epsilon: Epsilon = 2 ^ Least_normal_exponent * (1 + 1 * Mantissa_unit)
	native_real_width: 1+ Native_exponent_width + Native_mantissa_width = {PLATFORM}.Real_32_bits
	Native_exponent_mask:
		Native_exponent_mask = (2 ^ Native_exponent_width - 1).truncated_to_integer.as_natural_32
	Greatest_native_biased_exponent:
		Greatest_native_biased_exponent = Native_exponent_mask.as_integer_32
	Native_exponent_bias: Native_exponent_bias = (Greatest_native_biased_exponent - 1) // 2
	real_width: 1 + Exponent_width + Mantissa_width = 6
	Exponent_mask: Exponent_mask = (2 ^ Exponent_width - 1).truncated_to_integer.as_natural_32
	Greatest_biased_exponent: Greatest_biased_exponent = Exponent_mask.as_integer_32
	Exponent_bias: Exponent_bias = (Greatest_biased_exponent - 1) // 2
	Greatest_exponent: Greatest_exponent = Exponent_bias
	Least_normal_exponent: Least_normal_exponent = 1 - Exponent_bias
	Least_subnormal_exponent: Least_subnormal_exponent = Least_normal_exponent - Mantissa_width
	Mantissa_mask: Mantissa_mask = (2 ^ Mantissa_width - 1).truncated_to_integer.as_natural_32
	Greatest_mantissa: Greatest_mantissa = Mantissa_mask
	Mantissa_scale: Mantissa_scale = (2 ^ Mantissa_width).truncated_to_integer.as_natural_32
	Mantissa_unit: Mantissa_unit = 1 / Mantissa_scale

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
