﻿note
	description: "[
		Number in the form a+bi, where a and b are real numbers and i is the imaginary unit, i.e. i^2 = -1.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	STS_COMPLEX

inherit

	STS_NUMBER

feature -- Primitive

	a,
	real: like real_anchor
			-- Real part of this number
		deferred
		end

	b,
	imaginary: like real_anchor
			-- Imaginary part of this number
		deferred
		end

--feature -- Membership

--	is_in (s: STS_SET [STS_COMPLEX, STS_EQUALITY [STS_COMPLEX]]): BOOLEAN
--			-- Does `s` have this number?
--		do
--			Result := s ∋ Current
--		ensure
--			definition: Result = s ∋ Current
--		end

feature -- Access

	zero: like natural_anchor
			-- <Precursor>
		deferred
		ensure then
			definition: Result.value = 0
		end

	one: like natural_anchor
			-- <Precursor>
		deferred
		ensure then
			definition: Result.value = 1
		end

feature -- Quality

	is_invertible: BOOLEAN
			-- Does this number have a multiplicative `inverse' (AKA `reciprocal')?
			-- todo: redefine for rationals, integers and naturals?
		local
			abs_square: STS_REAL
		do
			if a ≭ zero then
				if b ≭ zero then
					abs_square := a * a + b * b
					Result := a.divisible (abs_square) and (- b).divisible (abs_square)
				else
					Result := (a.divisible (a) and (- b).divisible (a)) and then ((a / a).divisible (a) and (- b / a).divisible (a))
				end
			elseif b ≭ zero then
				Result := (a.divisible (b) and (- b).divisible (b)) and then ((a / b).divisible (b) and (- b / b).divisible (b))
			end
		ensure
			zero_zero: a ≍ zero and b ≍ zero implies not Result
			zero_bi: a ≍ zero and b ≭ zero implies Result = ((a.divisible (b) and (- b).divisible (b)) and then ((a / b).divisible (b) and (- b / b).divisible (b)))
			a_zero: a ≭ zero and b ≍ zero implies Result = ((a.divisible (a) and (- b).divisible (a)) and then ((a / a).divisible (a) and (- b / a).divisible (a)))
			a_bi: a ≭ zero and b ≭ zero implies Result = (a.divisible (a * a + b * b) and (- b).divisible (a * a + b * b))
		end

feature -- Status report

	divisible (other: STS_COMPLEX): BOOLEAN
			-- <Precursor>
		do
			Result := other.is_invertible
		ensure then
			definition: Result = other.is_invertible
		end

feature -- Comparison

	equals alias "≍" (z: STS_COMPLEX): BOOLEAN
			-- Do this complex number and `z' have the same `real' and `imaginary' parts, respectively?
		do
			Result := a ≍ z.a and b ≍ z.b
		ensure
			definition: Result = (a ≍ z.a and b ≍ z.b)
		end

	unequals alias "≭" (z: STS_COMPLEX): BOOLEAN
			-- Does not this complex number equal `z'?
		do
			Result := not (Current ≍ z)
		ensure
			definition: Result = not (Current ≍ z)
		end

feature -- Basic operations

	conjugate: like complex_anchor
			-- Complex number in the form `a' - `b'i
		deferred
		ensure
			real: Result.a ≍ a
			imaginary: Result.b ≍ - b
		end

	plus alias "+" (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		deferred
		ensure then
			real: Result.a ≍ (a + other.a)
			imaginary: Result.b ≍ (b + other.b)
		end

	identity alias "+": like Current
			-- <Precursor>
		do
			Result := Current
		ensure then
			real: Result.a ≍ a
			imaginary: Result.b ≍ b
		end

	minus alias "-" (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		deferred
		ensure then
			real: Result.a ≍ (a - other.a)
			imaginary: Result.b ≍ (b - other.b)
		end

	opposite alias "-": like complex_anchor
			-- <Precursor>
		do
			Result := zero.complex_minus (Current)
		ensure then
			real: Result.a ≍ - a
			imaginary: Result.b ≍ - b
		end

	product alias "*" (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		deferred
		ensure then
			real: Result.a ≍ (a * other.a - b * other.b)
			imaginary: Result.b ≍ (b * other.a + a * other.b)
		end

	quotient alias "/" (other: STS_COMPLEX): like complex_anchor
			-- <Precursor>
		do
				check
					is_invertible: other.is_invertible -- divisible (other)
				end
			Result := Current * other.inverse
		ensure then
			is_invertible: other.is_invertible -- divisible (other)
			real: Result.a ≍ (Current * other.inverse).a
			imaginary: Result.b ≍ (Current * other.inverse).b
		end

	reciprocal,
	inverse alias "⁻¹": like complex_anchor
			-- The multiplicative inverse (AKA reciprocal) of this bumber, i.e. `one' divided by this number.
		require
			is_invertible: is_invertible
		deferred
		ensure
			zero_bi_real: a ≍ zero and b ≭ zero implies (a.divisible (b) and (a / b).divisible (b)) -- `is_invertible' definition
				and then Result.a ≍ (a / b / b)
			a_zero_real: a ≭ zero and b ≍ zero implies (a.divisible (a) and (a / a).divisible (a)) -- `is_invertible' definition
				and then Result.a ≍ (a / a / a)
			a_bi_real: a ≭ zero and b ≭ zero implies a.divisible (a * a + b * b) -- `is_invertible' definition
				and then Result.a ≍ (a / (a * a + b * b))

			zero_bi_imaginary: a ≍ zero and b ≭ zero implies ((- b).divisible (b) and (- b / b).divisible (b)) -- `is_invertible' definition
				and then Result.b ≍ (- b / b / b)
			a_zero_imaginary: a ≭ zero and b ≍ zero implies ((- b).divisible (a) and (- b / a).divisible (a)) -- `is_invertible' definition
				and then Result.b ≍ (- b / a / a)
			a_bi_imaginary: a ≭ zero and b ≭ zero implies (- b).divisible (a * a + b * b) -- `is_invertible' definition
				and then Result.b ≍ (- b / (a * a + b * b))
		end

feature -- Anchor

	complex_anchor: STS_COMPLEX
			-- Anchor for complex numbers
		do
			Result := Current
		end

	real_anchor: STS_REAL
			-- Anchor for real numbers
		do
			Result := a
		end

--	opposite_anchor: like complex_anchor
--			-- Anchor for `opposite' operation
--		do
--			Result := - Current
--		end

--	reciprocal_anchor: like complex_anchor
--			-- Anchor for `reciprocal' operation
--		do
--			Result := reciprocal
--		end

	natural_anchor: STS_NATURAL
			-- Anchor for natural numbers
		do
			Result := zero
		end

invariant
	real: real ≍ a
	imaginary: imaginary ≍ b

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end

