﻿note
	description: "Element that models an integer number"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	STS_INTEGER

inherit

	STS_ELEMENT

--	STS_RATIONAL
--		rename
--				-- Primitive
--			value as real_value,
--				-- Status report
--			divisible as rational_divisible,
--				-- Comparison
--			equals as rational_equals,
--			unequals as rational_unequals,
--			is_less as rational_is_less,
--			is_less_equal as rational_is_less_equal,
--			is_greater as rational_is_greater,
--			is_greater_equal as rational_is_greater_equal,
--			native_three_way_comparison as rational_native_three_way_comparison,
--			three_way_comparison as rational_three_way_comparison,
--			min as rational_min,
--			max as rational_max,
--				-- Basic operations
--			plus as rational_plus,
--			minus as rational_minus,
--			product as rational_product,
--			quotient as rational_quotient,
--				-- Factory
--			normalized_value as real_normalized_value
--		redefine
--				-- Status report
--			exponentiable,
--				-- Basic operations
--			abs,
--			opposite
--		select
--				-- Status report
--			complex_divisible,
--				-- Comparison
--			real_is_less,
--			real_is_less_equal,
--			real_is_greater,
--			real_is_greater_equal,
--			real_native_three_way_comparison,
--			real_min,
--			real_max,
--				-- Basic operations
--			complex_plus,
--			complex_minus,
--			complex_product,
--			complex_quotient
--		end

--	STS_NUMBER
--		rename
--				-- Basic operations
--			quotient as integer_quotient alias "//"
--		undefine
--				-- Comparison
--			is_equal
--		end

--	COMPARABLE
--		rename
--				-- Comparison
--			three_way_comparison as native_three_way_comparison
--		redefine -- todo: properties, tests...
--				-- Comparison
--			is_less_equal,
--			is_greater,
--			is_greater_equal,
--			native_three_way_comparison,
--			min,
--			max
--		end

feature -- Primitive

	value: INTEGER
			-- Native value of this integer number
		deferred
		end

feature -- Access

	zero: STS_INTEGER
			-- Integer whose `value' is 0
		deferred
		end

	one: STS_INTEGER
			-- Integer whose `value' is 1
		deferred
		end

feature -- Relationship

	divisible (other: STS_INTEGER): BOOLEAN
			-- May current integer be divided by `other`?
		do
			Result := not (other ≍ zero)
		ensure
			definition: Result = not (other ≍ zero)
		end

--	exponentiable (other: NUMERIC): BOOLEAN
--			-- <Precursor>
--		do
--			Result := value.exponentiable (other)
--		ensure then
--			definition: Result = value.exponentiable (other)
--		end

feature -- Comparison

	equals alias "≍" (i: STS_INTEGER): BOOLEAN
			-- Do this integer number and `i' have the same numeric value?
		do
			Result := value = i.value
		ensure
			definition: Result = (value = i.value)
		end

	unequals alias "≭" (i: STS_INTEGER): BOOLEAN
			-- Does not this integer number equal `i'?
		do
			Result := not (Current ≍ i)
		ensure
			definition: Result = not (Current ≍ i)
		end

	is_less alias "<" (other: STS_INTEGER): BOOLEAN
			-- Is current integer less than `other`?
		do
			Result := value < other.value
		ensure
			definition: Result = (value < other.value)
		end

	is_less_equal alias "<=" (other: STS_INTEGER): BOOLEAN
			-- Is current integer less than or equal to `other`?
		do
			Result := Current < other or Current ≍ other
		ensure
			definition: Result = (Current < other or Current ≍ other)
		end

	is_greater alias ">" (other: STS_INTEGER): BOOLEAN
			-- Is current integer greater than `other`?
		do
			Result := other < Current
		ensure
			definition: Result = (other < Current)
		end

	is_greater_equal alias ">=" (other: STS_INTEGER): BOOLEAN
			-- Is current integer greater than or equal to `other`?
		do
			Result := other <= Current
		ensure
			definition: Result = (other <= Current)
		end

	native_three_way_comparison (other: STS_INTEGER): INTEGER
			-- If current integer equal to `other`, 0; if smaller, -1; if greater, 1.
		do
			if Current < other then
				Result := - 1
			elseif other < Current then
				Result := 1
			end
		ensure
			equal_zero: (Result = 0) = (Current ≍ other)
			smaller_negative: (Result = -1) = (Current < other)
			greater_positive: (Result = 1) = (Current > other)
		end

	three_way_comparison (other: STS_INTEGER): STS_INTEGER
			-- If current integer equal to `other`, 0; if smaller, -1; if greater, 1.
		do
			if Current < other then
				Result := - one
			elseif other < Current then
				Result := one
			else
				Result := zero
			end
		ensure
			equal_zero: (Result ≍ zero) = (Current ≍ other)
			smaller_negative: (Result ≍ - one) = (Current < other)
			greater_positive: (Result ≍ one) = (Current > other)
		end

	min (other: STS_INTEGER): STS_INTEGER
			-- The smaller of current integer and `other`
		do
			if Current <= other then
				Result := Current
			else
				Result := other
			end
		ensure
			current_if_not_greater: Current <= other implies Result ≍ Current
			other_if_greater: Current > other implies Result ≍ other
		end

	max (other: STS_INTEGER): STS_INTEGER
			-- The greater of current integer and `other`
		do
			if Current >= other then
				Result := Current
			else
				Result := other
			end
		ensure
			current_if_not_smaller: Current >= other implies Result ≍ Current
			other_if_smaller: Current < other implies Result ≍ other
		end

feature -- Operation

	abs: STS_INTEGER
			-- Absolute value of this integer
		do
			if Current < zero then
				Result := - Current
			else
				Result := Current
			end
		ensure
			negative: Current < zero implies Result ≍ - Current
			non_negative: Current >= zero implies Result ≍ Current
		end

	plus alias "+" (other: STS_INTEGER): STS_INTEGER
			-- Sum with `other` (commutative).
		deferred
		ensure
			definition: Result.value = normalized_value (value + other.value)
		end

	minus alias "-" (other: STS_INTEGER): STS_INTEGER
			-- Result of subtracting `other`
		deferred
		ensure
			definition: Result.value = normalized_value (value - other.value)
		end

	opposite alias "-": STS_INTEGER
			-- Unary minus
		do
			Result := zero - Current
		ensure
			definition: Result ≍ (zero - Current)
		end

	product alias "*" (other: STS_INTEGER): STS_INTEGER
			-- Product by `other`
		deferred
		ensure
			definition: Result.value = normalized_value (value * other.value)
		end

--	quotient alias "/" (other: STS_INTEGER): like rational_anchor
--			-- Division by `other`
--		require
--			good_divisor: divisible (other)
--		do
--			Result := reduced (Current, other)
--		ensure
--			numerator: Result.p ≍ reduced (Current, other).p
--			denominator: Result.q ≍ reduced (Current, other).q
--		end

	integer_quotient alias "//" (other: STS_INTEGER): STS_INTEGER
			-- Division by `other`
		require
			good_divisor: divisible (other)
		deferred
		ensure
			good_divisor: other.value /= 0 -- divisible (other)
			definition: Result.value = normalized_value (value // other.value)
		end

	integer_remainder alias "\\" (other: STS_INTEGER): STS_INTEGER
			-- Remainder of the integer division of `Current' by `other`
		require
			good_divisor: divisible (other)
		deferred
		ensure
			good_divisor: other.value /= 0 -- divisible (other)
			definition: Result.value = normalized_value (value \\ other.value)
		end

feature -- Factory

	normalized_value (v: INTEGER): INTEGER
			-- `v' adapted to be represented as the `value' of this number
		deferred
		ensure
			stable: normalized_value (Result) = Result
		end

invariant
--	is_integer: q ≍ one

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
