﻿note
	description: "Element that models a natural number"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	STS_NATURAL

inherit

	STS_INTEGER
		rename
				-- Primitive
			value as integer_value,
				-- Status report
			divisible as integer_divisible,
				-- Comparison
			equals as integer_equals,
			unequals as integer_unequals,
			is_less as integer_is_less,
			is_less_equal as integer_is_less_equal,
			is_greater as integer_is_greater,
			is_greater_equal as integer_is_greater_equal,
			native_three_way_comparison as integer_native_three_way_comparison,
			three_way_comparison as integer_three_way_comparison,
			min as integer_min,
			max as integer_max,
				-- Basic operations
			plus as integer_plus,
			minus as integer_minus,
			product as integer_product,
			integer_quotient as integer_quotient,
			integer_remainder as integer_remainder,
				-- Factory
			normalized_value as integer_normalized_value
		redefine
				-- Basic operations
			abs,
			sign,
			opposite
		select
				-- Status report
			complex_divisible,
				-- Comparison
			real_is_less,
			real_is_less_equal,
			real_is_greater,
			real_is_greater_equal,
			real_min,
			real_max,
				-- Basic operations
			complex_plus,
			complex_minus,
			complex_product,
			complex_quotient
		end

	STS_NUMBER
		rename
				-- Basic operations
			quotient as natural_quotient alias "//"
		undefine
				-- Comparison
			is_equal
		end

	COMPARABLE
		rename
				-- Comparison
			three_way_comparison as native_three_way_comparison
		redefine
				-- Comparison
			is_less_equal,
			is_greater,
			is_greater_equal,
			native_three_way_comparison,
			min,
			max
		select
				-- Comparison
			native_three_way_comparison -- todo: really?
		end

feature -- Primitive

	value: NATURAL
			-- Native value of this natural number
		deferred
		end

feature -- Access

	sign: like natural_anchor
			-- Sign of this number as a natural value (0 or 1)
		do
			Result := if zero < Current then one else zero end
		end

	zero: like natural_anchor
			-- <Precursor>
		deferred
		end

	one: like natural_anchor
			-- <Precursor>
		deferred
		end

feature -- Status report

	divisible (other: STS_NATURAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- May current object be divided by `other`?
		do
			Result := not (other ≍ zero)
		ensure then
			definition: Result = not (other ≍ zero)
		end

feature -- Comparison

	equals alias "≍" (n: STS_NATURAL): BOOLEAN
			-- Do this natural number and `n' have the same numeric value?
		do
			Result := value = n.value
		ensure
			definition: Result = (value = n.value)
		end

	unequals alias "≭" (n: STS_NATURAL): BOOLEAN
			-- Does not this natural number equal `n'?
		do
			Result := not (Current ≍ n)
		ensure
			definition: Result = not (Current ≍ n)
		end

	is_less alias "<" (other: STS_NATURAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Is current object less than `other`?
		do
			Result := value < other.value
		ensure then
			definition: Result = (value < other.value)
		end

	is_less_equal alias "<=" (other: STS_NATURAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Is current object less than or equal to `other`?
		do
			Result := not (other < Current) -- todo: redefine?
		end

	is_greater alias ">" (other: STS_NATURAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Is current object greater than `other`?
		do
			Result := other < Current -- todo: redefine?
		end

	is_greater_equal alias ">=" (other: STS_NATURAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Is current object greater than or equal to `other`?
		do
			Result := not (Current < other) -- todo: redefine?
		end

	native_three_way_comparison (other: STS_NATURAL): INTEGER
			-- <Precursor>
			-- todo: ancestor versions listed wrongly.
		do
			if Current < other then
				Result := -1
			elseif other < Current then
				Result := 1
			end
		end

	three_way_comparison (other: STS_NATURAL): like integer_anchor
			-- If current object equal to `other`, 0;
			-- if smaller, -1; if greater, 1
		do
			if Current < other then
				Result := - one
			elseif other < Current then
				Result := one
			else
				Result := zero
			end
		ensure
			equal_zero: (Result ≍ zero) = (Current ≍ other)
			smaller_negative: (Result ≍ - one) = (Current < other)
			greater_positive: (Result ≍ one) = (Current > other)
		end

	min (other: STS_NATURAL): STS_NATURAL
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- The smaller of current object and `other`
		do
			if Current <= other then
				Result := Current
			else
				Result := other
			end
		end

	max (other: STS_NATURAL): STS_NATURAL
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- The greater of current object and `other`
		do
			if Current >= other then
				Result := Current
			else
				Result := other
			end
		end

feature -- Basic operations

	abs: like Current
			-- <Precursor>
		do
			Result := Current
		end

	plus alias "+" (other: STS_NATURAL): STS_NATURAL
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Sum with `other` (commutative).
		deferred
		ensure then
			definition: Result.value = normalized_value (value + other.value)
		end

	minus alias "-" (other: STS_NATURAL): like natural_anchor
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Result of subtracting `other`
			-- todo: rely on integer minus instead?
		deferred
		ensure then
			definition: Result.value = normalized_value (value - other.value)
		end

	opposite alias "-": like integer_anchor
			-- <Precursor>
		do
			Result := Precursor {STS_INTEGER}
		end

	product alias "*" (other: STS_NATURAL): like natural_anchor
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Product by `other`
		deferred
		ensure then
			definition: Result.value = normalized_value (value * other.value)
		end

	natural_quotient alias "//" (other: STS_NATURAL): like natural_anchor
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Division by `other`
		deferred
		ensure then
			good_divisor: other.value /= 0 -- divisible (other)
			definition: Result.value = normalized_value (value // other.value)
		end

	natural_remainder alias "\\" (other: STS_NATURAL): like natural_anchor
			-- Remainder of the natural division of Current by `other`
		require
			good_divisor: divisible (other)
		deferred
		ensure then
			good_divisor: other.value /= 0 -- divisible (other)
			definition: Result.value = normalized_value (value \\ other.value)
		end

	factorial alias "﹗": like natural_anchor
			-- Product of all natural numbers from 1 to `Current'
		do
			if Current ≍ zero then
				Result := one
			else
				Result := Current * ﹗(Current - one)
			end
		ensure -- todo: rewrite as a reduction
			base: Current ≍ zero implies Result ≍ one
			induction: Current > zero implies Result ≍ (Current * ﹗(Current - one))
		end

feature -- Factory

	normalized_value (v: NATURAL): NATURAL
			-- `v' adapted to be represented as the `value' of this number
		deferred
		ensure
			stable: normalized_value (Result) = Result
		end

feature -- Anchor

--	natural_anchor: STS_NATURAL
--			-- Anchor for natural numbers
--		do
--			Result := Current
--		end

invariant
	consistent_value: value.as_integer_32 = integer_value
	non_negative: 0 <= integer_value

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
