﻿note
	description: "Set of ordered numbers"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	STS_ORDERED_NUMERIC_SET [A -> {STS_NUMBER, COMPARABLE}, EQ -> STS_EQUALITY [A]]

inherit

	STS_NUMERIC_SET [A, EQ]
		redefine
				-- Anchor
			set_anchor,
			subset_anchor,
			superset_anchor
		end

feature -- Interval

	closed_interval alias "[]" (a, b: A): like subset_anchor
			-- Set of every number x in this set such that `a' <= x <= `b', i.e. [`a', `b'].
		do
			Result := Current | agent anded
				(agent number_is_less_equal (a, ?), agent number_is_less_equal (?, b), ?)
		ensure
			definition: Result ≍ (
				Current | agent anded
					(agent number_is_less_equal (a, ?), agent number_is_less_equal (?, b), ?)
				)
		end

	left_closed_right_open_interval (a, b: A): like subset_anchor
			-- Set of every number x in this set such that `a' <= x < `b', i.e. [`a', `b').
		do
			Result := Current | agent anded
				(agent number_is_less_equal (a, ?), agent number_is_less (?, b), ?)
		ensure
			definition: Result ≍ (
				Current | agent anded
					(agent number_is_less_equal (a, ?), agent number_is_less (?, b), ?)
				)
		end

	left_open_right_closed_interval (a, b: A): like subset_anchor
			-- Set of every number x in this set such that `a' < x <= `b', i.e. (`a', `b'].
		do
			Result := Current | agent anded
				(agent number_is_less (a, ?), agent number_is_less_equal (?, b), ?)
		ensure
			definition: Result ≍ (
				Current | agent anded
					(agent number_is_less (a, ?), agent number_is_less_equal (?, b), ?)
				)
		end

	open_interval alias "()" (a, b: A): like subset_anchor
			-- Set of every number x in this set such that `a' < x < `b', i.e. (`a', `b').
		do
			Result := Current | agent anded
				(agent number_is_less (a, ?), agent number_is_less (?, b), ?)
		ensure
			definition: Result ≍ (
				Current | agent anded (agent number_is_less (a, ?), agent number_is_less (?, b), ?)
				)
		end

	left_closed_right_unbounded_interval (a: A): like subset_anchor
			-- Set of every number x in this set such that `a' <= x, i.e. [`a', →).
		do
			Result := Current | agent number_is_less_equal(a, ?)
		ensure
			definition: Result ≍ (Current | agent number_is_less_equal(a, ?))
		end

	left_open_right_unbounded_interval (a: A): like subset_anchor
			-- Set of every number x in this set such that `a' < x, i.e. (`a', →).
		do
			Result := Current | agent number_is_less(a, ?)
		ensure
			definition: Result ≍ (Current | agent number_is_less(a, ?))
		end

	left_unbounded_right_closed_interval (b: A): like subset_anchor
			-- Set of every number x in this set such that x <= `b', i.e. (←, `b'].
		do
			Result := Current | agent number_is_less_equal(?, b)
		ensure
			definition: Result ≍ (Current | agent number_is_less_equal(?, b))
		end

	left_unbounded_right_open_interval (b: A): like subset_anchor
			-- Set of every number x in this set such that x < `b', i.e. (←, `b')
		do
			Result := Current | agent number_is_less(?, b)
		ensure
			definition: Result ≍ (Current | agent number_is_less(?, b))
		end

	unbounded_interval: like Current
			-- This set itself, i.e. (←, →).
		do
			Result := Current
		ensure
			definition: Result ≍ Current
		end

feature -- Predicate

	number_is_less (x, y: A): BOOLEAN
			-- Is `x' less than `y'?
		do
			Result := x < y
		ensure then
			class
			definition: Result = (x < y)
		end

	number_is_less_equal (x, y: A): BOOLEAN
			-- Is `x' less than or equal to `y'?
		do
			Result := x <= y
		ensure then
			class
			definition: Result = (x <= y)
		end

feature -- Anchor

	set_anchor,
	subset_anchor,
	superset_anchor: STS_ORDERED_NUMERIC_SET [A, EQ]
			-- <Precursor>
		do
			Result := Current
		end

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
