﻿note
	description: "[
		Number in the form p/q, where p and q are integer numbers and q /= 0.
		]"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

deferred class
	STS_RATIONAL

inherit

	STS_REAL
		rename
				-- Status report
			divisible as real_divisible,
				-- Comparison
			equals as real_equals,
			unequals as real_unequals,
			is_less as real_is_less,
			is_less_equal as real_is_less_equal,
			is_greater as real_is_greater,
			is_greater_equal as real_is_greater_equal,
			native_three_way_comparison as real_native_three_way_comparison,
			three_way_comparison as real_three_way_comparison,
			min as real_min,
			max as real_max,
				-- Basic operations
			plus as real_plus,
			minus as real_minus,
			product as real_product,
			quotient as real_quotient,
			power as real_power alias "|^"
		redefine
				-- Status report
			exponentiable,
				-- Basic operations
			identity,
			opposite,
			reciprocal, inverse
		select
				-- Status report
			complex_divisible,
				-- Comparison
			real_is_less,
			real_is_less_equal,
			real_is_greater,
			real_is_greater_equal,
			real_native_three_way_comparison,
			real_min,
			real_max,
				-- Basic operations
			complex_plus,
			complex_minus,
			complex_product,
			complex_quotient
		end

	STS_NUMBER
		undefine
				-- Comparison
			is_equal
		end

	COMPARABLE
		rename
				-- Comparison
			three_way_comparison as native_three_way_comparison
		redefine -- todo: properties, tests...
				-- Comparison
			is_less_equal,
			is_greater,
			is_greater_equal,
			native_three_way_comparison,
			min,
			max
		end

feature -- Primitive

	p,
	numerator: like integer_anchor
			-- Numerator of this rational number
		deferred
		end

	q,
	denominator: like integer_anchor
			-- Denominator of this rational number
		deferred
		end

feature -- Access

	zero: like natural_anchor
			-- <Precursor>
		deferred
		end

	one: like natural_anchor
			-- <Precursor>
		deferred
		end

feature -- Status report

	divisible (other: STS_RATIONAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- May current number be divided by `other`?
		do
			Result := not (other ≍ zero)
		ensure then
			definition: Result = not (other ≍ zero)
		end

	exponentiable (other: NUMERIC): BOOLEAN
			-- <Precursor>
		do
			Result := p.exponentiable (other) and q.exponentiable (other)
		ensure then
			definition: Result = (p.exponentiable (other) and q.exponentiable (other)) -- todo: is it safe?
		end

feature -- Comparison

	equals alias "≍" (r: STS_RATIONAL): BOOLEAN
			-- Do this rational number and `r' have the same `numerator' and `denominator', respectively?
			-- NOTICE: Hence, every rational number must be represented in canonical, unique, form.
		do
			Result := p ≍ r.p and q ≍ r.q
		ensure
			definition: Result = (p ≍ r.p and q ≍ r.q)
		end

	unequals alias "≭" (r: STS_RATIONAL): BOOLEAN
			-- Does not this rational number equal `r'?
		do
			Result := not (Current ≍ r)
		ensure
			definition: Result = not (Current ≍ r)
		end

	is_less alias "<" (other: STS_RATIONAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Is current object less than `other`?
		do
				check
					good_divisor: p.divisible (q) -- class invariant
					other_good_divisor: other.p.divisible (other.q) -- class invariant
				end
			Result := (p // q < other.p // other.q) or ((p // q) ≍ (other.p // other.q) and (p \\ q < other.p \\ other.q))
		ensure then
			good_divisor: p.divisible (q) -- class invariant
			other_good_divisor: other.p.divisible (other.q) -- class invariant
			definition: Result = ((p // q < other.p // other.q) or ((p // q) ≍ (other.p // other.q) and (p \\ q < other.p \\ other.q)))
		end

	is_less_equal alias "<=" (other: STS_RATIONAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Is current object less than or equal to `other`?
		do
			Result := not (other < Current)
		end

	is_greater alias ">" (other: STS_RATIONAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Is current object greater than `other`?
		do
			Result := other < Current
		end

	is_greater_equal alias ">=" (other: STS_RATIONAL): BOOLEAN
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Is current object greater than or equal to `other`?
		do
			Result := not (Current < other)
		end

	native_three_way_comparison (other: STS_RATIONAL): INTEGER
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- If current object equal to `other`, 0;
			-- if smaller, -1; if greater, 1
		do
			if Current < other then
				Result := - 1
			elseif other < Current then
				Result := 1
			end
		end

	three_way_comparison (other: STS_RATIONAL): like integer_anchor
			-- If current object equal to `other`, 0;
			-- if smaller, -1; if greater, 1
		do
			if Current < other then
				Result := - one
			elseif other < Current then
				Result := one
			else
				Result := zero
			end
		ensure
			equal_zero: (Result ≍ zero) = (Current ≍ other)
			smaller_negative: (Result ≍ - one) = (Current < other)
			greater_positive: (Result ≍ one) = (Current > other)
		end

	min (other: STS_RATIONAL): STS_RATIONAL
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- The smaller of current object and `other`
		do
			if Current <= other then
				Result := Current
			else
				Result := other
			end
		end

	max (other: STS_RATIONAL): STS_RATIONAL
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- The greater of current object and `other`
		do
			if Current >= other then
				Result := Current
			else
				Result := other
			end
		end

feature -- Basic operations

	abs: like rational_anchor
			-- <Precursor>
		do
			if Current < zero then
				Result := - Current
			else
				Result := Current
			end
		ensure then
			opposite_numerator: Current < zero implies Result.p ≍ (- Current).p
			identity_numerator: Current >= zero implies Result.p ≍ p
			opposite_denominator: Current < zero implies Result.q ≍ (- Current).q
			identity_denominator: Current >= zero implies Result.q ≍ q
		end

	plus alias "+" (other: STS_RATIONAL): like rational_anchor
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Sum with `other` (commutative).
		do
				check
					q_good_divisor_gcd: q.divisible (gcd (q, other.q)) -- q /= 0, other.q /= 0
					other_q_good_divisor_gcd_: other.q.divisible (gcd (q, other.q)) -- q /= 0, other.q /= 0
					reduced_good_divisor: (
						p * (other.q \\ gcd (q, other.q)) +
						other.p * (q \\ gcd (q, other.q))
						).divisible ( -- q /= 0, other.q /= 0
							(q \\ gcd (q, other.q)) *
							(other.q \\ gcd (q, other.q))
							)
				end
			Result := reduced (
				p * (other.q \\ gcd (q, other.q)) +
				other.p * (q \\ gcd (q, other.q)),
				(q \\ gcd (q, other.q)) *
				(other.q \\ gcd (q, other.q))
				)
		ensure then
			q_good_divisor_gcd: q.divisible (gcd (q, other.q)) -- q /= 0, other.q /= 0
			other_q_good_divisor_gcd_: other.q.divisible (gcd (q, other.q)) -- q /= 0, other.q /= 0
			reduced_good_divisor: (
				p * (other.q \\ gcd (q, other.q)) +
				other.p * (q \\ gcd (q, other.q))
				).divisible ( -- q /= 0, other.q /= 0
					(q \\ gcd (q, other.q)) *
					(other.q \\ gcd (q, other.q))
					)
			numerator: Result.p ≍ reduced (
				p * (other.q \\ gcd (q, other.q)) +
				other.p * (q \\ gcd (q, other.q)),
				(q \\ gcd (q, other.q)) *
				(other.q \\ gcd (q, other.q))
				).p
			denominator: Result.q ≍ reduced (
				p * (other.q \\ gcd (q, other.q)) +
				other.p * (q \\ gcd (q, other.q)),
				(q \\ gcd (q, other.q)) *
				(other.q \\ gcd (q, other.q))
				).q
		end

	identity alias "+": like Current
			-- <Precursor>
		do
			Result := Current
		ensure then
			numerator: Result.p ≍ p
			denominator: Result.q ≍ q
		end

	minus alias "-" (other: STS_RATIONAL): like rational_anchor
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Result of subtracting `other`
		do
				check
					q_good_divisor_gcd: q.divisible (gcd (q, other.q)) -- q /= 0, other.q /= 0
					other_q_good_divisor_gcd_: other.q.divisible (gcd (q, other.q)) -- q /= 0, other.q /= 0
					reduced_good_divisor: (
						p * (other.q \\ gcd (q, other.q)) +
						other.p * (q \\ gcd (q, other.q))
						).divisible ( -- q /= 0, other.q /= 0
							(q \\ gcd (q, other.q)) *
							(other.q \\ gcd (q, other.q))
							)
				end
			Result := reduced (
				p * (other.q \\ gcd (q, other.q)) -
				other.p * (q \\ gcd (q, other.q)),
				(q \\ gcd (q, other.q)) *
				(other.q \\ gcd (q, other.q))
				)
		ensure then
			q_good_divisor_gcd: q.divisible (gcd (q, other.q)) -- q /= 0, other.q /= 0
			other_q_good_divisor_gcd_: other.q.divisible (gcd (q, other.q)) -- q /= 0, other.q /= 0
			reduced_good_divisor: (
				p * (other.q \\ gcd (q, other.q)) -
				other.p * (q \\ gcd (q, other.q))
				).divisible ( -- q /= 0, other.q /= 0
					(q \\ gcd (q, other.q)) *
					(other.q \\ gcd (q, other.q))
					)
			numerator: Result.p ≍ reduced (
				p * (other.q \\ gcd (q, other.q)) -
				other.p * (q \\ gcd (q, other.q)),
				(q \\ gcd (q, other.q)) *
				(other.q \\ gcd (q, other.q))
				).p
			denominator: Result.q ≍ reduced (
				p * (other.q \\ gcd (q, other.q)) -
				other.p * (q \\ gcd (q, other.q)),
				(q \\ gcd (q, other.q)) *
				(other.q \\ gcd (q, other.q))
				).q
		end

	opposite alias "-": like rational_anchor
			-- <Precursor>
		do
			Result := zero.rational_minus (Current)
		ensure then
			numerator: Result.p ≍ zero.rational_minus (Current).p
			denominator: Result.q ≍ zero.rational_minus (Current).q
		end

	product alias "*" (other: STS_RATIONAL): like rational_anchor
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Product by `other`
		do
				check
					good_divisor: (p * other.p).divisible (q * other.q) -- q /= 0, other.q /= 0
				end
			Result := reduced (p * other.p, q * other.q)
		ensure then
			good_divisor: (p * other.p).divisible (q * other.q) -- q /= 0, other.q /= 0
			numerator: Result.p ≍ reduced (p * other.p, q * other.q).p
			denominator: Result.q ≍ reduced (p * other.p, q * other.q).q
		end

	quotient alias "/" (other: STS_RATIONAL): like rational_anchor
			-- <Precursor>
			-- todo: why is <Precursor> empty?
			-- Division by `other`
		do
				check
					good_divisor: (p * other.q).divisible (q * other.p) -- q /= 0, other.p /= 0 <== divisible (other)
				end
			Result := reduced (p * other.q, q * other.p)
		ensure then
			good_divisor: (p * other.q).divisible (q * other.p) -- q /= 0, other.p /= 0 <== divisible (other)
			numerator: Result.p ≍ reduced (p * other.q, q * other.p).p
			denominator: Result.q ≍ reduced (p * other.q, q * other.p).q
		end

	reciprocal,
	inverse alias "⁻¹": like rational_anchor
			-- <Precursor>
		do
				check
					one.rational_divisible (Current) -- is_invertible
				end
			Result := one.rational_quotient (Current)
		ensure then
			good_divisor: one.rational_divisible (Current) -- is_invertible
			numerator: Result.p ≍ one.rational_quotient (Current).p
			denominator: Result.q ≍ one.rational_quotient (Current).q
		end

	power alias "^" (other: STS_INTEGER): like rational_anchor
			-- Current rational number to the power `other'
		require
			exponentiable: exponentiable (other)
		do
			if other < zero then
				Result := (⁻¹ Current) ^ other.abs -- todo: check this precondition
			elseif other ≍ zero then
				Result := one
			else
				Result := Current * (Current ^ (other - one)) -- todo: check this precondition
			end
		ensure
			negative: other < zero implies Result ≍ ((⁻¹ Current) ^ other.abs)
			base: other ≍ zero implies Result ≍ one
			induction: other > zero implies Result ≍ (Current * (Current ^ (other - one)))
		end

feature -- Arithmetic

	gcd (n, d: STS_INTEGER): like integer_anchor
			-- Greatest common divisor of `n' and `d'
		do
			if d ≍ zero then
				Result := n.abs
			else
					check
						good_divisor: n.divisible (d)
					end
				Result := gcd (d, n \\ d)
			end
		ensure
			base: d ≍ zero implies Result ≍ n.abs
			good_divisor: d ≭ zero implies n.divisible (d)
			induction: d ≭ zero implies Result ≍ gcd (d, n \\ d)
		end

feature -- Factory

	reduced (n, d: STS_INTEGER): like rational_anchor
			-- The rational number `n'/`d', reduced to its lowest terms.
		require
			good_divisor: n.divisible (d)
		deferred
		ensure
			n_good_divisor: n.abs.divisible (gcd (n, d)) -- gcd (n, d) /= 0
			d_good_divisor: d.abs.divisible (gcd (n, d)) -- gcd (n, d) /= 0
			numerator: Result.p.abs ≍ (n.abs // gcd (n, d))
			denominator: Result.q.abs ≍ (d.abs // gcd (n, d))
		end

feature -- Anchor

	rational_anchor: STS_RATIONAL
			-- Anchor for rational numbers
		do
			Result := Current
		end

invariant
	numerator: numerator ≍ p
	denominator: denominator ≍ q
	good_divisor: q ≭ zero
	irreducible: gcd (p, q) ≍ one
	divisible: q.value /= 0 -- q ≭ zero
	value: value = normalized_value ((p.value / q.value).truncated_to_real)

note
	copyright: "Copyright (c) 2012-2019, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
