﻿note
	description: "Test suite for {STI_COMPLEX_NUMBER}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	COMPLEX_NUMBER_TESTS

inherit
	ELEMENT_TESTS
		rename
			test_is_in as test_element_is_in,
			test_is_not_in as test_element_is_not_in,
			element_to_be_tested as complex_to_be_tested,
			some_integer_ref as some_native_integer_ref,
			some_direct_integer_ref as some_direct_native_integer_ref,
			new_integer_ref as new_native_integer_ref,
			some_integer as some_native_integer
		undefine
			some_native_integer,
			some_element
		redefine
			on_prepare,
			test_element_is_in,
			test_element_is_not_in,
			complex_to_be_tested
		end

	NUMBERS_TESTS_RESOURCES_C
		rename
			Z as Z_universe
		redefine
			on_prepare
		end

feature -- Access

	properties: COMPLEX_NUMBER_PROPERTIES
			-- Object that checks the set-theory properties of {STI_COMPLEX_NUMBER}

	homogeneous_pair_properties: HOMOGENEOUS_ORDERED_PAIR_PROPERTIES_R
			-- Object that checks the set-theory properties of {STS_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, ...]}

	composable_pair_properties: COMPOSABLE_ORDERED_PAIR_PROPERTIES_RRR
			-- Object that checks the set-theory properties of {STS_COMPOSABLE_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_NUMBER, ...]}

	resources_rr: SETS_TESTS_RESOURCES_BINARY_RR
			-- Resources of ordered pairs of real numbers

feature {NONE} -- Events

	on_prepare
			-- <Precursor>
		do
			Precursor {NUMBERS_TESTS_RESOURCES_C}
			create properties
			create homogeneous_pair_properties
			create composable_pair_properties
			create resources_rr
		end

feature -- Test routines (Initialization)

	test_default_create
			-- Test {STI_COMPLEX_NUMBER}.default_create.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.default_create"
		do
			assert ("default_create", attached (create {like complex_to_be_tested}.default_create))

			assert ("-NaN-NaN*i ok", properties.default_create_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.default_create_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.default_create_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.default_create_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.default_create_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.default_create_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.default_create_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.default_create_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.default_create_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.default_create_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.default_create_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.default_create_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.default_create_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.default_create_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.default_create_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.default_create_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.default_create_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.default_create_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.default_create_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.default_create_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.default_create_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.default_create_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.default_create_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.default_create_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.default_create_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.default_create_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.default_create_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.default_create_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.default_create_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.default_create_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.default_create_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.default_create_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.default_create_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.default_create_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.default_create_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.default_create_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.default_create_ok (-0.0*i))
			assert ("-0-0*i ok", properties.default_create_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.default_create_ok (0 - 0*i))
			assert ("a-0*i ok", properties.default_create_ok (some_real - 0*i))

			assert ("-NaN ok", properties.default_create_ok (-Nan))
			assert ("-NaN+0*i ok", properties.default_create_ok (-Nan + 0*i))
			assert ("NaN ok", properties.default_create_ok (Nan))
			assert ("NaN+0*i ok", properties.default_create_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.default_create_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.default_create_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.default_create_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.default_create_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.default_create_ok (Zero))
			assert ("0*i ok", properties.default_create_ok (0*i))
			assert ("-0+0*i ok", properties.default_create_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.default_create_ok (0 + 0*i))
			assert ("a ok", properties.default_create_ok (some_real))
			assert ("a+0*i ok", properties.default_create_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.default_create_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.default_create_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.default_create_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.default_create_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.default_create_ok (some_real*i))
			assert ("-0+bi ok", properties.default_create_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.default_create_ok (0 + some_real*i))
			assert ("a+bi ok", properties.default_create_ok (some_real + some_real*i))

			assert ("default_create_ok", properties.default_create_ok (complex_to_be_tested))
		end

	test_make
			-- Test {STI_COMPLEX_NUMBER}.make.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make"
		do
			assert ("make", attached (create {like complex_to_be_tested}.make (some_real, some_real)))
		end

	test_make_from_expanded
			-- Test {STI_COMPLEX_NUMBER}.make_from_expanded.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_expanded"
		do
			assert (
				"native naturals",
				attached (
					create {like complex_to_be_tested}.make_from_expanded
						(some_native_natural, some_native_natural_ref)
					)
				)
			assert (
				"naturals",
				attached (
					create {like complex_to_be_tested}.make_from_expanded
						(some_natural, some_natural)
					)
				)
			assert (
				"native integers",
				attached (
					create {like complex_to_be_tested}.make_from_expanded (
						some_native_integer,
						if attached some_native_integer_ref as nir
							then nir else 0 end
						)
					)
				)
			assert (
				"integers",
				attached (
					create {like complex_to_be_tested}.make_from_expanded
						(some_integer, some_integer)
					)
				)
			assert (
				"expanded integers",
				attached (
					create {like complex_to_be_tested}.make_from_expanded
						(some_integer, some_integer)
					)
				)
			assert (
				"native reals",
				attached (
					create {like complex_to_be_tested}.make_from_expanded
						(some_native_real, some_native_real_ref)
					)
				)
			assert (
				"reals",
				attached (
					create {like complex_to_be_tested}.make_from_expanded
						(some_real, some_expanded_real)
					)
				)
			assert (
				"native doubles",
				attached (
					create {like complex_to_be_tested}.make_from_expanded
						(some_native_double_real, some_native_double_real_ref)
					)
				)
		end

	test_make_from_complex
			-- Test {STI_COMPLEX_NUMBER}.make_from_complex.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_complex"
		do
			assert ("make_from_complex", attached (create {like complex_to_be_tested}.make_from_complex (some_complex)))
			assert ("convert_from_z", attached complex_from_z (some_complex))
		end

	test_make_from_native_real
			-- Test {STI_COMPLEX_NUMBER}.make_from_native_real.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_native_real"
		do
			assert ("make_from_native_real", attached (create {like complex_to_be_tested}.make_from_native_real (some_native_real)))
			assert ("convert_from_native_real", attached complex_from_native_real (some_native_real))
		end

	test_make_from_native_real_ref
			-- Test {STI_COMPLEX_NUMBER}.make_from_native_real_ref.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_native_real_ref"
		do
			assert ("make_from_native_real_ref", attached (create {like complex_to_be_tested}.make_from_native_real_ref (some_native_real_ref)))
			assert ("convert_from_native_real_ref", attached complex_from_native_real_ref (some_native_real_ref))
		end

	test_make_from_native_double_real
			-- Test {STI_COMPLEX_NUMBER}.make_from_native_double_real.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_native_double_real"
		do
			assert ("make_from_native_double_real", attached (create {like complex_to_be_tested}.make_from_native_double_real (some_native_double_real)))
			assert ("convert_from_native_double_real", attached complex_from_native_double_real (some_native_double_real))
		end

	test_make_from_native_double_real_ref
			-- Test {STI_COMPLEX_NUMBER}.make_from_native_double_real_ref.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_native_double_real_ref"
		do
			assert ("make_from_native_double_real_ref", attached (create {like complex_to_be_tested}.make_from_native_double_real_ref (some_native_double_real_ref)))
			assert ("convert_from_native_double_real_ref", attached complex_from_native_double_real_ref (some_native_double_real_ref))
		end

	test_make_from_real
			-- Test {STI_COMPLEX_NUMBER}.make_from_real.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_real"
		do
			assert ("make_from_real", attached (create {like complex_to_be_tested}.make_from_real (some_real)))
			assert ("convert_from_real", attached complex_from_real (some_real))
		end

	test_make_from_native_integer
			-- Test {STI_COMPLEX_NUMBER}.make_from_native_integer.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_native_integer"
		do
			assert ("make_from_native_integer", attached (create {like complex_to_be_tested}.make_from_native_integer (some_native_integer)))
			assert ("convert_from_native_integer", attached complex_from_native_integer (some_native_integer))
		end

	test_make_from_native_integer_ref
			-- Test {STI_COMPLEX_NUMBER}.make_from_native_integer_ref.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_native_integer_ref"
		local
			i_ref: INTEGER_REF
		do
			from
				i_ref := some_native_integer_ref
			until
				attached i_ref
			loop
				i_ref := some_native_integer_ref
			end
			assert ("make_from_native_integer_ref", attached (create {like complex_to_be_tested}.make_from_native_integer_ref (i_ref)))
			assert ("convert_from_native_integer_ref", attached complex_from_native_integer_ref (i_ref))
		end

	test_make_from_integer
			-- Test {STI_COMPLEX_NUMBER}.make_from_integer.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_integer"
		do
			assert ("make_from_integer", attached (create {like complex_to_be_tested}.make_from_integer (some_integer)))
			assert ("convert_from_integer", attached complex_from_integer (some_integer))
		end

	test_make_from_native_natural
			-- Test {STI_COMPLEX_NUMBER}.make_from_native_natural.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_native_natural"
		do
			assert ("make_from_native_natural", attached (create {like complex_to_be_tested}.make_from_native_natural (some_native_natural)))
			assert ("convert_from_native_natural", attached complex_from_native_natural (some_native_natural))
		end

	test_make_from_native_natural_ref
			-- Test {STI_COMPLEX_NUMBER}.make_from_native_natural_ref.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_native_natural_ref"
		do
			assert ("make_from_native_natural_ref", attached (create {like complex_to_be_tested}.make_from_native_natural_ref (some_native_natural_ref)))
			assert ("convert_from_native_natural_ref", attached complex_from_native_natural_ref (some_native_natural_ref))
		end

	test_make_from_ordered_pair
			-- Test {STI_COMPLEX_NUMBER}.make_from_ordered_pair.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.make_from_ordered_pair"
		do
			assert (
				"make_from_ordered_pair",
				attached (
					create {like complex_to_be_tested}.make_from_ordered_pair (
						create {STI_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]}.make (some_real, some_real)
						)
					)
				)
			assert (
				"convert_from_sts_pair",
				attached complex_from_sts_pair
					(create {STI_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]}.make (some_real, some_real))
				)
			assert (
				"convert_from_numeric_sti_pair",
				attached complex_from_numeric_sti_pair (
					create {
						STI_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER], STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]
						}.make (some_real, some_real)
					)
				)
			assert (
				"convert_from_real_sti_pair",
				attached complex_from_real_sti_pair
					(create {STI_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]}.make (some_real, some_real))
				)
			assert (
				"convert_from_expanded_pair",
				attached complex_from_expanded_pair (
					create {
						STI_ORDERED_PAIR [STI_REAL_NUMBER, STI_REAL_NUMBER, STS_NUMERIC_EQUALITY [STI_REAL_NUMBER], STS_NUMERIC_EQUALITY [STI_REAL_NUMBER]]
						}.make (some_expanded_real, some_expanded_real)
					)
				)
			assert (
				"convert_from_from_expanded_sti_pair",
				attached complex_from_expanded_sti_pair (
					create {
						STI_ORDERED_PAIR [STI_REAL_NUMBER, STI_REAL_NUMBER, STS_NUMERIC_EQUALITY [STI_REAL_NUMBER], STS_NUMERIC_EQUALITY [STI_REAL_NUMBER]]
						}.make (some_real, some_real)
					)
				)
			assert (
				"convert_from_homogeneous_pair",
				attached complex_from_homogeneous_pair (create {STI_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]}.make (some_real, some_real))
				)
			assert (
				"convert_from_numeric_sti_homogeneous_pair",
				attached complex_from_numeric_sti_homogeneous_pair
					(create {STI_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]}.make (some_real, some_real))
				)
			assert (
				"convert_from_real_sti_homogeneous_pair",
				attached complex_from_real_sti_homogeneous_pair (create {STI_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_EQUALITY]}.make (some_real, some_real))
				)
			assert (
				"convert_from_expanded_homogeneous_pair",
				attached complex_from_expanded_homogeneous_pair
					(create {STI_HOMOGENEOUS_ORDERED_PAIR [STI_REAL_NUMBER, STS_NUMERIC_EQUALITY [STI_REAL_NUMBER]]}.make (some_expanded_real, some_expanded_real))
				)
			assert (
				"convert_from_expanded_sti_homogeneous_pair",
				attached complex_from_expanded_sti_homogeneous_pair
					(create {STI_HOMOGENEOUS_ORDERED_PAIR [STI_REAL_NUMBER, STS_NUMERIC_EQUALITY [STI_REAL_NUMBER]]}.make (some_expanded_real, some_expanded_real))
				)
		end

feature -- Test routines (Primitive)

	test_a
			-- Test {STI_COMPLEX_NUMBER}.a.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.a"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("a", attached z.a)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.a_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.a_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.a_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.a_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.a_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.a_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.a_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.a_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.a_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.a_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.a_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.a_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.a_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.a_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.a_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.a_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.a_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.a_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.a_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.a_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.a_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.a_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.a_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.a_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.a_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.a_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.a_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.a_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.a_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.a_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.a_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.a_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.a_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.a_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.a_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.a_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.a_ok (-0.0*i))
			assert ("-0-0*i ok", properties.a_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.a_ok (0 - 0*i))
			assert ("a-0*i ok", properties.a_ok (some_real - 0*i))

			assert ("-NaN ok", properties.a_ok (-Nan))
			assert ("-NaN+0*i ok", properties.a_ok (-Nan + 0*i))
			assert ("NaN ok", properties.a_ok (Nan))
			assert ("NaN+0*i ok", properties.a_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.a_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.a_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.a_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.a_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.a_ok (Zero))
			assert ("0*i ok", properties.a_ok (0*i))
			assert ("-0+0*i ok", properties.a_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.a_ok (0 + 0*i))
			assert ("a ok", properties.a_ok (some_real))
			assert ("a+0*i ok", properties.a_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.a_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.a_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.a_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.a_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.a_ok (some_real*i))
			assert ("-0+bi ok", properties.a_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.a_ok (0 + some_real*i))
			assert ("a+bi ok", properties.a_ok (some_real + some_real*i))

			assert ("a_ok", properties.a_ok (complex_to_be_tested))
		end

	test_real
			-- Test {STI_COMPLEX_NUMBER}.real.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.real"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("real", attached z.real)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.real_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.real_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.real_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.real_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.real_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.real_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.real_ok (0 - Nan*i))
			assert ("real-NaN*i ok", properties.real_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.real_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.real_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.real_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.real_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.real_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.real_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.real_ok (0 + Nan*i))
			assert ("real+NaN*i ok", properties.real_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.real_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.real_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.real_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.real_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.real_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.real_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.real_ok (0 + Negative_infinity*i))
			assert ("real-Infinity*i ok", properties.real_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.real_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.real_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.real_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.real_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.real_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.real_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.real_ok (0 + Positive_infinity*i))
			assert ("real+Infinity*i ok", properties.real_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.real_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.real_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.real_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.real_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.real_ok (-0.0*i))
			assert ("-0-0*i ok", properties.real_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.real_ok (0 - 0*i))
			assert ("real-0*i ok", properties.real_ok (some_real - 0*i))

			assert ("-NaN ok", properties.real_ok (-Nan))
			assert ("-NaN+0*i ok", properties.real_ok (-Nan + 0*i))
			assert ("NaN ok", properties.real_ok (Nan))
			assert ("NaN+0*i ok", properties.real_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.real_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.real_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.real_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.real_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.real_ok (Zero))
			assert ("0*i ok", properties.real_ok (0*i))
			assert ("-0+0*i ok", properties.real_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.real_ok (0 + 0*i))
			assert ("real ok", properties.real_ok (some_real))
			assert ("real+0*i ok", properties.real_ok (some_real + 0*i))

			assert ("-NaN+imaginary*i ok", properties.real_ok (-Nan + some_real*i))
			assert ("NaN+imaginary*i ok", properties.real_ok (Nan + some_real*i))
			assert ("-Infinity+imaginary*i ok", properties.real_ok (Negative_infinity + some_real*i))
			assert ("Infinity+imaginary*i ok", properties.real_ok (Positive_infinity + some_real*i))
			assert ("+imaginary*i ok", properties.real_ok (some_real*i))
			assert ("-0+imaginary*i ok", properties.real_ok (-0.0 + some_real*i))
			assert ("0+imaginary*i ok", properties.real_ok (0 + some_real*i))
			assert ("real+imaginary*i ok", properties.real_ok (some_real + some_real*i))

			assert ("real_ok", properties.real_ok (complex_to_be_tested))
		end

	test_b
			-- Test {STI_COMPLEX_NUMBER}.b.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.b"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("b", attached z.b)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.b_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.b_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.b_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.b_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.b_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.b_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.b_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.b_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.b_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.b_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.b_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.b_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.b_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.b_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.b_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.b_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.b_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.b_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.b_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.b_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.b_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.b_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.b_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.b_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.b_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.b_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.b_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.b_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.b_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.b_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.b_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.b_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.b_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.b_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.b_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.b_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.b_ok (-0.0*i))
			assert ("-0-0*i ok", properties.b_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.b_ok (0 - 0*i))
			assert ("a-0*i ok", properties.b_ok (some_real - 0*i))

			assert ("-NaN ok", properties.b_ok (-Nan))
			assert ("-NaN+0*i ok", properties.b_ok (-Nan + 0*i))
			assert ("NaN ok", properties.b_ok (Nan))
			assert ("NaN+0*i ok", properties.b_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.b_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.b_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.b_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.b_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.b_ok (Zero))
			assert ("0*i ok", properties.b_ok (0*i))
			assert ("-0+0*i ok", properties.b_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.b_ok (0 + 0*i))
			assert ("a ok", properties.b_ok (some_real))
			assert ("a+0*i ok", properties.b_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.b_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.b_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.b_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.b_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.b_ok (some_real*i))
			assert ("-0+bi ok", properties.b_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.b_ok (0 + some_real*i))
			assert ("a+bi ok", properties.b_ok (some_real + some_real*i))

			assert ("b_ok", properties.b_ok (complex_to_be_tested))
		end

	test_imaginary
			-- Test {STI_COMPLEX_NUMBER}.imaginary.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.imaginary"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("imaginary", attached z.imaginary)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.imaginary_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.imaginary_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.imaginary_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.imaginary_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.imaginary_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.imaginary_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.imaginary_ok (0 - Nan*i))
			assert ("real-NaN*i ok", properties.imaginary_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.imaginary_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.imaginary_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.imaginary_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.imaginary_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.imaginary_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.imaginary_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.imaginary_ok (0 + Nan*i))
			assert ("real+NaN*i ok", properties.imaginary_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.imaginary_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.imaginary_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.imaginary_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.imaginary_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.imaginary_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.imaginary_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.imaginary_ok (0 + Negative_infinity*i))
			assert ("real-Infinity*i ok", properties.imaginary_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.imaginary_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.imaginary_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.imaginary_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.imaginary_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.imaginary_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.imaginary_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.imaginary_ok (0 + Positive_infinity*i))
			assert ("real+Infinity*i ok", properties.imaginary_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.imaginary_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.imaginary_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.imaginary_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.imaginary_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.imaginary_ok (-0.0*i))
			assert ("-0-0*i ok", properties.imaginary_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.imaginary_ok (0 - 0*i))
			assert ("real-0*i ok", properties.imaginary_ok (some_real - 0*i))

			assert ("-NaN ok", properties.imaginary_ok (-Nan))
			assert ("-NaN+0*i ok", properties.imaginary_ok (-Nan + 0*i))
			assert ("NaN ok", properties.imaginary_ok (Nan))
			assert ("NaN+0*i ok", properties.imaginary_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.imaginary_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.imaginary_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.imaginary_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.imaginary_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.imaginary_ok (Zero))
			assert ("0*i ok", properties.imaginary_ok (0*i))
			assert ("-0+0*i ok", properties.imaginary_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.imaginary_ok (0 + 0*i))
			assert ("real ok", properties.imaginary_ok (some_real))
			assert ("real+0*i ok", properties.imaginary_ok (some_real + 0*i))

			assert ("-NaN+imaginary*i ok", properties.imaginary_ok (-Nan + some_real*i))
			assert ("NaN+imaginary*i ok", properties.imaginary_ok (Nan + some_real*i))
			assert ("-Infinity+imaginary*i ok", properties.imaginary_ok (Negative_infinity + some_real*i))
			assert ("Infinity+imaginary*i ok", properties.imaginary_ok (Positive_infinity + some_real*i))
			assert ("+imaginary*i ok", properties.imaginary_ok (some_real*i))
			assert ("-0+imaginary*i ok", properties.imaginary_ok (-0.0 + some_real*i))
			assert ("0+imaginary*i ok", properties.imaginary_ok (0 + some_real*i))
			assert ("real+imaginary*i ok", properties.imaginary_ok (some_real + some_real*i))

			assert ("imaginary_ok", properties.imaginary_ok (complex_to_be_tested))
		end

	test_eq
			-- Test {STI_COMPLEX_NUMBER}.eq.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.eq"
		do
			assert ("eq", attached complex_to_be_tested.Eq)
		end

feature -- Test routines (Membership)

	test_element_is_in
			-- <Precursor>
			-- Test {STS_COMPLEX_NUMBER}.element_is_in.
			-- Test {STI_COMPLEX_NUMBER}.element_is_in.
		note
			testing: "covers/{STS_ELEMENT}.is_in"
			testing: "covers/{STS_COMPLEX_NUMBER}.element_is_in"
			testing: "covers/{STI_COMPLEX_NUMBER}.element_is_in"
		do
			Precursor {ELEMENT_TESTS}
		end

	test_element_is_not_in
			-- <Precursor>
			-- Test {STS_COMPLEX_NUMBER}.element_is_not_in.
			-- Test {STI_COMPLEX_NUMBER}.element_is_not_in.
		note
			testing: "covers/{STS_ELEMENT}.is_not_in"
			testing: "covers/{STS_COMPLEX_NUMBER}.element_is_not_in"
			testing: "covers/{STI_COMPLEX_NUMBER}.element_is_not_in"
		do
			Precursor {ELEMENT_TESTS}
		end

	test_pair_is_in
			-- Test {STS_ORDERED_PAIR}.is_in.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.is_in.
			-- Test {STI_COMPLEX_NUMBER}.pair_is_in.
		note
			testing: "covers/{STS_ORDERED_PAIR}.is_in"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.is_in"
			testing: "covers/{STI_COMPLEX_NUMBER}.pair_is_in"
		local
			z: like complex_to_be_tested
			abs: like resources_rr.some_set_ab
			chk_ops: BOOLEAN
		do
			z := complex_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_rr.some_references_ab & z
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_reference", z.pair_is_in (abs))
			assert ("same_reference_ok", homogeneous_pair_properties.is_in_ok (z, abs))

			z := complex_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_rr.some_objects_ab & z.twin
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_value", z.pair_is_in (abs))
			assert ("same_value_ok", homogeneous_pair_properties.is_in_ok (z, abs))

			z := complex_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_rr.some_math_set_ab & resources_rr.same_pair_ab (z)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_math_value", z.pair_is_in (abs))
			assert ("same_math_value_ok", homogeneous_pair_properties.is_in_ok (z, abs))

			z := complex_to_be_tested
			abs := resources_rr.some_set_ab
			assert ("some_pairs", z.pair_is_in (abs) implies True)
			assert ("is_in_ok", homogeneous_pair_properties.is_in_ok (z, abs))
		end

	test_pair_is_not_in
			-- Test {STS_ORDERED_PAIR}.is_not_in.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.is_not_in.
			-- Test {STI_COMPLEX_NUMBER}.pair_is_not_in.
		note
			testing: "covers/{STS_ORDERED_PAIR}.is_not_in"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.is_not_in"
			testing: "covers/{STI_COMPLEX_NUMBER}.pair_is_not_in"
		local
			z: like complex_to_be_tested
			abs: like resources_rr.some_set_ab
			chk_ops: BOOLEAN
		do
			z := complex_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_rr.some_references_ab / z
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_reference", z.pair_is_not_in (abs))
			assert ("same_reference_ok", homogeneous_pair_properties.is_not_in_ok (z, abs))

			z := complex_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_rr.some_objects_ab / z.twin
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_value", z.pair_is_not_in (abs))
			assert ("same_value_ok", homogeneous_pair_properties.is_not_in_ok (z, abs))

			z := complex_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_rr.some_math_set_ab / resources_rr.same_pair_ab (z)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_math_value", z.pair_is_not_in (abs))
			assert ("same_math_value_ok", homogeneous_pair_properties.is_not_in_ok (z, abs))

			z := complex_to_be_tested
			abs := resources_rr.some_set_ab
			assert ("some_pairs", z.pair_is_not_in (abs) implies True)
			assert ("is_not_in_ok", homogeneous_pair_properties.is_not_in_ok (z, abs))
		end

	test_is_in
			-- Test {STS_COMPLEX_NUMBER}.is_in.
			-- Test {STI_COMPLEX_NUMBER}.is_in.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_in"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_in"
		local
			z: like complex_to_be_tested
			s: like some_complexes
		do
			z := complex_to_be_tested
			s := some_complex_references & z
			assert ("same_reference", z ∈ s)
			assert ("same_reference_ok", properties.is_in_ok (z, s))

			z := complex_to_be_tested
			s := some_complex_objects & z.twin
			assert ("same_value", z ∈ s)
			assert ("same_value_ok", properties.is_in_ok (z, s))

			z := complex_to_be_tested
			s := some_complex_numbers & same_complex (z)
			assert ("same_math_value", z ∈ s)
			assert ("same_math_value_ok", properties.is_in_ok (z, s))

			z := complex_to_be_tested
			s := some_strict_complex_numbers & same_complex (z)
			assert ("same_strict_math_value", z ∈ s)
			assert ("same_strict_math_value_ok", properties.is_in_ok (z, s))

			z := complex_to_be_tested
			s := some_complexes
			assert ("some_complexes", z ∈ s implies True)
			assert ("some_complexes_ok", properties.is_in_ok (z, s))
		end

	test_is_not_in
			-- Test {STS_COMPLEX_NUMBER}.is_not_in.
			-- Test {STI_COMPLEX_NUMBER}.is_not_in.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_not_in"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_not_in"
		local
			z: like complex_to_be_tested
			s: like some_complexes
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			z := complex_to_be_tested
			s := some_complex_references / z
			assert ("same_reference", z ∉ s)
			assert ("same_reference ok", Properties.is_not_in_ok (z, s))

			z := complex_to_be_tested
			s := some_complex_objects / z.twin
			assert ("same_value", z ∉ s)
			assert ("same_value", Properties.is_not_in_ok (z, s))

			z := complex_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_complex_numbers / same_complex (z)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_math_value", z ∉ s)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("same_math_value", Properties.is_not_in_ok (z, s))

			z := complex_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_strict_complex_numbers / same_complex (z)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_strict_math_value", z ∉ s)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("same_strict_math_value", Properties.is_not_in_ok (z, s))

			z := complex_to_be_tested
			s := some_complexes
			assert ("some_complexes", z ∉ s implies True)
			assert ("some_complexes_ok", Properties.is_not_in_ok (z, s))
		end

feature -- Test routines (Access)

	test_zero
			-- Test {STI_COMPLEX_NUMBER}.Zero.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.zero"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("Zero", attached z.Zero)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.zero_ok (-Nan - Nan*i, some_complex))
			assert ("NaN-NaN*i ok", properties.zero_ok (Nan - Nan*i, some_complex))
			assert ("-Infinity-NaN*i ok", properties.zero_ok (Negative_infinity - Nan*i, some_complex))
			assert ("Infinity-NaN*i ok", properties.zero_ok (Positive_infinity - Nan*i, some_complex))
			assert ("-NaN*i ok", properties.zero_ok (-Nan*i, some_complex))
			assert ("-0-NaN*i ok", properties.zero_ok (-0.0 - Nan*i, some_complex))
			assert ("0-NaN*i ok", properties.zero_ok (0 - Nan*i, some_complex))
			assert ("a-NaN*i ok", properties.zero_ok (some_real - Nan*i, some_complex))

			assert ("-NaN+NaN*i ok", properties.zero_ok (-Nan + Nan*i, some_complex))
			assert ("NaN+NaN*i ok", properties.zero_ok (Nan + Nan*i, some_complex))
			assert ("-Infinity+NaN*i ok", properties.zero_ok (Negative_infinity + Nan*i, some_complex))
			assert ("Infinity+NaN*i ok", properties.zero_ok (Positive_infinity + Nan*i, some_complex))
			assert ("NaN*i ok", properties.zero_ok (Nan*i, some_complex))
			assert ("-0+NaN*i ok", properties.zero_ok (-0.0 + Nan*i, some_complex))
			assert ("0+NaN*i ok", properties.zero_ok (0 + Nan*i, some_complex))
			assert ("a+NaN*i ok", properties.zero_ok (some_real + Nan*i, some_complex))

			assert ("-NaN-Infinity*i ok", properties.zero_ok (-Nan + Negative_infinity*i, some_complex))
			assert ("NaN-Infinity*i ok", properties.zero_ok (Nan + Negative_infinity*i, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.zero_ok (Negative_infinity + Negative_infinity*i, some_complex))
			assert ("Infinity-Infinity*i ok", properties.zero_ok (Positive_infinity + Negative_infinity*i, some_complex))
			assert ("-Infinity*i ok", properties.zero_ok (Negative_infinity*i, some_complex))
			assert ("-0-Infinity*i ok", properties.zero_ok (-0.0 + Negative_infinity*i, some_complex))
			assert ("0-Infinity*i ok", properties.zero_ok (0 + Negative_infinity*i, some_complex))
			assert ("a-Infinity*i ok", properties.zero_ok (some_real + Negative_infinity*i, some_complex))

			assert ("-NaN+Infinity*i ok", properties.zero_ok (-Nan + Positive_infinity*i, some_complex))
			assert ("NaN+Infinity*i ok", properties.zero_ok (Nan + Positive_infinity*i, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.zero_ok (Negative_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity+Infinity*i ok", properties.zero_ok (Positive_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity*i ok", properties.zero_ok (Positive_infinity*i, some_complex))
			assert ("-0+Infinity*i ok", properties.zero_ok (-0.0 + Positive_infinity*i, some_complex))
			assert ("0+Infinity*i ok", properties.zero_ok (0 + Positive_infinity*i, some_complex))
			assert ("a+Infinity*i ok", properties.zero_ok (some_real + Positive_infinity*i, some_complex))

			assert ("-NaN-0*i ok", properties.zero_ok (-Nan - 0*i, some_complex))
			assert ("NaN-0*i ok", properties.zero_ok (Nan - 0*i, some_complex))
			assert ("-Infinity-0*i ok", properties.zero_ok (Negative_infinity - 0*i, some_complex))
			assert ("Infinity-0*i ok", properties.zero_ok (Positive_infinity - 0*i, some_complex))
			assert ("-0*i ok", properties.zero_ok (-0.0*i, some_complex))
			assert ("-0-0*i ok", properties.zero_ok (-0.0 - 0*i, some_complex))
			assert ("0-0*i ok", properties.zero_ok (0 - 0*i, some_complex))
			assert ("a-0*i ok", properties.zero_ok (some_real - 0*i, some_complex))

			assert ("-NaN ok", properties.zero_ok (-Nan, some_complex))
			assert ("-NaN+0*i ok", properties.zero_ok (-Nan + 0*i, some_complex))
			assert ("NaN ok", properties.zero_ok (Nan, some_complex))
			assert ("NaN+0*i ok", properties.zero_ok (Nan + 0*i, some_complex))
			assert ("-Infinity ok", properties.zero_ok (Negative_infinity, some_complex))
			assert ("-Infinity+0*i ok", properties.zero_ok (Negative_infinity + 0*i, some_complex))
			assert ("Infinity ok", properties.zero_ok (Positive_infinity, some_complex))
			assert ("Infinity+0*i ok", properties.zero_ok (Positive_infinity + 0*i, some_complex))
			assert ("0 ok", properties.zero_ok (Zero, some_complex))
			assert ("0*i ok", properties.zero_ok (0*i, some_complex))
			assert ("-0+0*i ok", properties.zero_ok (-0.0 + 0*i, some_complex))
			assert ("0+0*i ok", properties.zero_ok (0 + 0*i, some_complex))
			assert ("a ok", properties.zero_ok (some_real, some_complex))
			assert ("a+0*i ok", properties.zero_ok (some_real + 0*i, some_complex))

			assert ("-NaN+bi ok", properties.zero_ok (-Nan + some_real*i, some_complex))
			assert ("NaN+bi ok", properties.zero_ok (Nan + some_real*i, some_complex))
			assert ("-Infinity+bi ok", properties.zero_ok (Negative_infinity + some_real*i, some_complex))
			assert ("Infinity+bi ok", properties.zero_ok (Positive_infinity + some_real*i, some_complex))
			assert ("bi ok", properties.zero_ok (some_real*i, some_complex))
			assert ("-0+bi ok", properties.zero_ok (-0.0 + some_real*i, some_complex))
			assert ("0+bi ok", properties.zero_ok (0 + some_real*i, some_complex))
			assert ("a+bi ok", properties.zero_ok (some_real + some_real*i, some_complex))

			assert ("zero_ok", properties.zero_ok (complex_to_be_tested, some_complex))
		end

	test_one
			-- Test {STI_COMPLEX_NUMBER}.One.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.one"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("One", attached z.One)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.one_ok (-Nan - Nan*i, some_complex))
			assert ("NaN-NaN*i ok", properties.one_ok (Nan - Nan*i, some_complex))
			assert ("-Infinity-NaN*i ok", properties.one_ok (Negative_infinity - Nan*i, some_complex))
			assert ("Infinity-NaN*i ok", properties.one_ok (Positive_infinity - Nan*i, some_complex))
			assert ("-NaN*i ok", properties.one_ok (-Nan*i, some_complex))
			assert ("-0-NaN*i ok", properties.one_ok (-0.0 - Nan*i, some_complex))
			assert ("0-NaN*i ok", properties.one_ok (0 - Nan*i, some_complex))
			assert ("a-NaN*i ok", properties.one_ok (some_real - Nan*i, some_complex))

			assert ("-NaN+NaN*i ok", properties.one_ok (-Nan + Nan*i, some_complex))
			assert ("NaN+NaN*i ok", properties.one_ok (Nan + Nan*i, some_complex))
			assert ("-Infinity+NaN*i ok", properties.one_ok (Negative_infinity + Nan*i, some_complex))
			assert ("Infinity+NaN*i ok", properties.one_ok (Positive_infinity + Nan*i, some_complex))
			assert ("NaN*i ok", properties.one_ok (Nan*i, some_complex))
			assert ("-0+NaN*i ok", properties.one_ok (-0.0 + Nan*i, some_complex))
			assert ("0+NaN*i ok", properties.one_ok (0 + Nan*i, some_complex))
			assert ("a+NaN*i ok", properties.one_ok (some_real + Nan*i, some_complex))

			assert ("-NaN-Infinity*i ok", properties.one_ok (-Nan + Negative_infinity*i, some_complex))
			assert ("NaN-Infinity*i ok", properties.one_ok (Nan + Negative_infinity*i, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.one_ok (Negative_infinity + Negative_infinity*i, some_complex))
			assert ("Infinity-Infinity*i ok", properties.one_ok (Positive_infinity + Negative_infinity*i, some_complex))
			assert ("-Infinity*i ok", properties.one_ok (Negative_infinity*i, some_complex))
			assert ("-0-Infinity*i ok", properties.one_ok (-0.0 + Negative_infinity*i, some_complex))
			assert ("0-Infinity*i ok", properties.one_ok (0 + Negative_infinity*i, some_complex))
			assert ("a-Infinity*i ok", properties.one_ok (some_real + Negative_infinity*i, some_complex))

			assert ("-NaN+Infinity*i ok", properties.one_ok (-Nan + Positive_infinity*i, some_complex))
			assert ("NaN+Infinity*i ok", properties.one_ok (Nan + Positive_infinity*i, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.one_ok (Negative_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity+Infinity*i ok", properties.one_ok (Positive_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity*i ok", properties.one_ok (Positive_infinity*i, some_complex))
			assert ("-0+Infinity*i ok", properties.one_ok (-0.0 + Positive_infinity*i, some_complex))
			assert ("0+Infinity*i ok", properties.one_ok (0 + Positive_infinity*i, some_complex))
			assert ("a+Infinity*i ok", properties.one_ok (some_real + Positive_infinity*i, some_complex))

			assert ("-NaN-0*i ok", properties.one_ok (-Nan - 0*i, some_complex))
			assert ("NaN-0*i ok", properties.one_ok (Nan - 0*i, some_complex))
			assert ("-Infinity-0*i ok", properties.one_ok (Negative_infinity - 0*i, some_complex))
			assert ("Infinity-0*i ok", properties.one_ok (Positive_infinity - 0*i, some_complex))
			assert ("-0*i ok", properties.one_ok (-0.0*i, some_complex))
			assert ("-0-0*i ok", properties.one_ok (-0.0 - 0*i, some_complex))
			assert ("0-0*i ok", properties.one_ok (0 - 0*i, some_complex))
			assert ("a-0*i ok", properties.one_ok (some_real - 0*i, some_complex))

			assert ("-NaN ok", properties.one_ok (-Nan, some_complex))
			assert ("-NaN+0*i ok", properties.one_ok (-Nan + 0*i, some_complex))
			assert ("NaN ok", properties.one_ok (Nan, some_complex))
			assert ("NaN+0*i ok", properties.one_ok (Nan + 0*i, some_complex))
			assert ("-Infinity ok", properties.one_ok (Negative_infinity, some_complex))
			assert ("-Infinity+0*i ok", properties.one_ok (Negative_infinity + 0*i, some_complex))
			assert ("Infinity ok", properties.one_ok (Positive_infinity, some_complex))
			assert ("Infinity+0*i ok", properties.one_ok (Positive_infinity + 0*i, some_complex))
			assert ("0 ok", properties.one_ok (Zero, some_complex))
			assert ("0*i ok", properties.one_ok (0*i, some_complex))
			assert ("-0+0*i ok", properties.one_ok (-0.0 + 0*i, some_complex))
			assert ("0+0*i ok", properties.one_ok (0 + 0*i, some_complex))
			assert ("a ok", properties.one_ok (some_real, some_complex))
			assert ("a+0*i ok", properties.one_ok (some_real + 0*i, some_complex))

			assert ("-NaN+bi ok", properties.one_ok (-Nan + some_real*i, some_complex))
			assert ("NaN+bi ok", properties.one_ok (Nan + some_real*i, some_complex))
			assert ("-Infinity+bi ok", properties.one_ok (Negative_infinity + some_real*i, some_complex))
			assert ("Infinity+bi ok", properties.one_ok (Positive_infinity + some_real*i, some_complex))
			assert ("bi ok", properties.one_ok (some_real*i, some_complex))
			assert ("-0+bi ok", properties.one_ok (-0.0 + some_real*i, some_complex))
			assert ("0+bi ok", properties.one_ok (0 + some_real*i, some_complex))
			assert ("a+bi ok", properties.one_ok (some_real + some_real*i, some_complex))

			assert ("one_ok", properties.one_ok (complex_to_be_tested, some_complex))
		end

	test_two
			-- Test {STI_COMPLEX_NUMBER}.two.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.two"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("two", attached z.two)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.two_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.two_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.two_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.two_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.two_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.two_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.two_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.two_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.two_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.two_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.two_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.two_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.two_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.two_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.two_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.two_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.two_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.two_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.two_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.two_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.two_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.two_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.two_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.two_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.two_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.two_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.two_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.two_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.two_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.two_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.two_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.two_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.two_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.two_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.two_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.two_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.two_ok (-0.0*i))
			assert ("-0-0*i ok", properties.two_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.two_ok (0 - 0*i))
			assert ("a-0*i ok", properties.two_ok (some_real - 0*i))

			assert ("-NaN ok", properties.two_ok (-Nan))
			assert ("-NaN+0*i ok", properties.two_ok (-Nan + 0*i))
			assert ("NaN ok", properties.two_ok (Nan))
			assert ("NaN+0*i ok", properties.two_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.two_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.two_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.two_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.two_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.two_ok (Zero))
			assert ("0*i ok", properties.two_ok (0*i))
			assert ("-0+0*i ok", properties.two_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.two_ok (0 + 0*i))
			assert ("a ok", properties.two_ok (some_real))
			assert ("a+0*i ok", properties.two_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.two_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.two_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.two_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.two_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.two_ok (some_real*i))
			assert ("-0+bi ok", properties.two_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.two_ok (0 + some_real*i))
			assert ("a+bi ok", properties.two_ok (some_real + some_real*i))

			assert ("two_ok", properties.two_ok (complex_to_be_tested))
		end

	test_i
			-- Test {STI_COMPLEX_NUMBER}.i.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.i"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			xs: like R.subset_anchor
			x: like R.Any_anchor
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("i", attached z.i)
				assert ("i_ok", properties.i_ok (z))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

				-- The outsiders...
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				create z.make_from_expanded (x, -Nan)
				assert ("x - Nan*i", attached z.i)
				assert ("x - Nan*i ok", properties.i_ok (z))

				create z.make_from_expanded (-Nan, x)
				assert ("-Nan + xi", attached z.i)
				assert ("-Nan + xi ok", properties.i_ok (z))

				create z.make_from_expanded (x, -0.0)
				assert ("x - 0i", attached z.i)
				assert ("x - 0i ok", properties.i_ok (z))

				create z.make_from_expanded (-0.0, x)
				assert ("-0 + xi", attached z.i)
				assert ("-0 + xi ok", properties.i_ok (z))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			create z.make_from_expanded (-Nan, -Nan)
			assert ("-Nan - Nan*i", attached z.i)
			assert ("-Nan - Nan*i ok", properties.i_ok (z))

			create z.make_from_expanded (-Nan, -0.0)
			assert ("-Nan - 0i", attached z.i)
			assert ("-Nan - 0i ok", properties.i_ok (z))

			create z.make_from_expanded (-0.0, -Nan)
			assert ("-0 - Nan*i", attached z.i)
			assert ("-0 - Nan*i ok", properties.i_ok (z))

			create z.make_from_expanded (-0.0, -0.0)
			assert ("-0 - 0i", attached z.i)
			assert ("-0 - 0i ok", properties.i_ok (z))
		end

	test_imaginary_unit
			-- Test {STI_COMPLEX_NUMBER}.imaginary_unit.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.imaginary_unit"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			xs: like R.subset_anchor
			x: like R.Any_anchor
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("imaginary_unit", attached z.imaginary_unit)
				assert ("imaginary_unit_ok", properties.imaginary_unit_ok (z))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

				-- The outsiders...
			from
				xs := R
			until
				xs.is_empty
			loop
				x := xs.any
				create z.make_from_expanded (x, -Nan)
				assert ("x - Nan*imaginary_unit", attached z.imaginary_unit)
				assert ("x - Nan*imaginary_unit ok", properties.imaginary_unit_ok (z))

				create z.make_from_expanded (-Nan, x)
				assert ("-Nan + x*imaginary_unit", attached z.imaginary_unit)
				assert ("-Nan + x*imaginary_unit ok", properties.imaginary_unit_ok (z))

				create z.make_from_expanded (x, -0.0)
				assert ("x - 0*imaginary_unit", attached z.imaginary_unit)
				assert ("x - 0*imaginary_unit ok", properties.imaginary_unit_ok (z))

				create z.make_from_expanded (-0.0, x)
				assert ("-0 + x*imaginary_unit", attached z.imaginary_unit)
				assert ("-0 + x*imaginary_unit ok", properties.imaginary_unit_ok (z))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
			create z.make_from_expanded (-Nan, -Nan)
			assert ("-Nan - Nan*imaginary_unit", attached z.imaginary_unit)
			assert ("-Nan - Nan*imaginary_unit ok", properties.imaginary_unit_ok (z))

			create z.make_from_expanded (-Nan, -0.0)
			assert ("-Nan - 0*imaginary_unit", attached z.imaginary_unit)
			assert ("-Nan - 0*imaginary_unit ok", properties.imaginary_unit_ok (z))

			create z.make_from_expanded (-0.0, -Nan)
			assert ("-0 - Nan*imaginary_unit", attached z.imaginary_unit)
			assert ("-0 - Nan*imaginary_unit ok", properties.imaginary_unit_ok (z))

			create z.make_from_expanded (-0.0, -0.0)
			assert ("-0 - 0*imaginary_unit", attached z.imaginary_unit)
			assert ("-0 - 0*imaginary_unit ok", properties.imaginary_unit_ok (z))
		end

feature -- Test routines (Quality)

	test_is_real
			-- Test {STS_COMPLEX_NUMBER}.is_real.
			-- Test {STI_COMPLEX_NUMBER}.is_real.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_real"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_real"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_real", z.is_real implies True)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			z := some_real
			assert ("z.is_real", z.is_real)
			assert ("z.is_real ok", properties.is_real_ok (some_real))

			from
				y := some_real
			until
				y ≭ Zero
			loop
				y := some_real
			end
			z := some_real + y*i
			assert ("not z.is_real", not z.is_real)
			assert ("not z.is_real ok", properties.is_real_ok (z))

			assert ("-NaN-NaN*i ok", properties.is_real_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.is_real_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.is_real_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.is_real_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.is_real_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.is_real_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.is_real_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.is_real_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.is_real_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.is_real_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.is_real_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.is_real_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.is_real_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.is_real_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.is_real_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.is_real_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.is_real_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.is_real_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.is_real_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.is_real_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.is_real_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.is_real_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.is_real_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.is_real_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.is_real_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.is_real_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.is_real_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.is_real_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.is_real_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.is_real_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.is_real_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.is_real_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.is_real_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.is_real_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.is_real_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.is_real_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.is_real_ok (-0.0*i))
			assert ("-0-0*i ok", properties.is_real_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.is_real_ok (0 - 0*i))
			assert ("a-0*i ok", properties.is_real_ok (some_real - 0*i))

			assert ("-NaN ok", properties.is_real_ok (-Nan))
			assert ("-NaN+0*i ok", properties.is_real_ok (-Nan + 0*i))
			assert ("NaN ok", properties.is_real_ok (Nan))
			assert ("NaN+0*i ok", properties.is_real_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.is_real_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.is_real_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.is_real_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.is_real_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.is_real_ok (Zero))
			assert ("0*i ok", properties.is_real_ok (0*i))
			assert ("-0+0*i ok", properties.is_real_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.is_real_ok (0 + 0*i))
			assert ("a ok", properties.is_real_ok (some_real))
			assert ("a+0*i ok", properties.is_real_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.is_real_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.is_real_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.is_real_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.is_real_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.is_real_ok (some_real*i))
			assert ("-0+bi ok", properties.is_real_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.is_real_ok (0 + some_real*i))
			assert ("a+bi ok", properties.is_real_ok (some_real + some_real*i))

			assert ("is_real_ok", properties.is_real_ok (complex_to_be_tested))
		end

	test_is_proper_real
			-- Test {STS_COMPLEX_NUMBER}.is_proper_real.
			-- Test {STI_COMPLEX_NUMBER}.is_proper_real.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_proper_real"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_proper_real"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			x, y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_proper_real", z.is_proper_real implies True)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			from
				x := some_real
			until
				x ≭ Zero
			loop
				x := some_real
			end
			z := x
			assert ("z.is_proper_real", z.is_proper_real)
			assert ("z.is_proper_real ok", properties.is_proper_real_ok (z))

			x := some_real
			from
				y := some_real
			until
				y ≍ Zero implies x ≍ Zero
			loop
				y := some_real
			end
			z := x + y*i
			assert ("not z.is_proper_real", not z.is_proper_real)
			assert ("not z.is_proper_real ok", properties.is_proper_real_ok (z))

			assert ("-NaN-NaN*i ok", properties.is_proper_real_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.is_proper_real_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.is_proper_real_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.is_proper_real_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.is_proper_real_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.is_proper_real_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.is_proper_real_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.is_proper_real_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.is_proper_real_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.is_proper_real_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.is_proper_real_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.is_proper_real_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.is_proper_real_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.is_proper_real_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.is_proper_real_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.is_proper_real_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.is_proper_real_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.is_proper_real_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.is_proper_real_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.is_proper_real_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.is_proper_real_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.is_proper_real_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.is_proper_real_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.is_proper_real_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.is_proper_real_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.is_proper_real_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.is_proper_real_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.is_proper_real_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.is_proper_real_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.is_proper_real_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.is_proper_real_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.is_proper_real_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.is_proper_real_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.is_proper_real_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.is_proper_real_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.is_proper_real_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.is_proper_real_ok (-0.0*i))
			assert ("-0-0*i ok", properties.is_proper_real_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.is_proper_real_ok (0 - 0*i))
			assert ("a-0*i ok", properties.is_proper_real_ok (some_real - 0*i))

			assert ("-NaN ok", properties.is_proper_real_ok (-Nan))
			assert ("-NaN+0*i ok", properties.is_proper_real_ok (-Nan + 0*i))
			assert ("NaN ok", properties.is_proper_real_ok (Nan))
			assert ("NaN+0*i ok", properties.is_proper_real_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.is_proper_real_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.is_proper_real_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.is_proper_real_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.is_proper_real_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.is_proper_real_ok (Zero))
			assert ("0*i ok", properties.is_proper_real_ok (0*i))
			assert ("-0+0*i ok", properties.is_proper_real_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.is_proper_real_ok (0 + 0*i))
			assert ("a ok", properties.is_proper_real_ok (some_real))
			assert ("a+0*i ok", properties.is_proper_real_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.is_proper_real_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.is_proper_real_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.is_proper_real_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.is_proper_real_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.is_proper_real_ok (some_real*i))
			assert ("-0+bi ok", properties.is_proper_real_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.is_proper_real_ok (0 + some_real*i))
			assert ("a+bi ok", properties.is_proper_real_ok (some_real + some_real*i))

			assert ("is_proper_real_ok", properties.is_proper_real_ok (complex_to_be_tested))
		end

	test_is_imaginary
			-- Test {STS_COMPLEX_NUMBER}.is_imaginary.
			-- Test {STI_COMPLEX_NUMBER}.is_imaginary.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_imaginary"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_imaginary"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			x: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_imaginary", z.is_imaginary implies True)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			z := some_real*i
			assert ("z.is_imaginary", z.is_imaginary)
			assert ("z.is_imaginary ok", properties.is_imaginary_ok (z))

			from
				x := some_real
			until
				x ≭ Zero
			loop
				x := some_real
			end
			z := x + some_real*i
			assert ("not z.is_imaginary", not z.is_imaginary)
			assert ("not z.is_imaginary ok", properties.is_imaginary_ok (z))

			assert ("-NaN-NaN*i ok", properties.is_imaginary_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.is_imaginary_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.is_imaginary_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.is_imaginary_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.is_imaginary_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.is_imaginary_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.is_imaginary_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.is_imaginary_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.is_imaginary_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.is_imaginary_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.is_imaginary_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.is_imaginary_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.is_imaginary_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.is_imaginary_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.is_imaginary_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.is_imaginary_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.is_imaginary_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.is_imaginary_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.is_imaginary_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.is_imaginary_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.is_imaginary_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.is_imaginary_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.is_imaginary_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.is_imaginary_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.is_imaginary_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.is_imaginary_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.is_imaginary_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.is_imaginary_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.is_imaginary_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.is_imaginary_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.is_imaginary_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.is_imaginary_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.is_imaginary_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.is_imaginary_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.is_imaginary_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.is_imaginary_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.is_imaginary_ok (-0.0*i))
			assert ("-0-0*i ok", properties.is_imaginary_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.is_imaginary_ok (0 - 0*i))
			assert ("a-0*i ok", properties.is_imaginary_ok (some_real - 0*i))

			assert ("-NaN ok", properties.is_imaginary_ok (-Nan))
			assert ("-NaN+0*i ok", properties.is_imaginary_ok (-Nan + 0*i))
			assert ("NaN ok", properties.is_imaginary_ok (Nan))
			assert ("NaN+0*i ok", properties.is_imaginary_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.is_imaginary_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.is_imaginary_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.is_imaginary_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.is_imaginary_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.is_imaginary_ok (Zero))
			assert ("0*i ok", properties.is_imaginary_ok (0*i))
			assert ("-0+0*i ok", properties.is_imaginary_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.is_imaginary_ok (0 + 0*i))
			assert ("a ok", properties.is_imaginary_ok (some_real))
			assert ("a+0*i ok", properties.is_imaginary_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.is_imaginary_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.is_imaginary_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.is_imaginary_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.is_imaginary_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.is_imaginary_ok (some_real*i))
			assert ("-0+bi ok", properties.is_imaginary_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.is_imaginary_ok (0 + some_real*i))
			assert ("a+bi ok", properties.is_imaginary_ok (some_real + some_real*i))

			assert ("is_imaginary_ok", properties.is_imaginary_ok (complex_to_be_tested))
		end

	test_is_proper_imaginary
			-- Test {STS_COMPLEX_NUMBER}.is_proper_imaginary.
			-- Test {STI_COMPLEX_NUMBER}.is_proper_imaginary.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_proper_imaginary"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_proper_imaginary"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			x, y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_proper_imaginary", z.is_proper_imaginary implies True)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			from
				y := some_real
			until
				y ≭ Zero
			loop
				y := some_real
			end
			z := y*i
			assert ("z.is_proper_imaginary", z.is_proper_imaginary)
			assert ("z.is_proper_imaginary ok", properties.is_proper_imaginary_ok (z))

			y := some_real
			from
				x := some_real
			until
				x ≍ Zero implies y ≍ Zero
			loop
				x := some_real
			end
			z := x + y*i
			assert ("not z.is_proper_imaginary", not z.is_proper_imaginary)
			assert ("not z.is_proper_imaginary ok", properties.is_proper_imaginary_ok (z))

			assert ("-NaN-NaN*i ok", properties.is_proper_imaginary_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.is_proper_imaginary_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.is_proper_imaginary_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.is_proper_imaginary_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.is_proper_imaginary_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.is_proper_imaginary_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.is_proper_imaginary_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.is_proper_imaginary_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.is_proper_imaginary_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.is_proper_imaginary_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.is_proper_imaginary_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.is_proper_imaginary_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.is_proper_imaginary_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.is_proper_imaginary_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.is_proper_imaginary_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.is_proper_imaginary_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.is_proper_imaginary_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.is_proper_imaginary_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.is_proper_imaginary_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.is_proper_imaginary_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.is_proper_imaginary_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.is_proper_imaginary_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.is_proper_imaginary_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.is_proper_imaginary_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.is_proper_imaginary_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.is_proper_imaginary_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.is_proper_imaginary_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.is_proper_imaginary_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.is_proper_imaginary_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.is_proper_imaginary_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.is_proper_imaginary_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.is_proper_imaginary_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.is_proper_imaginary_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.is_proper_imaginary_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.is_proper_imaginary_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.is_proper_imaginary_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.is_proper_imaginary_ok (-0.0*i))
			assert ("-0-0*i ok", properties.is_proper_imaginary_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.is_proper_imaginary_ok (0 - 0*i))
			assert ("a-0*i ok", properties.is_proper_imaginary_ok (some_real - 0*i))

			assert ("-NaN ok", properties.is_proper_imaginary_ok (-Nan))
			assert ("-NaN+0*i ok", properties.is_proper_imaginary_ok (-Nan + 0*i))
			assert ("NaN ok", properties.is_proper_imaginary_ok (Nan))
			assert ("NaN+0*i ok", properties.is_proper_imaginary_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.is_proper_imaginary_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.is_proper_imaginary_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.is_proper_imaginary_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.is_proper_imaginary_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.is_proper_imaginary_ok (Zero))
			assert ("0*i ok", properties.is_proper_imaginary_ok (0*i))
			assert ("-0+0*i ok", properties.is_proper_imaginary_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.is_proper_imaginary_ok (0 + 0*i))
			assert ("a ok", properties.is_proper_imaginary_ok (some_real))
			assert ("a+0*i ok", properties.is_proper_imaginary_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.is_proper_imaginary_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.is_proper_imaginary_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.is_proper_imaginary_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.is_proper_imaginary_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.is_proper_imaginary_ok (some_real*i))
			assert ("-0+bi ok", properties.is_proper_imaginary_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.is_proper_imaginary_ok (0 + some_real*i))
			assert ("a+bi ok", properties.is_proper_imaginary_ok (some_real + some_real*i))

			assert ("is_proper_imaginary_ok", properties.is_proper_imaginary_ok (complex_to_be_tested))
		end

	test_is_proper_complex
			-- Test {STS_COMPLEX_NUMBER}.is_proper_complex.
			-- Test {STI_COMPLEX_NUMBER}.is_proper_complex.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_proper_complex"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_proper_complex"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			x, y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_proper_complex", z.is_proper_complex implies True)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			from
				x := some_real
			until
				x ≭ Zero
			loop
				x := some_real
			end
			from
				y := some_real
			until
				y ≭ Zero
			loop
				y := some_real
			end
			z := x + y*i
			assert ("z.is_proper_complex", z.is_proper_complex)
			assert ("z.is_proper_complex ok", Properties.is_proper_complex_ok (z))
			if next_random_item \\ 2 = 0 then
				z := some_real
			else
				z := some_real*i
			end
			assert ("not z.is_proper_complex", not z.is_proper_complex)
			assert ("not z.is_proper_complex ok", Properties.is_proper_complex_ok (z))

			assert ("-NaN-NaN*i ok", properties.is_proper_complex_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.is_proper_complex_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.is_proper_complex_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.is_proper_complex_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.is_proper_complex_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.is_proper_complex_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.is_proper_complex_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.is_proper_complex_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.is_proper_complex_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.is_proper_complex_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.is_proper_complex_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.is_proper_complex_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.is_proper_complex_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.is_proper_complex_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.is_proper_complex_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.is_proper_complex_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.is_proper_complex_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.is_proper_complex_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.is_proper_complex_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.is_proper_complex_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.is_proper_complex_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.is_proper_complex_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.is_proper_complex_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.is_proper_complex_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.is_proper_complex_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.is_proper_complex_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.is_proper_complex_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.is_proper_complex_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.is_proper_complex_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.is_proper_complex_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.is_proper_complex_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.is_proper_complex_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.is_proper_complex_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.is_proper_complex_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.is_proper_complex_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.is_proper_complex_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.is_proper_complex_ok (-0.0*i))
			assert ("-0-0*i ok", properties.is_proper_complex_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.is_proper_complex_ok (0 - 0*i))
			assert ("a-0*i ok", properties.is_proper_complex_ok (some_real - 0*i))

			assert ("-NaN ok", properties.is_proper_complex_ok (-Nan))
			assert ("-NaN+0*i ok", properties.is_proper_complex_ok (-Nan + 0*i))
			assert ("NaN ok", properties.is_proper_complex_ok (Nan))
			assert ("NaN+0*i ok", properties.is_proper_complex_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.is_proper_complex_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.is_proper_complex_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.is_proper_complex_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.is_proper_complex_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.is_proper_complex_ok (Zero))
			assert ("0*i ok", properties.is_proper_complex_ok (0*i))
			assert ("-0+0*i ok", properties.is_proper_complex_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.is_proper_complex_ok (0 + 0*i))
			assert ("a ok", properties.is_proper_complex_ok (some_real))
			assert ("a+0*i ok", properties.is_proper_complex_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.is_proper_complex_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.is_proper_complex_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.is_proper_complex_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.is_proper_complex_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.is_proper_complex_ok (some_real*i))
			assert ("-0+bi ok", properties.is_proper_complex_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.is_proper_complex_ok (0 + some_real*i))
			assert ("a+bi ok", properties.is_proper_complex_ok (some_real + some_real*i))

			assert ("is_proper_complex_ok", properties.is_proper_complex_ok (complex_to_be_tested))
		end

	test_is_infinite
			-- Test {STS_COMPLEX_NUMBER}.is_infinite.
			-- Test {STI_COMPLEX_NUMBER}.is_infinite.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_infinite"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_infinite"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			x, y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_infinite", z.is_infinite implies True)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			if next_random_item \\ 2 = 0 then
				if next_random_item \\ 2 = 0 then
					x := Negative_infinity
				else
					x := Positive_infinity
				end
				y := some_real
			else
				x := some_real
				if next_random_item \\ 2 = 0 then
					y := Negative_infinity
				else
					y := Positive_infinity
				end
			end
			z := x + y*i
			assert ("z.is_infinite", z.is_infinite)
			assert ("z.is_infinite ok", Properties.is_infinite_ok (z, some_complex))

			from
				x := some_real
			until
				not x.is_infinite
			loop
				x := some_real
			end
			from
				y := some_real
			until
				not y.is_infinite
			loop
				y := some_real
			end
			z := x + y*i
			assert ("not z.is_infinite", not z.is_infinite)
			assert ("not z.is_infinite ok", Properties.is_infinite_ok (z, some_complex))

			assert ("-NaN-NaN*i ok", properties.is_infinite_ok (-Nan - Nan*i, some_complex))
			assert ("NaN-NaN*i ok", properties.is_infinite_ok (Nan - Nan*i, some_complex))
			assert ("-Infinity-NaN*i ok", properties.is_infinite_ok (Negative_infinity - Nan*i, some_complex))
			assert ("Infinity-NaN*i ok", properties.is_infinite_ok (Positive_infinity - Nan*i, some_complex))
			assert ("-NaN*i ok", properties.is_infinite_ok (-Nan*i, some_complex))
			assert ("-0-NaN*i ok", properties.is_infinite_ok (-0.0 - Nan*i, some_complex))
			assert ("0-NaN*i ok", properties.is_infinite_ok (0 - Nan*i, some_complex))
			assert ("a-NaN*i ok", properties.is_infinite_ok (some_real - Nan*i, some_complex))

			assert ("-NaN+NaN*i ok", properties.is_infinite_ok (-Nan + Nan*i, some_complex))
			assert ("NaN+NaN*i ok", properties.is_infinite_ok (Nan + Nan*i, some_complex))
			assert ("-Infinity+NaN*i ok", properties.is_infinite_ok (Negative_infinity + Nan*i, some_complex))
			assert ("Infinity+NaN*i ok", properties.is_infinite_ok (Positive_infinity + Nan*i, some_complex))
			assert ("NaN*i ok", properties.is_infinite_ok (Nan*i, some_complex))
			assert ("-0+NaN*i ok", properties.is_infinite_ok (-0.0 + Nan*i, some_complex))
			assert ("0+NaN*i ok", properties.is_infinite_ok (0 + Nan*i, some_complex))
			assert ("a+NaN*i ok", properties.is_infinite_ok (some_real + Nan*i, some_complex))

			assert ("-NaN-Infinity*i ok", properties.is_infinite_ok (-Nan + Negative_infinity*i, some_complex))
			assert ("NaN-Infinity*i ok", properties.is_infinite_ok (Nan + Negative_infinity*i, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.is_infinite_ok (Negative_infinity + Negative_infinity*i, some_complex))
			assert ("Infinity-Infinity*i ok", properties.is_infinite_ok (Positive_infinity + Negative_infinity*i, some_complex))
			assert ("-Infinity*i ok", properties.is_infinite_ok (Negative_infinity*i, some_complex))
			assert ("-0-Infinity*i ok", properties.is_infinite_ok (-0.0 + Negative_infinity*i, some_complex))
			assert ("0-Infinity*i ok", properties.is_infinite_ok (0 + Negative_infinity*i, some_complex))
			assert ("a-Infinity*i ok", properties.is_infinite_ok (some_real + Negative_infinity*i, some_complex))

			assert ("-NaN+Infinity*i ok", properties.is_infinite_ok (-Nan + Positive_infinity*i, some_complex))
			assert ("NaN+Infinity*i ok", properties.is_infinite_ok (Nan + Positive_infinity*i, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.is_infinite_ok (Negative_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity+Infinity*i ok", properties.is_infinite_ok (Positive_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity*i ok", properties.is_infinite_ok (Positive_infinity*i, some_complex))
			assert ("-0+Infinity*i ok", properties.is_infinite_ok (-0.0 + Positive_infinity*i, some_complex))
			assert ("0+Infinity*i ok", properties.is_infinite_ok (0 + Positive_infinity*i, some_complex))
			assert ("a+Infinity*i ok", properties.is_infinite_ok (some_real + Positive_infinity*i, some_complex))

			assert ("-NaN-0*i ok", properties.is_infinite_ok (-Nan - 0*i, some_complex))
			assert ("NaN-0*i ok", properties.is_infinite_ok (Nan - 0*i, some_complex))
			assert ("-Infinity-0*i ok", properties.is_infinite_ok (Negative_infinity - 0*i, some_complex))
			assert ("Infinity-0*i ok", properties.is_infinite_ok (Positive_infinity - 0*i, some_complex))
			assert ("-0*i ok", properties.is_infinite_ok (-0.0*i, some_complex))
			assert ("-0-0*i ok", properties.is_infinite_ok (-0.0 - 0*i, some_complex))
			assert ("0-0*i ok", properties.is_infinite_ok (0 - 0*i, some_complex))
			assert ("a-0*i ok", properties.is_infinite_ok (some_real - 0*i, some_complex))

			assert ("-NaN ok", properties.is_infinite_ok (-Nan, some_complex))
			assert ("-NaN+0*i ok", properties.is_infinite_ok (-Nan + 0*i, some_complex))
			assert ("NaN ok", properties.is_infinite_ok (Nan, some_complex))
			assert ("NaN+0*i ok", properties.is_infinite_ok (Nan + 0*i, some_complex))
			assert ("-Infinity ok", properties.is_infinite_ok (Negative_infinity, some_complex))
			assert ("-Infinity+0*i ok", properties.is_infinite_ok (Negative_infinity + 0*i, some_complex))
			assert ("Infinity ok", properties.is_infinite_ok (Positive_infinity, some_complex))
			assert ("Infinity+0*i ok", properties.is_infinite_ok (Positive_infinity + 0*i, some_complex))
			assert ("0 ok", properties.is_infinite_ok (Zero, some_complex))
			assert ("0*i ok", properties.is_infinite_ok (0*i, some_complex))
			assert ("-0+0*i ok", properties.is_infinite_ok (-0.0 + 0*i, some_complex))
			assert ("0+0*i ok", properties.is_infinite_ok (0 + 0*i, some_complex))
			assert ("a ok", properties.is_infinite_ok (some_real, some_complex))
			assert ("a+0*i ok", properties.is_infinite_ok (some_real + 0*i, some_complex))

			assert ("-NaN+bi ok", properties.is_infinite_ok (-Nan + some_real*i, some_complex))
			assert ("NaN+bi ok", properties.is_infinite_ok (Nan + some_real*i, some_complex))
			assert ("-Infinity+bi ok", properties.is_infinite_ok (Negative_infinity + some_real*i, some_complex))
			assert ("Infinity+bi ok", properties.is_infinite_ok (Positive_infinity + some_real*i, some_complex))
			assert ("bi ok", properties.is_infinite_ok (some_real*i, some_complex))
			assert ("-0+bi ok", properties.is_infinite_ok (-0.0 + some_real*i, some_complex))
			assert ("0+bi ok", properties.is_infinite_ok (0 + some_real*i, some_complex))
			assert ("a+bi ok", properties.is_infinite_ok (some_real + some_real*i, some_complex))

			assert ("is_infinite_ok", properties.is_infinite_ok (complex_to_be_tested, some_complex))
		end

	test_is_finite
			-- Test {STS_COMPLEX_NUMBER}.is_finite.
			-- Test {STI_COMPLEX_NUMBER}.is_finite.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_finite"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_finite"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
			x, y: like some_real
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_finite", z.is_finite implies True)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			from
				x := some_real
			until
				x.is_finite
			loop
				x := some_real
			end
			from
				y := some_real
			until
				y.is_finite
			loop
				y := some_real
			end
			z := x + y*i
			assert ("z.is_finite", z.is_finite)

			if next_random_item \\ 2 = 0 then
				from
					x := some_real
				until
					not x.is_finite
				loop
					x := some_real
				end
				y := some_real
			else
				x := some_real
				from
					y := some_real
				until
					not y.is_finite
				loop
					y := some_real
				end
			end
			z := x + y*i
			assert ("not z.is_finite", not z.is_finite)
		end

	test_is_rational
			-- Test {STS_COMPLEX_NUMBER}.is_rational.
			-- Test {STI_COMPLEX_NUMBER}.is_rational.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_rational"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_rational"
		local
			z: like complex_to_be_tested
			zs: like some_quick_complex_set
			zt: like some_quick_complex_set.as_tuple
			j: like some_set.cardinality
		do
			zs := Quick_c | agent (w: STS_COMPLEX_NUMBER): BOOLEAN
				do
					Result := w.b ≍ Zero and w.a.is_rational
				end
			zt := zs.as_tuple
			j := (next_random_item.as_natural_32 \\ zt.n + 1)
				check
					positive: 0 < j -- next_random_item \\ zt.n + 1 > 0
					small_enough: j ≤ zt.n -- next_random_item \\ zt.n + 1 ≤ zt.n
				end
			z := zt [j]
			assert ("z.is_rational", z.is_rational)
			assert ("z.is_rational ok", properties.is_rational_ok (z))

			zs := Quick_c ∖ zs
			zt := zs.as_tuple
			j := (next_random_item.as_natural_32 \\ zt.n + 1)
				check
					positive: 0 < j -- next_random_item \\ zt.n + 1 > 0
					small_enough: j ≤ zt.n -- next_random_item \\ zt.n + 1 ≤ zt.n
				end
			z := zt [j]
			assert ("not z.is_rational", not z.is_rational)
			assert ("not z.is_rational ok", properties.is_rational_ok (z))

			from
				zs := Quick_c
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_rational", z.is_rational implies True)
				assert ("is_rational ok", properties.is_rational_ok (z))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end
		end

	test_is_integer
			-- Test {STS_COMPLEX_NUMBER}.is_integer.
			-- Test {STI_COMPLEX_NUMBER}.is_integer.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_integer"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_integer"
		local
			z: like complex_to_be_tested
			zs: like some_quick_complex_set
			zt: like some_quick_complex_set.as_tuple
			j: like some_set.cardinality
		do
			zs := Quick_c | agent {STS_COMPLEX_NUMBER}.is_integer -- TODO: Take Z instead.
			zt := zs.as_tuple
			j := (next_random_item.as_natural_32 \\ zt.n + 1)
				check
					positive: 0 < j -- next_random_item \\ zt.n + 1 > 0
					small_enough: j ≤ zt.n -- next_random_item \\ zt.n + 1 ≤ zt.n
				end
			z := zt [j]
			assert ("z.is_integer", z.is_integer)

			zs := Quick_c ∖ zs
			zt := zs.as_tuple
			j := (next_random_item.as_natural_32 \\ zt.n + 1)
				check
					positive: 0 < j -- next_random_item \\ zt.n + 1 > 0
					small_enough: j ≤ zt.n -- next_random_item \\ zt.n + 1 ≤ zt.n
				end
			z := zt [j]
			assert ("not z.is_integer", not z.is_integer)

			from
				zs := Quick_c
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_integer", z.is_integer implies True)
				assert ("is_integer ok", properties.is_integer_ok (z))
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end
		end

	test_is_invertible
			-- Test {STI_COMPLEX_NUMBER}.is_invertible.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.is_invertible"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("is_invertible", z.is_invertible)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end
		end

	test_is_identical
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.is_identical.
			-- Test {STI_COMPLEX_NUMBER}.is_identical.
		note
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.is_identical"
			testing: "covers/{STI_COMPLEX_NUMBER}.is_identical"
		local
			x, y: like some_real
			z: like complex_to_be_tested
		do
			x := some_real
			y := same_real (x)
			create z.make (x, y)
			assert ("z.is_identical", z.is_identical)
			assert ("z.is_identical ok", homogeneous_pair_properties.is_identical_ok (z))

			from
				x := some_real
				y := some_real
			until
				x ≭ y
			loop
				y := some_real
			end
			create z.make (x, y)
			assert ("not z.is_identical", not z.is_identical)
			assert ("not z.is_identical ok", homogeneous_pair_properties.is_identical_ok (z))

			z := complex_to_be_tested
			assert ("is_identical", z.is_identical implies True)
			assert ("is_identical_ok", homogeneous_pair_properties.is_identical_ok (z))
		end

feature -- Test routines (Output)

	test_out
			-- Test {STI_COMPLEX_NUMBER}.out.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.out"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("out", attached z.out)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end
		end

feature -- Test routines (Comparison)

	test_pair_equals
			-- Test {STS_ORDERED_PAIR}.equals.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.equals.
			-- Test {STI_COMPLEX_NUMBER}.pair_equals.
		note
			testing: "covers/{STS_ORDERED_PAIR}.equals"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.equals"
			testing: "covers/{STI_COMPLEX_NUMBER}.pair_equals"
		local
			z: like complex_to_be_tested
			ab: like resources_rr.some_pair_ab
			x, y: like some_real
		do
			z := complex_to_be_tested
			assert ("same_entity", z.pair_equals (z))
			assert ("same_entity_ok", homogeneous_pair_properties.opp_equals_ok (z, z))

			z := complex_to_be_tested
			ab := resources_rr.same_pair_ab (z)
			assert ("same_pair", z.pair_equals (ab))
			assert ("same_pair_ok", homogeneous_pair_properties.opp_equals_ok (z, ab))

			z := complex_to_be_tested
			ab := resources_rr.some_pair_ab
			assert ("some_pair", z.pair_equals (ab) implies True)
			assert ("some_pair_ok", homogeneous_pair_properties.opp_equals_ok (z, ab))

			x := some_real
			y := same_real (x)
			assert ("xy_equals_yx", (x + y*I).pair_equals (homogeneous_pair_properties.pair (y, x)))
			assert ("xy_equals_yx_ok", homogeneous_pair_properties.equals_ok (x, y)) -- TODO: It doesn't test complex numbers.

			x := some_real
			y := some_real
			assert ("equals", (x + y*I).pair_equals (homogeneous_pair_properties.pair (y, x)) implies True)
			assert ("equals_ok", homogeneous_pair_properties.equals_ok (x, y))
		end

	test_equals
			-- Test {STS_COMPLEX_NUMBER}.equals.
			-- Test {STI_COMPLEX_NUMBER}.equals.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.equals"
			testing: "covers/{STI_COMPLEX_NUMBER}.equals"
		local
			z1: like complex_to_be_tested
			z2: like some_complex
		do
			z1 := complex_to_be_tested
			assert ("same_entity", z1 ≍ z1)
			assert ("same_entity_ok", properties.equals_ok (z1, z1, z1))

			z1 := complex_to_be_tested
			z2 := same_complex (z1)
			assert ("same_complex", z1 ≍ z2)
			assert ("same_complex_ok", properties.equals_ok (z1, z2, same_complex (z2)))

			z1 := complex_to_be_tested
			z2 := some_complex
			assert ("some_complex", z1 ≍ z2 implies True)
			assert ("some_complex", properties.equals_ok (z1, z2, some_complex))
		end

	test_pair_unequals
			-- Test {STS_ORDERED_PAIR}.unequals.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.unequals.
			-- Test {STI_COMPLEX_NUMBER}.pair_unequals.
		note
			testing: "covers/{STS_ORDERED_PAIR}.unequals"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.unequals"
			testing: "covers/{STI_COMPLEX_NUMBER}.pair_unequals"
		local
			z: like complex_to_be_tested
			ab: like resources_rr.some_pair_ab
		do
			z := complex_to_be_tested
			assert ("same_entity", not z.pair_unequals (z))
			assert ("same_entity_ok", homogeneous_pair_properties.unequals_ok (z, z))

			z := complex_to_be_tested
			ab := resources_rr.same_pair_ab (z)
			assert ("same_pair", not z.pair_unequals (ab))
			assert ("same_pair_ok", homogeneous_pair_properties.unequals_ok (z, ab))

			z := complex_to_be_tested
			ab := resources_rr.some_pair_ab
			assert ("some_pair", not z.pair_unequals (ab) implies True)
			assert ("some_pair_ok", homogeneous_pair_properties.unequals_ok (z, ab))
		end

	test_unequals
			-- Test {STS_COMPLEX_NUMBER}.unequals.
			-- Test {STI_COMPLEX_NUMBER}.unequals.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.unequals"
			testing: "covers/{STI_COMPLEX_NUMBER}.unequals"
		local
			z1: like complex_to_be_tested
			z2: like some_complex
		do
			z1 := complex_to_be_tested
			assert ("same_entity", not (z1 ≭ z1))
			assert ("same_entity_ok", properties.unequals_ok (z1, z1))

			z1 := complex_to_be_tested
			z2 := same_complex (z1)
			assert ("same_complex", not (z1 ≭ z2))
			assert ("same_complex_ok", properties.unequals_ok (z1, z2))

			z1 := complex_to_be_tested
			z2 := some_complex
			assert ("some_complex", not (z1 ≭ z2) implies True)
			assert ("some_complex_ok", properties.unequals_ok (z1, z2))
		end

	test_succeeds
			-- Test {STS_ORDERED_PAIR}.succeeds.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.succeeds.
			-- Test {STI_COMPLEX_NUMBER}.succeeds.
		note
			testing: "covers/{STS_ORDERED_PAIR}.succeeds"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.succeeds"
			testing: "covers/{STI_COMPLEX_NUMBER}.succeeds"
		local
			z: like complex_to_be_tested
			dsa_x: like resources_rr.some_preceding_pair_dsa_a
			x: like some_real
		do
			z := complex_to_be_tested
			dsa_x := resources_rr.some_preceding_pair_dsa_a (z.a)
			assert ("z ≽ dsa_x", z ≽ dsa_x)
			assert ("z ≽ dsa_x ok", homogeneous_pair_properties.succeeds_ok (z, dsa_x))

			z := complex_to_be_tested
			from
				x := some_real
			until
				x ≭ z.a
			loop
				x := some_real
			end
			dsa_x := resources_rr.some_preceding_pair_dsa_a (x)
			assert ("not (z ≽ dsa_x)", not (z ≽ dsa_x))
			assert ("not (z ≽ dsa_x) ok", homogeneous_pair_properties.succeeds_ok (z, dsa_x))

			z := complex_to_be_tested
			dsa_x := resources_rr.some_preceding_pair_dsa_a (some_real)
			assert ("succeeds", z ≽ dsa_x implies True)
			assert ("succeeds_ok", homogeneous_pair_properties.succeeds_ok (z, dsa_x))
		end

feature -- Test routines (Relationship)

	test_divisible
			-- Test {STS_COMPLEX_NUMBER}.divisible.
			-- Test {STI_COMPLEX_NUMBER}.divisible.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.divisible"
			testing: "covers/{STI_COMPLEX_NUMBER}.divisible"
		local
			zs: like Quick_C.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			z1: like complex_to_be_tested
			z2: like some_complex
		do
			from
				zs := Quick_C
					from
						zt0 := zs.as_tuple
						if next_random_item \\ 2 = 0 then
							zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
						else
							zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
						end
					until
						zt0.n × # zs <= 457_620 -- TODO: no magic number!
					loop
						if next_random_item \\ 2 = 0 then
							zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
						else
							zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
						end
					end
			until
				zs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					z1 := zs.any
					z2 := same_complex (zt.first)
					assert ("divisible", z1.divisible (z2))
					zt := zt.tail
				variant
					n: {like Quick_C}.natural_as_integer (zt.n)
				end
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end
		end

feature -- Test routines (Operation)

	test_composed
			-- Test {STI_COMPLEX_NUMBER}.composed.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.composed"
		local
			z: like complex_to_be_tested
			aa_2, aa_1: like resources_rr.some_preceding_pair_aa
			ba: like resources_rr.some_preceding_pair_ba
			bb: like resources_rr.some_preceding_pair_bb
		do
			z := complex_to_be_tested
			aa_2 := resources_rr.some_preceding_pair_aa (z.a)
			assert ("z_succeeds_aa_2", z ≽ aa_2)
			assert ("z_oh_aa_2", attached (z ∘ aa_2))
			aa_1 := resources_rr.some_preceding_pair_aa (aa_2.first)
			assert ("aa_2_succeeds_aa_1", aa_2 ≽ aa_1)
			assert ("z_oh_aa_2_aa_1_ok", homogeneous_pair_properties.composed_ok (z, aa_2, aa_1))
			assert ("z_o_aa_2_aa_1_ok", composable_pair_properties.composed_ok (z, aa_2, aa_1))

			z := complex_to_be_tested
			aa_2 := resources_rr.some_preceding_pair_aa (some_real)
			aa_1 := resources_rr.some_preceding_pair_aa (some_real)
			assert ("composed", z ≽ aa_2 implies attached (z ∘ aa_2))
			assert ("composed_ok", homogeneous_pair_properties.composed_ok (z, aa_2, aa_1))
			assert ("composable_composed_ok", composable_pair_properties.composed_ok (z, aa_2, aa_1))

			ba := resources_rr.some_preceding_pair_ba (z.a)
			assert ("z_succeeds_ba", z ≽ ba)
			assert ("z_oi_ba", attached (z ∘ ba))
			bb := resources_rr.some_preceding_pair_bb (ba.first)
			assert ("z_oi_ba_bb_ok", homogeneous_pair_properties.composed_with_inverse_like_ok (z, ba, bb))
			assert ("z_o_ba_bb_ok", composable_pair_properties.composed_ok (z, ba, bb))

			bb := resources_rr.some_preceding_pair_bb (some_real)
			ba := resources_rr.some_preceding_pair_ba (some_real)
			z := complex_to_be_tested
			assert ("composed_with_inverse_like", z ≽ ba implies attached (z ∘ ba))
			assert ("composed_with_inverse_like_ok", homogeneous_pair_properties.composed_with_inverse_like_ok (z, ba, bb))
			assert ("composable_composed_ok", composable_pair_properties.composed_ok (z, ba, bb))
		end

	test_modulus
			-- Test {STS_COMPLEX_NUMBER}.modulus.
			-- Test {STI_COMPLEX_NUMBER}.modulus.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.modulus"
			testing: "covers/{STI_COMPLEX_NUMBER}.modulus"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("modulus", attached z.modulus)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.modulus_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.modulus_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.modulus_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.modulus_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.modulus_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.modulus_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.modulus_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.modulus_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.modulus_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.modulus_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.modulus_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.modulus_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.modulus_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.modulus_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.modulus_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.modulus_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.modulus_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.modulus_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.modulus_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.modulus_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.modulus_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.modulus_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.modulus_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.modulus_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.modulus_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.modulus_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.modulus_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.modulus_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.modulus_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.modulus_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.modulus_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.modulus_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.modulus_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.modulus_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.modulus_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.modulus_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.modulus_ok (-0.0*i))
			assert ("-0-0*i ok", properties.modulus_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.modulus_ok (0 - 0*i))
			assert ("a-0*i ok", properties.modulus_ok (some_real - 0*i))

			assert ("-NaN ok", properties.modulus_ok (-Nan))
			assert ("-NaN+0*i ok", properties.modulus_ok (-Nan + 0*i))
			assert ("NaN ok", properties.modulus_ok (Nan))
			assert ("NaN+0*i ok", properties.modulus_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.modulus_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.modulus_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.modulus_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.modulus_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.modulus_ok (Zero))
			assert ("0*i ok", properties.modulus_ok (0*i))
			assert ("-0+0*i ok", properties.modulus_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.modulus_ok (0 + 0*i))
			assert ("a ok", properties.modulus_ok (some_real))
			assert ("a+0*i ok", properties.modulus_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.modulus_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.modulus_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.modulus_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.modulus_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.modulus_ok (some_real*i))
			assert ("-0+bi ok", properties.modulus_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.modulus_ok (0 + some_real*i))
			assert ("a+bi ok", properties.modulus_ok (some_real + some_real*i))

			assert ("modulus_ok", properties.modulus_ok (complex_to_be_tested))
		end

	test_abs
			-- Test {STS_COMPLEX_NUMBER}.abs.
			-- Test {STI_COMPLEX_NUMBER}.abs.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.abs"
			testing: "covers/{STI_COMPLEX_NUMBER}.abs"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("abs", attached z.abs)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.abs_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.abs_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.abs_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.abs_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.abs_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.abs_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.abs_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.abs_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.abs_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.abs_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.abs_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.abs_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.abs_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.abs_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.abs_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.abs_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.abs_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.abs_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.abs_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.abs_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.abs_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.abs_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.abs_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.abs_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.abs_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.abs_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.abs_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.abs_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.abs_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.abs_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.abs_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.abs_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.abs_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.abs_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.abs_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.abs_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.abs_ok (-0.0*i))
			assert ("-0-0*i ok", properties.abs_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.abs_ok (0 - 0*i))
			assert ("a-0*i ok", properties.abs_ok (some_real - 0*i))

			assert ("-NaN ok", properties.abs_ok (-Nan))
			assert ("-NaN+0*i ok", properties.abs_ok (-Nan + 0*i))
			assert ("NaN ok", properties.abs_ok (Nan))
			assert ("NaN+0*i ok", properties.abs_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.abs_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.abs_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.abs_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.abs_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.abs_ok (Zero))
			assert ("0*i ok", properties.abs_ok (0*i))
			assert ("-0+0*i ok", properties.abs_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.abs_ok (0 + 0*i))
			assert ("a ok", properties.abs_ok (some_real))
			assert ("a+0*i ok", properties.abs_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.abs_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.abs_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.abs_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.abs_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.abs_ok (some_real*i))
			assert ("-0+bi ok", properties.abs_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.abs_ok (0 + some_real*i))
			assert ("a+bi ok", properties.abs_ok (some_real + some_real*i))

			assert ("abs_ok", properties.abs_ok (complex_to_be_tested))
		end

	test_argument
			-- Test {STS_COMPLEX_NUMBER}.argument.
			-- Test {STI_COMPLEX_NUMBER}.argument.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.argument"
			testing: "covers/{STI_COMPLEX_NUMBER}.argument"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("argument", attached z.argument)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.argument_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.argument_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.argument_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.argument_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.argument_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.argument_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.argument_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.argument_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.argument_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.argument_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.argument_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.argument_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.argument_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.argument_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.argument_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.argument_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.argument_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.argument_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.argument_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.argument_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.argument_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.argument_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.argument_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.argument_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.argument_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.argument_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.argument_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.argument_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.argument_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.argument_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.argument_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.argument_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.argument_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.argument_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.argument_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.argument_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.argument_ok (-0.0*i))
			assert ("-0-0*i ok", properties.argument_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.argument_ok (0 - 0*i))
			assert ("a-0*i ok", properties.argument_ok (some_real - 0*i))

			assert ("-NaN ok", properties.argument_ok (-Nan))
			assert ("-NaN+0*i ok", properties.argument_ok (-Nan + 0*i))
			assert ("NaN ok", properties.argument_ok (Nan))
			assert ("NaN+0*i ok", properties.argument_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.argument_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.argument_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.argument_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.argument_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.argument_ok (Zero))
			assert ("0*i ok", properties.argument_ok (0*i))
			assert ("-0+0*i ok", properties.argument_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.argument_ok (0 + 0*i))
			assert ("a ok", properties.argument_ok (some_real))
			assert ("a+0*i ok", properties.argument_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.argument_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.argument_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.argument_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.argument_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.argument_ok (some_real*i))
			assert ("-0+bi ok", properties.argument_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.argument_ok (0 + some_real*i))
			assert ("a+bi ok", properties.argument_ok (some_real + some_real*i))

			assert ("argument_ok", properties.argument_ok (complex_to_be_tested))
		end

	test_phase
			-- Test {STS_COMPLEX_NUMBER}.phase.
			-- Test {STI_COMPLEX_NUMBER}.phase.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.phase"
			testing: "covers/{STI_COMPLEX_NUMBER}.phase"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("phase", attached z.phase)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.phase_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.phase_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.phase_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.phase_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.phase_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.phase_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.phase_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.phase_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.phase_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.phase_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.phase_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.phase_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.phase_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.phase_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.phase_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.phase_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.phase_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.phase_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.phase_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.phase_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.phase_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.phase_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.phase_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.phase_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.phase_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.phase_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.phase_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.phase_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.phase_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.phase_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.phase_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.phase_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.phase_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.phase_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.phase_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.phase_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.phase_ok (-0.0*i))
			assert ("-0-0*i ok", properties.phase_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.phase_ok (0 - 0*i))
			assert ("a-0*i ok", properties.phase_ok (some_real - 0*i))

			assert ("-NaN ok", properties.phase_ok (-Nan))
			assert ("-NaN+0*i ok", properties.phase_ok (-Nan + 0*i))
			assert ("NaN ok", properties.phase_ok (Nan))
			assert ("NaN+0*i ok", properties.phase_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.phase_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.phase_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.phase_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.phase_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.phase_ok (Zero))
			assert ("0*i ok", properties.phase_ok (0*i))
			assert ("-0+0*i ok", properties.phase_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.phase_ok (0 + 0*i))
			assert ("a ok", properties.phase_ok (some_real))
			assert ("a+0*i ok", properties.phase_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.phase_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.phase_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.phase_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.phase_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.phase_ok (some_real*i))
			assert ("-0+bi ok", properties.phase_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.phase_ok (0 + some_real*i))
			assert ("a+bi ok", properties.phase_ok (some_real + some_real*i))

			assert ("phase_ok", properties.phase_ok (complex_to_be_tested))
		end

	test_conjugate
			-- Test {STI_COMPLEX_NUMBER}.conjugate.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.conjugate"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("conjugate", attached z.conjugate)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.conjugate_ok (-Nan - Nan*i, some_complex))
			assert ("NaN-NaN*i ok", properties.conjugate_ok (Nan - Nan*i, some_complex))
			assert ("-Infinity-NaN*i ok", properties.conjugate_ok (Negative_infinity - Nan*i, some_complex))
			assert ("Infinity-NaN*i ok", properties.conjugate_ok (Positive_infinity - Nan*i, some_complex))
			assert ("-NaN*i ok", properties.conjugate_ok (-Nan*i, some_complex))
			assert ("-0-NaN*i ok", properties.conjugate_ok (-0.0 - Nan*i, some_complex))
			assert ("0-NaN*i ok", properties.conjugate_ok (0 - Nan*i, some_complex))
			assert ("a-NaN*i ok", properties.conjugate_ok (some_real - Nan*i, some_complex))

			assert ("-NaN+NaN*i ok", properties.conjugate_ok (-Nan + Nan*i, some_complex))
			assert ("NaN+NaN*i ok", properties.conjugate_ok (Nan + Nan*i, some_complex))
			assert ("-Infinity+NaN*i ok", properties.conjugate_ok (Negative_infinity + Nan*i, some_complex))
			assert ("Infinity+NaN*i ok", properties.conjugate_ok (Positive_infinity + Nan*i, some_complex))
			assert ("NaN*i ok", properties.conjugate_ok (Nan*i, some_complex))
			assert ("-0+NaN*i ok", properties.conjugate_ok (-0.0 + Nan*i, some_complex))
			assert ("0+NaN*i ok", properties.conjugate_ok (0 + Nan*i, some_complex))
			assert ("a+NaN*i ok", properties.conjugate_ok (some_real + Nan*i, some_complex))

			assert ("-NaN-Infinity*i ok", properties.conjugate_ok (-Nan + Negative_infinity*i, some_complex))
			assert ("NaN-Infinity*i ok", properties.conjugate_ok (Nan + Negative_infinity*i, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.conjugate_ok (Negative_infinity + Negative_infinity*i, some_complex))
			assert ("Infinity-Infinity*i ok", properties.conjugate_ok (Positive_infinity + Negative_infinity*i, some_complex))
			assert ("-Infinity*i ok", properties.conjugate_ok (Negative_infinity*i, some_complex))
			assert ("-0-Infinity*i ok", properties.conjugate_ok (-0.0 + Negative_infinity*i, some_complex))
			assert ("0-Infinity*i ok", properties.conjugate_ok (0 + Negative_infinity*i, some_complex))
			assert ("a-Infinity*i ok", properties.conjugate_ok (some_real + Negative_infinity*i, some_complex))

			assert ("-NaN+Infinity*i ok", properties.conjugate_ok (-Nan + Positive_infinity*i, some_complex))
			assert ("NaN+Infinity*i ok", properties.conjugate_ok (Nan + Positive_infinity*i, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.conjugate_ok (Negative_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity+Infinity*i ok", properties.conjugate_ok (Positive_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity*i ok", properties.conjugate_ok (Positive_infinity*i, some_complex))
			assert ("-0+Infinity*i ok", properties.conjugate_ok (-0.0 + Positive_infinity*i, some_complex))
			assert ("0+Infinity*i ok", properties.conjugate_ok (0 + Positive_infinity*i, some_complex))
			assert ("a+Infinity*i ok", properties.conjugate_ok (some_real + Positive_infinity*i, some_complex))

			assert ("-NaN-0*i ok", properties.conjugate_ok (-Nan - 0*i, some_complex))
			assert ("NaN-0*i ok", properties.conjugate_ok (Nan - 0*i, some_complex))
			assert ("-Infinity-0*i ok", properties.conjugate_ok (Negative_infinity - 0*i, some_complex))
			assert ("Infinity-0*i ok", properties.conjugate_ok (Positive_infinity - 0*i, some_complex))
			assert ("-0*i ok", properties.conjugate_ok (-0.0*i, some_complex))
			assert ("-0-0*i ok", properties.conjugate_ok (-0.0 - 0*i, some_complex))
			assert ("0-0*i ok", properties.conjugate_ok (0 - 0*i, some_complex))
			assert ("a-0*i ok", properties.conjugate_ok (some_real - 0*i, some_complex))

			assert ("-NaN ok", properties.conjugate_ok (-Nan, some_complex))
			assert ("-NaN+0*i ok", properties.conjugate_ok (-Nan + 0*i, some_complex))
			assert ("NaN ok", properties.conjugate_ok (Nan, some_complex))
			assert ("NaN+0*i ok", properties.conjugate_ok (Nan + 0*i, some_complex))
			assert ("-Infinity ok", properties.conjugate_ok (Negative_infinity, some_complex))
			assert ("-Infinity+0*i ok", properties.conjugate_ok (Negative_infinity + 0*i, some_complex))
			assert ("Infinity ok", properties.conjugate_ok (Positive_infinity, some_complex))
			assert ("Infinity+0*i ok", properties.conjugate_ok (Positive_infinity + 0*i, some_complex))
			assert ("0 ok", properties.conjugate_ok (Zero, some_complex))
			assert ("0*i ok", properties.conjugate_ok (0*i, some_complex))
			assert ("-0+0*i ok", properties.conjugate_ok (-0.0 + 0*i, some_complex))
			assert ("0+0*i ok", properties.conjugate_ok (0 + 0*i, some_complex))
			assert ("a ok", properties.conjugate_ok (some_real, some_complex))
			assert ("a+0*i ok", properties.conjugate_ok (some_real + 0*i, some_complex))

			assert ("-NaN+bi ok", properties.conjugate_ok (-Nan + some_real*i, some_complex))
			assert ("NaN+bi ok", properties.conjugate_ok (Nan + some_real*i, some_complex))
			assert ("-Infinity+bi ok", properties.conjugate_ok (Negative_infinity + some_real*i, some_complex))
			assert ("Infinity+bi ok", properties.conjugate_ok (Positive_infinity + some_real*i, some_complex))
			assert ("bi ok", properties.conjugate_ok (some_real*i, some_complex))
			assert ("-0+bi ok", properties.conjugate_ok (-0.0 + some_real*i, some_complex))
			assert ("0+bi ok", properties.conjugate_ok (0 + some_real*i, some_complex))
			assert ("a+bi ok", properties.conjugate_ok (some_real + some_real*i, some_complex))

			assert ("conjugate_ok", properties.conjugate_ok (complex_to_be_tested, some_complex))
		end

	test_plus
			-- Test {STI_COMPLEX_NUMBER}.plus.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.plus"
		local
			zs: like Quick_c.subset_anchor
			zt0, zt: like Quick_c.as_tuple
			z1: like complex_to_be_tested
			z2: like some_complex
		do
			from
				zs := Quick_c
				from
					zt0 := zs.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # zs <= 161_307 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# C * zt0.n) print ('%N')
			until
				zs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					z1 := zs.any
					z2 := same_complex (zt.first)
					assert ("plus", attached (z1 + z2))
					zt := zt.tail
				variant
					n: {like Quick_C}.natural_as_integer (zt.n)
				end
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.plus_ok (-Nan - Nan*i, some_complex))
			assert ("NaN-NaN*i ok", properties.plus_ok (Nan - Nan*i, some_complex))
			assert ("-Infinity-NaN*i ok", properties.plus_ok (Negative_infinity - Nan*i, some_complex))
			assert ("Infinity-NaN*i ok", properties.plus_ok (Positive_infinity - Nan*i, some_complex))
			assert ("-NaN*i ok", properties.plus_ok (-Nan*i, some_complex))
			assert ("-0-NaN*i ok", properties.plus_ok (-0.0 - Nan*i, some_complex))
			assert ("0-NaN*i ok", properties.plus_ok (0 - Nan*i, some_complex))
			assert ("a-NaN*i ok", properties.plus_ok (some_real - Nan*i, some_complex))

			assert ("-NaN+NaN*i ok", properties.plus_ok (-Nan + Nan*i, some_complex))
			assert ("NaN+NaN*i ok", properties.plus_ok (Nan + Nan*i, some_complex))
			assert ("-Infinity+NaN*i ok", properties.plus_ok (Negative_infinity + Nan*i, some_complex))
			assert ("Infinity+NaN*i ok", properties.plus_ok (Positive_infinity + Nan*i, some_complex))
			assert ("NaN*i ok", properties.plus_ok (Nan*i, some_complex))
			assert ("-0+NaN*i ok", properties.plus_ok (-0.0 + Nan*i, some_complex))
			assert ("0+NaN*i ok", properties.plus_ok (0 + Nan*i, some_complex))
			assert ("a+NaN*i ok", properties.plus_ok (some_real + Nan*i, some_complex))

			assert ("-NaN-Infinity*i ok", properties.plus_ok (-Nan + Negative_infinity*i, some_complex))
			assert ("NaN-Infinity*i ok", properties.plus_ok (Nan + Negative_infinity*i, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.plus_ok (Negative_infinity + Negative_infinity*i, some_complex))
			assert ("Infinity-Infinity*i ok", properties.plus_ok (Positive_infinity + Negative_infinity*i, some_complex))
			assert ("-Infinity*i ok", properties.plus_ok (Negative_infinity*i, some_complex))
			assert ("-0-Infinity*i ok", properties.plus_ok (-0.0 + Negative_infinity*i, some_complex))
			assert ("0-Infinity*i ok", properties.plus_ok (0 + Negative_infinity*i, some_complex))
			assert ("a-Infinity*i ok", properties.plus_ok (some_real + Negative_infinity*i, some_complex))

			assert ("-NaN+Infinity*i ok", properties.plus_ok (-Nan + Positive_infinity*i, some_complex))
			assert ("NaN+Infinity*i ok", properties.plus_ok (Nan + Positive_infinity*i, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.plus_ok (Negative_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity+Infinity*i ok", properties.plus_ok (Positive_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity*i ok", properties.plus_ok (Positive_infinity*i, some_complex))
			assert ("-0+Infinity*i ok", properties.plus_ok (-0.0 + Positive_infinity*i, some_complex))
			assert ("0+Infinity*i ok", properties.plus_ok (0 + Positive_infinity*i, some_complex))
			assert ("a+Infinity*i ok", properties.plus_ok (some_real + Positive_infinity*i, some_complex))

			assert ("-NaN-0*i ok", properties.plus_ok (-Nan - 0*i, some_complex))
			assert ("NaN-0*i ok", properties.plus_ok (Nan - 0*i, some_complex))
			assert ("-Infinity-0*i ok", properties.plus_ok (Negative_infinity - 0*i, some_complex))
			assert ("Infinity-0*i ok", properties.plus_ok (Positive_infinity - 0*i, some_complex))
			assert ("-0*i ok", properties.plus_ok (-0.0*i, some_complex))
			assert ("-0-0*i ok", properties.plus_ok (-0.0 - 0*i, some_complex))
			assert ("0-0*i ok", properties.plus_ok (0 - 0*i, some_complex))
			assert ("a-0*i ok", properties.plus_ok (some_real - 0*i, some_complex))

			assert ("-NaN ok", properties.plus_ok (-Nan, some_complex))
			assert ("-NaN+0*i ok", properties.plus_ok (-Nan + 0*i, some_complex))
			assert ("NaN ok", properties.plus_ok (Nan, some_complex))
			assert ("NaN+0*i ok", properties.plus_ok (Nan + 0*i, some_complex))
			assert ("-Infinity ok", properties.plus_ok (Negative_infinity, some_complex))
			assert ("-Infinity+0*i ok", properties.plus_ok (Negative_infinity + 0*i, some_complex))
			assert ("Infinity ok", properties.plus_ok (Positive_infinity, some_complex))
			assert ("Infinity+0*i ok", properties.plus_ok (Positive_infinity + 0*i, some_complex))
			assert ("0 ok", properties.plus_ok (Zero, some_complex))
			assert ("0*i ok", properties.plus_ok (0*i, some_complex))
			assert ("-0+0*i ok", properties.plus_ok (-0.0 + 0*i, some_complex))
			assert ("0+0*i ok", properties.plus_ok (0 + 0*i, some_complex))
			assert ("a ok", properties.plus_ok (some_real, some_complex))
			assert ("a+0*i ok", properties.plus_ok (some_real + 0*i, some_complex))

			assert ("-NaN+bi ok", properties.plus_ok (-Nan + some_real*i, some_complex))
			assert ("NaN+bi ok", properties.plus_ok (Nan + some_real*i, some_complex))
			assert ("-Infinity+bi ok", properties.plus_ok (Negative_infinity + some_real*i, some_complex))
			assert ("Infinity+bi ok", properties.plus_ok (Positive_infinity + some_real*i, some_complex))
			assert ("bi ok", properties.plus_ok (some_real*i, some_complex))
			assert ("-0+bi ok", properties.plus_ok (-0.0 + some_real*i, some_complex))
			assert ("0+bi ok", properties.plus_ok (0 + some_real*i, some_complex))
			assert ("a+bi ok", properties.plus_ok (some_real + some_real*i, some_complex))

			assert ("plus_ok", properties.plus_ok (complex_to_be_tested, some_complex))
		end

	test_identity
			-- Test {STS_COMPLEX_NUMBER}.identity.
			-- Test {STI_COMPLEX_NUMBER}.identity.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.identity"
			testing: "covers/{STI_COMPLEX_NUMBER}.identity"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("identity", attached (+z))
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.identity_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.identity_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.identity_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.identity_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.identity_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.identity_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.identity_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.identity_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.identity_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.identity_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.identity_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.identity_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.identity_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.identity_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.identity_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.identity_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.identity_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.identity_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.identity_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.identity_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.identity_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.identity_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.identity_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.identity_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.identity_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.identity_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.identity_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.identity_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.identity_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.identity_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.identity_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.identity_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.identity_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.identity_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.identity_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.identity_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.identity_ok (-0.0*i))
			assert ("-0-0*i ok", properties.identity_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.identity_ok (0 - 0*i))
			assert ("a-0*i ok", properties.identity_ok (some_real - 0*i))

			assert ("-NaN ok", properties.identity_ok (-Nan))
			assert ("-NaN+0*i ok", properties.identity_ok (-Nan + 0*i))
			assert ("NaN ok", properties.identity_ok (Nan))
			assert ("NaN+0*i ok", properties.identity_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.identity_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.identity_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.identity_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.identity_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.identity_ok (Zero))
			assert ("0*i ok", properties.identity_ok (0*i))
			assert ("-0+0*i ok", properties.identity_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.identity_ok (0 + 0*i))
			assert ("a ok", properties.identity_ok (some_real))
			assert ("a+0*i ok", properties.identity_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.identity_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.identity_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.identity_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.identity_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.identity_ok (some_real*i))
			assert ("-0+bi ok", properties.identity_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.identity_ok (0 + some_real*i))
			assert ("a+bi ok", properties.identity_ok (some_real + some_real*i))

			assert ("identity_ok", properties.identity_ok (complex_to_be_tested))
		end

	test_minus
			-- Test {STI_COMPLEX_NUMBER}.minus.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.minus"
		local
			zs: like Quick_C.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			z1: like complex_to_be_tested
			z2: like some_complex
		do
			from
				zs := Quick_C
				from
					zt0 := zs.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # zs <= 147_003 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# C * zt0.n) print ('%N')
			until
				zs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					z1 := zs.any
					z2 := same_complex (zt.first)
					assert ("minus", attached (z1 - z2))
					zt := zt.tail
				variant
					n: {like Quick_C}.natural_as_integer (zt.n)
				end
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.minus_ok (-Nan - Nan*i, some_complex))
			assert ("NaN-NaN*i ok", properties.minus_ok (Nan - Nan*i, some_complex))
			assert ("-Infinity-NaN*i ok", properties.minus_ok (Negative_infinity - Nan*i, some_complex))
			assert ("Infinity-NaN*i ok", properties.minus_ok (Positive_infinity - Nan*i, some_complex))
			assert ("-NaN*i ok", properties.minus_ok (-Nan*i, some_complex))
			assert ("-0-NaN*i ok", properties.minus_ok (-0.0 - Nan*i, some_complex))
			assert ("0-NaN*i ok", properties.minus_ok (0 - Nan*i, some_complex))
			assert ("a-NaN*i ok", properties.minus_ok (some_real - Nan*i, some_complex))

			assert ("-NaN+NaN*i ok", properties.minus_ok (-Nan + Nan*i, some_complex))
			assert ("NaN+NaN*i ok", properties.minus_ok (Nan + Nan*i, some_complex))
			assert ("-Infinity+NaN*i ok", properties.minus_ok (Negative_infinity + Nan*i, some_complex))
			assert ("Infinity+NaN*i ok", properties.minus_ok (Positive_infinity + Nan*i, some_complex))
			assert ("NaN*i ok", properties.minus_ok (Nan*i, some_complex))
			assert ("-0+NaN*i ok", properties.minus_ok (-0.0 + Nan*i, some_complex))
			assert ("0+NaN*i ok", properties.minus_ok (0 + Nan*i, some_complex))
			assert ("a+NaN*i ok", properties.minus_ok (some_real + Nan*i, some_complex))

			assert ("-NaN-Infinity*i ok", properties.minus_ok (-Nan + Negative_infinity*i, some_complex))
			assert ("NaN-Infinity*i ok", properties.minus_ok (Nan + Negative_infinity*i, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.minus_ok (Negative_infinity + Negative_infinity*i, some_complex))
			assert ("Infinity-Infinity*i ok", properties.minus_ok (Positive_infinity + Negative_infinity*i, some_complex))
			assert ("-Infinity*i ok", properties.minus_ok (Negative_infinity*i, some_complex))
			assert ("-0-Infinity*i ok", properties.minus_ok (-0.0 + Negative_infinity*i, some_complex))
			assert ("0-Infinity*i ok", properties.minus_ok (0 + Negative_infinity*i, some_complex))
			assert ("a-Infinity*i ok", properties.minus_ok (some_real + Negative_infinity*i, some_complex))

			assert ("-NaN+Infinity*i ok", properties.minus_ok (-Nan + Positive_infinity*i, some_complex))
			assert ("NaN+Infinity*i ok", properties.minus_ok (Nan + Positive_infinity*i, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.minus_ok (Negative_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity+Infinity*i ok", properties.minus_ok (Positive_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity*i ok", properties.minus_ok (Positive_infinity*i, some_complex))
			assert ("-0+Infinity*i ok", properties.minus_ok (-0.0 + Positive_infinity*i, some_complex))
			assert ("0+Infinity*i ok", properties.minus_ok (0 + Positive_infinity*i, some_complex))
			assert ("a+Infinity*i ok", properties.minus_ok (some_real + Positive_infinity*i, some_complex))

			assert ("-NaN-0*i ok", properties.minus_ok (-Nan - 0*i, some_complex))
			assert ("NaN-0*i ok", properties.minus_ok (Nan - 0*i, some_complex))
			assert ("-Infinity-0*i ok", properties.minus_ok (Negative_infinity - 0*i, some_complex))
			assert ("Infinity-0*i ok", properties.minus_ok (Positive_infinity - 0*i, some_complex))
			assert ("-0*i ok", properties.minus_ok (-0.0*i, some_complex))
			assert ("-0-0*i ok", properties.minus_ok (-0.0 - 0*i, some_complex))
			assert ("0-0*i ok", properties.minus_ok (0 - 0*i, some_complex))
			assert ("a-0*i ok", properties.minus_ok (some_real - 0*i, some_complex))

			assert ("-NaN ok", properties.minus_ok (-Nan, some_complex))
			assert ("-NaN+0*i ok", properties.minus_ok (-Nan + 0*i, some_complex))
			assert ("NaN ok", properties.minus_ok (Nan, some_complex))
			assert ("NaN+0*i ok", properties.minus_ok (Nan + 0*i, some_complex))
			assert ("-Infinity ok", properties.minus_ok (Negative_infinity, some_complex))
			assert ("-Infinity+0*i ok", properties.minus_ok (Negative_infinity + 0*i, some_complex))
			assert ("Infinity ok", properties.minus_ok (Positive_infinity, some_complex))
			assert ("Infinity+0*i ok", properties.minus_ok (Positive_infinity + 0*i, some_complex))
			assert ("0 ok", properties.minus_ok (Zero, some_complex))
			assert ("0*i ok", properties.minus_ok (0*i, some_complex))
			assert ("-0+0*i ok", properties.minus_ok (-0.0 + 0*i, some_complex))
			assert ("0+0*i ok", properties.minus_ok (0 + 0*i, some_complex))
			assert ("a ok", properties.minus_ok (some_real, some_complex))
			assert ("a+0*i ok", properties.minus_ok (some_real + 0*i, some_complex))

			assert ("-NaN+bi ok", properties.minus_ok (-Nan + some_real*i, some_complex))
			assert ("NaN+bi ok", properties.minus_ok (Nan + some_real*i, some_complex))
			assert ("-Infinity+bi ok", properties.minus_ok (Negative_infinity + some_real*i, some_complex))
			assert ("Infinity+bi ok", properties.minus_ok (Positive_infinity + some_real*i, some_complex))
			assert ("bi ok", properties.minus_ok (some_real*i, some_complex))
			assert ("-0+bi ok", properties.minus_ok (-0.0 + some_real*i, some_complex))
			assert ("0+bi ok", properties.minus_ok (0 + some_real*i, some_complex))
			assert ("a+bi ok", properties.minus_ok (some_real + some_real*i, some_complex))

			assert ("minus_ok", properties.minus_ok (complex_to_be_tested, some_complex))
		end

	test_opposite
			-- Test {STI_COMPLEX_NUMBER}.opposite.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.opposite"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("opposite", attached (- z))
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.opposite_ok (-Nan - Nan*i))
			assert ("NaN-NaN*i ok", properties.opposite_ok (Nan - Nan*i))
			assert ("-Infinity-NaN*i ok", properties.opposite_ok (Negative_infinity - Nan*i))
			assert ("Infinity-NaN*i ok", properties.opposite_ok (Positive_infinity - Nan*i))
			assert ("-NaN*i ok", properties.opposite_ok (-Nan*i))
			assert ("-0-NaN*i ok", properties.opposite_ok (-0.0 - Nan*i))
			assert ("0-NaN*i ok", properties.opposite_ok (0 - Nan*i))
			assert ("a-NaN*i ok", properties.opposite_ok (some_real - Nan*i))

			assert ("-NaN+NaN*i ok", properties.opposite_ok (-Nan + Nan*i))
			assert ("NaN+NaN*i ok", properties.opposite_ok (Nan + Nan*i))
			assert ("-Infinity+NaN*i ok", properties.opposite_ok (Negative_infinity + Nan*i))
			assert ("Infinity+NaN*i ok", properties.opposite_ok (Positive_infinity + Nan*i))
			assert ("NaN*i ok", properties.opposite_ok (Nan*i))
			assert ("-0+NaN*i ok", properties.opposite_ok (-0.0 + Nan*i))
			assert ("0+NaN*i ok", properties.opposite_ok (0 + Nan*i))
			assert ("a+NaN*i ok", properties.opposite_ok (some_real + Nan*i))

			assert ("-NaN-Infinity*i ok", properties.opposite_ok (-Nan + Negative_infinity*i))
			assert ("NaN-Infinity*i ok", properties.opposite_ok (Nan + Negative_infinity*i))
			assert ("-Infinity-Infinity*i ok", properties.opposite_ok (Negative_infinity + Negative_infinity*i))
			assert ("Infinity-Infinity*i ok", properties.opposite_ok (Positive_infinity + Negative_infinity*i))
			assert ("-Infinity*i ok", properties.opposite_ok (Negative_infinity*i))
			assert ("-0-Infinity*i ok", properties.opposite_ok (-0.0 + Negative_infinity*i))
			assert ("0-Infinity*i ok", properties.opposite_ok (0 + Negative_infinity*i))
			assert ("a-Infinity*i ok", properties.opposite_ok (some_real + Negative_infinity*i))

			assert ("-NaN+Infinity*i ok", properties.opposite_ok (-Nan + Positive_infinity*i))
			assert ("NaN+Infinity*i ok", properties.opposite_ok (Nan + Positive_infinity*i))
			assert ("-Infinity+Infinity*i ok", properties.opposite_ok (Negative_infinity + Positive_infinity*i))
			assert ("Infinity+Infinity*i ok", properties.opposite_ok (Positive_infinity + Positive_infinity*i))
			assert ("Infinity*i ok", properties.opposite_ok (Positive_infinity*i))
			assert ("-0+Infinity*i ok", properties.opposite_ok (-0.0 + Positive_infinity*i))
			assert ("0+Infinity*i ok", properties.opposite_ok (0 + Positive_infinity*i))
			assert ("a+Infinity*i ok", properties.opposite_ok (some_real + Positive_infinity*i))

			assert ("-NaN-0*i ok", properties.opposite_ok (-Nan - 0*i))
			assert ("NaN-0*i ok", properties.opposite_ok (Nan - 0*i))
			assert ("-Infinity-0*i ok", properties.opposite_ok (Negative_infinity - 0*i))
			assert ("Infinity-0*i ok", properties.opposite_ok (Positive_infinity - 0*i))
			assert ("-0*i ok", properties.opposite_ok (-0.0*i))
			assert ("-0-0*i ok", properties.opposite_ok (-0.0 - 0*i))
			assert ("0-0*i ok", properties.opposite_ok (0 - 0*i))
			assert ("a-0*i ok", properties.opposite_ok (some_real - 0*i))

			assert ("-NaN ok", properties.opposite_ok (-Nan))
			assert ("-NaN+0*i ok", properties.opposite_ok (-Nan + 0*i))
			assert ("NaN ok", properties.opposite_ok (Nan))
			assert ("NaN+0*i ok", properties.opposite_ok (Nan + 0*i))
			assert ("-Infinity ok", properties.opposite_ok (Negative_infinity))
			assert ("-Infinity+0*i ok", properties.opposite_ok (Negative_infinity + 0*i))
			assert ("Infinity ok", properties.opposite_ok (Positive_infinity))
			assert ("Infinity+0*i ok", properties.opposite_ok (Positive_infinity + 0*i))
			assert ("0 ok", properties.opposite_ok (Zero))
			assert ("0*i ok", properties.opposite_ok (0*i))
			assert ("-0+0*i ok", properties.opposite_ok (-0.0 + 0*i))
			assert ("0+0*i ok", properties.opposite_ok (0 + 0*i))
			assert ("a ok", properties.opposite_ok (some_real))
			assert ("a+0*i ok", properties.opposite_ok (some_real + 0*i))

			assert ("-NaN+bi ok", properties.opposite_ok (-Nan + some_real*i))
			assert ("NaN+bi ok", properties.opposite_ok (Nan + some_real*i))
			assert ("-Infinity+bi ok", properties.opposite_ok (Negative_infinity + some_real*i))
			assert ("Infinity+bi ok", properties.opposite_ok (Positive_infinity + some_real*i))
			assert ("bi ok", properties.opposite_ok (some_real*i))
			assert ("-0+bi ok", properties.opposite_ok (-0.0 + some_real*i))
			assert ("0+bi ok", properties.opposite_ok (0 + some_real*i))
			assert ("a+bi ok", properties.opposite_ok (some_real + some_real*i))

			assert ("opposite_ok", properties.opposite_ok (complex_to_be_tested))
		end

	test_product
			-- Test {STI_COMPLEX_NUMBER}.product.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.product"
		local
			zs: like Quick_C.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			z1: like complex_to_be_tested
			z2: like some_complex
		do
			from
				zs := Quick_C
				from
					zt0 := zs.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # zs <= 80_003 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# C * zt0.n) print ('%N')
			until
				zs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					z1 := zs.any
					z2 := same_complex (zt.first)
					assert ("product", attached (z1 ⋅ z2))
					zt := zt.tail
				variant
					n: {like Quick_C}.natural_as_integer (zt.n)
				end
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.product_ok (-Nan - Nan*i, some_complex, some_complex))
			assert ("NaN-NaN*i ok", properties.product_ok (Nan - Nan*i, some_complex, some_complex))
			assert ("-Infinity-NaN*i ok", properties.product_ok (Negative_infinity - Nan*i, some_complex, some_complex))
			assert ("Infinity-NaN*i ok", properties.product_ok (Positive_infinity - Nan*i, some_complex, some_complex))
			assert ("-NaN*i ok", properties.product_ok (-Nan*i, some_complex, some_complex))
			assert ("-0-NaN*i ok", properties.product_ok (-0.0 - Nan*i, some_complex, some_complex))
			assert ("0-NaN*i ok", properties.product_ok (0 - Nan*i, some_complex, some_complex))
			assert ("a-NaN*i ok", properties.product_ok (some_real - Nan*i, some_complex, some_complex))

			assert ("-NaN+NaN*i ok", properties.product_ok (-Nan + Nan*i, some_complex, some_complex))
			assert ("NaN+NaN*i ok", properties.product_ok (Nan + Nan*i, some_complex, some_complex))
			assert ("-Infinity+NaN*i ok", properties.product_ok (Negative_infinity + Nan*i, some_complex, some_complex))
			assert ("Infinity+NaN*i ok", properties.product_ok (Positive_infinity + Nan*i, some_complex, some_complex))
			assert ("NaN*i ok", properties.product_ok (Nan*i, some_complex, some_complex))
			assert ("-0+NaN*i ok", properties.product_ok (-0.0 + Nan*i, some_complex, some_complex))
			assert ("0+NaN*i ok", properties.product_ok (0 + Nan*i, some_complex, some_complex))
			assert ("a+NaN*i ok", properties.product_ok (some_real + Nan*i, some_complex, some_complex))

			assert ("-NaN-Infinity*i ok", properties.product_ok (-Nan + Negative_infinity*i, some_complex, some_complex))
			assert ("NaN-Infinity*i ok", properties.product_ok (Nan + Negative_infinity*i, some_complex, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.product_ok (Negative_infinity + Negative_infinity*i, some_complex, some_complex))
			assert ("Infinity-Infinity*i ok", properties.product_ok (Positive_infinity + Negative_infinity*i, some_complex, some_complex))
			assert ("-Infinity*i ok", properties.product_ok (Negative_infinity*i, some_complex, some_complex))
			assert ("-0-Infinity*i ok", properties.product_ok (-0.0 + Negative_infinity*i, some_complex, some_complex))
			assert ("0-Infinity*i ok", properties.product_ok (0 + Negative_infinity*i, some_complex, some_complex))
			assert ("a-Infinity*i ok", properties.product_ok (some_real + Negative_infinity*i, some_complex, some_complex))

			assert ("-NaN+Infinity*i ok", properties.product_ok (-Nan + Positive_infinity*i, some_complex, some_complex))
			assert ("NaN+Infinity*i ok", properties.product_ok (Nan + Positive_infinity*i, some_complex, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.product_ok (Negative_infinity + Positive_infinity*i, some_complex, some_complex))
			assert ("Infinity+Infinity*i ok", properties.product_ok (Positive_infinity + Positive_infinity*i, some_complex, some_complex))
			assert ("Infinity*i ok", properties.product_ok (Positive_infinity*i, some_complex, some_complex))
			assert ("-0+Infinity*i ok", properties.product_ok (-0.0 + Positive_infinity*i, some_complex, some_complex))
			assert ("0+Infinity*i ok", properties.product_ok (0 + Positive_infinity*i, some_complex, some_complex))
			assert ("a+Infinity*i ok", properties.product_ok (some_real + Positive_infinity*i, some_complex, some_complex))

			assert ("-NaN-0*i ok", properties.product_ok (-Nan - 0*i, some_complex, some_complex))
			assert ("NaN-0*i ok", properties.product_ok (Nan - 0*i, some_complex, some_complex))
			assert ("-Infinity-0*i ok", properties.product_ok (Negative_infinity - 0*i, some_complex, some_complex))
			assert ("Infinity-0*i ok", properties.product_ok (Positive_infinity - 0*i, some_complex, some_complex))
			assert ("-0*i ok", properties.product_ok (-0.0*i, some_complex, some_complex))
			assert ("-0-0*i ok", properties.product_ok (-0.0 - 0*i, some_complex, some_complex))
			assert ("0-0*i ok", properties.product_ok (0 - 0*i, some_complex, some_complex))
			assert ("a-0*i ok", properties.product_ok (some_real - 0*i, some_complex, some_complex))

			assert ("-NaN ok", properties.product_ok (-Nan, some_complex, some_complex))
			assert ("-NaN+0*i ok", properties.product_ok (-Nan + 0*i, some_complex, some_complex))
			assert ("NaN ok", properties.product_ok (Nan, some_complex, some_complex))
			assert ("NaN+0*i ok", properties.product_ok (Nan + 0*i, some_complex, some_complex))
			assert ("-Infinity ok", properties.product_ok (Negative_infinity, some_complex, some_complex))
			assert ("-Infinity+0*i ok", properties.product_ok (Negative_infinity + 0*i, some_complex, some_complex))
			assert ("Infinity ok", properties.product_ok (Positive_infinity, some_complex, some_complex))
			assert ("Infinity+0*i ok", properties.product_ok (Positive_infinity + 0*i, some_complex, some_complex))
			assert ("0 ok", properties.product_ok (Zero, some_complex, some_complex))
			assert ("0*i ok", properties.product_ok (0*i, some_complex, some_complex))
			assert ("-0+0*i ok", properties.product_ok (-0.0 + 0*i, some_complex, some_complex))
			assert ("0+0*i ok", properties.product_ok (0 + 0*i, some_complex, some_complex))
			assert ("a ok", properties.product_ok (some_real, some_complex, some_complex))
			assert ("a+0*i ok", properties.product_ok (some_real + 0*i, some_complex, some_complex))

			assert ("-NaN+bi ok", properties.product_ok (-Nan + some_real*i, some_complex, some_complex))
			assert ("NaN+bi ok", properties.product_ok (Nan + some_real*i, some_complex, some_complex))
			assert ("-Infinity+bi ok", properties.product_ok (Negative_infinity + some_real*i, some_complex, some_complex))
			assert ("Infinity+bi ok", properties.product_ok (Positive_infinity + some_real*i, some_complex, some_complex))
			assert ("bi ok", properties.product_ok (some_real*i, some_complex, some_complex))
			assert ("-0+bi ok", properties.product_ok (-0.0 + some_real*i, some_complex, some_complex))
			assert ("0+bi ok", properties.product_ok (0 + some_real*i, some_complex, some_complex))
			assert ("a+bi ok", properties.product_ok (some_real + some_real*i, some_complex, some_complex))

			assert ("product_ok", properties.product_ok (complex_to_be_tested, some_complex, some_complex))
		end

	test_quotient
			-- Test {STI_COMPLEX_NUMBER}.quotient.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.quotient"
		local
			zs: like Quick_C.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			z1: like complex_to_be_tested
			z2: like some_complex
		do
			from
				zs := Quick_C
				from
					zt0 := zs.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # zs <= 76_780 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# C * zt0.n) print ('%N')
			until
				zs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					z1 := zs.any
					z2 := same_complex (zt.first)
					assert ("quotient", attached (z1 / z2))
					zt := zt.tail
				variant
					n: {like Quick_C}.natural_as_integer (zt.n)
				end
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.quotient_ok (-Nan - Nan*i, some_complex, some_complex))
			assert ("NaN-NaN*i ok", properties.quotient_ok (Nan - Nan*i, some_complex, some_complex))
			assert ("-Infinity-NaN*i ok", properties.quotient_ok (Negative_infinity - Nan*i, some_complex, some_complex))
			assert ("Infinity-NaN*i ok", properties.quotient_ok (Positive_infinity - Nan*i, some_complex, some_complex))
			assert ("-NaN*i ok", properties.quotient_ok (-Nan*i, some_complex, some_complex))
			assert ("-0-NaN*i ok", properties.quotient_ok (-0.0 - Nan*i, some_complex, some_complex))
			assert ("0-NaN*i ok", properties.quotient_ok (0 - Nan*i, some_complex, some_complex))
			assert ("a-NaN*i ok", properties.quotient_ok (some_real - Nan*i, some_complex, some_complex))

			assert ("-NaN+NaN*i ok", properties.quotient_ok (-Nan + Nan*i, some_complex, some_complex))
			assert ("NaN+NaN*i ok", properties.quotient_ok (Nan + Nan*i, some_complex, some_complex))
			assert ("-Infinity+NaN*i ok", properties.quotient_ok (Negative_infinity + Nan*i, some_complex, some_complex))
			assert ("Infinity+NaN*i ok", properties.quotient_ok (Positive_infinity + Nan*i, some_complex, some_complex))
			assert ("NaN*i ok", properties.quotient_ok (Nan*i, some_complex, some_complex))
			assert ("-0+NaN*i ok", properties.quotient_ok (-0.0 + Nan*i, some_complex, some_complex))
			assert ("0+NaN*i ok", properties.quotient_ok (0 + Nan*i, some_complex, some_complex))
			assert ("a+NaN*i ok", properties.quotient_ok (some_real + Nan*i, some_complex, some_complex))

			assert ("-NaN-Infinity*i ok", properties.quotient_ok (-Nan + Negative_infinity*i, some_complex, some_complex))
			assert ("NaN-Infinity*i ok", properties.quotient_ok (Nan + Negative_infinity*i, some_complex, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.quotient_ok (Negative_infinity + Negative_infinity*i, some_complex, some_complex))
			assert ("Infinity-Infinity*i ok", properties.quotient_ok (Positive_infinity + Negative_infinity*i, some_complex, some_complex))
			assert ("-Infinity*i ok", properties.quotient_ok (Negative_infinity*i, some_complex, some_complex))
			assert ("-0-Infinity*i ok", properties.quotient_ok (-0.0 + Negative_infinity*i, some_complex, some_complex))
			assert ("0-Infinity*i ok", properties.quotient_ok (0 + Negative_infinity*i, some_complex, some_complex))
			assert ("a-Infinity*i ok", properties.quotient_ok (some_real + Negative_infinity*i, some_complex, some_complex))

			assert ("-NaN+Infinity*i ok", properties.quotient_ok (-Nan + Positive_infinity*i, some_complex, some_complex))
			assert ("NaN+Infinity*i ok", properties.quotient_ok (Nan + Positive_infinity*i, some_complex, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.quotient_ok (Negative_infinity + Positive_infinity*i, some_complex, some_complex))
			assert ("Infinity+Infinity*i ok", properties.quotient_ok (Positive_infinity + Positive_infinity*i, some_complex, some_complex))
			assert ("Infinity*i ok", properties.quotient_ok (Positive_infinity*i, some_complex, some_complex))
			assert ("-0+Infinity*i ok", properties.quotient_ok (-0.0 + Positive_infinity*i, some_complex, some_complex))
			assert ("0+Infinity*i ok", properties.quotient_ok (0 + Positive_infinity*i, some_complex, some_complex))
			assert ("a+Infinity*i ok", properties.quotient_ok (some_real + Positive_infinity*i, some_complex, some_complex))

			assert ("-NaN-0*i ok", properties.quotient_ok (-Nan - 0*i, some_complex, some_complex))
			assert ("NaN-0*i ok", properties.quotient_ok (Nan - 0*i, some_complex, some_complex))
			assert ("-Infinity-0*i ok", properties.quotient_ok (Negative_infinity - 0*i, some_complex, some_complex))
			assert ("Infinity-0*i ok", properties.quotient_ok (Positive_infinity - 0*i, some_complex, some_complex))
			assert ("-0*i ok", properties.quotient_ok (-0.0*i, some_complex, some_complex))
			assert ("-0-0*i ok", properties.quotient_ok (-0.0 - 0*i, some_complex, some_complex))
			assert ("0-0*i ok", properties.quotient_ok (0 - 0*i, some_complex, some_complex))
			assert ("a-0*i ok", properties.quotient_ok (some_real - 0*i, some_complex, some_complex))

			assert ("-NaN ok", properties.quotient_ok (-Nan, some_complex, some_complex))
			assert ("-NaN+0*i ok", properties.quotient_ok (-Nan + 0*i, some_complex, some_complex))
			assert ("NaN ok", properties.quotient_ok (Nan, some_complex, some_complex))
			assert ("NaN+0*i ok", properties.quotient_ok (Nan + 0*i, some_complex, some_complex))
			assert ("-Infinity ok", properties.quotient_ok (Negative_infinity, some_complex, some_complex))
			assert ("-Infinity+0*i ok", properties.quotient_ok (Negative_infinity + 0*i, some_complex, some_complex))
			assert ("Infinity ok", properties.quotient_ok (Positive_infinity, some_complex, some_complex))
			assert ("Infinity+0*i ok", properties.quotient_ok (Positive_infinity + 0*i, some_complex, some_complex))
			assert ("0 ok", properties.quotient_ok (Zero, some_complex, some_complex))
			assert ("0*i ok", properties.quotient_ok (0*i, some_complex, some_complex))
			assert ("-0+0*i ok", properties.quotient_ok (-0.0 + 0*i, some_complex, some_complex))
			assert ("0+0*i ok", properties.quotient_ok (0 + 0*i, some_complex, some_complex))
			assert ("a ok", properties.quotient_ok (some_real, some_complex, some_complex))
			assert ("a+0*i ok", properties.quotient_ok (some_real + 0*i, some_complex, some_complex))

			assert ("-NaN+bi ok", properties.quotient_ok (-Nan + some_real*i, some_complex, some_complex))
			assert ("NaN+bi ok", properties.quotient_ok (Nan + some_real*i, some_complex, some_complex))
			assert ("-Infinity+bi ok", properties.quotient_ok (Negative_infinity + some_real*i, some_complex, some_complex))
			assert ("Infinity+bi ok", properties.quotient_ok (Positive_infinity + some_real*i, some_complex, some_complex))
			assert ("bi ok", properties.quotient_ok (some_real*i, some_complex, some_complex))
			assert ("-0+bi ok", properties.quotient_ok (-0.0 + some_real*i, some_complex, some_complex))
			assert ("0+bi ok", properties.quotient_ok (0 + some_real*i, some_complex, some_complex))
			assert ("a+bi ok", properties.quotient_ok (some_real + some_real*i, some_complex, some_complex))

			assert ("quotient_ok", properties.quotient_ok (complex_to_be_tested, some_complex, some_complex))
		end

	test_reciprocal
			-- Test {STI_COMPLEX_NUMBER}.reciprocal.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.reciprocal"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("reciprocal", attached z.reciprocal)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.reciprocal_ok (-Nan - Nan*i, some_complex))
			assert ("NaN-NaN*i ok", properties.reciprocal_ok (Nan - Nan*i, some_complex))
			assert ("-Infinity-NaN*i ok", properties.reciprocal_ok (Negative_infinity - Nan*i, some_complex))
			assert ("Infinity-NaN*i ok", properties.reciprocal_ok (Positive_infinity - Nan*i, some_complex))
			assert ("-NaN*i ok", properties.reciprocal_ok (-Nan*i, some_complex))
			assert ("-0-NaN*i ok", properties.reciprocal_ok (-0.0 - Nan*i, some_complex))
			assert ("0-NaN*i ok", properties.reciprocal_ok (0 - Nan*i, some_complex))
			assert ("a-NaN*i ok", properties.reciprocal_ok (some_real - Nan*i, some_complex))

			assert ("-NaN+NaN*i ok", properties.reciprocal_ok (-Nan + Nan*i, some_complex))
			assert ("NaN+NaN*i ok", properties.reciprocal_ok (Nan + Nan*i, some_complex))
			assert ("-Infinity+NaN*i ok", properties.reciprocal_ok (Negative_infinity + Nan*i, some_complex))
			assert ("Infinity+NaN*i ok", properties.reciprocal_ok (Positive_infinity + Nan*i, some_complex))
			assert ("NaN*i ok", properties.reciprocal_ok (Nan*i, some_complex))
			assert ("-0+NaN*i ok", properties.reciprocal_ok (-0.0 + Nan*i, some_complex))
			assert ("0+NaN*i ok", properties.reciprocal_ok (0 + Nan*i, some_complex))
			assert ("a+NaN*i ok", properties.reciprocal_ok (some_real + Nan*i, some_complex))

			assert ("-NaN-Infinity*i ok", properties.reciprocal_ok (-Nan + Negative_infinity*i, some_complex))
			assert ("NaN-Infinity*i ok", properties.reciprocal_ok (Nan + Negative_infinity*i, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.reciprocal_ok (Negative_infinity + Negative_infinity*i, some_complex))
			assert ("Infinity-Infinity*i ok", properties.reciprocal_ok (Positive_infinity + Negative_infinity*i, some_complex))
			assert ("-Infinity*i ok", properties.reciprocal_ok (Negative_infinity*i, some_complex))
			assert ("-0-Infinity*i ok", properties.reciprocal_ok (-0.0 + Negative_infinity*i, some_complex))
			assert ("0-Infinity*i ok", properties.reciprocal_ok (0 + Negative_infinity*i, some_complex))
			assert ("a-Infinity*i ok", properties.reciprocal_ok (some_real + Negative_infinity*i, some_complex))

			assert ("-NaN+Infinity*i ok", properties.reciprocal_ok (-Nan + Positive_infinity*i, some_complex))
			assert ("NaN+Infinity*i ok", properties.reciprocal_ok (Nan + Positive_infinity*i, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.reciprocal_ok (Negative_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity+Infinity*i ok", properties.reciprocal_ok (Positive_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity*i ok", properties.reciprocal_ok (Positive_infinity*i, some_complex))
			assert ("-0+Infinity*i ok", properties.reciprocal_ok (-0.0 + Positive_infinity*i, some_complex))
			assert ("0+Infinity*i ok", properties.reciprocal_ok (0 + Positive_infinity*i, some_complex))
			assert ("a+Infinity*i ok", properties.reciprocal_ok (some_real + Positive_infinity*i, some_complex))

			assert ("-NaN-0*i ok", properties.reciprocal_ok (-Nan - 0*i, some_complex))
			assert ("NaN-0*i ok", properties.reciprocal_ok (Nan - 0*i, some_complex))
			assert ("-Infinity-0*i ok", properties.reciprocal_ok (Negative_infinity - 0*i, some_complex))
			assert ("Infinity-0*i ok", properties.reciprocal_ok (Positive_infinity - 0*i, some_complex))
			assert ("-0*i ok", properties.reciprocal_ok (-0.0*i, some_complex))
			assert ("-0-0*i ok", properties.reciprocal_ok (-0.0 - 0*i, some_complex))
			assert ("0-0*i ok", properties.reciprocal_ok (0 - 0*i, some_complex))
			assert ("a-0*i ok", properties.reciprocal_ok (some_real - 0*i, some_complex))

			assert ("-NaN ok", properties.reciprocal_ok (-Nan, some_complex))
			assert ("-NaN+0*i ok", properties.reciprocal_ok (-Nan + 0*i, some_complex))
			assert ("NaN ok", properties.reciprocal_ok (Nan, some_complex))
			assert ("NaN+0*i ok", properties.reciprocal_ok (Nan + 0*i, some_complex))
			assert ("-Infinity ok", properties.reciprocal_ok (Negative_infinity, some_complex))
			assert ("-Infinity+0*i ok", properties.reciprocal_ok (Negative_infinity + 0*i, some_complex))
			assert ("Infinity ok", properties.reciprocal_ok (Positive_infinity, some_complex))
			assert ("Infinity+0*i ok", properties.reciprocal_ok (Positive_infinity + 0*i, some_complex))
			assert ("0 ok", properties.reciprocal_ok (Zero, some_complex))
			assert ("0*i ok", properties.reciprocal_ok (0*i, some_complex))
			assert ("-0+0*i ok", properties.reciprocal_ok (-0.0 + 0*i, some_complex))
			assert ("0+0*i ok", properties.reciprocal_ok (0 + 0*i, some_complex))
			assert ("a ok", properties.reciprocal_ok (some_real, some_complex))
			assert ("a+0*i ok", properties.reciprocal_ok (some_real + 0*i, some_complex))

			assert ("-NaN+bi ok", properties.reciprocal_ok (-Nan + some_real*i, some_complex))
			assert ("NaN+bi ok", properties.reciprocal_ok (Nan + some_real*i, some_complex))
			assert ("-Infinity+bi ok", properties.reciprocal_ok (Negative_infinity + some_real*i, some_complex))
			assert ("Infinity+bi ok", properties.reciprocal_ok (Positive_infinity + some_real*i, some_complex))
			assert ("bi ok", properties.reciprocal_ok (some_real*i, some_complex))
			assert ("-0+bi ok", properties.reciprocal_ok (-0.0 + some_real*i, some_complex))
			assert ("0+bi ok", properties.reciprocal_ok (0 + some_real*i, some_complex))
			assert ("a+bi ok", properties.reciprocal_ok (some_real + some_real*i, some_complex))

			assert ("reciprocal_ok", properties.reciprocal_ok (complex_to_be_tested, some_complex))
		rescue
			print (z) print ('%N')
		end

	test_inverse
			-- Test {STI_COMPLEX_NUMBER}.inverse.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.inverse"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("inverse", attached z.inverse)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN-NaN*i ok", properties.inverse_ok (-Nan - Nan*i, some_complex))
			assert ("NaN-NaN*i ok", properties.inverse_ok (Nan - Nan*i, some_complex))
			assert ("-Infinity-NaN*i ok", properties.inverse_ok (Negative_infinity - Nan*i, some_complex))
			assert ("Infinity-NaN*i ok", properties.inverse_ok (Positive_infinity - Nan*i, some_complex))
			assert ("-NaN*i ok", properties.inverse_ok (-Nan*i, some_complex))
			assert ("-0-NaN*i ok", properties.inverse_ok (-0.0 - Nan*i, some_complex))
			assert ("0-NaN*i ok", properties.inverse_ok (0 - Nan*i, some_complex))
			assert ("a-NaN*i ok", properties.inverse_ok (some_real - Nan*i, some_complex))

			assert ("-NaN+NaN*i ok", properties.inverse_ok (-Nan + Nan*i, some_complex))
			assert ("NaN+NaN*i ok", properties.inverse_ok (Nan + Nan*i, some_complex))
			assert ("-Infinity+NaN*i ok", properties.inverse_ok (Negative_infinity + Nan*i, some_complex))
			assert ("Infinity+NaN*i ok", properties.inverse_ok (Positive_infinity + Nan*i, some_complex))
			assert ("NaN*i ok", properties.inverse_ok (Nan*i, some_complex))
			assert ("-0+NaN*i ok", properties.inverse_ok (-0.0 + Nan*i, some_complex))
			assert ("0+NaN*i ok", properties.inverse_ok (0 + Nan*i, some_complex))
			assert ("a+NaN*i ok", properties.inverse_ok (some_real + Nan*i, some_complex))

			assert ("-NaN-Infinity*i ok", properties.inverse_ok (-Nan + Negative_infinity*i, some_complex))
			assert ("NaN-Infinity*i ok", properties.inverse_ok (Nan + Negative_infinity*i, some_complex))
			assert ("-Infinity-Infinity*i ok", properties.inverse_ok (Negative_infinity + Negative_infinity*i, some_complex))
			assert ("Infinity-Infinity*i ok", properties.inverse_ok (Positive_infinity + Negative_infinity*i, some_complex))
			assert ("-Infinity*i ok", properties.inverse_ok (Negative_infinity*i, some_complex))
			assert ("-0-Infinity*i ok", properties.inverse_ok (-0.0 + Negative_infinity*i, some_complex))
			assert ("0-Infinity*i ok", properties.inverse_ok (0 + Negative_infinity*i, some_complex))
			assert ("a-Infinity*i ok", properties.inverse_ok (some_real + Negative_infinity*i, some_complex))

			assert ("-NaN+Infinity*i ok", properties.inverse_ok (-Nan + Positive_infinity*i, some_complex))
			assert ("NaN+Infinity*i ok", properties.inverse_ok (Nan + Positive_infinity*i, some_complex))
			assert ("-Infinity+Infinity*i ok", properties.inverse_ok (Negative_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity+Infinity*i ok", properties.inverse_ok (Positive_infinity + Positive_infinity*i, some_complex))
			assert ("Infinity*i ok", properties.inverse_ok (Positive_infinity*i, some_complex))
			assert ("-0+Infinity*i ok", properties.inverse_ok (-0.0 + Positive_infinity*i, some_complex))
			assert ("0+Infinity*i ok", properties.inverse_ok (0 + Positive_infinity*i, some_complex))
			assert ("a+Infinity*i ok", properties.inverse_ok (some_real + Positive_infinity*i, some_complex))

			assert ("-NaN-0*i ok", properties.inverse_ok (-Nan - 0*i, some_complex))
			assert ("NaN-0*i ok", properties.inverse_ok (Nan - 0*i, some_complex))
			assert ("-Infinity-0*i ok", properties.inverse_ok (Negative_infinity - 0*i, some_complex))
			assert ("Infinity-0*i ok", properties.inverse_ok (Positive_infinity - 0*i, some_complex))
			assert ("-0*i ok", properties.inverse_ok (-0.0*i, some_complex))
			assert ("-0-0*i ok", properties.inverse_ok (-0.0 - 0*i, some_complex))
			assert ("0-0*i ok", properties.inverse_ok (0 - 0*i, some_complex))
			assert ("a-0*i ok", properties.inverse_ok (some_real - 0*i, some_complex))

			assert ("-NaN ok", properties.inverse_ok (-Nan, some_complex))
			assert ("-NaN+0*i ok", properties.inverse_ok (-Nan + 0*i, some_complex))
			assert ("NaN ok", properties.inverse_ok (Nan, some_complex))
			assert ("NaN+0*i ok", properties.inverse_ok (Nan + 0*i, some_complex))
			assert ("-Infinity ok", properties.inverse_ok (Negative_infinity, some_complex))
			assert ("-Infinity+0*i ok", properties.inverse_ok (Negative_infinity + 0*i, some_complex))
			assert ("Infinity ok", properties.inverse_ok (Positive_infinity, some_complex))
			assert ("Infinity+0*i ok", properties.inverse_ok (Positive_infinity + 0*i, some_complex))
			assert ("0 ok", properties.inverse_ok (Zero, some_complex))
			assert ("0*i ok", properties.inverse_ok (0*i, some_complex))
			assert ("-0+0*i ok", properties.inverse_ok (-0.0 + 0*i, some_complex))
			assert ("0+0*i ok", properties.inverse_ok (0 + 0*i, some_complex))
			assert ("a ok", properties.inverse_ok (some_real, some_complex))
			assert ("a+0*i ok", properties.inverse_ok (some_real + 0*i, some_complex))

			assert ("-NaN+bi ok", properties.inverse_ok (-Nan + some_real*i, some_complex))
			assert ("NaN+bi ok", properties.inverse_ok (Nan + some_real*i, some_complex))
			assert ("-Infinity+bi ok", properties.inverse_ok (Negative_infinity + some_real*i, some_complex))
			assert ("Infinity+bi ok", properties.inverse_ok (Positive_infinity + some_real*i, some_complex))
			assert ("bi ok", properties.inverse_ok (some_real*i, some_complex))
			assert ("-0+bi ok", properties.inverse_ok (-0.0 + some_real*i, some_complex))
			assert ("0+bi ok", properties.inverse_ok (0 + some_real*i, some_complex))
			assert ("a+bi ok", properties.inverse_ok (some_real + some_real*i, some_complex))

			assert ("inverse_ok", properties.inverse_ok (complex_to_be_tested, some_complex))
		rescue
			print (z) print ('%N')
		end

	test_pair_inverse
			-- Test {STI_COMPLEX_NUMBER}.pair_inverse.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.pair_inverse"
		local
			z: like complex_to_be_tested
		do
			z := complex_to_be_tested
			assert ("pair_inverse", attached z.pair_inverse)
			assert
				("pair_inverse_ok", homogeneous_pair_properties.inverse_ok (z))
		end

feature -- Test routines (Conversion)

	test_as_singleton
			-- Test {STI_COMPLEX_NUMBER}.as_singleton.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.as_singleton"
		local
			z: like complex_to_be_tested
		do
			z := complex_to_be_tested
			assert ("as_singleton", attached z.as_singleton)
			assert (
				"as_singleton_ok",
				homogeneous_pair_properties.as_singleton_ok (z)
				)
		end

	test_as_tuple
			-- Test {STI_COMPLEX_NUMBER}.as_tuple.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.as_tuple"
		local
			z: like complex_to_be_tested
		do
			z := complex_to_be_tested
			assert ("as_tuple", attached z.as_tuple)
			assert ("as_tuple_ok", homogeneous_pair_properties.as_tuple_ok (z))
			assert (
				"complex_to_sti_n_tuple", attached
				complex_to_sti_n_tuple (complex_to_be_tested)
				)
			assert (
				"complex_to_sts_n_tuple",
				attached complex_to_sts_n_tuple (complex_to_be_tested)
				)
			assert (
				"complex_to_numeric_sti_n_tuple",
				attached complex_to_numeric_sti_n_tuple (complex_to_be_tested)
				)
			assert (
				"complex_to_numeric_sts_n_tuple",
				attached complex_to_numeric_sts_n_tuple (complex_to_be_tested)
				)
		end

	test_as_ordered_pair
			-- Test {STI_COMPLEX_NUMBER}.as_ordered_pair.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.as_ordered_pair"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("as_ordered_pair", attached z.as_ordered_pair)
				assert (
					"complex_to_sti_homogeneous_real_pair",
					attached complex_to_sti_homogeneous_real_pair (z)
					)
				assert (
					"complex_to_sti_homogeneous_numeric_pair",
					attached complex_to_sti_homogeneous_numeric_pair (z)
					)
				assert (
					"complex_to_sti_real_pair",
					attached complex_to_sti_real_pair (z)
					)
				assert (
					"complex_to_sti_numeric_pair",
					attached complex_to_sti_numeric_pair (z)
					)
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end
		end

feature -- Test routines (Math)

	test_pi
			-- Test {STS_COMPLEX_NUMBER}.pi.
			-- Test {STI_COMPLEX_NUMBER}.pi.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi"
			testing: "covers/{STI_COMPLEX_NUMBER}.pi"
		do
			assert ("pi", attached complex_to_be_tested.Pi)
		end

	test_pi_2
			-- Test {STS_COMPLEX_NUMBER}.pi_2.
			-- Test {STI_COMPLEX_NUMBER}.pi_2.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi_2"
			testing: "covers/{STI_COMPLEX_NUMBER}.pi_2"
		do
			assert ("pi_2", attached complex_to_be_tested.Pi_2)
		end

	test_pi_4
			-- Test {STS_COMPLEX_NUMBER}.pi_4.
			-- Test {STI_COMPLEX_NUMBER}.pi_4.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi_4"
			testing: "covers/{STI_COMPLEX_NUMBER}.pi_4"
		do
			assert ("pi_4", attached complex_to_be_tested.Pi_4)
		end

	test_value_is_infinite
			-- Test {STI_COMPLEX_NUMBER}.value_is_infinite.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.value_is_infinite"
		local
			v: like some_native_real
		do
			v := Negative_infinity
			assert ("Negative_infinity", complex_to_be_tested.value_is_infinite (v))

			v := Positive_infinity
			assert ("Positive_infinity", complex_to_be_tested.value_is_infinite (v))

			from
				v := some_native_real
			until
				not (v.is_negative_infinity or v.is_positive_infinity)
			loop
				v := some_native_real
			end
			assert ("finite", not complex_to_be_tested.value_is_infinite (v))

			assert ("value_is_infinite", complex_to_be_tested.value_is_infinite (some_native_real) implies True)
		end

	test_value_is_finite
			-- Test {STI_COMPLEX_NUMBER}.value_is_finite.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.value_is_finite"
		local
			v: like some_native_real
		do
			v := Nan
			assert ("NaN", not complex_to_be_tested.value_is_finite (v))

			v := Negative_infinity
			assert ("Negative_infinity", not complex_to_be_tested.value_is_finite (v))

			v := Positive_infinity
			assert ("Positive_infinity", not complex_to_be_tested.value_is_finite (v))

			from
				v := some_native_real
			until
				not (v.is_nan or v.is_negative_infinity or v.is_positive_infinity)
			loop
				v := some_native_real
			end
			assert ("finite", complex_to_be_tested.value_is_finite (v))

			assert ("value_is_finite", complex_to_be_tested.value_is_finite (some_native_real) implies True)
		end

	test_hypotenuse
			-- Test {STI_COMPLEX_NUMBER}.hypotenuse.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.hypotenuse"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("hypotenuse", attached z.hypotenuse (z.a, z.b))
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN, -NaN", attached {like complex_to_be_tested}.hypotenuse (-Nan, -Nan))
			assert ("NaN, -NaN", attached {like complex_to_be_tested}.hypotenuse (Nan, -Nan))
			assert ("-Infinity, -NaN", attached {like complex_to_be_tested}.hypotenuse (Negative_infinity, -Nan))
			assert ("Infinity, -NaN", attached {like complex_to_be_tested}.hypotenuse (Positive_infinity, -Nan))
			assert ("0, -NaN", attached {like complex_to_be_tested}.hypotenuse (Zero.a, -Nan))
			assert ("-0, -NaN", attached {like complex_to_be_tested}.hypotenuse (-Zero.a, -Nan))
			assert ("0, -NaN", attached {like complex_to_be_tested}.hypotenuse (Zero.a, -Nan))
			assert ("a, -NaN", attached {like complex_to_be_tested}.hypotenuse (some_real, -Nan))

			assert ("-NaN, NaN", attached {like complex_to_be_tested}.hypotenuse (-Nan, Nan))
			assert ("NaN, NaN", attached {like complex_to_be_tested}.hypotenuse (Nan, Nan))
			assert ("-Infinity, NaN", attached {like complex_to_be_tested}.hypotenuse (Negative_infinity, Nan))
			assert ("Infinity, NaN", attached {like complex_to_be_tested}.hypotenuse (Positive_infinity, Nan))
			assert ("NaN", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Nan))
			assert ("-0, NaN", attached {like complex_to_be_tested}.hypotenuse (-Zero.a, Nan))
			assert ("0, NaN", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Nan))
			assert ("a, NaN", attached {like complex_to_be_tested}.hypotenuse (some_real, Nan))

			assert ("-NaN, -Infinity", attached {like complex_to_be_tested}.hypotenuse (-Nan, Negative_infinity))
			assert ("NaN, -Infinity", attached {like complex_to_be_tested}.hypotenuse (Nan, Negative_infinity))
			assert ("-Infinity, -Infinity", attached {like complex_to_be_tested}.hypotenuse (Negative_infinity, Negative_infinity))
			assert ("Infinity, -Infinity", attached {like complex_to_be_tested}.hypotenuse (Positive_infinity, Negative_infinity))
			assert ("0, -Infinity", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Negative_infinity))
			assert ("-0, -Infinity", attached {like complex_to_be_tested}.hypotenuse (-Zero.a, Negative_infinity))
			assert ("0, -Infinity", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Negative_infinity))
			assert ("a, -Infinity", attached {like complex_to_be_tested}.hypotenuse (some_real, Negative_infinity))

			assert ("-NaN, Infinity", attached {like complex_to_be_tested}.hypotenuse (-Nan, Positive_infinity))
			assert ("NaN, Infinity", attached {like complex_to_be_tested}.hypotenuse (Nan, Positive_infinity))
			assert ("-Infinity, Infinity", attached {like complex_to_be_tested}.hypotenuse (Negative_infinity, Positive_infinity))
			assert ("Infinity, Infinity", attached {like complex_to_be_tested}.hypotenuse (Positive_infinity, Positive_infinity))
			assert ("0, Infinity", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Positive_infinity))
			assert ("-0, Infinity", attached {like complex_to_be_tested}.hypotenuse (-Zero.a, Positive_infinity))
			assert ("0, Infinity", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Positive_infinity))
			assert ("a, Infinity", attached {like complex_to_be_tested}.hypotenuse (some_real, Positive_infinity))

			assert ("-NaN, -0", attached {like complex_to_be_tested}.hypotenuse (-Nan, -Zero.a))
			assert ("NaN, -0", attached {like complex_to_be_tested}.hypotenuse (Nan, -Zero.a))
			assert ("-Infinity, -0", attached {like complex_to_be_tested}.hypotenuse (Negative_infinity, -Zero.a))
			assert ("Infinity, -0", attached {like complex_to_be_tested}.hypotenuse (Positive_infinity, -Zero.a))
			assert ("0, -0", attached {like complex_to_be_tested}.hypotenuse (Zero.a, -Zero.a))
			assert ("-0, -0", attached {like complex_to_be_tested}.hypotenuse (-Zero.a, -Zero.a))
			assert ("0, -0", attached {like complex_to_be_tested}.hypotenuse (Zero.a, -Zero.a))
			assert ("a, -0", attached {like complex_to_be_tested}.hypotenuse (some_real, -Zero.a))

			assert ("-NaN, 0", attached {like complex_to_be_tested}.hypotenuse (-Nan, Zero.a))
			assert ("-NaN, 0", attached {like complex_to_be_tested}.hypotenuse (-Nan, Zero.a))
			assert ("NaN, 0", attached {like complex_to_be_tested}.hypotenuse (Nan, Zero.a))
			assert ("NaN, 0", attached {like complex_to_be_tested}.hypotenuse (Nan, Zero.a))
			assert ("-Infinity, 0", attached {like complex_to_be_tested}.hypotenuse (Negative_infinity, Zero.a))
			assert ("-Infinity, 0", attached {like complex_to_be_tested}.hypotenuse (Negative_infinity, Zero.a))
			assert ("Infinity, 0", attached {like complex_to_be_tested}.hypotenuse (Positive_infinity, Zero.a))
			assert ("Infinity, 0", attached {like complex_to_be_tested}.hypotenuse (Positive_infinity, Zero.a))
			assert ("0, 0", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Zero.a))
			assert ("0, 0", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Zero.a))
			assert ("-0, 0", attached {like complex_to_be_tested}.hypotenuse (-Zero.a, Zero.a))
			assert ("0, 0", attached {like complex_to_be_tested}.hypotenuse (Zero.a, Zero.a))
			assert ("a, 0", attached {like complex_to_be_tested}.hypotenuse (some_real, Zero.a))
			assert ("a, 0", attached {like complex_to_be_tested}.hypotenuse (some_real, Zero.a))

			assert ("-NaN, b", attached {like complex_to_be_tested}.hypotenuse (-Nan, some_real))
			assert ("NaN, b", attached {like complex_to_be_tested}.hypotenuse (Nan, some_real))
			assert ("-Infinity, b", attached {like complex_to_be_tested}.hypotenuse (Negative_infinity, some_real))
			assert ("Infinity, b", attached {like complex_to_be_tested}.hypotenuse (Positive_infinity, some_real))
			assert ("0, bi", attached {like complex_to_be_tested}.hypotenuse (Zero.a, some_real))
			assert ("-0, b", attached {like complex_to_be_tested}.hypotenuse (-Zero.a, some_real))
			assert ("0, b", attached {like complex_to_be_tested}.hypotenuse (Zero.a, some_real))
			assert ("a, b", attached {like complex_to_be_tested}.hypotenuse (some_real, some_real))
		end

	test_arc_tangent_yx
			-- Test {STI_COMPLEX_NUMBER}.arc_tangent_yx.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.arc_tangent_yx"
		local
			zs: like Quick_C.subset_anchor
			z: like complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("arc_tangent_yx", attached z.arc_tangent_yx (z.b, z.a))
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN, -NaN", attached {like complex_to_be_tested}.arc_tangent_yx (-Nan, -Nan))
			assert ("NaN, -NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Nan, -Nan))
			assert ("-Infinity, -NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Negative_infinity, -Nan))
			assert ("Infinity, -NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Positive_infinity, -Nan))
			assert ("0, -NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, -Nan))
			assert ("-0, -NaN", attached {like complex_to_be_tested}.arc_tangent_yx (-Zero.a, -Nan))
			assert ("0, -NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, -Nan))
			assert ("a, -NaN", attached {like complex_to_be_tested}.arc_tangent_yx (some_real, -Nan))

			assert ("-NaN, NaN", attached {like complex_to_be_tested}.arc_tangent_yx (-Nan, Nan))
			assert ("NaN, NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Nan, Nan))
			assert ("-Infinity, NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Negative_infinity, Nan))
			assert ("Infinity, NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Positive_infinity, Nan))
			assert ("NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Nan))
			assert ("-0, NaN", attached {like complex_to_be_tested}.arc_tangent_yx (-Zero.a, Nan))
			assert ("0, NaN", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Nan))
			assert ("a, NaN", attached {like complex_to_be_tested}.arc_tangent_yx (some_real, Nan))

			assert ("-NaN, -Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (-Nan, Negative_infinity))
			assert ("NaN, -Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Nan, Negative_infinity))
			assert ("-Infinity, -Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Negative_infinity, Negative_infinity))
			assert ("Infinity, -Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Positive_infinity, Negative_infinity))
			assert ("0, -Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Negative_infinity))
			assert ("-0, -Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (-Zero.a, Negative_infinity))
			assert ("0, -Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Negative_infinity))
			assert ("a, -Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (some_real, Negative_infinity))

			assert ("-NaN, Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (-Nan, Positive_infinity))
			assert ("NaN, Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Nan, Positive_infinity))
			assert ("-Infinity, Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Negative_infinity, Positive_infinity))
			assert ("Infinity, Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Positive_infinity, Positive_infinity))
			assert ("0, Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Positive_infinity))
			assert ("-0, Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (-Zero.a, Positive_infinity))
			assert ("0, Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Positive_infinity))
			assert ("a, Infinity", attached {like complex_to_be_tested}.arc_tangent_yx (some_real, Positive_infinity))

			assert ("-NaN, -0", attached {like complex_to_be_tested}.arc_tangent_yx (-Nan, -Zero.a))
			assert ("NaN, -0", attached {like complex_to_be_tested}.arc_tangent_yx (Nan, -Zero.a))
			assert ("-Infinity, -0", attached {like complex_to_be_tested}.arc_tangent_yx (Negative_infinity, -Zero.a))
			assert ("Infinity, -0", attached {like complex_to_be_tested}.arc_tangent_yx (Positive_infinity, -Zero.a))
			assert ("0, -0", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, -Zero.a))
			assert ("-0, -0", attached {like complex_to_be_tested}.arc_tangent_yx (-Zero.a, -Zero.a))
			assert ("0, -0", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, -Zero.a))
			assert ("a, -0", attached {like complex_to_be_tested}.arc_tangent_yx (some_real, -Zero.a))

			assert ("-NaN, 0", attached {like complex_to_be_tested}.arc_tangent_yx (-Nan, Zero.a))
			assert ("-NaN, 0", attached {like complex_to_be_tested}.arc_tangent_yx (-Nan, Zero.a))
			assert ("NaN, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Nan, Zero.a))
			assert ("NaN, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Nan, Zero.a))
			assert ("-Infinity, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Negative_infinity, Zero.a))
			assert ("-Infinity, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Negative_infinity, Zero.a))
			assert ("Infinity, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Positive_infinity, Zero.a))
			assert ("Infinity, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Positive_infinity, Zero.a))
			assert ("0, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Zero.a))
			assert ("0, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Zero.a))
			assert ("-0, 0", attached {like complex_to_be_tested}.arc_tangent_yx (-Zero.a, Zero.a))
			assert ("0, 0", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, Zero.a))
			assert ("a, 0", attached {like complex_to_be_tested}.arc_tangent_yx (some_real, Zero.a))
			assert ("a, 0", attached {like complex_to_be_tested}.arc_tangent_yx (some_real, Zero.a))

			assert ("-NaN, b", attached {like complex_to_be_tested}.arc_tangent_yx (-Nan, some_real))
			assert ("NaN, b", attached {like complex_to_be_tested}.arc_tangent_yx (Nan, some_real))
			assert ("-Infinity, b", attached {like complex_to_be_tested}.arc_tangent_yx (Negative_infinity, some_real))
			assert ("Infinity, b", attached {like complex_to_be_tested}.arc_tangent_yx (Positive_infinity, some_real))
			assert ("0, bi", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, some_real))
			assert ("-0, b", attached {like complex_to_be_tested}.arc_tangent_yx (-Zero.a, some_real))
			assert ("0, b", attached {like complex_to_be_tested}.arc_tangent_yx (Zero.a, some_real))
			assert ("a, b", attached {like complex_to_be_tested}.arc_tangent_yx (some_real, some_real))
		end

	test_arc_tangent
			-- Test {STS_COMPLEX_NUMBER}.arc_tangent.
			-- Test {STI_COMPLEX_NUMBER}.arc_tangent.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.arc_tangent"
			testing: "covers/{STI_COMPLEX_NUMBER}.arc_tangent"
		do
			assert ("-NaN / -NaN", attached {like complex_to_be_tested}.arc_tangent (-Nan / -Nan))
			assert ("NaN / -NaN", attached {like complex_to_be_tested}.arc_tangent (Nan / -Nan))
			assert ("-Infinity / -NaN", attached {like complex_to_be_tested}.arc_tangent (Negative_infinity / -Nan))
			assert ("Infinity / -NaN", attached {like complex_to_be_tested}.arc_tangent (Positive_infinity / -Nan))
			assert ("0 / -NaN", attached {like complex_to_be_tested}.arc_tangent (Zero.a / -Nan))
			assert ("-0 / -NaN", attached {like complex_to_be_tested}.arc_tangent (-Zero.a / -Nan))
			assert ("0 / -NaN", attached {like complex_to_be_tested}.arc_tangent (Zero.a / -Nan))
			assert ("a / -NaN", attached {like complex_to_be_tested}.arc_tangent (some_real.value / -Nan))

			assert ("-NaN / NaN", attached {like complex_to_be_tested}.arc_tangent (-Nan / Nan))
			assert ("NaN / NaN", attached {like complex_to_be_tested}.arc_tangent (Nan / Nan))
			assert ("-Infinity / NaN", attached {like complex_to_be_tested}.arc_tangent (Negative_infinity / Nan))
			assert ("Infinity / NaN", attached {like complex_to_be_tested}.arc_tangent (Positive_infinity / Nan))
			assert ("NaN", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Nan))
			assert ("-0 / NaN", attached {like complex_to_be_tested}.arc_tangent (-Zero.a / Nan))
			assert ("0 / NaN", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Nan))
			assert ("a / NaN", attached {like complex_to_be_tested}.arc_tangent (some_real.value / Nan))

			assert ("-NaN / -Infinity", attached {like complex_to_be_tested}.arc_tangent (-Nan / Negative_infinity))
			assert ("NaN / -Infinity", attached {like complex_to_be_tested}.arc_tangent (Nan / Negative_infinity))
			assert ("-Infinity / -Infinity", attached {like complex_to_be_tested}.arc_tangent (Negative_infinity / Negative_infinity))
			assert ("Infinity / -Infinity", attached {like complex_to_be_tested}.arc_tangent (Positive_infinity / Negative_infinity))
			assert ("0 / -Infinity", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Negative_infinity))
			assert ("-0 / -Infinity", attached {like complex_to_be_tested}.arc_tangent (-Zero.a / Negative_infinity))
			assert ("0 / -Infinity", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Negative_infinity))
			assert ("a / -Infinity", attached {like complex_to_be_tested}.arc_tangent (some_real.value / Negative_infinity))

			assert ("-NaN / Infinity", attached {like complex_to_be_tested}.arc_tangent (-Nan / Positive_infinity))
			assert ("NaN / Infinity", attached {like complex_to_be_tested}.arc_tangent (Nan / Positive_infinity))
			assert ("-Infinity / Infinity", attached {like complex_to_be_tested}.arc_tangent (Negative_infinity / Positive_infinity))
			assert ("Infinity / Infinity", attached {like complex_to_be_tested}.arc_tangent (Positive_infinity / Positive_infinity))
			assert ("0 / Infinity", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Positive_infinity))
			assert ("-0 / Infinity", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Positive_infinity))
			assert ("0 / Infinity", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Positive_infinity))
			assert ("a / Infinity", attached {like complex_to_be_tested}.arc_tangent (some_real.value / Positive_infinity))

			assert ("-NaN / -0", attached {like complex_to_be_tested}.arc_tangent (-Nan / -Zero.a))
			assert ("NaN / -0", attached {like complex_to_be_tested}.arc_tangent (Nan / -Zero.a))
			assert ("-Infinity / -0", attached {like complex_to_be_tested}.arc_tangent (Negative_infinity / -Zero.a))
			assert ("Infinity / -0", attached {like complex_to_be_tested}.arc_tangent (Positive_infinity / -Zero.a))
			assert ("0 / -0", attached {like complex_to_be_tested}.arc_tangent (Zero.a / -Zero.a))
			assert ("-0 / -0", attached {like complex_to_be_tested}.arc_tangent (-Zero.a / -Zero.a))
			assert ("0 / -0", attached {like complex_to_be_tested}.arc_tangent (Zero.a / -Zero.a))
			assert ("a / -0", attached {like complex_to_be_tested}.arc_tangent (some_real.value / -Zero.a))

			assert ("-NaN / 0", attached {like complex_to_be_tested}.arc_tangent (-Nan / Zero.a))
			assert ("-NaN / 0", attached {like complex_to_be_tested}.arc_tangent (-Nan / Zero.a))
			assert ("NaN / 0", attached {like complex_to_be_tested}.arc_tangent (Nan / Zero.a))
			assert ("NaN / 0", attached {like complex_to_be_tested}.arc_tangent (Nan / Zero.a))
			assert ("-Infinity / 0", attached {like complex_to_be_tested}.arc_tangent (Negative_infinity / Zero.a))
			assert ("-Infinity / 0", attached {like complex_to_be_tested}.arc_tangent (Negative_infinity / Zero.a))
			assert ("Infinity / 0", attached {like complex_to_be_tested}.arc_tangent (Positive_infinity / Zero.a))
			assert ("Infinity / 0", attached {like complex_to_be_tested}.arc_tangent (Positive_infinity / Zero.a))
			assert ("0 / 0", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Zero.a))
			assert ("0 / 0", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Zero.a))
			assert ("-0 / 0", attached {like complex_to_be_tested}.arc_tangent (-Zero.a / Zero.a))
			assert ("0 / 0", attached {like complex_to_be_tested}.arc_tangent (Zero.a / Zero.a))
			assert ("a / 0", attached {like complex_to_be_tested}.arc_tangent (some_real.value / Zero.a))
			assert ("a / 0", attached {like complex_to_be_tested}.arc_tangent (some_real.value / Zero.a))

			assert ("-NaN / b", attached {like complex_to_be_tested}.arc_tangent (-Nan / some_real))
			assert ("NaN / b", attached {like complex_to_be_tested}.arc_tangent (Nan / some_real))
			assert ("-Infinity / b", attached {like complex_to_be_tested}.arc_tangent (Negative_infinity / some_real))
			assert ("Infinity / b", attached {like complex_to_be_tested}.arc_tangent (Positive_infinity / some_real))
			assert ("0 / bi", attached {like complex_to_be_tested}.arc_tangent (Zero.a / some_real))
			assert ("-0 / b", attached {like complex_to_be_tested}.arc_tangent (-Zero.a / some_real))
			assert ("0 / b", attached {like complex_to_be_tested}.arc_tangent (Zero.a / some_real))
			assert ("a / b", attached {like complex_to_be_tested}.arc_tangent (some_real.value / some_real.value))
		end

	test_value_logb
			-- Test {STS_COMPLEX_NUMBER}.value_logb.
			-- Test {STI_COMPLEX_NUMBER}.value_logb.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.value_logb"
			testing: "covers/{STI_COMPLEX_NUMBER}.value_logb"
		do
			assert ("-NaN", attached complex_to_be_tested.value_logb (-Nan))
			assert ("NaN", attached complex_to_be_tested.value_logb (Nan))
			assert ("-Infinity", attached complex_to_be_tested.value_logb (Negative_infinity))
			assert ("Infinity", attached complex_to_be_tested.value_logb (Positive_infinity))
			assert ("-0", attached complex_to_be_tested.value_logb (-0.0))
			assert ("0", attached complex_to_be_tested.value_logb (0))
			assert ("x", attached complex_to_be_tested.value_logb (some_native_real))
		end

	test_value_sign_bit
			-- Test {STS_COMPLEX_NUMBER}.value_sign_bit.
			-- Test {STI_COMPLEX_NUMBER}.value_sign_bit.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.value_sign_bit"
			testing: "covers/{STI_COMPLEX_NUMBER}.value_sign_bit"
		local
			v: like some_native_real
		do
			v := - Nan
			assert ("signed nan", complex_to_be_tested.value_sign_bit (v) = 1)

			v := Nan
			assert ("unsigned nan", complex_to_be_tested.value_sign_bit (v) = 0)

			from
				v := some_native_real
			until
				not v.is_nan and v < 0
			loop
				v := some_native_real
			end
			assert ("negative", complex_to_be_tested.value_sign_bit (v) = 1)

			v := 0.0
			assert ("negative zero", complex_to_be_tested.value_sign_bit (- v) = 1)
			assert ("positive zero", complex_to_be_tested.value_sign_bit (v) = 0)

			from
				v := some_native_real
			until
				0 < v
			loop
				v := some_native_real
			end
			assert ("positive", complex_to_be_tested.value_sign_bit (v) = 0)

			assert ("-NaN", attached complex_to_be_tested.value_sign_bit (-Nan))
			assert ("NaN", attached complex_to_be_tested.value_sign_bit (Nan))
			assert ("-Infinity", attached complex_to_be_tested.value_sign_bit (Negative_infinity))
			assert ("Infinity", attached complex_to_be_tested.value_sign_bit (Positive_infinity))
			assert ("-0", attached complex_to_be_tested.value_sign_bit (-0.0))
			assert ("0", attached complex_to_be_tested.value_sign_bit (0))
			assert ("x", attached complex_to_be_tested.value_sign_bit (some_native_real))
		end

feature -- Test routines (Factory)

	test_real_from_value
			-- Test {STI_COMPLEX_NUMBER}.real_from_value.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.real_from_value"
		do
			assert ("-NaN", attached complex_to_be_tested.real_from_value (-Nan))
			assert ("NaN", attached complex_to_be_tested.real_from_value (Nan))
			assert ("-Infinity", attached complex_to_be_tested.real_from_value (Negative_infinity))
			assert ("Infinity", attached complex_to_be_tested.real_from_value (Positive_infinity))
			assert ("-0", attached complex_to_be_tested.real_from_value (-0.0))
			assert ("0", attached complex_to_be_tested.real_from_value (0))
			assert ("x", attached complex_to_be_tested.real_from_value (some_native_real))
		end

	test_real_from_double_value
			-- Test {STI_COMPLEX_NUMBER}.real_from_double_value.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.real_from_double_value"
		do
			assert ("-NaN", attached complex_to_be_tested.real_from_double_value (-Nan.value))
			assert ("NaN", attached complex_to_be_tested.real_from_double_value (Nan.value))
			assert ("-Infinity", attached complex_to_be_tested.real_from_double_value (Negative_infinity.value))
			assert ("Infinity", attached complex_to_be_tested.real_from_double_value (Positive_infinity.value))
			assert ("-0", attached complex_to_be_tested.real_from_double_value (-0.0))
			assert ("0", attached complex_to_be_tested.real_from_double_value (0))
			assert ("x", attached complex_to_be_tested.real_from_double_value (some_native_double_real))
		end

	test_native_real_from_double_value
			-- Test {STS_COMPLEX_NUMBER}.native_real_from_double_value.
			-- Test {STI_COMPLEX_NUMBER}.native_real_from_double_value.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.native_real_from_double_value"
			testing: "covers/{STI_COMPLEX_NUMBER}.native_real_from_double_value"
		do
			assert ("-NaN", attached complex_to_be_tested.native_real_from_double_value (-Nan.value))
			assert ("NaN", attached complex_to_be_tested.native_real_from_double_value (Nan.value))
			assert ("-Infinity", attached complex_to_be_tested.native_real_from_double_value (Negative_infinity.value))
			assert ("Infinity", attached complex_to_be_tested.native_real_from_double_value (Positive_infinity.value))
			assert ("-0", attached complex_to_be_tested.native_real_from_double_value (-0.0))
			assert ("0", attached complex_to_be_tested.native_real_from_double_value (0))
			assert ("x", attached complex_to_be_tested.native_real_from_double_value (some_native_double_real))
		end

	test_converted_real
			-- Test {STI_COMPLEX_NUMBER}.converted_real.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.converted_real"
		do
			assert ("converted_real", attached complex_to_be_tested.converted_real (some_real))

			assert ("-NaN ok", attached Properties.converted_real_ok (-Nan))
			assert ("NaN ok", attached Properties.converted_real_ok (Nan))
			assert ("-Infinity ok", attached Properties.converted_real_ok (Negative_infinity))
			assert ("Infinity ok", attached Properties.converted_real_ok (Positive_infinity))
			assert ("-0 ok", attached Properties.converted_real_ok (-Zero.a))
			assert ("0 ok", attached Properties.converted_real_ok (Zero.a))

			assert ("converted_real ok", Properties.converted_real_ok (some_real))
		end

feature -- Test routines (Implementation)

	test_value_is_negative_zero
			-- Test {STI_COMPLEX_NUMBER}.value_is_negative_zero.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.value_is_negative_zero"
		do
			assert ("-0", exposed_complex_to_be_tested.value_is_negative_zero (-0.0))
			assert ("+ 0", not exposed_complex_to_be_tested.value_is_negative_zero (0.0))
			assert ("value_is_negative_zero", exposed_complex_to_be_tested.value_is_negative_zero (some_native_real) implies True)
		end

	test_value_is_positive_zero
			-- Test {STI_COMPLEX_NUMBER}.value_is_positive_zero.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.value_is_positive_zero"
		do
			assert ("-0", not exposed_complex_to_be_tested.value_is_positive_zero (-0.0))
			assert ("+ 0", exposed_complex_to_be_tested.value_is_positive_zero (0.0))
			assert ("value_is_positive_zero", exposed_complex_to_be_tested.value_is_positive_zero (some_native_real) implies True)
		end

	test_c_hypot
			-- Test {STI_COMPLEX_NUMBER}.c_hypot.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.c_hypot"
		local
			zs: like Quick_C.subset_anchor
			z: like exposed_complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("c_hypot", attached z.c_hypot (z.a, z.b))
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN, -NaN", attached {like exposed_complex_to_be_tested}.c_hypot (-Nan, -Nan))
			assert ("NaN, -NaN", attached {like exposed_complex_to_be_tested}.c_hypot (Nan, -Nan))
			assert ("-Infinity, -NaN", attached {like exposed_complex_to_be_tested}.c_hypot (Negative_infinity, -Nan))
			assert ("Infinity, -NaN", attached {like exposed_complex_to_be_tested}.c_hypot (Positive_infinity, -Nan))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.c_hypot (0, -Nan))
			assert ("-0, -NaN", attached {like exposed_complex_to_be_tested}.c_hypot (-0.0, -Nan))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.c_hypot (0, -Nan))
			assert ("a, -NaN", attached {like exposed_complex_to_be_tested}.c_hypot (some_real.value, -Nan))

			assert ("-NaN, NaN", attached {like exposed_complex_to_be_tested}.c_hypot (-Nan, Nan))
			assert ("NaN, NaN", attached {like exposed_complex_to_be_tested}.c_hypot (Nan, Nan))
			assert ("-Infinity, NaN", attached {like exposed_complex_to_be_tested}.c_hypot (Negative_infinity, Nan))
			assert ("Infinity, NaN", attached {like exposed_complex_to_be_tested}.c_hypot (Positive_infinity, Nan))
			assert ("NaN", attached {like exposed_complex_to_be_tested}.c_hypot (0, Nan))
			assert ("-0, NaN", attached {like exposed_complex_to_be_tested}.c_hypot (-0.0, Nan))
			assert ("0, NaN", attached {like exposed_complex_to_be_tested}.c_hypot (0, Nan))
			assert ("a, NaN", attached {like exposed_complex_to_be_tested}.c_hypot (some_real.value, Nan))

			assert ("-NaN, -Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (-Nan, Negative_infinity))
			assert ("NaN, -Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (Nan, Negative_infinity))
			assert ("-Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (Negative_infinity, Negative_infinity))
			assert ("Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (Positive_infinity, Negative_infinity))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (0, Negative_infinity))
			assert ("-0, -Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (-0.0, Negative_infinity))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (0, Negative_infinity))
			assert ("a, -Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (some_real.value, Negative_infinity))

			assert ("-NaN, Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (-Nan, Positive_infinity))
			assert ("NaN, Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (Nan, Positive_infinity))
			assert ("-Infinity, Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (Negative_infinity, Positive_infinity))
			assert ("Infinity, Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (Positive_infinity, Positive_infinity))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (0, Positive_infinity))
			assert ("-0, Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (-0.0, Positive_infinity))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (0, Positive_infinity))
			assert ("a, Infinity", attached {like exposed_complex_to_be_tested}.c_hypot (some_real.value, Positive_infinity))

			assert ("-NaN, -0", attached {like exposed_complex_to_be_tested}.c_hypot (-Nan, -0.0))
			assert ("NaN, -0", attached {like exposed_complex_to_be_tested}.c_hypot (Nan, -0.0))
			assert ("-Infinity, -0", attached {like exposed_complex_to_be_tested}.c_hypot (Negative_infinity, -0.0))
			assert ("Infinity, -0", attached {like exposed_complex_to_be_tested}.c_hypot (Positive_infinity, -0.0))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.c_hypot (0, -0.0))
			assert ("-0, -0", attached {like exposed_complex_to_be_tested}.c_hypot (-0.0, -0.0))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.c_hypot (0, -0.0))
			assert ("a, -0", attached {like exposed_complex_to_be_tested}.c_hypot (some_real.value, -0.0))

			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.c_hypot (-Nan, 0))
			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.c_hypot (-Nan, 0))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.c_hypot (Nan, 0))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.c_hypot (Nan, 0))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.c_hypot (Negative_infinity, 0))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.c_hypot (Negative_infinity, 0))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.c_hypot (Positive_infinity, 0))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.c_hypot (Positive_infinity, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_hypot (0, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_hypot (0, 0))
			assert ("-0, 0", attached {like exposed_complex_to_be_tested}.c_hypot (-0.0, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_hypot (0, 0))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.c_hypot (some_real.value, 0))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.c_hypot (some_real.value, 0))

			assert ("-NaN, b", attached {like exposed_complex_to_be_tested}.c_hypot (-Nan, some_real.value))
			assert ("NaN, b", attached {like exposed_complex_to_be_tested}.c_hypot (Nan, some_real.value))
			assert ("-Infinity, b", attached {like exposed_complex_to_be_tested}.c_hypot (Negative_infinity, some_real.value))
			assert ("Infinity, b", attached {like exposed_complex_to_be_tested}.c_hypot (Positive_infinity, some_real.value))
			assert ("0, bi", attached {like exposed_complex_to_be_tested}.c_hypot (0, some_real.value))
			assert ("-0, b", attached {like exposed_complex_to_be_tested}.c_hypot (-0.0, some_real.value))
			assert ("0, b", attached {like exposed_complex_to_be_tested}.c_hypot (0, some_real.value))
			assert ("a, b", attached {like exposed_complex_to_be_tested}.c_hypot (some_real.value, some_real.value))
		end

	test_c_atan2
			-- Test {STI_COMPLEX_NUMBER}.c_atan2.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.c_atan2"
		local
			zs: like Quick_C.subset_anchor
			z: like exposed_complex_to_be_tested
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				z := zs.any
				assert ("c_atan2", attached z.c_atan2 (z.b, z.a))
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN, -NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (-Nan, -Nan))
			assert ("NaN, -NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (Nan, -Nan))
			assert ("-Infinity, -NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (Negative_infinity, -Nan))
			assert ("Infinity, -NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (Positive_infinity, -Nan))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (0, -Nan))
			assert ("-0, -NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (-0.0, -Nan))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (0, -Nan))
			assert ("a, -NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (some_real.value, -Nan))

			assert ("-NaN, NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (-Nan, Nan))
			assert ("NaN, NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (Nan, Nan))
			assert ("-Infinity, NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (Negative_infinity, Nan))
			assert ("Infinity, NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (Positive_infinity, Nan))
			assert ("NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (0, Nan))
			assert ("-0, NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (-0.0, Nan))
			assert ("0, NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (0, Nan))
			assert ("a, NaN", attached {like exposed_complex_to_be_tested}.c_atan2 (some_real.value, Nan))

			assert ("-NaN, -Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (-Nan, Negative_infinity))
			assert ("NaN, -Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (Nan, Negative_infinity))
			assert ("-Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (Negative_infinity, Negative_infinity))
			assert ("Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (Positive_infinity, Negative_infinity))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (0, Negative_infinity))
			assert ("-0, -Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (-0.0, Negative_infinity))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (0, Negative_infinity))
			assert ("a, -Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (some_real.value, Negative_infinity))

			assert ("-NaN, Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (-Nan, Positive_infinity))
			assert ("NaN, Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (Nan, Positive_infinity))
			assert ("-Infinity, Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (Negative_infinity, Positive_infinity))
			assert ("Infinity, Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (Positive_infinity, Positive_infinity))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (0, Positive_infinity))
			assert ("-0, Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (-0.0, Positive_infinity))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (0, Positive_infinity))
			assert ("a, Infinity", attached {like exposed_complex_to_be_tested}.c_atan2 (some_real.value, Positive_infinity))

			assert ("-NaN, -0", attached {like exposed_complex_to_be_tested}.c_atan2 (-Nan, -0.0))
			assert ("NaN, -0", attached {like exposed_complex_to_be_tested}.c_atan2 (Nan, -0.0))
			assert ("-Infinity, -0", attached {like exposed_complex_to_be_tested}.c_atan2 (Negative_infinity, -0.0))
			assert ("Infinity, -0", attached {like exposed_complex_to_be_tested}.c_atan2 (Positive_infinity, -0.0))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.c_atan2 (0, -0.0))
			assert ("-0, -0", attached {like exposed_complex_to_be_tested}.c_atan2 (-0.0, -0.0))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.c_atan2 (0, -0.0))
			assert ("a, -0", attached {like exposed_complex_to_be_tested}.c_atan2 (some_real.value, -0.0))

			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (-Nan, 0))
			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (-Nan, 0))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (Nan, 0))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (Nan, 0))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (Negative_infinity, 0))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (Negative_infinity, 0))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (Positive_infinity, 0))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (Positive_infinity, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (0, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (0, 0))
			assert ("-0, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (-0.0, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (0, 0))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (some_real.value, 0))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.c_atan2 (some_real.value, 0))

			assert ("-NaN, b", attached {like exposed_complex_to_be_tested}.c_atan2 (-Nan, some_real.value))
			assert ("NaN, b", attached {like exposed_complex_to_be_tested}.c_atan2 (Nan, some_real.value))
			assert ("-Infinity, b", attached {like exposed_complex_to_be_tested}.c_atan2 (Negative_infinity, some_real.value))
			assert ("Infinity, b", attached {like exposed_complex_to_be_tested}.c_atan2 (Positive_infinity, some_real.value))
			assert ("0, bi", attached {like exposed_complex_to_be_tested}.c_atan2 (0, some_real.value))
			assert ("-0, b", attached {like exposed_complex_to_be_tested}.c_atan2 (-0.0, some_real.value))
			assert ("0, b", attached {like exposed_complex_to_be_tested}.c_atan2 (0, some_real.value))
			assert ("a, b", attached {like exposed_complex_to_be_tested}.c_atan2 (some_real.value, some_real.value))
		end

	test_product_from_native_reals
			-- Test {STI_COMPLEX_NUMBER}.product_from_native_reals.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.product_from_native_reals"
		local
			zs: like Quick_C.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			z1: like exposed_complex_to_be_tested
			z2: like some_complex
		do
			from
				zs := Quick_C
				from
					zt0 := zs.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # zs <= 80_003 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# C * zt0.n) print ('%N')
			until
				zs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					z1 := zs.any
					z2 := same_complex (zt.first)
					assert ("product_from_native_reals", attached {like exposed_complex_to_be_tested}.product_from_native_reals(z1.a, z1.b, z2.a.value, z2.b.value))
					zt := zt.tail
				variant
					n: {like Quick_C}.natural_as_integer (zt.n)
				end
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN, -NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-Nan, -Nan, some_native_real, some_native_real))
			assert ("NaN, -NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Nan, -Nan, some_native_real, some_native_real))
			assert ("-Infinity, -NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Negative_infinity, -Nan, some_native_real, some_native_real))
			assert ("Infinity, -NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Positive_infinity, -Nan, some_native_real, some_native_real))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, -Nan, some_native_real, some_native_real))
			assert ("-0, -NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-0.0, -Nan, some_native_real, some_native_real))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, -Nan, some_native_real, some_native_real))
			assert ("a, -NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (some_real.value, -Nan, some_native_real, some_native_real))

			assert ("-NaN, NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-Nan, Nan, some_native_real, some_native_real))
			assert ("NaN, NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Nan, Nan, some_native_real, some_native_real))
			assert ("-Infinity, NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Negative_infinity, Nan, some_native_real, some_native_real))
			assert ("Infinity, NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Positive_infinity, Nan, some_native_real, some_native_real))
			assert ("NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, Nan, some_native_real, some_native_real))
			assert ("-0, NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-0.0, Nan, some_native_real, some_native_real))
			assert ("0, NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, Nan, some_native_real, some_native_real))
			assert ("a, NaN", attached {like exposed_complex_to_be_tested}.product_from_native_reals (some_real.value, Nan, some_native_real, some_native_real))

			assert ("-NaN, -Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-Nan, Negative_infinity, some_native_real, some_native_real))
			assert ("NaN, -Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Nan, Negative_infinity, some_native_real, some_native_real))
			assert ("-Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Negative_infinity, Negative_infinity, some_native_real, some_native_real))
			assert ("Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Positive_infinity, Negative_infinity, some_native_real, some_native_real))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, Negative_infinity, some_native_real, some_native_real))
			assert ("-0, -Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-0.0, Negative_infinity, some_native_real, some_native_real))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, Negative_infinity, some_native_real, some_native_real))
			assert ("a, -Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (some_real.value, Negative_infinity, some_native_real, some_native_real))

			assert ("-NaN, Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-Nan, Positive_infinity, some_native_real, some_native_real))
			assert ("NaN, Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Nan, Positive_infinity, some_native_real, some_native_real))
			assert ("-Infinity, Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Negative_infinity, Positive_infinity, some_native_real, some_native_real))
			assert ("Infinity, Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Positive_infinity, Positive_infinity, some_native_real, some_native_real))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, Positive_infinity, some_native_real, some_native_real))
			assert ("-0, Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-0.0, Positive_infinity, some_native_real, some_native_real))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, Positive_infinity, some_native_real, some_native_real))
			assert ("a, Infinity", attached {like exposed_complex_to_be_tested}.product_from_native_reals (some_real.value, Positive_infinity, some_native_real, some_native_real))

			assert ("-NaN, -0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-Nan, -0.0, some_native_real, some_native_real))
			assert ("NaN, -0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Nan, -0.0, some_native_real, some_native_real))
			assert ("-Infinity, -0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Negative_infinity, -0.0, some_native_real, some_native_real))
			assert ("Infinity, -0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Positive_infinity, -0.0, some_native_real, some_native_real))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, -0.0, some_native_real, some_native_real))
			assert ("-0, -0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-0.0, -0.0, some_native_real, some_native_real))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, -0.0, some_native_real, some_native_real))
			assert ("a, -0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (some_real.value, -0.0, some_native_real, some_native_real))

			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-Nan, 0, some_native_real, some_native_real))
			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-Nan, 0, some_native_real, some_native_real))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Nan, 0, some_native_real, some_native_real))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Nan, 0, some_native_real, some_native_real))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Negative_infinity, 0, some_native_real, some_native_real))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Negative_infinity, 0, some_native_real, some_native_real))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Positive_infinity, 0, some_native_real, some_native_real))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Positive_infinity, 0, some_native_real, some_native_real))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, 0, some_native_real, some_native_real))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, 0, some_native_real, some_native_real))
			assert ("-0, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-0.0, 0, some_native_real, some_native_real))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, 0, some_native_real, some_native_real))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (some_real.value, 0, some_native_real, some_native_real))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.product_from_native_reals (some_real.value, 0, some_native_real, some_native_real))

			assert ("-NaN, b", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-Nan, some_real.value, some_native_real, some_native_real))
			assert ("NaN, b", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Nan, some_real.value, some_native_real, some_native_real))
			assert ("-Infinity, b", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Negative_infinity, some_real.value, some_native_real, some_native_real))
			assert ("Infinity, b", attached {like exposed_complex_to_be_tested}.product_from_native_reals (Positive_infinity, some_real.value, some_native_real, some_native_real))
			assert ("0, bi", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, some_real.value, some_native_real, some_native_real))
			assert ("-0, b", attached {like exposed_complex_to_be_tested}.product_from_native_reals (-0.0, some_real.value, some_native_real, some_native_real))
			assert ("0, b", attached {like exposed_complex_to_be_tested}.product_from_native_reals (0, some_real.value, some_native_real, some_native_real))
			assert ("a, b", attached {like exposed_complex_to_be_tested}.product_from_native_reals (some_real.value, some_real.value, some_native_real, some_native_real))
		end

	test_quotient_from_native_reals
			-- Test {STI_COMPLEX_NUMBER}.quotient_from_native_reals.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.quotient_from_native_reals"
		local
			zs: like Quick_C.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			z1: like exposed_complex_to_be_tested
			z2: like some_complex
		do
			from
				zs := Quick_C
				from
					zt0 := zs.as_tuple
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				until
					zt0.n × # zs <= 80_003 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					else
						zt0 := zt0.suffix ((next_random_item \\ (zt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# C * zt0.n) print ('%N')
			until
				zs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					z1 := zs.any
					z2 := same_complex (zt.first)
					assert ("quotient_from_native_reals", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals(z1.a, z1.b, z2.a.value, z2.b.value))
					zt := zt.tail
				variant
					n: {like Quick_C}.natural_as_integer (zt.n)
				end
				zs := zs.others
			variant
				cardinality: {like Quick_C}.natural_as_integer (# zs)
			end

			assert ("-NaN, -NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-Nan, -Nan, some_native_real, some_native_real))
			assert ("NaN, -NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Nan, -Nan, some_native_real, some_native_real))
			assert ("-Infinity, -NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Negative_infinity, -Nan, some_native_real, some_native_real))
			assert ("Infinity, -NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Positive_infinity, -Nan, some_native_real, some_native_real))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, -Nan, some_native_real, some_native_real))
			assert ("-0, -NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-0.0, -Nan, some_native_real, some_native_real))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, -Nan, some_native_real, some_native_real))
			assert ("a, -NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (some_real.value, -Nan, some_native_real, some_native_real))

			assert ("-NaN, NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-Nan, Nan, some_native_real, some_native_real))
			assert ("NaN, NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Nan, Nan, some_native_real, some_native_real))
			assert ("-Infinity, NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Negative_infinity, Nan, some_native_real, some_native_real))
			assert ("Infinity, NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Positive_infinity, Nan, some_native_real, some_native_real))
			assert ("NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, Nan, some_native_real, some_native_real))
			assert ("-0, NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-0.0, Nan, some_native_real, some_native_real))
			assert ("0, NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, Nan, some_native_real, some_native_real))
			assert ("a, NaN", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (some_real.value, Nan, some_native_real, some_native_real))

			assert ("-NaN, -Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-Nan, Negative_infinity, some_native_real, some_native_real))
			assert ("NaN, -Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Nan, Negative_infinity, some_native_real, some_native_real))
			assert ("-Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Negative_infinity, Negative_infinity, some_native_real, some_native_real))
			assert ("Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Positive_infinity, Negative_infinity, some_native_real, some_native_real))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, Negative_infinity, some_native_real, some_native_real))
			assert ("-0, -Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-0.0, Negative_infinity, some_native_real, some_native_real))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, Negative_infinity, some_native_real, some_native_real))
			assert ("a, -Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (some_real.value, Negative_infinity, some_native_real, some_native_real))

			assert ("-NaN, Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-Nan, Positive_infinity, some_native_real, some_native_real))
			assert ("NaN, Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Nan, Positive_infinity, some_native_real, some_native_real))
			assert ("-Infinity, Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Negative_infinity, Positive_infinity, some_native_real, some_native_real))
			assert ("Infinity, Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Positive_infinity, Positive_infinity, some_native_real, some_native_real))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, Positive_infinity, some_native_real, some_native_real))
			assert ("-0, Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-0.0, Positive_infinity, some_native_real, some_native_real))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, Positive_infinity, some_native_real, some_native_real))
			assert ("a, Infinity", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (some_real.value, Positive_infinity, some_native_real, some_native_real))

			assert ("-NaN, -0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-Nan, -0.0, some_native_real, some_native_real))
			assert ("NaN, -0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Nan, -0.0, some_native_real, some_native_real))
			assert ("-Infinity, -0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Negative_infinity, -0.0, some_native_real, some_native_real))
			assert ("Infinity, -0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Positive_infinity, -0.0, some_native_real, some_native_real))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, -0.0, some_native_real, some_native_real))
			assert ("-0, -0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-0.0, -0.0, some_native_real, some_native_real))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, -0.0, some_native_real, some_native_real))
			assert ("a, -0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (some_real.value, -0.0, some_native_real, some_native_real))

			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-Nan, 0, some_native_real, some_native_real))
			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-Nan, 0, some_native_real, some_native_real))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Nan, 0, some_native_real, some_native_real))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Nan, 0, some_native_real, some_native_real))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Negative_infinity, 0, some_native_real, some_native_real))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Negative_infinity, 0, some_native_real, some_native_real))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Positive_infinity, 0, some_native_real, some_native_real))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Positive_infinity, 0, some_native_real, some_native_real))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, 0, some_native_real, some_native_real))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, 0, some_native_real, some_native_real))
			assert ("-0, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-0.0, 0, some_native_real, some_native_real))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, 0, some_native_real, some_native_real))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (some_real.value, 0, some_native_real, some_native_real))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (some_real.value, 0, some_native_real, some_native_real))

			assert ("-NaN, b", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-Nan, some_real.value, some_native_real, some_native_real))
			assert ("NaN, b", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Nan, some_real.value, some_native_real, some_native_real))
			assert ("-Infinity, b", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Negative_infinity, some_real.value, some_native_real, some_native_real))
			assert ("Infinity, b", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (Positive_infinity, some_real.value, some_native_real, some_native_real))
			assert ("0, bi", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, some_real.value, some_native_real, some_native_real))
			assert ("-0, b", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (-0.0, some_real.value, some_native_real, some_native_real))
			assert ("0, b", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (0, some_real.value, some_native_real, some_native_real))
			assert ("a, b", attached {like exposed_complex_to_be_tested}.quotient_from_native_reals (some_real.value, some_real.value, some_native_real, some_native_real))
		end

	test_c_copysign
			-- Test {STI_COMPLEX_NUMBER}.c_copysign.
		note
			testing: "covers/{STI_COMPLEX_NUMBER}.c_copysign"
		do
			assert ("-NaN, -NaN", attached {like exposed_complex_to_be_tested}.c_copysign (-Nan, -Nan))
			assert ("NaN, -NaN", attached {like exposed_complex_to_be_tested}.c_copysign (Nan, -Nan))
			assert ("-Infinity, -NaN", attached {like exposed_complex_to_be_tested}.c_copysign (Negative_infinity, -Nan))
			assert ("Infinity, -NaN", attached {like exposed_complex_to_be_tested}.c_copysign (Positive_infinity, -Nan))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.c_copysign (0, -Nan))
			assert ("-0, -NaN", attached {like exposed_complex_to_be_tested}.c_copysign (-0.0, -Nan))
			assert ("0, -NaN", attached {like exposed_complex_to_be_tested}.c_copysign (0, -Nan))
			assert ("a, -NaN", attached {like exposed_complex_to_be_tested}.c_copysign (some_real.value, -Nan))

			assert ("-NaN, NaN", attached {like exposed_complex_to_be_tested}.c_copysign (-Nan, Nan))
			assert ("NaN, NaN", attached {like exposed_complex_to_be_tested}.c_copysign (Nan, Nan))
			assert ("-Infinity, NaN", attached {like exposed_complex_to_be_tested}.c_copysign (Negative_infinity, Nan))
			assert ("Infinity, NaN", attached {like exposed_complex_to_be_tested}.c_copysign (Positive_infinity, Nan))
			assert ("NaN", attached {like exposed_complex_to_be_tested}.c_copysign (0, Nan))
			assert ("-0, NaN", attached {like exposed_complex_to_be_tested}.c_copysign (-0.0, Nan))
			assert ("0, NaN", attached {like exposed_complex_to_be_tested}.c_copysign (0, Nan))
			assert ("a, NaN", attached {like exposed_complex_to_be_tested}.c_copysign (some_real.value, Nan))

			assert ("-NaN, -Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (-Nan, Negative_infinity))
			assert ("NaN, -Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (Nan, Negative_infinity))
			assert ("-Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (Negative_infinity, Negative_infinity))
			assert ("Infinity, -Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (Positive_infinity, Negative_infinity))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (0, Negative_infinity))
			assert ("-0, -Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (-0.0, Negative_infinity))
			assert ("0, -Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (0, Negative_infinity))
			assert ("a, -Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (some_real.value, Negative_infinity))

			assert ("-NaN, Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (-Nan, Positive_infinity))
			assert ("NaN, Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (Nan, Positive_infinity))
			assert ("-Infinity, Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (Negative_infinity, Positive_infinity))
			assert ("Infinity, Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (Positive_infinity, Positive_infinity))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (0, Positive_infinity))
			assert ("-0, Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (-0.0, Positive_infinity))
			assert ("0, Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (0, Positive_infinity))
			assert ("a, Infinity", attached {like exposed_complex_to_be_tested}.c_copysign (some_real.value, Positive_infinity))

			assert ("-NaN, -0", attached {like exposed_complex_to_be_tested}.c_copysign (-Nan, -0.0))
			assert ("NaN, -0", attached {like exposed_complex_to_be_tested}.c_copysign (Nan, -0.0))
			assert ("-Infinity, -0", attached {like exposed_complex_to_be_tested}.c_copysign (Negative_infinity, -0.0))
			assert ("Infinity, -0", attached {like exposed_complex_to_be_tested}.c_copysign (Positive_infinity, -0.0))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.c_copysign (0, -0.0))
			assert ("-0, -0", attached {like exposed_complex_to_be_tested}.c_copysign (-0.0, -0.0))
			assert ("0, -0", attached {like exposed_complex_to_be_tested}.c_copysign (0, -0.0))
			assert ("a, -0", attached {like exposed_complex_to_be_tested}.c_copysign (some_real.value, -0.0))

			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.c_copysign (-Nan, 0))
			assert ("-NaN, 0", attached {like exposed_complex_to_be_tested}.c_copysign (-Nan, 0))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.c_copysign (Nan, 0))
			assert ("NaN, 0", attached {like exposed_complex_to_be_tested}.c_copysign (Nan, 0))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.c_copysign (Negative_infinity, 0))
			assert ("-Infinity, 0", attached {like exposed_complex_to_be_tested}.c_copysign (Negative_infinity, 0))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.c_copysign (Positive_infinity, 0))
			assert ("Infinity, 0", attached {like exposed_complex_to_be_tested}.c_copysign (Positive_infinity, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_copysign (0, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_copysign (0, 0))
			assert ("-0, 0", attached {like exposed_complex_to_be_tested}.c_copysign (-0.0, 0))
			assert ("0, 0", attached {like exposed_complex_to_be_tested}.c_copysign (0, 0))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.c_copysign (some_real.value, 0))
			assert ("a, 0", attached {like exposed_complex_to_be_tested}.c_copysign (some_real.value, 0))

			assert ("-NaN, b", attached {like exposed_complex_to_be_tested}.c_copysign (-Nan, some_real.value))
			assert ("NaN, b", attached {like exposed_complex_to_be_tested}.c_copysign (Nan, some_real.value))
			assert ("-Infinity, b", attached {like exposed_complex_to_be_tested}.c_copysign (Negative_infinity, some_real.value))
			assert ("Infinity, b", attached {like exposed_complex_to_be_tested}.c_copysign (Positive_infinity, some_real.value))
			assert ("0, bi", attached {like exposed_complex_to_be_tested}.c_copysign (0, some_real.value))
			assert ("-0, b", attached {like exposed_complex_to_be_tested}.c_copysign (-0.0, some_real.value))
			assert ("0, b", attached {like exposed_complex_to_be_tested}.c_copysign (0, some_real.value))
			assert ("a, b", attached {like exposed_complex_to_be_tested}.c_copysign (some_real.value, some_real.value))
		end

feature {NONE} -- Factory (element to be tested)

	complex_to_be_tested: like some_expanded_complex
			-- Complex number meant to be under tests
		do
			Result := some_expanded_complex
		end

	exposed_complex_to_be_tested: like some_exposed_complex
			-- Complex number meant to have under tests its features with restricted access
		do
			Result := some_exposed_complex
		end

feature {NONE} -- Conversion

	complex_from_z (z: STS_COMPLEX_NUMBER): like complex_to_be_tested
			-- `z' converted to a complex number like `complex_to_be_tested'
		do
			Result := z
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (z.a.value)
			imaginary: Result.b.value = {STI_REAL_NUMBER}.adjusted_value (z.b.value)
		end

	complex_from_native_real (v: like some_native_real): like complex_to_be_tested
			-- `v' converted to a complex number like `complex_to_be_tested'
		do
			Result := v -- TODO: optimize semantics for expanded types.
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (v)
			imaginary: Result.b ≍ Zero
		end

	complex_from_native_real_ref (v_ref: like some_native_real_ref): like complex_to_be_tested
			-- `v_ref' converted to a complex number like `complex_to_be_tested'
		do
			Result := v_ref
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (v_ref.item)
			imaginary: Result.b ≍ Zero
		end

	complex_from_native_double_real (dbl_v: like some_native_double_real): like complex_to_be_tested
			-- `dbl_v' converted to a complex number like `complex_to_be_tested'
			--| TODO: Get rid of truncated_to_real.
		do
			Result := dbl_v
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (dbl_v.truncated_to_real)
			imaginary: Result.b ≍ Zero
		end

	complex_from_native_double_real_ref (dbl_v_ref: like some_native_double_real_ref): like complex_to_be_tested
			-- `dbl_v_ref' converted to a complex number like `complex_to_be_tested'
			--| TODO: Get rid of truncated_to_real.
		do
			Result := dbl_v_ref
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (dbl_v_ref.truncated_to_real)
			imaginary: Result.b ≍ Zero
		end

	complex_from_real (x: STS_REAL_NUMBER): like complex_to_be_tested
			-- `x' converted to a complex number like `complex_to_be_tested'
		do
			Result := x
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (x.value)
			imaginary: Result.b ≍ Zero
		end

	complex_from_native_integer (v: like some_native_integer): like complex_to_be_tested
			-- `v' converted to a complex number like `complex_to_be_tested'
		do
			Result := v
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (v.to_real)
			imaginary: Result.b ≍ Zero
		end

	complex_from_native_integer_ref (v_ref: attached like some_native_integer_ref): like complex_to_be_tested
			-- `v_ref' converted to a complex number like `complex_to_be_tested'
		do
			Result := v_ref
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (v_ref.to_real)
			imaginary: Result.b ≍ Zero
		end

	complex_from_integer (j: STS_INTEGER_NUMBER): like complex_to_be_tested
			-- `j' converted to a complex number like `complex_to_be_tested'
		do
			Result := j
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (j.value)
			imaginary: Result.b ≍ Zero
		end

	complex_from_native_natural (v: like some_native_natural): like complex_to_be_tested
			-- `v' converted to a complex number like `complex_to_be_tested'
		do
			Result := v
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (v.to_real_32)
			imaginary: Result.b ≍ Zero
		end

	complex_from_native_natural_ref (v_ref: like some_native_natural_ref): like complex_to_be_tested
			-- `v_ref' converted to a complex number like `complex_to_be_tested'
		do
			Result := v_ref
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (v_ref.to_real_32)
			imaginary: Result.b ≍ Zero
		end

	complex_from_sts_pair (ab: STS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER], STS_EQUALITY [STS_REAL_NUMBER]]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (ab.a.value)
			imaginary: Result.b.value = {STI_REAL_NUMBER}.adjusted_value (ab.b.value)
		end

	complex_from_numeric_sti_pair
		(ab: STI_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER], STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (ab.a.value)
			imaginary: Result.b.value = {STI_REAL_NUMBER}.adjusted_value (ab.b.value)
		end

	complex_from_real_sti_pair (ab: STI_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (ab.a.value)
			imaginary: Result.b.value = {STI_REAL_NUMBER}.adjusted_value (ab.b.value)
		end

	complex_from_expanded_pair (ab: STS_ORDERED_PAIR [STI_REAL_NUMBER, STI_REAL_NUMBER, STS_EQUALITY [STI_REAL_NUMBER], STS_EQUALITY [STI_REAL_NUMBER]]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a ≍ ab.a
			imaginary: Result.b ≍ ab.b
		end

	complex_from_expanded_sti_pair
		(ab: STI_ORDERED_PAIR [STI_REAL_NUMBER, STI_REAL_NUMBER, STS_NUMERIC_EQUALITY [STI_REAL_NUMBER], STS_NUMERIC_EQUALITY [STI_REAL_NUMBER]]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a ≍ ab.a
			imaginary: Result.b ≍ ab.b
		end

	complex_from_homogeneous_pair (ab: STS_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_EQUALITY [STS_REAL_NUMBER]]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (ab.a.value)
			imaginary: Result.b.value = {STI_REAL_NUMBER}.adjusted_value (ab.b.value)
		end

	complex_from_numeric_sti_homogeneous_pair (ab: STI_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (ab.a.value)
			imaginary: Result.b.value = {STI_REAL_NUMBER}.adjusted_value (ab.b.value)
		end

	complex_from_real_sti_homogeneous_pair (ab: STI_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_EQUALITY]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a.value = {STI_REAL_NUMBER}.adjusted_value (ab.a.value)
			imaginary: Result.b.value = {STI_REAL_NUMBER}.adjusted_value (ab.b.value)
		end

	complex_from_expanded_homogeneous_pair (ab: STS_HOMOGENEOUS_ORDERED_PAIR [STI_REAL_NUMBER, STS_EQUALITY [STI_REAL_NUMBER]]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a ≍ ab.a
			imaginary: Result.b ≍ ab.b
		end

	complex_from_expanded_sti_homogeneous_pair (ab: STI_HOMOGENEOUS_ORDERED_PAIR [STI_REAL_NUMBER, STS_NUMERIC_EQUALITY [STI_REAL_NUMBER]]): like complex_to_be_tested
			-- `ab' converted to a complex number like `complex_to_be_tested'
		do
			Result := ab
		ensure
			real: Result.a ≍ ab.a
			imaginary: Result.b ≍ ab.b
		end

	complex_to_sti_homogeneous_real_pair (z: STI_COMPLEX_NUMBER):
		STI_HOMOGENEOUS_ORDERED_PAIR [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- `z' converted into an effective homogeneous ordered pair of real
			-- numbers
		do
			Result := z
		ensure
			first: Result.a ≍ z.a
			second: Result.b ≍ z.b
		end

	complex_to_sti_homogeneous_numeric_pair (z: STI_COMPLEX_NUMBER):
		STI_HOMOGENEOUS_ORDERED_PAIR
			[STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]
			-- `z' converted into an effective homogeneous ordered pair of real
			-- numbers
		do
			Result := z
		ensure
			first: Result.a ≍ z.a
			second: Result.b ≍ z.b
		end

	complex_to_sti_real_pair (z: STI_COMPLEX_NUMBER): STI_ORDERED_PAIR [
		STS_REAL_NUMBER, STS_REAL_NUMBER, STS_REAL_EQUALITY, STS_REAL_EQUALITY
		]
			-- `z' converted into an effective ordered pair of real numbers
		do
			Result := z
		ensure
			first: Result.a ≍ z.a
			second: Result.b ≍ z.b
		end

	complex_to_sti_numeric_pair (z: STI_COMPLEX_NUMBER): STI_ORDERED_PAIR [
		STS_REAL_NUMBER, STS_REAL_NUMBER,
		STS_NUMERIC_EQUALITY [STS_REAL_NUMBER],
		STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]
		]
			-- `z' converted into an effective ordered pair of real numbers
		do
			Result := z
		ensure
			first: Result.a ≍ z.a
			second: Result.b ≍ z.b
		end

	complex_to_sti_n_tuple (z: STI_COMPLEX_NUMBER):
		STI_N_TUPLE [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- `z' converted into an effective n-tuple of real numbers
		do
			Result := z
		ensure
			pair: Result.n = 2
			first: Result [1] ≍ z.a
			second: Result [2] ≍ z.b
		end

	complex_to_sts_n_tuple (z: STI_COMPLEX_NUMBER):
		STS_N_TUPLE [STS_REAL_NUMBER, STS_REAL_EQUALITY]
			-- `z' converted into an n-tuple of real numbers
		do
			Result := z
		ensure
			pair: Result.n = 2
			first: Result [1] ≍ z.a
			second: Result [2] ≍ z.b
		end

	complex_to_numeric_sti_n_tuple (z: STI_COMPLEX_NUMBER):
		STI_N_TUPLE [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]
			-- `z' converted into an effective n-tuple of real numbers compared
			-- by {STS_NUMERIC_EQUALITY}
		do
			Result := z
		ensure
			pair: Result.n = 2
			first: Result.eq (Result [1], z.a)
			second: Result.eq (Result [2],  z.b)
		end

	complex_to_numeric_sts_n_tuple (z: STI_COMPLEX_NUMBER):
		STS_N_TUPLE [STS_REAL_NUMBER, STS_NUMERIC_EQUALITY [STS_REAL_NUMBER]]
			-- `z' converted into an effective n-tuple of real numbers compared
			-- by {STS_NUMERIC_EQUALITY}
		do
			Result := z
		ensure
			pair: Result.n = 2
			first: Result.eq (Result [1], z.a)
			second: Result.eq (Result [2],  z.b)
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"

end
