﻿note
	description: "Test suite for {STI_RATIONAL_NUMBER}"
	author: "Rosivaldo Fernandes Alves"
	date: "$Date$"
	revision: "$Revision$"

class
	RATIONAL_NUMBER_TESTS

inherit
	ELEMENT_TESTS
		rename
			test_is_in as test_element_is_in,
			test_is_not_in as test_element_is_not_in,
			element_to_be_tested as rational_to_be_tested,
			some_integer_ref as some_native_integer_ref,
			some_direct_integer_ref as some_direct_native_integer_ref,
			new_integer_ref as new_native_integer_ref,
			some_integer as some_native_integer
		undefine
			some_native_integer,
			some_element
		redefine
			on_prepare,
			test_element_is_in,
			test_element_is_not_in,
			rational_to_be_tested
		end

	NUMBERS_TESTS_RESOURCES_Q
		redefine
			on_prepare
		end

--create
--	test_continued_fraction -- TODO: Remove it!

feature -- Access

	One: STI_RATIONAL_NUMBER
			-- The rational number 1/1
		once
			Result := {STI_RATIONAL_NUMBER}.One
		ensure then
			class
			numerator: Result.p = {STI_INTEGER_NUMBER}.One
			denominator: Result.p = {STI_INTEGER_NUMBER}.One
		end

	properties: RATIONAL_NUMBER_PROPERTIES
			-- Object that checks the set-theory properties of
			-- {STI_RATIONAL_NUMBER}

	homogeneous_pair_properties: HOMOGENEOUS_ORDERED_PAIR_PROPERTIES_I
			-- Object that checks the set-theory properties of
			-- {STS_HOMOGENEOUS_ORDERED_PAIR [STS_INTEGER_NUMBER, ...]}

	composable_pair_properties: COMPOSABLE_ORDERED_PAIR_PROPERTIES_III
			-- Object that checks the set-theory properties of {
			-- 	STS_COMPOSABLE_ORDERED_PAIR [
			--		STS_INTEGER_NUMBER,
			-- 		STS_INTEGER_NUMBER,
			-- 		STS_INTEGER_NUMBER, ...
			-- 		]
			-- 	}

	resources_ii: SETS_TESTS_RESOURCES_BINARY_INT_INT
			-- Resources of ordered pairs of integer numbers

feature {NONE} -- Events

	on_prepare
			-- <Precursor
		do
			Precursor {NUMBERS_TESTS_RESOURCES_Q}
			create properties
			create homogeneous_pair_properties
			create composable_pair_properties
			create resources_ii
		end

feature -- Test routines (Initialization)

	test_default_create
			-- Test {STI_RATIONAL_NUMBER}.default_create.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.default_create"
		do
			assert (
				"default_create",
				attached (create {like rational_to_be_tested})
				)
			assert ("zero", (create {like rational_to_be_tested}) ≍ Zero)
			assert (
				"default_create_ok",
				properties.default_create_ok (some_rational)
				)
		end

	test_make
			-- Test {STI_RATIONAL_NUMBER}.make.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.make"
		local
			num, den: like some_integer
		do
			num := some_integer
			from
				den := some_integer
			until
				den ≭ Zero
			loop
				den := some_integer
			end
			assert (
				"make",
				attached (create {like rational_to_be_tested}.make (num, den))
				)
		end

	test_make_from_expanded
			-- Test {STI_RATIONAL_NUMBER}.make_from_expanded.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.make_from_expanded"
		local
			native_n_ref: like some_native_natural_ref
			n: like some_natural
			native_i_ref: like some_native_integer_ref
			i: like some_integer
			exp_i: like some_expanded_integer
		do
			from
				native_n_ref := some_native_natural_ref
			until
				native_n_ref.item /= 0
			loop
				native_n_ref := some_native_natural_ref
			end
			assert (
				"native naturals",
				attached (
					create {like rational_to_be_tested}.make_from_expanded
						(some_native_natural, native_n_ref)
					)
				)

			from
				n := some_natural
			until
				n ≭ Zero
			loop
				n := some_natural
			end
			assert (
				"naturals",
				attached (
					create {like rational_to_be_tested}.make_from_expanded
						(some_natural, n)
					)
				)

			from
				native_i_ref := some_native_integer_ref
			until
				attached native_i_ref and then native_i_ref.item /= 0
			loop
				native_i_ref := some_native_integer_ref
			end
			assert (
				"native integers",
				attached (
					create {like rational_to_be_tested}.make_from_expanded
						(some_native_integer, native_i_ref)
					)
				)

			from
				i := some_integer
			until
				i ≭ Zero
			loop
				i := some_integer
			end
			assert (
				"integers",
				attached (
					create {like rational_to_be_tested}.make_from_expanded
						(some_integer, i)
					)
				)

			from
				exp_i := some_expanded_integer
			until
				exp_i ≭ Zero
			loop
				exp_i := some_expanded_integer
			end
			assert (
				"expanded integers",
				attached (
					create {like rational_to_be_tested}.make_from_expanded
						(some_expanded_integer, exp_i)
					)
				)
		end

	test_make_from_rational
			-- Test {STI_RATIONAL_NUMBER}.make_from_rational.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.make_from_rational"
		do
			assert (
				"make_from_rational",
				attached (create {like rational_to_be_tested}.make_from_rational
					(some_rational))
				)
			assert
				("rational_from_pq", attached rational_from_pq (some_rational))
		end

	test_make_from_native_integer
			-- Test {STI_RATIONAL_NUMBER}.make_from_native_integer.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.make_from_native_integer"
		do
			assert (
				"make_from_native_integer",
				attached (
					create {like rational_to_be_tested}.
						make_from_native_integer (next_random_item)
					)
				)
			assert (
				"rational_from_native_integer",
				attached rational_from_native_integer (next_random_item)
				)
		end

	test_make_from_native_integer_ref
			-- Test {STI_RATIONAL_NUMBER}.make_from_native_integer_ref.
		note
			testing:
				"covers/{STI_RATIONAL_NUMBER}.make_from_native_integer_ref"
		local
			i_ref: like some_native_integer_ref
		do
			from
				i_ref := some_native_integer_ref
			until
				attached i_ref
			loop
				i_ref := some_native_integer_ref
			end
			assert (
				"make_from_native_integer_ref",
				attached (
					create {like rational_to_be_tested}.
						make_from_native_integer_ref (i_ref)
					)
				)
			assert (
				"rational_from_native_integer_ref",
				attached rational_from_native_integer_ref (i_ref)
				)
		end

	test_make_from_integer
			-- Test {STI_RATIONAL_NUMBER}.make_from_integer.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.make_from_integer"
		do
			assert (
				"make_from_integer",
				attached (
					create {like rational_to_be_tested}.make_from_integer
						(some_integer)
					)
				)
			assert (
				"rational_from_integer",
				attached rational_from_integer (some_integer)
				)
		end

	test_make_from_native_natural
			-- Test {STI_RATIONAL_NUMBER}.make_from_native_natural.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.make_from_native_natural"
		do
			assert (
				"make_from_native_natural",
				attached (
					create {like rational_to_be_tested}.make_from_native_natural
						(some_native_natural)
					)
				)
			assert (
				"rational_from_native_natural",
				attached rational_from_native_natural (some_native_natural)
				)
		end

	test_make_from_native_natural_ref
			-- Test {STI_RATIONAL_NUMBER}.make_from_native_natural_ref.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.make_from_native_natural_ref"
		do
			assert (
				"make_from_native_natural_ref",
				attached (
					create {like rational_to_be_tested}.
						make_from_native_natural_ref (some_native_natural_ref)
					)
				)
			assert (
				"rational_from_native_natural_ref",
				attached rational_from_native_natural_ref
					(some_native_natural_ref)
				)
		end

feature -- Test routines (Primitive)

	test_p
			-- Test {STI_RATIONAL_NUMBER}.p.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.p"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("p", attached pq.p)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_numerator
			-- Test {STI_RATIONAL_NUMBER}.numerator.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.numerator"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("numerator", attached pq.numerator)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_q
			-- Test {STI_RATIONAL_NUMBER}.q.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.q"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("q", attached pq.q)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_denominator
			-- Test {STI_RATIONAL_NUMBER}.denominator.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.denominator"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("denominator", attached pq.denominator)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_eq
			-- Test {STI_RATIONAL_NUMBER}.eq.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.eq"
		do
			assert ("eq", attached rational_to_be_tested.Eq)
		end

	test_value
			-- Test {STI_RATIONAL_NUMBER}.value.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.value"
		local
			pqs: like some_rational_set
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("value", attached pq.value)
				assert (
					"native_real_from_rational",
					attached native_real_from_rational (pq)
					)
				assert (
					"native_real_ref_from_rational",
					attached native_ref_from_rational (pq)
					)
				assert (
					"numeric_from_rational", attached numeric_from_rational (pq)
					)
				assert (
					"comparable_from_rational",
					attached comparable_from_rational (pq)
					)
				assert (
					"hashable_from_rational",
					attached hashable_from_rational (pq)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

feature -- Test routines (Membership)

	test_element_is_in
			-- <Precursor>
			-- Test {STS_COMPLEX_NUMBER}.element_is_in.
			-- Test {STS_REAL_NUMBER}.element_is_in.
			-- Test {STS_RATIONAL_NUMBER}.element_is_in.
			-- Test {STI_RATIONAL_NUMBER}.element_is_in.
		note
			testing: "covers/{STS_ELEMENT}.is_in"
			testing: "covers/{STS_COMPLEX_NUMBER}.element_is_in"
			testing: "covers/{STS_REAL_NUMBER}.element_is_in"
			testing: "covers/{STS_RATIONAL_NUMBER}.element_is_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.element_is_in"
		do
			Precursor {ELEMENT_TESTS}
		end

	test_element_is_not_in
			-- Test {STS_COMPLEX_NUMBER}.element_is_not_in.
			-- Test {STS_REAL_NUMBER}.element_is_not_in.
			-- Test {STS_RATIONAL_NUMBER}.element_is_not_in.
			-- Test {STI_RATIONAL_NUMBER}.element_is_not_in.
		note
			testing: "covers/{STS_ELEMENT}.is_not_in"
			testing: "covers/{STS_COMPLEX_NUMBER}.element_is_not_in"
			testing: "covers/{STS_REAL_NUMBER}.element_is_not_in"
			testing: "covers/{STS_RATIONAL_NUMBER}.element_is_not_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.element_is_not_in"
		do
			Precursor {ELEMENT_TESTS}
		end

	test_complex_is_in
			-- Test {STS_COMPLEX_NUMBER}.is_in.
			-- Test {STS_REAL_NUMBER}.complex_is_in.
			-- Test {STS_RATIONAL_NUMBER}.complex_is_in.
			-- Test {STI_RATIONAL_NUMBER}.complex_is_in.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_in"
			testing: "covers/{STS_REAL_NUMBER}.complex_is_in"
			testing: "covers/{STS_RATIONAL_NUMBER}.complex_is_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_is_in"
		local
			pq: like rational_to_be_tested
			s: like some_complexes
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			pq := rational_to_be_tested
			s := some_complex_references & pq
			assert ("same_reference", pq.complex_is_in (s))
			assert ("same_reference_ok", properties.complex_is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_complex_objects & pq.twin
			assert ("same_value", pq.complex_is_in (s))
			assert ("same_value_ok", properties.complex_is_in_ok (pq, s))

			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_complex_numbers & same_complex (pq)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_math_value", pq.complex_is_in (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("same_math_value_ok", properties.complex_is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_complexes
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("some_complexes", pq.complex_is_in (s) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("some_complexes_ok", properties.complex_is_in_ok (pq, s))
		end

	test_complex_is_not_in
			-- Test {STS_COMPLEX_NUMBER}.is_not_in.
			-- Test {STS_REAL_NUMBER}.complex_is_not_in.
			-- Test {STS_RATIONAL_NUMBER}.complex_is_not_in.
			-- Test {STI_RATIONAL_NUMBER}.complex_is_not_in.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.is_not_in"
			testing: "covers/{STS_REAL_NUMBER}.complex_is_not_in"
			testing: "covers/{STS_RATIONAL_NUMBER}.complex_is_not_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_is_not_in"
		local
			pq: like rational_to_be_tested
			s: like some_complexes
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			pq := rational_to_be_tested
			s := some_complex_references / pq
			assert ("same_reference", pq.complex_is_not_in (s))
			assert
				("same_reference ok", Properties.complex_is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_complex_objects / pq.twin
			assert ("same_value", pq.complex_is_not_in (s))
			assert ("same_value", Properties.complex_is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_complex_numbers / same_complex (pq)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_math_value", pq.complex_is_not_in (s))
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("same_math_value", Properties.complex_is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_complexes
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert
					("some_complexes", pq.complex_is_not_in (s) implies True)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert
				("some_complexes_ok", Properties.complex_is_not_in_ok (pq, s))
		end

	test_real_is_in
			-- Test {STS_REAL_NUMBER}.is_in.
			-- Test {STS_RATIONAL_NUMBER}.real_is_in.
			-- Test {STI_RATIONAL_NUMBER}.real_is_in.
			--| TODO: Errors due to CATcalls.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_in"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_is_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_is_in"
		local
			pq: like rational_to_be_tested
			s: like some_reals
		do
			pq := rational_to_be_tested
			s := some_real_references & pq
			assert ("same_reference", pq.real_is_in (s))
			assert ("same_reference ok", properties.real_is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_real_objects & pq.twin
			assert ("same_object_value", pq.real_is_in (s))
			assert ("same_object_value ok", properties.real_is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_real_numbers & same_real (pq)
			assert ("same_math_value", pq.real_is_in (s))
			assert ("same_math_value ok", properties.real_is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_strict_real_numbers & same_real (pq)
			assert ("same_strict_math_value", pq.real_is_in (s))
			assert
				("same_strict_math_value ok", properties.real_is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_reals
			assert ("some_reals", pq.real_is_in (s) implies True)
			assert ("some_reals ok", properties.real_is_in_ok (pq, s))
		end

	test_real_is_not_in
			-- Test {STS_REAL_NUMBER}.is_not_in.
			-- Test {STS_RATIONAL_NUMBER}.real_is_not_in.
			-- Test {STI_RATIONAL_NUMBER}.real_is_not_in.
			--| TODO: Errors due to CATcalls.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_not_in"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_is_not_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_is_not_in"
		local
			pq: like rational_to_be_tested
			s: like some_reals
		do
			pq := rational_to_be_tested
			s := some_real_references / pq
			assert ("same_reference", pq.real_is_not_in (s))
			assert ("same_reference ok", properties.real_is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_real_objects / pq.twin
			assert ("same_object_value", pq.real_is_not_in (s))
			assert
				("same_object_value ok", properties.real_is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_real_numbers / same_real (pq)
			assert ("same_math_value", pq.real_is_not_in (s))
			assert ("same_math_value ok", properties.real_is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_strict_real_numbers / same_real (pq)
			assert ("same_strict_math_value", pq.real_is_not_in (s))
			assert ("same_strict_math_value ok", properties.real_is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_reals
			assert ("some_reals", pq.real_is_not_in (s) implies True)
			assert ("some_reals ok", properties.real_is_not_in_ok (pq, s))
		end

	test_pair_is_in
			-- Test {STS_ORDERED_PAIR}.is_in.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.is_in.
			-- Test {STI_RATIONAL_NUMBER}.pair_is_in.
		note
			testing: "covers/{STS_ORDERED_PAIR}.is_in"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.is_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.pair_is_in"
		local
			pq: like rational_to_be_tested
			abs: like resources_ii.some_set_ab
			chk_ops: BOOLEAN
		do
			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_ii.some_references_ab & pq
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_reference", pq.pair_is_in (abs))
			assert ("same_reference_ok", homogeneous_pair_properties.is_in_ok (pq, abs))

			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_ii.some_objects_ab & pq.twin
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_value", pq.pair_is_in (abs))
			assert ("same_value_ok", homogeneous_pair_properties.is_in_ok (pq, abs))

			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_ii.some_math_set_ab & resources_ii.same_pair_ab (pq)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_math_value", pq.pair_is_in (abs))
			assert ("same_math_value_ok", homogeneous_pair_properties.is_in_ok (pq, abs))

			pq := rational_to_be_tested
			abs := resources_ii.some_set_ab
			assert ("some_pairs", pq.pair_is_in (abs) implies True)
			assert ("is_in_ok", homogeneous_pair_properties.is_in_ok (pq, abs))
		end

	test_pair_is_not_in
			-- Test {STS_ORDERED_PAIR}.is_not_in.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.is_not_in.
			-- Test {STI_RATIONAL_NUMBER}.pair_is_not_in.
		note
			testing: "covers/{STS_ORDERED_PAIR}.is_not_in"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.is_not_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.pair_is_not_in"
		local
			pq: like rational_to_be_tested
			abs: like resources_ii.some_set_ab
			chk_ops: BOOLEAN
		do
			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_ii.some_references_ab / pq
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_reference", pq.pair_is_not_in (abs))
			assert ("same_reference_ok", homogeneous_pair_properties.is_not_in_ok (pq, abs))

			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_ii.some_objects_ab / pq.twin
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_value", pq.pair_is_not_in (abs))
			assert ("same_value_ok", homogeneous_pair_properties.is_not_in_ok (pq, abs))

			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				abs := resources_ii.some_math_set_ab / resources_ii.same_pair_ab (pq)
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			assert ("same_math_value", pq.pair_is_not_in (abs))
			assert ("same_math_value_ok", homogeneous_pair_properties.is_not_in_ok (pq, abs))

			pq := rational_to_be_tested
			abs := resources_ii.some_set_ab
			assert ("some_pairs", pq.pair_is_not_in (abs) implies True)
			assert ("is_not_in_ok", homogeneous_pair_properties.is_not_in_ok (pq, abs))
		end

	test_is_in
			-- Test {STS_RATIONAL_NUMBER}.is_in.
			-- Test {STI_RATIONAL_NUMBER}.is_in.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.is_in"
		local
			pq: like rational_to_be_tested
			s: like some_rationals
		do
			pq := rational_to_be_tested
			s := some_rational_references & pq
			assert ("same_reference", pq ∈ s)
			assert ("same_reference_ok", properties.is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_rational_objects & pq.twin
			assert ("same_value", pq ∈ s)
			assert ("same_value_ok", properties.is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_rational_numbers & same_rational (pq)
			assert ("same_math_value", pq ∈ s)
			assert ("same_math_value_ok", properties.is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_strict_rational_numbers & same_rational (pq)
			assert ("same_strict_math_value", pq ∈ s)
			assert ("same_strict_math_value_ok", properties.is_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_rationals
			assert ("some_rationals", pq ∈ s implies True)
			assert ("some_rationals_ok", properties.is_in_ok (pq, s))
		end

	test_is_not_in
			-- Test {STS_RATIONAL_NUMBER}.is_not_in.
			-- Test {STI_RATIONAL_NUMBER}.is_not_in.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_not_in"
			testing: "covers/{STI_RATIONAL_NUMBER}.is_not_in"
		local
			pq: like rational_to_be_tested
			s: like some_rationals
			chk_ops, chk_tests: BOOLEAN
			n: like some_set.cardinality
		do
			pq := rational_to_be_tested
			s := some_rational_references / pq
			assert ("same_reference", pq ∉ s)
			assert ("same_reference ok", Properties.is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_rational_objects / pq.twin
			assert ("same_value", pq ∉ s)
			assert ("same_value", Properties.is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_rational_numbers / same_rational (pq)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_math_value", pq ∉ s)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("same_math_value", Properties.is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			chk_ops := {ISE_RUNTIME}.check_assert (False)
				s := some_strict_rational_numbers / same_rational (pq)
				n := # s
			chk_ops := {ISE_RUNTIME}.check_assert (chk_ops)
			if n > Max_asserted_elements then
				chk_tests := {ISE_RUNTIME}.check_assert (False)
			end
				assert ("same_strict_math_value", pq ∉ s)
			if chk_tests then
				chk_tests := {ISE_RUNTIME}.check_assert (True)
			end
			assert ("same_strict_math_value", Properties.is_not_in_ok (pq, s))

			pq := rational_to_be_tested
			s := some_rationals
			assert ("some_rationals", pq ∉ s implies True)
			assert ("some_rationals_ok", Properties.is_not_in_ok (pq, s))
		end

feature -- Test routines (Access)

	test_sign
			-- Test {STI_RATIONAL_NUMBER}.sign.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.sign"
		local
			pqs: like some_rational_set
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("pq.sign", attached pq.sign)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end

			from
				pq := rational_to_be_tested
			until
				pq < Zero
			loop
				pq := rational_to_be_tested
			end
			assert ("negative", pq.sign ≍ - One)

			pq := Zero
			assert ("unchanged zero", (- pq).sign ≍ Zero)
			assert ("just zero", pq.sign ≍ Zero)

			from
				pq := rational_to_be_tested
			until
				pq > Zero
			loop
				pq := rational_to_be_tested
			end
			assert ("positive", pq.sign ≍ One)
		end

	test_sign_bit
			-- Test {STI_RATIONAL_NUMBER}.sign_bit.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.sign_bit"
		local
			pqs: like some_rational_set
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("pq.sign_bit", attached pq.sign_bit)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end

			from
				pq := rational_to_be_tested
			until
				pq < Zero
			loop
				pq := rational_to_be_tested
			end
			assert ("negative", pq.sign_bit ≍ One)

			pq := Zero
			assert ("unchanged zero", (- pq).sign_bit ≍ Zero)
			assert ("just zero", pq.sign_bit ≍ Zero)

			from
				pq := rational_to_be_tested
			until
				pq > Zero
			loop
				pq := rational_to_be_tested
			end
			assert ("positive", pq.sign_bit ≍ Zero)
		end

	test_zero
			-- Test {STI_RATIONAL_NUMBER}.zero.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.zero"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("zero", attached pq.Zero)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end

			assert (
				"Min_value complex_zero_ok",
				properties.complex_zero_ok
					({like rational_to_be_tested}.Min_value, some_complex)
				)
			assert (
				"Min_value real_zero_ok",
				properties.real_zero_ok
					({like rational_to_be_tested}.Min_value, some_real)
				)
			assert (
				"Min_value",
				properties.zero_ok
					({like rational_to_be_tested}.Min_value, some_rational)
				)

			assert (
				"Zero complex_zero_ok",
				properties.complex_zero_ok
					({like rational_to_be_tested}.Zero, some_complex)
				)
			assert (
				"Zero real_zero_ok",
				properties.real_zero_ok
					({like rational_to_be_tested}.Zero, some_real)
				)
			assert (
				"Zero",
					properties.zero_ok (
						Zero,
						some_rational
						)
				)

			assert (
				"Max_value complex_zero_ok",
				properties.complex_zero_ok
					({like rational_to_be_tested}.Max_value, some_complex)
				)
			assert (
				"Max_value real_zero_ok",
				properties.real_zero_ok
					({like rational_to_be_tested}.Max_value, some_real)
				)
			assert (
				"Max_value",
				properties.zero_ok
					({like rational_to_be_tested}.Max_value, some_rational)
				)

			pq := rational_to_be_tested
			assert (
				"complex_zero_ok", properties.complex_zero_ok (pq, some_complex)
				)
			assert ("real_zero_ok", properties.complex_zero_ok (pq, some_real))
			assert ("zero_ok", properties.zero_ok (pq, some_rational))
		end

	test_one
			-- Test {STI_RATIONAL_NUMBER}.one.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.one"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("one", attached pq.One)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end

			assert (
				"Min_value complex_one_ok",
				properties.complex_one_ok
					({like rational_to_be_tested}.Min_value, some_complex)
				)
			assert (
				"Min_value real_one_ok",
				properties.real_one_ok
					({like rational_to_be_tested}.Min_value, some_real)
				)
			assert (
				"Min_value",
				properties.one_ok
					({like rational_to_be_tested}.Min_value, some_rational)
				)

			assert (
				"Zero complex_one_ok",
				properties.complex_one_ok
					({like rational_to_be_tested}.Zero, some_complex)
				)
			assert (
				"Zero real_one_ok",
				properties.real_one_ok
					({like rational_to_be_tested}.Zero, some_real)
				)
			assert ("Zero", properties.one_ok (Zero, some_rational))

			assert (
				"Max_value complex_one_ok",
				properties.complex_one_ok
					({like rational_to_be_tested}.Max_value, some_complex)
				)
			assert (
				"Max_value real_one_ok",
					properties.real_one_ok
						({like rational_to_be_tested}.Max_value, some_real)
					)
			assert (
				"Max_value",
				properties.one_ok
					({like rational_to_be_tested}.Max_value, some_rational)
				)

			pq := rational_to_be_tested
			assert
				("complex_one_ok", properties.complex_one_ok (pq, some_complex))
			assert ("real_one_ok", properties.complex_one_ok (pq, some_real))
			assert ("one_ok", properties.one_ok (pq, some_rational))
		end

	test_two
			-- Test {STI_RATIONAL_NUMBER}.two.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.two"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("two", attached pq.Two)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end

			assert (
				"Min_value",
				properties.two_ok ({like rational_to_be_tested}.Min_value)
				)
			assert (
				"Zero",
				properties.two_ok ({like rational_to_be_tested}.Zero)
				)
			assert (
				"Max_value",
				properties.two_ok ({like rational_to_be_tested}.Max_value)
				)
			assert ("two_ok", properties.two_ok (rational_to_be_tested))
		end

	test_imaginary_unit
			-- Test {STI_RATIONAL_NUMBER}.imaginary_unit.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.imaginary_unit"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("imaginary_unit", attached pq.imaginary_unit)
				assert ("imaginary_unit_ok", properties.imaginary_unit_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_machine_epsilon
			-- Test {STI_RATIONAL_NUMBER}.machine_epsilon.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.machine_epsilon" -- TODO: AutoTest gests confused by covers/{STI_RATIONAL_NUMBER}.Machine_epsilon
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("Machine_epsilon", attached pq.Machine_epsilon)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end

			pq := rational_to_be_tested
			assert (
				"srnp_machine_epsilon_ok",
				properties.srnp_machine_epsilon_ok (pq)
				)
		end

	test_ulp
			-- Test {STI_RATIONAL_NUMBER}.ulp.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.ulp"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("ulp", attached pq.ulp)
				print (pq); print ("; "); print (pq.ulp); print ('%N')
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end

			assert ("ulp_ok", properties.ulp_ok (rational_to_be_tested))
		end

	test_relative_epsilon
			-- Test {STI_RATIONAL_NUMBER}.relative_epsilon.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.relative_epsilon"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("relative_epsilon", attached pq.relative_epsilon)
				print (pq)
				print ("; ")
				print (pq.relative_epsilon)
				print ('%N')
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end

			assert (
				"relative_epsilon_ok",
				properties.relative_epsilon_ok (rational_to_be_tested)
				)
		end

	test_previous_float
			-- Test {STI_RATIONAL_NUMBER}.previous_float.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.previous_float"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("previous_float", attached pq.previous_float)
				print (pq); print ("; "); print (pq.previous_float); print ('%N')
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_next_float
			-- Test {STI_RATIONAL_NUMBER}.next_float.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.next_float"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q -- TODO: Take ℤ instead.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("next_float", attached pq.next_float)
				print (pq); print ("; "); print (pq.next_float); print ('%N')
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_min_value
			-- Test {STI_RATIONAL_NUMBER}.min_value.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.min_value"
		local
			pq: like rational_to_be_tested
		do
			pq := rational_to_be_tested
			assert ("min_value", attached pq.Min_value)
			assert ("min_value ok", attached properties.min_value_ok (pq))
		end

	test_max_value
			-- Test {STI_RATIONAL_NUMBER}.max_value.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.max_value"
		local
			pq: like rational_to_be_tested
		do
			pq := rational_to_be_tested
			assert ("max_value", attached pq.Max_value)
			assert ("max_value ok", attached properties.max_value_ok (pq))
		end

feature -- Test routines (Quality)

	test_is_real
			-- Test {STI_RATIONAL_NUMBER}.is_real.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_real"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q -- TODO: Take ℤ instead.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_real", pq.is_real)
				assert ("is_real_ok", properties.is_real_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_proper_real
			-- Test {STI_RATIONAL_NUMBER}.is_proper_real.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_proper_real"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			pq := Zero
			assert ("not pq.is_proper_real", not pq.is_proper_real)
			assert
				("not pq.is_proper_real ok", properties.is_proper_real_ok (pq))

			from
				pq := rational_to_be_tested
			until
				pq ≭ Zero
			loop
				pq := rational_to_be_tested
			end
			assert ("pq.is_proper_real", pq.is_proper_real)
			assert ("pq.is_proper_real ok", properties.is_proper_real_ok (pq))

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_proper_real", pq.is_proper_real implies True)
				assert ("is_proper_real ok", properties.is_proper_real_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_imaginary
			-- Test {STI_RATIONAL_NUMBER}.is_imaginary.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_imaginary"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			pq := Zero
			assert ("pq.is_imaginary", pq.is_imaginary)
			assert ("pq.is_imaginary ok", properties.is_imaginary_ok (pq))

			from
				pq := rational_to_be_tested
			until
				pq ≭ Zero
			loop
				pq := rational_to_be_tested
			end
			assert ("not pq.is_imaginary", not pq.is_imaginary)
			assert ("not pq.is_imaginary ok", properties.is_imaginary_ok (pq))

			from
				pqs := Q -- TODO: Take ℤ instead.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_imaginary", pq.is_imaginary implies True)
				assert ("is_imaginary ok", properties.is_imaginary_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_proper_imaginary
			-- Test {STI_RATIONAL_NUMBER}.is_proper_imaginary.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_proper_imaginary"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q -- TODO: Take ℤ instead.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_proper_imaginary", not pq.is_proper_imaginary)
				assert (
					"is_proper_imaginary_ok",
					properties.is_proper_imaginary_ok (pq)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_proper_complex
			-- Test {STI_RATIONAL_NUMBER}.is_proper_complex.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_proper_complex"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q -- TODO: Take ℤ instead.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_proper_complex", not pq.is_proper_complex)
				assert (
					"is_proper_complex_ok",
					properties.is_proper_complex_ok (pq)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_nan
			-- Test {STI_RATIONAL_NUMBER}.is_nan.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_nan"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_nan", not pq.is_nan)
				assert
					("real_is_nan_ok", properties.real_is_nan_ok (pq, some_real))
				assert ("is_nan_ok", properties.is_nan_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_negative_infinity
			-- Test {STI_RATIONAL_NUMBER}.is_negative_infinity.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_negative_infinity"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_negative_infinity", not pq.is_negative_infinity)
				assert (
					"real_is_negative_infinity_ok",
					properties.real_is_negative_infinity_ok (pq, some_real)
					)
				assert (
					"is_negative_infinity_ok",
					properties.is_negative_infinity_ok (pq)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_negative_zero
			-- Test {STI_RATIONAL_NUMBER}.is_negative_zero.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_negative_zero"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_negative_zero", not pq.is_negative_zero)
				assert (
					"is_negative_zero_ok",
					properties.is_negative_zero_ok (pq, some_real)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_positive_zero
			-- Test {STI_RATIONAL_NUMBER}.is_positive_zero.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_positive_zero"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			pq := Zero
			assert ("pq.is_positive_zero", pq.is_positive_zero)
			assert (
				"pq.is_positive_zero ok",
				properties.is_positive_zero_ok (pq, some_rational)
				)

			from
				pq := rational_to_be_tested
			until
				pq ≭ Zero
			loop
				pq := rational_to_be_tested
			end
			assert ("not pq.is_positive_zero", not pq.is_positive_zero)
			assert (
				"not pq.is_positive_zero ok",
				properties.is_positive_zero_ok (pq, some_real)
				)

			from
				pqs := Q -- TODO: Take ℤ instead.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_positive_zero", pq.is_positive_zero implies True)
				assert (
					"is_positive_zero_ok",
					properties.is_positive_zero_ok (pq, some_real)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_positive_infinity
			-- Test {STI_RATIONAL_NUMBER}.is_positive_infinity.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_positive_infinity"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_positive_infinity", not pq.is_positive_infinity)
				assert (
					"real_is_positive_infinity_ok",
					properties.real_is_positive_infinity_ok (pq, some_real)
					)
				assert (
					"is_positive_infinity_ok",
					properties.is_positive_infinity_ok (pq)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_infinite
			-- Test {STI_RATIONAL_NUMBER}.is_infinite.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_infinite"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q -- TODO: Take ℤ instead.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_infinite", not pq.is_infinite)
				assert (
					"complex_is_infinite ok",
					properties.complex_is_infinite_ok (pq, some_complex)
					)
				assert (
					"complex_is_infinite ok",
					properties.real_is_infinite_ok (pq, some_real)
					)
				assert ("is_infinite", properties.is_infinite_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_finite
			-- Test {STI_RATIONAL_NUMBER}.is_finite.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_finite"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_finite", pq.is_finite)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_is_integer
			-- Test {STS_REAL_NUMBER}.is_integer.
			-- Test {STS_RATIONAL_NUMBER}.real_is_integer.
			-- Test {STI_RATIONAL_NUMBER}.real_is_integer.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_integer"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_is_integer"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_is_integer"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
			pqt: like Q.as_tuple
			i: like some_set.cardinality
		do
			pqs := Q | agent {STS_RATIONAL_NUMBER}.real_is_integer
			pqt := pqs.as_tuple
			i := (next_random_item.as_natural_32 \\ pqt.n + 1)
				check
					positive: 0 < i -- next_random_item \\ pqt.n + 1 > 0
					small_enough:
						i ≤ pqt.n -- next_random_item \\ pqt.n + 1 ≤ pqt.n
				end
			pq := pqt [i]
			assert ("pq.real_is_integer", pq.real_is_integer)
			assert ("pq.real_is_integer ok", properties.real_is_integer_ok (pq))

			pqs := Q ∖ pqs
			pqt := pqs.as_tuple
			i := (next_random_item.as_natural_32 \\ pqt.n + 1)
				check
					positive: 0 < i -- next_random_item \\ pqt.n + 1 > 0
					small_enough:
						i ≤ pqt.n -- next_random_item \\ pqt.n + 1 ≤ pqt.n
				end
			pq := pqt [i]
			assert ("not pq.real_is_integer", not pq.real_is_integer)
			assert ("not pq.real_is_integer ok", properties.real_is_integer_ok (pq))

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("real_is_integer", pq.real_is_integer implies True)
				assert ("real_is_integer ok", properties.real_is_integer_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_rational
			-- Test {STI_RATIONAL_NUMBER}.is_rational.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_rational"
		local
			pq: like rational_to_be_tested
			pqs: like some_rational_set
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_rational", pq.is_rational)
				assert ("is_rational ok", properties.is_rational_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_integer
			-- Test {STI_RATIONAL_NUMBER}.is_integer.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.is_integer"
		local
			num, den, k: like some_integer
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				den := some_integer
				k := some_integer
			until
				den ≭ Zero and not properties.product_overflows (k, den)
			loop
				den := some_integer
				k := some_integer
			end
			num := k ⋅ den
				check
					good_divisor: num.divisible (den) -- den /= 0
				end
			pq := num / den
			assert ("pq.is_integer", pq.is_integer)
			assert ("pq.is_integer ok", properties.is_integer_ok (pq))

			from
				num := some_integer
				den := some_integer
			until
				den ≭ Zero and then (num \\ den) ≭ Zero
			loop
				num := some_integer
				den := some_integer
			end
				check
					good_divisor: num.divisible (den) -- den /= 0
				end
			pq := num / den
			assert ("not pq.is_integer", not pq.is_integer)
			assert ("not pq.is_integer ok", properties.is_integer_ok (pq))

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_integer", pq.is_integer implies True)
				assert ("is_integer ok", properties.is_integer_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_is_invertible
			-- Test {STI_RATIONAL_NUMBER}.real_is_invertible.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_is_invertible"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q -- TODO: Take ℤ instead.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("real_is_invertible", pq.real_is_invertible)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_invertible
			-- Test {STS_RATIONAL_NUMBER}.is_invertible.
			-- Test {STI_RATIONAL_NUMBER}.is_invertible.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_invertible"
			testing: "covers/{STI_RATIONAL_NUMBER}.is_invertible"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pq := rational_to_be_tested
			until
				pq ≭ Zero
			loop
				pq := rational_to_be_tested
			end
			assert ("pq.is_invertible", pq.is_invertible)

			pq := Zero
			assert ("not pq.is_invertible", not pq.is_invertible)

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("is_invertible", pq.is_invertible implies True)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_is_identical
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.is_identical.
			-- Test {STI_RATIONAL_NUMBER}.is_identical.
		note
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.is_identical"
			testing: "covers/{STI_RATIONAL_NUMBER}.is_identical"
		local
			i, j: like some_integer
			pq: like rational_to_be_tested
		do
			i := some_integer
			j := same_integer (i)
			create pq.make (i, j)
			assert ("pq.is_identical", pq.is_identical)
			assert (
				"pq.is_identical ok",
				homogeneous_pair_properties.is_identical_ok (pq)
				)

			from
				i := some_integer
				j := some_integer
			until
				i ≭ j
			loop
				j := some_integer
			end
			create pq.make (i, j)
			assert ("not pq.is_identical", not pq.is_identical)
			assert (
				"not pq.is_identical ok",
				homogeneous_pair_properties.is_identical_ok (pq)
				)

			pq := rational_to_be_tested
			assert ("is_identical", pq.is_identical implies True)
			assert (
				"is_identical_ok",
				homogeneous_pair_properties.is_identical_ok (pq)
				)
		end

feature -- Test routines (Output)

	test_out
			-- Test {STI_RATIONAL_NUMBER}.out.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.out"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("out", attached pq.out)
				print (pq.out) print ('%N')
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

feature -- Test routines (Comparison)

	test_complex_equals
			-- Test {STS_COMPLEX_NUMBER}.equals.
			-- Test {STS_REAL_NUMBER}.complex_equals.
			-- Test {STS_RATIONAL_NUMBER}.complex_equals.
			-- Test {STI_RATIONAL_NUMBER}.complex_equals.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.equals"
			testing: "covers/{STS_REAL_NUMBER}.complex_equals"
			testing: "covers/{STS_RATIONAL_NUMBER}.complex_equals"
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_equals"
		local
			pq: like rational_to_be_tested
			ab: like some_complex
		do
			pq := rational_to_be_tested
			assert ("same_entity", pq.complex_equals (pq))
			assert ("same_entity_ok", properties.complex_equals_ok (pq, pq, pq))

			pq := rational_to_be_tested
			ab := same_complex (pq)
			assert ("same_complex", pq.complex_equals (ab))
			assert (
				"same_complex_ok",
				properties.complex_equals_ok (pq, ab, same_complex (ab))
				)

			pq := rational_to_be_tested
			ab := some_complex
			assert ("complex_equals", pq.complex_equals (ab) implies True)
			assert (
				"complex_equals_ok",
				properties.complex_equals_ok (pq, ab, some_complex)
				)
		end

	test_complex_unequals
			-- Test {STS_COMPLEX_NUMBER}.unequals.
			-- Test {STS_REAL_NUMBER}.complex_unequals.
			-- Test {STS_RATIONAL_NUMBER}.complex_unequals.
			-- Test {STI_RATIONAL_NUMBER}.complex_unequals.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.unequals"
			testing: "covers/{STS_REAL_NUMBER}.complex_unequals"
			testing: "covers/{STS_RATIONAL_NUMBER}.complex_unequals"
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_unequals"
		local
			pq: like rational_to_be_tested
			ab: like some_complex
		do
			pq := rational_to_be_tested
			assert ("same_entity", not pq.complex_unequals (pq))
			assert ("same_entity_ok", properties.complex_unequals_ok (pq, pq))

			pq := rational_to_be_tested
			ab := same_complex (pq)
			assert ("same_complex", not pq.complex_unequals (ab))
			assert ("same_complex_ok", properties.complex_unequals_ok (pq, ab))

			pq := rational_to_be_tested
			ab := some_complex
			assert ("complex_equals", pq.complex_equals (ab) implies True)
			assert
				("complex_unequals_ok", properties.complex_unequals_ok (pq, ab))
		end

	test_real_equals
			-- Test {STS_REAL_NUMBER}.equals.
			-- Test {STS_RATIONAL_NUMBER}.real_equals.
			-- Test {STI_RATIONAL_NUMBER}.real_equals.
		note
			testing: "covers/{STS_REAL_NUMBER}.equals"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_equals"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_equals"
		local
			pqs: like Q.subset_anchor
			xs: like R.subset_anchor
			pq: like rational_to_be_tested
			x: like some_real
		do
			pq := rational_to_be_tested
			assert ("same_entity", pq.real_equals (pq))
			assert ("same_entity_ok", properties.real_equals_ok (pq, pq, pq))

			pq := rational_to_be_tested
			x := same_real (pq)
			assert ("same_real", pq.real_equals (x))
			assert ("same_real_ok", properties.real_equals_ok (pq, x, same_real (x)))

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				from
					pq := pqs.any
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert ("some_real", pq.real_equals (x) implies True)
					assert ("real_equals_ok", properties.real_equals_ok (pq, x, some_real))
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
				pqs := pqs.others
			variant
				pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_unequals
			-- Test {STS_REAL_NUMBER}.unequals.
			-- Test {STS_RATIONAL_NUMBER}.real_unequals.
			-- Test {STI_RATIONAL_NUMBER}.real_unequals.
		note
			testing: "covers/{STS_REAL_NUMBER}.unequals"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_unequals"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_unequals"
		local
			pq: like rational_to_be_tested
			x: like some_real
			pqs: like Q.subset_anchor
			xs: like R.subset_anchor
		do
			pq := rational_to_be_tested
			assert ("same_entity", not pq.real_unequals (pq))
			assert ("same_entity ok", properties.real_unequals_ok (pq, pq))

			pq := rational_to_be_tested
			x := same_real (pq)
			assert ("same_real", not (pq.real_unequals (x)))
			assert ("same_real ok", properties.real_unequals_ok (pq, x))

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				from
					pq := pqs.any
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert ("some_real", pq.real_unequals (x) implies True)
					assert ("real_unequals_ok", properties.real_unequals_ok (pq, x))
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
			variant
				pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
			end

		end

	test_real_is_less
			-- Test {STS_REAL_NUMBER}.is_less.
			-- Test {STS_RATIONAL_NUMBER}.real_is_less.
			-- Test {STI_RATIONAL_NUMBER}.real_is_less.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_less"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_is_less"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_is_less"
		local
			pq: like rational_to_be_tested
			x, y: like some_real
			xs: like R.subset_anchor
		do
			pq := rational_to_be_tested
			from
				y := some_real
			until
				y > Zero
			loop
				y := some_real
			end
			x := (y ∨ pq.ulp) + pq
			assert ("pq < x", pq.real_is_less (x))
			assert ("pq < x ok", properties.real_is_less_ok (pq, x, some_real))

			pq := rational_to_be_tested
			from
				y := some_real
			until
				y ≤ Zero
			loop
				y := some_real
			end
			x := y + pq
			assert ("not (pq < x)", not pq.real_is_less (x))
			assert (
				"not (pq < x) ok", properties.real_is_less_ok (pq, x, some_real)
				)

			from
				xs := R
			until
				xs.is_empty
			loop
				pq := rational_to_be_tested -- TODO: Take every possible rational.
				x := same_real (xs.any)
				assert ("real_is_less", pq.real_is_less (x) implies True)
				assert (
					"real_is_less_ok",
					properties.real_is_less_ok (pq, x, some_real)
					)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_real_is_less_equal
			-- Test {STS_REAL_NUMBER}.is_less_equal.
			-- Test {STS_RATIONAL_NUMBER}.real_is_less_equal.
			-- Test {STI_RATIONAL_NUMBER}.real_is_less_equal.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_less_equal"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_is_less_equal"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_is_less_equal"
		local
			pq: like rational_to_be_tested
			x, y: like some_real
			pqs: like Q.subset_anchor
			xs: like R.subset_anchor
		do
			pq := rational_to_be_tested
			from
				y := some_real
			until
				y ≥ Zero
			loop
				y := some_real
			end
			x := y + pq
			assert ("pq ≤ x", pq.real_is_less_equal (x))
			assert (
				"pq ≤ x ok", properties.real_is_less_equal_ok (pq, x, some_real)
				)

			pq := rational_to_be_tested
			from
				y := some_real
			until
				y < Zero
			loop
				y := some_real
			end
			x := (y ∧ -pq.ulp) + pq
			assert ("not (pq ≤ x)", not pq.real_is_less_equal (x))
			assert (
				"not (pq ≤ x) ok",
				properties.real_is_less_equal_ok (pq, x, some_real)
				)

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				from
					pq := pqs.any
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert ("real_is_less_equal", pq.real_is_less_equal (x) implies True)
					assert ("real_is_less_equal_ok", properties.real_is_less_equal_ok (pq, x, some_real))
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
				pqs := pqs.others
			variant
				pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_is_greater
			-- Test {STS_REAL_NUMBER}.is_greater.
			-- Test {STS_RATIONAL_NUMBER}.real_is_greater.
			-- Test {STI_RATIONAL_NUMBER}.real_is_greater.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_greater"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_is_greater"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_is_greater"
		local
			pq: like rational_to_be_tested
			x, y: like some_real
			xs: like R.subset_anchor
		do
			pq := rational_to_be_tested
			from
				y := some_real
			until
				y < Zero
			loop
				y := some_real
			end
			x := (y ∧ -pq.ulp) + pq
			assert ("pq > x", pq.real_is_greater (x))
			assert
				("pq > x ok", properties.real_is_greater_ok (pq, x, some_real))

			pq := rational_to_be_tested
			from
				y := some_real
			until
				y ≥ Zero
			loop
				y := some_real
			end
			x := y + pq
			assert ("not (pq > x)", not pq.real_is_greater (x))
			assert (
				"not (pq > x) ok",
				properties.real_is_greater_ok (pq, x, some_real)
				)

			from
				xs := R
			until
				xs.is_empty
			loop
				pq := rational_to_be_tested -- TODO: Take every possible rational.
				x := same_real (xs.any)
				assert ("real_is_greater", pq.real_is_greater (x) implies True)
				assert (
					"real_is_greater_ok",
					properties.real_is_greater_ok (pq, x, some_real)
					)
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_real_is_greater_equal
			-- Test {STS_REAL_NUMBER}.is_greater_equal.
			-- Test {STS_RATIONAL_NUMBER}.real_is_greater_equal.
			-- Test {STI_RATIONAL_NUMBER}.real_is_greater_equal.
		note
			testing: "covers/{STS_REAL_NUMBER}.is_greater_equal"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_is_greater_equal"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_is_greater_equal"
		local
			pq: like rational_to_be_tested
			x, y: like some_real
			xs: like R.subset_anchor
		do
			pq := rational_to_be_tested
			from
				y := some_real
			until
				y ≤ Zero
			loop
				y := some_real
			end
			x := y + pq
			assert ("pq ≥ x", pq.real_is_greater_equal (x))
			assert (
				"pq ≥ x ok",
				properties.real_is_greater_equal_ok (pq, x, some_real)
				)

			pq := rational_to_be_tested
			from
				y := some_real
			until
				y > Zero
			loop
				y := some_real
			end
			x := (y ∨ pq.ulp) + pq
			assert ("not (pq ≥ x)", not pq.real_is_greater_equal (x))
			assert (
				"not (pq ≥ x) ok",
				properties.real_is_greater_equal_ok (pq, x, some_real)
				)

			from
				xs := R
			until
				xs.is_empty
			loop
				pq := rational_to_be_tested -- TODO: Take every possible rational.
				x := same_real (xs.any)
				assert ("real_is_greater_equal", pq.real_is_greater_equal (x) implies True)
				assert ("real_is_greater_equal_ok", properties.real_is_greater_equal_ok (pq, x, some_real))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_real_three_way_comparison
			-- Test {STI_RATIONAL_NUMBER}.real_three_way_comparison.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_three_way_comparison"
		local
			pq: like rational_to_be_tested
			x, y, w: like some_real
			pqs: like Q.subset_anchor
			xs: like R.subset_anchor
		do
			pq := rational_to_be_tested
			w := some_real.abs ∨ pq.ulp
			y := w + pq
			assert ("negative", pq.real_three_way_comparison (y) ≍ - One)

			assert ("same_entity", pq.real_three_way_comparison (pq) ≍ Zero)
			assert (
				"same_real", pq.real_three_way_comparison (same_real (pq)) ≍ Zero
				)

			pq := rational_to_be_tested
			w := (- some_real.abs) ∧ (- pq.ulp)
			y := w + pq
			assert ("positive", pq.real_three_way_comparison (y) ≍ One)

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				from
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert (
						"real_three_way_comparison",
						attached pq.real_three_way_comparison (x)
						)
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
				pqs := pqs.others
			variant
				pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_min
			-- Test {STI_RATIONAL_NUMBER}.real_min.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_min"
		local
			pq: like rational_to_be_tested
			x, y, w: like some_real
			pqs: like Q.subset_anchor
			xs: like R.subset_anchor
		do
			pq := rational_to_be_tested
			w := some_real.abs ∨ Zero
			y := w + pq
			assert ("pq", pq.real_min (y) ≍ pq)
			assert ("pq ok", properties.real_min_ok (pq, y, some_real))

			pq := rational_to_be_tested
			w := (- some_real.abs) ∧ (- pq.ulp)
			y := w + pq
			assert ("y", pq.real_min (y) ≍ y)
			assert ("y ok", properties.real_min_ok (pq, y, some_real))

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				from
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert ("real_min", attached pq.real_min (x))
					assert (
						"real_min_ok", properties.real_min_ok (pq, x, some_real)
						)
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
			end
		end

	test_real_max
			-- Test {STI_RATIONAL_NUMBER}.real_max.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_max"
		local
			pq: like rational_to_be_tested
			x, y, w: like some_real
			pqs: like Q.subset_anchor
			xs: like R.subset_anchor
		do
			pq := rational_to_be_tested
			w := - some_real.abs
			y := w + pq
			assert ("pq", pq.real_max (y) ≍ pq)
			assert ("pq ok", properties.real_max_ok (pq, y, some_real))

			pq := rational_to_be_tested
			w := some_real.abs ∨ pq.ulp
			y := w + pq
			assert ("y", pq.real_max (y) ≍ y)
			assert ("y ok", properties.real_max_ok (pq, y, some_real))

			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				from
					xs := R
				until
					xs.is_empty
				loop
					x := same_real (xs.any)
					assert ("real_max", attached pq.real_max (x))
					assert (
						"real_max_ok", properties.real_max_ok (pq, x, some_real)
						)
					xs := xs.others
				variant
					xs_cardinality: {like some_set}.natural_as_integer (# xs)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs) -- TODO: Take Q instead.
			end
		end

	test_pair_equals
			-- Test {STS_ORDERED_PAIR}.equals.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.equals.
			-- Test {STI_RATIONAL_NUMBER}.pair_equals.
		note
			testing: "covers/{STS_ORDERED_PAIR}.equals"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.equals"
			testing: "covers/{STI_RATIONAL_NUMBER}.pair_equals"
		local
			pq: like rational_to_be_tested
			ab: like resources_ii.some_pair_ab
			i, j: like some_integer
		do
			pq := rational_to_be_tested
			assert ("same_entity", pq.pair_equals (pq))
			assert (
				"same_entity_ok",
				homogeneous_pair_properties.opp_equals_ok (pq, pq)
				)

			pq := rational_to_be_tested
			ab := resources_ii.same_pair_ab (pq)
			assert ("same_pair", pq.pair_equals (ab))
			assert (
				"same_pair_ok",
				homogeneous_pair_properties.opp_equals_ok (pq, ab)
				)

			pq := rational_to_be_tested
			ab := resources_ii.some_pair_ab
			assert ("some_pair", pq.pair_equals (ab) implies True)
			assert (
				"some_pair_ok",
				homogeneous_pair_properties.opp_equals_ok (pq, ab)
				)

			i := some_integer
			j := same_integer (i)
			assert (
				"ij_equals_ji",
				(create {like rational_to_be_tested}.make (i, j)).pair_equals
					(homogeneous_pair_properties.pair (j, i))
				)
			assert (
				"ij_equals_ji_ok", homogeneous_pair_properties.equals_ok (i, j)
				) -- TODO: It doesn't test rational numbers.

			i := some_integer
			j := some_integer
			assert (
				"equals",
				(create {like rational_to_be_tested}.make (i, j)).pair_equals
					(homogeneous_pair_properties.pair (j, i)) implies True
				)
			assert ("equals_ok", homogeneous_pair_properties.equals_ok (i, j))
		end

	test_pair_unequals
			-- Test {STS_ORDERED_PAIR}.unequals.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.unequals.
			-- Test {STI_RATIONAL_NUMBER}.pair_unequals.
		note
			testing: "covers/{STS_ORDERED_PAIR}.unequals"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.unequals"
			testing: "covers/{STI_RATIONAL_NUMBER}.pair_unequals"
		local
			pq: like rational_to_be_tested
			ab: like resources_ii.some_pair_ab
		do
			pq := rational_to_be_tested
			assert ("same_entity", not pq.pair_unequals (pq))
			assert (
				"same_entity_ok",
				homogeneous_pair_properties.unequals_ok (pq, pq)
				)

			pq := rational_to_be_tested
			ab := resources_ii.same_pair_ab (pq)
			assert ("same_pair", not pq.pair_unequals (ab))
			assert (
				"same_pair_ok",
				homogeneous_pair_properties.unequals_ok (pq, ab)
				)

			pq := rational_to_be_tested
			ab := resources_ii.some_pair_ab
			assert ("some_pair", not pq.pair_unequals (ab) implies True)
			assert (
				"some_pair_ok",
				homogeneous_pair_properties.unequals_ok (pq, ab)
				)
		end

	test_succeeds
			-- Test {STS_ORDERED_PAIR}.succeeds.
			-- Test {STS_HOMOGENEOUS_ORDERED_PAIR}.succeeds.
			-- Test {STI_RATIONAL_NUMBER}.succeeds.
		note
			testing: "covers/{STS_ORDERED_PAIR}.succeeds"
			testing: "covers/{STS_HOMOGENEOUS_ORDERED_PAIR}.succeeds"
			testing: "covers/{STI_RATIONAL_NUMBER}.succeeds"
		local
			pq: like rational_to_be_tested
			dsa_i: like resources_ii.some_preceding_pair_dsa_a
			i: like some_integer
		do
			pq := rational_to_be_tested
			dsa_i := resources_ii.some_preceding_pair_dsa_a (pq.p)
			assert ("pq ≽ dsa_i", pq ≽ dsa_i)
			assert (
				"pq ≽ dsa_i ok",
				homogeneous_pair_properties.succeeds_ok (pq, dsa_i)
				)

			pq := rational_to_be_tested
			from
				i := some_integer
			until
				i ≭ pq.p
			loop
				i := some_integer
			end
			dsa_i := resources_ii.some_preceding_pair_dsa_a (i)
			assert ("not (pq ≽ dsa_i)", not (pq ≽ dsa_i))
			assert (
				"not (pq ≽ dsa_i) ok",
				homogeneous_pair_properties.succeeds_ok (pq, dsa_i)
				)

			pq := rational_to_be_tested
			dsa_i := resources_ii.some_preceding_pair_dsa_a (some_integer)
			assert ("succeeds", pq ≽ dsa_i implies True)
			assert (
				"succeeds_ok",
				homogeneous_pair_properties.succeeds_ok (pq, dsa_i)
				)
		end

	test_equals
			-- Test {STS_RATIONAL_NUMBER}.equals.
			-- Test {STI_RATIONAL_NUMBER}.equals.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.equals"
			testing: "covers/{STI_RATIONAL_NUMBER}.equals"
		local
			pq_1: like rational_to_be_tested
			pq_2: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor
		do
			pq_1 := rational_to_be_tested
			assert ("same_entity", pq_1 ≍ pq_1)
			assert
				("same_entity_ok", properties.equals_ok (pq_1, pq_1, pq_1))

			pq_1 := rational_to_be_tested
			pq_2 := same_rational (pq_1)
			assert ("same_rational", pq_1 ≍ pq_2)
			assert (
				"same_rational_ok",
				properties.equals_ok (pq_1, pq_2, same_rational (pq_2))
				)

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("some_rational", pq_1 ≍ pq_2 implies True)
					assert ("equals_ok", properties.equals_ok (pq_1, pq_2, some_rational))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_unequals
			-- Test {STS_RATIONAL_NUMBER}.unequals.
			-- Test {STI_RATIONAL_NUMBER}.unequals.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.unequals"
			testing: "covers/{STI_RATIONAL_NUMBER}.unequals"
		local
			pq_1: like rational_to_be_tested
			pq_2: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			pq_1 := rational_to_be_tested
			assert ("same_entity", not (pq_1 ≭ pq_1))
			assert ("same_entity_ok", properties.unequals_ok (pq_1, pq_1))

			pq_1 := rational_to_be_tested
			pq_2 := same_rational (pq_1)
			assert ("same_rational", not (pq_1 ≭ pq_2))
			assert ("same_rational_ok", properties.unequals_ok (pq_1, pq_2))

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("some_rational", pq_1 ≭ pq_2 implies True)
					assert ("unequals_ok", properties.unequals_ok (pq_1, pq_2))
					pqs_2 := pqs_2.others
				variant
					cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_is_less
			-- Test {STS_RATIONAL_NUMBER}.is_less.
			-- Test {STI_RATIONAL_NUMBER}.is_less.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_less"
			testing: "covers/{STI_RATIONAL_NUMBER}.is_less"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_m: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor
		do
			pq_2 := some_rational
			from
				pq_m := some_rational
			until
				pq_m ≭ pq_2
			loop
				pq_m := some_rational
			end
			pq_1 := pq_2 ∧ pq_m
			pq_2 := pq_2 ∨ pq_m
			assert ("pq_1 < pq_2", pq_1 < pq_2)
			assert (
				"pq_1 < pq_2 ok",
				properties.is_less_ok (pq_1, pq_2, some_rational)
				)

			pq_2 := some_rational
			pq_m := some_rational
			pq_1 := pq_2 ∨ pq_m
			pq_2 := pq_2 ∧ pq_m
			assert ("not (pq_1 < pq_2)", not (pq_1 < pq_2))
			assert (
				"not (pq_1 < pq_2) ok",
				properties.is_less_ok (pq_1, pq_2, some_rational)
				)

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("is_less", pq_1 < pq_2 implies True)
					assert (
						"is_less_ok",
						properties.is_less_ok (pq_1, pq_2, some_rational)
						)
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality:
						{like some_set}.natural_as_integer (# pqs_2)
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality:
					{like some_set}.natural_as_integer (# pqs_1)
			end
		end

	test_is_less_equal
			-- Test {STS_RATIONAL_NUMBER}.is_less_equal.
			-- Test {STI_RATIONAL_NUMBER}.is_less_equal.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_less_equal"
			testing: "covers/{STI_RATIONAL_NUMBER}.is_less_equal"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_m: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			pq_2 := some_rational
			pq_m := some_rational
			pq_1 := pq_2 ∧ pq_m
			pq_2 := pq_2 ∨ pq_m
			assert ("pq_1 ≤ pq_2", pq_1 ≤ pq_2)
			assert ("pq_1 ≤ pq_2 ok", properties.is_less_equal_ok (pq_1, pq_2, some_rational))

			pq_2 := some_rational
			from
				pq_m := some_rational
			until
				pq_m ≭ pq_2
			loop
				pq_m := some_rational
			end
			pq_1 := pq_2 ∨ pq_m
			pq_2 := pq_2 ∧ pq_m
			assert ("not (pq_1 ≤ pq_2)", not (pq_1 ≤ pq_2))
			assert ("not (pq_1 ≤ pq_2) ok", properties.is_less_equal_ok (pq_1, pq_2, some_rational))

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("is_less_equal", pq_1 ≤ pq_2 implies True)
					assert ("is_less_equal_ok", properties.is_less_equal_ok (pq_1, pq_2, some_rational))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_is_greater
			-- Test {STS_RATIONAL_NUMBER}.is_greater.
			-- Test {STI_RATIONAL_NUMBER}.is_greater.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_greater"
			testing: "covers/{STI_RATIONAL_NUMBER}.is_greater"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_m: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			pq_2 := some_rational
			from
				pq_m := some_rational
			until
				pq_m ≭ pq_2
			loop
				pq_m := some_rational
			end
			pq_1 := pq_2 ∨ pq_m
			pq_2 := pq_2 ∧ pq_m
			assert ("pq_1 > pq_2", pq_1 > pq_2)
			assert ("pq_1 > pq_2 ok", properties.is_greater_ok (pq_1, pq_2, some_rational))

			pq_2 := some_rational
			pq_m := some_rational
			pq_1 := pq_2 ∧ pq_m
			pq_2 := pq_2 ∨ pq_m
			assert ("not (pq_1 > pq_2)", not (pq_1 > pq_2))
			assert ("not (pq_1 > pq_2) ok", properties.is_greater_ok (pq_1, pq_2, some_rational))

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("is_greater", pq_1 > pq_2 implies True)
					assert ("is_greater_ok", properties.is_greater_ok (pq_1, pq_2, some_rational))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_is_greater_equal
			-- Test {STS_RATIONAL_NUMBER}.is_greater_equal.
			-- Test {STI_RATIONAL_NUMBER}.is_greater_equal.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.is_greater_equal"
			testing: "covers/{STI_RATIONAL_NUMBER}.is_greater_equal"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_m: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			pq_2 := some_rational
			pq_m := some_rational
			pq_1 := pq_2 ∨ pq_m
			pq_2 := pq_2 ∧ pq_m
			assert ("pq_1 ≥ pq_2", pq_1 ≥ pq_2)
			assert ("pq_1 ≥ pq_2 ok", properties.is_greater_equal_ok (pq_1, pq_2, some_rational))

			pq_2 := some_rational
			from
				pq_m := some_rational
			until
				pq_m ≭ pq_2
			loop
				pq_m := some_rational
			end
			pq_1 := pq_2 ∧ pq_m
			pq_2 := pq_2 ∨ pq_m
			assert ("not (pq_1 ≥ pq_2)", not (pq_1 ≥ pq_2))
			assert ("not (pq_1 ≥ pq_2) ok", properties.is_greater_equal_ok (pq_1, pq_2, some_rational))

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("is_greater_equal", pq_1 ≥ pq_2 implies True)
					assert ("is_greater_equal_ok", properties.is_greater_equal_ok (pq_1, pq_2, some_rational))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_three_way_comparison
			-- Test {STS_RATIONAL_NUMBER}.three_way_comparison.
			-- Test {STI_RATIONAL_NUMBER}.three_way_comparison.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.three_way_comparison"
			testing: "covers/{STI_RATIONAL_NUMBER}.three_way_comparison"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_m: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			pq_2 := some_rational
			from
				pq_m := some_rational
			until
				pq_m ≭ pq_2
			loop
				pq_m := some_rational
			end
			pq_1 := pq_2 ∧ pq_m
			pq_2 := pq_2 ∨ pq_m
			assert ("negative", pq_1 ⋚ pq_2 ≍ - One)

			pq_1 := rational_to_be_tested
			assert ("same_entity", pq_1 ⋚ pq_1 ≍ Zero)
			assert ("same_rational", pq_1 ⋚ same_rational (pq_1) ≍ Zero)

			pq_2 := some_rational
			from
				pq_m := some_rational
			until
				pq_m ≭ pq_2
			loop
				pq_m := some_rational
			end
			pq_1 := pq_2 ∨ pq_m
			pq_2 := pq_2 ∧ pq_m
			assert ("positive", pq_1 ⋚ pq_2 ≍ One)

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("three_way_comparison", attached (pq_1 ⋚ pq_2))
					pqs_2 := pqs_2.others
				variant
					cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_min
			-- Test {STI_RATIONAL_NUMBER}.min.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.min"
		local
			pq_1: like rational_to_be_tested
			pq_2: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("min", attached (pq_1 ∧ pq_2))
					assert ("min_ok", properties.min_ok (pq_1, pq_2, some_rational))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_max
			-- Test {STI_RATIONAL_NUMBER}.max.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.max"
		local
			pq_1: like rational_to_be_tested
			pq_2: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("max", attached (pq_1 ∨ pq_2))
					assert ("max_ok", properties.max_ok (pq_1, pq_2, some_rational))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

feature -- Test routines (Relationship)

	test_complex_divisible
			-- Test {STS_COMPLEX_NUMBER}.divisible.
			-- Test {STS_REAL_NUMBER}.complex_divisible.
			-- Test {STS_RATIONAL_NUMBER}.complex_divisible.
			-- Test {STI_RATIONAL_NUMBER}.complex_divisible.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.divisible"
			testing: "covers/{STS_REAL_NUMBER}.complex_divisible"
			testing: "covers/{STS_RATIONAL_NUMBER}.complex_divisible"
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_divisible"
		local
			pqs: like some_rational_set
			zs: like Quick_C.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				from
					zs := Quick_C
				until
					zs.is_empty
				loop
					pq := pqs.any
					assert ("complex_divisible", pq.complex_divisible (zs.any))
					zs := zs.others
				variant
					zs_cardinality: {like some_set}.natural_as_integer (# zs)
				end
				pqs := pqs.others
			variant
				pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_divisible
			-- Test {STS_REAL_NUMBER}.divisible.
			-- Test {STS_RATIONAL_NUMBER}.real_divisible.
			-- Test {STI_RATIONAL_NUMBER}.real_divisible.
		note
			testing: "covers/{STS_REAL_NUMBER}.divisible"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_divisible"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_divisible"
		local
			pq: like rational_to_be_tested
			x: like some_real
			xs: like R.subset_anchor
		do
			from
				xs := R
			until
				xs.is_empty
			loop
				pq := rational_to_be_tested -- TODO: Take every possible rational.
				x := same_real (xs.any)
				assert ("real_divisible", pq.real_divisible (x))
				xs := xs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# xs)
			end
		end

	test_multipliable
			-- Test {STI_RATIONAL_NUMBER}.multipliable.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.multipliable"
		local
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
			divisors: like z.subset_anchor.n_tuple_anchor
			pq_1: like rational_to_be_tested
			pq_2: like some_rational
			p1, p2, q1, q2, j: like some_integer
			k: like some_n_tuple.n
		do
			divisors := (
				Z.Without_zero | agent (i: STS_INTEGER_NUMBER): BOOLEAN
					do
							check
								good_divisor: -- i ∈ Z.Without_zero
									{STI_INTEGER_NUMBER}.Min_value.divisible (i)
							end
						Result := ({STI_INTEGER_NUMBER}.Min_value \\ i) ≍ Zero
					end
				).as_tuple
			k := next_random_item.as_natural_32 \\ divisors.n + 1
				check
						-- As assigned above.		
					positive: 0 < k
					small_enough: k ≤ divisors.n
				end
			j := divisors [k]
				check
					good_divisor_j: -- j ∈ Z.Without_zero
						{STI_INTEGER_NUMBER}.Min_value.divisible (j)
				end
			q1 := {STI_INTEGER_NUMBER}.Min_value // j
				check
					good_divisor_q1: -- {STI_INTEGER_NUMBER}.Min_value // j /= 0
						{STI_INTEGER_NUMBER}.Min_value.divisible (q1)
				end
			from
				q2 := Two × ({STI_INTEGER_NUMBER}.Min_value // q1) × some_integer
			until
				q2 ≭ Zero -- Which is possible upon an overflow.
			loop
				q2 := Two × ({STI_INTEGER_NUMBER}.Min_value // q1) × some_integer
			end
			p2 := Two × some_integer + One.p
			p1 := Two × some_integer + One.p
				check
					good_divisor_1: p1.divisible (q1) -- good_divisor_q1
				end
			pq_1 := p1 / q1
				check
					good_divisor_1: p2.divisible (q2) -- q2 ≭ Zero
				end
			pq_2 := p2 / q2
			assert (
				"not pq_1.multipliable (pq_2)",
				not pq_1.multipliable (pq_2)
				)
			assert (
				"not pq_1.multipliable (pq_2) ok",
				properties.expanded_multipliable_ok (pq_1, pq_2)
				)

				-- TODO: Take a guaranteed valid product.

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				from
					pq_1 := pqs_1.any
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := pqs_2.any
					assert (
						"multipliable",
						pq_1.multipliable (pq_2) implies True
						)
					assert (
						"multipliable_ok",
						properties.expanded_multipliable_ok (pq_1, pq_2)
						)
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality:
						{like some_set}.natural_as_integer (# pqs_2)
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality: {like some_set}.natural_as_integer (# pqs_1)
			end
		end

	test_divisible
			-- Test {STS_RATIONAL_NUMBER}.divisible.
			-- Test {STI_RATIONAL_NUMBER}.divisible.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.divisible"
			testing: "covers/{STI_RATIONAL_NUMBER}.divisible"
		local
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
			divisors: like z.subset_anchor.n_tuple_anchor
			pq_1: like rational_to_be_tested
			pq_2: like some_rational
			p1, p2, q1, q2, j: like some_integer
			k: like some_n_tuple.n
		do
			divisors := (
				Z.Without_zero | agent (i: STS_INTEGER_NUMBER): BOOLEAN
					do
							check
								good_divisor: -- i ∈ Z.Without_zero
									{STI_INTEGER_NUMBER}.Min_value.divisible (i)
							end
						Result := ({STI_INTEGER_NUMBER}.Min_value \\ i) ≍ Zero
					end
				).as_tuple
			k := next_random_item.as_natural_32 \\ divisors.n + 1
				check
						-- As assigned above.		
					positive: 0 < k
					small_enough: k ≤ divisors.n
				end
			j := divisors [k]
				check
					good_divisor_j: -- j ∈ Z.Without_zero
						{STI_INTEGER_NUMBER}.Min_value.divisible (j)
				end
			q1 := {STI_INTEGER_NUMBER}.Min_value // j
				check
					good_divisor_q1: -- {STI_INTEGER_NUMBER}.Min_value // j /= 0
						{STI_INTEGER_NUMBER}.Min_value.divisible (q1)
				end
			p2 := Two × ({STI_INTEGER_NUMBER}.Min_value // q1) × some_integer
			q2 := Two × some_integer + One.p
			p1 := Two × some_integer + One.p
				check
					good_divisor_1: p1.divisible (q1) -- good_divisor_q1
				end
			pq_1 := p1 / q1
				check
					good_divisor_1: -- q2 = 2 × some_integer + 1 /= 0
						p2.divisible (q2)
				end
			pq_2 := p2 / q2
			assert ("not pq_1.divisible (pq_2)", not pq_1.divisible (pq_2))
			assert ("not pq_1.divisible (pq_2) ok", properties.divisible_ok (pq_1, pq_2))

			pq_1 := rational_to_be_tested
			from
				pq_2 := some_rational
			invariant
				good_divisor_1: pq_1.q.divisible -- pq_2.q, pq_1.q /= 0
					({STI_RATIONAL_NUMBER}.gcd (pq_2.p, pq_1.q))
				good_divisor_2: pq_2.p ≭ Zero implies pq_2.p.divisible
					({STI_RATIONAL_NUMBER}.gcd (pq_1.p, pq_2.p))
			until
				pq_2.p ≭ Zero and then (
				(pq_1.q // {STI_RATIONAL_NUMBER}.gcd (pq_2.q, pq_1.q)) ⋅
				(pq_2.p // {STI_RATIONAL_NUMBER}.gcd (pq_1.p, pq_2.p))
				) ≭ Zero
			loop
				pq_2 := some_rational
			end
			assert ("pq_1.divisible (pq_2)", pq_1.divisible (pq_2))
			assert ("pq_1.divisible (pq_2) ok", properties.divisible_ok (pq_1, pq_2))

			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("divisible", pq_1.divisible (pq_2) implies True)
					assert ("divisible_ok", properties.divisible_ok (pq_1, pq_2))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality: {like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality: {like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

feature -- Test routines (Operation)

	test_composed
			-- Test {STI_RATIONAL_NUMBER}.composed.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.composed"
		local
			pq: like rational_to_be_tested
			aa_2, aa_1: like resources_ii.some_preceding_pair_aa
			ba: like resources_ii.some_preceding_pair_ba
			bb: like resources_ii.some_preceding_pair_bb
		do
			pq := rational_to_be_tested
			aa_2 := resources_ii.some_preceding_pair_aa (pq.p)
			assert ("pq_succeeds_aa_2", pq ≽ aa_2)
			assert ("pq_oh_aa_2", attached (pq ∘ aa_2))
			aa_1 := resources_ii.some_preceding_pair_aa (aa_2.first)
			assert ("aa_2_succeeds_aa_1", aa_2 ≽ aa_1)
			assert (
				"pq_oh_aa_2_aa_1_ok",
				homogeneous_pair_properties.composed_ok (pq, aa_2, aa_1)
				)
			assert (
				"pq_o_aa_2_aa_1_ok",
				composable_pair_properties.composed_with_homogeneous_ok
					(pq, aa_2, aa_1)
				)

			pq := rational_to_be_tested
			aa_2 := resources_ii.some_preceding_pair_aa (some_integer)
			aa_1 := resources_ii.some_preceding_pair_aa (some_integer)
			assert ("composed", pq ≽ aa_2 implies attached (pq ∘ aa_2))
			assert (
				"composed_ok",
				homogeneous_pair_properties.composed_ok (pq, aa_2, aa_1)
				)
			assert (
				"composable_composed_ok",
				composable_pair_properties.composed_with_homogeneous_ok
					(pq, aa_2, aa_1)
				)

			ba := resources_ii.some_preceding_pair_ba (pq.p)
			assert ("pq_succeeds_ba", pq ≽ ba)
			assert ("pq_oi_ba", attached (pq ∘ ba))
			bb := resources_ii.some_preceding_pair_bb (ba.first)
			assert (
				"pq_oi_ba_bb_ok",
				homogeneous_pair_properties.composed_ok (pq, ba, bb)
				)
			assert (
				"pq_o_ba_bb_ok",
				composable_pair_properties.composed_with_homogeneous_ok
					(pq, ba, bb)
				)

			bb := resources_ii.some_preceding_pair_bb (some_integer)
			ba := resources_ii.some_preceding_pair_ba (some_integer)
			pq := rational_to_be_tested
			assert (
				"composed_with_inverse_like",
				pq ≽ ba implies attached (pq ∘ ba)
				)
			assert (
				"composed_with_inverse_like_ok",
				homogeneous_pair_properties.composed_ok (pq, ba, bb)
				)
			assert (
				"composable_composed_ok",
				composable_pair_properties.composed_with_homogeneous_ok
					(pq, ba, bb)
				)
		end

	test_composable_composed
			-- Test {STI_RATIONAL_NUMBER}.composable_composed.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.composable_composed"
		local
			pq: like rational_to_be_tested
			aa_2, aa_1: like resources_ii.some_preceding_pair_aa
			ba: like resources_ii.some_preceding_pair_ba
			bb: like resources_ii.some_preceding_pair_bb
		do
			pq := rational_to_be_tested
			aa_2 := resources_ii.some_preceding_pair_aa (pq.p)
			assert ("pq_succeeds_aa_2", pq ≽ aa_2)
			assert ("pq_oh_aa_2", attached pq.composable_composed (aa_2))
			aa_1 := resources_ii.some_preceding_pair_aa (aa_2.first)
			assert ("aa_2_succeeds_aa_1", aa_2 ≽ aa_1)
			assert (
				"pq_oh_aa_2_aa_1_ok",
				homogeneous_pair_properties.composed_with_inverse_like_ok
					(pq, aa_2, aa_1)
				)
			assert (
				"pq_o_aa_2_aa_1_ok",
				composable_pair_properties.composed_ok (pq, aa_2, aa_1)
				)
			assert (
				"pq_o_aa_2_aa_1_ok",
				composable_pair_properties.composed_with_inverse_like_ok
					(pq, aa_2, aa_1)
				)

			pq := rational_to_be_tested
			aa_2 := resources_ii.some_preceding_pair_aa (some_integer)
			aa_1 := resources_ii.some_preceding_pair_aa (some_integer)
			assert (
				"composable_composed",
				pq ≽ aa_2 implies attached pq.composable_composed (aa_2)
				)
			assert (
				"composable_composed_ok",
				homogeneous_pair_properties.composed_with_inverse_like_ok
					(pq, aa_2, aa_1)
				)
			assert (
				"composable_composable_composed_ok",
				composable_pair_properties.composed_ok (pq, aa_2, aa_1)
				)
			assert (
				"composable_composable_composed_ok",
				composable_pair_properties.composed_with_inverse_like_ok
					(pq, aa_2, aa_1)
				)

			ba := resources_ii.some_preceding_pair_ba (pq.p)
			assert ("pq_succeeds_ba", pq ≽ ba)
			assert ("pq_oi_ba", attached pq.composable_composed (ba))
			bb := resources_ii.some_preceding_pair_bb (ba.first)
			assert (
				"pq_oi_ba_bb_ok",
				homogeneous_pair_properties.composed_with_inverse_like_ok
					(pq, ba, bb)
				)
			assert (
				"pq_o_ba_bb_ok",
				composable_pair_properties.composed_ok (pq, ba, bb)
				)
			assert (
				"pq_o_ba_bb_ok",
				composable_pair_properties.composed_with_inverse_like_ok
					(pq, ba, bb)
				)

			bb := resources_ii.some_preceding_pair_bb (some_integer)
			ba := resources_ii.some_preceding_pair_ba (some_integer)
			pq := rational_to_be_tested
			assert (
				"composable_composed_with_inverse_like",
				pq ≽ ba implies attached pq.composable_composed (ba)
				)
			assert (
				"composable_composed_with_inverse_like_ok",
				homogeneous_pair_properties.composed_with_inverse_like_ok
					(pq, ba, bb)
				)
			assert (
				"composable_composable_composed_ok",
				composable_pair_properties.composed_ok (pq, ba, bb)
				)
			assert (
				"composable_composable_composed_ok",
				composable_pair_properties.composed_with_inverse_like_ok
					(pq, ba, bb)
				)
		end

	test_pair_inverse
			-- Test {STI_RATIONAL_NUMBER}.pair_inverse.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.pair_inverse"
		local
			pq: like rational_to_be_tested
		do
			pq := rational_to_be_tested
			assert ("pair_inverse", attached pq.pair_inverse)
			assert (
				"homogeneous pair_inverse ok",
				homogeneous_pair_properties.inverse_ok (pq)
				)
		end

	test_composable_pair_inverse
			-- Test {STI_RATIONAL_NUMBER}.composable_pair_inverse.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.composable_pair_inverse"
		local
			pq: like rational_to_be_tested
		do
			pq := rational_to_be_tested
			assert
				("composable_pair_inverse", attached pq.composable_pair_inverse)
			assert (
				"composable pair_inverse ok",
				composable_pair_properties.inverse_ok (pq)
				)
		end

	test_argument
			-- Test {STI_RATIONAL_NUMBER}.argument.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.argument"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q  -- TODO: Replace by Q.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("argument", attached pq.argument)
				assert ("srnp_argument_ok", properties.srnp_argument_ok (pq))
				assert ("argument_ok", properties.argument_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_phase
			-- Test {STI_RATIONAL_NUMBER}.phase.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.phase"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q  -- TODO: Replace by Q.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("phase", attached pq.phase)
				assert ("srnp_phase_ok", properties.srnp_phase_ok (pq))
				assert ("phase_ok", properties.phase_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_conjugate
			-- Test {STI_RATIONAL_NUMBER}.conjugate.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.conjugate"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q  -- TODO: Replace by Q.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("conjugate", attached pq.conjugate)
				assert
					("conjugate_ok", properties.conjugate_ok (pq, some_complex))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_complex_plus
			-- Test {STI_RATIONAL_NUMBER}.complex_plus.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_plus"
		local
			pqs: like Q.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			pq: like rational_to_be_tested
			ab: like some_complex
		do
			from
				pqs := Q  -- TODO: Replace by Q.
				from
					zt0 := Quick_C.as_tuple
				until
					zt0.n × # pqs ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						zt0 := zt0.suffix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# pqs * zt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					pq := pqs.any
					ab := same_complex (zt.first)
					assert ("complex_plus", attached pq.complex_plus (ab))
					assert
						("complex_plus_ok", properties.complex_plus_ok (pq, ab))
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_complex_minus
			-- Test {STI_RATIONAL_NUMBER}.complex_minus.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_minus"
		local
			pqs: like Q.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			pq: like rational_to_be_tested
			ab: like some_complex
		do
			from
				pqs := Q  -- TODO: Replace by Q.
				from
					zt0 := Quick_C.as_tuple
				until
					zt0.n × # pqs ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						zt0 := zt0.suffix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# pqs * zt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					pq := pqs.any
					ab := same_complex (zt.first)
					assert ("complex_minus", attached pq.complex_minus (ab))
					assert (
						"complex_minus_ok", properties.complex_minus_ok (pq, ab)
						)
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_complex_product
			-- Test {STI_RATIONAL_NUMBER}.complex_product.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_product"
		local
			pqs: like Q.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			pq: like rational_to_be_tested
			ab: like some_complex
		do
			from
				pqs := Q  -- TODO: Replace by Q.
				from
					zt0 := Quick_C.as_tuple
				until
					zt0.n × # pqs ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						zt0 := zt0.suffix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# pqs * zt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					pq := pqs.any
					ab := same_complex (zt.first)
					assert ("complex_product", attached pq.complex_product (ab))
					assert (
						"complex_product_ok",
						properties.complex_product_ok (pq, ab, some_complex)
						)
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_complex_quotient
			-- Test {STI_RATIONAL_NUMBER}.complex_quotient.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.complex_quotient"
		local
			pqs: like Q.subset_anchor
			zt0, zt: like Quick_C.as_tuple
			pq: like rational_to_be_tested
			ab: like some_complex
		do
			from
				pqs := Q  -- TODO: Replace by Q.
				from
					zt0 := Quick_C.as_tuple
				until
					zt0.n × # pqs ≤ 1_958 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						zt0 := zt0.prefix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						zt0 := zt0.suffix (
							(next_random_item \\ (zt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# pqs * zt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					zt := zt0
				until
					zt.is_unit
				loop
					pq := pqs.any
					ab := same_complex (zt.first)
					assert
						("complex_quotient", attached pq.complex_quotient (ab))
					assert (
						"complex_quotient_ok",
						properties.complex_quotient_ok (pq, ab, some_complex)
						)
					zt := zt.tail
				variant
					n: {like some_set}.natural_as_integer (zt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_plus
			-- Test {STS_REAL_NUMBER}.plus.
			-- Test {STS_RATIONAL_NUMBER}.real_plus.
			-- Test {STI_RATIONAL_NUMBER}.real_plus.
		note
			testing: "covers/{STS_REAL_NUMBER}.plus"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_plus"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_plus"
		local
			pqs: like Q.subset_anchor
			xt0, xt: like R.as_tuple
			pq: like rational_to_be_tested
			x: like some_real
		do
			from
				pqs := Q
				from
					xt0 := R.as_tuple
				until
					xt0.n × # pqs ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					else
						xt0 := xt0.suffix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# pqs * xt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					pq := pqs.any
					x := same_real (xt.first)
					assert ("real_plus", attached pq.real_plus (x))
					assert ("real_plus_ok", properties.real_plus_ok (pq, x, some_real))
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_minus
			-- Test {STS_REAL_NUMBER}.minus.
			-- Test {STS_RATIONAL_NUMBER}.real_minus.
			-- Test {STI_RATIONAL_NUMBER}.real_minus.
		note
			testing: "covers/{STS_REAL_NUMBER}.minus"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_minus"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_minus"
		local
			pqs: like Q.subset_anchor
			xt0, xt: like R.as_tuple
			pq: like rational_to_be_tested
			x: like some_real
		do
			from
				pqs := Q
				from
					xt0 := R.as_tuple
				until
--					xt0.n × # pqs ≤ 83_494 -- TODO: Syntax error!
					xt0.n × # pqs ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					else
						xt0 := xt0.suffix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# pqs * xt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					pq := pqs.any
					x := same_real (xt.first)
					assert ("real_minus", attached pq.real_minus (x))
					assert ("real_minus_ok", properties.real_minus_ok (pq, x, some_real))
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_opposite
			-- Test {STS_REAL_NUMBER}.opposite.
			-- Test {STI_RATIONAL_NUMBER}.real_opposite.
		note
			testing: "covers/{STS_REAL_NUMBER}.opposite"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_opposite"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("real_opposite", attached pq.real_opposite)
				assert (
					"real_opposite_ok",
					properties.real_opposite_ok (rational_to_be_tested)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_product
			-- Test {STS_REAL_NUMBER}.product.
			-- Test {STS_RATIONAL_NUMBER}.real_product.
			-- Test {STI_RATIONAL_NUMBER}.real_product.
		note
			testing: "covers/{STS_REAL_NUMBER}.product"
			testing: "covers/{STS_RATIONAL_NUMBER}.real_product"
			testing: "covers/{STI_RATIONAL_NUMBER}.real_product"
		local
			pqs: like Q.subset_anchor
			xt0, xt: like R.as_tuple
			pq: like rational_to_be_tested
			x: like some_real
		do
			from
				pqs := Q
				from
					xt0 := R.as_tuple
				until
					xt0.n × # pqs ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					else
						xt0 := xt0.suffix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# pqs * xt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					pq := pqs.any
					x := same_real (xt.first)
					assert ("real_product", attached pq.real_product (x))
					assert ("real_product_ok", properties.real_product_ok (pq, x, some_real))
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_quotient
			-- Test {STI_RATIONAL_NUMBER}.real_quotient.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_quotient"
		local
			pqs: like Q.subset_anchor
			xt0, xt: like R.as_tuple
			pq: like rational_to_be_tested
			x: like some_real
		do
			from
				pqs := Q
				from
					xt0 := R.as_tuple
				until
					xt0.n × # pqs ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix (
							(next_random_item \\ (xt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					else
						xt0 := xt0.suffix (
							(next_random_item \\ (xt0.n.as_integer_32 + 1)).
								as_natural_32
							)
					end
				end
				print (# pqs * xt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					pq := pqs.any
					x := same_real (xt.first)
					assert ("real_quotient", attached pq.real_quotient (x))
					assert (
						"real_quotient_ok",
						properties.real_quotient_ok (pq, x, some_real)
						)
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_reciprocal
			-- Test {STI_RATIONAL_NUMBER}.real_reciprocal.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_reciprocal"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q  -- TODO: Replace by Q.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("real_reciprocal", attached pq.real_reciprocal)
				assert (
					"real_reciprocal_ok",
					properties.real_reciprocal_ok (pq, some_real)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_real_inverse
			-- Test {STI_RATIONAL_NUMBER}.real_inverse.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_inverse"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q  -- TODO: Replace by Q.
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("real_inverse", attached pq.real_inverse)
				assert (
					"real_inverse_ok",
					properties.real_inverse_ok (pq, some_real)
					)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_modulus
			-- Test {STI_RATIONAL_NUMBER}.modulus.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.modulus"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			assert ("modulus", attached pq.modulus)
			assert ("modulus_ok", properties.modulus_ok (pq))
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("modulus", attached pq.modulus)
				assert ("modulus_ok", properties.modulus_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_abs
			-- Test {STI_RATIONAL_NUMBER}.abs.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.abs"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			assert ("abs", attached pq.abs)
			assert ("abs_ok", properties.abs_ok (pq))
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("abs", attached pq.abs)
				assert ("abs_ok", properties.abs_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_plus
			-- Test {STS_RATIONAL_NUMBER}.plus.
			-- Test {STI_RATIONAL_NUMBER}.plus.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.plus"
			testing: "covers/{STI_RATIONAL_NUMBER}.plus"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_3: like some_rational
			pqs_1, pqs_2, pqs_3: like Q.subset_anchor -- TODO: Take Q instead.
		do
			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("plus", attached (pq_1 + pq_2))
					from
						pqs_3 := Q
					until
						pqs_3.is_empty
					loop
						pq_3 := same_rational (pqs_3.any)
						assert
							("plus_ok", properties.plus_ok (pq_1, pq_2, pq_3))
						pqs_3 := pqs_3.others
					variant
						pqs_3_cardinality:
							{like some_set}.natural_as_integer (# pqs_3) -- TODO: Take Q instead.
					end
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality:
						{like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality:
					{like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_identity
			-- Test {STS_COMPLEX_NUMBER}.a.
			-- Test {STS_COMPLEX_NUMBER}.real.
			-- Test {STS_COMPLEX_NUMBER}.identity.
			-- Test {STS_REAL_NUMBER}.to_rational.
			-- Test {STS_REAL_NUMBER}.identity.
			-- Test {STS_RATIONAL_NUMBER}.identity.
			-- Test {STI_RATIONAL_NUMBER}.identity.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.a"
			testing: "covers/{STS_COMPLEX_NUMBER}.real"
			testing: "covers/{STS_COMPLEX_NUMBER}.identity"
			testing: "covers/{STS_REAL_NUMBER}.to_rational"
			testing: "covers/{STS_REAL_NUMBER}.identity"
			testing: "covers/{STS_RATIONAL_NUMBER}.identity"
			testing: "covers/{STI_RATIONAL_NUMBER}.identity"
		local
			pq: like rational_to_be_tested
			pqs: like some_rational_set
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("identity", attached (+ pq)) -- TODO: Anomalous join!
				assert ("identity_ok", properties.identity_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_minus
			-- Test {STS_RATIONAL_NUMBER}.minus.
			-- Test {STI_RATIONAL_NUMBER}.minus.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.minus"
			testing: "covers/{STI_RATIONAL_NUMBER}.minus"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_3: like some_rational
			pqs_1, pqs_2, pqs_3: like Q.subset_anchor -- TODO: Take Q instead.
		do
			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert ("minus", attached (pq_1 - pq_2))
					from
						pqs_3 := Q
					until
						pqs_3.is_empty
					loop
						pq_3 := same_rational (pqs_3.any)
						assert (
							"minus_ok",
							properties.minus_ok (pq_1, pq_2, pq_3)
							)
						pqs_3 := pqs_3.others
					variant
						pqs_3_cardinality:
							{like some_set}.natural_as_integer (# pqs_3) -- TODO: Take Q instead.
					end
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality:
						{like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality:
					{like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_opposite
			-- Test {STI_RATIONAL_NUMBER}.opposite.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.opposite"
		local
			pq: like rational_to_be_tested
			pqs: like some_rational_set
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("opposite", attached (- pq))
				assert ("opposite_ok", properties.opposite_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_product
			-- Test {STS_RATIONAL_NUMBER}.product.
			-- Test {STI_RATIONAL_NUMBER}.product.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.product"
			testing: "covers/{STI_RATIONAL_NUMBER}.product"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_3: like some_rational
			pqs_1, pqs_2, pqs_3: like Q.subset_anchor -- TODO: Take Q instead.
		do
			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert (
						"product",
						pq_1.multipliable (pq_2) implies
						attached (pq_1 ⋅ pq_2)
						)
					from
						pqs_3 := Q
					until
						pqs_3.is_empty
					loop
						pq_3 := same_rational (pqs_3.any)
						assert (
							"product_ok",
							properties.product_ok (pq_1, pq_2, pq_3)
							)
						pqs_3 := pqs_3.others
					variant
						pqs_3_cardinality:
							{like some_set}.natural_as_integer (# pqs_3) -- TODO: Take Q instead.
					end
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality:
						{like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality:
					{like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_quotient
			-- Test {STS_RATIONAL_NUMBER}.quotient.
			-- Test {STI_RATIONAL_NUMBER}.quotient.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.quotient"
			testing: "covers/{STI_RATIONAL_NUMBER}.quotient"
		local
			pq_1: like rational_to_be_tested
			pq_2, pq_3: like some_rational
			pqs_1, pqs_2, pqs_3: like Q.subset_anchor -- TODO: Take Q instead.
		do
			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert (
						"quotient",
						pq_1.divisible (pq_2) implies attached (pq_1 / pq_2)
						)
					from
						pqs_3 := Q
					until
						pqs_3.is_empty
					loop
						pq_3 := same_rational (pqs_3.any)
						assert (
							"quotient_ok",
							properties.quotient_ok (pq_1, pq_2, pq_3)
							)
						pqs_3 := pqs_3.others
					variant
						pqs_3_cardinality:
							{like some_set}.natural_as_integer (# pqs_3) -- TODO: Take Q instead.
					end
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality:
						{like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality:
					{like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_reciprocal
			-- Test {STS_RATIONAL_NUMBER}.reciprocal.
			-- Test {STI_RATIONAL_NUMBER}.reciprocal.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.reciprocal"
			testing: "covers/{STI_RATIONAL_NUMBER}.reciprocal"
		local
			pq_1: like rational_to_be_tested
			pq_2: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				assert (
					"reciprocal",
					pq_1.is_invertible implies attached pq_1.reciprocal
					)
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert
						("reciprocal_ok", properties.reciprocal_ok (pq_1, pq_2))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality:
						{like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality:
					{like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_inverse
			-- Test {STS_RATIONAL_NUMBER}.inverse.
			-- Test {STI_RATIONAL_NUMBER}.inverse.
		note
			testing: "covers/{STS_RATIONAL_NUMBER}.inverse"
			testing: "covers/{STI_RATIONAL_NUMBER}.inverse"
		local
			pq_1: like rational_to_be_tested
			pq_2: like some_rational
			pqs_1, pqs_2: like Q.subset_anchor -- TODO: Take Q instead.
		do
			from
				pqs_1 := Q
			until
				pqs_1.is_empty
			loop
				pq_1 := pqs_1.any
				assert (
					"inverse",
					pq_1.is_invertible implies attached pq_1.inverse
					)
				from
					pqs_2 := Q
				until
					pqs_2.is_empty
				loop
					pq_2 := same_rational (pqs_2.any)
					assert
						("inverse_ok", properties.inverse_ok (pq_1, pq_2))
					pqs_2 := pqs_2.others
				variant
					pqs_2_cardinality:
						{like some_set}.natural_as_integer (# pqs_2) -- TODO: Take Q instead.
				end
				pqs_1 := pqs_1.others
			variant
				pqs_1_cardinality:
					{like some_set}.natural_as_integer (# pqs_1) -- TODO: Take Q instead.
			end
		end

	test_power
			-- Test {STS_REAL_NUMBER}.power.
			-- Test {STS_RATIONAL_NUMBER}.power.
			-- Test {STI_RATIONAL_NUMBER}.power.
		note
			testing: "covers/{STS_REAL_NUMBER}.power"
			testing: "covers/{STS_RATIONAL_NUMBER}.power"
			testing: "covers/{STI_RATIONAL_NUMBER}.power"
		local
			pqs: like Q.subset_anchor
			xt0, xt: like R.as_tuple
			pq: like rational_to_be_tested
			x: like some_real
		do
			from
				pqs := Q
				from
					xt0 := R.as_tuple
				until
					xt0.n × # pqs ≤ 83_494 -- TODO: no magic number!
				loop
					if next_random_item \\ 2 = 0 then
						xt0 := xt0.prefix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					else
						xt0 := xt0.suffix ((next_random_item \\ (xt0.n.as_integer_32 + 1)).as_natural_32)
					end
				end
				print (# pqs * xt0.n) print ('%N')
			until
				pqs.is_empty
			loop
				from
					xt := xt0
				until
					xt.is_unit
				loop
					pq := pqs.any
					x := same_real (xt.first)
					assert ("power", attached (pq ^ x))
					assert ("power_ok", properties.power_ok (pq))
					xt := xt.tail
				variant
					n: {like some_set}.natural_as_integer (xt.n)
				end
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_square_root
			-- Test {STS_REAL_NUMBER}.square_root.
			-- Test {STS_RATIONAL_NUMBER}.square_root.
			-- Test {STI_RATIONAL_NUMBER}.square_root.
		note
			testing: "covers/{STS_REAL_NUMBER}.square_root"
			testing: "covers/{STS_RATIONAL_NUMBER}.square_root"
			testing: "covers/{STI_RATIONAL_NUMBER}.square_root"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
--				pqs := Q.Non_negative -- TODO: Fix it!
				pqs := Q | agent {STS_RATIONAL_NUMBER}.real_is_greater_equal (Zero)
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ( -- pqs ⊆ Z.Non_negative
					"non_negative", pq.real_is_greater_equal (Zero)
					)
				assert ("square_root", attached √¯pq)
				assert ("square_root_ok", properties.square_root_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

feature -- Test routines (Conversion)

	test_as_singleton
			-- Test {STI_RATIONAL_NUMBER}.as_singleton.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.as_singleton"
		local
			pq: like rational_to_be_tested
		do
			pq := rational_to_be_tested
			assert ("as_singleton", attached pq.as_singleton)
			assert (
				"as_singleton_ok",
				homogeneous_pair_properties.as_singleton_ok (pq)
				)
		end

	test_as_tuple
			-- Test {STI_RATIONAL_NUMBER}.as_tuple.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.as_tuple"
		local
			pq: like rational_to_be_tested
		do
			pq := rational_to_be_tested
			assert ("as_tuple", attached pq.as_tuple)
			assert
				("as_tuple_ok", homogeneous_pair_properties.as_tuple_ok (pq))
			assert (
				"rational_to_sti_n_tuple", attached
				rational_to_sti_n_tuple (rational_to_be_tested)
				)
			assert (
				"rational_to_sts_n_tuple",
				attached rational_to_sts_n_tuple (rational_to_be_tested)
				)
			assert (
				"rational_to_numeric_sti_n_tuple",
				attached rational_to_numeric_sti_n_tuple
					(rational_to_be_tested)
				)
			assert (
				"rational_to_numeric_sts_n_tuple",
				attached rational_to_numeric_sts_n_tuple
					(rational_to_be_tested)
				)
		end

	test_as_ordered_pair
			-- Test {STI_RATIONAL_NUMBER}.as_ordered_pair.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.as_ordered_pair"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("as_ordered_pair", attached pq.as_ordered_pair)
				assert (
					"rational_to_sti_homogeneous_integer_pair",
					attached rational_to_sti_homogeneous_integer_pair (pq)
					)
				assert (
					"rational_to_sti_homogeneous_numeric_pair",
					attached rational_to_sti_homogeneous_numeric_pair (pq)
					)
				assert (
					"rational_to_sti_integer_pair",
					attached rational_to_sti_integer_pair (pq)
					)
				assert (
					"rational_to_sti_numeric_pair",
					attached rational_to_sti_numeric_pair (pq)
					)
				pqs := pqs.others
			variant
				cardinality: {like Q}.natural_as_integer (# pqs)
			end
		end

	test_as_complex
			-- Test {STI_RATIONAL_NUMBER}.as_complex.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.as_complex"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("as_complex", attached pq.as_complex)
				assert (
					"complex_from_rational",
					attached complex_from_rational (pq)
					)
				assert ("as_complex_ok", properties.as_complex_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_as_real
			-- Test {STI_RATIONAL_NUMBER}.as_real.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.as_real"
		local
			pq: like rational_to_be_tested
			pqs: like Q.subset_anchor
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert ("as_real", attached pq.as_real)
				assert ("real_from_rational", attached real_from_rational (pq))
				assert ("as_real_ok", properties.as_real_ok (pq))
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_truncated_to_integer
			-- Test {STI_RATIONAL_NUMBER}.truncated_to_integer.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.truncated_to_integer"
		local
			pqs: like some_rational_set
			pq: like rational_to_be_tested
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				pq := pqs.any
				assert
					("truncated_to_integer", attached pq.truncated_to_integer)
				pqs := pqs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

feature -- Test routines (Math)

	test_pi
			-- Test {STS_COMPLEX_NUMBER}.pi.
			-- Test {STS_REAL_NUMBER}.pi.
			-- Test {STS_RATIONAL_NUMBER}.pi.
			-- Test {STI_RATIONAL_NUMBER}.pi.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi"
			testing: "covers/{STS_REAL_NUMBER}.pi"
			testing: "covers/{STS_RATIONAL_NUMBER}.pi"
			testing: "covers/{STI_RATIONAL_NUMBER}.pi"
		do
			assert ("pi", attached rational_to_be_tested.Pi)
		end

	test_pi_2
			-- Test {STS_COMPLEX_NUMBER}.pi_2.
			-- Test {STS_REAL_NUMBER}.pi_2.
			-- Test {STS_RATIONAL_NUMBER}.pi_2.
			-- Test {STI_RATIONAL_NUMBER}.pi_2.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi_2"
			testing: "covers/{STS_REAL_NUMBER}.pi_2"
			testing: "covers/{STS_RATIONAL_NUMBER}.pi_2"
			testing: "covers/{STI_RATIONAL_NUMBER}.pi_2"
		do
			assert ("pi_2", attached rational_to_be_tested.Pi_2)
		end

	test_pi_4
			-- Test {STS_COMPLEX_NUMBER}.pi_4.
			-- Test {STS_REAL_NUMBER}.pi_4.
			-- Test {STS_RATIONAL_NUMBER}.pi_4.
			-- Test {STI_RATIONAL_NUMBER}.pi_4.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.pi_4"
			testing: "covers/{STS_REAL_NUMBER}.pi_4"
			testing: "covers/{STS_RATIONAL_NUMBER}.pi_4"
			testing: "covers/{STI_RATIONAL_NUMBER}.pi_4"
		do
			assert ("pi_4", attached rational_to_be_tested.Pi_4)
		end

	test_value_is_infinite
			-- Test {STI_RATIONAL_NUMBER}.value_is_infinite.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.value_is_infinite"
		local
			v: like some_native_real
		do
			v := Negative_infinity
			assert (
				"Negative_infinity",
				rational_to_be_tested.value_is_infinite (v)
				)

			v := Positive_infinity
			assert (
				"Positive_infinity",
				rational_to_be_tested.value_is_infinite (v)
				)

			from
				v := some_native_real
			until
				not (v.is_negative_infinity or v.is_positive_infinity)
			loop
				v := some_native_real
			end
			assert ("finite", not rational_to_be_tested.value_is_infinite (v))

			assert (
				"value_is_infinite",
				rational_to_be_tested.value_is_infinite
					(some_native_real) implies True
				)
		end

	test_value_is_finite
			-- Test {STI_RATIONAL_NUMBER}.value_is_finite.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.value_is_finite"
		local
			v: like some_native_real
		do
			v := Nan
			assert ("NaN", not rational_to_be_tested.value_is_finite (v))

			v := Negative_infinity
			assert (
				"Negative_infinity",
				not rational_to_be_tested.value_is_finite (v)
				)

			v := Positive_infinity
			assert (
				"Positive_infinity",
				not rational_to_be_tested.value_is_finite (v)
				)

			from
				v := some_native_real
			until
				not (
					v.is_nan or
					v.is_negative_infinity or
					v.is_positive_infinity
					)
			loop
				v := some_native_real
			end
			assert ("finite", rational_to_be_tested.value_is_finite (v))

			assert (
				"value_is_finite",
				rational_to_be_tested.value_is_finite
					(some_native_real) implies True
				)
		end

	test_hypotenuse
			-- Test {STI_RATIONAL_NUMBER}.hypotenuse.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.hypotenuse"
		local
			zs: like Quick_C.subset_anchor
			ab: like some_expanded_complex
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				ab := zs.any
				assert (
					"hypotenuse",
					attached {like rational_to_be_tested}.hypotenuse
						(ab.a, ab.b)
					)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert (
				"-NaN, -NaN",
				attached {like rational_to_be_tested}.hypotenuse (-Nan, -Nan)
				)
			assert (
				"NaN, -NaN",
				attached {like rational_to_be_tested}.hypotenuse (Nan, -Nan)
				)
			assert (
				"-Infinity, -NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(Negative_infinity, -Nan)
				)
			assert (
				"Infinity, -NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(Positive_infinity, -Nan)
				)
			assert (
				"0, -NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, -Nan)
				)
			assert (
				"-0, -NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(-Zero.as_real, -Nan)
				)
			assert (
				"0, -NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, -Nan)
				)
			assert (
				"a, -NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(some_real, -Nan)
				)

			assert (
				"-NaN, NaN",
				attached {like rational_to_be_tested}.hypotenuse (-Nan, Nan)
				)
			assert (
				"NaN, NaN",
				attached {like rational_to_be_tested}.hypotenuse (Nan, Nan)
				)
			assert (
				"-Infinity, NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(Negative_infinity, Nan)
				)
			assert (
				"Infinity, NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(Positive_infinity, Nan)
				)
			assert (
				"NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Nan)
				)
			assert (
				"-0, NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(-Zero.as_real, Nan)
				)
			assert (
				"0, NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Nan)
				)
			assert (
				"a, NaN",
				attached {like rational_to_be_tested}.hypotenuse
					(some_real, Nan)
				)

			assert (
				"-NaN, -Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(-Nan, Negative_infinity)
				)
			assert (
				"NaN, -Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Nan, Negative_infinity)
				)
			assert (
				"-Infinity, -Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Negative_infinity, Negative_infinity)
				)
			assert (
				"Infinity, -Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Positive_infinity, Negative_infinity)
				)
			assert (
				"0, -Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Negative_infinity)
				)
			assert (
				"-0, -Infinity",
				attached {like rational_to_be_tested}.hypotenuse
				(-Zero.as_real, Negative_infinity)
				)
			assert (
				"0, -Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Negative_infinity)
				)
			assert (
				"a, -Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(some_real, Negative_infinity)
				)

			assert (
				"-NaN, Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(-Nan, Positive_infinity)
				)
			assert (
				"NaN, Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Nan, Positive_infinity)
				)
			assert (
				"-Infinity, Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Negative_infinity, Positive_infinity)
				)
			assert (
				"Infinity, Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Positive_infinity, Positive_infinity)
				)
			assert (
				"0, Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Positive_infinity)
				)
			assert (
				"-0, Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(-Zero.as_real, Positive_infinity)
					)
			assert (
				"0, Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Positive_infinity)
				)
			assert (
				"a, Infinity",
				attached {like rational_to_be_tested}.hypotenuse
					(some_real, Positive_infinity)
					)

			assert (
				"-NaN, -0",
				attached {like rational_to_be_tested}.hypotenuse
					(-Nan, -Zero.as_real)
				)
			assert (
				"NaN, -0",
				attached {like rational_to_be_tested}.hypotenuse
					(Nan, -Zero.as_real)
				)
			assert (
				"-Infinity, -0",
				attached {like rational_to_be_tested}.hypotenuse
					(Negative_infinity, -Zero.as_real)
				)
			assert (
				"Infinity, -0",
				attached {like rational_to_be_tested}.hypotenuse
					(Positive_infinity, -Zero.as_real)
				)
			assert (
				"0, -0",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, -Zero.as_real)
				)
			assert (
				"-0, -0",
				attached {like rational_to_be_tested}.hypotenuse
					(-Zero.as_real, -Zero.as_real)
				)
			assert (
				"0, -0",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, -Zero.as_real)
				)
			assert (
				"a, -0", attached {like rational_to_be_tested}.hypotenuse
					(some_real, -Zero.as_real)
				)

			assert (
				"-NaN, 0",
				attached {like rational_to_be_tested}.hypotenuse (-Nan, Zero)
				)
			assert (
				"-NaN, 0",
				attached {like rational_to_be_tested}.hypotenuse (-Nan, Zero)
				)
			assert (
				"NaN, 0",
				attached {like rational_to_be_tested}.hypotenuse (Nan, Zero)
				)
			assert (
				"NaN, 0", attached {like rational_to_be_tested}.hypotenuse
					(Nan, Zero)
				)
			assert (
				"-Infinity, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(Negative_infinity, Zero)
				)
			assert (
				"-Infinity, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(Negative_infinity, Zero)
				)
			assert (
				"Infinity, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(Positive_infinity, Zero)
				)
			assert (
				"Infinity, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(Positive_infinity, Zero)
				)
			assert (
				"0, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Zero)
				)
			assert (
				"0, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Zero)
				)
			assert (
				"-0, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(-Zero.as_real, Zero)
				)
			assert (
				"0, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, Zero)
				)
			assert (
				"a, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(some_real, Zero)
				)
			assert (
				"a, 0",
				attached {like rational_to_be_tested}.hypotenuse
					(some_real, Zero)
				)

			assert (
				"-NaN, b",
				attached {like rational_to_be_tested}.hypotenuse
					(-Nan, some_real)
				)
			assert (
				"NaN, b",
				attached {like rational_to_be_tested}.hypotenuse
					(Nan, some_real)
					)
			assert (
				"-Infinity, b",
				attached {like rational_to_be_tested}.hypotenuse
					(Negative_infinity, some_real)
				)
			assert (
				"Infinity, b",
				attached {like rational_to_be_tested}.hypotenuse
					(Positive_infinity, some_real)
				)
			assert (
				"0, bi", attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, some_real)
				)
			assert (
				"-0, b",
				attached {like rational_to_be_tested}.hypotenuse
					(-Zero.as_real, some_real)
				)
			assert (
				"0, b",
				attached {like rational_to_be_tested}.hypotenuse
					(Zero.as_real, some_real)
				)
			assert (
				"a, b",
				attached {like rational_to_be_tested}.hypotenuse
					(some_real, some_real)
				)
		end

	test_arc_tangent
			-- Test {STS_COMPLEX_NUMBER}.arc_tangent.
			-- Test {STS_REAL_NUMBER}.arc_tangent.
			-- Test {STS_RATIONAL_NUMBER}.arc_tangent.
			-- Test {STI_RATIONAL_NUMBER}.arc_tangent.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.arc_tangent"
			testing: "covers/{STS_REAL_NUMBER}.arc_tangent"
			testing: "covers/{STS_RATIONAL_NUMBER}.arc_tangent"
			testing: "covers/{STI_RATIONAL_NUMBER}.arc_tangent"
		do
			assert ("-NaN / -NaN", attached {like rational_to_be_tested}.arc_tangent (-Nan / -Nan))
			assert ("NaN / -NaN", attached {like rational_to_be_tested}.arc_tangent (Nan / -Nan))
			assert ("-Infinity / -NaN", attached {like rational_to_be_tested}.arc_tangent (Negative_infinity / -Nan))
			assert ("Infinity / -NaN", attached {like rational_to_be_tested}.arc_tangent (Positive_infinity / -Nan))
			assert ("0 / -NaN", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / -Nan))
			assert ("-0 / -NaN", attached {like rational_to_be_tested}.arc_tangent (-Zero.as_real / -Nan))
			assert ("0 / -NaN", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / -Nan))
			assert ("a / -NaN", attached {like rational_to_be_tested}.arc_tangent (some_real.value / -Nan))

			assert ("-NaN / NaN", attached {like rational_to_be_tested}.arc_tangent (-Nan / Nan))
			assert ("NaN / NaN", attached {like rational_to_be_tested}.arc_tangent (Nan / Nan))
			assert ("-Infinity / NaN", attached {like rational_to_be_tested}.arc_tangent (Negative_infinity / Nan))
			assert ("Infinity / NaN", attached {like rational_to_be_tested}.arc_tangent (Positive_infinity / Nan))
			assert ("NaN", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Nan))
			assert ("-0 / NaN", attached {like rational_to_be_tested}.arc_tangent (-Zero.as_real / Nan))
			assert ("0 / NaN", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Nan))
			assert ("a / NaN", attached {like rational_to_be_tested}.arc_tangent (some_real.value / Nan))

			assert ("-NaN / -Infinity", attached {like rational_to_be_tested}.arc_tangent (-Nan / Negative_infinity))
			assert ("NaN / -Infinity", attached {like rational_to_be_tested}.arc_tangent (Nan / Negative_infinity))
			assert ("-Infinity / -Infinity", attached {like rational_to_be_tested}.arc_tangent (Negative_infinity / Negative_infinity))
			assert ("Infinity / -Infinity", attached {like rational_to_be_tested}.arc_tangent (Positive_infinity / Negative_infinity))
			assert ("0 / -Infinity", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Negative_infinity))
			assert ("-0 / -Infinity", attached {like rational_to_be_tested}.arc_tangent (-Zero.as_real / Negative_infinity))
			assert ("0 / -Infinity", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Negative_infinity))
			assert ("a / -Infinity", attached {like rational_to_be_tested}.arc_tangent (some_real.value / Negative_infinity))

			assert ("-NaN / Infinity", attached {like rational_to_be_tested}.arc_tangent (-Nan / Positive_infinity))
			assert ("NaN / Infinity", attached {like rational_to_be_tested}.arc_tangent (Nan / Positive_infinity))
			assert ("-Infinity / Infinity", attached {like rational_to_be_tested}.arc_tangent (Negative_infinity / Positive_infinity))
			assert ("Infinity / Infinity", attached {like rational_to_be_tested}.arc_tangent (Positive_infinity / Positive_infinity))
			assert ("0 / Infinity", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Positive_infinity))
			assert ("-0 / Infinity", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Positive_infinity))
			assert ("0 / Infinity", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Positive_infinity))
			assert ("a / Infinity", attached {like rational_to_be_tested}.arc_tangent (some_real.value / Positive_infinity))

			assert ("-NaN / -0", attached {like rational_to_be_tested}.arc_tangent (-Nan / -Zero.as_real))
			assert ("NaN / -0", attached {like rational_to_be_tested}.arc_tangent (Nan / -Zero.as_real))
			assert ("-Infinity / -0", attached {like rational_to_be_tested}.arc_tangent (Negative_infinity / -Zero.as_real))
			assert ("Infinity / -0", attached {like rational_to_be_tested}.arc_tangent (Positive_infinity / -Zero.as_real))
			assert ("0 / -0", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / -Zero.as_real))
			assert ("-0 / -0", attached {like rational_to_be_tested}.arc_tangent (-Zero.as_real / -Zero.as_real))
			assert ("0 / -0", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / -Zero.as_real))
			assert ("a / -0", attached {like rational_to_be_tested}.arc_tangent (some_real.value / -Zero.as_real))

			assert ("-NaN / 0", attached {like rational_to_be_tested}.arc_tangent (-Nan / Zero))
			assert ("-NaN / 0", attached {like rational_to_be_tested}.arc_tangent (-Nan / Zero))
			assert ("NaN / 0", attached {like rational_to_be_tested}.arc_tangent (Nan / Zero))
			assert ("NaN / 0", attached {like rational_to_be_tested}.arc_tangent (Nan / Zero))
			assert ("-Infinity / 0", attached {like rational_to_be_tested}.arc_tangent (Negative_infinity / Zero))
			assert ("-Infinity / 0", attached {like rational_to_be_tested}.arc_tangent (Negative_infinity / Zero))
			assert ("Infinity / 0", attached {like rational_to_be_tested}.arc_tangent (Positive_infinity / Zero))
			assert ("Infinity / 0", attached {like rational_to_be_tested}.arc_tangent (Positive_infinity / Zero))
			assert ("0 / 0", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Zero))
			assert ("0 / 0", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Zero))
			assert ("-0 / 0", attached {like rational_to_be_tested}.arc_tangent (-Zero.as_real / Zero))
			assert ("0 / 0", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / Zero))
			assert ("a / 0", attached {like rational_to_be_tested}.arc_tangent (some_real.value / Zero))
			assert ("a / 0", attached {like rational_to_be_tested}.arc_tangent (some_real.value / Zero))

			assert ("-NaN / b", attached {like rational_to_be_tested}.arc_tangent (-Nan / some_real))
			assert ("NaN / b", attached {like rational_to_be_tested}.arc_tangent (Nan / some_real))
			assert ("-Infinity / b", attached {like rational_to_be_tested}.arc_tangent (Negative_infinity / some_real))
			assert ("Infinity / b", attached {like rational_to_be_tested}.arc_tangent (Positive_infinity / some_real))
			assert ("0 / bi", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / some_real))
			assert ("-0 / b", attached {like rational_to_be_tested}.arc_tangent (-Zero.as_real / some_real))
			assert ("0 / b", attached {like rational_to_be_tested}.arc_tangent (Zero.as_real / some_real))
			assert ("a / b", attached {like rational_to_be_tested}.arc_tangent (some_real.value / some_real.value))
		end

	test_arc_tangent_yx
			-- Test {STI_RATIONAL_NUMBER}.arc_tangent_yx.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.arc_tangent_yx"
		local
			zs: like Quick_C.subset_anchor
			ab: like some_expanded_complex
		do
			from
				zs := Quick_C
			until
				zs.is_empty
			loop
				ab := zs.any
				assert (
					"arc_tangent_yx",
					attached {like rational_to_be_tested}.arc_tangent_yx
						(ab.a, ab.b)
					)
				zs := zs.others
			variant
				cardinality: {like some_set}.natural_as_integer (# zs)
			end

			assert (
				"-NaN, -NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Nan, -Nan)
				)
			assert (
				"NaN, -NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Nan, -Nan)
				)
			assert (
				"-Infinity, -NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Negative_infinity, -Nan)
				)
			assert (
				"Infinity, -NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Positive_infinity, -Nan)
				)
			assert (
				"0, -NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, -Nan)
				)
			assert (
				"-0, -NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, -Nan)
				)
			assert (
				"0, -NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, -Nan)
				)
			assert (
				"a, -NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(some_real, -Nan)
				)

			assert (
				"-NaN, NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Nan, Nan)
				)
			assert (
				"NaN, NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx (Nan, Nan)
				)
			assert (
				"-Infinity, NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Nan)
				)
			assert (
				"Infinity, NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Nan)
				)
			assert (
				"NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Nan)
				)
			assert (
				"-0, NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, Nan)
				)
			assert (
				"0, NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Nan)
				)
			assert (
				"a, NaN",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(some_real, Nan)
				)

			assert (
				"-NaN, -Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Nan, Negative_infinity)
				)
			assert (
				"NaN, -Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Nan, Negative_infinity)
				)
			assert (
				"-Infinity, -Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Negative_infinity)
				)
			assert (
				"Infinity, -Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Negative_infinity)
				)
			assert (
				"0, -Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Negative_infinity)
				)
			assert (
				"-0, -Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
				(-Zero.as_real, Negative_infinity)
				)
			assert (
				"0, -Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Negative_infinity)
				)
			assert (
				"a, -Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(some_real, Negative_infinity)
				)

			assert (
				"-NaN, Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Nan, Positive_infinity)
				)
			assert (
				"NaN, Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Nan, Positive_infinity)
				)
			assert (
				"-Infinity, Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Positive_infinity)
				)
			assert (
				"Infinity, Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Positive_infinity)
				)
			assert (
				"0, Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Positive_infinity)
				)
			assert (
				"-0, Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, Positive_infinity)
					)
			assert (
				"0, Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Positive_infinity)
				)
			assert (
				"a, Infinity",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(some_real, Positive_infinity)
					)

			assert (
				"-NaN, -0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Nan, -Zero.as_real)
				)
			assert (
				"NaN, -0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Nan, -Zero.as_real)
				)
			assert (
				"-Infinity, -0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Negative_infinity, -Zero.as_real)
				)
			assert (
				"Infinity, -0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Positive_infinity, -Zero.as_real)
				)
			assert (
				"0, -0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, -Zero.as_real)
				)
			assert (
				"-0, -0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, -Zero.as_real)
				)
			assert (
				"0, -0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, -Zero.as_real)
				)
			assert (
				"a, -0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(some_real, -Zero.as_real)
				)

			assert (
				"-NaN, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Nan, Zero)
				)
			assert (
				"-NaN, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Nan, Zero)
				)
			assert (
				"NaN, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx (Nan, Zero)
				)
			assert (
				"NaN, 0", attached {like rational_to_be_tested}.arc_tangent_yx
					(Nan, Zero)
				)
			assert (
				"-Infinity, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Zero)
				)
			assert (
				"-Infinity, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Negative_infinity, Zero)
				)
			assert (
				"Infinity, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Zero)
				)
			assert (
				"Infinity, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Positive_infinity, Zero)
				)
			assert (
				"0, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Zero)
				)
			assert (
				"0, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Zero)
				)
			assert (
				"-0, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, Zero)
				)
			assert (
				"0, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, Zero)
				)
			assert (
				"a, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(some_real, Zero)
				)
			assert (
				"a, 0",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(some_real, Zero)
				)

			assert (
				"-NaN, b",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Nan, some_real)
				)
			assert (
				"NaN, b",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Nan, some_real)
					)
			assert (
				"-Infinity, b",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Negative_infinity, some_real)
				)
			assert (
				"Infinity, b",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Positive_infinity, some_real)
				)
			assert (
				"0, bi", attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, some_real)
				)
			assert (
				"-0, b",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(-Zero.as_real, some_real)
				)
			assert (
				"0, b",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(Zero.as_real, some_real)
				)
			assert (
				"a, b",
				attached {like rational_to_be_tested}.arc_tangent_yx
					(some_real, some_real)
				)
		end

	test_value_logb
			-- Test {STS_COMPLEX_NUMBER}.value_logb.
			-- Test {STS_REAL_NUMBER}.value_logb.
			-- Test {STS_RATIONAL_NUMBER}.value_logb.
			-- Test {STI_RATIONAL_NUMBER}.value_logb.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.value_logb"
			testing: "covers/{STS_REAL_NUMBER}.value_logb"
			testing: "covers/{STS_RATIONAL_NUMBER}.value_logb"
			testing: "covers/{STI_RATIONAL_NUMBER}.value_logb"
		do
			assert ("-NaN", attached rational_to_be_tested.value_logb (-Nan))
			assert ("NaN", attached rational_to_be_tested.value_logb (Nan))
			assert ("-Infinity", attached rational_to_be_tested.value_logb (Negative_infinity))
			assert ("Infinity", attached rational_to_be_tested.value_logb (Positive_infinity))
			assert ("-0", attached rational_to_be_tested.value_logb (-0.0))
			assert ("0", attached rational_to_be_tested.value_logb (0))
			assert ("x", attached rational_to_be_tested.value_logb (some_native_real))
		end

	test_value_sign_bit
			-- Test {STS_COMPLEX_NUMBER}.value_sign_bit.
			-- Test {STS_REAL_NUMBER}.value_sign_bit.
			-- Test {STS_RATIONAL_NUMBER}.value_sign_bit.
			-- Test {STI_RATIONAL_NUMBER}.value_sign_bit.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.value_sign_bit"
			testing: "covers/{STS_REAL_NUMBER}.value_sign_bit"
			testing: "covers/{STS_RATIONAL_NUMBER}.value_sign_bit"
			testing: "covers/{STI_RATIONAL_NUMBER}.value_sign_bit"
		local
			v: like some_native_real
		do
			v := - Nan
			assert ("signed nan", rational_to_be_tested.value_sign_bit (v) = 1)

			v := Nan
			assert ("unsigned nan", rational_to_be_tested.value_sign_bit (v) = 0)

			from
				v := some_native_real
			until
				not v.is_nan and v < 0
			loop
				v := some_native_real
			end
			assert ("negative", rational_to_be_tested.value_sign_bit (v) = 1)

			v := 0.0
			assert ("negative zero", rational_to_be_tested.value_sign_bit (- v) = 1)
			assert ("positive zero", rational_to_be_tested.value_sign_bit (v) = 0)

			from
				v := some_native_real
			until
				0 < v
			loop
				v := some_native_real
			end
			assert ("positive", rational_to_be_tested.value_sign_bit (v) = 0)

			assert ("-NaN", attached rational_to_be_tested.value_sign_bit (-Nan))
			assert ("NaN", attached rational_to_be_tested.value_sign_bit (Nan))
			assert ("-Infinity", attached rational_to_be_tested.value_sign_bit (Negative_infinity))
			assert ("Infinity", attached rational_to_be_tested.value_sign_bit (Positive_infinity))
			assert ("-0", attached rational_to_be_tested.value_sign_bit (-0.0))
			assert ("0", attached rational_to_be_tested.value_sign_bit (0))
			assert ("x", attached rational_to_be_tested.value_sign_bit (some_native_real))
		end

	test_sqrt
			-- Test {STS_REAL_NUMBER}.sqrt.
			-- Test {STS_RATIONAL_NUMBER}.sqrt.
			-- Test {STI_RATIONAL_NUMBER}.sqrt.
		note
			testing: "covers/{STS_REAL_NUMBER}.sqrt"
			testing: "covers/{STS_RATIONAL_NUMBER}.sqrt"
			testing: "covers/{STI_RATIONAL_NUMBER}.sqrt"
		do
			assert ("-NaN", attached rational_to_be_tested.sqrt (-Nan))
			assert ("NaN", attached rational_to_be_tested.sqrt (Nan))
			assert ("-Infinity", attached rational_to_be_tested.sqrt (Negative_infinity))
			assert ("Infinity", attached rational_to_be_tested.sqrt (Positive_infinity))
			assert ("-0", attached rational_to_be_tested.sqrt (-0.0))
			assert ("0", attached rational_to_be_tested.sqrt (0))
			assert ("x", attached rational_to_be_tested.sqrt (some_native_real))
		end

	test_splitted
			-- Test {STI_RATIONAL_NUMBER}.splitted.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.splitted"
		local
			pqs: like Q.subset_anchor
			pq: like rational_to_be_tested
			i_s: like some_integer_set
			i: like some_integer
		do
			from
				pqs := Q
			until
				pqs.is_empty
			loop
				from
					pq := pqs.any
					i_s := Z
				until
					i_s.is_empty
				loop
					i := same_integer (i_s.any)
					assert ("splitted", attached pq.splitted (i))
					assert ("splitted ok", properties.splitted_ok (pq, i))
					i_s := i_s.others
				variant
					i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
				end
				pqs := pqs.others
			variant
				pqs_cardinality: {like some_set}.natural_as_integer (# pqs)
			end
		end

	test_gcd
			-- Test {STI_RATIONAL_NUMBER}.gcd.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.gcd"
		local
			pq: like rational_to_be_tested
			i_s, js, ks: like Z.subset_anchor -- TODO: Take Q instead.
			i, j, k: like some_integer
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := same_integer (i_s.any)
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					pq := rational_to_be_tested
					assert ("gcd", attached pq.gcd (i, j))
					from
						ks := Z
					until
						ks.is_empty
					loop
						k := same_integer (ks.any)
						assert ("gcd_ok", properties.gcd_ok (pq, i, j, k))
						ks := ks.others
					variant
						ks_cardinality: {like some_set}.natural_as_integer (# ks)
					end
					js := js.others
				variant
					js_cardinality: {like some_set}.natural_as_integer (# js)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_div
			-- Test {STI_RATIONAL_NUMBER}.div.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.div"
		local
			pq: like rational_to_be_tested
			i_s, js: like Z.subset_anchor -- TODO: Take Q instead.
			i, j: like some_integer
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := same_integer (i_s.any)
					js := Z.without_zero
				until
					js.is_empty
				loop
					j := same_integer (js.any)
						check
							good_divisor: i.divisible (j) -- j ∈ Z.without_zero
						end
					pq := rational_to_be_tested
					assert ("div", attached pq.div (i, j))
					assert ("div_ok", properties.div_ok (pq, i, j))
					js := js.others
				variant
					js_cardinality: {like some_set}.natural_as_integer (# js)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

	test_rem
			-- Test {STI_RATIONAL_NUMBER}.rem.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.rem"
		local
			pq: like rational_to_be_tested
			i_s, js: like Z.subset_anchor -- TODO: Take Q instead.
			i, j: like some_integer
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := same_integer (i_s.any)
					js := Z.without_zero
				until
					js.is_empty
				loop
					j := same_integer (js.any)
						check
							good_divisor: i.divisible (j) -- j ∈ Z.without_zero
						end
					pq := rational_to_be_tested
					assert ("rem", attached pq.rem (i, j))
					assert
						("rem_ok", properties.rem_ok (pq, i, j, some_integer))
					js := js.others
				variant
					js_cardinality: {like some_set}.natural_as_integer (# js)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

feature -- Test routines (Factory)

	test_real_from_value
			-- Test {STI_RATIONAL_NUMBER}.real_from_value.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_from_value"
		do
			assert
				("-NaN", attached rational_to_be_tested.real_from_value (-Nan))
			assert
				("NaN", attached rational_to_be_tested.real_from_value (Nan))
			assert (
				"-Infinity",
				attached rational_to_be_tested.real_from_value
					(Negative_infinity)
				)
			assert (
				"Infinity",
				attached rational_to_be_tested.real_from_value
					(Positive_infinity)
				)
			assert
				("-0", attached rational_to_be_tested.real_from_value (-0.0))
			assert ("0", attached rational_to_be_tested.real_from_value (0))
			assert (
				"x",
				attached rational_to_be_tested.real_from_value
					(some_native_real)
				)
		end

	test_real_from_double_value
			-- Test {STI_RATIONAL_NUMBER}.real_from_double_value.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.real_from_double_value"
		do
			assert (
				"-NaN",
				attached rational_to_be_tested.real_from_double_value
					(-Nan.value)
				)
			assert (
				"NaN",
				attached rational_to_be_tested.real_from_double_value
					(Nan.value)
				)
			assert (
				"-Infinity",
				attached rational_to_be_tested.real_from_double_value
					(Negative_infinity.value)
				)
			assert (
				"Infinity",
				attached rational_to_be_tested.real_from_double_value
					(Positive_infinity.value)
				)
			assert (
				"-0",
				attached rational_to_be_tested.real_from_double_value (-0.0)
				)
			assert (
				"0",
				attached rational_to_be_tested.real_from_double_value (0)
				)
			assert (
				"x",
				attached rational_to_be_tested.real_from_double_value
					(some_native_double_real)
				)
		end

	test_native_real_from_double_value
			-- Test {STS_COMPLEX_NUMBER}.native_real_from_double_value.
			-- Test {STS_REAL_NUMBER}.native_real_from_double_value.
			-- Test {STS_RATIONAL_NUMBER}.native_real_from_double_value.
			-- Test {STI_RATIONAL_NUMBER}.native_real_from_double_value.
		note
			testing: "covers/{STS_COMPLEX_NUMBER}.native_real_from_double_value"
			testing: "covers/{STS_REAL_NUMBER}.native_real_from_double_value"
			testing: "covers/{STS_RATIONAL_NUMBER}.native_real_from_double_value"
			testing: "covers/{STI_RATIONAL_NUMBER}.native_real_from_double_value"
		do
			assert ("-NaN", attached rational_to_be_tested.native_real_from_double_value (-Nan.value))
			assert ("NaN", attached rational_to_be_tested.native_real_from_double_value (Nan.value))
			assert ("-Infinity", attached rational_to_be_tested.native_real_from_double_value (Negative_infinity.value))
			assert ("Infinity", attached rational_to_be_tested.native_real_from_double_value (Positive_infinity.value))
			assert ("-0", attached rational_to_be_tested.native_real_from_double_value (-0.0))
			assert ("0", attached rational_to_be_tested.native_real_from_double_value (0))
			assert ("x", attached rational_to_be_tested.native_real_from_double_value (some_native_double_real))
		end

	test_converted_integer
			-- Test {STI_RATIONAL_NUMBER}.converted_integer.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.converted_integer"
		local
			i_s: like Z.subset_anchor
			i: like Z.Any
		do
			from
				i_s := Z
			until
				i_s.is_empty
			loop
				i := i_s.any
				assert (
					"converted_integer",
					attached rational_to_be_tested.converted_integer (i)
					)
				i_s := i_s.others
			variant
				cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

feature -- Test routines (Predicate)

	test_integer_product_overflows
			-- Test {STI_RATIONAL_NUMBER}.integer_product_overflows.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.integer_product_overflows"
		local
			i_s, js: like Z.subset_anchor -- TODO: Take Q instead.
			pq: like rational_to_be_tested
			i, j: like some_integer
		do
			if next_random_item \\ 2 = 0 then
				j := - some_integer.abs ∧ - One.p
				if next_random_item \\ 2 = 0 then
					i := (- some_integer.abs ∧ - One.p) ∧
						({STI_INTEGER_NUMBER}.Max_value // j - One.p)
				else
					j := j ∧ - Two
					i := (some_integer.abs ∨ One.p) ∨
						({STI_INTEGER_NUMBER}.Min_value // j + One.p)
				end
			else
				j := some_integer.abs ∨ One.p
				if next_random_item \\ 2 = 0 then
					i := (- some_integer.abs ∧ - One.p) ∧
						({STI_INTEGER_NUMBER}.Min_value // j - One.p)
				else
					i := (some_integer.abs ∨ One.p) ∨
						({STI_INTEGER_NUMBER}.Max_value // j + One.p)
				end
			end
			pq := rational_to_be_tested
			assert (
				"product_overflows (i, j)",
				pq.integer_product_overflows (i, j)
				)
			assert (
				"product_overflows (i, j) ok",
				properties.integer_product_overflows_ok (pq, i, j)
				)

			if next_random_item \\ 2 = 0 then
				j := - some_integer.abs
				i := some_integer
				if j < Zero then
					if next_random_item \\ 2 = 0 then
						i := - i.abs ∨ ({STI_INTEGER_NUMBER}.Max_value // j)
					else
						i := i.abs ∧ ({STI_INTEGER_NUMBER}.Min_value // j)
					end
				end
			else
				j := some_integer.abs
				i := some_integer
				if j > Zero then
					if next_random_item \\ 2 = 0 then
						i := - i.abs ∨ ({STI_INTEGER_NUMBER}.Min_value // j)
					else
						i := i.abs ∧ ({STI_INTEGER_NUMBER}.Max_value // j)
					end
				end
			end
			pq := rational_to_be_tested
			assert (
				"not product_overflows (i, j)",
				not pq.integer_product_overflows (i, j)
				)
			assert (
				"not product_overflows (i, j) ok",
				properties.integer_product_overflows_ok (pq, i, j)
				)

			from
				i_s := Z
			until
				i_s.is_empty
			loop
				from
					i := same_integer (i_s.any)
					js := Z
				until
					js.is_empty
				loop
					j := same_integer (js.any)
					pq := rational_to_be_tested
					assert (
						"product_overflows",
						pq.integer_product_overflows (i, j) implies True
						)
					assert (
						"integer_product_overflows_ok",
						properties.integer_product_overflows_ok (pq, i, j)
						)
					js := js.others
				variant
					js_cardinality: {like some_set}.natural_as_integer (# js)
				end
				i_s := i_s.others
			variant
				i_s_cardinality: {like some_set}.natural_as_integer (# i_s)
			end
		end

feature -- Test routines (Implementation)

	test_adjusted_value
			-- Test {STI_RATIONAL_NUMBER}.adjusted_value.
		note
			testing: "covers/{STI_RATIONAL_NUMBER}.adjusted_value"
		do
			assert
				("-NaN", attached rational_to_be_tested.adjusted_value (-Nan))
			assert ("NaN", attached rational_to_be_tested.adjusted_value (Nan))
			assert (
				"-Infinity",
				attached rational_to_be_tested.adjusted_value
					(Negative_infinity)
				)
			assert (
				"Infinity",
				attached rational_to_be_tested.adjusted_value
					(Positive_infinity)
				)
			assert ("-0", attached rational_to_be_tested.adjusted_value (-0.0))
			assert ("0", attached rational_to_be_tested.adjusted_value (0))
			assert (
				"x",
				attached rational_to_be_tested.adjusted_value
					(some_native_real)
				)
		end

feature {NONE} -- Factory (element to be tested)

	rational_to_be_tested: like some_expanded_rational
			-- Rational number meant to be under tests
		do
			Result := some_expanded_rational
		end

feature {NONE} -- Conversion

	rational_from_pq (pq: STS_RATIONAL_NUMBER): like rational_to_be_tested
			-- `pq' converted to a rational number like `rational_to_be_tested'
		do
			Result := pq
		ensure
			numerator: Result.p ≍ pq.p
			denominator: Result.q ≍ pq.q
		end

	rational_from_native_integer
		(v: like some_native_integer): like rational_to_be_tested
			-- `v' converted to a rational number like `rational_to_be_tested'
		do
			Result := v
		ensure
			numerator: Result.p.value =
				{like some_expanded_integer}.adjusted_value (v)
			denominator: Result.q ≍ One
		end

	rational_from_native_integer_ref (
		v_ref: attached like some_native_integer_ref
		): like rational_to_be_tested
			-- `v_ref' converted to a rational number like
			-- `rational_to_be_tested'
		do
			Result := v_ref
		ensure
			numerator: Result.p.value =
				{like some_expanded_integer}.adjusted_value (v_ref.item)
			denominator: Result.q ≍ One
		end

	rational_from_integer (i: STS_INTEGER_NUMBER): like rational_to_be_tested
			-- `i' converted to a rational number like `rational_to_be_tested'
		do
			Result := i
		ensure
			numerator: Result.p.value =
				{like some_expanded_integer}.adjusted_value (i.value)
			denominator: Result.q ≍ One
		end

	rational_from_native_natural
		(v: like some_native_natural): like rational_to_be_tested
			-- `v' converted to a rational number like `rational_to_be_tested'
		do
			Result := v
		ensure
			numerator: Result.p.value =
				{like some_expanded_integer}.adjusted_value (v.as_integer_32)
			denominator: Result.q ≍ One
		end

	rational_from_native_natural_ref (v_ref: like some_native_natural_ref):
		like rational_to_be_tested
			-- `v_ref' converted to a rational number like
			-- `rational_to_be_tested'
		do
			Result := v_ref
		ensure
			numerator: Result.p.value = {like some_expanded_integer}.
				adjusted_value (v_ref.as_integer_32)
			denominator: Result.q ≍ One
		end

	native_real_from_rational (pq: STI_RATIONAL_NUMBER): like some_native_real
			-- `pq' converted to a native real number
		do
			Result := pq
		ensure
			value: Result = pq.value
		end

	native_ref_from_rational (pq: STI_RATIONAL_NUMBER):
		like some_native_real_ref
			-- `pq' converted to a native real number reference
		do
			Result := pq
		ensure
			value: Result = pq.value
		end

	numeric_from_rational (pq: STI_RATIONAL_NUMBER): NUMERIC
			-- `pq' converted to a {NUMERIC} object
		do
			Result := pq
		ensure
			value: Result = pq.value
		end

	comparable_from_rational (pq: STI_RATIONAL_NUMBER): COMPARABLE
			-- `pq' converted to a {COMPARABLE} object
		do
			Result := pq
		ensure
			value: Result = pq.value
		end

	hashable_from_rational (pq: STI_RATIONAL_NUMBER): HASHABLE
			-- `pq' converted to a {HASHABLE} object
		do
			Result := pq
		ensure
			value: Result = pq.value
		end

	rational_to_sti_n_tuple (pq: STI_RATIONAL_NUMBER):
		STI_N_TUPLE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- `pq' converted into an effective n-tuple of integer numbers
		do
			Result := pq
		ensure
			pair: Result.n = 2
			first: Result [1] ≍ pq.p
			second: Result [2] ≍ pq.q
		end

	rational_to_sts_n_tuple (pq: STI_RATIONAL_NUMBER):
		STS_N_TUPLE [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- `pq' converted into an n-tuple of integer numbers
		do
			Result := pq
		ensure
			pair: Result.n = 2
			first: Result [1] ≍ pq.p
			second: Result [2] ≍ pq.q
		end

	rational_to_numeric_sti_n_tuple (pq: STI_RATIONAL_NUMBER): STI_N_TUPLE
		[STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]]
			-- `pq' converted into an effective n-tuple of integer numbers
			-- compared by {STS_NUMERIC_EQUALITY}
		do
			Result := pq
		ensure
			pair: Result.n = 2
			first: Result.eq (Result [1], pq.p)
			second: Result.eq (Result [2],  pq.q)
		end

	rational_to_numeric_sts_n_tuple (pq: STI_RATIONAL_NUMBER): STS_N_TUPLE
		[STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]]
			-- `pq' converted into an effective n-tuple of integer numbers
			-- compared by {STS_NUMERIC_EQUALITY}
		do
			Result := pq
		ensure
			pair: Result.n = 2
			first: Result.eq (Result [1], pq.p)
			second: Result.eq (Result [2],  pq.q)
		end

	rational_to_sti_homogeneous_integer_pair (pq: STI_RATIONAL_NUMBER):
		STI_HOMOGENEOUS_ORDERED_PAIR [STS_INTEGER_NUMBER, STS_INTEGER_EQUALITY]
			-- `pq' converted into an effective homogeneous ordered pair of
			-- integer numbers
		do
			Result := pq
		ensure
			first: Result.a ≍ pq.p
			second: Result.b ≍ pq.q
		end

	rational_to_sti_homogeneous_numeric_pair (pq: STI_RATIONAL_NUMBER):
		STI_HOMOGENEOUS_ORDERED_PAIR
			[STS_INTEGER_NUMBER, STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]]
			-- `pq' converted into an effective homogeneous ordered pair of
			-- integer numbers
		do
			Result := pq
		ensure
			first: Result.a ≍ pq.p
			second: Result.b ≍ pq.q
		end

	rational_to_sti_integer_pair (pq: STI_RATIONAL_NUMBER): STI_ORDERED_PAIR [
		STS_INTEGER_NUMBER, STS_INTEGER_NUMBER,
		STS_INTEGER_EQUALITY, STS_INTEGER_EQUALITY
		]
			-- `pq' converted into an effective ordered pair of integer numbers
		do
			Result := pq
		ensure
			first: Result.a ≍ pq.p
			second: Result.b ≍ pq.q
		end

	rational_to_sti_numeric_pair (pq: STI_RATIONAL_NUMBER): STI_ORDERED_PAIR [
		STS_INTEGER_NUMBER, STS_INTEGER_NUMBER,
		STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER],
		STS_NUMERIC_EQUALITY [STS_INTEGER_NUMBER]
		]
			-- `pq' converted into an effective ordered pair of integer numbers
		do
			Result := pq
		ensure
			first: Result.a ≍ pq.p
			second: Result.b ≍ pq.q
		end

	complex_from_rational (pq: STI_RATIONAL_NUMBER): STI_COMPLEX_NUMBER
			-- `pq' converted to an {STI_COMPLEX_NUMBER}
		do
			Result := pq
		ensure
			real: Result.a ≍ pq
			imaginary: Result.b ≍ Zero
		end

	real_from_rational (pq: STI_RATIONAL_NUMBER): STI_REAL_NUMBER
			-- `pq' converted to an {STI_REAL_NUMBER}
		do
			Result := pq
		ensure
			definition: Result ≍ pq
		end

note
	copyright: "Copyright (c) 2012-2022, Rosivaldo Fernandes Alves"
	license: "[
		Eiffel Forum License v2
		(see http://www.eiffel.com/licensing/forum.txt)
		]"
	source: "https://launchpad.net/set-theory"
end
